<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:ns10="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
    xmlns:ns2="http://jazz.net/xmlns/alm/qm/v0.1/" xmlns:ns3="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:ns4="http://schema.ibm.com/vega/2008/"
    xmlns:ns5="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
    xmlns:ns6="http://jazz.net/xmlns/alm/qm/qmadapter/task/v0.1"
    xmlns:ns7="http://jazz.net/xmlns/alm/qm/qmadapter/v0.1"
    xmlns:ns8="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:ns9="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/" xmlns:atom="http://www.w3.org/2005/Atom"
    xmlns:print="xalan://com.ibm.rqm.xslfo.service.extension.PrintExecutionResultExtension"
    exclude-result-prefixes="ns10 ns9 ns8 ns7 ns6 ns5 ns4 ns3 ns2 atom dc xsl">
    <!-- included stylesheets -->
    <xsl:include href="cache://stylesheets/execution-result/table.xsl"/>
    
    <!-- variables -->
    <xsl:variable name="lang" select="//locale"/>
    <xsl:variable name="strings" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]"/>
    <xsl:variable name="font" select="font"/>
    
    <xsl:template match="transformArgs">
        <xsl:choose>
            <xsl:when test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
                <fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
                    initial-page-number="1" force-page-count="no-force"
                    master-reference="PageMaster">
                    <fo:static-content flow-name="xsl-region-before">
                        <fo:block font-size="8pt" text-align="right">
                            <fo:page-number/> / <fo:page-number-citation ref-id="{reqNumber}"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:static-content flow-name="xsl-region-after">
                    	<fo:block-container height="0.1in" padding-after="0.07in">
							<fo:block text-align="center">
								<fo:leader leader-pattern='rule' rule-thickness='1pt' leader-length='100%'/>
							</fo:block>
						</fo:block-container>
                        <fo:block text-align="center">
                            <fo:retrieve-marker retrieve-class-name="artifactUri"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:flow flow-name="xsl-region-body">
                        <fo:marker marker-class-name="artifactUri">
                            <fo:block font-size="6pt">
                                <fo:basic-link external-destination="{originUrl}" color="blue"
                                    text-decoration="underline">
                                    <xsl:value-of select="originUrl"/>
                                </fo:basic-link>
                            </fo:block>
                        </fo:marker>
                        <xsl:apply-templates select="descendant::ns2:executionworkitem[not(@href)]"/>
                        <fo:block id="{reqNumber}"/>
                    </fo:flow>
                </fo:page-sequence>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="descendant::ns2:executionworkitem[not(@href)]"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ns2:executionworkitem">
        <xsl:choose>
            <xsl:when test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
                <xsl:choose>
                    <xsl:when test="../../componentType='STAND_ALONE_SUMMARY' or (../../componentType='COMPOSITE_SUMMARY')">
                        <xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
                            <fo:block font-size="14pt" space-before="6mm" space-after="2mm">
                                <xsl:value-of select="$strings/labelSummary"/>: <xsl:value-of select="$strings/labelTestExecutionRecord"/>
                            </fo:block>
                        </xsl:if>
                        
                        <!-- EXECUTION WORK ITEM TABLE -->
                        <fo:table xsl:use-attribute-sets="artifact-table">
                            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                                <!-- name -->
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block><xsl:value-of select="$strings/labelName"/></fo:block>
                                </fo:table-cell>
                                <!-- test case -->
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block><xsl:value-of select="$strings/labelTestCase"/></fo:block>
                                </fo:table-cell>
                                <!-- test script -->
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block><xsl:value-of select="$strings/labelTestScript"/></fo:block>
                                </fo:table-cell>
                                <!-- last result -->
                                <fo:table-cell xsl:use-attribute-sets="border" width="47">
                                    <fo:block><xsl:value-of select="$strings/labelLastResult"/></fo:block>
                                </fo:table-cell>
                                <!-- modified date -->
                                <fo:table-cell xsl:use-attribute-sets="border timestamp-header" width="45">
                                    <fo:block><xsl:value-of select="$strings/labelModifiedDate"/></fo:block>
                                </fo:table-cell>
                                <!-- weight -->
                                <fo:table-cell xsl:use-attribute-sets="border" width="30">
                                    <fo:block><xsl:value-of select="$strings/labelWeight"/></fo:block>
                                </fo:table-cell>
                            </fo:table-header>
                            <fo:table-body>
                                <fo:table-row>
                                    <!-- name -->
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:apply-templates select="dc:title"/></fo:block>
                                    </fo:table-cell>
                                    <!-- test case -->
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:apply-templates select="../../atom:titles/atom:testCaseTitle"/></fo:block>
                                    </fo:table-cell>
                                    <!-- test script -->
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:apply-templates select="../../atom:titles/atom:testScriptTitle"/></fo:block>
                                    </fo:table-cell>
                                    <!-- last result -->
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block><xsl:apply-templates select="../../atom:titles/atom:currentResultState"/></fo:block>
                                    </fo:table-cell>
                                    <!-- modified date -->
                                    <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                                        <fo:block><xsl:value-of select="ns3:updated"/></fo:block>
                                    </fo:table-cell>
                                    <!-- weight -->
                                    <fo:table-cell xsl:use-attribute-sets="border" text-align="right">
                                        <fo:block><xsl:value-of select="ns10:weight"/></fo:block>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-body>
                        </fo:table>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row>
                            <!-- name -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="dc:title"/></fo:block>
                            </fo:table-cell>
                            <!-- test case -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="../../atom:titles/atom:testCaseTitle"/></fo:block>
                            </fo:table-cell>
                            <!-- test script -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="../../atom:titles/atom:testScriptTitle"/></fo:block>
                            </fo:table-cell>
                            <!-- last result -->
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="../../atom:titles/atom:currentResultState"/></fo:block>
                            </fo:table-cell>
                            <!-- modified date -->
                            <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                                <fo:block><xsl:value-of select="ns3:updated"/></fo:block>
                            </fo:table-cell>
                            <!-- weight -->
                            <fo:table-cell xsl:use-attribute-sets="border" text-align="right">
                                <fo:block><xsl:value-of select="ns10:weight"/></fo:block>
                            </fo:table-cell>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="(../../componentType='COMPOSITE') and (../../recursive)">
                <fo:wrapper>
                    <fo:table-row>
                        <!-- name -->
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:apply-templates select="dc:title"/></fo:block>
                        </fo:table-cell>
                        <!-- test case -->
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:apply-templates select="../../atom:titles/atom:testCaseTitle"/></fo:block>
                        </fo:table-cell>
                        <!-- test script -->
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:apply-templates select="../../atom:titles/atom:testScriptTitle"/></fo:block>
                        </fo:table-cell>
                        <!-- last result -->
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:apply-templates select="../../atom:titles/atom:currentResultState"/></fo:block>
                        </fo:table-cell>
                        <!-- modified date -->
                        <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                            <fo:block><xsl:value-of select="ns3:updated"/></fo:block>
                        </fo:table-cell>
                        <!-- weight -->
                        <fo:table-cell xsl:use-attribute-sets="border" text-align="right">
                            <fo:block><xsl:value-of select="ns10:weight"/></fo:block>
                        </fo:table-cell>
                    </fo:table-row>
                    
                    <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format" id="ns3:identifier">
                        <fo:block font-size="14pt" space-before="2mm" space-after="2mm"
                            border-after-style="solid" border-after-width="thin">
                            <xsl:value-of select="$strings/labelTestExecutionRecord"/>: <xsl:value-of select="dc:title"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of select="$strings/labelModifiedDate"/>: <xsl:value-of select="ns3:updated"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of select="$strings/reqLabel_created_by"/>: <xsl:value-of select="dc:creator"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of select="$strings/labelOwnedBy"/>: <xsl:value-of select="ns3:owner"/>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of select="$strings/labelTestCase"/>: <xsl:value-of select="../../atom:titles/atom:testCaseTitle"/>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of select="$strings/labelTestPlan"/>: <xsl:value-of select="../../atom:titles/atom:testPlanTitle"/>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of select="$strings/labelTestScript"/>: <xsl:value-of select="../../atom:titles/atom:testScriptTitle"/>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of select="$strings/labelTestEnvironment"/>: <xsl:value-of select="../../atom:titles/atom:configurations/atom:title"/>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of select="$strings/labelTestMilestone"/>: <xsl:value-of select="../../atom:titles/atom:testPhaseTitle"/>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of select="$strings/labelLastResult"/>: <xsl:apply-templates select="../../atom:titles/atom:currentResultState"/>
                        </fo:block>
                        
                        <!-- execution results -->
                        <fo:block font-size="10pt" space-before="4mm" space-after="4mm" font-weight="bold" text-decoration="underline" text-align="left"
                            keep-with-next="always">
                            <xsl:value-of select="$strings/executionResults"/>
                        </fo:block>
                        
                        <xsl:if test="ns2:executionresult">
                            <!-- results table -->
                            <xsl:call-template name="buildExecutionResultTable">
                                <xsl:with-param name="strings" select="$strings"/>
                                <xsl:with-param name="results" select="document(concat('rest://', ns2:executionresult/@href))/ns2:executionresult"/>
                                
                                <xsl:with-param name="render-id" select="true()"/>
                                <xsl:with-param name="render-name" select="true()"/>
                                <xsl:with-param name="render-state" select="true()"/>
                            </xsl:call-template>
                            
                            <!-- recursive results pages -->
                            <xsl:for-each select="ns2:executionresult">
                                <xsl:variable name="artifact-summary" select="print:printArtifactSummary($lang, @href)"/>
                                <fo:block break-before="page">
                                    <xsl:copy-of select="$artifact-summary/fo:root/fo:page-sequence/fo:flow[@flow-name='xsl-region-body']/*"/>    
                                </fo:block>
                            </xsl:for-each>
                        </xsl:if>
                        
                        
                    </fo:block>
                </fo:wrapper>
            </xsl:when>
            <xsl:otherwise>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                    <xsl:if test="(../../componentType='COMPOSITE' )">
                        <xsl:if test="//pageBreaks = 'true'">
                            <xsl:attribute name="break-after">page</xsl:attribute>
                        </xsl:if>
                        <xsl:attribute name="id">
                            <xsl:value-of select="ns3:identifier"/>
                        </xsl:attribute>
                    </xsl:if>
                    <fo:block font-size="14pt" space-before="2mm" space-after="2mm"
                        border-after-style="solid" border-after-width="thin">
                        <xsl:value-of select="$strings/labelTestExecutionRecord"/>: <xsl:value-of select="dc:title"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelModifiedDate"/>: <xsl:value-of select="ns3:updated"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/reqLabel_created_by"/>: <xsl:value-of select="dc:creator"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of select="$strings/labelOwnedBy"/>: <xsl:value-of select="ns3:owner"/>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of select="$strings/labelTestCase"/>: <xsl:value-of select="../../atom:titles/atom:testCaseTitle"/>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of select="$strings/labelTestPlan"/>: <xsl:value-of select="../../atom:titles/atom:testPlanTitle"/>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of select="$strings/labelTestScript"/>: <xsl:value-of select="../../atom:titles/atom:testScriptTitle"/>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of select="$strings/labelTestEnvironment"/>: <xsl:value-of select="../../atom:titles/atom:configurations/atom:title"/>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of select="$strings/labelTestMilestone"/>: <xsl:value-of select="../../atom:titles/atom:testPhaseTitle"/>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of select="$strings/labelLastResult"/>: <xsl:apply-templates select="../../atom:titles/atom:currentResultState"/>
                    </fo:block>
                    
                    <!-- execution results -->
                    <fo:block font-size="10pt" space-before="4mm" space-after="4mm" font-weight="bold" text-decoration="underline" text-align="left"
                        keep-with-next="always">
                        <xsl:value-of select="$strings/executionResults"/>
                    </fo:block>
                    
                    <xsl:if test="ns2:executionresult">
                        <!-- execution results table -->
                        <xsl:call-template name="buildExecutionResultTable">
                            <xsl:with-param name="strings" select="$strings"/>
                            <xsl:with-param name="results" select="document(concat('rest://', ns2:executionresult/@href))/ns2:executionresult"/>
                            
                            <xsl:with-param name="render-id" select="true()"/>
                            <xsl:with-param name="render-name" select="true()"/>
                            <xsl:with-param name="render-state" select="true()"/>
                        </xsl:call-template>
                    </xsl:if>
                </fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- result state -->
    <xsl:template match="atom:currentResultState">
        <xsl:choose>
            <!-- in progress -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.inprogress'">
                <xsl:value-of select="$strings/InProgress"/></xsl:when>
            <!-- passed -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.passed'">
                <xsl:value-of select="$strings/labelPassed"/></xsl:when>
            <!-- failed -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.failed'">
                <xsl:value-of select="$strings/labelFailed"/></xsl:when>
            <!-- blocked -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.blocked'">
                <xsl:value-of select="$strings/labelBlocked"/></xsl:when>
            <!-- partially blocked -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.part_blocked'">
                <xsl:value-of select="$strings/labelPartBlocked"/></xsl:when>
            <!-- complete -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.complete'">
                <xsl:value-of select="$strings/labelComplete"/></xsl:when>
            <!-- inconclusive -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.inconclusive'">
                <xsl:value-of select="$strings/labelInconclusive"/></xsl:when>
            <!-- incomplete -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.incomplete'">
                <xsl:value-of select="$strings/labelIncomplete"/></xsl:when>
            <!-- error -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.error'">
                <xsl:value-of select="$strings/labelError"/></xsl:when>
            <!-- perm fail -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.perm_failed'">
                <xsl:value-of select="$strings/labelPermFailed"/></xsl:when>
            <!-- paused -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.state.paused'">
                <xsl:value-of select="$strings/labelPaused"/></xsl:when>
            <!-- default -->
            <xsl:otherwise>
                <xsl:apply-templates/></xsl:otherwise>    
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>
