<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:req="http://purl.org/dc/elements/1.1/"
    xmlns:ns2="http://jazz.net/xmlns/alm/qm/v0.1/" xmlns:ns3="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:ns4="http://schema.ibm.com/vega/2008/"
    xmlns:ns5="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
    xmlns:ns6="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
    xmlns:ns7="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:ns8="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
    exclude-result-prefixes="ns8 ns7 ns6 ns5 ns4 ns3 ns2 req xsl">
    
    <!-- included stylesheets -->
    <xsl:include href="wrapping-rules.xsl"/>
    <xsl:include href="attribute-sets.xsl"/>
    <!--<xsl:include href="../stylesheets/attribute-sets.xsl"/>-->
    
    <!-- variables -->
    <xsl:variable name="lang" select="locale"/>
    <xsl:variable name="strings" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]"/>
    <xsl:variable name="font" select="font"/>
	
    <xsl:template match="transformArgs">
        <xsl:choose>
            <xsl:when test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
                <fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
                    initial-page-number="1" force-page-count="no-force"
                    master-reference="PageMaster">
                    <fo:static-content flow-name="xsl-region-before">
                        <fo:block font-size="8pt" text-align="right">
                            <fo:page-number/> / <fo:page-number-citation ref-id="{reqNumber}"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:static-content flow-name="xsl-region-after">
                    	<fo:block-container height="0.1in" padding-after="0.07in">
							<fo:block text-align="center">
								<fo:leader leader-pattern='rule' rule-thickness='1pt' leader-length='100%'/>
							</fo:block>
						</fo:block-container>
                        <fo:block text-align="center">
                            <fo:retrieve-marker retrieve-class-name="artifactUri"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:flow flow-name="xsl-region-body">
                        <fo:marker marker-class-name="artifactUri">
                            <fo:block font-size="6pt">
                                <fo:basic-link external-destination="{originUrl}" color="blue"
                                    text-decoration="underline">
                                    <xsl:value-of select="originUrl"/>
                                </fo:basic-link>
                            </fo:block>
                        </fo:marker>
                        <xsl:apply-templates select="descendant::ns2:request"/>
                        <fo:block id="{reqNumber}"/>
                    </fo:flow>
                </fo:page-sequence>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="descendant::ns2:request"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- request -->
    <xsl:template match="ns2:request">
        <xsl:choose>
            <xsl:when
                test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
                <xsl:choose>
                    <xsl:when
                        test="../../componentType='STAND_ALONE_SUMMARY' or (../../componentType='COMPOSITE_SUMMARY')">
                        <xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
                            <fo:block font-size="14pt" space-before="6mm" space-after="2mm">
								<xsl:value-of select="$strings/labelSummary"/>
								: 
								<xsl:value-of select="$strings/reqLabel_request"/>
							</fo:block>
                        </xsl:if>
                        <fo:table xsl:use-attribute-sets="artifact-table">
                            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                                <fo:table-row>
                                    <fo:table-cell xsl:use-attribute-sets="border id-header" width="10mm">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="$strings/labelId"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="$strings/labelSummary"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border timestamp-header" width="20mm">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="$strings/reqLabel_created_on"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border" width="20mm">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="$strings/reqLabel_confirm_by"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border" width="20mm">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="$strings/reqLabel_assigned"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border" width="15mm">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="$strings/reqLabel_priority"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border" width="20mm">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="$strings/reqLabel_status"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-header>
                            <fo:table-body>
                                <fo:table-row>
                                    <fo:table-cell xsl:use-attribute-sets="border id-cell">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="ns2:visibleId"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:apply-templates select="ns2:summary"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="ns2:createDate"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="ns2:confirmationDate"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:apply-templates select="ns2:fulfiller"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                            	<xsl:apply-templates select="ns2:priority"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell xsl:use-attribute-sets="border">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:apply-templates select="ns3:state"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-body>
                        </fo:table>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row>
                            <fo:table-cell xsl:use-attribute-sets="border id-cell">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:value-of select="ns2:visibleId"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:apply-templates select="ns2:summary"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:value-of select="ns2:createDate"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:value-of select="ns2:confirmationDate"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:apply-templates select="ns2:fulfiller"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                    	<xsl:apply-templates select="ns2:priority"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:apply-templates select="ns3:state"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                    <xsl:if test="(../../componentType='COMPOSITE' )">
                        <xsl:attribute name="break-after">page</xsl:attribute>
                    </xsl:if>
                    <fo:block/>
                    
                    <!-- page header -->
                    <!-- Request: <ns2:title> -->
                    <fo:block font-size="14pt" space-before="4mm" border-after-style="solid" border-after-width="thin">
                        <xsl:value-of select="$strings/reqLabel_request"/><xsl:text> </xsl:text>
						<xsl:value-of select="ns2:visibleId"/>
                    </fo:block>
                    
                    <!-- creation date -->
                    <!-- Created On: <ns2:createDate> -->
                    <fo:block font-size="10pt" space-before="2mm" space-after="4mm">
                        <xsl:value-of select="$strings/reqLabel_created_on"/>: <xsl:value-of select="ns2:createDate"/>
                    </fo:block>
                    
                    <fo:table xsl:use-attribute-sets="simple-text">
                        <fo:table-body>
                            <!-- requester -->                        <!-- status -->                    <!-- priority -->
                            <fo:table-row>
                                <fo:table-cell>
                                    <fo:block><xsl:value-of select="$strings/reqLabel_created_by"/>: <xsl:apply-templates select="ns2:requester"/></fo:block>
                                </fo:table-cell>
                                <fo:table-cell>
                                    <fo:block><xsl:value-of select="$strings/reqLabel_status"/>: <xsl:apply-templates select="ns3:state"/></fo:block>
                                </fo:table-cell>
                                <fo:table-cell>
                                    <fo:block><xsl:value-of select="$strings/reqLabel_priority"/>: <xsl:apply-templates select="ns2:priority"/></fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                           
                            <!-- fulfiller -->                        <!-- confirmation date -->
                            <fo:table-row>
                                <fo:table-cell>
                                    <fo:block><xsl:value-of select="$strings/reqLabel_assigned"/>: <xsl:apply-templates select="ns2:fulfiller"/></fo:block>
                                </fo:table-cell>
                                <fo:table-cell number-columns-spanned="2">
                                    <fo:block><xsl:value-of select="$strings/reqLabel_confirm_by"/>: <xsl:value-of select="ns2:confirmationDate"/></fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                           
                            <!-- summary -->
                            <fo:table-row>
                                <fo:table-cell number-columns-spanned="3">
                                    <fo:block><xsl:value-of select="$strings/labeldescription"/>: <xsl:apply-templates select="ns2:summary"/></fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                            
                            <!-- reserve from -->
                            <fo:table-row>
                                <fo:table-cell number-columns-spanned="3">
                                    <fo:block><xsl:value-of select="$strings/resreservefrom"/>: <xsl:apply-templates select="ns2:reserveFrom"/></fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                            
                            <!-- reserve to -->
                            <fo:table-row>
                                <fo:table-cell number-columns-spanned="3">
                                    <fo:block><xsl:value-of select="$strings/resreserveTo"/>: <xsl:apply-templates select="ns2:reserveTo"/></fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                        </fo:table-body>
                    </fo:table>
					
					<!-- Assets -->
                    <xsl:for-each select="ns2:configuration/Assets/AssetInstance">
                        <fo:block font-size="10pt" font-weight="bold" text-decoration="underline" space-before="4mm">
                            <xsl:value-of select="Label"/>
                        </fo:block>
                        
                        <xsl:apply-templates select="AssetType"/>
                    </xsl:for-each>
                    
					<!-- Comments -->
                    <fo:block font-weight="bold" text-decoration="underline" space-before="4mm">
						<xsl:apply-templates select="$strings/reqLabel_comments"/>
                    </fo:block>
                
                    <xsl:value-of select="ns2:comments"/>
                </fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- asset type -->
    <xsl:template match="AssetType">
        <fo:block space-before="4mm" font-size="8pt">
            <xsl:choose>
                <xsl:when test="type='Machine'">
                    <xsl:value-of select="$strings/Machine"/>: </xsl:when>
                <xsl:when test="type='RemoteConnection'">
                    <xsl:value-of select="$strings/RemoteConnectivity"/>: </xsl:when>
                <xsl:when test="type='OperatingSystem'">
                    <xsl:value-of select="$strings/operatingsystem"/>: </xsl:when>
                <xsl:otherwise>
                    <xsl:if test="type">
                        <xsl:value-of select="type"/>: 
                    </xsl:if>
                </xsl:otherwise>
                    
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="name='Machine'">
                    <xsl:value-of select="$strings/Machine"/></xsl:when>
                <xsl:when test="name='PhysicalMachine'">
                    <xsl:value-of select="$strings/PhysicalMachine"/></xsl:when>
                <xsl:when test="name='VirtualMachine'">
                    <xsl:value-of select="$strings/VirtualMachine"/></xsl:when>
                <xsl:when test="name='Version'">
                    <xsl:value-of select="$strings/version"/></xsl:when>
                <xsl:otherwise>
                        <xsl:value-of select="name"/></xsl:otherwise>
            </xsl:choose>
        </fo:block>
        
        <xsl:apply-templates select="Attributes/Attribute"/>
    </xsl:template>
    
    <xsl:template match="Attribute">
        <fo:block xsl:use-attribute-sets="simple-text">
            <xsl:attribute name="text-indent">
                <xsl:value-of select="concat(count(ancestor::AssetInstance) * .75,'cm')"/>
            </xsl:attribute>
            
            <xsl:variable name="reqLabel" select="translate(@name,' ','')"/>
            
            <xsl:choose>
                <xsl:when test="@type='StringType'">
                    <xsl:value-of select="$strings//*[name()=$reqLabel]"/>: <xsl:value-of select="value/string"/></xsl:when>
                <xsl:when test="@type='PointerType'">
                    <xsl:value-of select="$strings//*[name()=$reqLabel]"/>: <xsl:value-of select="value/DisplayValue/displayString"/></xsl:when>
                <xsl:when test="@type='NumericType'">
                    <xsl:value-of select="$strings//*[name()=$reqLabel]"/>: <xsl:value-of select="value/int"/></xsl:when>
                <xsl:when test="@type='ConfigurationPartType'">
                    <xsl:value-of select="$strings//*[name()=$reqLabel]"/>: <xsl:apply-templates select="value/AssetInstance/AssetType"/></xsl:when>
            </xsl:choose>
        </fo:block>
    </xsl:template>
    
    <xsl:template match="ns3:state">
        <xsl:choose>
            <!-- draft -->
            <xsl:when test="current()='Draft'">
                <xsl:value-of select="$strings/reqStateDraft"/></xsl:when>
            <!-- submitted -->
            <xsl:when test="current()='Submitted'">
                <xsl:value-of select="$strings/reqStateSubmitted"/></xsl:when>
            <!-- fulfilled -->
            <xsl:when test="current()='Fulfilled'">
                <xsl:value-of select="$strings/reqStateFulfilled"/></xsl:when>
			<!-- assigned -->
            <xsl:when test="current()='Assigned'">
                <xsl:value-of select="$strings/reqStateAssigned"/></xsl:when>
            <!-- returned -->            
            <xsl:when test="current()='Returned'">
                <xsl:value-of select="$strings/reqStateReturned"/></xsl:when>
            <!-- default -->
            <xsl:otherwise>
                <xsl:apply-templates/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ns2:priority">
        <xsl:choose>
            <!-- low -->
            <xsl:when test="current()='Low'">
                <xsl:value-of select="$strings/reqPriorityLow"/></xsl:when>
            <!-- normal -->
            <xsl:when test="current()='Normal'">
                <xsl:value-of select="$strings/reqPriorityNormal"/></xsl:when>
            <!-- high -->
            <xsl:when test="current()='High'">
                <xsl:value-of select="$strings/reqPriorityHigh"/></xsl:when>
            <!-- default -->
            <xsl:otherwise>
                <xsl:apply-templates/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>
