<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:ns10="http://purl.org/dc/elements/1.1/" xmlns:ns11="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
	xmlns:ns12="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
	xmlns:ns2="http://jazz.net/xmlns/alm/qm/v0.1/" xmlns:ns3="http://jazz.net/xmlns/alm/v0.1/"
	xmlns:ns4="http://jazz.net/xmlns/alm/qm/v0.1/catalog/v0.1" xmlns:ns5="http://schema.ibm.com/vega/2008/"
	xmlns:ns6="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
	xmlns:ns7="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/" xmlns:ns8="http://jazz.net/xmlns/alm/qm/qmadapter/task/v0.1"
	xmlns:ns9="http://jazz.net/xmlns/alm/qm/qmadapter/v0.1" xmlns:ns15="http://jazz.net/xmlns/alm/qm/v0.1/#extension"
	exclude-result-prefixes="ns9 ns15 ns10 ns11 ns12 ns8 ns7 ns6 ns5 ns4 ns3 ns2 xsl">
	
	<!-- included stylesheets -->
	<xsl:include href="xhtml-to-xslfo-ns.xsl"/>
	<xsl:include href="wrapping-rules.xsl"/>
	<xsl:include href="attribute-sets.xsl"/>
	
	<!-- variables -->
	<xsl:variable name="lang" select="locale"/>
	<xsl:variable name="strings" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]"/>
	<xsl:variable name="font" select="font"/>
	
	<xsl:template match="transformArgs">
		<xsl:choose>
			<xsl:when
				test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
				<fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
					initial-page-number="1" force-page-count="no-force"
					master-reference="PageMaster">
					<fo:static-content flow-name="xsl-region-before">
						<fo:block font-size="8pt" text-align="right">
							<fo:page-number />
							/
							<fo:page-number-citation ref-id="{reqNumber}" />
						</fo:block>
					</fo:static-content>
					<fo:static-content flow-name="xsl-region-after">
						<fo:block-container height="0.1in"
							padding-after="0.07in">
							<fo:block text-align="center">
								<fo:leader leader-pattern='rule' rule-thickness='1pt'
									leader-length='100%' />
							</fo:block>
						</fo:block-container>
						<fo:block text-align="center">
							<fo:retrieve-marker retrieve-class-name="artifactUri" />
						</fo:block>
					</fo:static-content>
					<fo:flow flow-name="xsl-region-body">
						<fo:marker marker-class-name="artifactUri">
							<fo:block font-size="6pt">
								<fo:basic-link external-destination="{originUrl}"
									color="blue" text-decoration="underline">
									<xsl:value-of select="originUrl" />
								</fo:basic-link>
							</fo:block>
						</fo:marker>
						<xsl:apply-templates select="descendant::ns2:testcase" />
						<fo:block id="{reqNumber}" />
					</fo:flow>
				</fo:page-sequence>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="descendant::ns2:testcase" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="ns2:testcase">
		<xsl:choose>
			<xsl:when
				test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
				<xsl:choose>
					<xsl:when
						test="../../componentType='STAND_ALONE_SUMMARY' or (../../componentType='COMPOSITE_SUMMARY')">
						<xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
							<fo:block font-size="14pt" space-before="6mm"
								space-after="2mm">
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary" />
								:
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestCase" />
							</fo:block>
						</xsl:if>
						<fo:table xsl:use-attribute-sets="artifact-table">
							<fo:table-header xsl:use-attribute-sets="artifact-table-header">
								<!-- id -->
								<fo:table-cell xsl:use-attribute-sets="border id-header">
									<fo:block><xsl:value-of select="$strings/labelId"/></fo:block>
								</fo:table-cell>
								<!-- name -->
								<fo:table-cell xsl:use-attribute-sets="border">
									<fo:block><xsl:value-of select="$strings/labelName"/></fo:block>
								</fo:table-cell>
								<!-- state -->
								<fo:table-cell xsl:use-attribute-sets="border" width="60">
									<fo:block><xsl:value-of select="$strings/labelState"/></fo:block>
								</fo:table-cell>
								<!-- weight -->
								<fo:table-cell xsl:use-attribute-sets="border" width="25">
									<fo:block><xsl:value-of select="$strings/labelWeight"/></fo:block>
								</fo:table-cell>
								<!-- last modified -->
								<fo:table-cell xsl:use-attribute-sets="border timestamp-header">
									<fo:block><xsl:value-of select="$strings/labelModifiedDate"/></fo:block>
								</fo:table-cell>
							</fo:table-header>
							<fo:table-body>
								<fo:table-row>
									<!-- id -->
									<fo:table-cell xsl:use-attribute-sets="border id-cell">
										<fo:block><xsl:value-of select="//generatedId"/></fo:block>
									</fo:table-cell>
									<!-- name -->
									<fo:table-cell xsl:use-attribute-sets="border">
										<fo:block><xsl:apply-templates select="ns10:title"/></fo:block>
									</fo:table-cell>
									<!-- state -->
									<fo:table-cell xsl:use-attribute-sets="border">
										<fo:block><xsl:apply-templates select="ns3:state"/></fo:block>
									</fo:table-cell>
									<!-- weight -->
									<fo:table-cell xsl:use-attribute-sets="border" text-align="right">
										<fo:block><xsl:value-of select="ns2:weight"/></fo:block>
									</fo:table-cell>
									<!-- last modified -->
									<fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
										<fo:block><xsl:value-of select="ns3:updated"/></fo:block>
									</fo:table-cell>
								</fo:table-row>
							</fo:table-body>
						</fo:table>
					</xsl:when>
					<xsl:otherwise>
						<fo:table-row>
							<!-- id -->
							<fo:table-cell xsl:use-attribute-sets="border id-cell">
								<fo:block><xsl:value-of select="//generatedId"/></fo:block>
							</fo:table-cell>
							<!-- name -->
							<fo:table-cell xsl:use-attribute-sets="border">
								<fo:block><xsl:apply-templates select="ns10:title"/></fo:block>
							</fo:table-cell>
							<!-- state -->
							<fo:table-cell xsl:use-attribute-sets="border">
								<fo:block><xsl:apply-templates select="ns3:state"/></fo:block>
							</fo:table-cell>
							<!-- weight -->
							<fo:table-cell xsl:use-attribute-sets="border" text-align="right">
								<fo:block><xsl:value-of select="ns2:weight"/></fo:block>
							</fo:table-cell>
							<!-- last modified -->
							<fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
								<fo:block><xsl:value-of select="ns3:updated"/></fo:block>
							</fo:table-cell>
						</fo:table-row>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<fo:wrapper>
					<xsl:if test="(../../componentType='COMPOSITE') and (../../recursive)">
						<fo:table-row>
							<!-- id -->
							<fo:table-cell xsl:use-attribute-sets="border id-cell">
								<fo:block><xsl:value-of select="//generatedId"/></fo:block>
							</fo:table-cell>
							<!-- name -->
							<fo:table-cell xsl:use-attribute-sets="border">
								<fo:block><xsl:apply-templates select="ns10:title"/></fo:block>
							</fo:table-cell>
							<!-- state -->
							<fo:table-cell xsl:use-attribute-sets="border">
								<fo:block><xsl:apply-templates select="ns3:state"/></fo:block>
							</fo:table-cell>
							<!-- weight -->
							<fo:table-cell xsl:use-attribute-sets="border" text-align="right">
								<fo:block><xsl:value-of select="ns2:weight"/></fo:block>
							</fo:table-cell>
							<!-- last modified -->
							<fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
								<fo:block><xsl:value-of select="ns3:updated"/></fo:block>
							</fo:table-cell>
						</fo:table-row>
					</xsl:if>
				
					<fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format"> <!-- font-size="8pt" ? -->
	
						<xsl:if test="(../../componentType='COMPOSITE')">
							<!--
								<xsl:attribute name="break-after">page</xsl:attribute>
							-->
							<xsl:attribute name="id">
								<xsl:value-of select="ns10:identifier" />
							</xsl:attribute>
						</xsl:if>
	
	
						<fo:block font-size="14pt" space-after="6mm" text-align="left"
							font-weight="bold" text-decoration="underline" color="rgb(0,0,0)">
							<xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestCase" />
							<xsl:text> </xsl:text><xsl:value-of select="//generatedId" />: <xsl:value-of select="ns10:title" />						
						</fo:block>
						<fo:block space-after="2mm" text-align="left" font-weight="bold">
							<xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeloriginatorLabel" />
							:
							<xsl:value-of select="ns10:creator" />
						</fo:block>
						<fo:block space-after="4mm" text-align="left" font-weight="bold">
							<xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelState" />
							:
							<xsl:choose>
								<xsl:when test="ns3:state='com.ibm.rqm.planning.common.new'">
									<xsl:value-of
										select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/com.ibm.rqm.planning.common.new" />
								</xsl:when>
								<xsl:when test="ns3:state='com.ibm.rqm.planning.common.underreview'">
									<xsl:value-of
										select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/com.ibm.rqm.planning.common.underreview" />
								</xsl:when>
								<xsl:when test="ns3:state='com.ibm.rqm.planning.common.approved'">
									<xsl:value-of
										select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/com.ibm.rqm.planning.common.approved" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of
										select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
								</xsl:otherwise>
							</xsl:choose>
						</fo:block>
	
						<xsl:for-each select="ns2:section">
							<xsl:choose>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-SUMMARY-TITLE'">
	
									<fo:block space-after="4mm" space-before="4mm"
										font-weight="bold" text-decoration="underline" text-align="left">
										<xsl:value-of
											select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary" />
									</fo:block>
									<xsl:for-each select="../../../categoryType">
										<fo:block>
											<xsl:value-of select="current()" />
											:
											<xsl:choose>
												<xsl:when test="../artifactXML/ns2:testcase/ns2:category[@term=current()]">
													<xsl:value-of select="../artifactXML/ns2:testcase/ns2:category[@term=current()]/@value" />
												</xsl:when>
												<xsl:otherwise>
													<xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
												</xsl:otherwise>
											</xsl:choose>
										</fo:block>
									</xsl:for-each>
									
									<!-- Test Activity -->
									<fo:block>
										<xsl:variable name="activity" select="../ns2:activity"/>
										
										<xsl:if test="$activity">
											<xsl:variable name="value" select="$strings//*[name()=$activity]"/>
											
											<xsl:choose>
												<xsl:when test="$value">
													<xsl:value-of select="$strings/labelActivity"/>: <xsl:value-of select="$value"/></xsl:when>
												<xsl:otherwise>
													<xsl:value-of select="$strings/labelActivity"/>: <xsl:value-of select="$strings/labelUnassigned"/></xsl:otherwise>
											</xsl:choose>
										</xsl:if>	
									</fo:block>
									
									<!-- Trigger -->
									<fo:block>
										<xsl:variable name="trigger" select="../ns2:trigger"/>
										
										<xsl:if test="$trigger">
											<xsl:variable name="value" select="$strings//*[name()=$trigger]"/>
											
											<xsl:choose>
												<xsl:when test="$value">
													<xsl:value-of select="$strings/labelTrigger"/>: <xsl:value-of select="$value"/></xsl:when>
												<xsl:otherwise>
													<xsl:value-of select="$strings/labelTrigger"/>: <xsl:value-of select="$strings/labelUnassigned"/></xsl:otherwise>
											</xsl:choose>
										</xsl:if>
									</fo:block>
									
									<!-- Weight -->
									<fo:block>
										<xsl:value-of
											select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelWeight" />
										:
										<xsl:value-of select="../ns2:weight" />
									</fo:block>
									
									<!-- Description -->
									<fo:block space-before="4mm" space-after="1mm">
										<xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"/>:
									</fo:block>
									
									<fo:block space-after="4mm" white-space-collapse="false" linefeed-treatment="preserve">
										<xsl:value-of select="../ns10:description"/>
									</fo:block>
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-DESIGN-TITLE'">
	
									<xsl:if
										test="../ns2:com.ibm.rqm.planning.editor.section.testCaseDesign">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/RQM-KEY-TC-DESIGN-TITLE" />
										</fo:block>
										<fo:block>
											<xsl:apply-templates
												select="../ns2:com.ibm.rqm.planning.editor.section.testCaseDesign" />
										</fo:block>
									</xsl:if>
	
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-PROC-REVIEW-TITLE'">
	
									<xsl:if test="../ns2:approvals/ns2:approvalDescriptor">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelFormalReview" />
										</fo:block>
										<fo:table font-size="8pt">
											<fo:table-header>
												<fo:table-row background-color="rgb(249,245,219)">
													<fo:table-cell width="30mm" padding="3pt" border="0.5pt solid black">
														<fo:block-container overflow="hidden">
															<fo:block font-weight="bold">
																<xsl:value-of
																	select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTitleApprovalTable" />
															</fo:block>
														</fo:block-container>
													</fo:table-cell>
													<fo:table-cell width="30mm" padding="3pt" border="0.5pt solid black">
														<fo:block-container overflow="hidden">
															<fo:block font-weight="bold">
																<xsl:value-of
																	select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelName" />
															</fo:block>
														</fo:block-container>
													</fo:table-cell>
													<fo:table-cell padding="3pt" border="0.5pt solid black">
														<fo:block-container overflow="hidden">
															<fo:block font-weight="bold">
																<xsl:value-of
																	select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_status" />
															</fo:block>
														</fo:block-container>
													</fo:table-cell>
													<fo:table-cell padding="3pt" border="0.5pt solid black">
														<fo:block-container overflow="hidden">
															<fo:block font-weight="bold">
																<xsl:value-of
																	select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_comments" />
															</fo:block>
														</fo:block-container>
													</fo:table-cell>
													<fo:table-cell padding="3pt" border="0.5pt solid black">
														<fo:block-container overflow="hidden">
															<fo:block font-weight="bold">
																<xsl:value-of
																	select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate" />
															</fo:block>
														</fo:block-container>
													</fo:table-cell>
												</fo:table-row>
											</fo:table-header>
											<fo:table-body>
												<xsl:apply-templates
													select="../ns2:approvals/ns2:approvalDescriptor" />
											</fo:table-body>
										</fo:table>
									</xsl:if>
	
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-REQ-TITLE'">
	
									<xsl:if test="../ns2:requirement">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelRequirements" />
										</fo:block>
										<xsl:choose>
											<!-- COMPOSITE RECURSIVE PRINT -->
											<xsl:when
												test="(../ns2:requirement/fo:wrapper/fo:block) and (../ns2:requirement/fo:wrapper/fo:table-row)">
												<fo:table border="0.5pt solid black" font-size="8pt">
													<fo:table-header>
														<fo:table-row background-color="rgb(249,245,219)">
															<fo:table-cell width="7mm" padding="3pt" border="0.5pt solid black">
																<fo:block-container overflow="hidden">
																	<fo:block font-weight="bold">
																		<xsl:value-of
																			select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelId" />
																	</fo:block>
																</fo:block-container>
															</fo:table-cell>
															<fo:table-cell width="20mm" padding="3pt" border="0.5pt solid black">
																<fo:block-container overflow="hidden">
																	<fo:block font-weight="bold">
																		<xsl:value-of
																			select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_status" />
																	</fo:block>
																</fo:block-container>
															</fo:table-cell>
															<fo:table-cell padding="3pt" border="0.5pt solid black">
																<fo:block-container overflow="hidden">
																	<fo:block font-weight="bold">
																		<xsl:value-of
																			select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelName" />
																	</fo:block>
																</fo:block-container>
															</fo:table-cell>
															<fo:table-cell padding="3pt" border="0.5pt solid black">
																<fo:block-container overflow="hidden">
																	<fo:block font-weight="bold">
																		<xsl:value-of
																			select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription" />
																	</fo:block>
																</fo:block-container>
															</fo:table-cell>
															<fo:table-cell width="30mm" padding="3pt" border="0.5pt solid black">
																<fo:block-container overflow="hidden">
																	<fo:block font-weight="bold">
																		<xsl:value-of
																			select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwner" />
																	</fo:block>
																</fo:block-container>
															</fo:table-cell>
														</fo:table-row>
													</fo:table-header>
													<fo:table-body>
														<xsl:copy-of select="../ns2:requirement/fo:wrapper/fo:table-row" />
													</fo:table-body>
												</fo:table>
												<xsl:for-each select="../ns2:requirement/fo:wrapper/fo:block">
													<xsl:choose>
														<xsl:when test="//pageBreaks = 'false'">
															<fo:block>
																<fo:leader/>
																<xsl:copy-of select="." />
															</fo:block>
														</xsl:when>
														<xsl:otherwise>
															<fo:block break-before="page">
																<xsl:if test="position()=last()">
																	<xsl:attribute name="break-after">page</xsl:attribute>
																</xsl:if>
																<xsl:copy-of select="." />
															</fo:block>
														</xsl:otherwise>
													</xsl:choose>
												</xsl:for-each>
											</xsl:when>
											<!--									COMPOSITE SUMMARY VIEW-->
											<xsl:when test="../ns2:requirement/fo:table">
												<fo:table border="0.5pt solid black" font-size="8pt">
													<xsl:copy-of select="../ns2:requirement/fo:table/fo:table-header" />
													<fo:table-body>
														<xsl:copy-of
															select="../ns2:requirement/fo:table/fo:table-body/fo:table-row" />
														<xsl:copy-of select="../ns2:requirement/fo:table-row" />
													</fo:table-body>
												</fo:table>
											</xsl:when>
											<!--										COMPOSITE STAND ALONE-->
											<xsl:when test="../ns2:requirement/fo:block">
												<xsl:copy-of select="../ns2:requirement/fo:block" />
											</xsl:when>
										</xsl:choose>
									</xsl:if>
	
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-PRE-COND-TITLE'">
	
									<xsl:if
										test="../ns2:com.ibm.rqm.planning.editor.section.testCasePreCondition">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelPreConditions" />
										</fo:block>
										<fo:block>
											<xsl:apply-templates
												select="../ns2:com.ibm.rqm.planning.editor.section.testCasePreCondition" />
										</fo:block>
									</xsl:if>
	
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-POST-COND-TITLE'">
									<xsl:if
										test="../ns2:com.ibm.rqm.planning.editor.section.testCasePostCondition">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelPostConditions" />
										</fo:block>
										<fo:block>
											<xsl:apply-templates
												select="../ns2:com.ibm.rqm.planning.editor.section.testCasePostCondition" />
										</fo:block>
									</xsl:if>
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-EXP-RESULTS-TITLE'">
	
									<xsl:if
										test="../ns2:com.ibm.rqm.planning.editor.section.testCaseExpectedResults">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/RQM-KEY-TC-EXP-RESULTS-TITLE" />
										</fo:block>
										<fo:block>
											<xsl:apply-templates
												select="../ns2:com.ibm.rqm.planning.editor.section.testCaseExpectedResults" />
										</fo:block>
									</xsl:if>
	
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-SCRIPTS-TITLE'">
									<xsl:if test="../ns2:testscript">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestScripts" />
										</fo:block>
										<xsl:choose>
											<!-- COMPOSITE RECURSIVE PRINT -->
											<xsl:when
												test="(../ns2:testscript/fo:wrapper/fo:block) and (../ns2:testscript/fo:wrapper/fo:table-row)">
												<fo:table xsl:use-attribute-sets="artifact-table">
													<fo:table-header xsl:use-attribute-sets="artifact-table-header">
														<fo:table-cell xsl:use-attribute-sets="border id-header">
															<fo:block><xsl:value-of select="$strings/labelId" /></fo:block>
														</fo:table-cell>
														<fo:table-cell xsl:use-attribute-sets="border">
															<fo:block><xsl:value-of select="$strings/labelName" /></fo:block>
														</fo:table-cell>
														<fo:table-cell xsl:use-attribute-sets="border">
															<fo:block><xsl:value-of select="$strings/labelState" /></fo:block>
														</fo:table-cell>
														<fo:table-cell xsl:use-attribute-sets="border">
															<fo:block><xsl:value-of select="$strings/labelScriptType" /></fo:block>
														</fo:table-cell>
														<fo:table-cell xsl:use-attribute-sets="border timestamp-header">
															<fo:block><xsl:value-of select="$strings/labelModifiedDate" /></fo:block>
														</fo:table-cell>
													</fo:table-header>
													<fo:table-body>
														<xsl:copy-of select="../ns2:testscript/fo:wrapper/fo:table-row" />
													</fo:table-body>
												</fo:table>
												<xsl:for-each select="../ns2:testscript/fo:wrapper/fo:block">
													<xsl:choose>
														<xsl:when test="//pageBreaks = 'false'">
															<fo:block>
																<fo:leader/>
																<xsl:copy-of select="." />
															</fo:block>
														</xsl:when>
														<xsl:otherwise>
															<fo:block break-before="page">
																<xsl:if test="position()=last()">
																	<xsl:attribute name="break-after">page</xsl:attribute>
																</xsl:if>
																<xsl:copy-of select="." />
															</fo:block>
														</xsl:otherwise>
													</xsl:choose>
												</xsl:for-each>
											</xsl:when>
											<!--									COMPOSITE SUMMARY VIEW-->
											<xsl:when test="../ns2:testscript/fo:block/fo:table">
												<fo:table border="0.5pt solid black" font-size="8pt">
													<xsl:copy-of select="../ns2:testscript/fo:block/fo:table/fo:table-header" />
													<fo:table-body>
														<xsl:copy-of
															select="../ns2:testscript/fo:block/fo:table/fo:table-body/fo:table-row" />
														<xsl:copy-of select="../ns2:testscript/fo:table-row" />
													</fo:table-body>
												</fo:table>
											</xsl:when>
											<!--										COMPOSITE STAND ALONE-->
											<xsl:when test="ns2:testscript/fo:block">
												<xsl:copy-of select="ns2:testscript/fo:block" />
											</xsl:when>
										</xsl:choose>
									</xsl:if>
	
								</xsl:when>
								
								<!-- EXECUTION WORK ITEMS (TERS) -->
								<xsl:when test="@name='RQM-KEY-EXE-EWIS-TITLE'">
									<xsl:if test="../ns2:executionworkitems/ns2:executionworkitem">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelEWIs"/>
										</fo:block>
										<xsl:choose>
											<!-- COMPOSITE RECURSIVE PRINT -->
											<xsl:when test="(../ns2:executionworkitems/ns2:executionworkitem/fo:wrapper/fo:block) and (../ns2:executionworkitems/ns2:executionworkitem/fo:wrapper/fo:table-row)">
												<fo:table xsl:use-attribute-sets="artifact-table">
													<fo:table-header xsl:use-attribute-sets="artifact-table-header">
														<!-- name -->
														<fo:table-cell xsl:use-attribute-sets="border">
															<fo:block><xsl:value-of select="$strings/labelName"/></fo:block>
														</fo:table-cell>
														<!-- test case -->
														<fo:table-cell xsl:use-attribute-sets="border">
															<fo:block><xsl:value-of select="$strings/labelTestCase"/></fo:block>
														</fo:table-cell>
														<!-- test script -->
														<fo:table-cell xsl:use-attribute-sets="border">
															<fo:block><xsl:value-of select="$strings/labelTestScript"/></fo:block>
														</fo:table-cell>
														<!-- last result -->
														<fo:table-cell xsl:use-attribute-sets="border" width="47">
															<fo:block><xsl:value-of select="$strings/labelLastResult"/></fo:block>
														</fo:table-cell>
														<!-- modified date -->
														<fo:table-cell xsl:use-attribute-sets="border timestamp-header" width="45">
															<fo:block><xsl:value-of select="$strings/labelModifiedDate"/></fo:block>
														</fo:table-cell>
														<!-- weight -->
														<fo:table-cell xsl:use-attribute-sets="border" width="30">
															<fo:block><xsl:value-of select="$strings/labelWeight"/></fo:block>
														</fo:table-cell>
													</fo:table-header>
													<fo:table-body>
														<xsl:copy-of select="../ns2:executionworkitems/ns2:executionworkitem/fo:wrapper/fo:table-row"/>
													</fo:table-body>
												</fo:table>
												<xsl:for-each select="../ns2:executionworkitems/ns2:executionworkitem/fo:wrapper/fo:block">
													<xsl:choose>
														<xsl:when test="//pageBreaks = 'false'">
															<fo:block>
																<fo:leader/>
																<xsl:copy-of select="." />
															</fo:block>
														</xsl:when>
														<xsl:otherwise>
															<fo:block break-before="page">
																<xsl:if test="position()=last() and (//ns2:attachment or //ns2:*[contains(local-name(),'dynamic')])">
																	<xsl:attribute name="break-after">page</xsl:attribute>
																</xsl:if>
																<xsl:copy-of select="." />
															</fo:block>
														</xsl:otherwise>
													</xsl:choose>
												</xsl:for-each>
											</xsl:when>
											
											<!-- COMPOSITE SUMMARY VIEW -->
											<xsl:when test="../ns2:executionworkitems/ns2:executionworkitem/fo:table">
												<fo:table xsl:use-attribute-sets="artifact-table">
													<xsl:copy-of select="../ns2:executionworkitems/ns2:executionworkitem/fo:table/fo:table-header"/>
													<fo:table-body>
														<xsl:copy-of select="../ns2:executionworkitems/ns2:executionworkitem/fo:table/fo:table-body/fo:table-row"/>
														<xsl:copy-of select="../ns2:executionworkitems/ns2:executionworkitem/fo:table-row"/>
													</fo:table-body>
												</fo:table>
											</xsl:when>
											
											<!-- COMPOSITE STAND ALONE -->
											<xsl:when test="../ns2:executionworkitems/ns2:executionworkitem/fo:block">
												<xsl:copy-of select="../ns2:executionworkitems/ns2:executionworkitem/fo:block"/>
											</xsl:when>
										</xsl:choose>
									</xsl:if>
								</xsl:when>
								
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-ATTACHMENTS-TITLE'">
								
								<xsl:if test="../ns2:attachment">
									<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
										text-decoration="underline" text-align="left" keep-with-next="always">
										<xsl:value-of
											select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelAttachments" />
									</fo:block>
										
									<fo:table xsl:use-attribute-sets="artifact-table">
										<fo:table-header xsl:use-attribute-sets="artifact-table-header">
											<!-- id -->
											<fo:table-cell xsl:use-attribute-sets="border id-header">
												<fo:block><xsl:value-of select="$strings/labelId"/></fo:block>
											</fo:table-cell>
											<!-- file name -->
											<fo:table-cell xsl:use-attribute-sets="border">
												<fo:block><xsl:value-of select="$strings/labelFileName"/></fo:block>	
											</fo:table-cell>
											<!-- label size -->
											<fo:table-cell xsl:use-attribute-sets="border" width="50">
												<fo:block><xsl:value-of select="$strings/labelSize"/></fo:block>
											</fo:table-cell>
										</fo:table-header>
										<fo:table-body>
											<xsl:apply-templates select="../ns2:attachment"/>
										</fo:table-body>
									</fo:table>
								</xsl:if>
								
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-WI-SECTION-TITLE'">
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-RISK-TITLE'">
	
									<xsl:if test="../ns2:risk">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of select="$strings/labelRiskAssessment" />
										</fo:block>
										<fo:block space-after="4mm">
											<xsl:apply-templates
												select="../ns2:com.ibm.rqm.risk.editor.section.planAssessment" />
										</fo:block>
										<fo:block space-after="4mm">
											<xsl:value-of select="$strings/labelRiskAssessment" />:
											<xsl:value-of select="../ns2:risk/ns2:riskLevel" />
										</fo:block>
										<xsl:if test="../ns2:risk/ns2:factors/ns2:factor">
											<fo:table space-after="4mm" font-size="8pt">
												<fo:table-header>
													<fo:table-row background-color="rgb(249,245,219)">
														<fo:table-cell padding="3pt" border="0.5pt solid black">
															<fo:block-container overflow="hidden">
																<fo:block font-weight="bold">
																	<xsl:value-of select="$strings/labelRisk" />
																</fo:block>
															</fo:block-container>
														</fo:table-cell>
														<fo:table-cell padding="3pt" border="0.5pt solid black">
															<fo:block-container overflow="hidden">
																<fo:block font-weight="bold">
																	<xsl:value-of select="$strings/labelImportance" />
																</fo:block>
															</fo:block-container>
														</fo:table-cell>
													</fo:table-row>
												</fo:table-header>
												<fo:table-body>
													<xsl:for-each select="../ns2:risk/ns2:factors/ns2:factor">
														<fo:table-row>
															<fo:table-cell padding="3pt" border="0.5pt solid black">
																<fo:block-container overflow="hidden">
																	<fo:block>
																		<xsl:apply-templates select="ns2:name" />
																	</fo:block>
																</fo:block-container>
															</fo:table-cell>
															<fo:table-cell padding="3pt" border="0.5pt solid black">
																<fo:block-container overflow="hidden">
																	<fo:block>
																		<xsl:value-of select="ns2:weight" />
																	</fo:block>
																</fo:block-container>
															</fo:table-cell>
														</fo:table-row>
													</xsl:for-each>
												</fo:table-body>
											</fo:table>
										</xsl:if>
										<xsl:if test="../ns2:risk/ns2:commnunityAssessment">
											<fo:table font-size="8pt">
												<fo:table-column column-width="25mm" />
												<fo:table-column column-width="25mm" />
												<fo:table-body>
													<fo:table-row>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of select="$strings/veryHigh"/>:
															</fo:block>
														</fo:table-cell>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of
																	select="../ns2:risk/ns2:commnunityAssessment/ns2:veryHighAmount" />
															</fo:block>
														</fo:table-cell>
													</fo:table-row>
													<fo:table-row>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of select="$strings/high"/>:
															</fo:block>
														</fo:table-cell>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of
																	select="../ns2:risk/ns2:commnunityAssessment/ns2:highAmount" />
															</fo:block>
														</fo:table-cell>
													</fo:table-row>
													<fo:table-row>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of select="$strings/neutral"/>:
															</fo:block>
														</fo:table-cell>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of
																	select="../ns2:risk/ns2:commnunityAssessment/ns2:neutralAmount" />
															</fo:block>
														</fo:table-cell>
													</fo:table-row>
													<fo:table-row>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of select="$strings/low"/>:
															</fo:block>
														</fo:table-cell>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of
																	select="../ns2:risk/ns2:commnunityAssessment/ns2:lowAmount" />
															</fo:block>
														</fo:table-cell>
													</fo:table-row>
													<fo:table-row>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of select="$strings/veryLow"/>:
															</fo:block>
														</fo:table-cell>
														<fo:table-cell>
															<fo:block>
																<xsl:value-of
																	select="../ns2:risk/ns2:commnunityAssessment/ns2:veryLowAmount" />
															</fo:block>
														</fo:table-cell>
													</fo:table-row>
												</fo:table-body>
											</fo:table>
										</xsl:if>
									</xsl:if>
	
								</xsl:when>
								<!--
	
									This will print the summary section of the testplan
								-->
								<xsl:when test="@name='RQM-KEY-TC-NOTES-TITLE'">
								
								
									<xsl:if
										test="../ns2:com.ibm.rqm.planning.editor.section.testCaseNotes">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of select="$strings/Notes"/>:
										</fo:block>
										<fo:block space-after="4mm">
											<xsl:apply-templates
												select="../ns2:com.ibm.rqm.planning.editor.section.testCaseNotes" />
										</fo:block>
									</xsl:if>
								
								
								</xsl:when>
	
								<!--
	
									This will print any custom formatted sections
								-->
								<xsl:otherwise>
								
								<xsl:if test="../ns2:*[@extensionDisplayName=current()/@name]">
										<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
											text-decoration="underline" text-align="left" keep-with-next="always">
											<xsl:value-of select="current()/@name" />
										</fo:block>
										<fo:block>
											<xsl:apply-templates
												select="../ns2:*[@extensionDisplayName=current()/@name]" />
										</fo:block>
									</xsl:if>
								
								</xsl:otherwise>
							</xsl:choose>
						</xsl:for-each>
	
	
						<xsl:if test="ns2:workitem">
							<fo:block space-before="4mm" space-after="4mm" font-weight="bold" 
								text-decoration="underline" text-align="left" keep-with-next="always">
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelWorkItems" />
							</fo:block>
							<xsl:choose>
								<!--								COMPOSITE RECURSIVE PRINT-->
								<xsl:when
									test="(ns2:workitem/fo:wrapper/fo:block) and (ns2:workitem/fo:wrapper/fo:table-row)">
									<fo:table border="0.5pt solid black" font-size="8pt">
										<fo:table-header>
											<fo:table-row 
												background-color="rgb(249,245,219)">
												<fo:table-cell width="25mm" padding="3pt" border="0.5pt solid black">
													<fo:block-container overflow="hidden">
														<fo:block font-weight="bold">
															<xsl:value-of
																select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelType" />
														</fo:block>
													</fo:block-container>
												</fo:table-cell>
												<fo:table-cell width="7mm" padding="3pt" border="0.5pt solid black">
													<fo:block-container overflow="hidden">
														<fo:block font-weight="bold">
															<xsl:value-of
																select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelId" />
														</fo:block>
													</fo:block-container>
												</fo:table-cell>
												<fo:table-cell padding="3pt" border="0.5pt solid black">
													<fo:block-container overflow="hidden">
														<fo:block font-weight="bold">
															<xsl:value-of
																select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary" />
														</fo:block>
													</fo:block-container>
												</fo:table-cell>
												<fo:table-cell width="20mm" padding="3pt" border="0.5pt solid black">
													<fo:block-container overflow="hidden">
														<fo:block font-weight="bold">
															<xsl:value-of
																select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelState" />
														</fo:block>
													</fo:block-container>
												</fo:table-cell>
											</fo:table-row>
										</fo:table-header>
										<fo:table-body>
											<xsl:copy-of select="ns2:workitem/fo:wrapper/fo:table-row" />
										</fo:table-body>
									</fo:table>
									<xsl:for-each select="ns2:tworkitem/fo:wrapper/fo:block">
										<xsl:choose>
											<xsl:when test="//pageBreaks = 'false'">
												<fo:block>
													<fo:leader/>
													<xsl:copy-of select="." />
												</fo:block>
											</xsl:when>
											<xsl:otherwise>
												<fo:block break-before="page">
													<xsl:if test="position()=last()">
														<xsl:attribute name="break-after">page</xsl:attribute>
													</xsl:if>
													<xsl:copy-of select="." />
												</fo:block>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:for-each>
								</xsl:when>
								<!--									COMPOSITE SUMMARY VIEW-->
								<xsl:when test="ns2:workitem/fo:table">
									<fo:table border="0.5pt solid black" font-size="8pt">
										<xsl:copy-of select="ns2:workitem/fo:table/fo:table-header" />
										<fo:table-body>
											<xsl:copy-of
												select="ns2:workitem/fo:table/fo:table-body/fo:table-row" />
											<xsl:copy-of select="ns2:workitem/fo:table-row" />
										</fo:table-body>
									</fo:table>
								</xsl:when>
								<!--										COMPOSITE STAND ALONE-->
								<xsl:when test="ns2:workitem/fo:block">
									<xsl:copy-of select="ns2:workitem/fo:block" />
								</xsl:when>
							</xsl:choose>
						</xsl:if>
					</fo:block>
				</fo:wrapper>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- attachments table body -->
	<xsl:template match="ns2:attachment">
		<xsl:if test="name">
			<fo:table-row>
				<!-- id -->
				<fo:table-cell xsl:use-attribute-sets="border id-cell">
					<fo:block><xsl:value-of select="id"/></fo:block>
				</fo:table-cell>
				<!-- file name -->
				<fo:table-cell xsl:use-attribute-sets="border">
					<fo:block>
						<fo:basic-link external-destination="{download-link}" xsl:use-attribute-sets="simple-link">
							<xsl:apply-templates select="name"/>
						</fo:basic-link>
					</fo:block>
				</fo:table-cell>
				<!-- size -->
				<fo:table-cell xsl:use-attribute-sets="border" text-align="right">
					<fo:block><xsl:value-of select="size"/></fo:block>
				</fo:table-cell>
			</fo:table-row>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="ns2:approvals/ns2:approvalDescriptor">
		<fo:table-row font-size="8pt">
			<fo:table-cell padding="3pt" border="0.5pt solid black">
				<fo:block-container overflow="hidden">
					<fo:block>
						<xsl:choose>
							<xsl:when
								test="ns2:approvalType='com.ibm.team.workitem.approvalType.approval'">
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelApprovalApprovalTable" />
							</xsl:when>
							<xsl:when
								test="ns2:approvalType='com.ibm.team.workitem.approvalType.review'">
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelReviewerApprovalTable" />
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
							</xsl:otherwise>
						</xsl:choose>
					</fo:block>
				</fo:block-container>
			</fo:table-cell>
			<fo:table-cell padding="3pt" border="0.5pt solid black">
				<fo:block-container overflow="hidden">
					<fo:block>
						<xsl:call-template name="wrap-on-all-characters">
							<xsl:with-param name="text-content"
								select="ns2:approval/ns2:approvalOwner" />
						</xsl:call-template>
					</fo:block>
				</fo:block-container>
			</fo:table-cell>
			<fo:table-cell padding="3pt" border="0.5pt solid black">
				<fo:block-container overflow="hidden">
					<fo:block>
						<xsl:choose>
							<xsl:when
								test="ns2:approval/ns2:approvalStatus='com.ibm.team.workitem.approvalState.approved'">
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/com.ibm.rqm.planning.common.approved" />
							</xsl:when>
							<xsl:when
								test="ns2:approval/ns2:approvalStatus='com.ibm.team.workitem.approvalState.pending'">
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/com.ibm.rqm.planning.common.new" />
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
							</xsl:otherwise>
						</xsl:choose>
					</fo:block>
				</fo:block-container>
			</fo:table-cell>
			<fo:table-cell padding="3pt" border="0.5pt solid black">
				<fo:block-container overflow="hidden">
					<fo:block>
						<xsl:value-of select="comment/content" />
					</fo:block>
				</fo:block-container>
			</fo:table-cell>
			<fo:table-cell padding="3pt" border="0.5pt solid black">
				<fo:block-container overflow="hidden">
					<fo:block>
						<xsl:value-of select="comment/date" />
					</fo:block>
				</fo:block-container>
			</fo:table-cell>
		</fo:table-row>
	</xsl:template>
	
	<!-- state -->
	<xsl:template match="ns3:state">
		<xsl:choose>
			<xsl:when test="current()='com.ibm.rqm.planning.common.new'">
				<!-- new -->
				<xsl:apply-templates select="$strings/com.ibm.rqm.planning.common.new"/></xsl:when>
			<xsl:when test="current()='com.ibm.rqm.planning.common.underreview'">
				<!-- under review -->
				<xsl:apply-templates select="$strings/com.ibm.rqm.planning.common.underreview"/></xsl:when>
			<xsl:when test="current()='com.ibm.rqm.planning.common.approved'">
				<!-- approved -->
				<xsl:apply-templates select="$strings/com.ibm.rqm.planning.common.approved"/></xsl:when>
			<xsl:when test="current()='com.ibm.rqm.planning.common.retired'">
				<!-- retired -->
				<xsl:apply-templates select="$strings/com.ibm.rqm.planning.common.retired"/></xsl:when>
			<xsl:otherwise>
				<!-- unassigned -->
				<xsl:apply-templates select="$strings/labelUnassigned"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- name -->
	<xsl:template match="ns2:name">
		<!-- FIXME: For RQM 3.0, strip whitespace for label and make tag name
			case sensitive. -->
		<xsl:choose>
			<xsl:when test="current()='Test Case Weight'">
				<!-- Test Case Weight -->
				<xsl:apply-templates select="$strings/testCaseWeight"/></xsl:when>
			<xsl:when test="current()='Insufficient test resources'">
				<!-- Insufficient test resources -->
				<xsl:apply-templates select="$strings/insufficientTestResources"/></xsl:when>
			<xsl:when test="current()='Insufficient test configurations'">
				<!-- Insufficient test configurations -->
				<xsl:apply-templates select="$strings/insufficientTestConfigurations"/></xsl:when>
			<xsl:when test="current()='Insufficient static review in design'">
				<!-- Insufficient static review in design -->
				<xsl:apply-templates select="$strings/insufficientStaticReviewInDesign"/></xsl:when>
			<xsl:when test="current()='Lack of skills'">
				<!-- Lack of skills -->
				<xsl:apply-templates select="$strings/lackOfSkills"/></xsl:when>
			<xsl:when test="current()='New technology'">
				<!-- New technology -->
				<xsl:apply-templates select="$strings/newTechnology"/></xsl:when>
			<xsl:when test="current()='Aggressive schedule'">
				<!-- Aggressive schedule -->
				<xsl:apply-templates select="$strings/aggressiveSchedule"/></xsl:when>
			<xsl:when test="current()='Inexperienced team'">
				<!-- Inexperienced team -->
				<xsl:apply-templates select="$strings/inexperiencedTeam"/></xsl:when>
			<xsl:when test="current()='Large development team'">
				<!-- Large development team -->
				<xsl:apply-templates select="$strings/largeDevelopmentTeam"/></xsl:when>
			<xsl:when test="current()='Geographically distributed team'">
				<!-- Geographically distributed team -->
				<xsl:apply-templates select="$strings/geographicallyDistributedTeam"/></xsl:when>
			<xsl:when test="current()='Historically problematic area'">
				<!-- Historically problematic area -->
				<xsl:apply-templates select="$strings/historicallyProblematicArea"/></xsl:when>
			<xsl:when test="current()='Lack of formal development process'">
				<!-- Lack of formal development process -->
				<xsl:apply-templates select="$strings/lackOfFormalDevelopmentProcess"/></xsl:when>
			<xsl:when test="current()='Complex System integration involved'">
				<!-- Complex System integration involved -->
				<xsl:apply-templates select="$strings/complexSystemIntegrationInvolved"/></xsl:when>
			<xsl:when test="current()='Insufficient developer test'">
				<!-- Insufficient developer test -->
				<xsl:apply-templates select="$strings/insufficientDeveloperTest"/></xsl:when>
			<xsl:when test="current()='Loss of revenue'">
				<!-- Loss of revenue -->
				<xsl:apply-templates select="$strings/lossOfRevenue"/></xsl:when>
			<xsl:when test="current()='Damage to third parties'">
				<!-- Damage to third parties -->
				<xsl:apply-templates select="$strings/damageToThirdParties"/></xsl:when>
			<xsl:when test="current()='Economic Loss'">
				<!-- Economic Loss -->
				<xsl:apply-templates select="$strings/economicLoss"/></xsl:when>
			<xsl:when test="current()='Physical damage'">
				<!-- Physical damage -->
				<xsl:apply-templates select="$strings/physicalDamage"/></xsl:when>
			<xsl:when test="current()='Costs of corrections and repair'">
				<!-- Costs of corrections and repair -->
				<xsl:apply-templates select="$strings/costsOfCorrectionsAndRepair"/></xsl:when>
			<xsl:when test="current()='Damage to reputation'">
				<!-- Damage to reputation -->
				<xsl:apply-templates select="$strings/damageToReputation"/></xsl:when>
			<xsl:when test="current()='Loss of customer trust'">
				<!-- Loss of customer trust -->
				<xsl:apply-templates select="$strings/lossOfCustomerTrust"/></xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- category -->
	<xsl:template match="ns2:category">
		<xsl:choose>
			<xsl:when test="@term=$strings/labelCategory">
				<xsl:apply-templates select="$strings/labelCategory/@value"/></xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="$strings/labelUnassigned"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
</xsl:stylesheet>
