<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:atom="http://www.w3.org/2005/Atom"
    xmlns:fox="http://xml.apache.org/fop/extensions"
    xmlns:ns10="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
    xmlns:ns11="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
    xmlns:ns2="http://schema.ibm.com/vega/2008/" xmlns:ns3="http://purl.org/dc/elements/1.1/"
    xmlns:ns4="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:ns5="http://jazz.net/xmlns/alm/qm/qmadapter/task/v0.1"
    xmlns:ns6="http://jazz.net/xmlns/alm/qm/qmadapter/v0.1"
    xmlns:ns7="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:ns8="http://jazz.net/xmlns/alm/qm/v0.1/catalog/v0.1"
    xmlns:ns9="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
    xmlns:ts="http://jazz.net/xmlns/alm/qm/v0.1/"
    exclude-result-prefixes="ns9 ns8 ns7 ns6 ns5 ns4 ns3 ns2 ts xsl">
    <xsl:include href="xhtml-to-xslfo-ns.xsl"/>
    <xsl:include href="wrapping-rules.xsl"/>
    <xsl:include href="attribute-sets.xsl"/>
    
    <!-- variables -->
    <xsl:variable name="lang" select="locale"/>
    <xsl:variable name="strings" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]"/>
    <xsl:variable name="font" select="font"/>
    
    <xsl:template match="transformArgs">
        <xsl:choose>
            <xsl:when test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
                <fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
                    initial-page-number="1" force-page-count="no-force"
                    master-reference="PageMaster">
                    <fo:static-content flow-name="xsl-region-before">
                        <fo:block font-size="8pt" text-align="right">
                            <fo:page-number/> / <fo:page-number-citation ref-id="{reqNumber}"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:static-content flow-name="xsl-region-after">
                    	<fo:block-container height="0.1in" padding-after="0.07in">
							<fo:block text-align="center">
								<fo:leader leader-pattern='rule' rule-thickness='1pt' leader-length='100%'/>
							</fo:block>
						</fo:block-container>
                        <fo:block text-align="center">
                            <fo:retrieve-marker retrieve-class-name="artifactUri"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:flow flow-name="xsl-region-body">
                        <fo:marker marker-class-name="artifactUri">
                            <fo:block font-size="6pt">
                                <fo:basic-link external-destination="{originUrl}" color="blue"
                                    text-decoration="underline">
                                    <xsl:value-of select="originUrl"/>
                                </fo:basic-link>
                            </fo:block>
                        </fo:marker>
                        <xsl:apply-templates select="descendant::ts:testscript"/>
                        <fo:block id="{reqNumber}"/>
                    </fo:flow>
                </fo:page-sequence>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="descendant::ts:testscript"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ts:testscript">
        <xsl:choose>
            <xsl:when
                test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
                <xsl:choose>
                    <xsl:when
                        test="../../componentType='STAND_ALONE_SUMMARY' or (../../componentType='COMPOSITE_SUMMARY')">
                        <fo:block>
	                        <xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
	                            <fo:block font-size="14pt" space-before="6mm" space-after="2mm">
	                                <xsl:value-of
	                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary"
	                                /> : <xsl:value-of
	                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestScript"
	                                />
	                            </fo:block>
	                        </xsl:if>
	                        <xsl:if test="../../componentType='COMPOSITE_SUMMARY'">
	                            <fo:block font-size="14pt" space-before="6mm" space-after="2mm">
	                            	<fo:block>
	                            		<xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestScript"/> : 
	                            		<xsl:value-of select="ns3:title"/>
	                            	</fo:block>
	                            	<fo:block>
	                            		<xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelDatapool"/> : 
	                            		<xsl:apply-templates select="ts:datapool"/>
	                        		</fo:block>
	                                <xsl:apply-templates select="ts:steps"/> 
	                            </fo:block>
	                        </xsl:if>    
	                        <fo:table xsl:use-attribute-sets="artifact-table">
	                            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
	                                <fo:table-cell xsl:use-attribute-sets="border id-header">
	                                    <fo:block><xsl:value-of select="$strings/labelId" /></fo:block>
	                                </fo:table-cell>
	                                <fo:table-cell xsl:use-attribute-sets="border">
	                                    <fo:block><xsl:value-of select="$strings/labelName" /></fo:block>
	                                </fo:table-cell>
	                                <fo:table-cell xsl:use-attribute-sets="border">
	                                    <fo:block><xsl:value-of select="$strings/labelState" /></fo:block>
	                                </fo:table-cell>
	                                <fo:table-cell xsl:use-attribute-sets="border">
	                                    <fo:block><xsl:value-of select="$strings/labelScriptType" /></fo:block>
	                                </fo:table-cell>
	                                <fo:table-cell xsl:use-attribute-sets="border timestamp-header">
	                                    <fo:block><xsl:value-of select="$strings/labelModifiedDate" /></fo:block>
	                                </fo:table-cell>
	                            </fo:table-header>
	                            <fo:table-body>
	                                <fo:table-row>
	                                    <fo:table-cell xsl:use-attribute-sets="border id-cell">
	                                        <fo:block>
	                                            <xsl:call-template name="wrap-on-all-characters">
	                                                <xsl:with-param name="text-content" select="//generatedId" />	
	                                            </xsl:call-template>	
	                                        </fo:block>
	                                    </fo:table-cell>
	                                    <fo:table-cell xsl:use-attribute-sets="border">
	                                        <fo:block>
	                                            <xsl:call-template name="wrap-on-all-characters">
	                                                <xsl:with-param name="text-content" select="ns3:title"/>
	                                            </xsl:call-template>
	                                        </fo:block>
	                                    </fo:table-cell>
	                                    <fo:table-cell xsl:use-attribute-sets="border">
	                                        <fo:block>
	                                            <xsl:variable name="state" select="ns4:state"/>
	                                            <xsl:call-template name="wrap-on-all-characters">
	                                                <xsl:with-param name="text-content" select="$strings/*[name()=$state]"/>
	                                            </xsl:call-template>
	                                        </fo:block>
	                                    </fo:table-cell>
	                                    <fo:table-cell xsl:use-attribute-sets="border">
	                                        <fo:block>
	                                            <xsl:choose>
	                                                <xsl:when test="ts:scripttype='Manual'">
	                                                    <xsl:call-template name="wrap-on-all-characters">
	                                                        <xsl:with-param name="text-content" select="$strings/manual" />
	                                                    </xsl:call-template>
	                                                </xsl:when>
	                                                <xsl:otherwise>
	                                                    <xsl:call-template name="wrap-on-all-characters">
	                                                        <xsl:with-param name="text-content" select="ts:scripttype" />
	                                                    </xsl:call-template>
	                                                </xsl:otherwise>
	                                            </xsl:choose>
	                                        </fo:block>
	                                    </fo:table-cell>
	                                    <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
	                                        <fo:block><xsl:value-of select="ns4:updated"/></fo:block>
	                                    </fo:table-cell>
	                                </fo:table-row>
	                            </fo:table-body>
	                        </fo:table>
                        </fo:block>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row>
                            <fo:table-cell xsl:use-attribute-sets="border id-cell">
                                <fo:block>
                                    <xsl:call-template name="wrap-on-all-characters">
                                        <xsl:with-param name="text-content" select="//generatedId" />	
                                    </xsl:call-template>	
                                </fo:block>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block>
                                    <xsl:call-template name="wrap-on-all-characters">
                                        <xsl:with-param name="text-content" select="ns3:title"/>
                                    </xsl:call-template>
                                </fo:block>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block>
                                    <xsl:variable name="state" select="ns4:state"/>
                                    <xsl:call-template name="wrap-on-all-characters">
                                        <xsl:with-param name="text-content" select="$strings/*[name()=$state]"/>
                                    </xsl:call-template>
                                </fo:block>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block>
                                    <xsl:choose>
                                        <xsl:when test="ts:scripttype='Manual'">
                                            <xsl:call-template name="wrap-on-all-characters">
                                                <xsl:with-param name="text-content" select="$strings/manual" />
                                            </xsl:call-template>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:call-template name="wrap-on-all-characters">
                                                <xsl:with-param name="text-content" select="ts:scripttype" />
                                            </xsl:call-template>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </fo:block>
                            </fo:table-cell>
                            <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                                <fo:block><xsl:value-of select="ns4:updated"/></fo:block>
                            </fo:table-cell>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="(../../componentType='COMPOSITE') and (../../recursive)">
                <fo:wrapper>
                    <fo:table-row>
                        <fo:table-cell xsl:use-attribute-sets="border id-cell">
                            <fo:block>
                                <xsl:variable name="ident" select="*[local-name()='identifier']"/>
                                <xsl:call-template name="wrap-on-all-characters">
                                    <xsl:with-param name="text-content" select="substring-after($ident,'testscript:')"/>
                                </xsl:call-template>
                            </fo:block>
                        </fo:table-cell>
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block>
                                <fo:basic-link
                                    internal-destination="{*[local-name()='identifier']}"
                                    color="blue" text-decoration="underline">
                                    <xsl:call-template name="wrap-on-all-characters">
                                        <xsl:with-param name="text-content" select="ns3:title"/>
                                    </xsl:call-template>
                                </fo:basic-link>
                            </fo:block>
                        </fo:table-cell>
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block>
                                <xsl:variable name="state" select="ns4:state"/>
                                <xsl:call-template name="wrap-on-all-characters">
                                    <xsl:with-param name="text-content" select="$strings/*[name()=$state]"/>
                                </xsl:call-template>
                            </fo:block>
                        </fo:table-cell>
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block>
                                <xsl:choose>
                                    <xsl:when test="ts:scripttype='Manual'">
                                        <xsl:call-template name="wrap-on-all-characters">
                                            <xsl:with-param name="text-content" select="$strings/manual" />
                                        </xsl:call-template>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:call-template name="wrap-on-all-characters">
                                            <xsl:with-param name="text-content" select="ts:scripttype" />
                                        </xsl:call-template>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </fo:block>
                        </fo:table-cell>
                        <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                            <fo:block><xsl:value-of select="ns4:updated"/></fo:block>
                        </fo:table-cell>
                    </fo:table-row>
                    
                    <!-- TEST SCRIPT SUMMARY -->
                    <!-- This is the section that prints out the test script attributes other than steps in a testscript PDF -->
                    <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                        <!-- This part doesn't make any sense and I have no idea what this does so I'm just going to leave it alone... -->
                        <xsl:attribute name="id">
                            <xsl:value-of select="*[local-name()='identifier']"/>
                        </xsl:attribute>
                        
                        <!-- name (page header) -->
                        <!-- Test Script <#>: <name> -->
                        <fo:block font-size="14pt" space-before="2mm" space-after="2mm" border-after-style="solid" border-after-width="thin">
                            <xsl:value-of select="$strings/labelTestScript"/> <xsl:value-of select="//generatedId"/>: <xsl:value-of select="ns3:title"/>
                        </fo:block>
                        
                        <!-- script type -->
                        <!-- Script Type: <type> -->
                        <fo:block space-after="2mm">
                            <xsl:value-of select="$strings/labelScriptType"/>: <xsl:apply-templates select="ts:scripttype"/> 
                        </fo:block>
                        
                        <!-- state -->
                        <fo:block space-after="2mm">
                            <xsl:value-of select="$strings/labelState"/>: <xsl:apply-templates select="ns4:state"/>
                        </fo:block>
                        
                        <!-- datapool -->
                        <!-- Datapool: <datapool name> -->
                         <fo:block space-before="2mm" space-after="2mm">
                            <xsl:value-of select="$strings/labelDatapool"/>: <xsl:apply-templates select="ts:datapool"/>
                         </fo:block>
                        
                        <!-- description -->
                        <fo:block space-after="1mm">
                            <xsl:value-of select="$strings/labeldescription"/>: 
                        </fo:block>
                        <fo:block space-after="4mm" white-space-collapse="false" linefeed-treatment="preserve">
                            <xsl:value-of select="ns3:description"/>
                        </fo:block>
                        
                        <!-- steps -->
                        <xsl:apply-templates select="ts:steps"/>
                    </fo:block>
                    
                </fo:wrapper>
            </xsl:when>
            <xsl:otherwise>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format"> <!--font-size="8pt" ?-->
                    <xsl:if test="(../../componentType='COMPOSITE' )">
                        <xsl:attribute name="id">
                            <xsl:value-of select="ns3:identifier"/>
                        </xsl:attribute>
                        <xsl:if test="//pageBreaks = 'true'">
                        	<xsl:attribute name="break-after">page</xsl:attribute>
                        </xsl:if>
                    </xsl:if>
                    
                    <!-- TEST SCRIPT SUMMARY -->
                    <!-- This is the section that prints out the test script attributes other than steps in a testscript PDF -->
                    <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                        <!-- This part doesn't make any sense and I have no idea what this does so I'm just going to leave it alone... -->
                        <xsl:attribute name="id">
                            <xsl:value-of select="*[local-name()='identifier']"/>
                        </xsl:attribute>
                        
                        <!-- name (page header) -->
                        <!-- Test Script <#>: <name> -->
                        <fo:block font-size="14pt" space-before="2mm" space-after="2mm" border-after-style="solid" border-after-width="thin">
                            <xsl:value-of select="$strings/labelTestScript"/> <xsl:value-of select="//generatedId"/>: <xsl:value-of select="ns3:title"/>
                        </fo:block>
                        
                        <!-- script type -->
                        <!-- Script Type: <type> -->
                        <fo:block space-after="2mm">
                            <xsl:value-of select="$strings/labelScriptType"/>: <xsl:apply-templates select="ts:scripttype"/> 
                        </fo:block>
                        
                        <!-- state -->
                        <fo:block space-after="2mm">
                            <xsl:value-of select="$strings/labelState"/>: <xsl:apply-templates select="ns4:state"/>
                        </fo:block>
                        
                        <!-- datapool -->
                        <!-- Datapool: <datapool name> -->
                        <fo:block space-before="2mm" space-after="2mm">
                            <xsl:value-of select="$strings/labelDatapool"/>: <xsl:apply-templates select="ts:datapool"/>
                        </fo:block>
                        
                        <!-- description -->
                        <fo:block space-after="1mm">
                            <xsl:value-of select="$strings/labeldescription"/>: 
                        </fo:block>
                        <fo:block space-after="4mm" white-space-collapse="false" linefeed-treatment="preserve">
                            <xsl:value-of select="ns3:description"/>
                        </fo:block>
                        
                        <!-- steps -->
                        <xsl:apply-templates select="ts:steps"/>
                    </fo:block>
                    
                </fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ts:steps">
        <xsl:if test="ns9:step">
            <fo:block space-after="4mm">
                <xsl:value-of
                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelScriptSteps"
                />
            </fo:block>
            <fo:table border="0.5pt solid black" font-size="8pt">
                <fo:table-column column-width="6mm"/>
                <fo:table-column column-width="28mm"/>
                <fo:table-column/>
                <fo:table-column/>
                <fo:table-header>
                    <fo:table-cell keep-together.within-column="always"
                        background-color="rgb(222,242,236)">
                        <fo:block-container overflow="hidden">
                            <fo:block font-weight="bold" text-align="left">#</fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                    <fo:table-cell keep-together.within-column="always"
                        background-color="rgb(222,242,236)">
                        <fo:block-container overflow="hidden">
                            <fo:block font-weight="bold" text-align="left">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelType"
                                />
                            </fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                    <fo:table-cell keep-together.within-column="always"
                        background-color="rgb(222,242,236)">
                        <fo:block-container overflow="hidden">
                            <fo:block font-weight="bold" text-align="left">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"
                                />
                            </fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                    <fo:table-cell keep-together.within-column="always"
                        background-color="rgb(222,242,236)">
                        <fo:block-container overflow="hidden">
                            <fo:block font-weight="bold" text-align="left">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelExpectedResult"
                                />
                            </fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                </fo:table-header>
                <fo:table-body>
                    <xsl:apply-templates select="ns9:step"/>
                </fo:table-body>
            </fo:table>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="ns9:step">
        <xsl:choose>
            <xsl:when test="count(ancestor::ns9:step) &gt; 0">
                <fo:block font-size="8pt">
                    <xsl:attribute name="text-indent">
                        <xsl:value-of select="concat(count(ancestor::ns9:step) * .75,'cm')"/>
                    </xsl:attribute>
                    <xsl:apply-templates select="ns9:description"/>
                    <xsl:apply-templates select="ns9:step"/>
                </fo:block>
            </xsl:when>
            <xsl:otherwise>
                <fo:table-row font-size="8pt">
                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                        <fo:block-container overflow="hidden">
                            <fo:block>
                                <xsl:number/>
                            </fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                        <fo:block-container overflow="hidden">	
                            <fo:block text-align="left">
                                <xsl:choose>
                                    <xsl:when test="@type='execution'">
                                    	<xsl:call-template name="wrap-on-all-characters">
                                        	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelExecutionStep" />
                                        </xsl:call-template>
                                    </xsl:when>
                                    <xsl:when test="@type='verification'">
                                    	<xsl:call-template name="wrap-on-all-characters">
                                        	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelVerificationPoint" />
                                    	</xsl:call-template>
                                    </xsl:when>
                                    <xsl:when test="@type='reporting'">
                                    	<xsl:call-template name="wrap-on-all-characters">
                                        	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelReportingStep" />
                                    	</xsl:call-template>
                                    </xsl:when>
                                    <xsl:when test="@type='keyword'">
                                    	<xsl:call-template name="wrap-on-all-characters">
                                        	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelKeywordStep" />
                                    	</xsl:call-template>
                                    </xsl:when>
                                </xsl:choose>
                            </fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                        <xsl:if test="ns9:link/@href">
						 <fo:block>
						  <xsl:apply-templates select="ns9:link/fo:page-sequence/fo:flow"/>
						 </fo:block>
						</xsl:if>
                        <fo:block-container overflow="hidden">
                            <fo:block text-align="left">
                                <xsl:apply-templates select="ns9:description"/>
                                <xsl:apply-templates select="ns9:step"/>
                        </fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                        <fo:block-container overflow="hidden">
                            <fo:block text-align="left">
                                <xsl:apply-templates select="ns9:expectedResult"/>
                            </fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                </fo:table-row>
                
                <!--<xsl:if test="@type='keyword'">
                    <xsl:copy-of select="ns9:link/fo:page-sequence/fo:flow/fo:block/fo:table-body/fo:table-row"/>
                </xsl:if>-->
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ns9:link/fo:page-sequence/fo:flow">
      <fo:block text-align="left">
        <fo:basic-link color="blue" text-decoration="underline">
          <xsl:attribute name="external-destination">
            <xsl:value-of select="../../../ns9:link/@href"/>
          </xsl:attribute>	
          <xsl:value-of select="../../../ns9:link/fo:page-sequence/fo:flow/fo:block"/>
        </fo:basic-link>
      </fo:block>
    </xsl:template>
    
    <!-- script type -->
    <xsl:template match="ts:scripttype">
        <xsl:choose>
            <!-- manual -->
            <xsl:when test="current()='Manual'">
                <xsl:value-of select="$strings/manual"/></xsl:when>
            <!-- default -->
            <xsl:otherwise>
                <xsl:value-of select="ts:scripttype"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- state -->
    <xsl:template match="ns4:state">
        <xsl:choose>
            <!-- under review -->
            <xsl:when test="current()='com.ibm.rqm.planning.common.underreview'">
                <xsl:value-of select="$strings/com.ibm.rqm.planning.common.underreview"/></xsl:when>
            <!-- new -->
            <xsl:when test="current()='com.ibm.rqm.planning.common.new'">                
                <xsl:value-of select="$strings/com.ibm.rqm.planning.common.new"/></xsl:when>
            <!-- approved -->
            <xsl:when test="current()='com.ibm.rqm.planning.common.approved'">
                <xsl:value-of select="$strings/com.ibm.rqm.planning.common.approved"/></xsl:when>
            <!-- default -->
            <xsl:otherwise>
                <xsl:apply-templates/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- datapool -->
    <xsl:template match="ts:datapool">
        <fo:basic-link color="blue" text-decoration="underline">
            <xsl:attribute name="external-destination">
                <xsl:value-of select="fo:page-sequence/fo:flow/fo:marker/fo:block/fo:basic-link"/>
            </xsl:attribute>	
            <xsl:value-of select="../ts:datapool/fo:page-sequence/fo:flow/fo:block"/>
        </fo:basic-link>
    </xsl:template>
    
    <xsl:template match="ns9:*">
        <xsl:apply-templates select="*|text()"/>
    </xsl:template>
    
</xsl:stylesheet>
