/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.client.AbstractOSLCManager");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.defects.web.util.OSLCServicesDocUtil");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.rqm.defects.web.client.OSLCCmProvider");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ServiceProviderDialog");

dojo.require("jazz.util.error");

(function(){
    var INIT_UNINITIALIZED = 0;
    var INIT_DONE = 1;
    var INIT_UNRECOVERABLE = 2;
    var INIT_RECOVERABLE = 3;
    
    var PROVIDER_LINK_TYPE_CQREST = "cqrest";
    
    var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
    var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    
    // object class for cm services like picker dialogs etc 
    dojo.declare("com.ibm.rqm.defects.web.client.OSLCServices", null, {
        _creationDialogs: [],
        _selectionDialogs: [],
        constructor: function(paramObj){
            this._creationDialogs = paramObj.creationDialogs;
            this._selectionDialogs = paramObj.selectionDialogs;
        },
        getCreationDialog: function(id){
            var result = null;
            dojo.forEach(this._creationDialogs, function(creationDialog){
                if (creationDialog.id == id) {
                    result = creationDialog;
                }
            });
            if (result) {
                return result;
            }
            else {
                return this.getDefaultCreationDialog();
            }
        },
        
        getDefaultCreationDialog: function(){
            var result = null;
            dojo.forEach(this._creationDialogs, function(creationDialog){
                if (creationDialog.isDefault) {
                    result = creationDialog;
                }
            });
            if (result) {
                return result;
            }
            else {
                return this._creationDialogs[0];
            }
        },
        
        getSelectionDialog: function(id, isMultipleAllowed){
            var result = null;
            dojo.forEach(this._selectionDialogs, function(selectionDialog){
                if (selectionDialog.id == id) {
                    result = selectionDialog;
                }
            });
            if (!result) {
                result = this.getDefaultSelectionDialog()
            }
            if (isMultipleAllowed && result.uri.indexOf("allowsMultiple=true") == -1) {
                if (result.uri.indexOf('?') != -1) {
                    result.uri = result.uri + '&allowsMultiple=true';
                }
                else {
                    result.uri = result.uri + '?allowsMultiple=true';
                }
            }
            return result;
        },
        
        getDefaultSelectionDialog: function(){
            var result = null;
            dojo.forEach(this._selectionDialogs, function(selectionDialog){
                if (selectionDialog.isDefault) {
                    result = selectionDialog;
                }
            });
            if (result) {
                return result;
            }
            else {
                return this._selectionDialogs[0];
            }
        }
    });
    
    dojo.declare("com.ibm.rqm.defects.web.client.AbstractOSLCManager", null, {
    
        _currentProvider: null,
        
        constructor: function(){
            this._initializationState = INIT_UNINITIALIZED;
        },
        
        /* Abstract methods which the sub class has to provide implementation */
        getAPIConstructor: function(){
        },
        
        getProjectLinkType: function(){
        },
        
        getCurrentServiceProvider: function(callback){
            if (this._initializationState == INIT_UNINITIALIZED || this._initializationState == INIT_RECOVERABLE) {
                var ProgressDialog = com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
                ProgressDialog.doRunWithProgress(DefectsUIUtil.getMessage("labelInitializingDialog"), dojo.hitch(this, function(monitor){
                    monitor.beginTask(DefectsUIUtil.getMessage("msgAboutToInitializeProvider"));
                    this._chooseCurrentProvider(dojo.hitch(this, function(currentProvider){// After a single provider is selected
                        monitor.done();
                        if (currentProvider) {// 1. The selected provider is connected successfully
                            this._currentProvider = currentProvider;
                            this._initializationState = INIT_DONE;
                            // get api implementation 
                            var providerConstructor = this.getAPIConstructor();
                            var api = new providerConstructor(this._currentProvider);
                            this._currentProvider.postLink = dojo.hitch(api, "postLink");
                            this._currentProvider.deleteLink = dojo.hitch(api, "deleteLink");
                            // FIXME - for back compatiable only, but shouldn't use a global constant for state indicator.
                            if (this.getProjectLinkType() == "tests") {
                                com.ibm.rqm.defects.web.client.ChangeRequestManager.INIT_INITIALIZED = true;
                            }
                            else 
                                if (this.getProjectLinkType() == "validates") {
                                }
                            callback(this._currentProvider);
                        }
                        else {// 2. No project linkings are set up or unrecoverable error occurs
                            callback(null);
                        }
                        
                    }), monitor);
                }));
                
            }
            else {
                callback(this._currentProvider);
            }
        },
        
        getServiceProviders: function(callback){
            var dummyMonitor = {};
            dummyMonitor.beginTask = function(label){
            };
            this._initProjectLinks(callback, dummyMonitor);
        },
        
        initServiceProvider: function(projectLink, callback){
            var dummyMonitor = {};
            dummyMonitor.beginTask = function(label){
            };
            this._discoverServices(projectLink, callback, dummyMonitor);
        },
        
        // ---- implementation -------------------------------------------------------------------------------------------------
        
        _chooseCurrentProvider: function(runAfterChosed, monitor){
            this._discoverServices(null, dojo.hitch(this, function(serviceProvider){
                runAfterChosed(serviceProvider);
            }), monitor);
        },
        
        _discoverServices: function(projectLink, runAfterServiceProvidersInitiated, monitor){
            if (projectLink) {
                this._handleDiscoveringServices(monitor, projectLink, runAfterServiceProvidersInitiated);
            }
            else {
                this._initProjectLinks(dojo.hitch(this, function(projectLinks){ // After project linkings are found
                    // if more than one service provider is configured, ask user to select one
                    if (projectLinks && projectLinks.length > 1) {
                        var serviceProviderDialog = new com.ibm.rqm.defects.web.ui.view.common.ServiceProviderDialog({
                            onOk: dojo.hitch(this, function(currrentProvider){
                                this._handleDiscoveringServices(monitor, currrentProvider, runAfterServiceProvidersInitiated);
                            }),
                            onClose: dojo.hitch(this, function(){
                                monitor.done();
                            }),
                            serviceProviders: projectLinks
                        })
                    }
                    else 
                        if (projectLinks && projectLinks.length == 1) {// If only one linking is set up, it will be selected by default
                            this._handleDiscoveringServices(monitor, projectLinks[0], runAfterServiceProvidersInitiated);
                        }
                        else {// No linking is set up
                            runAfterServiceProvidersInitiated(null);
                        }
                }), monitor);
            }
        },
        
        _handleDiscoveringServices: function(monitor, currrentProvider, runAfterServiceProvidersInitiated){
            var projectLink = currrentProvider;
            //FIXME - remove the hack
            if (projectLink.servicesUrl.indexOf(PROVIDER_LINK_TYPE_CQREST) != -1) {
                projectLink.linkType = PROVIDER_LINK_TYPE_CQREST;
            }
            monitor.beginTask(DefectsUIUtil.getMessage("msgDiscoverURIs"));
            var deferred = jazz.client.xhrGet({
                url: projectLink.servicesUrl,
                handleAs: "xml",
                error: dojo.hitch(this, function(error){
                    console.debug(error);
                    monitor.done();
                    this._handleErrorDiscoveringServices(deferred, error);
                }),
                load: dojo.hitch(this, function(result){
                    var services = com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readServiceDescription(result, this.getProjectLinkType());
                    // for back compatiable
                    projectLink.providerUris = services;
                    projectLink.services = new com.ibm.rqm.defects.web.client.OSLCServices({
                        creationDialogs: services.creationDialogs,
                        selectionDialogs: services.selectionDialogs
                    });
                    dojo.mixin(projectLink.services, services);
                    runAfterServiceProvidersInitiated(projectLink);
                })
            });
        },
        
        _handleErrorDiscoveringServices: function(deferred, error){// These errors are all recoverable thus call back won't be invoked.
            this._initializationState = INIT_RECOVERABLE
            // 1. The challenge pop up is blocked or canceled
            // 2. The selected provider is unavailable, e.g, the server is down or proxy setting is incorrect 
            // 3. All other unknown issues
            if (error.dojoType === "popupBlocked" || error.dojoType == "cancel") { //$NON-NLS-1$
                this._showErrorAlert(DefectsUIUtil.getMessage("msgPopupBlockedOrCanceled"));
            }
            else 
                if (error.status == 500) {
                    this._showErrorAlert(DefectsUIUtil.getMessage("msgService500", error.responseText || error.message));
                }
                else {
                    this._showErrorAlert(DefectsUIUtil.getMessage("msgFailedToConfigProvider", error.responseText || error.message || error));
                }
        },
        
        _initProjectLinks: function(runAfterProjectLinkInitiated, monitor){
            monitor.beginTask(DefectsUIUtil.getMessage("msgDiscoverProvider"));
            var linksUrl = location.protocol + "//" + location.host + net.jazz.ajax._contextRoot + "/process/project-areas/" + getCurrentProjectArea().itemId + "/links";
            var self = this;
            jazz.client.xhrGet({
                url: linksUrl,
                handleAs: "xml",
                load: dojo.hitch(self, function(result){
                    this._handleInitiatingProjectLinks(result, runAfterProjectLinkInitiated);
                }),
                error: dojo.hitch(this, function(error){
                    monitor.done();
                    this._handleErrorInitiatingProjectLinks(error);
                })
            });
        },
        
        _handleErrorInitiatingProjectLinks: function(error, callback){// This in general won't happen. 
            this._initializationState = INIT_UNRECOVERABLE;
            this._showErrorAlert(DefectsUIUtil.getMessage("msgFailedToConfigProvider", error));
            callback(null);
        },
        
        _handleInitiatingProjectLinks: function(result, callback){
            var projectLinks = [];
            var links = com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readProjectLinksDescription(result, this.getProjectLinkType());
            dojo.forEach(links, function(link){
                projectLinks.push(link);
            });
            callback(projectLinks);
        },
        
        _showErrorAlert: function(errorMessage){
            var alertDialog = null;
            alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog();
            alertDialog.setMessage(errorMessage);
            alertDialog.open();
        },
        __sentinel: null // terminates this class definition
    });
})();
