/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.client.internal.CategoryClient");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() { //module container

var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var getCurrentPA            = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

com.ibm.rqm.planning.web.client.internal.CategoryClient = {

	CATEGORY_SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.ICategoryRestService",
	CATEGORY_TYPE_SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.ICategoryTypeRestService",

	_invokeCategoryService: function(responseHandler, method, args) {
		if (args && !args.processAreaUUID) {
			args.processAreaUUID = getCurrentPA().itemId;
		}
		var serviceRequest= new ServiceRequest(this.CATEGORY_SERVICE_URI, method, args);
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},

    _invokeCategoryTypeService: function(responseHandler, method, args) {
        if (args && !args.processAreaUUID) {
            args.processAreaUUID = getCurrentPA().itemId;
        }
        var serviceRequest= new ServiceRequest(this.CATEGORY_TYPE_SERVICE_URI, method, args);
        TeamServerClient.invokeService(serviceRequest, responseHandler);
    },
	
	_getItemType: function(/*string*/ modelType) {
        return modelType.substr(modelType.lastIndexOf(".") + 1)		
	},
	
    getCategories: function(serviceResponseHandler, modelType, showArchived) {
		var args = {
			typeItemType: this._getItemType(modelType),
			resolveCategories: true,
			includeArchived: showArchived && showArchived == true ? true : false
		}
        this._invokeCategoryService(serviceResponseHandler, "getCategoriesDTO", args);
    },
	
    getCategoryTypes: function(serviceResponseHandler, modelType, showArchived) {
        var args = {
            itemType: this._getItemType(modelType),
            resolveCategories: true,
            includeArchived: showArchived && showArchived == true ? true : false
        }
        this._invokeCategoryTypeService(serviceResponseHandler, "getCategoryTypesDTO", args);
    },	
	
	getCategoryTypeSummaries: function(serviceResponseHandler, modelType, includeArchived) {
        var args = {
            itemType: this._getItemType(modelType),
            resolveCategories: true,
            includeArchived: includeArchived?includeArchived:false
        }
        this._invokeCategoryTypeService(serviceResponseHandler, "getCategoryTypesSummaryDTO", args);
    },	
	
	getCategoryReferences: function(serviceResponseHandler, modelType, itemId, states) {
        var args = {
            itemType: this._getItemType(modelType),
            itemId: itemId,
            includeArchived: false
        }	
		if (states && states.length) {
			args.states = states;
		}	
		this._invokeCategoryService(serviceResponseHandler, "getCategoryReferences", args);
	},
	   
    postCategory: function(serviceResponseHandler, categoryObj) {
		//categoryObj: newCategory, itemId, name, typeName/typeItemType || typeItemId   
		if (categoryObj.typeItemType) {
			categoryObj.typeItemType = this._getItemType(categoryObj.typeItemType);
		} 
        this._invokeCategoryService(serviceResponseHandler, "postCategoryDTO", categoryObj);
    },
	
    postCategoryType: function(serviceResponseHandler, categoryTypeObj) {
        //categoryObj: newCategoryType, itemId, name, itemType
        if (categoryTypeObj.itemType) {
            categoryTypeObj.itemType = this._getItemType(categoryTypeObj.itemType);
        }		  
        this._invokeCategoryTypeService(serviceResponseHandler, "postCategoryTypeDTO", categoryTypeObj);
    }
};

})();
