/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");

dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.client.internal.user");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.GroupUtil");

//Workaround Jazz dashboard problem of not declaring ServiceResponseHandler in couple of their classes.
//https://jazz.net/jazz/web/projects/Jazz%20Project#action=com.ibm.team.workitem.viewWorkItem&id=48174
//This problem is fixed in Jazz M6, so can remove when upgrade to M6.
dojo.require("com.ibm.team.dashboard.web.ui.IMemento");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var InitializationData     = com.ibm.team.repository.web.client.init.InitializationData;

(function() {

	
// Summary:
//    Initializes common data for the RTLM application. The application
//    must call the initializeData() method first before accessing the
//    cached application data.
//
com.ibm.rational.test.lm.web.client.internal.RtlmInitializer = {

    externalRespHandler: null,
	userList: null,
	userNameList: null,
    
	// Summary:
	//    Main method for this initializer object. This method must
	//    called before accessing other public methods in this class.
	//
	initializeData: function(responseHandler)
	{
        if (!dojo.hasClass(dojo.body(), "com-ibm-rational-asq")) {
            dojo.addClass(dojo.body(), "com-ibm-rational-asq");
        }

        // set the global var for the CS help provider id for RTLM
        dojo.setObject("com.ibm.rational.test.lm.web.CSHelpID", "RTLM");
        // set CS help URL for RTLM CS help provider
        CSHelpEngine.registerCSHelpProvider("RTLM", "/help/rtopic/com.ibm.rational.test.lm.cshelp/topics/");

		// tug away the handler, we're going to let the last
		// data fetcher to use it.
		this.externalRespHandler = responseHandler;
        
        com.ibm.rational.test.lm.web.client.internal.RtlmInitializer._fetchUsers();
	},
	
	// Summary:
	//    Returns the list of users. Each user is a full object with its name and itemId fields.
	//
	getUsers: function()
	{
		return this.userList;
	},
	
    // Summary:
    //    Returns the list of user (displayable) names only. 
    //
    getUserNames: function()
    {
        return this.userNameList;
    },

    _initializeCatalog: function() 
    {
        var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_initCatalogSuccess", "_initCatalogFailure");
        catalog.initializeTemplates(responseHandler);
    },
	_initCatalogSuccess: function(){
		GroupUtil.init(this.externalRespHandler);
	},
    
	_initCatalogFailure: function(response){
        // need to record the error. Otherwise, if the group init correctly,
        // the error is never trapped.
        dojo.setObject("com.ibm.rqm.planning.web.ui.initError", {errorMessage: ((response && response.message) || "")});
		GroupUtil.init(this.externalRespHandler);
	},
    
    _fetchUsers: function()
    {
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_fetchUsersSuccess", "_fetchDataFailure");
        var user = new com.ibm.rational.test.lm.web.client.internal.projectmembers();
        user.getMembers(responseHandler); 
		this._getCurrentUser();
		this._initializeCatalog();
    },

    _fetchDataFailure: function(errorType, errorMessage){},
        
    _fetchUsersSuccess: function(response)
    {
         this.userList = response;
         this.userNameList = [];
         
         for(var i = 0; i < response.length; i++) {
            //filter out archived users
		 	if (!response[i].archived) {
                if (com.ibm.asq.common.web.ui.internal.util.Utility.isUnassignedUserId(response[i].itemId)) {
                    response[i].name = MessageUtil.getMessage("RTLM", "cfgUnassigned");
                }
                this.userNameList.push(response[i].name);
            }
         }
    },
	
    // same as _fetchUsers but not as one of the chain action of init
    updateAllUsers: function(responseHandler)
    {
        var _responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper({
            util: this,
            _successHandler: function (successResponse) {
                this.util.userList = successResponse;
                this.util.userNameList = [];
         
                for (var i = 0; i < successResponse.length; i++) {
                    //filter out archived users
					if (!successResponse[i].archived) {
                        if (com.ibm.asq.common.web.ui.internal.util.Utility.isUnassignedUserId(successResponse[i].itemId)) {
                            successResponse[i].name = MessageUtil.getMessage("RTLM", "cfgUnassigned");
                        }
                        this.util.userNameList.push(successResponse[i].name);
                    }
                }
                // invoke caller's response handler
                responseHandler._contextObject[responseHandler._callbackName](successResponse);
            },
            _failureHandler: function (failureResponse) {
                responseHandler._contextObject[responseHandler._errorCallbackName](failureResponse);
            }
        },
        "_successHandler",
        "_failureHandler"
        );

        var user = new com.ibm.rational.test.lm.web.client.internal.projectmembers();
        user.getMembers(_responseHandler); 
    },
    
    _getCurrentUser: function()
	{        
        // daisy-chain to initialize logged in user
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getUserSuccess", "_getUserFailure");
        var user = new com.ibm.rational.test.lm.web.client.internal.user();
        user.getCurrent(responseHandler);
    },
    
    _getUserFailure: function(errorType, errorMessage)
    {
         
    },
        
    _getUserSuccess: function(response)
	{
        // override the logged in user in the InitializationData with ours which is
        // a subclass of the user object with user role info
        var user = { currentContributor: response };
        var userData = {
            "com.ibm.team.repository.service.currentContributorInitializer": user
        };
        InitializationData._load(userData);
        
        // set the global var for the logged-in contributor
        dojo.setObject(
            "com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", response);

    }
};
 
})();
