/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.asq.common.web.client.internal.SimpleCache");

(function() {
	
var TTL_INFINITE= -1;

dojo.declare("com.ibm.asq.common.web.client.internal.SimpleCache", null, {
	
	maxSize: 50,
	
	constructor: function(args) {
		dojo.mixin(this, args);
		this._cache = [];
	},
	
	get: function(key) {
		var entry = this._findEntry(key);
		if (entry != null) {
			var now = (new Date()).getTime();
			if (entry.ttl == TTL_INFINITE || (entry.timestamp + entry.ttl > now)) {
				return entry.item;
			} else {
				this.clear(key); //expired, remove from cache
			}
		}
		return null;
	},
	
	put: function(key, item, ttl) {
		var entry= { 
			key: key,
			item: item,
			added: (new Date()).getTime(),
			ttl: ttl || TTL_INFINITE
		};
		while (this._cache.length >= this.maxSize) {
			//remove an entry to keep the cache at max size
			this._removeOldest();
		}
		this._cache.push(entry);
		return entry;
	},
	
	getAll: function() {
		var values = [];
		var cache = this._cache;
		var cLen = cache.length;
		for (var i = 0; i < cLen; i++) {
			values.push(this.get(cache[i].key));
		}		
		return values;
	},
	
	remove: function(key) {
		this._removeEntry(key);
	},

	clear: function() {
		delete this._cache;
		this._cache = [];
	},
	
	/*********** Internals ***********/	
	_findEntry: function(key) {
		var index = this._findIndex(key);
		if (index != null) {
			return this._cache[index];
		}
		return null;
	},
	
	_removeEntry: function(key) {
		var index = this._findIndex(key);
		if (index != null) {
			this._cache.splice(index, 1);
		}
	},	
	
	_removeOldest: function() {
		var cache = this._cache;
		var cLen = cache.length;
		var oldest = null;
		for (var i = 0; i < cLen; i++) {
			var entry = cache[i];
			if (oldest == null || (entry.added < oldest.added)) {
				oldest = entry;
			}
		}
		if (oldest != null) {
			this._removeEntry(oldest.key);
		}
	},	
	
	_findIndex: function(key) {
		var cache = this._cache;
		var cLen = cache.length;
		for (var i = 0; i < cLen; i++) {
			var entry = cache[i];
			if (entry.key == key) {
				return i;
			}
		}
		return null;	
	}
	
});

com.ibm.asq.common.web.client.internal.SimpleCache.TTL_INFINITE = TTL_INFINITE;
com.ibm.asq.common.web.client.internal.SimpleCache.TTL_MINUTE = 60000;
com.ibm.asq.common.web.client.internal.SimpleCache.TTL_HOUR = 3600000;

})();