/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal._catalog._ConfigurationBase");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._DocumentBase");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Template");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.RootAttribute");
dojo.declare("com.ibm.asq.common.web.client.internal._catalog._ConfigurationBase", com.ibm.asq.common.web.client.internal._catalog._DocumentBase, 
{
    // construct a configuration from a json object. If not object is provided, construct
    // an empty configuration from the template.
    constructor: function(obj, typeName){
        this.swappedPart = {};
        if (typeof obj == 'string'){
            this.instanceObj = dojo.fromJson(obj);
        }
        if (!obj){
            this.instanceObj = { Assets: {
                AssetInstance: []
            }};
        }
        if(this.instanceObj == null) {
            this.instanceObj = obj;
        }
        if(this.typeName == null) {
            var assets = this.instanceObj.Assets;
            if(assets == null){
                throw new Error("Cannot construct configuration without either an object or a typeName!");
            }
            var ai = assets.AssetInstance;
            if(ai != null) {
                var instances = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
                if(instances.length > 0){
                    var name = instances[0].AssetType.type;
                    this.typeName = name;
                }
            }
            if(this.typeName == null){
                 throw new Error("Cannot construct configuration without either an object or a typeName!");
            }

        }
        this.rootAttribute = new com.ibm.asq.common.web.client.internal._catalog.RootAttribute(
            this, this, this.instanceObj, null, null);
    },
    // Get the root attribute of the Configuration
    getRootAttribute: function(){
        return this.rootAttribute;
    },
    // Add a new root to the configuration.  The name of 
    // the type MUST be passed in to this method.
    addRoot: function(typeName){
        if(typeName != null) {
            var att = this.rootAttribute;
            if(att.isArray()) {
                att.addArrayValue(typeName);
            } else {
                att.setArrayvalue(typeName);
            }
        }
    },
    // Set a root Part of the Configuration.  This is only
    // valid for a Configuration where there can be multiple roots.
    setRootPart: function(index, typeName) {
        var att = this.rootAttribute;
        att.setArrayValue(index, typeName);
    },        
    // Remove a root Part from the Configuration.  This is only
    // valid for a Configuration where there can be multiple roots.
    removeRootPart: function(index) {
        var att = this.rootAttribute;
        att.removeArrayValue(index);
    },        
    // Get the root Parts of the Configuration.  This is only
    // valid for a Configuration where there can be multiple roots.
    getRootParts: function(){
        var att = this.rootAttribute;
        return att.getArrayValues();
    },
    // Temporary Convienience method to keep the UI from breaking.
    // This should be considered depricated
    getRoot: function(){
        return this.getRootPart(0);
    },
    // Get a root Part of the Configuration.  This is only
    // valid for a Configuration where there can be multiple roots.
    getRootPart: function(index){
        var att = this.rootAttribute;
        return att.getArrayValue(index);
    },
    // return the object as JSON.  Optionally pretty-print it.
    toJson: function(prettyPrint){
        var tempConfig = this.clone();
        tempConfig.prune();
        return dojo.toJson(tempConfig.instanceObj,prettyPrint);
    },
    // Remove Parts that are both template roots and
    // contain no attributes with values.
    prune: function() {
        // Walk the attribute chain starting at the roots
        // TODO remove this marker
        var rootParts = this.rootAttribute.partCache;
        for(var i=0;i<rootParts.length;i++){
            var root = rootParts[i];
            this._prunePart(root);
        }
    },
    _prunePart: function(part) {
        // check if there are any values
        var attributes = part.instanceObj.Attributes;
        if(attributes.Attribute && attributes.Attribute.length == 0){
            return;
        }
        var attNames = part.getAttributeNames();
        var attNameLength=attNames.length;
        for(var j=0; j<attNameLength;j++){
            var att = part.getAttribute(attNames[j]);
            if(att.instanceObj != null) {
                if (att.isArray()) {
                    var vals = att.getArrayValues();
                    if (att.getType() == "ConfigurationPartType") {
                        for (var k = 0; k < vals.length; k++) {
                        	var val = vals[k];
                            this._prunePart(val);
                            var attArray = val.instanceObj.Attributes.Attribute;
                            if (val.isGeneric() && attArray.length == 0) {
                                // TODO:  This may need looking at
                                vals = this._util._removeFromArray(vals, k);
                                k = k - 1;
                            }
                        }
                    }
                    if (vals.length == 0) {
                        if (att.instanceObj.value == null) {
                            this._removeAttributeFromPart(part, attNames[j]);
                        }
                        else {
                            att.instanceObj.value = [];
                            //att.instanceObj.value = { "null": "" };
                        }
                    }
                }
                else {
                    var val = att._getValue(true);
                    // Check if it's a part
                    if (val == null) {
                        this._removeAttributeFromPart(part, attNames[j]);
                    }
                    else if (val["null"] == "") {
                        // do nothing
                    }
                    else if (att.getType() == "ConfigurationPartType") {
                        this._prunePart(val);
                        attArray = val.instanceObj.Attributes.Attribute;
                        if (val.isGeneric() && attArray.length == 0) {
                            this._removeAttributeFromPart(part, attNames[j]);
                        }
                    }
                }
            }
        }
    },
    _removeAttributeFromPart: function(part, attName) {
        var attributes = part.instanceObj.Attributes;
        var attributeArray = attributes.Attribute;
        var found = false;
        var index = 0;
        var attrLength=attributeArray.length ;
        for (var index = 0; (index < attrLength); index++) {
            if(attributeArray[index].name == attName) {
                attributeArray = this._util._removeFromArray(attributeArray, index);
                attributes.Attribute = attributeArray;
                break;
            }
        }
    },
    _getSwappedPart: function(attributeName, partName) {
        var attributeName = this.swappedPart[attributeName];
        if(attributeName == null){
            return null;
        }
        return attributeName[partName];
    },
    _setSwappedPart: function(attributeName, partName, part) {
        this.swappedPart = {};
        var tempObj = {};
        tempObj[partName] = part;
        this.swappedPart[attributeName] = tempObj;
    },
    _clearSwappedPart: function(){
        this.swappedPart = {};
    }
    
});
})();
