//BM
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager");

dojo.require("jazz.core.loader");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("jazz.core.RegistryFactory");
dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.rqm.buildintegration.web.client.internal.build.BuildClient");
dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil");

(function() {

var INIT_UNINITIALIZED= 0;
var INIT_IN_PROGRESS= 1;
var INIT_DONE= 2;
var INIT_FAILURE= 3;
var loader					= jazz.core.loader;
var RegistryFactory 		= jazz.core.RegistryFactory;
var BuildClient				= com.ibm.rqm.buildintegration.web.client.internal.build.BuildClient;
var Status					= com.ibm.team.repository.web.foundation.Status;
var BuildUIUtil           = com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil;

dojo.declare("com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager", null, {

	_buildsProviders: null,
	_currentProviderId: null,

	constructor: function() {
		this._initializationState= INIT_UNINITIALIZED;
		this._buildsProviders= null;
	},
	ensureInitialized: function(monitor, initializationCallback) {
		this._runInitialized(dojo.hitch(this, function() {
			initializationCallback();
		}), monitor);
	},

	getCurrentBuildProviderId: function() {
		return this._currentProviderId;
	},

	getBuildProviders: function() {
		return this._buildsProviders;
	},

	getBuildProvider: function(providerId) {
		return this._buildsProviders && this._buildsProviders[providerId]? this._buildsProviders[providerId].instance : null;
	},
	_runWithCurrentProvider: function(runWithProvider, errorCallback, monitor) {
		this._runInitialized(dojo.hitch(this, function() {
			var provider= this._buildsProviders[this._currentProviderId].instance;
			if (provider == null || provider == undefined) {
				errorCallback(new Status(Status.ERROR, BuildUIUtil.getMessage("labelUnknwonBuildsProvider", this._currentProviderId), null), null);
			} else if (!provider.isConfigured()) {
				errorCallback(new Status(Status.ERROR, BuildUIUtil.getMessage("labelBuildsProviderNotInitialized", this._currentProviderId), null), null);
			} else {
				runWithProvider(provider);
			}
		}), monitor);
	},

	_runInitialized: function(runInitialized, monitor) {
		if (this._initializationState == INIT_UNINITIALIZED) {
			if (dojo.isObject(monitor)) {
				monitor.beginTask(BuildUIUtil.getMessage("labelLoadBuildsProviderConfiguration"));
			}
			BuildClient.getProviderTypes(dojo.hitch(this, function(status, result) {
				if (status.isOK()) {
					this._currentProviderId= result[0].providerId;

					if (dojo.isObject(monitor)) {
						monitor.beginTask(BuildUIUtil.getMessage("labelInitializeBuildsProviders"));
					}
					this._doInitialize(result, dojo.hitch(this, function() {
						this._initializationState= INIT_DONE;
						runInitialized();
					}));
				} else {
					runInitialized();
				}
			}));
		} else {
			runInitialized();
		}
	},
	
	_doInitialize: function(buildsProviders, runAfterInitialization) {
		var extensionPoint= RegistryFactory.getRegistry().getExtensionPoint("com.ibm.rqm.buildintegration.web.buildsProviders");
		var allProviderIds= [];
		if (buildsProviders) {
			this._buildsProviders = {};
			for (var i= 0; i < buildsProviders.length; i++) {
				this._buildsProviders[buildsProviders[i].providerId]= {
					id: buildsProviders[i].providerId,
					name: buildsProviders[i].providerName,
					providerImplementation: null,
					instance: null,
					providerTypeId:buildsProviders[i].providerTypeId
				};
				allProviderIds.push(buildsProviders[i].providerId);
			}
		}
		dojo.forEach(extensionPoint.getExtensions(), dojo.hitch(this, function(extension) {
				dojo.forEach(extension.getConfigurationElements(), dojo.hitch(this, function(configElement) {
					var id= configElement.getAttribute("id");
					var providerImplementation= configElement.getAttribute("providerImplementation");
					if (this._buildsProviders[id]) {
						if (this._buildsProviders[id].providerImplementation == null) {
							this._buildsProviders[id].providerImplementation= providerImplementation;
						} else {
							console.error(BuildUIUtil.getMessage("labelFoundDuplicateBuildsProvider", id));
						}
					} else {
						console.error(BuildUIUtil.getMessage("labelFoundUnknownBuildsProvider", id));
					}
				}));
			}));

		this._doInitializeProvider(allProviderIds, 0, function() {
			runAfterInitialization();
		});
	},

	_doInitializeProvider: function(allProviderIds, providerIndex, runAfterInitialization) {
		if (providerIndex < allProviderIds.length) {
			var providerDescriptor= this._buildsProviders[allProviderIds[providerIndex]];
			loader.load_async(providerDescriptor.providerImplementation, dojo.hitch(this, function() {
				var ctor= dojo.getObject(providerDescriptor.providerImplementation);
				var providerInstance= new ctor(providerDescriptor.id, providerDescriptor.name,providerDescriptor.providerTypeId);
				providerInstance.initialize(dojo.hitch(this, function() {
					this._buildsProviders[providerDescriptor.id].instance= providerInstance;
					this._doInitializeProvider(allProviderIds, providerIndex + 1, runAfterInitialization);
				}));
			}));
		} else {
			runAfterInitialization();
		}
	},
	
	setUninitialized: function(){
		this._initializationState = INIT_UNINITIALIZED;
	},
	
	__sentinel: null // terminates this class definition
});

var MANAGER_INSTANCE= null;

com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager.getInstance= function() {
	if (MANAGER_INSTANCE == null) {
		var instance= new com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager();
		MANAGER_INSTANCE= instance;
	}
	
	return MANAGER_INSTANCE;
};

})();
