/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// Author: Paul Beaulieu
//
// Summary:
//

(function(){
dojo.provide("com.ibm.rational.test.lm.web.client.internal.inventory");
	
dojo.require("com.ibm.asq.common.web.client.internal.container");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.declare("com.ibm.rational.test.lm.web.client.internal.inventory",
			 com.ibm.asq.common.web.client.internal.container,
	{
		constructor: function()
		{
			this.setServiceInterface('com.ibm.rational.test.lm.common.IProviderRestService');			
		},
		
        _getProjectAreaArgs : function()
		{
            var args = { "projectAreaItemId" : com.ibm.team.process.web.projectArea.itemId };
            return args;
        },
		
		getOperations: function( responseHandler )
		{
			return( this._invokeService( responseHandler, 'getInventoryOperations', this._getProjectAreaArgs() ) );	
		},
		
		updateAsset: function( responseHandler, assetId )
		{
			var args = {"assetIds": ( ( dojo.isArray( assetId ) ) ? assetId : [assetId] )};
			return( this._invokeService( responseHandler, 'postUpdateAsset', args ) );	
		},
		
		updateAutomations: function( responseHandler )
		{
			this._invokeService( responseHandler, 'postUpdateAvailableAutomations', this._getProjectAreaArgs() );
		},
		
		updateAllAssets: function( responseHandler )
		{
			return( this._invokeService( responseHandler, 
			                             'postUpdateAllAssets',
										 this._getProjectAreaArgs() ) );
		},
		
		discoverAsset: function( responseHandler, assetId )
		{
			var args = {"assetIds": ( ( dojo.isArray( assetId ) ) ? assetId : [assetId] )};
			return( this._invokeService( responseHandler, 'postDiscoverAsset', args ) );	
		},
		
		/**
		 * Cancel a running discovery operation.
		 * 
		 * @param inventoryOperationUUID
		 *            UUID of InventoryOperation in RTLM repository
		 * @throws TeamRepositoryException
		 */
		cancelDiscovery: function(responseHandler, uuid)
		{
			var args = {"uuid": uuid};
			this._invokeService( responseHandler, 'postCancelDiscovery', args );	
		},
		
		discoverAllAssets: function( responseHandler )
		{
			return( this._invokeService( responseHandler, 'postDiscoverAllAssets', this._getProjectAreaArgs() ) );	
		},
		
		deleteAssetFromProviders: function( responseHandler, assetId )
		{
            var args = {"assetIds": ( ( dojo.isArray( assetId ) ) ? assetId : [assetId] )};
            return( this._invokeService( responseHandler, 'postDeleteAssets', args ) );
		}, 
		
		isInventoryAvailable: function( responseHandler, assetId )
		{
			var args = {"assetIds": ( ( dojo.isArray( assetId ) ) ? assetId : [assetId] )};
			return( this._invokeService( responseHandler, 'postIsInventoryAvailable', args ) );	
		},
		
		getAvailableAssetsByProvider: function( responseHandler, providerName, providerInstance )
		{
			var args = {"providerName": providerName, "providerInstance": providerInstance };
			
			var innerHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
			(
				{
					inventory: this,
					_success: function( response )
					{
						var results = { "Data": [], "Header": ["machine.id", "machine.name"] };
                        var response_length = response.length;
						for( var iMachine = 0; response && ( iMachine < response_length ); iMachine++ )
						{
							results.Data.push( [response[iMachine].id, response[ iMachine ].name] );
						}
						
						responseHandler.doCallback(results);
					},
					_failure: function( errObj )
					{
						responseHandler.doErrorCallback( errObj );						
					}
				},
				"_success",
				"_failure"
			);
			
			this._invokeService( innerHandler, 'getAvailableAssetByProvider', args );
		},
		
		getAutomationOperations: function( responseHandler )
		{
			this._invokeService( responseHandler, 'getAutomationOperations', this._getProjectAreaArgs() );	
		},
		
		getAutomationsForAsset: function( responseHandler, assetId )
		{
			var args = {"assetIds": ( ( dojo.isArray( assetId ) ) ? assetId : [assetId] )};
			return( this._invokeService( responseHandler, 'postAutomationsForAsset', args ) );	
		},
		
		isAutomationAvailableForAsset: function( responseHandler, assetId )
		{
			var args = {"assetIds": ( ( dojo.isArray( assetId ) ) ? assetId : [assetId] )};
			return( this._invokeService( responseHandler, 'postIsAutomationAvailableForAsset', args ) );	
		},
		
		getAllAutomations: function( responseHandler )
		{
			return( this._invokeService( responseHandler, 'getAllAutomations', this._getProjectAreaArgs() ) );	
		},
		
		runAutomation: function(responseHandler, ids, idOfAutomation, serviceProviderName, serverProviderInstance, params){
			var auto_params = ( params ) ? params : [];
			var names = [];
			var values = [];
						
            var auto_params_length = auto_params.length;
			for( var i = 0; i < auto_params_length; i++ )
			{
				names.push( auto_params[i].name );
				values.push( auto_params[i].value );
			}
			
			var args = {"assetIds": ( ( dojo.isArray( ids ) ) ? ids : [ids] ),
						"automationId": idOfAutomation,
						"providerName": serviceProviderName,
						"providerInstance": serverProviderInstance,
						"parameterNames": names,
						"parameterValues": values };
						
			return (this._invokeService(responseHandler, 'postRunAutomation', this._removeNulls( args ) ) );
		},
		
		cancelJob: function( responseHandler, jobId )
		{
			var args = { "runningJobIds": [jobId] };
			
			this._invokeService(responseHandler, 'postCancelJobs', args );
		},
		
		getAllDeployingOperations: function( responseHandler )
		{
			this._invokeService(responseHandler, 'getAllDeployingOperations', this._getProjectAreaArgs() );
		},
		
		deployVirtualImage: function( responseHandler, viUUID )
		{
			var args = { "uuid": viUUID };
		
			this._invokeService(responseHandler, 'postDeployVirtualImage', args );
		},	
			
		syncAllVirtualImages: function( responseHandler )
		{
			this._invokeService(responseHandler, 
			                    'postSyncAllVirtualImages',
								this._getProjectAreaArgs() );
		},
		
		syncAllVirtualMachines: function( responseHandler )
		{
			this._invokeService(responseHandler, 
								'postSyncAllVirtualMachines', 
								this._getProjectAreaArgs() );
		},
			
		syncVirtualMachines: function( responseHandler )
		{
			this._invokeService(responseHandler, 'postSyncVirtualMachines' );
		},
			
		updateVirtualImage: function( responseHandler, viUUID )
		{
			var args = { "virtualImageUUID": viUUID };
		
			this._invokeService(responseHandler, 'postUpdateVirtualImage', args );
		},	
		
		updateVirtualMachine: function( responseHandler, vmUUID )
		{
			var args = { "virtualMachineUUID": vmUUID };
		
			this._invokeService(responseHandler, 'postUpdateVirtualMachine', args );
		},
		
		deployMutipleVirtualImages: function( responseHandler, uuids )
		{
			var ids = ( dojo.isArray( uuids ) ? uuids : [uuids] );
			var processed = 0;
			var results = [];
			
            var ids_length = ids.length;
			for( var iDeploy = 0; iDeploy < ids_length; iDeploy++ )
			{
				var innerHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
				(
					{
						_success: function( response )
						{
							processed++;
							if( response )
							{								
								results.push( response )
							}
							
							if( processed == ids_length )
							{
								responseHandler.doCallback(results);
							}
						},
						_failure: function( errObj )
						{
							responseHandler.doErrorCallback( errObj );						
						}
					},
					"_success",
					"_failure"
				);
				
				this.deployVirtualImage( innerHandler, ids[ iDeploy ] );
			}
			
			if( ids_length == 0 )
			{
				responseHandler.doCallback( results );
			}
		},
		
		discoverMutipleAssets: function( responseHandler, uuids )
		{
			var ids = ( dojo.isArray( uuids ) ? uuids : [uuids] );
			var processed = 0;
			var results = [];
			
            var ids_length = ids.length;
			for( var iAsset = 0; iAsset < ids_length; iAsset++ )
			{
				var innerHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
				(
					{
						_success: function( response )
						{
							processed++;
							if( response )
							{								
								results.push( response )
							}
							
							if( processed == ids_length )
							{
								responseHandler.doCallback(results);
							}
						},
						_failure: function( errObj )
						{
							responseHandler.doErrorCallback( errObj );						
						}
					},
					"_success",
					"_failure"
				);
				
				this.discoverAsset( innerHandler, ids[ iAsset ] );
			}
			
			if( ids_length == 0 )
			{
				responseHandler.doCallback( results );
			}
		},
		
		getJobLog: function( responseHandler, providerName, providerInstance, jobId )
		{
			var args = { "providerName": providerName,
			             "providerInstance": providerInstance,
						 "jobId": jobId, 
						 "projectAreaItemId": com.ibm.team.process.web.projectArea.itemId };
			this._invokeService(responseHandler, 'getJobLog', args );			
		},
		
		undeployVirtualMachine: function( responseHandler, uuid )
		{
			var args = {"uuid": uuid};
			this._invokeService(responseHandler, 'postUndeployVirtualMachine', args );	
		},
		
		undeployMultipleVMs: function( responseHandler, uuids )
		{
			var ids = ( dojo.isArray( uuids ) ? uuids : [uuids] );
			var processed = 0;
			var results = [];
			
            var ids_length = ids.length;
			for( var iAsset = 0; iAsset < ids_length; iAsset++ )
			{
				var innerHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
				(
					{
						_success: function( response )
						{
							processed++;
							if( response )
							{								
								results.push( response )
							}
							
							if( processed == ids_length )
							{
								responseHandler.doCallback(results);
							}
						},
						_failure: function( errObj )
						{
							responseHandler.doErrorCallback( errObj );						
						}
					},
					"_success",
					"_failure"
				);
				
				this.undeployVirtualMachine( innerHandler, ids[ iAsset ] );
			}
			
			if( ids_length == 0 )
			{
				responseHandler.doCallback( results );
			}
		},
		
		isVirtualizationAvailable: function( responseHandler, assetIds )
		{
			var ids = ( dojo.isArray( assetIds ) ? assetIds : [assetIds] );
			var args = {"assetIds": ids}
			this._invokeService(responseHandler, 'postIsVirtualizationAvailable', args );	
		},
		
		addAssetToProvider: function( responseHandler, assetUuid, providerUuid, typeIndex, params )
		{
			var add_params = (params) ? params : {};
			var names = [];
			var values = [];
						
			for( var param in add_params )
			{
				names.push( param );
				values.push( add_params[ param ] );
			}
			var args = 
				{
					"assetUuid": assetUuid,
					"providerUuid": providerUuid,
					"providerType": typeIndex,
					"paramNames": names,
					"paramValues": values	
				};
			
			this._invokeService(responseHandler, 'postAddAssetToProvider', args );				
		},
		
		getInventoryProviders: function( responseHandler, assetId )
		{
			var args = { 
			     "assetId": assetId,
			     "projectAreaItemId": com.ibm.team.process.web.projectArea.itemId};
			this._invokeService(responseHandler, 'getInventoryProviders', args );	
		},
        
        deployVirtualCollection: function( responseHandler, collectionId )
        {
            var args = { "uuid": collectionId };
            this._invokeService(responseHandler, 'postDeployVirtualImageCollection', args );  
        },

        undeployVirtualCollection: function( responseHandler, collectionId )
        {
            var args = { "uuid": collectionId };
            this._invokeService(responseHandler, 'postUndeployVirtualMachineCollection', args );  
        }
	}
);

})();
