//EWIMapper
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.client.internal.mapper.EWIMapper");
dojo.require("com.ibm.asq.common.web.client.internal.mapper.AbstractMapper");

dojo.declare("com.ibm.rqm.execution.web.client.internal.mapper.EWIMapper",
             com.ibm.asq.common.web.client.internal.mapper.AbstractMapper, {			

	SEARCH_RESULT_DTO_CLASSNAME: "com.ibm.rqm.planning.rest.dto:SearchResultDTO",
	PAGED_SEARCH_RESULT_DTO_CLASSNAME:"common_dto.rest:PagedSearchResultDTO",
					
    /*
     * Handle mapping results contained within a SearchResultDTO object 
     * in addition to handling the normal modelled object and arrays
     */
    doCallback: function(result) {
        
        //handle normal array result
        if (result && dojo.isArray(result)) {
            result = this.mapArray(result);

        } else if (result && dojo.isObject(result) && (result._eQualifiedClassName == this.SEARCH_RESULT_DTO_CLASSNAME || 
			 result._eQualifiedClassName == this.PAGED_SEARCH_RESULT_DTO_CLASSNAME)) {
            //handle case of search result object
			result.results = result.results || [];
            if (dojo.isArray(result.results)) {
                result.results = this.mapArray(result.results);
            }        
        }else if(result && !dojo.isArray(result)){
			result = this.mapArray([result]);
		}
        //do callback
        var obj= this._contextObject;
        obj[this._callbackName].call(obj, result);          
    }, 
				
	// transforms a summary DTO to a full DTO
	map: function(summaryDTO) {
		
		// default script
		if(summaryDTO.defaultScriptId){
			summaryDTO.defaultScript = {};
			summaryDTO.defaultScript.versionableItemId = summaryDTO.defaultScriptId;
			summaryDTO.defaultScript.name = summaryDTO.defaultScriptName;
			summaryDTO.defaultScriptId = null;
			summaryDTO.defaultScriptName = null;
		}	
		
		// test plan
		if(summaryDTO.testPlanId){
			summaryDTO.testPlan = {};
			summaryDTO.testPlan.id = summaryDTO.testPlanId;
			summaryDTO.testPlan.versionableItemId = summaryDTO.testPlanVersionableItemId;
			summaryDTO.testPlan.name = summaryDTO.testPlanName;
			summaryDTO.testPlan.sections = summaryDTO.testPlanSections;
			summaryDTO.testPlanId = null;
			summaryDTO.testPlanVersionableItemId = null;
			summaryDTO.testPlanName = null;
		}	
		
		// phase
		if(summaryDTO.phaseItemId){
			summaryDTO.phase = {};
			summaryDTO.phase.itemId = summaryDTO.phaseItemId;
			summaryDTO.phase.name = summaryDTO.phaseName;
			summaryDTO.phaseItemId = null;
			summaryDTO.phaseName = null;
		}	
		
		// test case
		summaryDTO.testCase = {};
		summaryDTO.testCase.id = summaryDTO.testCaseId;
		summaryDTO.testCase.name = summaryDTO.testCaseName;
		summaryDTO.testCase.versionableItemId = summaryDTO.testCaseVersionableItemId;
		summaryDTO.testCase.itemId = summaryDTO.testCaseItemId;
		summaryDTO.testCaseId = null;
		summaryDTO.testCaseItemId = null;
		summaryDTO.testCaseName = null;
		
		// test case scripts
		var scripts = summaryDTO.testCaseScriptIds;
		if (scripts!=null) {
			summaryDTO.testCase.scripts = [];
			for (var i=0; i<summaryDTO.testCaseScriptIds.length; i++) {
				summaryDTO.testCase.scripts[i] = {};
				summaryDTO.testCase.scripts[i].versionableItemId = summaryDTO.testCaseScriptIds[i];
				summaryDTO.testCase.scripts[i].name = summaryDTO.testCaseScriptNames[i];
			}
		}
		
		// owner
		if(summaryDTO.ownerUserId){
			summaryDTO.owner = {};
			summaryDTO.owner.userId = summaryDTO.ownerUserId;
			summaryDTO.owner.name = summaryDTO.ownerName;
			summaryDTO.ownerUserId = null;
			summaryDTO.ownerName = null;
		}	
		
		// current result
		if (summaryDTO.currentResultStartTime!=null) {
			summaryDTO.currentResult = {};
			summaryDTO.currentResult.startTime = summaryDTO.currentResultStartTime;
			summaryDTO.currentResult.state = {}
			summaryDTO.currentResult.state.name = summaryDTO.currentResultStateName;
			summaryDTO.currentResult.state.id = summaryDTO.currentResultStateId;
			summaryDTO.currentResult.itemId = summaryDTO.currentResultItemId;
			summaryDTO.currentResultStartTime = null;
			summaryDTO.currentResultStateName = null;
			summaryDTO.currentResultItemId = null;
			if(summaryDTO.currentResultScriptId){
				summaryDTO.currentResult.script = {};
				summaryDTO.currentResult.script.versionableItemId = summaryDTO.currentResultScriptId;
				summaryDTO.currentResultScriptId = null;
			} 
		}
		
		// configuration
		if(summaryDTO.configId){
			summaryDTO.config = {};
			summaryDTO.config.id = summaryDTO.configId;
			summaryDTO.config.name = summaryDTO.configName;
			summaryDTO.configuration = summaryDTO.configId;//Some sections in editors need configuration variable.
			summaryDTO.configId = null;
			summaryDTO.configName = null;
		}	
		
//		// blocking defects
		if (summaryDTO.defectIds !=null) {
			summaryDTO.defects = [];
			for (var i=0; i<summaryDTO.defectIds.length; i++) {
				var defect = {workItem:{}};
                defect.workItem.attributes=[];
                defect.workItem.attributes.push({key:"id",value:{id:summaryDTO.defectIds[i],label:summaryDTO.defectIds[i]}});
                defect.workItem.attributes.push({key:"summary",value:{content:summaryDTO.defectNames[i]}});
                defect.workItem.attributes.push({key:"internalResolution",value:{label:summaryDTO.defectIResolutions[i]}});
                defect.workItem.attributes.push({key:"internalState",value:{id:summaryDTO.defectIStateIDs[i],label:summaryDTO.defectIStates[i]}});
                summaryDTO.defects.push(defect);
			}
            summaryDTO.defectIds = null;
            summaryDTO.defectNames = null;
            summaryDTO.defectIResolutions = null;
            summaryDTO.defectIStates = null;
		}
		
		if (summaryDTO.results && summaryDTO.results.length > 0){
			var results = summaryDTO.results;	
			summaryDTO.results = [];
			for (var i = 0; i < results.length; i++){
				var result = this.mapResultSummary(results[i]);
				summaryDTO.results[i] = result;
			}
		}
		
		return summaryDTO;
	},
	
	mapResultSummary: function(summaryDTO) {
		// script
		summaryDTO.script = {};
		summaryDTO.script.id = summaryDTO.scriptId;
		summaryDTO.script.name = summaryDTO.scriptName;
		summaryDTO.scriptId = null;
		summaryDTO.scriptName = null;
		
		if(summaryDTO.scriptTypeId){
			summaryDTO.script.scriptType= {};
			summaryDTO.script.scriptType.id= summaryDTO.scriptTypeId;
			summaryDTO.script.scriptType.name= summaryDTO.scriptTypeName;
			summaryDTO.scriptTypeId = null;
			summaryDTO.scriptTypeName = null;
			summaryDTO.script.versionableItemId = summaryDTO.scriptVersionableItemId;
		}
		
		// test case
		summaryDTO.testCase = {};
		summaryDTO.testCase.id = summaryDTO.testCaseId;
		summaryDTO.testCase.name = summaryDTO.testCaseName;
		summaryDTO.testCaseId = null;
		summaryDTO.testCaseName = null;
		
		// state
		if (summaryDTO.state != null) {
			var stateId = summaryDTO.state;
			summaryDTO.state = { name : summaryDTO.stateName, id: stateId};
			summaryDTO.stateName = null;
		}
		
		return summaryDTO;
	}
	
});