/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.process.web.client.projectarea");

(function() { // module container 	

var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var getCurrentProjectArea	= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient = {

    REQ_WORKITEM_SERVICE_URI: "com.ibm.rqm.requirement.common.service.rest.IRequirementWorkItemRestService",
    
    _invokeService: function(responseHandler, method, appArgs){
        var actualArgs = {};
        if (appArgs.testPlanItemId) {
            actualArgs.testPlanItemId = appArgs.testPlanItemId;
        }
        if (appArgs.testCaseItemId) {
            actualArgs.testCaseItemId = appArgs.testCaseItemId;
        }
        if (appArgs.addedRequirements) {
			actualArgs.jsonString = dojo.toJson(appArgs.addedRequirements);
		}
		if (appArgs.ids) {
            actualArgs.ids = appArgs.ids;
        }
        if (appArgs.itemIds) {
            actualArgs.itemIds = appArgs.itemIds;
        }
		if (appArgs.workItemId) {
            actualArgs.workItemId = appArgs.workItemId;
        }
		if (appArgs.reqWorkItemUUID) {
			actualArgs.reqWorkItemUUID = appArgs.reqWorkItemUUID;
		}
		if (appArgs.relatedTestCaseItemId) {
			actualArgs.relatedTestCaseItemId = appArgs.relatedTestCaseItemId;
		}
		if (appArgs.isSingleTestCase) {
			actualArgs.isSingleTestCase = appArgs.isSingleTestCase;
		}
		if (appArgs.names) {
			actualArgs.names = appArgs.names;
		}
		if (appArgs.ownerIds) {
			actualArgs.ownerIds = appArgs.ownerIds;
		}
		if (appArgs.descriptions) {
			actualArgs.descriptions = appArgs.descriptions;
		}
		if (appArgs.weights) {
			actualArgs.weights = appArgs.weights;
		}
		if (appArgs.planId) {
			actualArgs.planId = appArgs.planId;
		}
		if (appArgs.categories) {
			actualArgs.categories = appArgs.categories;
		}
		if (appArgs.templateItemId) {
			actualArgs.templateItemId = appArgs.templateItemId;
		}
		if (appArgs.templateStateId) {
			actualArgs.templateStateId = appArgs.templateStateId;
		}
		if (appArgs.ownerUUID) {
			actualArgs.ownerUUID = appArgs.ownerUUID;
		}
		if (appArgs.workItemsJsonStrings) {
			actualArgs.workItemsJsonStrings = appArgs.workItemsJsonStrings;
		}
        if(appArgs.queryToken) {
            actualArgs.queryToken = appArgs.queryToken; 
        }       
        if(appArgs.page) {
            actualArgs.page = appArgs.page; 
        }       
        if(appArgs.pageSize) {
            actualArgs.pageSize = appArgs.pageSize; 
        }       
        if(appArgs.sortColumns) {
            actualArgs.sortColumns = appArgs.sortColumns; 
        }                               
        if(appArgs.kindOfHandles) {
            actualArgs.kindOfHandles = appArgs.kindOfHandles; 
        } 
        if(appArgs.processArea)
        	actualArgs.processArea =appArgs.processArea;
        else
        	actualArgs.processArea = getCurrentProjectArea().itemId;
        var serviceRequest = new ServiceRequest(this.REQ_WORKITEM_SERVICE_URI, method, actualArgs);
        TeamServerClient.invokeService(serviceRequest, responseHandler);
    },
    
    getRequirementWorkItemDTOs: function(serviceResponseHandler, params){
        this._invokeService(serviceResponseHandler, "getWorkItemDTOs", params);
    },
    
    postRequirementWorkItems: function(serviceResponseHandler, params){
        this._invokeService(serviceResponseHandler, "postWorkItems", params);
    },
	
	postLatestWorkItemDTOs: function(serviceResponseHandler, params){
        this._invokeService(serviceResponseHandler, "postLatestWorkItemDTOs", params);
    },
	
	getTestCasesByRequirement: function(serviceResponseHandler, params) {
		this._invokeService(serviceResponseHandler, "getTestCasesByRequirement", params);
	},

	getTestCasesByRequirements: function(serviceResponseHandler, params) {
		this._invokeService(serviceResponseHandler, "getTestCasesByRequirements", params);
	},
		
	getTestPlansByRequirement: function(serviceResponseHandler, params) {
		this._invokeService(serviceResponseHandler, "getTestPlansByRequirement", params);
	},
	
	postArchiveRequirements: function(serviceResponseHandler, params) {
		this._invokeService(serviceResponseHandler, "postArchiveRequirements", params);
	},
	
	postCreateTestCasesByRequirement: function(serviceResponseHandler, params) {
		this._invokeService(serviceResponseHandler, "postCreateTestCasesByRequirement", params);
	},
	
	postReassignRequirementOwner: function(serviceResponseHandler, params) {
		this._invokeService(serviceResponseHandler, "postReassignRequirementOwner", params);
	},
	
    getAllSearchResultItemIds: function(serviceResponseHandler, args)
    {
        /*
                VALID PARAMETERS
                --------------------------------
                String queryToken;
                String idType;
         */         
        var typeMap = {
            'id': 'Id',
            'itemId': 'ItemHandle',
            'versionableItemId': 'VersionedItemHandle'
        };
        var type = args.kindOfHandles;
        args.kindOfHandles = typeMap[type];
        return this._invokeService(serviceResponseHandler, "getAllItemHandles", args);
    },   
           
	searchRequirements: function(serviceResponseHandler, params) {
        /*
         valid parameters:
            public String projectArea;
            public String name;
            String queryToken;
            int page;
            int pageSize;
            String[] sortColumns;
            String textSearch;
        */
		params = params ? params : {};
        if (!params.processArea) {
            params.processArea = getCurrentProjectArea().itemId;
        }           
		 var serviceRequest = new ServiceRequest(this.REQ_WORKITEM_SERVICE_URI, "getPagedSearchResult", params);
        TeamServerClient.invokeService(serviceRequest, serviceResponseHandler);
	}
};

})();