/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.reports.cognos.web.ui.internal.CognosReportsBrowseDialog");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.reports.cognos.web.client.internal.CognosReportsClient");
dojo.require("com.ibm.rqm.reports.cognos.web.client.internal.Constants");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.Tree");
dojo.require("dojox.data.dom");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("net.jazz.ajax.ui.internal.Dialog");


(function() {

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var CognosReportsClient = com.ibm.rqm.reports.cognos.web.client.internal.CognosReportsClient;
var Constants = com.ibm.rqm.reports.cognos.web.client.internal.Constants;
var AlertDialog 	= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;

dojo.declare("com.ibm.rqm.reports.cognos.web.ui.internal.CognosReportsBrowseDialog", [dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.rqm.reports.cognos.web", "ui/internal/templates/CognosReportsBrowseDialog.html"),
	labelOK: Constants.getMessage("labelOK"),
	labelCancel: Constants.getMessage("labelCancel"),
	widgetsInTemplate: true,
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		this.userName = args.username;
		this.password = args.password;
		this.namespace = args.namespace;
		this.ip 		= args.ip;
		this.url = net.jazz.ajax._contextRoot + "/service/com.ibm.rqm.reports." +
		  "service.cognos.internal.ICognosReportService/?cognosurl=" + this.ip + "&user=" + this.userName
		+ "&pw=" + this.password + "&ns=" + this.namespace;
		this.elementsToBeDisplayed = args.elementsToBeDisplayed;
	},
	
	postCreate: function() {
		this.dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: this.contentNode,
			modal: true,
			primaryTitle: Constants.getMessage("labelReportBrowse"),
			width: "460px",
			height: "270px",
			isClosable: true
		});
		this.connect(this._okButton, "onclick", this._onOk);
		this.connect(this._cancelButton, "onclick", this._onCancel);
		this.textLoading = document.createTextNode("Loading...");
		this._treeContainer.appendChild(this.textLoading);
		
		var self = this;
		if(this.elementsToBeDisplayed == "ReportTemplate"){
			dojo.xhrGet
			({
			  url: this.url,
			  load: function(response, ioArgs){
			  	var reportStore = new dojo.data.ItemFileReadStore({data: response});
				var reportTree = new dijit.Tree({           
					store: reportStore,
					query: {type: 'content'},
					labelAttr: "name",
					typeAttr: "type",
					onClick: dojo.hitch(self, function(item, node){
						if(node.item.type == "report"){
							self.reportPath = node.item.path;
							self.node = node;
							self.reportId = node.item.reportId;
							self._okButton.disabled = false;
						}
					})}, dojo.byId("_treeContainer"));
				self.reportTree = reportTree;
			//self._tree = tree;
			self._treeContainer.removeChild(self.textLoading);
			self._treeContainer.appendChild(reportTree.domNode);
			},error: this._handleError,
			  handleAs: "json"
			});
		}
		else{
			var context = this;
			// Create the service object
			var serviceObject = {
				success: function(item) {
				context._createTree(item);
			},
			failure: function(errorObject) {
				(new AlertDialog({'message': Constants.getMessage("errorCantCommunicate") })).open();
			}
			};
			// Create the handler
			var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
			CognosReportsClient.getCognosReportDescriptors(handler, {});
		}
	},
	
	uninitialize: function() {
		if (this.reportTree){
			this.reportTree.destroy();
		}
		if (this.dialog){
			this.dialog.destroy();
		}
	},
	
	_createTree: function(reports){
		var reportStore = this._createTemplateDataStore(reports);
		this._createTemplateTree(reportStore);
	},
	
	_createTemplateTree: function(dataStore) {
		var self = this;
		var treeStore = new dojo.data.ItemFileReadStore({
			data: dataStore
		});
		this.tree = new dijit.Tree({
			store: treeStore,
			widgetId: "templateTreeWidget",
			onClick: dojo.hitch(self, function(item, node){
				self._okButton.disabled = false;
				self._loadReport(item, node);
		})
		});

		this._templateTreeWidget = this.tree;
		this._treeContainer.removeChild(this.textLoading);
		this._treeContainer.appendChild(this.tree.domNode);
	},
	
	_loadReport: function(item, node){
		this.reportUUID = item.reportUUID;
		this.cognosReportName = node.label;
	},
		
	_onOk: function(item, node) {
		this.dialog.close();
	},
	
	_onCancel: function() {
		this.dialog.close();
	},
	
	_handleError: function(data){
		(new AlertDialog({'message': data.responseText})).open();
	},

	close: function() {
		this.dialog.close();
	},
	
	_createTemplateDataStore: function(reports) {
			var store = new Object();
			store.label = "name";
			store.identifier = "name";
			var items = new Array();
			store.items = reports;
			return store;
		}
});

})();