/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.LinksTableView");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.RichHoverCellRenderer");
dojo.require("com.ibm.rqm.defects.web.util.RichHoverUtil");

dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");


(function(){
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
    var RichHoverCellRenderer = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.RichHoverCellRenderer;
    var RichHoverUtil = com.ibm.rqm.defects.web.util.RichHoverUtil;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
    dojo.declare("LinksTableView.TableListener", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener, {
        constructor: function(args){
            dojo.mixin(this, args);
        },
        
        // Summary:
        //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
        //   called.
        //
        rowCheckStateChanged: function(tableViewer, rowObject, state){
            var selection = tableViewer.getCheckedRows();
            var selectionCount = selection.length;
            
            if (this.parentView.deleteActionNode) 
                // can delete archived or non-archived test cases
                this.parentView.deleteActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
            if (this.parentView.createTestCasesActionNode) {
                this.parentView.createTestCasesActionNode.setDisabled(selectionCount != 1 || !tableViewer.isEditable());
            }
            
            //if table view has a rowCheckListener, invoke it
            var p = this.parentView;
            if (p && p.rowCheckedListener && dojo.isFunction(p.rowCheckedListener)) {
                p.rowCheckedListener(rowObject, state);
            }
        },
        
        tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){
            if (callback != null && dojo.isFunction(callback)) {
                if (callbackData != null) {
                    callback(callbackData);
                }
                else {
                    callback();
                }
            }
        }
    });
    
    
    dojo.declare("com.ibm.rqm.defects.web.ui.internal.LinksTableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        getNumColumns: function(content){
            return 1;
        },
        getColumnHeading: function(content, index){
            var headings = [DefectsUIUtil.getMessage("labelSummaryHeading")];
            return headings[index];
        },
        getColumnHeadingImage: function(content, atColumnIndex){
            return null;
        },
        getColumnHeadingAlignment: function(atColumnIndex){
            return this.getCellAlignment(null, atColumnIndex);
        },
        getIsColumnGroupable: function(content, atColumnIndex){
            return false;
        },
        getIsColumnSortable: function(content, atColumnIndex){
            return false;
        },
        getColumnWidth: function(content, atColumnIndex){
            return 100;
        },
        getContentObjects: function(content){
            return content;
        },
        getRowDataId: function(rowData){
        	var id = rowData.url || rowData.uri;
            return "_table_row_for_external_wi" + id;
        },
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_TEXT;
            switch (atColumnIndex) {
                case 0:
                    type = "RENDER_TYPE_RICHHOVERLINK";
                    break;
            }
            return type;
        },
        getCellAlignment: function(rowData, atColumnIndex){
            if (atColumnIndex == 0) 
                return TableConstants.ALIGN_CENTER;
            return TableConstants.ALIGN_LEFT;
        },
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "";
            switch (atColumnIndex) {
                case 0:
                    val = this._restoreTags(rowData.summary);
                    break;
            }
            return val;
        },
        getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
            var val = {};
            if (atColumnIndex == 0) {
                val.uri = RichHoverUtil.getWorkItemUri(rowData.url || rowData.uri);
                val.presentationUri = RichHoverUtil.getWorkItemPresentationUri(rowData.url || rowData.uri);
                val.label = this._restoreTags(rowData.summary);
            }
            
            return val;
        },
        
        // function from com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil
        _restoreTags: function(value){
            var rawText = value;
            rawText = rawText.replace(/\&gt;/gi, ">");
            rawText = rawText.replace(/\&lt;/gi, "<");
            rawText = rawText.replace(/\&amp;/gi, "&");
            rawText = rawText.replace(/\&tilde;/gi, "~");
            rawText = rawText.replace(/\&quot;/gi, "\"");
            rawText = rawText.replace(/\&circ;/gi, "^");
            rawText = rawText.replace(/\&nbsp;/gi, " ");
            rawText = rawText.replace(/<br\/>/gi, "\n");
            rawText = rawText.replace(/\&#09;/g, "\t");
            return rawText
        }
    });
    
    
    dojo.declare("com.ibm.rqm.defects.web.ui.internal.LinksTableView", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
        '</div>',
        
        widgetsInTemplate: true,
        editable: true,
        
        constructor: function(){
        },
        
        postCreate: function(){
            this.inherited("postCreate", arguments);
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
                allowAdjustSettings: true,
                autoSaveSettings: true,
                tableId: "RQM-WORKITEMLINKS-TABLEVIEWER"
            });
            tableViewer.registerCellRenderer("RENDER_TYPE_RICHHOVERLINK", new RichHoverCellRenderer());
            this._tableViewerNode.appendChild(tableViewer.domNode);
            
            var content = this.wiLinks ? this.wiLinks : [];
            tableViewer.setContentPageSize(5);
            tableViewer.setContent(content);
            var tableProvider = new com.ibm.rqm.defects.web.ui.internal.LinksTableProvider();
            tableViewer.setContentProvider(tableProvider);
            tableViewer.setTableListener(new LinksTableView.TableListener({
                parentView: this
            }));
            
            tableViewer.setTableRowsCheckable(true, /* isSingleSelect */ false);
            tableViewer.show();
            for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
                var action = this.actions[i];
                if (action.label == DefectsUIUtil.getMessage("labelRemoveRmLinks") || action.label == DefectsUIUtil.getMessage("labelRemoveWorkitem")) {
                    this.deleteActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
                    this.deleteActionNode.setDisabled(true);
                }
                else 
                    if (action.label == Constants.getMessage("labelCreateTestCases")) {
                        this.createTestCasesActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
                        this.createTestCasesActionNode.setDisabled(true);
                    }
                    else {
                        var actionButton = tableViewer.addAction("", action.label, null, action.iconClass, action.actionHandler);
                        actionButton.setDisabled(!this.editable);
                    }
            }
            tableViewer.setEditable(this.editable);
            this.tableViewer = tableViewer;
        },
        
        refresh: function(links){
            this.tableViewer.setContent(links);
            this.tableViewer.refresh();
        },
        
        getCheckedRows: function(){
            return this.tableViewer.getCheckedRows();
        }
    });
    
})();
