/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.auth.ESignatureDialog");

dojo.require("dojo.cookie");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");


(function(){
	
	var getCurrentContributor       = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
	var ServiceResponseHandler      = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    
	var SECURE_RESOURCE_PATH        = "/service/com.ibm.team.repository.common.internal.IContributorRestService/currentContributor";
	var JAZZ_FORM_COOKIE            = "JazzFormAuth";
	var JAZZ_LOGIN_COOKIE           = "JazzLoginCookie";
    var X_AUTH_MSG                  = "X-com-ibm-team-repository-web-auth-msg";
	var SESSIONID_COOKIE            = "JSESSIONID";
	
    dojo.declare("com.ibm.asq.common.web.ui.internal.auth.ESignatureDialog", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/auth/templates/ESignatureDialog.html"),

        numFailuresAllowed: 2,
		width: "500px",
    	title: MessageUtil.getMessage(MessageBundle.COMMON, "labelESig"),
		labelRequires: MessageUtil.getMessage(MessageBundle.COMMON, "labelESigRequired"),
		labelSign: MessageUtil.getMessage(MessageBundle.COMMON, "labelESigSign"),
		labelUserId: MessageUtil.getMessage(MessageBundle.COMMON, "labelUserId"),
		labelPassword: MessageUtil.getMessage(MessageBundle.COMMON, "labelPassword"),		
		   
        constructor: function(args){
			//callback functions
            this.onSuccess = function(){};
			this.onFailure = function(){};
			this.onCancel = function(){};
			
			this._origSessionId = null;
			this._origSessionCookies = null;
			this._failureCount = 0;
        },
		
		postCreate: function() {          
		    var dialogArgs = {
				primaryTitle: this.title,
				okCallback: dojo.hitch(this, "_onSign"),
				cancelCallback: dojo.hitch(this, "_onCancel"),
		        isClosable: false
		    };                
		    this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
		    this._dialog.setWidth(this.width);
			this._dialog.getOkButton().disabled = false;
			this._dialog.getOkButton().title = this.labelSign;
			//this._dialog.getCancelButton().style.display = "none";
		    this._dialog.contentContainer.appendChild(this.domNode);
			this._dialog.open();
			
            //auto-fill userId
            this._userId.value = getCurrentContributor().userId;	
			
			//focus on password field
			setTimeout(dojo.hitch(this, function() {this._password.focus()}), 250);		
		},
		
		_onCancel: function() {
			if (this._failureCount > 0 && this._failureCount <= this.numFailuresAllowed) {
				this._restoreSession()
			}
            if (dojo.isFunction(this.onCancel)) {
				this.onCancel();
			}
		},
		
		_onKeypress: function(event) {
			if (event.keyCode == dojo.keys.ENTER) {
                this._onSign();
			}
		},
		
		_onSign: function() {
			//ensure all inputs have values
			if (dojo.trim(this._userId.value).length == 0) {
				this._status.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "msgFieldRequired", this.labelUserId);
				return false;
			} else if (dojo.trim(this._password.value).length == 0) {
				this._status.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "msgFieldRequired", this.labelPassword);
				return false;
			} else {
				this._status.innerHTML = ""; //MessageUtil.getMessage(MessageBundle.COMMON, "msgAuthenticating");
			}
			//determine authentication scheme
			var method = document.cookie.match(JAZZ_FORM_COOKIE + "=") ? "FORM" : "BASIC";
			if (method != "FORM") {
				console.warn("Cannot determine authentication scheme.  The JazzFormAuth Cookie is missing or the server is configured for BASIC authentication which is not yet supported for E-Signature.");
			} 
			
            this._doFormAuth();
            
            return false;
		}, 
				
		_doFormAuth: function() {
			//on first attempt, save off cookies for later restoration
			if (this._failureCount == 0) {
				//save off existing cookie(s)
				var sessionId = dojo.cookie(SESSIONID_COOKIE);
				if (sessionId != null) {
					//TOMCAT/JETTY 
					//save original session id
					this._origSessionId = sessionId;
					//remove session cookie to force new login
					dojo.cookie(SESSIONID_COOKIE, "", {
						expires: -1,
						path: net.jazz.ajax._contextRoot
					});
					
				} else {
					//WEBSPHERE/OTHER
					this._origSessionCookies = [];
					//Example of document.cookie: "JazzFormAuth=Form; JSESSION_ID=876c876d87c; JazzLoginCookie=ADMIN"
					var cookies = document.cookie.split(";");
					for (var i = 0; i < cookies.length; i++) {
						var cookie = cookies[i].split("=");
						//each array element will contain Name at 0, Value at 1 
						var name = dojo.trim(cookie[0]);
						if (name != JAZZ_FORM_COOKIE && name != JAZZ_LOGIN_COOKIE) {
							//save the cookie info so it can be restored after authentication
							this._origSessionCookies.push({
								name: name,
								value: cookie[1]
							});
							//remove cookie to force new login
							dojo.cookie(name, "", {
								expires: -1,
								path: net.jazz.ajax._contextRoot
							});
						}
					}
				}
			}
			
            //hit another secure resource - forces login			
			this._forceAuthentication(new ServiceResponseHandler(this, "_doLogin", "_error"));
		},

        _formAuthSuccess: function() {
			console.debug(this.declaredClass + ": Before '_restoreSession'; document.cookie: '" + document.cookie + "'");
            this._restoreSession();
			//this._status.innerHTML = "Successfully Authenticated '" + this._userId.value + "'...";
			this._onSuccess();
			console.debug(this.declaredClass + ": On complete; document.cookie: '" + document.cookie + "'");
        },
				
		_formAuthFailure: function(error) {
			this._onFailure(error); 
		},	
		
		_restoreSession: function() {
            //restore original session id, if it was saved
			if (this._origSessionId != null) {
                var newSessionId = dojo.cookie(SESSIONID_COOKIE);
                dojo.cookie(SESSIONID_COOKIE, "", { expires: -1, path: net.jazz.ajax._contextRoot }); //remove new session id
                dojo.cookie(SESSIONID_COOKIE, this._origSessionId, { path: net.jazz.ajax._contextRoot });		
                
			}
			//restore original session cookies, if present
            if (dojo.isArray(this._origSessionCookies) && this._origSessionCookies.length > 0) {
                for (var i = 0; i < this._origSessionCookies.length; i++) {
					var cookie = this._origSessionCookies[i];
	                dojo.cookie(cookie.name, "", { expires: -1, path: net.jazz.ajax._contextRoot });   //remove new cookie
	                dojo.cookie(cookie.name, cookie.value, { path: net.jazz.ajax._contextRoot });      //restore original cookie
				}
            }
		},
		
		_forceAuthentication: function(responseHandler) {
	        var bindArgs= {
	            url: net.jazz.ajax._contextRoot + SECURE_RESOURCE_PATH,
	            content: { },
	            handleAs: "json",
	            headers: { },
	            handle: dojo.hitch(this, function(response, ioArgs){
	                if (response instanceof Error) {
	                    var errorObj= {
	                        status: ioArgs.xhr.status
	                    };
	                    // format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
	                    errorObj.message= dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
	                    responseHandler.doErrorCallback(errorObj);
	                }
	                else {
                        responseHandler.doCallback(ioArgs.xhr.getResponseHeader(X_AUTH_MSG));
	                }
	            })
	        };
	        dojo.xhrGet(bindArgs);			
		},
		
		_doLogin: function() {
            //perform login
	        var bindArgs= {
	            //url: net.jazz.ajax._contextRoot + "/j_security_check",
				url: "j_security_check",
				contentType: "application/x-www-form-urlencoded; charset=utf-8",
				
	            content: {
	                j_username: this._userId.value,
	                j_password: this._password.value
	            },
	            handleAs: "json",
	            headers: {
	                accept: "text/json"
	            },
	            handle: dojo.hitch(this, function(response, ioArgs){
	                if (response instanceof Error) {
	                    var errorObj= {
	                        status: ioArgs.xhr.status
	                    };
	                    // format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
	                    errorObj.message= dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
	                    this._formAuthFailure(errorObj);
						
	                } else if (ioArgs.xhr.getResponseHeader(X_AUTH_MSG) == "authfailed") {
	                    var errorObj= {
	                        message: MessageUtil.getMessage(MessageBundle.COMMON, "msgAuthFailed")
	                    };
	                    this._formAuthFailure(errorObj);
						
	                } else {
	                    this._formAuthSuccess(true);
	                }
	            })
	        };
	        dojo.xhrPost(bindArgs);
		},
		
		_onSuccess: function() {
			this._close();
			if (dojo.isFunction(this.onSuccess)) {
				this.onSuccess();
			}
		},
		
        _onFailure: function(error) {
			this._failureCount++;
			if (this._failureCount > this.numFailuresAllowed) {
	            this._close();
	            if (dojo.isFunction(this.onFailure)) {
	                this.onFailure(error);
	            }				
			} else {
				this._status.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "msgAuthFailed");
			}

        },		
		
		_close: function() {
			if (this._dialog) {
				this._dialog.close();
			}
		},
		
		_error: function(e) {
			//this._status.innerHTML = "Error authenticating: '" + (e ? e.message : "") + "'..."; 
            console.error(this.declaredClass + ": error: " + (e ? e.message : ""));          
        }
	
    });
      
})();
