/**TeamConcertBuildWizard**************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.buildintegration.web.ui.internal.config.TeamConcertBuildWizard");
dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.config.BuildPageRepository");
dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.config.BuildPageProjectArea");
dojo.require("com.ibm.rqm.buildintegration.web.client.internal.TeamConcertConnectionClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.foundation.Status");

dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil");

(function(){

var TeamConcertConnectionClient			= com.ibm.rqm.buildintegration.web.client.internal.TeamConcertConnectionClient;
var ServiceResponseHandler				= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Status 								= com.ibm.team.repository.web.foundation.Status;
var BuildUiUtil 					= com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil;

dojo.declare("com.ibm.rqm.buildintegration.web.ui.internal.config.TeamConcertBuildWizard", null, {

	_context: null,
	_type: null,
	_teamConcertBuildProvider: null,
	_title: null,
	_pages: null,

	constructor: function(teamConcertBuildProvider) {
		this._pages= [];
		this._context= {
			url: teamConcertBuildProvider.getRepositoryUrl() || "",
			name: teamConcertBuildProvider.getConnectionName() || "",
			description: teamConcertBuildProvider.getRepositoryDescription() || "",
			projectArea: teamConcertBuildProvider.getProjectAreaName() || ""
		};
		this._title= "Rational Team Concert";
		this._pages.push(com.ibm.rqm.buildintegration.web.ui.internal.config.BuildPageRepository.create(this._context));
		this._pages.push(com.ibm.rqm.buildintegration.web.ui.internal.config.BuildPageProjectArea.create(this._context));
		
		this._teamConcertBuildProvider= teamConcertBuildProvider;
	},
	getTitle: function() {
		return this._title;
	},

	getPages: function() {
		return this._pages;
	},

	getPreviousPage: function(currentPage) {
		var index= dojo.indexOf(this._pages, currentPage);
		return index > 0 ? this._pages[index - 1] : null;
	},

	getNextPage: function(currentPage) {
		var index= dojo.indexOf(this._pages, currentPage);
		return index < this._pages.length - 1 ? this._pages[index + 1] : null;
	},

	canFinish: function() {
		return dojo.every(this._pages, function(page) {
			return page.isPageComplete();
		});
	},

	getInitialPage: function() {
		return this._context.url.length > 0 ? this._pages[1] : this._pages[0];
	},

	performFinish: function(runWhenFinished, monitor) {
		monitor.beginTask(BuildUiUtil.getMessage("labelSaveConfiguration"));
		var _providerTypeId = this._teamConcertBuildProvider.getProviderTypeId();
		var connectionParams= {
			name: this._context.name, 
			description: this._context.description, 
			repositoryUrl: this._context.url, 
			projectAreaName: this._context.projectArea,
			providerTypeId: _providerTypeId
		};
		TeamConcertConnectionClient.postDefaultTeamConcertConnection(connectionParams, dojo.hitch(this, function(status, response) {
			if (status.isOK()) {
				this._teamConcertBuildProvider.setup(function(status) {
					runWhenFinished(status);
				}, monitor);
			} else {
				runWhenFinished(status);
			}
		}));
	}
});

})();
