/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    An action to handle bulk changes in supported list views: 
 //     test plan
 //     test case
 //     test suite
 //    This includes changing of state, weight, risk, categories, and owner.
 //
dojo.provide("com.ibm.rqm.planning.web.ui.internal.model.action.BulkChangeAction");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressStatusMonitor");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.AbstractAction");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.ActionEntityType");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectCategoryPopup");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectWeightPopup");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectStatePopup");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectOwnerPopup");
dojo.require("com.ibm.asq.common.web.client.internal.ContributorClient");

// NOTE: these modules are dynamically loaded when the respective list views are invoked (non-static load)
//
// com.ibm.rqm.risk.web.client.internal.risk.RiskClient
// com.ibm.rqm.execution.web.client.internal.execution.TestExecutionSuiteClient

(function()
{
var Constants        = com.ibm.rqm.planning.web.client.internal.Constants;    
var ActionEntityType = com.ibm.rqm.planning.web.ui.internal.model.action.ActionEntityType;      
var AbstractAction = com.ibm.rqm.planning.web.ui.internal.model.action.AbstractAction;
var TestPlanClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
var TestCaseClient = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var ProgressUtility = com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressUtility;
var UserClient      = com.ibm.asq.common.web.client.internal.ContributorClient;
var CommonUtility   = com.ibm.asq.common.web.ui.internal.util.Utility;
var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
 
dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.action.BulkChangeAction", AbstractAction, 
{
    artifactType: null,
    actionId: null,
    itemIds: null,
    categoryType: null,
    itemIdsArgName: 'uid',
    
    execute: function()
    {
        var actionId = this.actionId;
        var categoryType = this.categoryType;
        var dlg;
        this.opTitle = Constants.getMessage("labelChangeProperties", actionId);
                                    
        if (actionId == Constants.getMessage("labelRiskAssessment") || actionId == "risk") {
            //handle risk assessment
            dlg = new com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskProfileTemplateDialog({
                artifactType: this.artifactType,
                actionId: actionId,
                okCallback: dojo.hitch(this, "_doRiskProfile")
            });
        }
        else if (actionId == Constants.getMessage("labelWeight")) {
            var args = {
                actionId: actionId,
                okCallback: dojo.hitch(this, "_doBulkChange")
            }
            this.isChangeWeight = true;
            dlg = new com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectWeightPopup();
            dlg.initialize(args);
        }
        else if (actionId == Constants.getMessage("labelState")) {
            var args = {
                actionId: actionId,
                okCallback: dojo.hitch(this, "_doBulkChange")
            }
            dlg = new com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectStatePopup();
            dlg.initialize(args);
        }
        else if (this.actionId == Constants.getMessage("labelOwner") || actionId == "owner") {
            var handler = {
                self: this,
                actionId: actionId,
                
                success: function(users){
                    if (dojo.isArray(users) && users.length > 0) {
                        var owners = [];
                        for (var i = 0; i < users.length; i++) {
                            if (users[i].name != "ADMIN" && users[i].name != "GUEST" && !users[i].archived) {
                                if (CommonUtility.isUnassignedUserId(users[i].itemId)) 
                                    users[i].name = Constants.getMessage("labelUnassigned");
                                owners.push(users[i]);
                            }
                        }
                        
                        var args = {
                            owners: owners,
                            actionId: this.actionId,
                            okCallback: dojo.hitch(this.self, "_doBulkChange")
                        }
                        
                        var dlg = new com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectOwnerPopup();
                        dlg.initialize(args);
                    }
                },
                error: function(e){
                    this.self.errorCallback(e);
                }
            };
            var responseHandler = new ServiceResponseHandler(handler, "success", "error");
            UserClient.getContributors(responseHandler);
        }
        else if(categoryType){
            var args = {
                categoryType: categoryType,
                actionId: actionId,
                okCallback: dojo.hitch(this, "_doBulkChange")
            }
            dlg = new com.ibm.rqm.planning.web.ui.internal.view.common.properties.SelectCategoryPopup();
            dlg.initialize(args);
        }
    },
    
    _doBulkChange: function(args){
        var params = { };
        
        // initialize the default params for bulk change.
        // if there's no weight change, we send a -1 as an
        // indicator. This allows 0 to be a valid value
        // for bulk change.
        params[this.itemIdsArgName] = this.itemIds;
        params.weight = -1;
         
        var handler = {
            self: this,
            success: function(cases) {
                var self = this.self;
                if(self.isChangeWeight && self.versionableItemIds) {
                    //re-calculate risk value after weight changed
                    var riskHandler = {
                        success: function(riskAssessments){
                            ProgressUtility.endTaskMonitor(self.monitorId);
                            self.completeCallback();
                        },
                        error: function(e){
                            ProgressUtility.endTaskMonitor(self.monitorId, (e && e.message) ? e.message : null);
                            self.completeCallback();
                        }
                    }
                    var newParam = {
                        artifactType: self.artifactType,
                        testCaseItemIds: self.versionableItemIds
                    }
                    var riskRespHandler = new ServiceResponseHandler(riskHandler, "success", "error"); 
                    com.ibm.rqm.risk.web.client.internal.risk.RiskClient.postChangeRiskAssessmentsByWeight(riskRespHandler, newParam);
                    return;
                }
                ProgressUtility.endTaskMonitor(self.monitorId);
                self.completeCallback();           
            },
            error: function(e) {
                var self = this.self;
                //  This substitution message comes from a very particular problem resolved on server side.
                //  Since this is fixed on iFix3 we are no able to create new messages, and server has no any accurate
                //  message to present regarding this problem, but UI does; so there is an agreement here between server
                //  side and UI. If an error message comes with this particular "key msgCannotTransitionNotAllTestScriptsApproved"
                //  from server, UI look for translation of said key and reported as the next couple of lines below do.
                //  This has been fixed accurately on 30 and a translatable message has been added to server.
                if(e && e.message=="msgCannotTransitionNotAllTestScriptsApproved")
                    e.message=Constants.getMessage("msgCannotTransitionNotAllTestScriptsApproved",
                                                    Constants.getMessage("com.ibm.rqm.planning.common.approved"));
                ProgressUtility.endTaskMonitor(self.monitorId, (e && e.message) ? e.message : null);
                self.errorCallback();
            }
        };
        
        if (args.actionId == Constants.getMessage("labelWeight")) {
            params.weight = args.weight;
        }
        else if (args.actionId == Constants.getMessage("labelState")) {
            params.state = args.state;
        }
        else if (args.actionId == Constants.getMessage("labelOwner") || args.actionId == "owner") {
            params.ownerId = args.OwnerItemId;
        }
        else {
            params.category = args.categoryValueItemId;
            params.categoryType = args.categoryTypeItemId;
        }
        var responseHandler = new ServiceResponseHandler(handler, "success", "error"); 
        this.invokePost(responseHandler, params);
    },
    
    _doRiskProfile: function(args) {
        if(args && args.selectedProfile) {            
            var self = this;
            var handler = {
                success: function(riskAssessments){
                    ProgressUtility.endTaskMonitor(self.monitorId);
                    self.completeCallback();           
                },
                error: function(err){
                	var error = (err && err.message)?err.message:err;
                    ProgressUtility.endTaskMonitor(self.monitorId, error);
                    self.completeCallback(err);           
                }
            }
            var params = {
                artifactType: this.artifactType,
                riskProfileId: args.selectedProfile.id,
                operation: args.handleAction,
                artifactItemIds: this.itemIds
            };
            
            var responseHandler = new ServiceResponseHandler(handler, "success", "error");
            this.invokePost(responseHandler, params);
        }  
    },
    
    invokePost: function(handler, params)
    {
        this.monitorId = ProgressUtility.beginTaskMonitor(this.opTitle, 0, null);
    }
});

// Summary:
//    Specific class to handle test case bulk changes.
//
dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.action.TestCaseBulkChangeAction", 
             com.ibm.rqm.planning.web.ui.internal.model.action.BulkChangeAction, 
{
    artifactType: Constants.MODEL_TYPE_CASE,
    
    invokePost: function(responseHandler, params)
    {
        this.inherited("invokePost", arguments);

        if (this.actionId == Constants.getMessage("labelRiskAssessment")) {
            com.ibm.rqm.risk.web.client.internal.risk.RiskClient.postSwitchRiskProfile(responseHandler, params);
        }
        else {
            TestCaseClient.postBulkChangeTestCase(responseHandler, params);
        }
    }
});

// Summary:
//    Specific class to handle test plan bulk changes.
//
dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.action.TestPlanBulkChangeAction", 
             com.ibm.rqm.planning.web.ui.internal.model.action.BulkChangeAction, 
{
    artifactType: Constants.MODEL_TYPE_PLAN,
    
    invokePost: function(responseHandler, params)
    {
        this.inherited("invokePost", arguments);

        if (this.actionId == Constants.getMessage("labelRiskAssessment")) {
            com.ibm.rqm.risk.web.client.internal.risk.RiskClient.postSwitchRiskProfile(responseHandler, params);
        }
        else {
            TestPlanClient.postBulkChangeTestPlan(responseHandler, params);
        }
    }
});

// Summary:
//    Specific class to handle test suite bulk changes.
//
dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.action.TestSuiteBulkChangeAction", 
             com.ibm.rqm.planning.web.ui.internal.model.action.BulkChangeAction, 
{
    artifactType: Constants.TYPE_SUITE,
    
    invokePost: function(responseHandler, params)
    {
        this.inherited("invokePost", arguments);
        
        // the suite bulk change service expects an different name for the ids...sigh.
        params.ids = params.uid;
        delete params.uid;
        com.ibm.rqm.execution.web.client.internal.execution.TestExecutionSuiteClient.postBulkChangeTestSuite(responseHandler, params);    
    }
});

})();
