/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation  2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

 //
 // Summary:
 //   Single event handler for button and menu Duplicate Events.
 //	  Manages invoking and monitoring via Progress the duplicate action. 
 //


dojo.provide("com.ibm.rqm.planning.web.ui.internal.model.action.DuplicateAction");
dojo.require("dijit.Menu");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.CopyClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.DuplicateDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.AbstractAction");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.ActionEntityType");	
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.MonitorableAction");
dojo.require("com.ibm.rqm.planning.web.ui.internal.wizard.ArtifactCopyWizard");
dojo.require("com.ibm.asq.common.web.ui.internal.wizard.WizardDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.QueryableProjectArea");

(function()
{
	var QueryableProjectArea = com.ibm.rqm.planning.web.client.internal.QueryableProjectArea;	
	var CopyClient = com.ibm.rqm.planning.web.client.internal.CopyClient;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;	
	var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
	var DuplicateDialog = com.ibm.rqm.planning.web.ui.internal.view.construction.DuplicateDialog;
	var Widget = dojo.widget;
	var DuplicateAction = com.ibm.rqm.planning.web.ui.internal.model.action.DuplicateAction;
	var ActionEntityType = com.ibm.rqm.planning.web.ui.internal.model.action.ActionEntityType;	
	var MonitorableAction = com.ibm.rqm.planning.web.ui.internal.model.action.MonitorableAction;	
	var ArtifactCopyWizard = com.ibm.rqm.planning.web.ui.internal.wizard.ArtifactCopyWizard;
	var WizardDialog = com.ibm.asq.common.web.ui.internal.wizard.WizardDialog;
	var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

	var ProgressMonitor = {};	
	DuplicateAction.TASK_SIZE = 100;	
	DuplicateAction.INTERVAL = 1000;		

	dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.action.DuplicateAction", MonitorableAction, {
		_completeCallback: null,
		_errorCallback: null,
		_button: null,
		_dupDialog: null,
		_pm: null,
		_idArray: null,
		_typeTable: null,
		_type: null,
		_handle: null,
		_haveFeedback: true,
		_useCurrentProjectArea: false,
		_openAfterComplete: false,
		_correlationWizard: null,

	   // Setup a table of the various entities that can be copied
	  constructor: function(args){
			
			this._typeTable = []
			this._typeTable[ActionEntityType.TEST_CASE] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeTC"),
						itemTypeName : "VersionedTestCase",
						itemTypeNSUri: "com.ibm.rqm.planning"
					};
			this._typeTable[ActionEntityType.AUTOMATED_TEST_SCRIPT] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeScript"),
						itemTypeName : "RemoteExecutionScript",
						itemTypeNSUri: "com.ibm.rqm.planning"
					};		
			// Just for display purposes, when multiple scripts are selected of different types display this type, and let the 
			// action figure out the details
			this._typeTable[ActionEntityType.TEST_SCRIPT] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeScript"),
						itemTypeName : "VersionedExecutionScript",
						itemTypeNSUri: "com.ibm.rqm.planning"
					};
			this._typeTable[ActionEntityType.MANUAL_TEST_SCRIPT] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeScript"),
						itemTypeName : "VersionedExecutionScript",
						itemTypeNSUri: "com.ibm.rqm.planning"
					};
			this._typeTable[ActionEntityType.TEST_SUITE] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeTSuite"),
						itemTypeName : "VersionedExecutionRecord",
						itemTypeNSUri: "com.ibm.rqm.execution"
					};

			this._typeTable[ActionEntityType.TEST_PLAN] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeTP"),
						itemTypeName : "VersionedTestPlan",
						itemTypeNSUri: "com.ibm.rqm.planning"
					};
			this._typeTable[ActionEntityType.KEYWORD] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeKeyword"),
						itemTypeName : "Keyword",
						itemTypeNSUri: "com.ibm.rqm.keyword"
					};
			this._typeTable[ActionEntityType.REQUIREMENT] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeRequirement"),
						itemTypeName : "WorkItem",
						itemTypeNSUri: "com.ibm.team.workitem"						
					};
			this._typeTable[ActionEntityType.DATAPOOL] =	{ 
						displayString : Constants.getMessage("labelDuplicatingTypeDatapool"),
						itemTypeName : "Datapool",
						itemTypeNSUri: "com.ibm.rqm.datapool"						
					};
		},
		
		// Two interfaces for determining if duplicate is allowed. These call back to
		// callback. The first looks to see if there are any project areas to duplicate to.
		// If so duplicate is allowed. The second looks to see if the current project area
		// is on the list of project areas that the user is allowed to duplicate to.
		// 
		// Permissions are determined by entity type.
		
		isDuplicateAllowed: function(type, callback){
			var args = {};				
			args.maxResults = 1;
			args.itemTypeNSUri = this._typeTable[type].itemTypeNSUri;
			args.itemTypeName = this._typeTable[type].itemTypeName;
			this.isAllowedCallback = callback;
			
			var srh = new ServiceResponseHandler(this, "_isAllAllowed", "_buildProjectAreaListFailed");
			QueryableProjectArea.getInitialProjectAreas(srh, args);
		},
		
		isDuplicateAllowedInCurrentProjectArea: function(type, callback){
			var args = {};				
			args.maxResults = 50;
			args.itemTypeNSUri = this._typeTable[type].itemTypeNSUri;
			args.itemTypeName = this._typeTable[type].itemTypeName;
			this.isAllowedCallback = callback;			
			this.current = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			if(this.current){
				args.namePrefix = this.current.name; 
				var srh = new ServiceResponseHandler(this, "_isCurrentAllowed", "_buildProjectAreaListFailed");
				QueryableProjectArea.getSomeProjectAreas(srh, args);
			}
			else {
				this._isCurrentAllowed(null);				
			}				
		},

		// Callback to see if there are any project areas that the user can duplicate to.		
		_isAllAllowed: function(userProjectAreas){
			var val = false;
			if(userProjectAreas && userProjectAreas.length > 0){
				val = true;
			}
			this.isAllowedCallback(val);
		},		
		
		// Callback from query. Build the items based on the return of the query
		_isCurrentAllowed: function(userProjectAreas)
		{
			var val = false;
			if(userProjectAreas && userProjectAreas.length > 0){
				for(var i = 0; i < userProjectAreas.length; i++){
					if(userProjectAreas[i].name == this.current.name){
						val = true;
						break;	
					}
				}
			}
			this.isAllowedCallback(val);
		},
		
		_buildProjectAreaListFailed: function()
		{
			this.isAllowedCallback(false);
		},
		
		openAfterComplete: function()
		{
			this._openAfterComplete = true;	
		},
		
		runUsingCurrentProjectArea: function(){
			this._useCurrentProjectArea = true;	
		},
		
		setType: function(type){
			this._type = type;	
		},
		
		// Context as in duplicate this test case, by the way the test case
		// is part of a test plan, update the test plan to include the newly 
		// created test case.
		
		setContext: function(containerType, containerId){
			this._containerType = containerType;
			this._containerId = containerId;
		},
		
		// Execute the copy callback on completed or error
		execute: function(idArray, completeCallback, errorCallback)
		{
			this._completeCallback = completeCallback;
			this._errorCallback = errorCallback;
			this._idArray = idArray;
			if(!this._idArray)
				return;
			
			var dialogTitle = Constants.getMessage("msgDuplicating1", this._idArray.length, this._typeTable[this._type].displayString);
			var dialogArgs = {onDuplicateCancel: dojo.hitch(this, "_onCancel"),
							  onDuplicateOk: dojo.hitch(this, "_onOk"),
							  title: dialogTitle,
							  appArgs: {  "itemTypeName" :this._typeTable[this._type].itemTypeName,
										  "itemTypeNSUri": this._typeTable[this._type].itemTypeNSUri }							  
							 };

			var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			this._sourceProjectArea = currentProjectArea;
		    if(this._useCurrentProjectArea){
				this._onOk(currentProjectArea);
			}							 
			else{
				this._dupDialog = new DuplicateDialog(dialogArgs);
			}
		},

		_onCorrelationCancel: function(jobId){
			this._correlationWizard._dialog.close();
			this._correlationWizard._dialog.destroy();
		    var callbackObj = {
                success: function(results){},
                failure: function(error){}
            }
            var responseHandler = new ServiceResponseHandler(callbackObj, "success", "failure");
            CopyClient.postCancel(responseHandler,{uuid:jobId});
		},

		// Get the percentage of the job done given a handle to a status object
		getWorkDone: function(copyStatus){
			if(copyStatus && dojo.isArray(copyStatus.items)){
				var total = copyStatus.items.length;
				var current = 0;
				if(dojo.isArray(copyStatus.copiedItems))
					current = copyStatus.copiedItems.length;
				if(!this._correlationWizard && this._getCorrelations(copyStatus)) {      
				    var dialogContent = new ArtifactCopyWizard(
				        {jobId: copyStatus.itemId,
				        sourcePA: this._sourceProjectArea.name,
				        targetPA: this._targetProjectArea.name,
				        cancelCallback:dojo.hitch(this,"_onCorrelationCancel"),
				        params:copyStatus}
				    );
                    this._correlationWizard = WizardDialog.create(dialogContent,true);
				}
				return current*DuplicateAction.TASK_SIZE/total;
			}
			return 100;
		},
		
		// The copy completed notify client, and open in editor if requested 
		actionComplete: function(copyStatus){
			if(this._completeCallback)
				this._completeCallback(copyStatus);
			if(this._openAfterComplete){
				if(copyStatus && dojo.isArray(copyStatus.copiedItems) && copyStatus.copiedItems.length > 0){
					var copiedItem = copyStatus.copiedItems[0];
					if(copiedItem){
						var itemId = copiedItem.internalId;
						// Datapool doesn't have an internal id. TODO Is this a server-side defect?
						if(itemId == null){
							itemId = copiedItem.itemId;
						}
						if(itemId != null)
							this.openItem(itemId);
					}
				}
			}
		},
		
		// Action failed, notify client
		actionFailed: function(e) {
			if(this._errorCallback)
				this._errorCallback(e);
		}, 
		
		// Call back for cancel from progress monitor, call goes through MonitorableAction
		cancelCallback: function(responseHandler, jobId)
		{
            CopyClient.postCancel(responseHandler,{uuid:jobId});
		},
		
		// poll server for progress
		pollServerForStatus: function(srh, copyStatus)
		{
			if(copyStatus){
				var appArgs = {};
				appArgs.uuid =  copyStatus.itemId;
				CopyClient.getStatus(srh, appArgs);
			}
		},

		// Get the feedback message for the operation		
		getFeedbackMessage: function(){
			return Constants.getMessage("msgDuplicating2",  this._targetProjectArea.name);
		},
		
		//Cancel options from the Project Selector dialog
		_onCancel: function()
		{
		},
		
		_onOk:function(projectArea)
		{
			this._targetProjectArea = projectArea;
			
			var appArgs = new Object();
			if(this._containerType){
				appArgs.containerItemTypeNSUri =  this._typeTable[this._containerType].itemTypeNSUri;
				appArgs.containerItemTypeName = this._typeTable[this._containerType].itemTypeName;
			}	
			if(this._containerId)
				appArgs.containerItemId = this._containerId;				
			appArgs.itemTypeName =  this._typeTable[this._type].itemTypeName;
			appArgs.itemTypeNSUri = this._typeTable[this._type].itemTypeNSUri;
			appArgs.targetProjectAreaItemId = projectArea.itemId;
			appArgs.itemIds = [];
									
			for (var i = 0; i < this._idArray.length; i++) {				
				appArgs.itemIds[i] =  this._idArray[i];
			}
			var srh = this.monitorService(DuplicateAction.INTERVAL, DuplicateAction.TASK_SIZE);
			CopyClient.postCopyItemsAsynch(srh, appArgs);
		},
		
		openItem: function(internalId){
			var hash = this._getArtifactUri(internalId);
			if(hash)
				window.location.hash = hash;
		},
		
		// Get artifact type to open in editor
		_getArtifactUri: function(internalId) {
            var artifactType = this._type;
			

            switch(artifactType){
                case ActionEntityType.TEST_PLAN:
                    return Utility.getTestPlanURI({id: internalId});   
                case ActionEntityType.TEST_CASE:
                    return Utility.getTestCaseURI(internalId);
				case ActionEntityType.TEST_SUITE: 
					return Utility.getTestSuiteURI(internalId);
				case ActionEntityType.TEST_SCRIPT: 								
				case ActionEntityType.MANUAL_TEST_SCRIPT: 								
				case ActionEntityType.AUTOMATED_TEST_SCRIPT: 				
					return Utility.getTestScriptURI(internalId);
				case ActionEntityType.DATAPOOL:					
					return Utility.getViewDatapoolURI(internalId)
					break;								
                default:
                    break;
            }
            return null;      
        },
        
        // returns true if there are correlations to get and false otherwise
        _getCorrelations: function(copyStatus) {
            return copyStatus.planCategories || 
                   copyStatus.caseCategories || 
                   copyStatus.suiteCategories || 
                   copyStatus.risks ||
                   copyStatus.objectives ||
                   copyStatus.planTemplates ||
                   copyStatus.caseTemplates ||
                   copyStatus.resourceLocs ||
                   copyStatus.teams;            
        }
	
	});
})();
