//debug MonitorableAction
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation  2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

 //
 // Abstract action interface
 // Support executing an RQM action. Implementers of the  action create a cetral location where all actions are shared. 
 // 

dojo.provide("com.ibm.rqm.planning.web.ui.internal.model.action.MonitorableAction");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.AbstractAction");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressStatusMonitor");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function()
{
	var AbstractAction = com.ibm.rqm.planning.web.ui.internal.model.action.AbstractAction;	
	var ProgressUtility = com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressUtility; 
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
		
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.action.MonitorableAction", AbstractAction, {
		_haveFeedback: true,
		_status: null,
		_taskSize: 0,
		_interval: 1000, 
		_cancelPending: false, 
  		 
			// Create any monitorable service via this monitorService. Watches the 
			monitorService: function(interval, taskSize)
			{
				this._interval = interval;
				this._taskSize = taskSize;
				return new ServiceResponseHandler(this, "_startActionSuccess", "_startActionFail");																				
			},	   	
				   
			//  Clents call to direct service to run with no feedback
			runWithNoFeedback:function()
			{
				this._haveFeedback = false;
			},
			
			// Clients implement to tell user how much work has been done so far, relative to total work
			getWorkCompleted: function(status){
			},
		
			// Clients implent to poll server for progress status
			pollServerForStatus: function(){
			},

			// Clients implemnt to supply a message during feedback, called after some amount of work is done
			getFeedbackMessage: function(){
			},
			
			// Clients subclasses implement cancelCallback to tell the server that the operation was cancelled.
			// srh - The cancel service request handler
			// id - the id of the job being canceled.
			cancelCallback: function(srh, id){
			},
			
			// Client implements to tell external UI objects that the action was cancelled
			actionCancelled : function(){
			},
			
			// Client implements to tell external UI objects that the action was complete
			actionComplete: function(status){
			},
		
			// Client implemetns to tell external UI objects that the action failed
			actionFailed: function(e) {
			},
			
			// Service monitors for cancel selection
			_monitorCancelCallback: function()
			{
				var srh =  new ServiceResponseHandler(this, "_monitorCancelSuccess", "_monitorCancelFail");																																
				this._cancelPending = true;
				var jobId = this._status.itemId;
				this.cancelCallback(srh, jobId);
			},
			
			// The cancel request failed. This shouldn't happen
			_monitorCancelFail: function(e){
				this._cancelPending = false;
				this.actionFailed(e);				
			},
			
			// The operation was cancelled. This is a server notification that the cancel was complete
			_monitorCancelSuccess: function(){
				this._cancelPending = false;
				ProgressUtility.endTaskMonitor(this.pm);
				this._endTimer();				
				this.actionCancelled();
			},
			
			// Service was started, now poll it for status
		   _startActionSuccess: function(status) {
				this.progressTimer = setInterval(dojo.hitch(this, "_pollMonitor"), this._interval);
			 	this._status = status;												   	
				if(this._haveFeedback)
					this.pm = ProgressUtility.beginTaskMonitor(this.getFeedbackMessage(), this._taskSize, dojo.hitch(this, "_monitorCancelCallback"));					
	       },
		   
			// Service failed to start
          _startActionFail: function(e) {
                this.actionFailed(e);
          },
			
		   // Poll the for status on the server, subclass impleents actual polling
		   _pollMonitor: function(){
				var srh = new ServiceResponseHandler(this, "_pollMonitorSuccess", "_pollMonitorFailure");
				this.pollServerForStatus(srh, this._status);																									
		   },
			   
	       _pollMonitorSuccess: function(status) {
				var workDone = this.getWorkDone(status);
				if(this._haveFeedback)									
					ProgressUtility.updateTask(this.pm, workDone);
				if(workDone == this._taskSize){
					if(this._haveFeedback)
						ProgressUtility.endTaskMonitor(this.pm);	
					this._endTimer();
					this.actionComplete(status);
				}
	        },

			// A polling pass failed, something has gone wrong on the server
            _pollMonitorFailure: function(err) {
            	var error = (err && err.message)?err.message:err;
				if(this._haveFeedback)
			   		ProgressUtility.endTaskMonitor(this.pm, error);
				this._endTimer();						
				this.actionFailed(err);
			},

			// Stop the polling timer, called when the action completes or fails				
			_endTimer:function(){
				if(this.progressTimer){
					clearInterval(this.progressTimer);
					delete this.progressTimer;
			     }
			}
	});
})();
