/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function()
{
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.QMPage");
    
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.GlobalTaskBar");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ActionBar");
dojo.require("com.ibm.rqm.planning.web.ui.internal.page.home.HomePageController");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

var PlatformUI 			 = net.jazz.ajax.ui.PlatformUI;
var X_AUTH_MSG           = "X-com-ibm-team-repository-web-auth-msg";
var Constants            = com.ibm.rqm.planning.web.client.internal.Constants;
var UserPreferenceClient = com.ibm.rqm.planning.web.client.internal.PreferenceClient;
var GlobalEvents         = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;

var ACTION_ROOT = "com.ibm.rqm.planning.home.";
var DISPATCHER  = "actionDispatcher";
var HOME_ACTION = "viewUserHome";
var SHOW_QUERY_ACTION = "showCurrentQuery";
var DISPATCHER_ACTION = "actionDispatcher";

dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.QMPage", null,
{
    isFromNonRQMPage: false,
    
    constructor: function(args) {
        // NOTE: The 'ui' object is constructed by the base jazz application class, jazz.app._JazzApplication
        // It's of the type jazz.app.JazzUI defined in net.jazz.web.app. 
        this.ui = args.ui;

        this.bannerDashboardButton = null;
        this.bannerDashboardButtonNotFound = false;

        // Actions should be in the scope of the HomePageView and not in the scope of the application.
        // However, to have our actions appear in the navBar of the common banner, we need to have the
        // actions registered so that loading a Jazz page within RQM, without HomePageView loaded, can
        // have the actions available in the navBar to dispatch the actions to the proper RQM page.
        // 
        this.controller = new com.ibm.rqm.planning.web.ui.internal.page.home.HomePageController(this);
        dojo.setObject("com.ibm.rqm.web.homePageController", this.controller);

        // register actions to the Action Registry
        var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
        ActionRegistry.registerAction(ACTION_ROOT + SHOW_QUERY_ACTION, this.controller, SHOW_QUERY_ACTION);
        ActionRegistry.registerAction(ACTION_ROOT + DISPATCHER_ACTION, this.controller, DISPATCHER_ACTION);

        // set the global navBar for the horizontal menu bar to create menu item entries
        dojo.setObject("com.ibm.rqm.web.globalNavBar", this.ui._navbar);
        
        // create the global task bar with Admin and Preference menu, and search box
        this._globalTaskBar = new com.ibm.rqm.planning.web.ui.internal.view.common.GlobalTaskBar();
        this._globalTaskBar.initialize({
            navbar: this.ui._navbar,
            containerLeft: this.ui.navbarLeft,
            containerRight: this.ui._banner._rightButtons
        });

        // check user preference to create horizontal menu bar
        UserPreferenceClient.getCurrentUserPreferences(this, this.showHorizontalActionBar, Constants.USER_UI_CUSTOMISE_ACTIONBAR_CATEGORY.key);

        // set rtl attribute:
        if(com.ibm.rqm.planning.web.ui.internal.util.EditorUtil.isDirectionRtl()) {
            this.ui._content.setAttribute("dir", "rtl");
        }

        // subscribe to viewPage event from the workbench to check if the user
        // navigated to a Jazz page away from the RQM page
        dojo.subscribe(net.jazz.ajax.internal.events.viewPage, this, "_viewPage");
    },
    
    showHorizontalActionBar: function(userPreference)
    {
        if (userPreference.key == Constants.USER_UI_CUSTOMISE_ACTIONBAR_CATEGORY.key) {
            if (userPreference.value == "actionBarHorizontal") {
                var dispatcherName = ACTION_ROOT + DISPATCHER;
                // create the horizontal menu bar
                var menuBar = new com.ibm.rqm.planning.web.ui.internal.view.common.ActionBar();
                // HomePageView may not be created yet so content area for the action entries
                // is not set but will be initialized upon run time
                menuBar.initialize(null, null, dispatcherName, true);
            }
        }
    },
    
    _viewPage: function(page) {
        var homelink = dojo.getObject("com-ibm-rqm-planning-web-home-link");
        var homeButton = dojo.getObject("com-ibm-rqm-planning-web-home-widget");
        
        // run this once to locate the banner RQM dashboard button
        if (!this.bannerDashboardButton && !this.bannerDashboardButtonNotFound) {
            var headerButtons = jazz.app.currentApplication.ui._banner.buttons;
            for (var i = 0; i < headerButtons.length; i++) {
                if (headerButtons[i].label == Constants.getMessage("labelDashboards")) {
                    this.bannerDashboardButton = headerButtons[i]._widget;
                    break;
                }
            }
            this.bannerDashboardButtonNotFound = this.bannerDashboardButton == null;
        }

        if (page && page.id && page.id == "com.ibm.rqm.planning.web.HomePageView") {
            // restore the banner RQM dashboard button style when the RQM page is shown
            if (this.bannerDashboardButton) {
                if (dojo.getObject("com-ibm-rqm-planning-web-dashboard-selected")
                    && !dojo.hasClass(this.bannerDashboardButton.domNode, "selected")) {
                    dojo.addClass(this.bannerDashboardButton.domNode, "selected");
                }
            }

            // showing a RQM page, the id is defined in plugin.xml
            if (homelink) {
                homelink.style.visibility = "hidden";
            }
            if (homeButton) {
                // update home button style to reflect if the "Home" tab is selected
                var homeSelected = window.location.hash.indexOf("&subAction=viewUserHome") > 0;
                homeButton.attr('disabled', homeSelected);
            }
            
            
            var workbench = PlatformUI.getWorkbench();
    		var rootNode = workbench.rootNode();
            if (workbench.browserTitle && workbench.browserTitle != Constants.getMessage("labelLoading")) {
                workbench.setTitle(workbench.browserTitle);
            }
            
            if(this.isFromNonRQMPage){
                dojo.publish("RQM-Event-Back-From-Print-Preview",[]);
                this.isFromNonRQMPage = false;
                
                // we came back from the Jazz's pages. Make sure that
                // all of its unload/body click handlers are all clear.
                // Otherwise, navigational click could still trigger
                // unwanted confirm dialog about unsaved changes, but the
                // editor has long been unloaded.
                workbench._onBeforeUnloadHandlers = new Array(); 
                workbench._bodyClickHandlers = new Array();
                
                // clear out non-RQM widgets from the workbench page cache                
		    	var widgetCache = workbench._pageWidgetCache;
		    	for (var wid in widgetCache) {
		    		if (wid == "com.ibm.team.workitem") {
		    			var widget = widgetCache[wid];

		    			// clean up recent work item list
						var recentWorkItemView = widget._recentWorkItemWidget;
						var rows=recentWorkItemView.getRows();
						for(var i =0; i< rows.length; i++ ) {
							recentWorkItemView._disconnectNode(rows[i].node);
							rows.splice(i,1);
						}
		    			var multipaneController = widget._multipaneContentWidget.controller;
		    			var widgets = multipaneController._widgetCache;
    					dojo.forEach(widgets, function(child) {
							try {
								var w = child.widget;
								var id = w.id;
								recentWorkItemView.removeItem(id);
								if (w.destroyWidget){
									w.destroyWidget();
								}
								if(w.declaredClass != "com.ibm.team.workitem.web.ui.internal.page.WorkItemWelcome") {
									// destroying the workitemwelcome causes an error
									// when you open the view requirements page, click on a requirement id,
									// in the opened requirement click "Back to View Requirements" and then 
									// click on another requirement id.
									if(w.destroy) w.destroy();
								}
							} catch(e) {
								console.debug('Error destroying: ' + child);// ignore
							}
						});
		    			
		    			multipaneController.destroy();
		    			multipaneController._widgetCache=[];
		    			widget.resetFilter();
		    			// THIS CAUSES BAD THINGS: widget.destroy();
//		    			if (widget && widget.destroyRecursive) {
//		    				widget.destroyRecursive();
//		    				delete widgetCache[wid];
//		    			}
		    		}
		    	}
//		    	if (this.nonRQMPage && this.nonRQMPage.domNode){
//		    		
//		    		var list = dojo.query('[widgetId]', this.nonRQMPage.domNode);
//					var a = list.map(dijit.byNode);		// Array
//					dojo.forEach(a, function(widget){ 
//						if(widget.destroy){
//							widget.destroy();
//						}
//		    		});
//		    		this.nonRQMPage=null;
//		    	}
            }
            
            if(window.location.hash.indexOf(HOME_ACTION) > 0) {
                //when change to Home tab, publish an event that viewlets can subscribe to
                dojo.publish(GlobalEvents.DashboardFocus, []);
            }
            
            // save the uri of the latest non-rqm page
            this.lastPageURI = window.location.hash;
        } else {
//        	this.nonRQMPage = page;
            // deselect the banner RQM dashboard button if not showing a RQM page
            if (this.bannerDashboardButton) {
                dojo.removeClass(this.bannerDashboardButton.domNode, "selected");
            }

            // showing a non-RQM page, ensure the back to RQM home link is visible
            if (homelink && !this.isFromNonRQMPage) {

                // we only have a single widget, homePageView, in the Jazz page
                // of which always has a single unique content area
                var homePageViewContentArea = dijit.byId("one-and-only-one-com-ibm-rqm-web-home-page-content-area");
            
                // get the selected tab of the content area for its title
                var selectedTab = homePageViewContentArea ? homePageViewContentArea.getLastSelectedTab() : null;

                // get the label of the selected tab
                var selectedTabTitle = selectedTab ? selectedTab.controlButton.label : null;
                
                // the selected RQM tab will not be defined if a non-RQM page is refreshed
                homelink.innerHTML = selectedTabTitle ? Constants.getMessage("labelBackRQMPage", selectedTabTitle) : Constants.getMessage("labelBackRQMHome");

                homelink.style.visibility = "visible";
                // Use previous RQM page URI for the link.  The link handler will use the Home page URI
                // if this.lastPageURI is not defined.
                homelink.backURI = this.lastPageURI;

                this.isFromNonRQMPage = true;
            }
            if (homeButton) {
                homeButton.attr('disabled', false);
                homeButton._focused = false;
				//WI 29743
				if(!this.isFromNonRQMPage)
					this.isFromNonRQMPage = true;
            }
        }
    }
});

})();
