/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.home.HomePageController");

dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function()
{/** * Main page controller */

var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.home.HomePageController", null, {
        constructor: function(myView){
            this._view = myView;
            this._subActionMap = {};
            
            // subscribe/listen to these page control events: 
            //    update the tab page's URI
            //    update the tab page's display label
            //
            var uriEvent = GlobalEvents.UpdateTabUri;
            var labelEvent = GlobalEvents.UpdateTabLabel;
            
            dojo.subscribe(uriEvent, this, "onUpdateTabPageUri");
            dojo.subscribe(labelEvent, this, "onUpdateTabPageLabel");
            
            // subscribe/listen to artifacts changed events
            dojo.subscribe(GlobalEvents.ArtifactsUpdated, this, "onArtifactsUpdated");
            dojo.subscribe(GlobalEvents.ArtifactsDeleted, this, "onArtifactsDeleted");
            dojo.subscribe(GlobalEvents.ArtifactsUndeleted, this, "onArtifactsUndeleted");
            dojo.subscribe(GlobalEvents.ArtifactsAdded, this, "onArtifactsAdded");
            
            dojo.subscribe(GlobalEvents.CloseTab, this, "onTabClose");
        },
        
        setView: function(myView) {
            this._view = myView;
        },
        
        // Summary:
        //    This method hooks the controller to the view's contentArea's tab container's page change
        //    events. When the tab's changed by the user, the controller will receives the event and 
        //    it updates the window's location to match the tab page's URI.
        //
        initTrackTabChange: function(){
            dojo.subscribe(this._view._contentArea._tabs.id + "-selectChild", this, "onTabChanged");
            this.onUpdateUri = false;
        },
        
        showCurrentQuery: function(){
            this._view.showCurrentQuery();
        },
        
        actionDispatcher: function(args){
            if (this.onUpdateUri) {
                this.onUpdateUri = false;
                return;
            }
            // don't have to dispatch to the sub action handler if the content area
            // already has page. We'll just bring it up.
            //
            if(this._hasCachedPageForCurrentLocation(true)) {
                return;
            }
			
			// CML: check for existence of Home page
			if (!this._view._contentArea.hasHomeTab())
			{
                // if the action is to be dispatched is the home view action,
                // don't have to explicitly create the home tab. The home tab
                // will be created as a result of this action dispatch.
                if(args.subAction && args.subAction != this._view.homeViewActionTag)
				    this._view.showUserDashboard({});
			}
            
            if (args.subAction) {
                var func = this._subActionMap[args.subAction];
                
                if (func) {
                    func(args);
                }
                else {
                    var actionReg = dojo.getObject(com.ibm.rqm.planning.web.client.internal.Constants.ACTION_BAR_REGISTRY_OBJECT_ID);
                    var deferredArg = args;
                    var deferredActionDispatch = args.subAction;
                    if (actionReg && actionReg._doInitCallback) {
                        this.actionRegistryCompleted = dojo.connect(actionReg, "_doInitCallback", this, function(){
                            var func = this._subActionMap[deferredActionDispatch];
                            
                            if (func) {
                                func(deferredArg);
                            }
                            else {
                                console.debug("Non registered dispatch action: " + deferredActionDispatch);
                            }
                            dojo.disconnect(this.actionRegistryCompleted);
                            delete this.actionRegistryCompleted;
                        })
                    }
                }
            }
        },
        
        // Summary:
        //    Returns true if the current window.location already maps to an existing
        //    page. Optionally, if the 'show' parameter is set to true, the matched page
        //    is also brought to the front. 
        //    
        _hasCachedPageForCurrentLocation: function(show)
        {
            var uri = window.location.hash;
            var hasIt = this._view._contentArea.hasPage(uri);
            
            if(show && hasIt)
                this._showCachedPageForCurrentLocation();
                
            return hasIt; 
        },
        
        // Summary:
        //    Shows (brings to the front) the page in our content area that matches the 
        //    current window location.
        _showCachedPageForCurrentLocation: function()
        {
            var uri = window.location.hash;
            this._view._contentArea.showPageByUri(uri);
        },

        // Summary:
        //    Registers a sub action to be dispatched by the controller. When
        //    an URI that maps to this controller's 'actionDispatcher' method,
        //    this controller searches for the registered sub action and fires
        //    the corresponding method to carry out the URI action.
        //
        registerDispatchAction: function(name, method){
            this._subActionMap[name] = method;
        },
        
        // Summary:
        //    Handler that gets fired when the tab's current selection has changed.
        //    Update the window's URI to the tab's.
        // 
        onTabChanged: function(page){
            setTimeout(function(){
                var windowLoc = window.location;
                var curr_uri = windowLoc.hash;
                var pageUri = page.uri;
                // only update the uri if it's different.
                if (pageUri && pageUri != curr_uri) {
                    this.onUpdateUri = true;
                    windowLoc.hash = pageUri;
                }
            })
            
        },
        
        // Summary:
        //   Handler for the update Tab page's URI event. This occurs when an artifact
        //   page's id has changed and its URI needs to be updated.
        //
        onUpdateTabPageUri: function(oldUri, newUri) {
            var updated = this._view._contentArea.updatePageUri(oldUri, newUri);
            
            // replace the current window's location with the new URI. This
            // removes the old URI and uses the new one in its place. Any back/forward
            // button changes will now go to the replaced URI.
            if(updated) {
                this.onUpdateUri = true;
                window.location.replace(newUri);
            }
        },

        // Summary:
        //   Handler for the update Tab page's label event. This occurs when an artifact
        //   page's name has changed and its tab label needs to be updated.
        //
        onUpdateTabPageLabel: function(uri, newLabel, iconClass, dirty, skipCompactLabel) {
            this._view._contentArea.updatePageLabel(uri, newLabel, iconClass, dirty, skipCompactLabel);
        },
        
        // Summary:
        //   Handler for the artifacts updated event. This occurs when an artifact
        //   has changed and other tabs that have reference to it need to be updated.
        //
        onArtifactsUpdated: function(artifactType, artifactArray, fromURI) {
            this._view._contentArea.pageContentChanged(GlobalEvents.ArtifactsUpdated, artifactType, artifactArray, fromURI);
        },
        
        // Summary:
        //   Handler for the artifacts deleted event. This occurs when artifacts are
        //   deleted and other tabs that have reference to them need to be updated.
        //
        onArtifactsDeleted: function(artifactType, artifactArray, fromURI) {
            this._view._contentArea.pageContentChanged(GlobalEvents.ArtifactsDeleted, artifactType, artifactArray, fromURI);
        },

        // Summary:
        //   Handler for the artifacts deleted event. This occurs when artifacts are
        //   deleted and other tabs that have reference to them need to be updated.
        //
        onArtifactsUndeleted: function(artifactType, artifactArray, fromURI) {
            this._view._contentArea.pageContentChanged(GlobalEvents.ArtifactsUndeleted, artifactType, artifactArray, fromURI);
        },

        // Summary:
        //   Handler for the artifacts added event.
        //
        onArtifactsAdded: function(artifactType, artifactArray, fromURI) {
            this._view._contentArea.pageContentChanged(GlobalEvents.ArtifactsAdded, artifactType, artifactArray, fromURI);
        },

        onTabClose: function(uri) {
            this._view._contentArea.closeTab(uri);
        }
        
    });
})();
