/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

(function(){
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
   	var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyEditor", [dijit._Widget, dijit._Templated], {
        templateString: '<div class="com-ibm-rqm-test-administrator">' +
                        '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" "noline"=true dojoAttachPoint="viewHeaderNode">' +
                        '   <table dojoAttachPoint="_tableNode" class="top-table">' +
                        '       <tr><td colspan="2"><span dojoAttachPoint="_title" class="title"></span></td></tr>' +
                        '       <tr>' + 
                        '           <td class="selector">' +
                        '                  <label><select dojoAttachPoint="_nameSelector" dojoAttachEvent="onchange: displayContent" size="21" class="table-content"></select></label>' +
                        '           </td>' +
                        '           <td class="content-area" dojoAttachPoint="_contentArea">' +
                        '           </td>' + 
                        '       </tr>' + 
                        '       <tr align="right"><td colspan="2" class="button-area" align="right" dojoAttachPoint="_buttonArea"></td></tr>' +
                        '   </table>' +
                        '  </div>' +
                        '</div>',
						
		widgetsInTemplate: true,
        
        labelSelectArea: Constants.getMessage("labelSelectArea"),
        defaultProperty: "", //default selection name
        _nameContentPairs: [],
        
        postCreate: function(){
            this.setTitle();
            this.setNameContentPairs();
            this.setButtonArea();
            
            this._initNameSelector();
            this._displayDefaultContent();
        },
		
		uninitialize: function(){
            this._destroyAllcontentsWidgets();
		},
        
        // Summary:
        //    The editor's contents are shown one page at a time. If the user
        //    opened more then one page in the editor, the other non-visible
        //    pages' widgets would not be attached to the visible dom tree.
        //    Calling individual widget's destroyRecursive would not find/
        //    destroy all decendant widgets. Create a temp node and attach 
        //    all non-attached content widget's domNode to it and destroy
        //    all widgets under it.
        //    
        _destroyAllcontentsWidgets: function(){
            var nameContentPairs = this._nameContentPairs;
            var length = nameContentPairs.length;
            var parentNode = this.domNode;
            var tempNode = null;
                    
            if (parentNode) {
                tempNode = document.createElement('div');
                
                // make the temp node non-visible.
                dojo.style(tempNode, "visibility", "hidden");
            }
            
            for (var i = 0; i < length; i++) {
                var widget = nameContentPairs[i].content;
                
                // attach the content's widget to the temp node, but only
                // if it's not attached to the dom tree.
                if (widget) {
                    // make sure the widget has a dom node, it should
                    // Otherwise, just call the destroy.
                    if (tempNode && widget.domNode) {
                        tempNode.appendChild(widget.domNode)
                    }
                    else if (widget.destroyRecursive) {
                        widget.destroyRecursive();
                    }
                }
            }
            
            // we have created a temp node, clean its widgets.
            if(tempNode) {
                parentNode.appendChild(tempNode);
                com.ibm.asq.common.web.ui.internal.util.Utility.destroyWidgetsFromNode(tempNode);
                parentNode.removeChild(tempNode);
                dojo._destroyElement(tempNode);
            }   
        },
        
        setTitle: function(){
			this.viewHeaderNode.setTitle(this.headerTitle);
        },
        
        /*
         * Abstract
         */
        setNameContentPairs: function(){
        
        },
        
        /*
         * Abstract
         */
        setButtonArea: function(){
        
        },
        
        _initNameSelector: function(){
            dojox.data.dom.removeChildren(this._nameSelector);
			var nameContentPairs = this._nameContentPairs;
			var length = nameContentPairs.length;
			var defaultProperty = this.defaultProperty;
			var nameSelector = this._nameSelector;
            for (var i = 0; i < length; i++) {
				var nameContentPair = nameContentPairs[i];
                var option = new Option(nameContentPair.name, nameContentPair.name);
				option.tabindex = "-1";
				option.title = nameContentPair.name;
				option.selected = (defaultProperty == nameContentPair.name);
				SelectUtility.add( nameSelector, option );
            }
        },
        
        _displayDefaultContent: function(){
			this._selectContent(this.defaultProperty);
        },

		_selectContent: function(name) {
            dojox.data.dom.removeChildren(this._contentArea);
			var nameContentPairs = this._nameContentPairs;
			var length = nameContentPairs.length;
            for (var i = 0; i < length; i++) {
				var nameContentPair = nameContentPairs[i];
                if (nameContentPair.name == name) {
					var content = nameContentPair.content;
					if(!content){
						nameContentPair.content = new nameContentPair.jsClass(nameContentPair.widgetArgs);
					}
					this._contentArea.appendChild(nameContentPair.content.domNode);
					this._currentContentArea = nameContentPair.content;
                    this._currentContentArea.onSelected();
                    break;
                }
            }
		},
		
		
        _reflectDirtySelector: function(name, dirty){
            var nameSelector = this._nameSelector;
            var optionSelector;

            for (var i = 0; i < nameSelector.options.length; i++) {
                if (nameSelector.options[i].value == name) {
                    optionSelector = nameSelector.options[i];
                    if(optionSelector){
                        if(optionSelector.text.substring(optionSelector.text.length-2)==" *"&& !dirty){
                            optionSelector.text = name;
                        }
        	            else if(optionSelector.text.substring(optionSelector.text.length-2)!=" *"&& dirty){
    	                   optionSelector.text = name + " *";
    	                }
                    }
                    return;
                }
            }
		   
        },
		
		        
        displayContent: function(event){
            this._selectContent(this.getName());
            if(event){
                dojo.stopEvent(event);
            }
        },
		
        getCurrentContentArea: function(){
            return this._currentContentArea;
        },
        
        getName: function(){
            return this._nameSelector.value;
        }
    });
    
})();
