/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactOptionsTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

(function() {

var TableConstants 		= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var Utility             = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var OptionsDialog    	= com.ibm.asq.common.web.ui.internal.widgets.OptionsDialog;
var Constants      		= com.ibm.rqm.planning.web.client.internal.Constants;

var RENDER_TYPE_OPT_CHECKBOX = "renderType.optBox";

dojo.declare("ArtifactOptionsTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	// Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        //do nothing     
    },
    
    // Summary:
    //    A group check box is shown when the tables rows are checkable and the in multi-select mode.
    //    This method is called when the group's check box is checked/unchecked. The affected rows and 
    //    their states are passed.
    //
    groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    {
        //do nothing
    },
	
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {                
        //do nothing
    }     
});

dojo.declare("ArtifactOptionsTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    constructor: function(args)   {	
        dojo.mixin(this, args);
		//expected params: provider
    },
			
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        //update the row object
        var profile = this.provider.getProfile();
        if (profile) {
			var colName = profile[columnIndex].name;
			switch (colName) {
				case this.provider.COLUMNS.LOCKING.name:
					rowObject.lockingEnabled = newValue;
					break;
                case this.provider.COLUMNS.SIGN_REVIEWS.name:
                    rowObject.reviewSignEnabled = newValue;
                    break;					
				case this.provider.COLUMNS.MAX_RESULTS.name:
					rowObject.maxResults = newValue;
					break;
				default:
					break;
			}
		}

        //invoke listener, if present
        if (dojo.isFunction(this.onChangeListener)) {
            this.onChangeListener(rowObject);
        }
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		//do nothing
    }
});

dojo.declare("ArtifactOptionsTableView.CheckboxCellRenderer",
            com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex)
    {
        var span = document.createElement("span");
        var isChecked = false;
        var tooltip = null;
        
        if(extendedDisplayInfo && (typeof extendedDisplayInfo['state'] != "undefined")) {
            isChecked = extendedDisplayInfo['state'];
            tooltip = extendedDisplayInfo['tooltip'];
        }
        
		var provider = viewer.getContentProvider();
		var column = provider.getProfile()[columnIndex];
		if (this._isOptionAvailable(provider, column, rowObject)) {
			var args = {
				'checked': isChecked
			};
			var button = new dijit.form.CheckBox(args);
			var editable = viewer.isEditable() && (provider ? provider.getIsCellEditable(rowObject, columnIndex) : false);
			button.setDisabled(!editable);
			
			span.appendChild(button.domNode);
	        // the parent class's attach tooltip will not create a tooltip
	        // if the string is null, or undefined. Don't have to check it here.
	        this._attachTooltipToCell(button.domNode, tooltip);    

	        // keep the associated attributes in the button.
	        // 
	        button.cellModifier = cellModifier;
	        button.rowObject = rowObject;
	        button.columnIndex = columnIndex;
	        button.onChange = function() {
	            if(this.cellModifier)
	                this.cellModifier.cellValueChanged(!this.checked, this.checked, this.rowObject, this.columnIndex);
	        }
			
			var title = this.getCellLabel(viewer, rowObject, columnIndex);
			if(title){
				var label = document.createElement("label");
				label.appendChild(document.createTextNode(title));
				label.setAttribute("for", button.focusNode.id); 
				label.style.display = "none";
				button.domNode.appendChild(label);
			}		
		}
        span.appendChild(document.createTextNode(cellData));
        return span;
    },
	
	_isOptionAvailable: function(provider, column, rowObject) {
        switch(column.name) {
            case provider.COLUMNS.LOCKING.name: 
                return rowObject.lockable;
            case provider.COLUMNS.SIGN_REVIEWS.name: 
                return rowObject.reviewable;
            default:
                return false;            
        }		
	}
    
});

dojo.declare("ArtifactOptionsTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    constructor: function(args) {
        this.COLUMNS = {
            ICON: this.toColumn(" ", 5, false, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_IMAGE),
            NAME: this.toColumn(Constants.getMessage("labelName"), 35, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT),
            LOCKING: this.toColumn(Constants.getMessage("labelLockingEnabled"), 15, false, TableConstants.ALIGN_CENTER, RENDER_TYPE_OPT_CHECKBOX),
			SIGN_REVIEWS: this.toColumn(Constants.getMessage("labelSignReviews"), 15, false, TableConstants.ALIGN_CENTER, RENDER_TYPE_OPT_CHECKBOX),
			MAX_RESULTS: this.toColumn(Constants.getMessage("labelMaxResults"), 30, true, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_EDITABLE_TEXT)
        }
		
        this.COLUMNS.ALL = [this.COLUMNS.ICON, this.COLUMNS.NAME, this.COLUMNS.SIGN_REVIEWS, this.COLUMNS.LOCKING, this.COLUMNS.MAX_RESULTS];
		
        this.columnProfile = null;
        this.columnProfileName = null;
        this.setProfile(this.profile ? this.profile : "ALL"); //default to 'ALL' (full) column profile 	
    },
    
    toColumn: function(name, width, groupable, alignment, type) {
        return {
            name: name,
            width: width,
            groupable: groupable,
            alignment: alignment,
            type: type
        }
    },  
	
    setProfile: function(/* string */ profile) {
        if (this.COLUMNS[profile]) {
            this.columnProfileName = profile;
            this.columnProfile = this.COLUMNS[profile];
        }
    },	
	
    getProfile: function() {
        return this.columnProfile;
    },	

    getColumnIndex: function(name) {
		var columnProfile = this.columnProfile;
		var length = columnProfile.length;
        for (var i = 0; i < length; i++) {
            if (columnProfile[i].name == name) {
                return i;
            }
        }
        return -1;
    },	
			
    // Summary:
    //
    getNumColumns: function(content)
    {
        return this.columnProfile.length;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        return this.columnProfile[index].name;
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {  
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].alignment;
    },   
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return false;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return false;     
    },

    getIsCellEditable: function(content, atColumnIndex)
    {
		var colName = this.columnProfile[atColumnIndex].name;
		if (colName == this.COLUMNS.LOCKING.name) {
			return content.lockable;
		}
        if (colName == this.COLUMNS.SIGN_REVIEWS.name) {
            return content.reviewable;
        }		
		return true;
    },
	    
    getColumnWidth: function(content, atColumnIndex)
    {
        var column = this.columnProfile[atColumnIndex];
        return column.width;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_artifactOption_" + rowData.typeName;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].alignment;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "";
        var colName = this.columnProfile[atColumnIndex].name;
        switch(colName) {
            case this.COLUMNS.NAME.name: 
                val = rowData.typeName;
                break;
            case this.COLUMNS.MAX_RESULTS.name: 
                val = rowData.maxResults;
                break;   
            default:
                break;            
        }
        return val;		
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = {};
        var colName = this.columnProfile[atColumnIndex].name;
        switch(colName) {
            case this.COLUMNS.ICON.name:
                val.image = Utility.getArtifactIconURL(rowData.typeName);			
				break;
            case this.COLUMNS.LOCKING.name: 
                val.state = rowData.lockingEnabled;			 
                break;
            case this.COLUMNS.SIGN_REVIEWS.name: 
                val.state = rowData.reviewSignEnabled;          
                break;				
            default:
                break;			        
        }
        return val; 		
    }
    
    
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactOptionsTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
                    '  <div>' +
                    '    <div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '  </div>' +
				    '</div>',

    editable: true,

    constructor: function() {
	 	this.artifactOptions = [];
		this.onChangeListener = null;
        this.showInitLoadProgress = false;
	 },

    postCreate: function() {
        this.inherited("postCreate", arguments);		

        var content = this.artifactOptions ? this.artifactOptions : [];
				
        //create table viewer       
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();		      
        this._tableViewerNode.appendChild(tableViewer.domNode);
        tableViewer.setContentPageSize(10);
        tableViewer.setContent(content);
        this.provider = new ArtifactOptionsTableView.TableProvider({
            profile: Constants.TABLE_PROFILE_ALL     
        }); 		
        tableViewer.registerCellRenderer(RENDER_TYPE_OPT_CHECKBOX, new ArtifactOptionsTableView.CheckboxCellRenderer());	
        tableViewer.setContentProvider(this.provider);
        tableViewer.setTableListener(new ArtifactOptionsTableView.TableListener(this));
        tableViewer.setCellModifier(new ArtifactOptionsTableView.CellModifier({
			provider: this.provider,
			onChangeListener: this.onChangeListener
		}));       
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelArtifactOptions")));
       		  
		this.tableViewer = tableViewer;
        if(this.showInitLoadProgress) {
            tableViewer.showLoadingProgress();
        }
        this.tableViewer.show();
        this.tableViewer.setEditable(this.editable);		  
    },
	
	// API
	getOptions: function() {
        return this.tableViewer.getContent();
	},
	
    refresh: function(content, skipClearStatus) {
        this.artifactOptions = content || [];
        this.tableViewer.setContent(content);
        this.tableViewer.refresh();
     }	
});

})();
