/***ExecutionStatesEditor******************************************************* 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.administration.ExecutionStatesEditor");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractContentArea");

dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStates");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStateClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveContext");

(function(){
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var SaveParticipant = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
	var SaveContext = com.ibm.rqm.planning.web.ui.internal.model.SaveContext;
	
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var ExecutionResultStates = com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStates;
	var ExecutionResultStateClient = com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStateClient;

	var DIALOG_STYLE = "position: absolute; top: 15%; left: 15%; width: 600px; height: 500px; z-index: 999";
	
	dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.administration.ExecutionStatesEditor",
	[dijit._Widget, dijit._Templated, com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractContentArea],
	{
        //		
	    // Provides editing capability for ExecutionStates
	    //
	    templateString: '<div class="com-ibm-rqm-execution-states-editor">' +
		                '  <div dojoAttachPoint="_loadingNode" class="hidden">${labelLoading}</div>' +
                        '  <div class="action-option-bar hidden">' +
                        '      <div class="actions" dojoAttachPoint="_actionsNode"></div>' +
						'      <div class="clear"></div>' +
						'  </div>' +
						'  <div dojoAttachPoint="_executionStatesTableNode" class="execution-states">' +
                        '      <table dojoAttachPoint="_executionStateEditorNode" class="execution-states-editor-table">' +
                        '          <tr class="execution-states-header">' + 
						'              <th dojoAttachPoint="_iconNode" class="icon-column"></th>' +
                        '              <th dojoAttachPoint="_nameNode" class="content-column">${labelName}</th>' +
                        '              <th dojoAttachPoint="_enabledNode" class="content-column">${labelEnabled}</th>' +
                        '              <!--th dojoAttachPoint="_actionsNode" class="actions-column"></th-->' +
                        '          </tr>' +  
                        '         <tbody dojoAttachPoint="_executionStatesNode">' +
                        '         <!-- individual execution state editor rows will be appended here -->' +
                        '         </tbody>' +
                        '      </table>' +					
						'  </div>' +
						'  <!--div class="options">' +
						'      <label><input type="checkbox" dojoAttachPoint="_showArchivedCheckbox" dojoAttachEvent="onchange: _onArchivedChange"></input></label>' +
						'      <span class="" dojoAttachPoint="_showArchivedLabel"></span>' +
						'  </div-->' +	                
		                '</div>',
        
		labelName: Constants.getMessage("labelName"),
		labelEnabled: Constants.getMessage("labelEnabled"),
		labelLoading: Constants.getMessage("labelLoading"),

		constructor: function() {
			this.csHelpId = "rqm_admin_prop.html";
			this.isDialog = false;
            this.executionStates = [];
			this.onSaved = null;  //optional callback fired on successful save
			this._editors = [];  //editor cache
			this._permissions = {};
		},		
	    
	    postCreate: function() {
			//determine the current user's permissions
            var operations = [];
            operations.push({
                id: PermissionsClient.OP_EXECUTION_STATE, 
                actions: [PermissionsClient.ACTION_EDIT]
            });
			
			var responseHandler = new ServiceResponseHandler(this, "_preinitPermissions", "_error");
            PermissionsClient.checkAllPermissions(responseHandler, operations);				
	    },
		
		_preinitPermissions: function(permissions) {
			this._permissions = permissions;			
	        this.initialize();
		},

        initialize: function() {			
			dojo.removeClass(this._loadingNode, "hidden");
			
			//retrieve the categoryTypes for display/edit			
			var self = this;
			var handlerObj = {
				success: function(executionStates) {
                    dojo.addClass(self._loadingNode, "hidden");
					self.setExecutionStates(executionStates);
					self.showExecutionStates();
					//if this is being shown as a dialog, init the dialog
		            if (self.isDialog) {
		                self.show();
		            }
				},
				
				failure: function(e) {
					self._error(e);
				}
			}
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            ExecutionResultStateClient.getExecutionResultStates(responseHandler, {includeHidden: true});
        },
		
        //////////////////////////////////////////////////////////////////////////////////////////
        ///                                        API                                         ///
        //////////////////////////////////////////////////////////////////////////////////////////  		
        show: function() {
            if (!this._dialog) {           
                var dialogArgs = {
					okCallback: dojo.hitch(this, "doSave"),
					cancelCallback: dojo.hitch(this, "hide")					
                };
                
                this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
                this._dialog.setWidth("600px");
                this._dialog.contentContainer.appendChild(this.domNode);
                this._dialog.setSecondaryTitle(Constants.getMessage("labelManageCategories"));
                this._dialog.getOkButton().disabled = false;                            
            }
            this._dialog.open();    						
		},		
		
		hide: function() {
			if (this._dialog && dojo.isFunction(this._dialog.hide)) {
				this._dialog.hide();
			}
		},
		
		setExecutionStates: function(states) {
			this.executionStates = states.sort(function(s1, s2) {
				//temp to deal with odd severities, append negative severities to end
				var sort1 = s1.severity >= 0 ? s1.severity : 1000;
				var sort2 = s2.severity >= 0 ? s2.severity : 1001;
				return sort1 < sort2 ? -1 : 1;
			});
		},
	
		showExecutionStates: function() {
			this._editors = [];
			dojox.data.dom.removeChildren(this._executionStatesNode);
			var executionStates = this.executionStates;
			var executionStatesLength = executionStates.length;
			var totalEnabled = 0;			
			for (var i = 0; i < executionStatesLength; i++) {
				if (executionStates[i].visible) {
					totalEnabled++;
				}
				var editor = new com.ibm.rqm.planning.web.ui.internal.view.administration.ExecutionStateEditor({
					parent: this,
					executionState: executionStates[i]
				});
				this._editors.push(editor);   //cache the editor
				this._executionStatesNode.appendChild(editor.domNode);    //show the editor
			}
			this.totalEnabled = totalEnabled;
		},    
		updateEnabledEditors: function(state){
			if(state){
				this.totalEnabled++;
				if(this.totalEnabled == 1){
					if(this.container && this.container._notifyEditorUpdate){
						this.container._notifyEditorUpdate(false);
					}
				}
			}else{
				this.totalEnabled--;
				if(this.totalEnabled == 0){
					if(this.container && this.container._notifyEditorUpdate){
						this.container._notifyEditorUpdate(true);
					}
				}
			}
		},
        getExecutionState: function(name) {
			var _editors = this._editors;
			var len = this._editors ? this._editors.length : 0;
            for (var i = 0; i < len; i++) {
                var state = _editors[i].getExecutionState();
                if (state && state.name.toUpperCase() == name.toUpperCase()) {
                    return state;
                }            
            }
            return null;
        },		
				
        hasPermission: function(actionId) {
            var permission = this._permissions[PermissionsClient.OP_EXECUTION_STATE];
            return (permission && permission[actionId] && permission[actionId] == true);         
        },
		
		isDirty: function() {
			return this._dirty;
		},
		
		setDirty: function(dirty) {
			this._dirty = dirty;
			if (this._dialog) {
				this._dialog.getOkButton().disabled = !dirty;
			}
		},		
		
		doSave: function() {
			//init save context
			var context = new SaveContext();
			
			//for each ExecutionResultState editor
			var _editors = this._editors;
			var len = _editors.length;
			for (var i = 0; i < len; i++) {
				//add save participant to context (if its dirty)
				if (_editors[i].isDirty()) {
					context.addParticipant(_editors[i]);
				}
			}
			
			//commit the save context
			context.setOnSaveFinished(dojo.hitch(this, "_saveFinished"));
			var status = context.commit();
			
			//if return code from commit != SAVING, return false so dialog stays open
			if (status != Constants.STATUS_SAVING) {
				//TODO add some message
				alert("Bad Status from SaveContext.commit() ==> " + status);
			}
			return false;
		},
		
		//INTERNALS
		
		_saveFinished: function(status) {
			if (this._dialog) {
				this.hide();
			} else {
				this.initialize();
			}		
			//fire onSaved callback, if specified
			if (dojo.isFunction(this.onSaved)) {
				this.onSaved(status);
			}
		},		
		
        _error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
        }
	});  
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.ExecutionStateEditor", [dijit._Widget, dijit._Templated, SaveParticipant],
    {                   
        templateString: '    <tr class="execution-state-row" dojoAttachPoint="_executionStateEditorNode" dojoAttachEvent="onclick:_handleRowClick">' +                        
                        '        <td dojoAttachPoint="_iconNode" class="icon-column">' +
                        '            <img dojoAttachPoint="_iconImg"></input>' +
                        '        </td>' +
                        '        <td dojoAttachPoint="_nameNode" class="content-column">' +
                        '            <span dojoAttachPoint="_nameText"></span>' +
                        '            <input dojoAttachPoint="_name" dojoAttachEvent="onchange:_doEdit,onkeypress:_doEditKeypress" type="text" size="${nameInputSize}" maxlength="${nameInputMax}" class="hidden"></input>' +
                        '        </td>' +												
						'        <td dojoAttachPoint="_enabledNode" class="enabled-column">' +
						'            <!--span dojoAttachPoint="_enabledText" dojoAttachEvent="onclick:_toggleEditMode"></span-->' +
						'            <input dojoAttachPoint="_enabledCheckbox" type="checkbox" dojoAttachEvent="onclick:_doEdit" class="enabled-checkbox"></input>' +
						'        </td>' +
                        '        <!--td dojoAttachPoint="_actionsCell" class="actions-column">' +
						'            <div dojoAttachPoint="_actionsNode" class="hidden"></div>' +
						'        </td-->' +						
                        '    </tr>',

        nameInputSize: 30,          
        nameInputMax: 50,
								
        constructor: function() {
			this.parent = null;
            this.executionState = null;
			this.editable = false;
			this.inEditMode = false;
			this._isSaving = false;
        },
        
        postCreate: function() {
            if (!this.parent) {
                throw new Error("Parameter 'parent' REQUIRED!");
            }
            if (!this.executionState) {
                throw new Error("Parameter 'executionState' REQUIRED!");
			}
			this.inEditMode = false;
			this.editable = this.parent.hasPermission(PermissionsClient.ACTION_EDIT);
			this._initialize();
        },
		
        getExecutionState: function() {
            return this.executionState;
        },

        _initialize: function() {
			//set saveparticipant info
			this.setItemType(Constants.MODEL_TYPE_RESULT_STATE);
			this.setItemId(this.executionState.itemId);
			this.setDirty(false); //don't save unless modified
							
            this._showInfo();
            //this._showActions();
			//this._setupHover();
		},
				
        _showInfo: function() {
			//show state's icon
			this._iconImg.src = ExecutionResultStates.ICONS[this.executionState.id];
			
		    //enabled
            this._enabledCheckbox.checked = this.executionState.visible;
				
			//if in edit mode
            if (this.editable && this.inEditMode) {
				//name
                dojo.removeClass(this._name, "hidden");
                dojo.addClass(this._nameText, "hidden");
                this._name.value = this.executionState.name;                
				
				//enable checkbox
                this._enabledCheckbox.disabled = false;	
											
				//toggle row's editable style
                dojo.removeClass(this._executionStateEditorNode, "editable");
								
            } else {
				//name
                this._nameText.innerHTML = this.executionState.name;
                dojo.removeClass(this._nameText, "hidden");
                dojo.addClass(this._name, "hidden");	
				
                //disable checkbox
                //this._enabledCheckbox.disabled = true; 				
				
                //toggle row's editable style
                dojo.addClass(this._executionStateEditorNode, "editable");				 				  
            }
        },
		
        _showActions: function() {
            dojox.data.dom.removeChildren(this._actionsNode);
			var actions = this._getActions();
			var actionsLength = actions.length;
			for (var i = 0; i < actionsLength; i++) {
			 	this._addAction(actions[i]);   
			}            
        },		
		
		_setupHover: function() {
			this.connect(this.domNode, "onmouseover", function() {
				dojo.removeClass(this._actionsNode, "hidden")
			});
            this.connect(this.domNode, "onmouseout", function() {
				dojo.addClass(this._actionsNode, "hidden");
			});
		},
		
		_handleRowClick: function(event) {
			if (event && event.target && event.target.type == "checkbox") {
				//checkbox clicks do not toggle into edit mode
				return;
			}
			if (this.editable && !this.inEditMode) {
				this._toggleEditMode();
			}
			if(event){
                 dojo.stopEvent(event);
            }
		},
	
		_toggleEditMode: function(event) {
			this.inEditMode = !this.inEditMode;
			this._showInfo();
			if(event){
                 dojo.stopEvent(event);
            }
		},
		
		_doEditKeypress: function(e) {
			//dojo.stopEvent(e);
            if (e.keyCode == dojo.keys.ENTER) {
			    //perform the edit on ENTER pressed 
			    this._doEdit();
            } else if (e.keyCode == dojo.keys.ESCAPE) {
                //on ESCAPE - cancel out if existing
                if (this.inEditMode) {
                    this._toggleEditMode();
                }				
				dojo.stopEvent(e); //prevent Dialog from closing
			}
		},
		
		_doEdit: function(event) {
			if (this._name.value) {
				if (!this.validate()) {
					//revert if invalid for edit
					this._name.value = this.category.name;
                    //this._toggleEditMode();
					return;
                }
                this.executionState.name = this._name.value;
			}
			this.setDirty(true);
			var checked = this._enabledCheckbox.checked;
			if(this.executionState.visible != checked){
				this.parent.updateEnabledEditors(checked);
				this.executionState.visible = checked;
			}
			if (this.inEditMode) this._toggleEditMode();
		},
		
		//@Override for SaveParticipant
		setDirty: function(dirty) {
			this.inherited('setDirty', arguments);
			if (dirty) {
				this.parent.setDirty(dirty);
			}
		},
		
		//@Override for SaveParticipant
		getAttributes: function() {
            var attributes = {        
                itemId: this.executionState.itemId,
				name: this.inEditMode ? this._name.value : this.executionState.name,
                visible: this._enabledCheckbox.checked             
            }			
			return attributes;
		},		
		
		//@Override for SaveParticipant
		validate: function() {	
            //validate the name
			var name = this.inEditMode ? this._name.value : this.executionState.name;
			var existing = this.parent.getExecutionState(name);
            if (name == this.executionState.name && name.toUpperCase() != this.executionState.name.toUpperCase()) {
                //allow renaming of same state with change to case only
                return true; 
            } else if (existing && existing.itemId != this.executionState.itemId) {
                //disallow naming/renaming of category to already existing category
                alert(Constants.getMessage("msgStateExists", name));
                return false;               
            }
			return true;
		},

		/*
		_setSaving: function() {
			this._isSaving = true;
			this._nameText.innerHTML = "Saving...";
		},
		
		_endSaving: function() {
			this._isSaving = false;
		},
	
        _save: function() {
            if (this._isSaving) {
                return false;
            } else {
                this._setSaving();
            }
            var postObj = {        
                itemId: this.executionState.itemId,
				name: this._name.value,
                visible: this._enabledCheckbox.checked             
            }
            var srh = new ServiceResponseHandler(this, "_saved", "_error");
            ExecutionResultStateClient.postExecutionResultState(srh, postObj);                      
        },		
		
		_saved: function(executionState) {
			this.executionState = executionState;
			this.inEditMode = false;
			this._initialize();
			this._endSaving();		
		},		
		*/
		
		_error: function(e) {
			console.error(this.declaredClass + ": Error: " + e.message);
		}
		
    });		
})();


