//debug PartAttributeTableView

/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.PartAttributeTableView");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");


dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter");
dojo.require("com.ibm.asq.common.web.ui.internal.catalog.CatalogExplorer");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ListCellRenderer");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.HiddenTextCellRenderer"); 
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ListCellWidget");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;
var ListCellWidgetAction = com.ibm.rqm.planning.web.ui.internal.view.administration.ListCellWidgetAction;

// Local Constants

var PartAttributeTableViewer = {};
PartAttributeTableViewer.RENDER_TYPE_LIST = "RenderType.List";		
PartAttributeTableViewer.RENDER_TYPE_TEXT_HIDDEN = "RenderType.HiddenText";		


// Attribute types
var ViewerType = {};
ViewerType.STRING_TYPE = "StringType";
ViewerType.NUMERIC_TYPE = "NumericType";
ViewerType.PART_TYPE = "ConfigurationPartType";
ViewerType.DATE_TYPE = "TimeStampType";

// Localized strings
var ViewerDisplay = {};
ViewerDisplay.STRING_TYPE = Constants.getMessage("labelString");
ViewerDisplay.NUMERIC_TYPE = Constants.getMessage("labelNumber");
ViewerDisplay.PART_TYPE = Constants.getMessage("labelConfigurationPart");
ViewerDisplay.PART_TYPE_ARRAY = Constants.getMessage("labelMultipleConfigurationParts");
ViewerDisplay.DATE_TYPE = Constants.getMessage("labelDate");


// Column Names
var ColumnName = {};
ColumnName.NAME = "name";
ColumnName.TYPE = "type";
ColumnName.VALUE = "value";


// Seed for ids
var idSeed = 0;

dojo.declare("PartAttributeTableView.TableListener", 
    com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
	{
		_parentView: null,

		 constructor: function(parentView) {
	        this._parentView = parentView;
    	},
    
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called. 
	// Gray buttons according to selection 
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
		var rows = tableViewer.getCheckedRows();
		var hiddenCount = 0;
		var visibleCount = 0;
		var allowedToHide = true;
		var isRequired = false;
		
		var allowedToDelete = true;
		for(var i = 0; i < rows.length; i++){
			if(rows[i].getIsHidden())
				hiddenCount++;
			else 
				visibleCount++;
			if(rows[i].isRequired())
				isRequired = true;
			if(!rows[i].canBeDeleted())
				allowedToDelete = false;			
		}				
        
        if (this._parentView._removeActionNode)
            this._parentView._removeActionNode.setDisabled(rows.length == 0 || !allowedToDelete);
	    if (this._parentView._restoreActionNode)
            this._parentView._restoreActionNode.setDisabled(hiddenCount == 0);
		if (this._parentView._hideActionNode)
            this._parentView._hideActionNode.setDisabled(isRequired || visibleCount == 0)
	},
	
	tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){                
        if (callback == null && callbackData != null && dojo.isFunction(callbackData))
            callbackData();		
		if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null)
                callback(callbackData);
            else
                callback();
        } 
    }
});

dojo.declare("PartAttributeTableView.HiddenText", null,
{
	text: null,
	hidden: false,
	
	constructor: function(args) {
		this.text = args.text;
		this.hidden = args.hidden;
    }
	

});

  

// Wrapper class for attribute. An wrapper represents either a new attribute or an existing attribute.
// If the attribute exists in the catalog, the catalog's attribute is non-null. Otherwise the attribute is null
// and the name and the type represent the newly created, but not save attribute. 
// Attributes are still editable in the catalog until they are committed to the database, so evenwhen 
// an attribute object is associated with an AttributeWrapper, it's possible that the object hasn't yet been
// committed.

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.AttributeWrapper", null,  
{	
	  _name: null,
	  _type: null,
	  _values : null,
	  _attribute: null,
	  _containingPart: null,
	  _isNew: true,
	  _isHidden: false,
	  _dirty: false,
	  _selectedPart: "",
	  _isArray: false,
	  _markDeleted: false,
	  _linkMode: true,
	  _id: 0,
	  _isRequired: false,
	  
	  init: function(name, type, part){
			this._name = name;
			this._type = type;
					
			this._containingPart = part;
			this._values = [];
			this._isNew = true;
			this._isHidden = false;
			this._isRequired = false;
			this._id = ++idSeed;
			
	   },
	   
	   setLinkMode: function(){
	   		this._linkMode = true;
	   },
	    
	   setEditMode: function(){
			this._linkMode = false;
	   },

	   isLinkMode: function(){
		  return this._linkMode; 
	   },
	   
	   isEditMode: function(){
	   	  return !this._linkMode;
	   },
	   
	   markDeleted: function()
	   {
	   		this._markDeleted = true;
			this._dirty = true;
	   },
	   
	   canBeDeleted: function(){
	   	 return this._attribute == null || this._attribute.isEditable();
	   },
	   
	   getDisplayType: function()
	   {
			var mapper = {};
			mapper[ViewerType.STRING_TYPE] = ViewerDisplay.STRING_TYPE; 
			mapper[ViewerType.NUMERIC_TYPE] = ViewerDisplay.NUMERIC_TYPE;
			mapper[ViewerType.PART_TYPE] = ViewerDisplay.PART_TYPE;
			mapper[ViewerType.DATE_TYPE]= ViewerDisplay.DATE_TYPE;
			
			var display = mapper[this._type];
			if(this._type == ViewerType.PART_TYPE && this._isArray)
				display = ViewerDisplay.PART_TYPE_ARRAY;
			return display;
  	  },		   
	   
	   setIsArray: function(isArray){
		   this._isArray = isArray;
	   },
	   
	   getIsArray: function(){
		   return this._isArray;
	   },
	   

	   
	   setSelectedPart: function(part)
	   {
			this._selectedPart = part;
		},
	
		getSelectedPart: function()
		{
			return this._selectedPart;
		},
		
	   save: function(){
	   		this.setDirty(false);
			var id = this._containingPart.getTemplate().getId();			
		 	if(this._isNew){
		   		if(this._attribute == null)
				{
					this._attribute = this._containingPart.createAttribute(this._type, this._name, this.getIsArray(), this.getSelectedPart(), id, this.getEnumValues());
				}
				else {
					if(this._markDeleted)
						this._attribute.deleteAttribute();
			    	else 
						this._attribute.edit(this._name,  this.getIsArray(),  this.getType(),this.getSelectedPart(), id,  this.getEnumValues());
				}
	  	 	}
			else  // must mean that only the enum changed
				this._attribute.edit(this._name, false, this.getType(), this.getSelectedPart(),  id, this.getEnumValues());
			
			return true;		
	   },
	   
	   isDirty: function()
	   {
	   		return this._dirty;
	   },
	   
	   setDirty: function(dirty)
	   {	
	   		this._dirty = dirty;
	   },
	   
	   
	   initWithAttribute: function(attribute, part){
		 this._attribute = attribute;	   	
	   	 this._name = attribute.getName();
		 this._isArray = attribute.isArray();
		 this._containingPart =  part;
		 this._values = [];
		 if(attribute.isEnum())
		 	this._values = attribute.getEnumValues();
		 this._type = attribute.getType();
		 this._isNew = attribute.isEditable();
		 var isRequired = attribute.isRequired();
		 if(isRequired || isRequired == "true")
		 	this._isRequired = true;
		 else
			this._isRequired = false;
		 // TODO weird behavior
		 var isHidden = attribute.isHidden();
		 if(isHidden == true || isHidden == "true")
		 	this._isHidden = true;
		 else
			this._isHidden = false;
		 this._id = ++idSeed; 			 
		 if(this._type == ViewerType.PART_TYPE) 
		     this.setSelectedPart(attribute.getPartTypeName());			 
		 
	   },
	   
	   setIsHidden: function(val){
	   	 this._isHidden = val;
		 if(this._attribute != null)
		 	if(val)
		 		this._attribute.hide();
			else
				this._attribute.show();
  	   },
	   		 	   
	   getIsNew: function(){
	   		return this._isNew;
	   },
	   getName: function() {
	   		return this._name;
	   },
       getDisplayName: function() {
           var name = Constants.getLmMessage("pl_" + this._name);
           
           // not a known translated attribute name, should be a custom one. Just use 
           // its stored name.
           return (name.indexOf("pl_") > -1) ? this._name : name;
       },
	   setType: function(type)
	   {
	   	  this._type = type;
	   },
	   
	   getType: function(){
	   		return this._type;
	   },
	   setName: function(name){
	   	 this._name = name;
	   },
	   
	   getEnumValues: function(){
			return this._values;
	   },
	   
	   clearEnumeratedValues: function(){
		 this._values = [];
	   },
	   
	   setEnumValues: function(values){
		  this._values = values;
	   },
	   hasEnumValues: function()
	   {
	   		if(this._values == null)
				return false;
			return this._values.length > 0;
	   },
	   couldBeEnum: function(){
			return  this._type == ViewerType.STRING_TYPE || this._type == ViewerType.NUMERIC_TYPE;
	   },
	   getIsHidden: function(){
			return this._isHidden;
	   },
	   isRequired: function() {
	   		return this._isRequired;
	   } 
	  
});

dojo.declare("PartAttributeTableView.Sorter",
			com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter,
{
	_sortComparator: function(obj1, obj2){

				if(this.sortCol == 0){
					if(this.sortDir == TableConstants.SORT_DIR_ASCEND)					
						return obj1.getDisplayName() < obj2.getDisplayName();
					else
						return obj2.getDisplayName() < obj1.getDisplayName();
				}
				if(this.sortCol == 1){
					var v1 = obj1.getType();
					var v2 = obj2.getType();

					if(this.sortDir == TableConstants.SORT_DIR_ASCEND)
						return v1 < v2;
					else
						return v2 < v1;
				}
				if(this.sortCol == 2){
					var v1 = 0;
					var v2 = 0;
					if(obj1.hasEnumValues())
						v1  = 1;
					if(obj2.hasEnumValues())
						v2 = 1;
					if(this.sortDir == TableConstants.SORT_DIR_ASCEND)
						return v1 < v2;
					else
						return v2 < v1;
				}
			}
});


// Get the data for the table viewer
dojo.declare("PartAttributeTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
			
{
 	_templates: null,
	_types: null,
	_columnName: null,	
	
	constructor: function(params) {
		this._types = [ViewerDisplay.STRING_TYPE, ViewerDisplay.NUMERIC_TYPE, ViewerDisplay.PART_TYPE_ARRAY, ViewerDisplay.PART_TYPE, ViewerDisplay.DATE_TYPE ];
		this._columnNames = [ ColumnName.NAME, ColumnName.TYPE, ColumnName.VALUE];
		this._templates = params.templates; 
	 },
	 	
    getNumColumns: function(content){
        return this._columnNames.length;;
    },
    getColumnHeading: function(content, index){
        var headings = [
			Constants.getMessage("labelAttributeName"),
			Constants.getMessage("labelDataType"),
			Constants.getMessage("labelValue")
        ];
        return headings[index];
    },
    getColumnHeadingImage: function(content, atColumnIndex){
		return null;
    },
    getColumnHeadingAlignment: function(atColumnIndex){
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    getIsColumnGroupable: function(content, atColumnIndex){
        return false;
    },
    getIsColumnSortable: function(content, atColumnIndex){
        return true;   
    },
    getColumnWidth: function(content, atColumnIndex){
        switch(atColumnIndex) {
            case 0: 
                return 30;
            case 1: 
                return 30;
            case 2: 
                return 35;
        }
        return 0;
    },
    getContentObjects: function(content){
        return content;
    },
	
    getRowDataId: function(rowData){
        return  rowData._id;
    },
	
    getCellDisplayType: function(rowData, atColumnIndex){
		if (rowData.getIsNew()) {
			if (atColumnIndex == 0){
				return TableConstants.RENDER_TYPE_EDITABLE_TEXT;
			}
			if (atColumnIndex == 1) 
				return TableConstants.RENDER_TYPE_ENUM;
			if (atColumnIndex == 2)
				if(rowData.couldBeEnum()){
					if(rowData.isLinkMode())
					 	return TableConstants.RENDER_TYPE_ACTION;
					return PartAttributeTableViewer.RENDER_TYPE_LIST;
				}
				else if(rowData.getType() == ViewerType.PART_TYPE) 
					return TableConstants.RENDER_TYPE_ENUM;
				else
					return TableConstants.RENDER_TYPE_TEXT;
		}
		else {
			if (atColumnIndex == 2){
				if (rowData.couldBeEnum()) {
				    if(rowData.isLinkMode())
			 		  return TableConstants.RENDER_TYPE_ACTION;
				    return PartAttributeTableViewer.RENDER_TYPE_LIST;
				}
				else if (rowData.getType() == ViewerType.PART_TYPE)
				    return TableConstants.RENDER_TYPE_TEXT;
			}
		}
		return PartAttributeTableViewer.RENDER_TYPE_TEXT_HIDDEN;
    },
	
    getCellAlignment: function(rowData, atColumnIndex){
        return TableConstants.ALIGN_LEFT;
    },

      
	getCellDisplayData: function(rowData, atColumnIndex){
        var val = "";
        switch(atColumnIndex) {
     	   case 0:
                val = new PartAttributeTableView.HiddenText({ text: rowData.getDisplayName(), hidden: rowData.getIsHidden() }) ;		   
		   		if(rowData.getIsNew()){
					val = rowData.getName(); // new custom attributes, no need to use the message catalog.					
				}
                break;
           case 1:
				val = new PartAttributeTableView.HiddenText({ text: rowData.getDisplayType(), hidden: rowData.getIsHidden() });			
                break;
           case 2: 
    			if(rowData.couldBeEnum()){
						val = rowData.getEnumValues();				
						if(rowData.isLinkMode()){
							val = Constants.getMessage("labelAddEnum");
							if(rowData.hasEnumValues())
								val = Constants.getMessage("labelEditEnum");
						}
				}
				else if(rowData.getType() == ViewerType.PART_TYPE) {
				    if (rowData.getIsNew()) {
                        val = this._templates;
				    }
				    else {
				    	var typeName = rowData.getSelectedPart();
				    	var messageString = typeName;
				    	var message = Constants.getLmMessage(messageString);
				    	if (message != messageString) {
				    		val = message;
				    	}
				    	else { 
				    		messageString = "pl_" + typeName;
				    		message = Constants.getLmMessage(messageString);
				    		
				    		if (message != messageString) {
				    			val = message;
				    		}
				    		else {
				    			val = typeName;
				    		}
				    	}
				    }
				}
				break;
				
        }
        return val;
    },
	
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
		  var val = {};	
		  val.select = "";
		  val.choices = [];
		  if(rowData.getIsNew()){
		  	if (atColumnIndex == 1) {
		  		for (var i = 0; i < this._types.length; i++) {
					val.select = rowData.getDisplayType();
		  			val.choices.push({
		  				text: this._types[i],
		  				value: this._types[i]
		  			})
			  	}
		 	 }
			  else 
			  	if (atColumnIndex == 2) {
		  			if (rowData.getType() == ViewerType.PART_TYPE) {
						val.select = rowData.getSelectedPart();						
		  				for ( i = 0; i < this._templates.length; i++) {
		  					val.choices.push({text: this._templates[i],
		  						value: this._templates[i]})
			  			};
		  		}
		  	}
		}
		return val;
	}
});

dojo.declare("PartAttributeTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
	action: 0,
	templates: null,
	tableViewer: null,
	okButtonMgr: null,
	
    constructor: function(args) {
		this._tableViewer = args.tableViewer;
		this._templates = args.templates;
		this._okButtonMgr = args.okButtonMgr;
    },

	setChangeListener: function(listener)
	{
	},
	
	// Validate this object before notifying of cellValue Change 
	cellValueValidate : function(oldValue, newValue, rowObject, columnIndex, action)
	{
		this._action = action;
		if(columnIndex == 0)
		{
		} else if(columnIndex == 1)
		{
			
		} else if (columnIndex == 2)
		{

			if(newValue == "" && action == ListCellWidgetAction.ADD)
				return { valid: false, reason: "" }
						
		}
		return { valid: true, reason: "" };	
	},

    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex, action)
    {
		
		var mapper = {};
		mapper[ViewerDisplay.STRING_TYPE] = ViewerType.STRING_TYPE;
		mapper[ViewerDisplay.NUMERIC_TYPE] = ViewerType.NUMERIC_TYPE;
		mapper[ViewerDisplay.PART_TYPE] = ViewerType.PART_TYPE;
		mapper[ViewerDisplay.PART_TYPE_ARRAY]= ViewerType.PART_TYPE;
		mapper[ViewerDisplay.DATE_TYPE]= ViewerType.DATE_TYPE;

		
		if(columnIndex == 0){
            rowObject.setDirty(true);       
			rowObject.setName(newValue);
			this.setOkState(rowObject);			
		}							
	    else if(columnIndex == 1)
		{
            rowObject.setDirty(true);       

			if(oldValue == null)// Fix this, some sort of Comobox defect
				oldValue = ViewerDisplay.PART_TYPE;
			if(oldValue != ViewerDisplay.NUMERIC_TYPE && newValue != ViewerDisplay.STRING_TYPE)
				rowObject.clearEnumeratedValues();			
		   	rowObject.setType(mapper[newValue]);
			rowObject.setIsArray(false); 			
			if(newValue == ViewerDisplay.PART_TYPE_ARRAY)
				rowObject.setIsArray(true); 
			this._tableViewer.refresh();
	   }
	   else if(columnIndex == 2){
   			 if(rowObject.getType() == ViewerType.STRING_TYPE || rowObject.getType() == ViewerType.NUMERIC_TYPE){
			 	if(action == ListCellWidgetAction.CLOSE){ // Not a row modifier
					rowObject.setLinkMode();
					this._tableViewer.refresh();
				}
				else if(action == ListCellWidgetAction.REMOVE){
                    rowObject.setDirty(true);       
					var removeValues = newValue;
					var values = rowObject.getEnumValues();
					for(var i = 0; i < values.length; i++){
						for(var j = 0; j < newValue.length; j++)
							if(values[i] == removeValues[j])
								values.splice(i, 1); 
					}
					rowObject.setEnumValues(values);						
				}
				else if(action == ListCellWidgetAction.ADD){
                    rowObject.setDirty(true);       
					values = rowObject.getEnumValues();
					values.push(newValue);
					rowObject.setEnumValues(values);					
				}
			}
			else {
                rowObject.setDirty(true);       
				rowObject.setSelectedPart(newValue);
				this._tableViewer.refresh();
			}
		}
    },
	
	setOkState: function()
	{
		var allRows = this._okButtonMgr.allRows;
		var okButton = this._okButtonMgr.okButton;
		okButton.disabled = false;
		for(var i = 0; i < allRows.length; i++){
			if(allRows[i].getName() == null || allRows[i].getName().length == 0 && allRows[i].getType() != ViewerType.PART_TYPE){
				okButton.disabled = true;		
			}
		}
	},
	
	cellActionInvoked: function(rowObject, columnIndex)
    {
		rowObject.setEditMode();
	   	this._tableViewer.refresh();
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.PartAttributeTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-asq-common-web-ui-internal-catalog" tabindex="-1"></div>',
    _widgetsInTemplate: true,
	_showHidden: false, 
	_templates: null, 
	_ttributeWrappers: null,
	_attributesDeleted: null,	
	_tableViewer: null,
	_okButton: null,
	
	constructor: function(params) {
		this._actionButtons = [];		
	 	this._attributeWrappers = [];
		this._attributesDeleted = [];
		this._templates = params.templates;
		this._containingPart = params.containingPart;
		this._showHidden = params.showHidden;
		this._okButton = params.okButton;
		
	},
	
	hasModified: function(){
		var hasModified = false;
		for(var i = 0; i < this._attributeWrappers.length; i++){
				if(this._attributeWrappers[i].isDirty()){
					return true;
				}
		}
	},
	
	save: function(){
			for(var i = 0; i < this._attributeWrappers.length; i++){
				if(this._attributeWrappers[i].isDirty()){
					this._attributeWrappers[i].save();
				}
			}
			// Save the deleted items also			
			for(i =0 ; i < this._attributesDeleted.length; i++)
				this._attributesDeleted[i].save();
			this._attributesDeleted = [];
	},
	 
	postCreate: function(){
        this.inherited("postCreate", arguments);
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();
		        
		tableViewer.registerCellRenderer(PartAttributeTableViewer.RENDER_TYPE_LIST, 
           new com.ibm.rqm.planning.web.ui.internal.view.administration.ListCellRenderer());
		tableViewer.registerCellRenderer(PartAttributeTableViewer.RENDER_TYPE_TEXT_HIDDEN, 
           new com.ibm.rqm.planning.web.ui.internal.view.administration.HiddenTextCellRenderer());
			
        this._tableViewerNode.appendChild(tableViewer.domNode);
		var cellModifier = new PartAttributeTableView.CellModifier({tableViewer: tableViewer, templates: this._templates, okButtonMgr: { okButton: this._okButton, allRows: this._attributeWrappers}});
		tableViewer.setCellModifier(cellModifier);				
		var provider = new PartAttributeTableView.TableProvider({templates: this._templates });
		tableViewer.setContentProvider(provider);

		this._listener = new PartAttributeTableView.TableListener(this)
        tableViewer.setTableListener(this._listener);
		tableViewer.setSorter(new PartAttributeTableView.Sorter());		
		tableViewer.setTableSummary("summary", this.queryName);
		tableViewer.setTableRowsCheckable(true);
		this._actions = this.getTableActions();
		for (var i = 0; i < (this._actions ? this._actions.length : 0); i++) {
            var action = this._actions[i];
            var actionButton = tableViewer.addAction(action.actionId, action.label, action.actionHandler, action.iconClass);
            if (action.disabled && action.disabled == true)
                actionButton.setDisabled(true);
            if (action.hidden && action.hidden == true)
                dojo.addClass(actionButton.domNode, "dijitDisplayNone");
			if(action.actionId == "add")
				this._addActionNode = actionButton;
			if(action.actionId == "remove")
				this._removeActionNode = actionButton;
			if(action.actionId == "hide")			
				this._hideActionNode = actionButton;
			if(action.actionId == "restore")
				this._restoreActionNode = actionButton;			
            this._actionButtons.push(actionButton);
        }
        tableViewer.setContent([]);
        tableViewer.show();
		this._tableViewer = tableViewer;
    },
	
	refresh:function(/*Attribute [] */attributes){
		
		var j = 0;
		for(var i = 0; i < attributes.length; i++){
			var isHidden = attributes[i].isHidden();
			// TODO weird behavior
			if(isHidden == "true" || isHidden == true)
				isHidden = true;
			else
				isHidden = false;
			if(this._showHidden || !isHidden){
				this._attributeWrappers[j] = new com.ibm.rqm.planning.web.ui.internal.view.administration.AttributeWrapper();
				this._attributeWrappers[j].initWithAttribute(attributes[i], this._containingPart);
				j++;
			}			
		}
		this._tableViewer.setContent(this._attributeWrappers);
		this._tableViewer.refresh();
	},
	
	_handleAdd: function()
	{
		var attribute = new com.ibm.rqm.planning.web.ui.internal.view.administration.AttributeWrapper();
		attribute.init("", ViewerType.STRING_TYPE, this._containingPart);
		attribute.setSelectedPart(this._templates[0]);
		this._attributeWrappers.splice(0, 0, attribute);
		this._okButton.disabled = true;			
		this._tableViewer.refresh();
    	
	},
	_handleHide: function()
	{
		var rows = this._tableViewer.getCheckedRows();
		for(var i = 0; i < rows.length; i++){
			rows[i].setIsHidden(true);
			rows[i].setDirty(true);
			this._listener.rowCheckStateChanged(this._tableViewer, rows[i]);			
		}
		this._tableViewer.refresh();
		
				
	},
	_handleRestore: function()
	{
		var rows = this._tableViewer.getCheckedRows();
		for(var i = 0; i < rows.length; i++){
			rows[i].setIsHidden(false);
			rows[i].setDirty(true);			
			this._listener.rowCheckStateChanged(this._tableViewer, rows[i]);			
		}
		this._tableViewer.refresh();		
	},
	
	_handleRemove: function()
	{
		var rows = this._tableViewer.getCheckedRows();
		for(var i = 0; i < rows.length; i++){
			this._tableViewer.deleteItem(rows[i]);
			rows[i].markDeleted();
			this._attributesDeleted.push(rows[i]);
		}
		this._tableViewer.refresh();		
	},
	
	getTableActions: function(){
		var actions = [];
		actions.push({
                    actionId: "add",
                    label: Constants.getMessage("labelAdd"),
                    actionHandler: dojo.hitch(this, "_handleAdd"),
                    iconClass: "add-icon-image",
                    disabled: false
                });
		actions.push({
                    actionId: "remove",
                    label: Constants.getMessage("labelRemove"),
                    actionHandler: dojo.hitch(this, "_handleRemove"),
                    iconClass: "delete-icon-image",
                    disabled: true
                });
		actions.push({
                    actionId: "restore",
                    label: Constants.getMessage("labelRestore"),
                    actionHandler: dojo.hitch(this, "_handleRestore"),
                    iconClass: "restore-icon-image",
                    disabled: true
                });
		actions.push({
                    actionId: "hide",
                    label: Constants.getMessage("labelHide"),
                    actionHandler: dojo.hitch(this, "_handleHide"),
                    iconClass: "hide-icon-image",
                    disabled: true
                });
		return actions;		
	},
	
	addAction:function(actionId,actionLabel,callbackData,iconClass,callback){
		return this._tableViewer.addAction(actionId,actionLabel,callbackData,iconClass,callback);
	}	
});

})();
