/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.ProgressPanelContentArea");

dojo.require("dijit.form.CheckBox");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea");


(function(){
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.ProgressPanelContentArea", 
             com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPreferenceContentArea, 
{
    templateString: '<div id="${id}" class="progress-panel-content-area" dojoAttachPoint="domNode">' +
                    '  <div style="margin-bottom: 12px;">' +
                    '    <div>' +
                    '      <span class="content-name" dojoAttachPoint="_name"></span>' + 
                    '      <span dojoAttachPoint="_cshelp"></span>' + 
                    '    </div>' +
                    '  </div>' +
                    '  <div dojoAttachPoint="_content">' +
                    '    <input name="autoClose" type="checkbox" dojoType="dijit.form.CheckBox" dojoAttachPoint="autoCloseNode" dojoAttachEvent="onClick:_autoCloseChanged"></input>'+
                    '    <span>${labelAutoClose}</span>' + 
                    '    <span>' + 
                    '        <label><input type="text" size="1" dojoAttachEvent="onkeyup:_secondsInputChanged" dojoAttachPoint="secondInputNode"></input></label>' +
                    '    </span>' + 
                    '    <span dojoAttachPoint="secondLabelNode">${labelSeconds}</span>' + 
                    '    <div style="margin-top: 1.0em; margin-left:0.8em;">' +
                    '        <div style="margin-top: 0.2em; margin-bottom:0.6em;">${labelAlignment}</div>' +
                    '        <label><input type="radio" style="margin-bottom:0.2em; margin-right:0.4em;" name="alignTo" dojoType="dijit.form.RadioButton" dojoAttachPoint="alignLeftNode" dojoAttachEvent="onChange:onChange"></input>${labelAlignLeft}</label>' +
                    '        <br/>' +
                    '        <label><input type="radio" style="margin-bottom:0.2em; margin-right:0.4em;" name="alignTo" dojoType="dijit.form.RadioButton" dojoAttachPoint="alignRightNode" dojoAttachEvent="onChange:onChange"></input>${labelAlignRight}</label>' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',
                    
    widgetsInTemplate: true,
    key: Constants.USER_UI_CUSTOMISE_PROGRESS_PANEL_CATEGORY.key,
    
    labelInputSeconds: Constants.getCommonMessage("labelDateRelativeStyle"),
    labelAutoClose: Constants.getCommonMessage("labelAutoCloseBar"),
    labelSeconds: Constants.getCommonMessage("labelAutoCloseAfterSec"),
    labelAlignment: Constants.getCommonMessage("labelPlaceAt"),
    labelAlignLeft: Constants.getCommonMessage("labelPositionLeft"),
    labelAlignRight: Constants.getCommonMessage("labelPositionRight"),
    
    postCreate: function(){
        this.csHelpId = "rqm_pref_date_display.html";
        this.csHelpLabel = Constants.getMessage("labelDateDisplayformatHelp");
        this.badInput = false;
        
        this.inherited("postCreate", arguments);
    },
    
    // @override for AbstractContentArea
    setName: function(){
        dojox.data.dom.removeChildren(this._name);
        this._name.appendChild(document.createTextNode(Constants.getMessage("labelProgressPanelSettings")));
    },
    
    // @overrie for AbstractPreferenceContentArea
    getValues: function(){
        var autoCloseVal = (this.autoCloseNode.attr("checked")) ? this.secondInputNode.value : -1;
        var align = (this.alignLeftNode.attr("checked")) ? "alignLeft" : "alignRight";
        
        return [{
            key: this.key,
            value: {
                autoClose: autoCloseVal,
                alignTo: align
            }
        }];
    },

    // @override set the corresponding user preference for this content area
    setUserPreference: function(userPreference) {
        // flush our old preferences first, before loading
        // the new values.
        this.userPreference = [];
        
        for(var i = 0; i < userPreference.length; i++){
            if(userPreference[i].key == this.key){
                this.displayUserPreference(userPreference[i]);
                break;            
            }
        }
    },
    
    // @override
    displayUserPreference: function(preference){
        var value = preference.value; 
        var isAutoClose = value.autoClose > 0;

        this.autoCloseNode.attr("checked", isAutoClose);
        this.secondInputNode.value = (isAutoClose) ? value.autoClose : "8"; // provide a default value even if it's not turned on.
        
        this._autoCloseChanged();       
        this.alignLeftNode.attr("checked", (value.alignTo == "alignLeft"));
        this.alignRightNode.attr("checked", (value.alignTo == "alignRight"));
    },
   
    _autoCloseChanged: function(event)
    {    
        var isAutoClose = this.autoCloseNode.attr("checked");
        this.secondInputNode.disabled = (!isAutoClose);
        dojo.toggleClass(this.secondLabelNode, "disabled-text", !isAutoClose);    
        
        if(isAutoClose) {
            dijit.focus(this.secondInputNode);
        }   
        this.onChange();
        if(event){
            dojo.stopEvent(event);
        }
    },
   
    _secondsInputChanged: function(event)
    {
        var secs = this.secondInputNode.value;
        if(secs) {
            this.badInput = isNaN(secs) || parseInt(secs) > 3000;
            dojo.toggleClass(this.secondInputNode, "invalid-input-text", this.badInput);    
        }
        this.onChange();
        if(event){
            dojo.stopEvent(event);
        }
    },

    hasInvalidInput: function()
    {
        return this.badInput;  
    },
    
    // @override for AbstractPreferenceContentArea
    isHotSwitchable: function(){
        return true;
    }
});
    
})();
