/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.WhitchCatalogEditor");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CatalogEditor");

(function(){
    
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("WhitchCatalogEditor.AcceptNameDialog", 
             com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog, 
{  
    defaultName: null,
    attrPaths: null,
    selectedPath: null,
    enteredName: null,
      
    postCreate: function()
    {
        this.inherited("postCreate", arguments);  
        
        var content = '<div dojoAttachPoint="domNode" style="width:100%; margin-bottom:5px;">' +
                      '  <div dojoAttachPoint="promptNode" style="margin-bottom:10px;"></div>' +
                      '  <div style="width:100%;">' +
                      '    <div dojoAttachPoint="labelNode" style="float:left; width:28%; margin-right:10px;"></div>' +
                      '    <input dojoAttachPoint="nameInputNode" type="text" style="width:65%;"></input>' +
                      '  </div>' +
                      '  <div dojoAttachPoint="pathDisplayNode" style="display:none;width:100%;margin-top:5px;">' +
                      '    <div dojoAttachPoint="pathLabelNode" style="float:left; width:28%; margin-right:10px;"></div>' +
                      '    <select dojoAttachPoint="pathSelectNode"  style="width:66%;"></select>' +
                      '  </div>' +
                      '</div>';
                      
        this.contentDom = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(content);
		var contentDom = this.contentDom;
        contentDom.promptNode.appendChild(document.createTextNode(Constants.getMessage("labelTestEnvTypeNamePrompt")));
        contentDom.labelNode.appendChild(document.createTextNode(Constants.getMessage("labelIndentifier")));
        contentDom.pathLabelNode.appendChild(document.createTextNode("Attribute Path:"));
        this.contentContainer.appendChild(contentDom.domNode);
        this.connect(contentDom.nameInputNode, "onkeyup", "_inputEntered");
        
        // The type has more then one attribute pointing to it. Need
        // to show the path selection. The user need to choice which one 
        // to create.
        if(this.attrPaths && this.attrPaths.length > 1) {
            dojo.style(contentDom.pathDisplayNode, "display", "");
            this.connect(contentDom.pathSelectNode, "onchange", "_selectChanged");
            this.loadPathSelections();
        }
        this.selectedPath = this.attrPaths[0];
        
        if (this.defaultName && this.defaultName.length > 0) {
            contentDom.nameInputNode.value = this.defaultName;
            this.enteredName = this.defaultName;
        }   
        this.enableOK(); 
        this.firstFocusNode = contentDom.nameInputNode;
    },
    _selectChanged: function() {
        var select = this.contentDom.pathSelectNode;
		var options = select.options;
		var length = options.length;
        for(var i = 0; i < length; i++) {
            if(options[i].selected) {
                this.selectedPath = this.attrPaths[i];
            }
        }
        this.enableOK();
    },    
    _inputEntered: function()
    {
        this.enteredName = this.contentDom.nameInputNode.value;
        this.enableOK();
    },
    enableOK: function(e)
    {
        var name = this.getEnteredName();
        var disable = (!name || name.length == 0 || !this.selectedPath);  
        this.getOkButton().disabled = disable;
    },
    getSelectedAttributePath: function()
    {
        return this.selectedPath;
    },
    getEnteredName: function()
    {
        return this.enteredName;
    },
    loadPathSelections: function()
    {
        var select = this.contentDom.pathSelectNode;
		var attrPaths = this.attrPaths;
		var length = attrPaths.length;
        for(var i = 0; i < length; i++) {
            var path = attrPaths[i];
            var pathStr = "";
            var components = path.components.component;
            
			var length2 = components.length;
            for(var j = 0; j < length2; j++) {
				var component = components[j];
                var prefix = (component != "Machine") ? "pl_" : ""; 
                var tempPath = Constants.getLmMessage(prefix + component);
                pathStr += (tempPath.indexOf("pl_") == -1) ? tempPath : component;
                if(j < length2 - 1) {
                    pathStr += " > ";
                }
            }
            var option = new Option(pathStr);
            if(dojo.isIE)
                option.title = pathStr; 
            select.options[i] = option;
        };
    },
	handleEnterKeyPressed: function()
	{
		this.getOkButton().click();
	}
});

dojo.declare("WhitchCatalogEditor.ActionRegistry",
             com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.ActionRegistry,
{   
    init: function()
    {
        this.actions = {
            actions: [
                {
                    label: Constants.getMessage("labelSetType"),
                    needSelectionEnablement: true,
                    enablementCallback: dojo.hitch(this.parentView, "_enableAddAction"),
                    iconClass: "assign-test-env-icon-image",
                    callback: dojo.hitch(this.parentView, "_addTypeToWhitch")
                },
                {
                    label: Constants.getMessage("labelRemoveType"),
                    needSelectionEnablement: true,
                    enablementCallback: dojo.hitch(this.parentView, "_enableRemoveAction"),
                    iconClass: "delete-icon-image",
                    callback: dojo.hitch(this.parentView, "_removeTypeFromWhitch")
                }
            ]
        };
    },

    getPanelActions: function(model, item)
    {
        if (item.type == "part" || item.type == "root") {
            return this.actions; 
        }
        return null;
    },
    
    getContextActions: function(model, item)
    {
        if (item.type == "part") {
            return this.actions; 
        }
        return null;
    }
});
    
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.WhitchCatalogEditor", 
             com.ibm.rqm.planning.web.ui.internal.view.administration.CatalogEditor, 
{    
    templateString: '<div id="${id}" class="com-ibm-rqm-catalog-editor" dojoAttachPoint="domNode">' +
                    '  <div style="margin-bottom: 12px;">' +
                    '    <div class="content-name" dojoAttachPoint="_name"></div>' +
                    '    <div>' +
                    '      <span class="content-text" dojoAttachPoint="_text" style="float:left;"></span>' + 
                    '      <span dojoAttachPoint="_cshelp"></span>' + 
                    '    </div>' +
                    '  </div>' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.catalog.CatalogExplorer" dojoAttachPoint="explorer" filterNonWhitchTagParts="true" showWhitchTag="true" allowAutoResize="false" numVisiblePanels="3" canShowHidden="false" partIsAlwaysContainer="false" width="100%" height="400px">' +
                    '  </div>' +
                    '</div>',

    widgetsInTemplate: true,

    initIds: function()
	{
        this.csHelpId = "rtlm_catalog_tpenvironment.html";
        this.csHelpLabel = Constants.getMessage("labelTestPlanEnvEditorHelp");		
        this.editorId = "CatalogWhitchEditor";
        this.editorName = Constants.getMessage("labelWhitchCustomizeCatalog");                
	},
	
    postCreate: function()
    {
        this.inherited("postCreate", arguments);
    },

    //@Abstract: Override
    initCatalogExplorer: function()
    {
        this.explorer.installActionRegistry(new WhitchCatalogEditor.ActionRegistry({parentView: this}));
        this.explorer.htree.getIconClass = dojo.hitch(this, "_getIconClass");  
        
        this.explorer.loadModelContents();
    },
    
    //@Abstract: Override
    setName: function()
    {
        this.name = Constants.getMessage("labelWhitchCustomizeCatalog");
        dojox.data.dom.removeChildren(this._name);
        var name = document.createTextNode(this.name);
        this._name.appendChild(name);
    },
    
    //@Abstract: Override
    setText: function()
    {
        this.text = Constants.getMessage("labelWhitchCustomizeCatalogText");
        dojox.data.dom.removeChildren(this._text);
        var text = document.createTextNode(this.text);
        this._text.appendChild(text);
    },
    
    _addTypeToWhitch: function(parentItem, item)
    {
        var accessiblePaths = this.model.getPartAccessiblePaths(item);
        var whitchTag = this.model.getWhitchTagOnItem(item);
        var identifier = (whitchTag) ? whitchTag.name : this.model.getLabel(item, /* noWhitchTag */true);
        var args = {
            width: "380px",
            attrPaths: accessiblePaths,
            primaryTitle: Constants.getMessage("labelSetTestEnvTypeName"),
            isClosable: true,
            defaultName: identifier,
            okCallback: dojo.hitch(this, function(){
                var tagName = this.dialog.getEnteredName();
                var path = this.dialog.getSelectedAttributePath();
                
                // get the user selected info to store in the whitch tag.
                this.model.setWhitchTagOnItem(item, tagName, path);
                
                this.explorer.htree.updateItem(parentItem, item);
                this.explorer.htree.updateAllPanelActions();
                this.setDirty(true);
                return true;
            })
        };
        this.dialog = new WhitchCatalogEditor.AcceptNameDialog(args);
        this.dialog.open();
    },

    _removeTypeFromWhitch: function(parentItem, item)
    {
        this.model.removeWhitchTagOnItem(item);
        this.explorer.htree.updateItem(parentItem, item);
        this.explorer.htree.updateAllPanelActions();
        this.setDirty(true);
    },
    
    _enableAddAction: function(model, item)
    {
        var subparts = item.part.getAllSubPartNames();
        var noSubparts = !subparts || subparts.length == 0;      
        return !(noSubparts || this._checkPredAttr(item, "whitchTagged") || this._checkDescAttr(item, "whitchTagged")); 
    },
    
    _enableRemoveAction: function(model, item)
    {
        return this._checkAttr(item, "whitchTagged");
    },
       
    _getIconClass: function(item, opened)
    {
        var iconClass = this._blankGif;
        var isTagged = (this._checkAttr(item, "whitchTagged") || this._checkPredAttr(item, "whitchTagged"));
        
        if(isTagged)
            iconClass = "assigned-to-env-icon-image";
        return iconClass;
    },

    _getLabelClass: function(item)
    {   
        // no label style changes...
        return null;
    }   
});
    
})();
