/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.process.web.ui.internal.view.approval.ApprovalExtendTable");
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.approval.ApprovalExtendTable");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");

(function(){
    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
    dojo.declare("ApprovalExtendTable.TableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        // Summary:
        //
        getNumColumns: function(content){
            return 4;
        },
        
        // Summary:
        //
        getColumnHeading: function(content, index){
            var headings = ["Test Case", "Approved Number", "Rejected Number", "Reviewed Number"];
            return headings[index];
        },
        
        getIsColumnSortable: function(content, atColumnIndex){
            return false;
        },
        
        getColumnWidth: function(content, atColumnIndex){
            var width = 25;
            switch (atColumnIndex) {
                case 0:
                    width = 40;
                    break;
                case 1:
                    width = 20;
                    break;
                case 2:
                    width = 20;
                    break;
                case 3:
                    width = 20;
                    break;
            }
            return width;
        },
		
        getIsCellEditable: function(rowData, atColumnIndex){
            return false;
        },

        getContentObjects: function(content){
            // our content is already an array of objects
            return content;
        },
        
        getRowDataId: function(rowData){
            return rowData.id;
        },
        
        // Summary:
        //    Returns the type of the cell to be displayed. By default, the TableViewer
        //    will support a set of cell types. The type of the cell will map to a specific
        //    cell renderer (to be discussed).
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_TEXT;
            return type;
        },
        
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "data";
            switch (atColumnIndex) {
                case 0:
                    val = rowData.col1;
                    break;
                case 1:
                    val = rowData.col2;
                    break;
                case 2:
                    val = rowData.col3;
                    break;
                case 3:
                    val = rowData.col4;
                    break;
            }
            return val;
        }
    });
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.approval.ApprovalExtendTable", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="domNode" tabindex="-1">' +
        //        '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" dojoAttachPoint="viewHeaderNode">' +
        //        '  </div>' +
        '</div>',
        widgetsInTemplate: true,
        
        constructor: function(args){
            this._childrenApprovalDTO = null;
            this._itemId = null;
            this.content = null;
            this._id = 0;
        },
        
        postCreate: function(){
            this.inherited("postCreate", arguments);
        },
        
        initialize: function(args){
			this._childrenApprovalDTO = args.childrenApprovalDTO;
			
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();
            this.domNode.appendChild(tableViewer.domNode);
            this.tableViewer = tableViewer;
            
            this.content = this.createContent();
            var contentProvider = new ApprovalExtendTable.TableProvider();
            tableViewer.setContent(this.content);
            tableViewer.setContentProvider(contentProvider);
			tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("msgReviewSummaryApproval")));
            tableViewer.show();
        },
        
        createContent: function(){
            var rows = [];
            if (this._childrenApprovalDTO == null) {
                return rows;
            }
            for (var i = 0; i < this._childrenApprovalDTO.length; i++) {
                var res = this._childrenApprovalDTO[i];
				var summary = res.summary;
				var approvedNum = "";
				var rejectedNum = "";
				var reviewedNum = "";
				if(summary.approvalTotal!=0){
					approvedNum = summary.approveNum + " " + Constants.APPROVAL_OF_APPROVAL + " " + summary.approvalTotal + " " + Constants.APPROVAL_APPROVED_APPROVAL_TABLE;
					rejectedNum = summary.rejectNum + " " + Constants.APPROVAL_OF_APPROVAL + " " + summary.approvalTotal + " " + Constants.APPROVAL_REJECTED_APPROVAL_TABLE;
				}
				if (summary.reviewTotal != 0) {
					reviewedNum = summary.reviewNum + " " + Constants.APPROVAL_OF_APPROVAL + " " + summary.reviewTotal + " " + Constants.APPROVAL_REVIEWED_APPROVAL_TABLE;
				}
                var obj = {
                    col1: res.name,
                    col2: approvedNum,
                    col3: rejectedNum,
                    col4: reviewedNum,
                    id: res.relatedItemId
                };
                rows.push(obj);
            }
            return rows;
        }
    });
    
})();
