/******************ExecutionConsole******************************************/
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.common.ExecutionConsole");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.core.loader");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionClient");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionWorkItemClient");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient");
dojo.require("com.ibm.rqm.execution.web.client.internal.mapper.ExecutionResultGroupMapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.execution.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container 	

var ServiceResponseHandler			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var ExecutionClient 				= com.ibm.rqm.execution.web.client.internal.execution.ExecutionClient;
var ExecutionWorkItemClient			= com.ibm.rqm.execution.web.client.internal.execution.ExecutionWorkItemClient;
var ERGClient 						= com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient;
var GlobalEvents 					= com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
var ImageCache  	   				= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility     		   			= com.ibm.rqm.execution.web.ui.internal.util.Utility;
var Constants                       = com.ibm.rqm.planning.web.client.internal.Constants;
var ERGMapper						= com.ibm.rqm.execution.web.client.internal.mapper.ExecutionResultGroupMapper;

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.common.ExecutionConsole",
			 [dijit._Widget, dijit._Templated], 
{
		
	templateString: "<div class=\"\">"+
					"<div dojoAttachPoint=\"_table\"></div>"+
					"</div>",
	
	consoleNode: true,
	
	postCreate: function(){
		this.initialize();
	},
		
	initialize: function(){
		if (!dojo.getObject("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionTableView")) {
			jazz.core.loader.load_async("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionTableView", dojo.hitch(this, "_createExecutionTable"));
		}else{
			this._createExecutionTable();
		}
	},
	
	_createExecutionTable: function () {
		if(!this._executionTable){
			var actions = [{
		        actionId: "refresh",
		        label: Constants.getMessage("labelRefresh"),
		        actionHandler: dojo.hitch(this, "onRefresh"),
		        iconClass: "refresh-icon-image",
				disabled: false
	    	}];
			var args = {
				requests: [],				
				actions: actions,
				queryName: this.name,
                isConsoleNode: this.consoleNode,
                // initialize options for the view builder
                _searchFunction: dojo.hitch(ERGClient, "getSearchExeConsole"),
                _searchCallback: dojo.hitch(this, "_showTable")
            };

			if(this.ergId){
				args.ergId = this.ergId;
			}
			var _executionTable = new com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionTableView(args);
            if(_executionTable.viewHeaderNode && !_executionTable.viewHeaderNode.uninitialized){
				_executionTable.viewHeaderNode.setCSHelpId("rqm_exec_console.html");
				_executionTable.viewHeaderNode.setCSHelpButtonLabel(Constants.getMessage("labelExecutionTableViewHelp"));
			}	
			_executionTable.setInProgress(Constants.getMessage("msgQueryTableList", this.name));
			dojox.data.dom.removeChildren(this._table);
			this._table.appendChild(_executionTable.domNode);
			this._executionTable = _executionTable;

		} else {
		    this._executionTable.refreshQuery();
		    this._executionTable.disableRefresh();
		}
	},
	
	_showTable: function(results){
	    // display TER results
        this._createTable((results.length > 0) ? results[0] : []);
        if (this.consoleNode) {
            this._setAllERGs((results.length > 1) ? results[1] : []);
        }
	},
	
	_createTable: function(requests){
		this.requests = requests;
		this._executionTable.setConsoleNode(this.consoleNode);
		if(this.consoleNode){
			this.requests = [];
			if(requests){//Since IMPORT requests are also getting returned
				var len = requests.length;
				for(var i=0;i<len;i++){
					if(requests[i].requestType == "EXECUTE"){//non-nls
						this.requests.push(requests[i]);
					}
				}	
			}
			requests = this.requests;
		}
		this._executionTable.setContent(requests ? requests : []);
		if(this.consoleNode)
			this._executionTable.showTable();
	},
	
	_setAllERGs: function(ergs){
		this.suiteRequests = [];
		if(ergs){
			var len = ergs.length;
			var labelMultiple = Constants.getMessage("labelMultiple");
			for(var i=0;i<len;i++){
				var erg = ergs[i];
				var suiteRequest ={};
				suiteRequest.scriptTypeName = erg.name;
				suiteRequest.id = erg.id;
				suiteRequest.scriptName = labelMultiple;
				suiteRequest.configuration = labelMultiple;
				suiteRequest.machine = labelMultiple;
				suiteRequest.testCaseName = erg.name;
				suiteRequest.icon = ImageCache.TEST_SUITE;
				suiteRequest.modified = erg.modified;
				suiteRequest.link = Utility.runSuite({groupId: erg.id});
				suiteRequest.progress = 0;
				var suiteProgress = 0;
				var suiteState = null;
				var len2 = erg.elements ? erg.elements.length: 0;
				for(var j=0;j<len2;j++){
					var element = erg.elements[j];
					var request = element.request;
					if (request) {
						suiteProgress += parseInt(request.progress);
						if(	request.state && 
							request.state.id == "com.ibm.rqm.executionframework.common.requeststate.cancelled" && 
							erg.sequentialExecution && suiteState == null )
						{
							//In case of sequential Exec if one req is cancelled, suite is cancelled
							suiteState = {
								id: "com.ibm.rqm.executionframework.common.requeststate.cancelled",
								name: Constants.getMessage("msgSuiteExecCancelled") 
							}
						}
					}
				}
				if(suiteProgress != 0 && len2 != 0){
					suiteRequest.progress = parseInt(suiteProgress /  len2) ; 
				}
				//TODO: HACK to be removed. Change state to result here since 'state' filed is already used up
				suiteRequest.resultState = erg.state;
				
				//Build Suite state here
				if(suiteState == null){
					suiteRequest.state = {
						id: "com.ibm.rqm.executionframework.common.requeststate.taken",
						name: Constants.getMessage("msgSuiteExecRunning")
					}
					if(!suiteRequest.progress || suiteRequest.progress == 0){
						suiteRequest.state.id = "com.ibm.rqm.executionframework.common.requeststate.nottaken";
						suiteRequest.state.name = Constants.getMessage("msgSuiteExecQueued"); 
					}else if(suiteRequest.progress == 100){
						suiteRequest.state.id = "com.ibm.rqm.executionframework.common.requeststate.complete";
						suiteRequest.state.name = Constants.getMessage("msgSuiteExecCompleted");
					}
				}else{
					suiteRequest.state = suiteState;
				}	
				this.suiteRequests.push(suiteRequest);
			}
		}
		this._executionTable.appendItem(this.suiteRequests);
		this._executionTable.enableRefresh();
	},
	
	onRefresh: function(){
		this.initialize();
	},

//TODO: Add updates to the console.
//	 // handler on artifacts changed
//    onArtifactChanged: function(changeType, artifactType, artifactArray) {
//        // only care about requests
//        if (artifactType == Constants.TYPE_REQUEST) {
//            if (changeType == GlobalEvents.ArtifactsAdded) {
//                // add requests to the table
//                if (!this.requests) {
//                    this.requests = artifactArray;
//                } else {
//                    this.requests = this.requests.concat(artifactArray);
//                }
//                if(this._executionTable)
//					this._executionTable.setContent(this.requests);
//            } else {
//                // iterate through the array of artifacts
//                for (var i = 0; i < artifactArray.length; i++) {
//                    // iterate through the array of requests in the table
//                    for (var j = 0; j < this.requests.length; j++) {
//                        // handle the matching requests
//                        if (artifactArray[i].id == this.requests[j].id) {
//                            if (changeType == GlobalEvents.ArtifactsUpdated) {
//                                // swap requests on update
//                                this.requests.splice(j, 1, artifactArray[i]);
//                                if(this._executionTable)
//									this._executionTable.setContent(this.requests);
//                            } else if (changeType == GlobalEvents.ArtifactsDeleted) {
//                                // remove requests from table on delete
//                                this.requests.splice(j, 1);
//                                if(this._executionTable)
//									this._executionTable.setContent(this.requests);
//                            }
//                            break;
//                        }
//                    }
//                }
//            }
//        }
//    },
	
	_error: function(errorObj){
		console.error("ERROR in Execution console "+errorObj.message);
	}
	
});
	
})();