/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.common.ListNavigator");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.ListNavigator");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

(function()
{
var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;

/**
 * Widget to display list of link elements in the LeftNavigation.
 */
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.ListNavigator",
				 [dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/ListNavigator.html"),
//		templateCssPath: dojo.uri.dojoUri("../com.ibm.rqm.planning.web/ui/internal/view/templates/common/ListNavigator.css"),

		constructor: function() 
		{
			this.controller = null;
			this._entries = [];
		},

		postCreate: function() 
		{
//			this.controller = args.controller;
//			this.parentController = args.parentController;
//			this._bold = args.bold;			
		},
		
		_entries: [],
		_bold: false,
		
		/**
		 * Adds a collection of queries (with 'name' property) to the widget
		 */
		addListEntries: function(newEntries) 
		{
			if (newEntries != null) 
			{
				// Sort them first.
				//var sorted = this._sort(newEntries);
				for (var i = 0; i < newEntries.length; i++) 
				{
					//this.addElement(sorted[i]);
					this.addEntry(newEntries[i]);
				}
			}
		},

		/**
		 * Adds an query to the widget.
		 */
		addEntry: function(entry) 
		{
			//if this is the first entry being added, clear the list div
			if( this._entries == null || this._entries.length == 0 ) 
			{
				dojox.data.dom.removeChildren(this.elementListDiv);	
			}
						
			this._entries.push(entry);
			entry.id = this._entries.length;
             
            var entryDiv = document.createElement("div");
			
			entryDiv.className = (this.bold)? "element-div-unselected-bold" : "element-div-unselected";

            entryDiv.appendChild(this._buildEntryLink(entry));
    		entry.div = entryDiv;
				
            this.elementListDiv.appendChild(entryDiv);
		},

		/**
		 * Adds a link for the query to the list.
		 */
		_buildEntryLink: function(entry) 
		{
			var linkText = entry.name;
			var anchor= document.createElement("a");

			anchor.title = linkText;
			
			// set as index into this._entries array
			anchor.id = entry.id-1;
			anchor.className = (this.bold) ? "element-div-unselected-bold" : "element-div-unselected";

			if (entry.isUnique)
			{
				this.connect(anchor, "onclick", this._onClickUniqueUrl);
				entry.href = Utility.reuniquifyUrl(entry.href);
			}
			else
			{
				anchor.href = entry.href;
			}
			
			anchor.appendChild(document.createTextNode(linkText));
			
			return anchor;
		},
		
		_onClickUniqueUrl: function(e)
		{
			var index = e.target.id;
			var href = this._entries[index].href;
			this._entries[index].href = Utility.reuniquifyUrl(this._entries[index].href);
			window.location.href = href;
		},
			
		selectEntry: function(name) 
		{
			/*
			 * Highlight the selected option
			 */
			var entryLinks = this.elementListDiv.childNodes;
			for (var i=0; i < entryLinks.length; i++) 
			{
				var anchorText = entryLinks[i].textContent;
				if(anchorText == name) 
				{
					entryLinks[i].className = (this.bold)? "element-div-selected-bold" : "element-div-selected";
				} 
				else
				{
					entryLinks[i].className = (this.bold)? "element-div-unselected-bold" : "element-div-unselected";
				}
			}
		},
		
		/**
		 * Helper function to sort a list of report descriptors
		 * Note: Assume that report names are unique
		 */
		_sort: function(elements) 
		{
			var names = [];
			for (var i = 0; i < elements.length; i++) 
			{
				names[i] = elements[i].name;
			}
			names = names.sort();
			var result = [];
			for (var i = 0; i < names.length; i++) 
			{
				result[i] = this._find(names[i], elements);
			}
			return result;
		},
		
		_find: function(name, elements) 
		{
			for (var i = 0; i < elements.length; i++) 
			{
				if (elements[i].name == name) 
				{
					return elements[i];
				}
			}
			return null;
		}
	}
);

})();
