/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.common.NewDefectDataInitializer");


dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.client.internal.config");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient");
dojo.require("com.ibm.rqm.defects.web.client.ChangeRequestManager");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient");

(function() { // module container   

var Constants                       = com.ibm.rqm.planning.web.client.internal.Constants;
var ServiceResponseHandler          = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var RequirementWorkItemClient       = com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient;
var ChangeRequestManager			= com.ibm.rqm.defects.web.client.ChangeRequestManager;
var getCurrentProjectArea           = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var ExecutionResultGroupClient      = com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient;

com.ibm.rqm.execution.web.ui.internal.view.common.NewDefectDataInitializer = {

    DESCRIPTION_ATTR: "description",
    LINKTYPES_ATTR: "linkTypes",
    TESTCASE_ATTR: "com.ibm.rqm.workitem.related.testCase",
    REQUIREMENT_ATTR: "com.ibm.rqm.workitem.related.requirement",
    TEST_ENV_ATTR: "com.ibm.rqm.workitem.related.testEnvironment",
    BUILD_ID_ATTR: "com.ibm.rqm.workitem.related.buildId",
    FAILED_VER_POINT_ATTR: "com.ibm.rqm.workitem.related.failedVerificationPoint",
	REQUIREMENT_NAME: "com.ibm.rqm.workitem.related.requirement.name",

    // Obtains pertinent RQM data for a new defect. These data will be used to
    // seed a new defect form. Some data need to be fetch asychronously, the entire
    // data will be returned via the passed in callback. The callback function should
    // expect a single parameter containing the seed attribute values.
    // 
    getDefectSeedValuesFromEWI: function(ewi,scriptName,ergId,resultId, callback)
    {
        var respondObject = {
            self: this,
            cb: callback,
            ewiObj: ewi,
            reqs: null,
            scriptName:scriptName,
			ergId:ergId,
			resultId:resultId,
            saveReqs: function(reqs){
                this.reqs = reqs;
                
                // success or failed, clear the flag, we already have fetched the reqs
                this.fetchReqs = false;
                
                if(this.ewiObj.configuration) {
                    var responseHandler = new ServiceResponseHandler(this, "saveConfig", "failure");
                    var config = new com.ibm.rational.test.lm.web.client.internal.config();
                    config.retrieveConfigurationWithDetails(responseHandler, this.ewiObj.configuration);
                    return;
                }
                this.self._getTestSuitesFromERG(this.ewiObj, this.scriptName,reqs, null, this.cb, this.ergId,this.resultId);
            },
            saveConfig: function(config)
            {
                this.self._getTestSuitesFromERG(this.ewiObj, this.scriptName,this.reqs, config, this.cb, this.ergId,this.resultId);
            },
            failure: function(){
                // If the fetch requirements failed, continue to fetch the configuration's
                // data. 
                if (this.fetchReqs) {
                    this.saveReqs(null);
                }
                else {
                    this.self._getTestSuitesFromERG(this.ewiObj,this.scriptName, this.reqs, null, this.cb, this.ergId,this.resultId);
                }                  
            }
        }
        
        var responseHandler = new ServiceResponseHandler(respondObject, "saveReqs", "failure");
        if (ewi.testCase) {
            // fetch the related requirments for the testcase.
            respondObject.fetchReqs = true;
            RequirementWorkItemClient.getRequirementWorkItemDTOs(responseHandler, {
                testCaseItemId: ewi.testCase.versionableItemId
            });
        }
        else {
            respondObject.saveReqs(null);
        }
        
    },
    
	_getERGIdFromResultId:function(ewi, scriptName, reqs, config, cb, ergId,resultId){
		var self = this;			
		var responseObj ={
			cb: cb,
			ewi: ewi,
			reqs: reqs,
			config: config,
			scriptName:scriptName,
			success: function(ergs){
				var ergId = ergs? ergs[0].id:null;
				self._getTestSuitesFromERG(this.ewi, this.scriptName, this.reqs, this.config,this.cb,ergId,null);
				
			},
			failure: function(errorObj){
				self._getTestSuitesFromERG(this.ewi, this.scriptName, this.reqs, this.config,this.cb,null,null);
			}
		}
		if(resultId){
			var responseHandler = new ServiceResponseHandler(responseObj, "success", "failure");
			ExecutionResultGroupClient.getExecutionResultGroupDTOs(responseHandler, {
				resultId: resultId,
				includeElements: false
			});
		}else
			this._getTestSuitesFromERG(ewi, scriptName, reqs, config,cb,ergId,null);
	},
	_getTestSuitesFromERG: function(ewi, scriptName, reqs, config, cb, ergId,resultId) {
		var responseObj = {
			self: this,
			ergId: ergId,
			cb: cb,
			ewi: ewi,
			reqs: reqs,
			config: config,
			scriptName:scriptName,
			success: function(suites){
				if(suites){
					this.self._constructSeedValues(this.ewi, this.scriptName, this.reqs, this.config, suites, this.cb);
				}
				else{
					this.self._constructSeedValues(this.ewi, this.scriptName, this.reqs, this.config, null, this.cb);	
				}
			},
			failure: function(){
				// if it failed still continue to build the description 
				console.error("failed to fetch test suites for ewi " + ewi.itemId);
				this.self._constructSeedValues(this.ewi, this.scriptName, this.reqs, this.config, null, this.cb);
			}
		}
		if(ergId){
			var responseHandler = new ServiceResponseHandler(responseObj, "success", "failure");
			ExecutionResultGroupClient.getTestExecutionSuiteMinimalSummaryDTOs(responseHandler, {
				resultGroupId: ergId
			});
		}
		else if(resultId){
			this._getERGIdFromResultId(ewi, scriptName, reqs, config, cb, ergId,resultId);
		}else{
			this._constructSeedValues(ewi, scriptName, reqs, config, null, cb);
		}
	},
	
	getDefectDescription: function(ewi, ergId, resultId, currentProvider, callback) {
		var self = this;
		this.getDefectSeedValuesFromEWI(ewi, null, ergId, resultId, dojo.hitch(self, function(seedValues) {
			var desc = seedValues[self.DESCRIPTION_ATTR];
			var relatedReq = seedValues[self.REQUIREMENT_NAME];
 			desc = self._appendToField(desc, "labelRequirement", relatedReq);

			var relatedTestEnv = seedValues[self.TEST_ENV_ATTR];
			desc = self._appendToField(desc, "labelTestEnvironment", relatedTestEnv);

			if (currentProvider.linkType == ChangeRequestManager.PROVIDER_LINK_TYPE_CQREST) {
				var resourceUri = location.protocol + '//' + location.host + net.jazz.ajax._contextRoot + "/oslc_qm/contexts/" + getCurrentProjectArea().itemId + '/resources/com.ibm.rqm.planning.VersionedTestCase/' + ewi.testCase.versionableItemId;
				desc = self._appendToField(desc, "labelCalmLinks", resourceUri);
			}
			
			if (dojo.isFunction(callback)) {
				callback(desc);
			}
		}));
	},
	
	_appendToField: function(field, labelKey, value) {
		var newField = field;
		if (value) {
			newField += "\r\n";
			newField += Constants.getMessage(labelKey) + ": " + value;
			newField += "\r\n";
		}
		return newField;
	},

    _constructSeedValues: function(ewi, scriptName,reqObjects, configObj, suites, callback)
    {
        var desc = "";
        var seedValues = {};
        var tcName = (ewi.testCase && ewi.testCase.name) ? ewi.testCase.name : "";
        if(reqObjects && dojo.isArray(reqObjects)) {
        	// FIX for 17850, as 'related requirement' field changed to 'work item' type in 17087,
			// it can only accept one work item uuid instead of multiple. 
            //var reqNames = [];
            //for(var i = 0; i < reqObjects.length; i++) {
            //    reqNames.push(reqObjects[i].name);
            //}
            if (reqObjects.length > 0) {
            	seedValues[this.REQUIREMENT_ATTR] = reqObjects[0].itemId;
				seedValues[this.REQUIREMENT_NAME] = reqObjects[0].name;
            }
        }
        
        var testplan = ewi.testPlan;
        if (testplan) {
        	desc = this._appendToField(desc, "labelTestPlan", testplan.name);
        }

       	desc = this._appendToField(desc, "labelTestCase", tcName);
        
		var scriptNames = ewi.testCaseScriptNames;
		if (scriptNames!=null && scriptNames.length>0) {
        	desc = this._appendToField(desc, "labelTestScript", scriptNames[0]);
		}
		
		var projectArea = getCurrentProjectArea();
       	desc = this._appendToField(desc, "labelProjectArea", projectArea.name);
       	
        if (suites && dojo.isArray(suites)) {
        	desc = this._appendToField(desc, "labelTestSuite", suites[0].name);
        }
        
        seedValues[this.DESCRIPTION_ATTR] = desc;
        seedValues[this.TESTCASE_ATTR] = tcName;
        
        if(configObj) {
            var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
            var config = catalog.createConfigurationFromJson(configObj.configuration);
            var attributeIds = catalog.getDisplayTypes();
            var envDetails = [];
            var attributeIdsLength = attributeIds.length;
            for(var i = 0; i < attributeIdsLength; i++){
                var attribute = config.getValue(attributeIds[i]);
                if(attribute) {
                    envDetails.push(attribute);
                }
            }
            if(envDetails.length > 0)
                seedValues[this.TEST_ENV_ATTR] = envDetails.join();
        }
        if(ewi.testCase && ewi.testCase.id)
            seedValues[this.LINKTYPES_ATTR] = this._createLink(ewi);
            
        if(callback)    
            callback(seedValues);
    },
    
    _createLink: function(ewi)
    {
        var links = [];
        var link = {
            _eQualifiedClassName: "com.ibm.team.workitem.rest.dto:LinkDTO",
            displayName: Constants.getMessage("labelRelatedArtifacts"),
            _isNew: true,
            id: "com.ibm.team.workitem.linktype.relatedartifact",
            isSingleValued: false
        };
        
        var dtos = [];
        var uri = this._getTestCaseURI(ewi.testCase);
        dtos.push({
            _eQualifiedClassName: "com.ibm.team.workitem.rest.dto:LinkDTO",
            comment: Constants.getMessage("labelTestCase") + ": " + ewi.testCase.name,
            url: uri
        });
        link.linkDTOs = dtos;
        links.push(link);
        return links;
    },
    
    _getTestCaseURI: function(testCase) {
        var result = window.location.protocol + "//" + window.location.hostname + ":" + window.location.port;
        result+= window.location.pathname.match(/(.*)\/web\/console\/[^\/]*$/)[1];
        result+= "/resource/itemOid/com.ibm.rqm.planning.VersionedTestCase/" + testCase.versionableItemId;
        return result;
    }
           
};

})();