/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableFormField");

dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog");

(function() {

var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableFormField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    currentValue: null,
    disabled: false,
    textFieldInvalidFormatMsg: "",
    artifactTemplate: '<div dojoAttachPoint="mainNode" ' +
                      // IE has the CSS "overflow: visible !important" for DIV and does not obey the overflow style in PopupFormField.TooltipDialog.
                      // Set the overflow here directly.  Somehow need to override overflow, can't just set overflow-x and overflow-y.
                      (dojo.isIE ? 'style="zoom:1;overflow:auto;overflow-y:hidden !important;overflow-x:auto !important">' : '>') +
                      '<table width="100%">' +
                      '  <tr><td><table dojoAttachPoint="simplePane">' +
                      '    <tr><td align="right" dojoAttachPoint="simpleButtonNode" style="white-space:nowrap"></td></tr>' +
                      '    <tr><td dojoAttachPoint="simpleNode" style="white-space:nowrap"></td></tr>' +
                      '  </table></td></tr>' +
                      '  <tr><td><table dojoAttachPoint="advancedPane" style="display:none">' +
                      '    <tr><td align="right" dojoAttachPoint="advancedButtonNode" style="white-space:nowrap"></td></tr>' +
                      '    <tr><td dojoAttachPoint="advancedNode" style="white-space:nowrap"></td></tr>' +
                      '  </table></td></tr>' +
                      '</table>' +
                      '</div>',
    
    constructor: function() {
        this.currentValue = null;
        this.artifactType = null;
        this.isSingleSelect = false;
        this.advancedDisplayMode = false;  // current display mode
        this.simpleWidth = 300;
        this.advancedWidth = 750;
    },
    
    postCreate: function() {
        if (!this.artifactType) {
            // validation to make sure consumer of this widget supplies the required argument
            throw new Error("artifactType must be defined");
        }
        if (!this.simpleKey) {
            // validation to make sure consumer of this widget supplies the simple key for the API param
            throw new Error("simpleKey must be defined for the API");
        }
        if (!this.advancedKey) {
            // validation to make sure consumer of this widget supplies the advanced key for the API param
            throw new Error("advancedKey must be defined for the API");
        }

        this.inherited("postCreate", arguments);
        
        this.name = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField({
            fieldLabel: Constants.getMessage("labelName"),
            showFieldLabel: !this.inDynamicControl,  // don't show label if simple mode only
            defaultValue: "",
            onValueChanged: dojo.hitch(this, "_onNameChanged")
        });

        // only support simple mode (name only) when within the dynamic filters pane to avoid
        // complicated UI nested pop-up
        //
        if (!this.inDynamicControl) {
            this.artifactField = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(this.artifactTemplate);

            dojo.style(this.artifactField.simplePane, "width", this.simpleWidth + "px");
            dojo.style(this.artifactField.simplePane, "margin", "0px !important");
            dojo.style(this.artifactField.simplePane, "padding", "0px !important");
            dojo.style(this.artifactField.advancedPane, "width", this.advancedWidth + "px");
            if (dojo.isIE) {
                // add the 5px offset to the tooltip dialog, see _setDisplayMode()
                dojo.style(this.artifactField.mainNode, "width", this.simpleWidth + 5 + "px");
            }
            var simpleButton = document.createElement("button");
            simpleButton.innerHTML = Constants.getMessage("labelAdvanced");
            simpleButton.className = "action-link";
            simpleButton.tabIndex = "0";
            this.connect(simpleButton, "onclick", "_toggleDisplayMode");
            this.artifactField.simpleButtonNode.appendChild(simpleButton);

            this.artifactField.simpleNode.appendChild(this.name.domNode);
        }

        // advanced pane is not created until the user explicitly changed from simple mode to advanced mode
        this.formField = this.inDynamicControl ? this.name.domNode : this.artifactField.mainNode;
        var parent = (this.alterHolderNode || this.fieldHolderNode);
        parent.appendChild(this.formField);
        
        if (this.defaultValue) {
            this.setValue(this.defaultValue);
        }
    },

    _createAdvancedPane: function() {
        if (!this.artifactTable) {
            var advancedButton = document.createElement("button");
            advancedButton.innerHTML = Constants.getMessage("labelSimple");
            advancedButton.className = "action-link";
            advancedButton.tabIndex = "0";
            this.connect(advancedButton, "onclick", "_toggleDisplayMode");
            this.artifactField.advancedButtonNode.appendChild(advancedButton);

            var params = {
                showQueryFilters: true,
                staticProfile: true,
                isSingleSelect: this.isSingleSelect,
                itemType: this.artifactType,
                tableContainer: this.artifactField.advancedNode,
                allowSelectAll: false,
                allowChangingTableSettings: false,
                pageSize: 6,
				advancedParams:this.advancedParams
            };
            this.artifactTable = new com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog(params);
        }
    },
    
    uninitialize: function() {
        this.name.destroy();
        if (this.artifactTable) {
            this.artifactTable.destroyRecursive();
        }
    },

    _toggleDisplayMode: function() {
        this.advancedDisplayMode = !this.advancedDisplayMode;
        this._setDisplayMode(this.advancedDisplayMode);
    },
    
    _setDisplayMode: function(isAdvanced) {
        if (isAdvanced) {
            this._createAdvancedPane();
            this.artifactField.simplePane.style.display = "none"
            this.artifactField.advancedPane.style.display = "";
            
            if (!this.show) {
                this.artifactTable.show();
                this.show = true;
            }
            if (dojo.isIE) {
                dojo.style(this.artifactField.mainNode, "width", this.advancedWidth + 5 + "px");
            }
            if (dojo.isFunction(this.setWidth)) {
                this.setWidth(this.advancedWidth + 5);
                // IE does not show dotted line for active DIV as in FF, the 3px padding for the containerNode
                // in PopupFormField.TooltipDialog would clipped the scroll bar.
                dojo.style(this.domNode, "width", this.advancedWidth + (dojo.isIE ? 6 : 0) + "px");
            }
        } else {
            this.artifactField.advancedPane.style.display = "none";
            this.artifactField.simplePane.style.display = ""
            if (dojo.isIE) {
                dojo.style(this.artifactField.mainNode, "width", this.simpleWidth + 5 + "px");
            }
            if (dojo.isFunction(this.setWidth)) {
                this.setWidth(this.simpleWidth + 5);
                dojo.style(this.domNode, "width", this.simpleWidth + (dojo.isIE ? 6 : 0) + "px");
            }
        }
    },
    
    _onNameChanged: function(widget, oldValue, newValue) {
        if (newValue) {
            this._valueChangedHandler();
        }
    },

    _valueChangedHandler: function() {
        var oldValue = this.currentValue;
        this.currentValue = this.getValue(); 
        
        this.onValueChanged(this, oldValue, this.currentValue);
    },
    
    onPopupOpen: function() {
        // The artifact table is only created when needed.  In the case the cached filter value
        // is in advanced display with the artifact table, we need to switch the default simple
        // display when instantiated to the advanced display upon the first user click on the toggle.
        //
        if (!this.popupOpenning && this.currentAdvancedValue && this.advancedDisplayMode) {
            // _setDisplayMode() will trigger the popup close and open and we 
            // don't want to run into a recursive infinite loop
            this.popupOpenning = true;
            this._setDisplayMode(this.advancedDisplayMode);
            this.setValue(this.currentAdvancedValue);
            delete this.currentAdvancedValue;
            delete this.popupOpenning;
        }
    },
    
    setValue: function(value) {
        if (value) {
            this.advancedDisplayMode = value[this.advancedKey] != undefined;
            if (this.advancedDisplayMode) {
                if (this.artifactTable) {
                    this.artifactTable.setSelection(value['advancedObj']); // artifactTable will validate the param
                } else {
                    // just cache value, no need to create the artifact table until user click to open
                    this.currentAdvancedValue = value;
                }
            } else {
                if (value[this.simpleKey]) {
                    this.name.setValue(value[this.simpleKey]);
                } else {
                    this.name.clearValue();
                }
            }
        }
    },

    getValue: function() {
        var value = {};
        if (!this.advancedDisplayMode) {
            value[this.simpleKey] = this.name.getValue();
        } else if (this.artifactTable) {
            var selection = this.artifactTable.getSelection() || [];
            var count = selection.length;
            var selectionVersionID = [];
            for (var i = 0; i < count; i++) {
                selectionVersionID.push(selection[i].versionableItemId);
            }
            value[this.advancedKey] = selectionVersionID;
            value['advancedObj'] = selection;
        } else if (this.currentAdvancedValue) {
            // advanced value was set but table is not created yet, use cached value
            value = this.currentAdvancedValue;
        }
        return value;
    },
    
    getValueInString: function() {
        if (!this.advancedDisplayMode) {
            return this.name.getValue();
        } else {
            var selection = [];
            if (this.artifactTable) {
                selection = this.artifactTable.getSelection();
            } else if (this.currentAdvancedValue) {
                // advanced value was set but table is not created yet, use cached value
                selection = this.currentAdvancedValue['advancedObj'];
            }
            var count = selection.length;
            if (count == 0) {
                return "";
            } else {
                var selectionStr = selection[0].id;
                for (var i = 1; i < count; i++) {
                    selectionStr = Constants.getCommonMessage("labelOrValues", selectionStr, selection[i].id);
                }
                return selectionStr;
            }
        }
        return "";
    },
    
    clearValue: function() {
        if (this.advancedDisplayMode) {
            // advanced value was set but table is not created yet, use cached value
            if (this.artifactTable) {
                // setSelection is invoked in a delay manner. Need this to happen immediately
                // set the force flag to true.
                this.artifactTable.setSelection([], /*force*/true);
            }
            if (this.currentAdvancedValue) {
                delete this.currentAdvancedValue;
            }
        }
        this.name.clearValue();
    },

    _getBaseClassName: function() {
        return "text-field";
    }
});

})();
