/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.planning.web.ui.internal.view.common.SortableFilterableTable");

SortableFilterableTable = dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SortableFilterableTable");

SortableFilterableTable.DEFAULT_MAX_ROWS = 50;

SortableFilterableTable.TYPE_ICON   = 1;
SortableFilterableTable.TYPE_DATE   = 2;
SortableFilterableTable.TYPE_STRING = 3;
SortableFilterableTable.TYPE_NUM    = 4;
SortableFilterableTable.TYPE_SELECT = 5;
SortableFilterableTable.TYPE_BUTTON = 6;

SortableFilterableTable.SORT_ASC = "ascending";
SortableFilterableTable.SORT_DSC = "descending";
SortableFilterableTable.DEFAULT_SORT_DIRECTION = SortableFilterableTable.SORT_DSC;


dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

// Require local widgets
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");

(function() 
{
	var Utility 		= com.ibm.rqm.planning.web.ui.internal.util.Utility;
	var TooltipBuilder 	= com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder;
	var ImageCache 		= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;

	dojo.declare(
		"com.ibm.rqm.planning.web.ui.internal.view.common.SortableFilterableTable",
		[dijit._Widget, dijit._Templated],
		{
			templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/SortableFilterableTable.html"),
		
			/*******************************************************************************
			 * Constructor functions
			 * 
			 *	showPagination = show/hide "Previous" & "Next" and currently displaying rows 
			 *	maxRows = max number of rows to show in the table, # smaller than total results in paging, if enabled
			 *	rangeStart = the row number to start displaying from, counting starting from 1
			 *	isFilterable = should the filter text box be shown/enabled
			 *	filterText = default filter text, used to prefilter the table
			 *	showQuery = show/hide the title for the table
			 *	queryTitle = title for the table
			 *	runQueryHref    = HREF for repopulating the table
			 *	isQueryEditable = allow the original query definition to be edited
			 *	editQueryHref = HREF to edit the current query
			 *	showHeader = show/hide header, which just displays paging info
			 *	showFooter = show/hide footer, which just displays paging info
			 *	centerFooter = display footer info centered
			 *	sortColumn = default sort column, counting starts at 0
			 *	sortDirection = use the constant for this setting
			 *	isRowSelectable = selection enabled (only one thing selectable) when mouse is anywhere over the row
			 *	isDeleteColumn = show/hide delete column, always in the last column position
			 *	isMultiselect = show/hide checkboxes, always becomes the first column, does not affect specifying default sort column
			 *	columnHeadings = array of column headings, see addColumn for a definition of the parameters for a column.
			 *	rows = array of rows, see addRow for a definition of the parameters for a row.
			 *
			 *******************************************************************************/
			constructor: function(args) 
			{
				this.showPagination = true;
				this.maxRows = SortableFilterableTable.DEFAULT_MAX_ROWS;
				this.rangeStart =  1;
				this.isFilterable = true;
				this.filterText = "";
				this.showQuery = true;
				this.queryTitle = "";
				this.runQueryHref    = "";
				this.isQueryEditable = false;
				this.editQueryHref = "";
				this.showHeader = true;
				this.showFooter = true;
				this.centerFooter = false;
				this.sortColumn = -1;
				this.sortDirection = SortableFilterableTable.DEFAULT_SORT_DIRECTION;
				this.isRowSelectable = false;
				this.isDeleteColumn = false;
				this.isMultiselect = false;
				this.columnHeadings = [];
				this.rows = [];
			},
			
			postCreate: function() 
			{
			},
			
			initialize: function(args)
			{
				for (var element in args)
				{
					this[element] = args[element];
				}
				
				if (this.maxRows < 1)
				{
					this.maxRows = SortableFilterableTable.DEFAULT_MAX_ROWS;
				}

				if (this.rangeStart < 1)
				{
					this.rangeStart = 1;
				}

				if (this.sortDirection != SortableFilterableTable.SORT_ASC && this.sortDirection != SortableFilterableTable.SORT_DSC)
				{
					this.sortDirection = SortableFilterableTable.DEFAULT_SORT_DIRECTION;
				}
				
				if (!this.showPagination)
				{
					this.showHeader = false;
					this.showFooter = false;
				}
				
				this.filterInProgress   = false;
				this.queuedFilter       = false;
				this.filterValueEntered = false;
				this.filterableIndices  = [];
				this.filteredRows       = [];
			},

			/*******************************************************************************
			 * Externally callable define and populate table functions
			 * 
			 * Column Header info
			 *	- title <str> -- default is ""
			 *	- isSortable <bool> -- default is TRUE
			 *	- includeInFilter <bool> -- default is TRUE
			 *	- width <int> -- default is no setting
			 *******************************************************************************/

			addColumn: function(col)
			{
				this.columnHeadings.push(col);
				if (col.includeInFilter)
				{
					this.filterableIndices.push(this.columnHeadings.length - 1);
				}
			},
			
			addColumns: function(cols)
			{
				for (var i = 0; i < cols.length; i++)
				{
					this.addColumn(cols[i]);
				}
			},
			
			/*******************************************************************************
			 * Row info
			 *	- href <str> -- default is ""
			 *	- removeParms {} -- {obj = this; parm = whatever is expected by removeMethod}
			 *	- removeMethod -- method to be called when the delete is pressed for a row
			 *  - callbackData -- data to be passed back on either an HREF or callback
			 *	- elements[] -- length should correspond to the number of columns
			 *		- type <int> -- need for icon, select and date columns
			 *		- data <str> -- default is "", need for icon and date type too for sorting;
			 *						special for selects, see below
			 *		- image <str> -- default is "", location of icon image, overrides data field 
			 *						 if present
			 *		- tooltip <str> -- default is "", builds a FunctionalTooltip
			 *		- isLink <bool> -- default is FALSE
			 *      - isVisible <bool> -- default is TRUE
			 *		- href <str> -- URL to use in anchor if isLink == TRUE and isRowSelectable == FALSE
			 *		- callback <function> -- method and object reference to call when a link is selected, 
			 *								 don't specify this if you have an HREF specified, the HREF 
			 *								 takes precedence. Use dojo.hitch to create the function callback
			 *		- callbackObj <obj> -- object to be passed when invoking the callback function, can 
			 *							   be whatever you want
			 *
			 *	NOTE: if the type is SELECT, then pass an array of objects for "data". The objects in the array
			 *		  need to have specified an "id" and "displayName". For example, 
			 * 		  [{id: 1, displayName: "One"}, {id:2, displayName: "Two"}].
			 *******************************************************************************/

			addRow: function(row)
			{
				this.rows.push(row);
			},
			
			addRows: function(rows)
			{
				for (var i = 0; i < rows.length; i++)
				{
					this.addRow(rows[i]);
				}
			},
			
			clear: function()
			{
				this.rows = [];
				this.filteredRows = [];
			},
			
			/*******************************************************************************
			 * Externally callable display function 
			 * 
			 *******************************************************************************/
			displayTable: function()
			{
				// apply initial filter if there is one				
				if ((this.isFilterable) &&
					(this.filterText != ""))
				{
					this._filterControl.value = this.filterText;
					this._filterControl.className = "";
					this._applyFilter();
				}
				else
				{
					this._disableFilterEntry();
					this._disableClearFilterImg();
				}
				
				// apply initial sort if there is one
				if (this.sortColumn >= 0)
				{
					this._sortData();
				}				
				
				if (!this.showPagination)
				{
					this.maxRows = this.rows.length;
				}
				
				this._displayHeader();

				this._displayTable();
			},
			
			/*******************************************************************************
			 * Get selected table rows 
			 *
			 *******************************************************************************/
			getSelectedRows: function()
			{
				var results = [];
				
				if ((this.isMultiselect) &&
					(this.rows != null))
				{
					for (var i = 0; i < this.rows.length; i++)
					{
						if (this.rows[i].checked)
						{
							var row = this.rows[i];
							var data = [];
							
							for (var j = 0; j < row.elements.length; j++)
							{
								var element = row.elements[j];
								
								if (element.type == SortableFilterableTable.TYPE_SELECT)
								{
									data.push(element.select.value);
								}
								else
								{
									data.push(element.data);
								}
							}
							//TODO: this has to make adjustments if there are selects
							results.push(data);
						}
					}
				}
				
				return(results);
			},
			
			/*******************************************************************************
			 * Remove selected table rows 
			 *
			 *******************************************************************************/
			removeSelectedRows: function()
			{
				if ((this.isMultiselect) &&
					(this.rows != null))
				{
					var toReserved = [];
					for (var i = 0; i < this.rows.length; i++)
					{
						if (!this.rows[i].checked)
						{
							toReserved.push(this.rows[i]);		
						}
					}
					this.rows = toReserved;
					this._displayTable();
				}
			},

			setMaxRows: function(num)
			{
				if ((num > 0) &&
				    (num != this.maxRows))
				{
					this.maxRows = num;

					this._displayHeader();

					this._displayTable();
				}
			},
			
			/*******************************************************************************
			 * Sorting functions
			 * 
			 *******************************************************************************/
			_sortData: function(target)
			{
				if ((this.rows) &&
					(this.rows.length > 0))
				{
					Utility.qsort_obj(this.rows, this.sortColumn, 0, this.rows.length);
					
					if (this.rows[0].elements[this.sortColumn].type == SortableFilterableTable.TYPE_DATE)
					{
						if (this.sortDirection == SortableFilterableTable.SORT_ASC)
						{
							this.rows.reverse();
						}
					}
					else if (this.sortDirection == SortableFilterableTable.SORT_DSC) 
					{
						this.rows.reverse();
					}
					
					if (this.filterText != "")
					{
						this._applyFilter();
					}						
							
					this.rangeStart = 1;
					
//					this._displayTable();
				}
			},

			/*******************************************************************************
			 * Filtering functions
			 * 
			 *******************************************************************************/
			_applyFilter: function()
			{
				if (this.filterInProgress)
				{
					console.debug("queueing filter: " + this.filterText);
					this.queuedFilter = true;
					return;
				}
				
				//
				// BEGIN GUARDED SECTION
				//
				this.filterInProgress = true;
				if (this.filterText == "")
				{
					this._doUnfilterAll();
					this._setNoFilterMatchesVisibility(/* isVisible */ false);
				}
				else
				{
					this._doFilter(dojo.trim(this.filterText).toLowerCase());
				}
				this.filterInProgress = false;
				//
				// END GUARDED SECTION
				//
	
				// if we queued a filter during processing, clear it and call
				// ourselves with the queued filter
				if (this.queuedFilter)
				{
					console.debug("Applying queued filter: " + this.filterText);
					this.queuedFilter = false;
					this._applyFilter();
				}
			},

			_doFilter: function(filterStr)
			{
				var noMatches = true;

				this.filteredRows = [];
								
				for (var i=0; i<this.rows.length; i++)
				{
					var row = this.rows[i];

					if (this._isMatch(row, filterStr))
					{
						this.filteredRows.push(row);
						noMatches = false;
					}
				}
				this._setNoFilterMatchesVisibility(/* isVisible */ noMatches);
			},
			
			_doUnfilterAll: function()
			{
				this.filteredRows = [];
			},
			
			_isMatch: function(row, filterStr)
			{
				for (var i=0; i<this.filterableIndices.length; i++)
				{
					var data = row.elements[this.filterableIndices[i]].data;
					
					var type = typeof(data);
					if (typeof data != "string")
					{
						data = "" + data;
					}
					
					if (data.toLowerCase().indexOf(filterStr) !== -1)
					{
						return true;
					}
				}
				return false;
			},

			_setNoFilterMatchesVisibility: function(isVisible)
			{
				this._noFilterMatches.className = isVisible ? "" : "hidden";
			},
			
			_onClearFilterClick: function(e)
			{
				e.preventDefault();
				
				this.rangeStart = 1;
				this.filterValueEntered = false;
				this._resetFilter();
			},
			
			_onSetFilterValue: function(e)
			{
				this.filterText = this._filterControl.value;
				this.filterValueEntered = true;
				this.rangeStart = 1;
				this._applyFilter();
				this._enableClearFilterImg();
				this._displayTable();				
			},
			
			_onFilterFocus: function(e)
			{
				if (this._filterControl._jazzIsReset)
				{
					this._filterControl.value = "";
					this._filterControl.className = "";
				}
				
				this._filterControl._jazzIsReset = false;
			},
			
			_onFilterBlur: function(e)
			{
				var filterVal = dojo.trim(this._filterControl.value);
				
				if (filterVal.length === 0)
				{
					if (this.filterValueEntered)
					{
						this.rangeStart = 1;
						this._resetFilter();
					}
					else
					{
						this._disableFilterEntry();
					}
				}
				else
				{
					this.filterText = filterVal;
					this._filterControl.value = filterVal;
				}
			},

			_disableFilterEntry: function()
			{
				this._filterControl.className = "disabled-text";
				this._filterControl.value = "Type Filter Text";
				this._filterControl._jazzIsReset = true;
			},
						
			_resetFilter: function()
			 {
				// unfilter results
				this._disableFilterEntry();

				this.filterText = "";
				this._applyFilter();
				
				// show disabled form of 'clear filter' icon
				this._disableClearFilterImg();
				
				this._displayTable();				
			},
			
			_enableClearFilterImg: function()
			{
				if (this._clearFilterImg._jazzShowingDisabledImg || this._clearFilterImg.src == "")
				{
					this._clearFilterImg.src = ImageCache.CLEAR_FILTER_ENABLED;
				}
				this._clearFilterImg._jazzShowingDisabledImg = false;
			},
			
			_disableClearFilterImg: function()
			{
				if (!this._clearFilterImg._jazzShowingDisabledImg)
				{
					this._clearFilterImg.src = ImageCache.CLEAR_FILTER_DISABLED;
				}
				this._clearFilterImg._jazzShowingDisabledImg = true;
			},

			/*******************************************************************************
			 * Internal page display functions
			 * 
			 *******************************************************************************/
			_displayHeader: function()
			{
				dojox.data.dom.removeChildren(this._queryRow);

				if ((this.showQuery) && 
				    (this.queryTitle != ""))
				{
					if (this.isQueryEditable)
					{
						var anchor = document.createElement("A");
						anchor.href = this.editQueryHref;
						anchor.appendChild(document.createTextNode("Edit Current Query"));
						
						this._queryRow.appendChild(anchor);
					}
				
					var titleSpan = document.createElement("span");
					titleSpan.className = "";
					
					titleSpan.innerHTML = "<b>Current Query: </b>";

					if (this.runQueryHref != "")
					{
						var anchor = document.createElement("A");
						anchor.className = "";
						anchor.href = this.runQueryHref;
						anchor.appendChild(document.createTextNode(this.queryTitle));
						titleSpan.appendChild(anchor);
					}
					else
					{
						titleSpan.appendChild(document.createTextNode(this.queryTitle));
					}
					
					this._queryRow.appendChild(titleSpan);
				}
				
				this._filterArea.className = "filter-area" + ((this.isFilterable) ? "" : " hidden");
			},
			
			_updateRangeInfo: function(container)
			{
				var list = (this.filterText == "") ? this.rows : this.filteredRows;
				
				// ensure that the rangeStart was not initially set to a number larger than the list size, 
				// and that it hasn't somehow gotten out of whack after the fact
				if (this.rangeStart > list.length)
				{
					this.rangeStart = 1;
				}
				
				if (this.rangeStart > 1)
				{
					var prev = document.createElement("A");
					prev.id = "prev";
					this.connect(prev, "onclick", this._previousClicked);
				}
				else
				{
					var prev = document.createElement("span");
					prev.className = "disabled-text";
				}	

				prev.appendChild(document.createTextNode("Previous"));

				// modify _rangeStart by 1 because it does not start at 0
				var rangeEnd = ((this.rangeStart + this.maxRows - 1) > list.length) ? list.length : (this.rangeStart + this.maxRows - 1);  
				var rangeStart = (rangeEnd == 0) ? 0 : this.rangeStart ;
				
				var range = document.createElement("span");
				range.className = "page-info-span";

				if (this.filterText == "")
				{
					range.innerHTML = "<b>" + rangeStart + " - " + rangeEnd + "</b> of " + list.length;
				}
				else
				{
					range.innerHTML = "<b>" + rangeStart + " - " + rangeEnd + "</b> of " + list.length + " (" + this.rows.length + ")";
				}

				// modify _rangeStart by 1 because it does not start at 0
				if (list.length > (this.rangeStart + this.maxRows - 1))
				{
					var next = document.createElement("A");
					next.id = "next";
					this.connect(next, "onclick", this._nextClicked);
				}	
				else
				{
					var next = document.createElement("span");
					next.className = "disabled-text";
				}			

				next.appendChild(document.createTextNode("Next"));

				dojox.data.dom.removeChildren(container);
				container.appendChild(prev);
				container.appendChild(range);
				container.appendChild(next);
			},
			
			_nextClicked: function(e)
			{
				e.preventDefault();
				this.rangeStart += this.maxRows;
				this._displayTable();
			},
			
			_previousClicked: function(e)
			{
				e.preventDefault();
				this.rangeStart -= this.maxRows;
				if (this.rangeStart < 1)
				{
					this.rangeStart = 1;
				}
				this._displayTable();
			},
			
			_displayTable: function()
			{
				if (this.showHeader)
				{
					this._updateRangeInfo(this._rangeContainer);
				}

				dojox.data.dom.removeChildren(this._tableBody);
				var tableDiv = document.createElement("div");
				this._tableBody.appendChild(tableDiv);
				tableDiv.className = "table-div";								
				var table = document.createElement("table");
				tableDiv.appendChild(table);

				this._displayTableHeader(table);
				
				if (this.rows.length > 0)
				{
					this._noResultsMsg.className = "hidden";
					
					this._displayTableRows(table);
	
					if (this.showFooter)
					{
						this._updateRangeInfo(this._footer);
						
						if (this.centerFooter)
						{
							dojo.addClass(this._footer, "centered");
						}
					}
				}
				else
				{
					this._noResultsMsg.className = "";
				}
			},
			
			_displayTableHeader: function(table)
			{
				var thead = document.createElement("thead");								
				thead.className = "no-row-selection";
				
				var tr = document.createElement("tr");								

				// if isMultiselect, then add a column to the beginning of the table
				if (this.isMultiselect)
				{
					var th = document.createElement("th");
					th.className = "center";								
					
					//TODO: add CB here to select/deselect all
					
					tr.appendChild(th);		
				}								

				for (var i=0; i<this.columnHeadings.length; i++)
				{
					var col = this.columnHeadings[i];
					
					var th = document.createElement("th");								
					th.width = col.width;
					
					if (col.isSortable)
					{
						var anchor = document.createElement("a");
						anchor.appendChild(document.createTextNode(col.title));
						anchor.colIndex = i;
						anchor.title = "Sort " + ((i == this.sortColumn) ? ((this.sortDirection == SortableFilterableTable.SORT_ASC) ? SortableFilterableTable.SORT_DSC : SortableFilterableTable.SORT_ASC) : SortableFilterableTable.DEFAULT_SORT_DIRECTION);
						th.appendChild(anchor);
						this.connect(anchor, "onclick", this._handleSortClick);
						
						if (i == this.sortColumn)
						{
							// TODO: This is a terrible way to put a space between the text and image -- fix this 
							th.appendChild(document.createTextNode("  "));

							var img = document.createElement("img");
							img.src = (this.sortDirection == SortableFilterableTable.SORT_ASC) ? ImageCache.SORT_ASC : ImageCache.SORT_DSC ;
							th.appendChild(img);
						}
					}
					else
					{
						th.appendChild(document.createTextNode(col.title));
					}

					tr.appendChild(th);		
				}

				// if deletable a column, add another column
				if (this.isDeleteColumn)
				{
					var th = document.createElement("th");
					th.className = "center";								
					var img = document.createElement("img");
					img.src = ImageCache.REMOVE_ENABLED;
					th.appendChild(img);
					tr.appendChild(th);		
				}								

				thead.appendChild(tr);		
				table.appendChild(thead);
			},

			_getRow: function(e)
			{
				var element= e.target || e.srcElement;
				// walk up the DOM tree until we find a row element
				while (element && element.tagName !== "TR") {
					element= element.parentNode;
				}
				return element || null;
			},
			
			_onMouseOver: function(e)
			{
				if (this.isDeleteColumn)
				{
					var row = this._getRow(e);
					row.childNodes[row.childNodes.length - 1].firstChild.className = "";
				}
//				console.debug(">>>>>>>>>>> On mouse over <<<<<<<<<<<<<, row = " + row.rowIndex);
			},
			
			_onMouseOut: function(e)
			{
				if (this.isDeleteColumn)
				{
					var row = this._getRow(e);
					row.childNodes[row.childNodes.length - 1].firstChild.className = "hidden";
				}
//				console.debug("<<<<<<<<<<< On mouse out >>>>>>>>>>>>>, row = " + row.rowIndex);
			},

			_attachTooltip: function(anchor, element)
			{
				if (element.tooltip == null ) {
					anchor.title = element.data;				
				} else if ( dojo.isString(element.tooltip) ) {
					anchor.title = element.tooltip;
				} else if ( dojo.isObject(element.tooltip) ) {
					if ( element.tooltip.html != null ) {
						var tip = TooltipBuilder.addTooltip(element.tooltip.html, anchor);							
					}
				}
			},
			
			_displayTableRows: function(table)
			{
				var tbody = document.createElement("tbody");
				table.appendChild(tbody);

				this.connect(tbody, "onmouseover", this._onMouseOver);
				this.connect(tbody, "onmouseout", this._onMouseOut);

				tbody.className = (this.isRowSelectable) ? "" : "no-row-selection";

				var list = (this.filterText == "") ? this.rows : this.filteredRows;
				
				// modify _rangeStart by 1 because it does not start at 0
				var maxNum = ((list.length - this.rangeStart + 1) < this.maxRows) ? (list.length - this.rangeStart + 1) : this.maxRows;  

				for (var i=0; i<maxNum; i++)
				{
					// modify _rangeStart by 1 because it does not start at 0
					var row = list[this.rangeStart - 1 + i];
										
					var tr = document.createElement("tr");
					tbody.appendChild(tr);

					if ((this.isRowSelectable) &&
						(row.href != null)) 
					{
						this.connect(tr, "onclick", this._handleRowClick);
					}
					
					if (this.isMultiselect)
					{
						var td = document.createElement("td");
						td.className = "center";

						var inputLabel = document.createElement("label");
						var cb = document.createElement("input");
						cb.type = "checkbox";
						cb.checked = (row.checked != null) ? row.checked : false;
						
						this.connect(cb, "onclick", this._handleCheckBoxClick);
						
						inputLabel.appendChild(cb);
						td.appendChild(inputLabel);
						tr.appendChild(td);	
					}
					
					for (var j =0; j<row.elements.length; j++)
					{
						var element = row.elements[j];
						if (element.isVisible!=null && element.isVisible==false) {
							continue;
						}
						var td = document.createElement("td");
						tr.appendChild(td);	
						
						if ((element.image) && (element.image != ""))
						{
							if ((element.data != null) &&
								(element.data == ""))
							{
								td.className = "center";
							}
							
							var img = document.createElement("img");														
							img.id = "img_for_row" + i + "_element_" + j;							
							img.alt = element.tooltip;
							img.src = element.image;
							this._attachTooltip(img, element);
							
							if ((element.href) && (element.href!=null))
							{
								img.style.border = '0';							
								var anchor = document.createElement("a");
								anchor.setAttribute("href", element.href);
								anchor.appendChild(img);
								td.appendChild(anchor);
							}
							else
							{
								td.appendChild(img);
							}
							if ((element.data != null) &&
								(element.data != ""))
							{
								var txt = document.createTextNode("   ");
								td.appendChild(txt);
								txt = document.createTextNode(element.data);
								td.appendChild(txt);
							}
						}
						else if (element.isLink)
						{
							var anchor = document.createElement("a");							
							anchor.id = "link_for_row" + i + "_element_" + j;
							//anchor.title = element.tooltip != null ? element.tooltip : element.data;
							
							if (element.href)
							{
								anchor.href = (this.isRowSelectable) ? "" : element.href;
							}
							else if (element.callback)
							{
								this.connect(anchor, "onclick", this._anchorCallback);
							}
							else
							{
								anchor.href = "";
							}
							
							anchor.appendChild(document.createTextNode(element.data));
							td.appendChild(anchor);
							
							this._attachTooltip(anchor, element);
						}
						else if (element.type == SortableFilterableTable.TYPE_SELECT)
						{
							var select = document.createElement("select");
							
							for (var k = 0; k < element.data.length; k++)
							{
								var option = new Option(element.data[k].displayName, element.data[k].id);
								select.appendChild(option);
							}
							
							element.select = select;
							select.className = "select";
							td.appendChild(select);
						}
						else
						{
							if (element.type == SortableFilterableTable.TYPE_DATE)
							{
								var date = new Date()
								date.setTime(element.data);

								td.appendChild(document.createTextNode(Utility.formatDate(date)));
							}
							else
							{
								td.appendChild(document.createTextNode(element.data));
							}
						}
					}
					
					if (this.isDeleteColumn)
					{
						var td = document.createElement("td");
						td.className = "center";

						var img = document.createElement("img");
						img.className = "hidden";
						img.title = "Delete";
						img.alt = "Delete";
						img.src = ImageCache.REMOVE_DISABLED;
						this.connect(img,"onmouseover", this._overRemove);
						this.connect(img,"onmouseout", this._offRemove);
						this.connect(img,"onclick", this._clickRemove);
						td.appendChild(img);
						tr.appendChild(td);	
					}
				}
			},

			_overRemove: function(e)
			{
				e.target.src = ImageCache.REMOVE_ENABLED;
			},
			
			_offRemove: function(e)
			{
				e.target.src = ImageCache.REMOVE_DISABLED;
			},
			
			_clickRemove: function(e)
			{
				e.stopPropagation();

				var list = (this.filterText == "") ? this.rows : this.filteredRows;

				// have to modify _rangeStart by 1 because it starts at 1
				// and subtract 1 from the e.currentTarget.rowIndex because it includes the row for the THEAD
				var index = (this.rangeStart - 1) + (e.target.parentNode.parentNode.rowIndex - 1);
				var parms = list[index].removeParms;
				var removeMethod = list[index].removeMethod;
				// set the list&index in a instance global variable, used by removeElement()
				this.list = list;
				this.indexRemoved = index;
				removeMethod(parms.obj, parms.parm);

// TODO: this really shouldn't happen until the success callback is invoked
//				list.splice(index, 1);
//				this.displayTable();
//				alert("Deleting item at row = " + e.target.parentNode.parentNode.rowIndex);
			},
			
			removeElement: function(col, value)
			{
				//look up index based on col and value
				for (var i = 0; i < this.list.length; i++) {
					var column = this.list[i].elements[col];
					if (column.data == value) {
						//remove this row
						this.list.splice(i, 1);
						break;
					}
				}
				this._displayTable();
			},
			
			//now it can only be used as a callback function of removeMethod() in _clickRemove()
			removeRow: function()
			{
				this.list.splice(this.indexRemoved, 1);
				this._displayTable();
			},
			
			_handleCheckBoxClick: function(e)
			{
				var list = (this.filterText == "") ? this.rows : this.filteredRows;

				// have to modify _rangeStart by 1 because it starts at 1
				// and subtract 1 from the e.parentNode.parentNode.rowIndex because it includes the row for the THEAD
				var rowIndex = e.target.parentNode.parentNode.rowIndex;
				list[(this.rangeStart - 1) + (rowIndex - 1)].checked = e.target.checked;
			},
			
			_handleSortClick: function(e)
			{
				if (e.currentTarget.colIndex == this.sortColumn)
				{
					this.sortDirection = (this.sortDirection == SortableFilterableTable.SORT_ASC) ? SortableFilterableTable.SORT_DSC : SortableFilterableTable.SORT_ASC;
				}
				else
				{
					this.sortDirection = SortableFilterableTable.DEFAULT_SORT_DIRECTION;
				}
				
				this.sortColumn = e.currentTarget.colIndex;
				this._sortData(e.currentTarget);	
				this._displayTable();			
			},
			
			_anchorCallback: function(e)
			{
				e.preventDefault();

				var list = (this.filterText == "") ? this.rows : this.filteredRows;
				var elements = list[(this.rangeStart - 1) + (e.currentTarget.parentNode.parentNode.rowIndex - 1)].elements;
				var element = elements[e.currentTarget.parentNode.cellIndex];
				element.callback(element.callbackObj);
			},
			
			_handleRowClick: function(e)
			{
				e.preventDefault();

				var list = (this.filterText == "") ? this.rows : this.filteredRows;
				
				if (this.parent)
				{
					var parent = this.parent;
					var viewletId = parent.getViewletStrId();
					var container = parent.container;
					var viewletPage = container.viewletPage;
					var dashboardContainer = viewletPage.dashboardContainer;
					var currentPage = dashboardContainer.getCurrentPage();
					var columns = currentPage.columns;
					
					for (var j = 0; j < columns.length; j++)
					{
						var widgets = columns[j].widgets;

						for (var i = 0; i < widgets.length; i++)
						{
							var id = widgets[i].viewletDefinition.id;

							if (id == viewletId)
							{
								var viewlet = widgets[i]._viewlet;
								
								viewlet.artifactSelected(list[(this.rangeStart - 1) + (e.currentTarget.rowIndex - 1)].callbackData);
							}
						}
					}
				}
				else
				{
					// have to modify _rangeStart by 1 because it starts at 1
					// and subtract 1 from the e.currentTarget.rowIndex because it includes the row for the THEAD
					window.location.href = list[(this.rangeStart - 1) + (e.currentTarget.rowIndex - 1)].href;
				}
			},

			_error: function(e)
			{
				alert("An error occurred: [" + e.message + "] @ SortableFilterableTable#_error");
			}
		}
	);
}
)();
