/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIOverviewSection");
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIOverviewSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.configuration.web.util.TestConfigurationUtil");
dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.OverviewSection");
dojo.require( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );

(function(){ // module container
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var TimeboxClient = com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient;
    var PlanningPageClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
    var TestConfigurationUtil = com.ibm.rqm.configuration.web.util.TestConfigurationUtil;
	var Constants  = com.ibm.rqm.planning.web.client.internal.Constants;
    var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
    
    dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIOverviewSection", [dijit._Widget, dijit._Templated, com.ibm.rqm.configuration.web.ui.internal.view.configuration.OverviewSection], {
        
        constructor: function(args){
            if (args.controller == null) {
                throw new Error("GenerateEWIOverviewSection#constructor: Missing required parameter 'args.controller'");
            }
            this.testCases = args.testCases;
			this.parentTestPlan = args.parentTestPlan;
			this.testPlans = [];
			this.testPhases = [];
        },
        
        //Overriden from OverviewSection
        _initTestArtifact: function(){
//            this._testCaseNameLabel.appendChild(document.createTextNode(""));
        },
        
        //Overriden from OverviewSection
        _initOthers: function(){
            this._doInitScheduleTestPlan();
			this._inlineHelp.innerHTML = Constants.getMessage("labelOverviewInlineHelp");
        },
        
        _doInitScheduleTestPlan: function(){
			dojo.removeClass(this._testplan, "show");
            dojo.addClass(this._testplan, "hidden");
            var self = this;
            this.connect(this._scheduleTestPlan, "onchange", function(e){
                self._doInitSchedule(e.currentTarget.value);
                dojo.publish("TESTPLAN_CHANGED", [e.currentTarget.options[e.currentTarget.selectedIndex].id, e.currentTarget.options[e.currentTarget.selectedIndex].value]);
            });

            var responseHandler = new ServiceResponseHandler(this, "_setTestPlans", "_error");
			var args = {
				uuid: this.testCases[0].versionableItemId, 
				includeRetiredPlan: false,
				includeSnapshots: false
			}
            com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient.getRelatedTestPlanByCaseMinimalSummary(responseHandler, args);
        },
		
		_setTestPlans: function(testPlans) {
			testPlans = testPlans != null && testPlans.length > 0 ? testPlans : [];
			
			var planSelect = this._scheduleTestPlan;
            dojox.data.dom.removeChildren(planSelect);
            planSelect.disabled = false;
			
            var option = new Option(Constants.getMessage("labelUnassigned"), TestConfigurationUtil.NO_ASSOCIATED_TEST_PLAN);
            option.id = TestConfigurationUtil.NO_ASSOCIATED_TEST_PLAN;
			SelectUtility.add(planSelect, option);
            
			var planLen = testPlans.length;
            for (var i = 0; i < planLen; i++) {
				var plan = testPlans[i];
                var option = new Option(plan.name, plan.versionableItemId);
                option.id = plan.id;
				//when there is a specified parent test plan, disable this selection
                if (this.parentTestPlan !=null && this.parentTestPlan.versionableItemId == plan.versionableItemId) {
                    option.selected = true;
                    planSelect.disabled = true;
                }
				SelectUtility.add(planSelect, option);
            }
			this.testPlans = testPlans;
            this._doInitSchedule(planSelect.value);			
		},
        
        _doInitSchedule: function(testPlanUUID){
            var testPlan = testPlanUUID;
            dojox.data.dom.removeChildren(this._scheduleActivity);
            if (testPlanUUID == TestConfigurationUtil.NO_ASSOCIATED_TEST_PLAN) {
                this._scheduleActivity.disabled = true;
                return;
            }
            var option = new Option(Constants.getMessage("labelLoading"));
			SelectUtility.add( this._scheduleActivity, option );
            var self = this;
            var handler = {
                success: function(testPhases){
                    dojox.data.dom.removeChildren(self._scheduleActivity);
                    self._scheduleActivity.disabled = false;
                    if (dojo.isArray(testPhases) && testPhases.length > 0) {
						self.testPhases = testPhases;
                        for (var i = 0; i < testPhases.length; i++) {
                            var option = new Option(testPhases[i].name, testPhases[i].itemId);
                            if(testPhases.length == ( i + 1)){
                                option.selected = true;
                            }
							SelectUtility.add( self._scheduleActivity, option );
                        }
                    }
                },
                error: function(e){
                    console.debug(e);
                }
            }
            var responseHandler = new ServiceResponseHandler(handler, "success", "error");
            TimeboxClient.getTestPhases(responseHandler, {
                testPlanId: testPlan
            });
        },
        
        getTestCases: function(){
            return this._testCases;
        },
        
        getTestPlan: function(){
            var testPlan = null;
            var options = this._scheduleTestPlan.options;
            for (var i = 0; i < options.length; i++) {
                if (options[i].selected) {
					var testPlans = this.testPlans;
					var length = testPlans.length;
					for (var j = 0; j < length; j++) {
						if (options[i].value == testPlans[j].versionableItemId){
							testPlan = testPlans[j];
							break;
						}
					}
                    break;
                }
            }
            return testPlan;
        },
        
        getTestPhase: function(){
            var schedule = [];
            var options = this._scheduleActivity.options;
			var len = options.length;
			var testPhases = this.testPhases;
			var len2 = this.testPhases.length;
            for (var i = 0; i < len; i++) {
                if (options[i].selected) {
					for (var j = 0; j < len2; j++){
						if(options[i].value == testPhases[j].itemId){
							schedule.push(testPhases[j]);
						}
					}
                }
            }
			if(schedule.length == 0)
				schedule = [null];
            return schedule;
        },
		
		_error: function(e) {
			console.error(this.declaredClass + ": Error: " + e.message);
		}		
    });
    
})();
