/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.QualityObjectivesGrid");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");


dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog")

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.ObjectiveClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveContext");


(function(){
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var ObjectiveClient = com.ibm.rqm.planning.web.client.internal.ObjectiveClient;
	var SaveContext = com.ibm.rqm.planning.web.ui.internal.model.SaveContext;
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.QualityObjectivesGrid", [dijit._Widget, dijit._Templated], {
        templateString: '<div id="${id}">' +		                
						'    <div dojoAttachPoint="_actionsNode" style="text-align: right;"></div>' +
						'    <div dojoAttachPoint="_contentNode" style="overflow: hidden;"></div>' +
						'    <div dojoAttachPoint="_messageNode" style="font-color: #FF0000;"></div>' +
						'</div>',
        
		labelSave: Constants.getMessage("labelSave"),

        constructor: function() {
            //valid parameters
			this.editable = false;
			this.excluded = {};
			this.isDialog = false;
			this.dialogTitle = Constants.getMessage("labelManageObjectives");
			this.dialogWidth = "800px";
			this.dialogHeight = "425px";
			this.gridHeight = "280px";
			this.gridWidth = "100%";
			this.contentHeight = "325px"; //content height should be bigger than grid height, for grid headerRow will occupy some space
			this.pageSize = 10;
			this.okCallback = null;
			this.cancelCallback = null;
			this._conditionOptions = ["=", ">", ">=", "<", "<="];
		},
		
        postCreate: function(){
            this.inherited('postCreate', arguments);

            this._contentNode.style.height = this.contentHeight;
			this._contentNode.style.width = this.gridWidth;	
			
			this._initialize();
		},
		
		_initialize: function() {
			//initialize state of the editor
			this._saving = false;
			this._dirty = false;
			this._objectives = null;
			this._removed = {}; //map (by itemId) of any objectives removed
			this._messages = [];
						
            //fetch objectives
            var responseHandler = new ServiceResponseHandler(this, "_setObjectives", "_error");
            ObjectiveClient.getObjectives(responseHandler, /*showArchived*/ false);				
		},
		
		_setObjectives: function(objectives) {
            this._objectives = objectives || [];			
			if (this.isDialog) {
				this._showAsDialog();
			}
			this._initializeGrid();
			if (this.isDialog) {
				this._dialog.updatePosition();
			}			
		},

        _initializeGrid: function(){
            var dataAsGridModel = this._toGridModel(this._objectives);
			
            var model = new dojox.grid.data.Table([{}, {}, {}, {}], dataAsGridModel);
            
            // Layout
            var nameCell = {
                name: Constants.getMessage("labelName"),
                width: '200px'
            };
            var descCell = {
                name: Constants.getMessage("labelDescription"),
                width: '300px'
            };
            var conditionCell = {
                name: Constants.getMessage("labelCondition"),
                width: '75px',
				styles: 'text-align: center;'
            };
            var targetCell = {
                name: Constants.getMessage("labelTarget"),
                width: '75px',
				styles: 'text-align: center;'
            };
            if (this.editable) {
				nameCell.editor = descCell.editor = targetCell.editor = dojox.grid.editors.Input;
                conditionCell.editor = dojox.grid.editors.Select;
				conditionCell.options = this._conditionOptions;
            }
            var gridLayout = [{
                defaultCell: {
                    styles: 'text-align: left;'
                },
                rows: [[nameCell, descCell, conditionCell, targetCell]]
            }];
            
            if (!this._grid) {
                this._grid = new com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer({
                    rowsPerPage: this.pageSize,
                    model: model,
                    structure: gridLayout,
                    rowCount: this.pageSize,
                    elasticView: "2",
                    defaultStaticRow: ["", "", "=", "", null, null],
                    onchange: dojo.hitch(this, "_onGridChange"),
					onRemoveRows: dojo.hitch(this, "_onGridRemove"),
                    singleClickEdit: false,
					maxHeight: this.gridHeight,
					actionsNode: this._actionsNode
                });
				if (!this.editable) {
					dojo.connect(this._grid, "onRowClick", this, "_onGridClick");
					dojo.connect(this._grid, "_selectByKeys", this, "_onGridClick");
					dojo.connect(this._grid, "onRowDblClick", this, "_onGridDblClick");
				}

				dojox.data.dom.removeChildren(this._contentNode);
				this._contentNode.appendChild(this._grid.domNode);
				
				//render the Grid
                this._grid.render();
                if (this.editable) {
                    this._grid.initialize();
                }
            } else {
				//refresh the Grid with the current data
				this._grid.setModel(model);
				this._grid.refresh();
			}
        },	
		
		isDirty: function() {
			return this._dirty;
		},
		
		_onGridChange: function() {
			//placeholder for edit validation
			this._dirty = true;
			if (this._dialog) {
				this._dialog.getOkButton().disabled = false;
			}
		},	
		
		_onGridRemove: function(selectedRows) {
			var removals = this._fromGridModelRows(selectedRows) || [];
			var remLen = removals.length;
			for (var i = 0; i < remLen; i++) {
				var removed = removals[i];
				if (removed.itemId != null) {
					this._removed[removed.itemId] = removed;
				}
			}
		},
		
        _onGridClick: function(e) {
            var selected = this._grid.getSelectedRows();
            if (this._dialog) {
                this._dialog.getOkButton().disabled = !(selected != null && selected.length > 0);
            }         
        },
		
        _onGridDblClick: function(e) {
            var selected = this._grid.getSelectedRows();
            if (this._dialog && !this.editable && selected != null && selected.length > 0) {
                this._dialog.getOkButton().onclick();
            }         
        },			
		
        _showAsDialog: function() {
            if (!this._dialog) {            
                var dialogArgs = {
                    primaryTitle: this.dialogTitle,
					okCallback: dojo.hitch(this, "_onOK"),
                    cancelCallback: dojo.hitch(this, "_onCancel"),
                    _truncate: function() {}  //disable the Dialog's '_truncate'
                };
                if (this.editable) {
					dialogArgs.okLabel = this.labelSave;
				}
				
                this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
                //add cshelp to dialog
                var cshelp = new CSHelp({contextID: "rqm_manage_quality_objectives.html"});
                this._dialog.setSecondaryTitleNode(cshelp.domNode); 
                cshelp.setButtonLabel(this.dialogTitle);
                this._dialog.setWidth(this.dialogWidth);
				this._dialog.setHeight(this.dialogHeight);
                this._dialog.contentContainer.appendChild(this.domNode);
                //this._dialog.setSecondaryTitle(this.getDialogTitle());
                this._dialog.getOkButton().disabled = true;
            }
            this._dialog.open();            
        }, 
		
		_onOK: function() {
			if (!this.editable) {
				//process as selection
				var selected = this._fromGridModel(true);
				this._doCallback(this.okCallback, selected);
				return true;
			}
			//process as a save; get data from grid as modelled
            var asModelled = this._fromGridModel();
			if (this._removed && this._removed.length > 0) {
				asModelled = asModelled.concat(this._removed);
			}
            if (asModelled.length > 0) {
				return this._doSave(asModelled);
            }			
			return false;
		},
		
		_onCancel: function() {
            this._doCallback(this.cancelCallback);		  	
		},
		
		_doSave: function(objectives) {
            //set save in progress
            if (this._isSaving()) {
                return false; //protects against multiple save button clicks
            } else {
				this.clearMessages();
                this._setSaving(true);
            }
			
			//create save context and register callback
			var saveContext = new SaveContext();
			saveContext.setOnSaveFinished(dojo.hitch(this, "_saveFinished"));
			for (var i = 0; i < (objectives || []).length; i++) {
				var objective = objectives[i];
				if (objective.itemId != null) {
					var original = this._findIn(this._objectives, objective.itemId, "itemId");
					if (!this._equals(objective, original)) {
						//only save the objective if it's been updated
						saveContext.addParticipant(this._toSaveParticipant(objective, i));						
					}
				} else {
					saveContext.addParticipant(this._toSaveParticipant(objective, i));
				}
			}
			
			//add save participants for any removed objectives
			for (var itemId in this._removed) {
				var objective = this._removed[itemId];
				objective.archive = true;
				saveContext.addParticipant(this._toSaveParticipant(objective, -1));
			}
            
            //commit the save
            var status = saveContext.commit();
            if (status != Constants.STATUS_SAVING) {
                this._setSaving(false);
                if (this._dialog) this._dialog.getOkButton().disabled = false;
				if (status == Constants.STATUS_FAILURE) {
					this._showMessages();
				}
            }
			//keep the dialog open until we receive a response
			return false;
		},
				
        _saveFinished: function(status) {
			this._setSaving(false);
            if (status && status.ok == true) {
				this._doCallback(this.okCallback, status);
				if (this._dialog) {
					this._dialog.close();
				} else {
					this._initialize();
				}
			}
        }, 		
		
		_doCallback: function(/* Function */ callback, callbackData) {
            if (callback != null && dojo.isFunction(callback)) {
                callbackData != null ? callback(callbackData) : callback();
            }			
		},
        
		_fromGridModel: function(selectedOnly) {
			var asModelled = [];
			if (this._grid) {
				//each row is an array of values
				var rows = selectedOnly === true ? this._grid.getSelectedRows() : this._grid.getAllRows();
				asModelled = this._fromGridModelRows(rows);
			}
			return asModelled;
		},
		
		_fromGridModelRows: function(rows) {
			rows = rows || [];
			var rowLen = rows.length;
			var asModelled = [];
			for (var i = 0; i < rowLen; i++) {
				var row = rows[i];
				asModelled.push({
					//id:   row[1] || "",
					name: row[0] || "",
					description: row[1] || "",
					operator: row[2] || "",
					targetValue: row[3] != null ? row[3] : "",
					itemId: row[4] || null,
					stateId: row[5] || null,
					evaluatorId: row[6] || null
				});
			}		
			return asModelled;	
		},
		
		_toGridModel: function(objectives) {
            objectives = objectives || [];
			var asGridModel = [];			
			var objLen = objectives.length;
            for (var i = 0; i < objLen; i++) {
                var o = objectives[i];
				if (!this.excluded[o.itemId]) {
					asGridModel.push([o.name, o.description, o.operator, o.targetValue, o.itemId, o.stateId, o.evaluatorId]);
				}
            }
			return asGridModel;		
		},
		
		_toSaveParticipant: function(objective, rowNum) {
            var p = new com.ibm.rqm.planning.web.ui.internal.view.construction.ObjectiveSaveParticipant({
                parent: this,
				objective: objective,
				rowNum: rowNum
            });
			return p;
		},
		
	    _findIn: function(array, value, byProperty) {
	        for (var i = 0; i < (dojo.isArray(array) ? array.length : 0); i++) {
				if (dojo.isObject(array[i]) && byProperty != null && array[i][byProperty] && array[i][byProperty] == value) {
					return array[i]; 		 	
				} else if (array[i] == value) {
					return array[i];
				}			
			}
			return null;
	    },
		
		_equals: function(updated, original) {
			if (updated.name != original.name) return false;
			if (updated.description != original.description) return false;
			if (updated.operator != original.operator) return false;
			if (updated.targetValue != original.targetValue) return false;
			return true;
		},	
        
        _isSaving: function() {
            return this._saving;
        },

        _setSaving: function(bool) {
            this._saving = bool;
        },
		
		_showMessages: function() {
            if (this._messages.length > 0) {
				var messages = "";
				for (var i = 0; i < this._messages.length; i++) {
					messages += this._messages[i] + "<br />\n";
				}
				var args = {
	                message: messages
	            };		
	            var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(args);
	            alertDialog.setSecondaryTitle("");
	            alertDialog.open();
			}
		},
		
		addMessage: function(message) {	
			this._messages.push(message);
			//this._messageNode.appendChild(document.createTextNode(message));
			//this._messageNode.appendChild(document.createElement("br"));
		},
		
		clearMessages: function() {
			this._messages = [];
			//dojox.data.dom.removeChildren(this._messageNode);
		},
		
        _error: function(e){
            console.error(this.declaredClass + ": An error occurred: \n[" + e.message + "]");
        }
    });
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.ObjectiveSaveParticipant", 
	[com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant],
    {
        constructor: function(args) {
            if (!args.parent) {
                throw new Error("Parameter 'parent' is REQUIRED!");
            } else if (!args.objective) {
                throw new Error("Parameter 'objective' is REQUIRED!");
            } else if (args.rowNum == null) {
				if (!args.objective.archive) {
					throw new Error("Parameter 'rowNum' is REQUIRED!");
				}
			}
			this._parent = args.parent;
			this._objective = args.objective;
			this._rowNum = args.rowNum + 1; //adjust by 1 for display purposes
			this._itemId = this._isEmpty(this._objective.itemId) ? null : this._objective.itemId;
			this._stateId = this._isEmpty(this._objective.stateId) ? null : this._objective.stateId;
        },
        
        getItemId: function() {
            return this._itemId;
        },
		
        getStateId: function() {
            return this._stateId;
        },		
        
        getItemType: function(){
            return Constants.MODEL_TYPE_OBJECTIVE;
        },
        
        isNew: function(){
            return (this._itemId == null);
        },
        
        // @override for SaveParticipant
        validate: function(){		
            var o = this._objective;
			if (this._objective.archive == true) {
				return true;
			}
			//name
            if (this._isEmpty(o.name)) {
				this._parent.addMessage(Constants.getMessage("msgRequiredGridField", Constants.getMessage("labelName"), this._rowNum));
				return false; 
            } else if (o.name.length > Constants.DB_STRING_SIZE_SMALL) {
				var details = Constants.getMessage("msgFieldToLong", Constants.getMessage("labelName"), o.name.length, Constants.DB_STRING_SIZE_SMALL)
				this._parent.addMessage(Constants.getMessage("msgInvalidGridField", Constants.getMessage("labelName"), details, this._rowNum));
				return false;
			}
			//description
			if (!this._isEmpty(o.description) && o.description.length > Constants.DB_STRING_SIZE_MEDIUM) {
				var details = Constants.getMessage("msgFieldToLong", Constants.getMessage("labelDescription"), o.description.length, Constants.DB_STRING_SIZE_MEDIUM)
				this._parent.addMessage(Constants.getMessage("msgInvalidGridField", Constants.getMessage("labelDescription"), details, this._rowNum));
				return false;
			}
			//operator						
            if (this._isEmpty(o.operator)) {
                this._parent.addMessage(Constants.getMessage("msgRequiredGridField", Constants.getMessage("labelCondition"), this._rowNum));
				return false; 
            }
			//target value
			var target = new String(o.targetValue);
            if (dojo.trim(target) === "") {
                this._parent.addMessage(Constants.getMessage("msgRequiredGridField", Constants.getMessage("labelTarget"), this._rowNum));
				return false;
			} else if (this._isNonNumeric(target)) {
				this._parent.addMessage(
				    Constants.getMessage("msgInvalidGridField", Constants.getMessage("labelTarget"), Constants.getMessage("msgValidNumber"), this._rowNum)
				);
                return false;
            }
			return true;               
        },
        
        _isEmpty: function(value) {
            return (value == null || dojo.trim(value).length == 0);
        },  
		
		_isNonNumeric: function(value) {
			var asNum = new Number(value);
			return isNaN(asNum);
		},
		        
        // @override for SaveParticipant
        getAttributes: function(){
            var attributes = {
                isNewItem: this.isNew(),
                name: this._objective.name,
                description: this._objective.description,                               
                operator: this._objective.operator,
				targetValue: this._objective.targetValue,
				archive: this._objective.archive != null ? this._objective.archive : false
            };
            //generate an id for new objectives for now; ids are only used for computed objectives
            if (this.isNew() && this._isEmpty(this._objective.id)) {
				var uid = (new Date()).getTime() + "" + Math.ceil(Math.random() * 99999);
                attributes.id = "com.ibm.rqm.planning.objectives._" + uid; //$NON-NLS$
            }			
            return attributes;
        },
        
        // @override for SaveParticipant
        saveFinished: function(savedItem){
            this._objective = savedItem;
            this.setDirty(false);
        },
        
        // @override for SaveParticipant
        saveFailed: function(status){
            if (status && status.ok != true && status.message) {
				this._parent.addMessage(status.message);
			}
        }        
    
    });   	
	
})();
