/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotPane");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotPane");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.SnapshotTableView");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

(function() 
{
	var ImageCache 				= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
	var SnapshotTableView      = com.ibm.rqm.planning.web.ui.internal.view.table.SnapshotTableView;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var Utility					   = com.ibm.rqm.planning.web.ui.internal.util.Utility;
	var Constants				   = com.ibm.rqm.planning.web.client.internal.Constants;
	
	/**
	 * 
	 */
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotPane",
				 [dijit._Widget, dijit._Templated],
		{
			templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/construction/SnapshotPane.html"),
			
			labelLoading: Constants.getMessage("labelLoading"),

			constructor: function() 
			{
			},

			postCreate: function() 
			{
                this.showLoadingMessage(false);
			},

			initialize: function(args)
			{
				this._editor = args.editor;
			},

			_fillSnapshotsList: function(items){
				if (items == null) {
					//this.showLoadingMessage(false);				
					//return;
				}
                //add tooltips to each item
                for (var i = 0; i < (items ? items.length : 0); i++) {
                    items[i].tooltipHTML = this._getTooltipHtml(items[i]); 
                }
				if (this._tableViewer == null) {
					var args = {
                        snapshots: items,
						itemId: this._editor.getItemId(), 
						itemType: this._editor.getItemType(),
                        isEditable: this._editor.isEditable()						
					}					
					this._tableViewer = new SnapshotTableView(args);
				} else {
					//set itemId in case of new item & itemId was not avail when table created
					this._tableViewer.setItemId(this._editor.getItemId());
					this._tableViewer.refresh(items);					
				}

				// is there a better way to refresh the view?
				dojox.data.dom.removeChildren(this._snapshotsList);
				//this._list.displayTable();
				//this._snapshotsList.appendChild(this._list.domNode);
				this._snapshotsList.appendChild(this._tableViewer.domNode);
				//this.showLoadingMessage(false);
				
			},

		    refreshSnapshots: function(){
				//this.showLoadingMessage(true);
		        //ajax client call back method, fill snapshots table
		        var self = this;
				var responseHandler = new ServiceResponseHandler(this, "_fillSnapshotsList", "_error");
				// getSnapshots functions should be implemented by any SnapshotEnabledEditor
		        this._editor.getSnapshots(responseHandler);
		    },
			
			showLoadingMessage: function(loading) {
				if (loading) {
					this._loadingMessage.style.display="inline";
				} else {
					this._loadingMessage.style.display="none";
				}
			},
			
			_getTooltipHtml: function(snapshotInfo) {
				var tooltipHtml;
                var created = new Date();
                created.setTime(Utility.dateFromISOString(snapshotInfo.created));
				tooltipHtml = "<table style=\"font-size: smaller; font-family:Verdana;\" border=0>";
				tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labelname") + "</b></td><td>" + snapshotInfo.name + "</td></tr>";
				tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labelrevision") + "</b></td><td>" + snapshotInfo.revision + "</td></tr>";
				tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labeldescription") + "</b></td><td>" + snapshotInfo.description + "</td></tr>";
				tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labelcreated") + "</b></td><td>" + Utility.formatDate(created) + "</td></tr>";
				tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labelitemID") + "</b></td><td>" + snapshotInfo.itemId + "</td></tr>";
				if (snapshotInfo.snapshotOrigin!=null) {
					tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labelorigin") + "</b></td><td>"
					tooltipHtml += snapshotInfo.snapshotOrigin.itemType + " : " + snapshotInfo.snapshotOrigin.name + "</td></tr>";
				}
				tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labeloriginatorname") + "</b></td><td>" + snapshotInfo.originator.name + "</td></tr>";
				tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labeloriginatoremail") + "</b></td><td><a href='mailto:" + snapshotInfo.originator.emailAddress + "'>";
				tooltipHtml += snapshotInfo.originator.emailAddress + "</td></tr>";
				tooltipHtml += "<tr><td align=right><b>" + Constants.getMessage("labeloriginatorID") + "</b></td><td>" + snapshotInfo.originator.userId + "</td></tr>";
				tooltipHtml += "</table>";
				
				return tooltipHtml;
			},
		    
			_error: function(e)
			{
				alert("An error occurred: [" + e.message + "] @ SnapshotPane#_error");
			}
		}
	);
}
)();
