/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.ManualScriptEditorSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.ExecutionScriptElementList");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.DatapoolInTestScriptClient");
dojo.require("jazz.util.html");
(function(){ // module container
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var DatapoolClient = com.ibm.rqm.planning.web.client.internal.construction.DatapoolInTestScriptClient;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    
    var COMMAND_CREATE = "create"; //$NON-NLS$
    var COMMAND_UPDATE = "update"; //$NON-NLS$
    var COMMAND_REMOVE = "remove"; //$NON-NLS$
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.ManualScriptEditorSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: "<div dojoAttachPoint=\"_manualStepsSection\" class=\"com-ibm-rqm-planning-web-ui-internal-view-construction-extension-ManualScriptEditorSection\"></div>",
        
        constructor: function(){
        },
        
        postCreate: function(){
        },
        
        initialize: function(){
            var script = this.getParentEditor().getArtifact();
            this._steps = script && script.elements ? dojo.clone(script.elements) : [];
			var stepLen = this._steps.length;
			var steps = this._steps ;
			for (var i = 0; i < stepLen; i++) {//Workitem 27332
                var step = steps[i];
                if(step.content)
                    step.content = jazz.util.html.removeScript(step.content);
                if(step.expectedResult)                    
                    step.expectedResult = jazz.util.html.removeScript(step.expectedResult);
            }
         	this.unloadContent(this._manualStepsSection);
            this._stepsEditor = new com.ibm.rqm.planning.web.ui.internal.view.construction.ExecutionScriptElementList({
                save: dojo.hitch(this, function(){
                    var editor = this.getParentEditor();
                    if (dojo.isFunction(editor.onSave)) {
                        editor.onSave();
                    }
                })
            });
            this._manualStepsSection.appendChild(this._stepsEditor.domNode);
            this._stepsEditor.initialize({
                parentController: this.parentController,
                steps: this._steps,
                parentCallBack: this.parentEditor
            });
        },
        
        //@Abstract: Override to contribute attributes to the parent's save
        getAttributes: function(){
            //collect commands
            var commands = [];
            
            //get current steps from step editor
            var script = this.getParentEditor().getArtifact();
            var isCopy = this.getParentEditor().isCopy();
            var oldElements = script && script.elements ? script.elements : [];
            var newElements = this.currentElements;
            
            //for each element, create the appropriate command to update script
            var numNew = newElements.length;
            for (var i = 0; i < numNew; i++) {
                var element = newElements[i];
                if (!isCopy && element.itemId != null) {
                    //existing element; 'update' command
					var existing = this._findIn(oldElements, element);
                    var command = {
                        cmd: COMMAND_UPDATE,
                        elementObj: this._toCommandObj(element, existing)
                    }
                    commands.push(command);
                    
                } else {
                    //new element; 'create' command
                    var command = {
                        cmd: COMMAND_CREATE,
                        elementObj: this._toCommandObj(element)
                    }
                    commands.push(command);
                }
            }
            
            //handle any deletes
			if (!isCopy) {
				var numOld = oldElements.length;
				for (var i = 0; i < numOld; i++) {
					//if there's no entry for an existing element in the list of current 
					//elements, the existing element has been removed
					var original = oldElements[i];
					var updatedElement = this._findIn(newElements, original);
					if (original.itemId && updatedElement == null) {
						var command = {
							cmd: COMMAND_REMOVE,
							elementObj: {
								num: (i + 1),
								itemId: original.itemId
							}
						}
						commands.push(command);
					}
				}
			}
            
            return {
                elements: commands//this.currentElements
            };
        },
        
        _toCommandObj: function(element, existing){
            var commandObj = {
                num: element.num,
                typeName: element.elementType.name     
            }
            if (element.itemId) {
                commandObj.itemId = element.itemId;
            }
			if (element.summary != null) {
				if (!existing || existing.summary != element.summary) {
					commandObj.summary = element.summary;
				}
			}
            if (element.content != null) {
                if (!existing || existing.content != element.content) {
                    commandObj.content = element.content;
                }
            }	
            if (element.expectedResult != null) {
                if (!existing || existing.expectedResult != element.expectedResult) {
                    commandObj.expectedResult = element.expectedResult;
                }
            }					
            if (element.properties) {
                commandObj.properties = element.properties;
            }
			return commandObj;
        },
        
        validate: function(){
            this.currentElements = [];
            this._stepsEditor.setGlobalSaveInProgress();
            if (!this._stepsEditor.validateElements()) {
                return false;
            }
            this.currentElements = this._stepsEditor.getElements();
            return true;
        },
        
        isEmpty: function(){
            return this.currentElements ? this.currentElements.length == 0 : false;
        },
        
        //@Overriden from AbstractArtifactEditorSection
        getActions: function(){
            return [];
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
//            dojo.style(this._stepsEditor._stepsContainer, "height", "auto");
			this._steps = savedItem && savedItem.elements ? dojo.clone(savedItem.elements) : [];
            this._stepsEditor.updateSteps(this._steps);
            this._dealWithDatapool(savedItem.elements);
        },
        
        _dealWithDatapool: function(elements){
            if (elements && elements.length) {
                var stepPair = new Array();
				var len = elements.length;
                for (var i = 0; i < len; i++) {
                    var varArray = this._parseVariableId(elements[i].content);
                    var varArray2 = this._parseVariableId(elements[i].expectedResult);
                    if (varArray2 && varArray2.length) {
                        if (!varArray) 
                            varArray = varArray2;
                        else 
                            varArray = varArray.concat(varArray2);
                    }
                    if (varArray && varArray.length > 0) {
                        stepPair.push({
                            stepItemId: elements[i].itemId,
                            variables: varArray
                        });
                    }
                }
                if (stepPair.length > 0) {
                    var param = dojo.toJson(stepPair);
                    var callbackObj = {
                        success: function(results){
                        },
                        failure: function(e){
                            console.error("Datapool createLink failed");
                        }
                    }
                    var responseHandler = new ServiceResponseHandler(callbackObj, "success", "failure");
                    DatapoolClient.postDatapoolVariableLink2(responseHandler, {
                        jsonString: param
                    });
                }
            }
        },
        
        _findIn: function(elements, element){
            var len = (elements || []).length;
            for (var i = 0; i < len; i++) {
                if (elements[i].itemId != null && elements[i].itemId == element.itemId) {
                    return elements[i];
                }
            }
            return null;
        },
        
        _parseVariableId: function(content){
            var newContent = unescape(content);
            var contentDiv = document.createElement('div');
            contentDiv.innerHTML = newContent;
            var children = contentDiv.getElementsByTagName("BUTTON");//desDiv.childNodes;
            var newArray = [];
			var len = children ? children.length : 0;
            for (var i = 0; i < len; i++) {
                newArray.push(children[i]);
            }
            
            var idArray = null;
            if (newArray && newArray.length) {
                idArray = new Array();
				var len = newArray.length;
                for (var i = 0; i < len; i++) {
                    if (newArray[i].name == "variable") {
                        var itemId = newArray[i].id;
                        idArray.push(itemId);
                    }
                }
            }
            return idArray;
        },
        
        onArtifactChanged: function(changeType, artifactType, artifactArray){
            if (this._stepsEditor && this._stepsEditor._kwView) 
                this._stepsEditor._kwView.onArtifactChanged(changeType, artifactType, artifactArray);
        },
        
        _onDatapoolChange: function(){
            this._stepsEditor._removeAllVariables();
        },
        
        destroy: function(){
            if (this._stepsEditor) 
                this._stepsEditor.destroy();
            this.inherited(arguments);
        },
		onActivated:function(){
			if(this._stepsEditor && this._stepsEditor.onActivated)
				this._stepsEditor.onActivated();
		}
    });
})();
