/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestCaseSummarySection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestCaseSummarySection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");
dojo.require( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
dojo.require("com.ibm.rqm.planning.web.client.internal.RegistryClient");
dojo.require("com.ibm.asq.common.web.client.internal.option");

(function(){
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var FormNumericField = com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField;
	var wrapText  = com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText;
	var RegistryClient = com.ibm.rqm.planning.web.client.internal.RegistryClient;
    var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
	
    /**
     * Widget to display list of link elements in the LeftNavigation.
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestCaseSummarySection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/construction/extension/TestCaseSummarySection.html"),
        
		labelWeight: Constants.getMessage("labelWeight"),
		labelPoints: Constants.getMessage("labelPoints"),
		
        constructor: function(){
        },
        
        postCreate: function(args){
            this.inherited('postCreate', arguments);
			this.addWorkItemSupport(EditorUtil.getCurrentContributor().itemId);
			this.UNASSIGNED = Constants.getMessage("labelUnassigned");
			this.SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
			
			//create form widgets (but do not append to the dom until initialize)
			var testCase = this.getArtifact();
			var weightWidget = new FormNumericField({
				fieldLabel: this.labelWeight + ": *", 
                labelWidthPercent: (dojo.isIE) ? 33 : 40,
				required: true,
				disabled: !this.isEditable()
			});	
			this.weightWidget = weightWidget;		
			weightWidget.setValue(testCase != null ? testCase.weight : Constants.TEST_CASE_DEFAULT_WEIGHT);
			weightWidget.onValueChanged = dojo.hitch(this, "_onChange");
			weightWidget.setErrorMessage(Constants.getMessage("msgNumericWeightRequired"));
			weightWidget.setPromptMessage(Constants.getMessage("msgWeightRequired"));			
			
			this._categorySelect = new com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection({
				parent: this,
				artifactType: Constants.MODEL_TYPE_CASE,
				artifact: testCase,
				selectDefaults: this.getParentEditor().isNew(), //select default categories for NEW cases only
                
                // Adjust the non-category fields' label width to match that of the 
                // category labels. This ensures that all of the fields displayed in
                // the section would align correctly. Add a callback to do this after
                // all of the categories have been loaded.
                //
                afterInitialize: dojo.hitch(this, function() {
					if (this._categorySelect) {
						var labelWidth = this._categorySelect.getLabelWidth() - 2;
						
						// no categories, align our internal fields to a fixed width. 
						if (labelWidth <= 0) {
							labelWidth = 95;
						}
						// find all field label nodes and apply the width to them.
						var labelTDs = dojo.query(".tc-category-section-label-td", this._categoriesDisplayNode);
						for (var i = 0; i < labelTDs.length; i++) {
							labelTDs[i].width = labelWidth;
						}
						// the weight field is specially handled.
						if (this.weightWidget && this.weightWidget.domNode) {
							dojo.style(this.weightWidget.domNode, "width", "60%");
							this.weightWidget.setFormFieldWidths(labelWidth + 2, 120);
						}
					}
                })			
			});						           
        },
		
        uninitialize: function(){
            // orphan widgets would be leaked if they have not attached
            if(!this.initialized) {
                if (this.weightWidget && this.weightWidget.destroy) {
					this.weightWidget.destroy();
					this.weightWidget = null;
				}
                if (this._categorySelect && this._categorySelect.destroy) {
					this._categorySelect.destroy();
					this._categorySelect = null;
				}
            }
            this._categoriesEditor = null;
        },
        
        //@Overriden from AbstractArtifactEditorSection
        initialize: function(isEditable){
            var testCase = this.getArtifact();
			
			dojox.data.dom.removeChildren(this._weight);
			this._weight.appendChild(this.weightWidget.domNode);
			
            //category labels
            this._categoriesTitle.innerHTML = Constants.getMessage("labelCategories");          
            var args = {
                label: Constants.getMessage("labelManageCategories", Constants.TYPE_CASE),
                onClick: dojo.hitch(this, "_handleCategoriesClick"),
                iconClass: "categories-icon-image",
                showLabel: false
            };
            this._manageCategoriesActionButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(args);           
            this._manageCategoriesActionNode.appendChild(this._manageCategoriesActionButton.domNode);
            this._manageCategoriesActionButton.setDisabled(!this.isEditable()); 			
			
			//setup category selection
			dojox.data.dom.removeChildren(this._categoriesNode);
			this._categoriesNode.appendChild(this._categorySelect.domNode);
			this._categorySelect.setDisabled(!this.isEditable());
			
			var self = this;
			var handlerObj = {
				success: function(response){
					var prefs = response?dojo.fromJson(response.value):null;
					if (prefs && prefs.useTriggerActivity) {
						self._activityTitle.innerHTML = Constants.getMessage("labelActivity");
						self._initializeActivity();
						self._triggerTitle.innerHTML = Constants.getMessage("labelTrigger");
						self._initializeTriggers();
						if (self.isEditable()) {
							self.connect(self._activitySelect, "onchange", "_onChange");
							self.connect(self._triggerSelect, "onchange", "_onChange");
						}
					}else{
						self.activityRow.style.display = "none";
						self.triggerRow.style.display = "none";
					}
				},
				failure: function(e){
					self.activityRow.style.display = "none";
					self.triggerRow.style.display = "none";
					self._error(e);
				}
			}
			var option = new com.ibm.asq.common.web.client.internal.option();
			var handler = new ServiceResponseHandler(handlerObj, "success", "failure");
			option.getProjectPreferences(handler, getCurrentProjectArea().itemId, Constants.TESTCASE_SYSTEM_PREFERENCE);
			
            this.initialized = true;            
        },
        
        //@Overriden from AbstractArtifactEditorSection
        getActions: function(){
            return null;
        },
        
        //@Overridden from AbstractArtifactEditorSection --> disable 'remove' capability for Summary section          
        getHeaderContent: /* String || DOM Node */ function(isEditable) {
            return null;
        },        
		
		validate: function() {
            //weight
            if (dojo.trim(this.weightWidget.getValue()).length == 0) {
                var detailError = Constants.getMessage("msgRequiredField", Constants.getMessage("labelWeight"), Constants.getMessage("labelTitleSection", this.getDisplayTitle()));
				this.getParentEditor().setHeaderStatus(detailError, Constants.HEADER_STATUS_ERROR, detailError);
                return false;
            } else {
                var weight = Number(dojo.trim(this.weightWidget.getValue()));
                if (weight != NaN && weight >= 0) {
					//Validate if the weight is less than permissible.
					if(weight > parseInt(Constants.MAX_INT_VALUE_IN_HEX, 10)){//Constants.NUMBER_MAX_INT_VALUE){
					    var detailError = Constants.getMessage("msgInvalidWeightField", parseInt(Constants.MAX_INT_VALUE_IN_HEX, 10), this.getDisplayTitle());
						this.getParentEditor().setHeaderStatus(detailError, Constants.HEADER_STATUS_ERROR, detailError);
	                return false;
					}if(weight == 0){
					    var detailError = Constants.getMessage("labelWeightValueWarning"); 
						this.getParentEditor().setHeaderStatus(detailError, Constants.HEADER_STATUS_ERROR, detailError);
	                return false;
					}else if(!this.weightWidget.textField.isValid()){
                        var detailError = Constants.getMessage("msgInvalidField", Constants.getMessage("labelWeight"), Constants.getMessage("labelOnlyNonnegativeInteger"), this.getDisplayTitle());
						this.getParentEditor().setHeaderStatus(detailError, Constants.HEADER_STATUS_ERROR, detailError);
	                	return false;
					}
                } else {
                    var detailError = Constants.getMessage("msgInvalidField", Constants.getMessage("labelWeight"), Constants.getMessage("labelOnlyNonnegativeInteger"), this.getDisplayTitle());
					this.getParentEditor().setHeaderStatus(detailError, Constants.HEADER_STATUS_ERROR, detailError);
	                return false;
                }                
            }	
			//categories
			if (this._categorySelect && !this._categorySelect.validate()) {
				return false;
			}
			
			return true;
		},
		
        //@Overriden from AbstractArtifactEditorSection
        getAttributes: function(){
            var attributes = {};
            //weight
			var weight = Number(dojo.trim(this.weightWidget.getValue()));
			if (weight != NaN && weight >= 0) {
			    attributes.weight = this.weightWidget.getValue();	
			} else {
				attributes.weight = Constants.TEST_CASE_DEFAULT_WEIGHT;
			}                

			if (this._categorySelect) {
                //TODO send up an array instead of this comma delimited string
				attributes.categories = this._categorySelect.getSelectedItemIds().join();
			}
			if(this._triggerSelect.value != ""){
				attributes.triggerId = this._triggerSelect.value;
			}else{
				attributes.triggerId = "";
			}
			if(this._activitySelect.value != ""){
				attributes.activityId = this._activitySelect.value;
			}else
				attributes.activityId = "";
            return attributes;
        },
   
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
    //        this.initialize();
            this.addWorkItemSupport();
        },
		
		        
        _handleCategoriesClick: function() {
            this._categoriesEditor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor({
                artifactType: Constants.MODEL_TYPE_CASE,
                onClose: dojo.hitch(this, "_updateCategories")
            });
        },      
        
        _updateCategories: function() {
            //refresh the category selection widget; refreshes the list
            if (this._categorySelect) {
                this._categorySelect.refresh();
            }
        },
		_initializeActivity:function(){
			var self = this;
			var handlerObj = {
				success: function(allActivities) {
					var testCase = self.getArtifact();
					allActivities = allActivities?allActivities:[];
					if (allActivities.length > 0) {
						allActivities = allActivities.sort(function(c1, c2){
							var cSort1 = c1 && c1.name ? (c1.name).toLowerCase() : "";
							var cSort2 = c2 && c2.name ? (c2.name).toLowerCase() : "";
							if (cSort1 == cSort2) {
								return 0;
							}
							else 
								return cSort1 < cSort2 ? -1 : 1;//Ascending order
						});
					}
					self.SelectUtility.add(self._activitySelect, new Option(self.UNASSIGNED, ""));
		            for(var i=0;i<allActivities.length;i++){
						var opt = new Option(Constants.getMessage(allActivities[i].id),allActivities[i].id);
						if(testCase && testCase.activity && testCase.activity.id == allActivities[i].id)
							opt.selected = true
						self.SelectUtility.add(self._activitySelect,opt );
					}
					self._activitySelect.disabled = !self.isEditable();
		            					
				},
				failure: function(e) {
					self._error(e);
				}
			}
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            RegistryClient.getAllTestActivityTypes(responseHandler, {}); 
		},
		_initializeTriggers:function(){
			var self = this;
			var handlerObj = {
				success: function(allTriggers) {
					var testCase = self.getArtifact();
					allTriggers = allTriggers?allTriggers:[];
					if (allTriggers.length > 0) {
						allTriggers = allTriggers.sort(function(c1, c2){
							var cSort1 = c1 && c1.name ? (c1.name).toLowerCase() : "";
							var cSort2 = c2 && c2.name ? (c2.name).toLowerCase() : "";
							if (cSort1 == cSort2) {
								return 0;
							}
							else 
								return cSort1 < cSort2 ? -1 : 1;//Ascending order
						});
					}
					self.SelectUtility.add(self._triggerSelect, new Option(self.UNASSIGNED, ""));
		            for(var i=0;i<allTriggers.length;i++){
						var opt = new Option(Constants.getMessage(allTriggers[i].id),allTriggers[i].id);
						if(testCase && testCase.trigger && testCase.trigger.id == allTriggers[i].id)
							opt.selected = true
						self.SelectUtility.add(self._triggerSelect,opt );
						self._triggerSelect.disabled = !self.isEditable();
					}
				},
				failure: function(e) {
					self._error(e);
				}
			}		
			
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            RegistryClient.getAllTriggerTypes(responseHandler, {}); 
		},		
		_onChange: function(){
			this.setDirty();
		},
        
        _error: function(e){
            throw new Error("An error occurred: [" + e.message + "] @ TestCaseSummarySection#_error");
        }
    });
})();


