/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    Simple dialog that Allows to Add / Edit names / Remove the columns for Dynamic grids that are used
//    by test plans and test cases. empty names are not allowed and at least one column should remain
//    in the Dynamic grid.
// 
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicGridEditColumnsDialog");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicGridEditColumnsDialog", 
             com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog,
{

    columnSettingsSection: 
              '    <tr>' +
              '      <th nowrap dojoAttachPoint="columnHead" class="settings-table-header"></th>' +
              '      <th nowrap dojoAttachPoint="columnNameHead" class="settings-table-header"></th>' +
              '      <th nowrap dojoAttachPoint="actionsHead" class="settings-table-header" style="align:center"></th>' +
              '    </tr>',

    columnSettingsTemplate:
              '  <table class="settings-table" dojoAttachPoint="domNode">',
              
    
    //csHelpID: "rqm_table_viewer.html", // default CS help id  //Add this if help is available
    NEW_VALUE: -1,
    COLUMN_REMOVED: -2,
    
    

    postCreate: function() 
    {
        this.inherited('postCreate', arguments);
        //this.cshelp.setButtonLabel(MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_settingsTitle")+" "+MessageUtil.getMessage(MessageBundle.COMMON, "labelHelp"));
        this.setTitle(Constants.getMessage("labelEditColumnsTitle"));
       
       
        var columns = this.viewer.getColumnNames();	
        var columnArg = {
                parent: this,
                columnNames: this.viewer.getColumnNames()
            };
       this.columnsWidget= new com.ibm.asq.common.web.ui.internal.widgets.grid.GridColumnEditor(columnArg);
        
        //Add the message area
        this.contentContainer.appendChild(this.columnsWidget.domNode);    
        this.messageNode = document.createElement("span");
        dojo.addClass(this.messageNode, "settings-column-label-attention");
        this.contentContainer.appendChild(this.messageNode); 

        this._augmentDialogButtons();
        
         
    },


    //add the OK and Cancel button
    _augmentDialogButtons: function()
    {
        this.actionsContainer.innerHTML = "";
        var yesButtonDiv = document.createElement("span");
        this._yesButton = document.createElement("button");
        this._yesButton.innerHTML = this.yesLabel;
        this._yesButton.tabIndex = "0";
        this.connect(this._yesButton, "onclick", "_yesAction");
        yesButtonDiv.appendChild(this._yesButton);
        yesButtonDiv.className = "button-div";
        
        var noButtonDiv = document.createElement("span");
        this._noButton = document.createElement("button");
        this._noButton.innerHTML = this.noLabel;
        this._noButton.tabIndex = "0";
        this.connect(this._noButton, "onclick", "_noAction");
        noButtonDiv.appendChild(this._noButton);
        
        
        this.actionsContainer.appendChild(yesButtonDiv);
        this.actionsContainer.appendChild(noButtonDiv);
    },
    
    // Override the "_yesAction" handler to save the settings. 
    _yesAction: function() {
        
        this.messageNode.innerHTML = "";
        if(this.columnsWidget.checkColumnNames()){
                           var msg = this.columnsWidget.getErrorMessage();
                           this.messageNode.innerHTML = msg;
                           return false;
                       }
        
        this.columns = this.columnsWidget.getColumnNames();
        this.inherited('_yesAction', arguments);
    
     },
    
    open: function() 
    {
        this.inherited('open', arguments);
        
    },

    // Returns the user modified settings.
    // 
    getColumns: function()  
    {
        return this.columns;
    }
});

})();
