/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.requirement.web.ui.internal.view.extension.RequirementsActionBarEntry");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressStatusMonitor");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ActionBarEntry");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.requirement.NewTestCasesForReqs");

(function() 
{
    var Constants                   = com.ibm.rqm.planning.web.client.internal.Constants;
	var PermissionsClient           = com.ibm.asq.common.web.client.internal.PermissionsClient;
	var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
	var ServiceResponseHandler		= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var GlobalEvents   				= com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var RequirementWorkItemClient   = com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient;
	var ProgressUtility = com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressUtility;

	dojo.declare(
		"com.ibm.rqm.requirement.web.ui.internal.view.extension.RequirementsActionBarEntry",
		[com.ibm.rqm.planning.web.ui.internal.view.common.ActionBarEntry],
		{
			constructor: function()
			{
			},
	
			postCreate: function(args)
			{
			},
			
			initialize: function(args)
			{
				//com.ibm.rqm.planning.web.ui.internal.view.actionbarentries.RequirementsActionBarEntry.superclass.initialize.call(this, args);
				this.inherited('initialize', arguments);
				this._addNonDispatchableMenuEntry(Constants.getMessage("labelCreateRequirement"), dojo.hitch(this, "_onClickCreateRequirement"), false, null);
				this._addMenuSeparator();
				var reqPermission = PermissionsClient.getStoredPermission(PermissionsClient.OP_REQ_WORKITEM);
				if (!reqPermission || reqPermission[PermissionsClient.ACTION_IMPORT]) {
                    this._addMenuEntry(Constants.getMessage("labelLaunchReqmtTool"), dojo.hitch(this, "_onClickLaunchRequirementTool"), false, {
                        'subAction': "importRequirements"
                    });
					this._addMenuSeparator();
				}
				this._addMenuEntry(Constants.getMessage("labelAllReqmt"), dojo.hitch(this, "_onClickAllRequirements"), false, {'subAction':"allRequirements"});
				if(this.actionDispatcher) {
                    this.actionDispatcher.registerDispatchAction("viewAssociatedTestCases", dojo.hitch(this, "viewAssociatedTestCases"));
					this.actionDispatcher.registerDispatchAction("viewAssociatedTestPlans", dojo.hitch(this, "viewAssociatedTestPlans"));
                }
				dojo.subscribe(GlobalEvents.TestCasesCreated, this, "openInNewTab");
			},
			
			_onClickCreateRequirement: function()
			{
				var timestamp = (new Date()).getTime();
				window.location.hash = "action=com.ibm.team.workitem.newWorkItem&type=requirement&ts="+timestamp;
			},
			
			_onClickAllRequirements: function()
            {
                var loadFunc = function(tab, onComplete){
                    this._loadClassAsync("com.ibm.rqm.requirement.web.ui.internal.view.table.RequirementTableView", dojo.hitch(this, "_loadRequirements", tab, onComplete));
                }
                var name = Constants.getMessage("labelAllReqmt");
                var browserTitle = Constants.getMessage("labelRequirements") + Constants.getMessage("labelColen") + " " + Constants.getMessage("labelAllView");
                this._addPage( {
                    name: name,
                    onDeferredLoad: dojo.hitch(this, loadFunc),
                    closable: true,
                    browserTitle: browserTitle
                });  
            },            

            _loadRequirements: function(tab, onComplete)
            {
                var name = Constants.getMessage("labelAllReqmt");
                var queryArgs = {};
				var actions = [];
                //add delete action
                var reqPermission = PermissionsClient.getStoredPermission(PermissionsClient.OP_REQ_WORKITEM);
                if (!reqPermission || reqPermission[PermissionsClient.ACTION_ARCHIVE]) {
                    actions.push({
                        label: Constants.getMessage("labelDelete"),
                        actionHandler: dojo.hitch(this, "_preDeleteRequirements"),
                        iconClass: "delete-icon-image",
                        disabledOnReadOnly: true
                    });
                }				
				actions.push({
	                label: Constants.getMessage("labelAssociatedTestPlans"),
	                actionHandler: dojo.hitch(this, "_handleFindAssociatedTestPlan"),
	                iconClass: "tplans-w-requirmt-icon-image",
	                disabledOnReadOnly: true
            	});
				actions.push({
	                label: Constants.getMessage("labelAssociatedTestCases"),
	                actionHandler: dojo.hitch(this, "_handleFindAssociatedTestCase"),
	                iconClass: "tcases-w-requirmt-icon-image",
	                disabledOnReadOnly: true
            	});
				
				//add "create test cases" button
				var tcPerm = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEST_CASE);
				if (!tcPerm || tcPerm.create) {
					actions.push({
						label: Constants.getMessage("labelCreateTestCases"),
						actionHandler: dojo.hitch(this, "_handleCreateTestCases"),
						iconClass: "add-new-testcase-image",
						disabledOnReadOnly: true
					});
				}
				
                var tableViewArgs = {
					queryName: name,
                    requirements: [],
                    showInitLoadProgress: true,
					actions: actions,
					showQueryFilters: true
                }
	            var list = new com.ibm.rqm.requirement.web.ui.internal.view.table.ExtRequirementTableView(tableViewArgs);
                onComplete(tab, list);
            }, 
			
			
			_refreshRequirementList: function(tableView){
            	var self = this;
            	var handler = {
                	success: function(requirements){
                    	if (tableView) {
                        	tableView.refresh(requirements || []);
                    	}
                	},
                	error: function(e){
                    	self._error(e);
                	}
           		}
            
           		var responseHandler = new ServiceResponseHandler(handler, "success", "error");
            	RequirementWorkItemClient.getRequirementWorkItemDTOs(responseHandler, {});
        	},
			
			_preDeleteRequirements: function(tableView)
            {
                // handle virtual select all case...
                tableView.invokeAction("itemId", this, "_deleteRequirements", tableView);
            },
            
            _deleteRequirements: function(selectedReqs, tableView) {
				var len = selectedReqs.length;
                var self = this;
                var dialogArgs = {
                    message: Constants.getMessage("msgConfirmDeleteRequirements",len),
                    okCallback: function(args){
                        var handler = {
                            success: function(msg) {
                                ProgressUtility.endTaskMonitor(this.monitorId);
                                if (tableView.contentIsVirtual) {
                                    tableView._invalidateQuery();
                                }
                                else {
                                    self._refreshRequirementList(tableView);
                                }
                                if(msg && msg!=""){
                                    msg = msg.replace(/\n/gi, "<br>");
                                    var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({'message': msg});
                                    alertDialog.setSecondaryTitle(Constants.getMessage("labelAlert"));
                                    alertDialog.open();
                                }
                            },
                            error: function(e) {
                                tableView._invalidateQuery();
                                ProgressUtility.endTaskMonitor(this.monitorId, e.message);
                            }
                        }            
                        var delArgs={
                            itemIds: selectedReqs
                        };
                        var responseHandler = new ServiceResponseHandler(handler, "success", "error");          
                        com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient.postArchiveRequirements(responseHandler, delArgs);
                        handler.monitorId = ProgressUtility.beginTaskMonitor(Constants.getMessage("msgDeletingRequirements"), 0, null);  
                    },
                    cancelCallback: function(args){
                        return;
                    }
                };
                
                var floatingPane = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog(dialogArgs);
                floatingPane.setSecondaryTitle(Constants.getMessage("labelConfirmation"));
                floatingPane.open();                
            },			
			
			_handleFindAssociatedTestPlan: function(tableView) {
				var selectedReqs = tableView.getSelectedRows();
				if(!selectedReqs || !dojo.isArray(selectedReqs) || selectedReqs.length != 1) {
					return;
				}
				var selectedReq = selectedReqs[0];
				var args = {};
				args.subAction = "viewAssociatedTestPlans";
				args.itemId = selectedReq.itemId;
				args.id = selectedReq.id;
				this._routeAction(args);
            }, 
			
			viewAssociatedTestPlans: function(args) {
                var name = Constants.getMessage("labelAssociatedTestPlansTitle") + args.id;
                var createFunc = function(tab, onComplete){
                    // show only the test plans that are associated with the requirement.
                    var tableViewArgs = {
                        plans: [],
                        // important - must provide a unique id for the table. Otherwise,
                        // the cached filter mechanism in the base table will reuse the
                        // cached filters for the base TP list view.
                        tableId: "RQM-REQ-TP-TABLE-FOR-" + args.id,
                        parentItemIdArgName: "associatedReq",
                        parentVersionItemId: args.itemId,
                        showQueryFilters: true,
                        queryName: name
                    };
                    var list = new com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView(tableViewArgs);
                    onComplete(tab, list);
                }
                var loadFunc = function(tab, onComplete){
                    this._loadClassAsync("com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView", dojo.hitch(this, createFunc, tab, onComplete));
                }
                var pageName = name; 
				this._addPage( {
                    name: pageName,
                    onDeferredLoad: dojo.hitch(this, loadFunc),
                    closable: true
                });
			},
            
			_handleFindAssociatedTestCase: function(tableView) {
				var selectedReqs = tableView.getSelectedRows();
				if(!selectedReqs || !dojo.isArray(selectedReqs) || selectedReqs.length != 1) {
					return;
				}
				var selectedReq = selectedReqs[0];
				var args = {};
				args.subAction = "viewAssociatedTestCases";
				args.itemId = selectedReq.itemId;
				args.id = selectedReq.id;
				this._routeAction(args);
            }, 
			
			viewAssociatedTestCases: function(args) {
				var name = Constants.getMessage("labelAssociatedTestCasesTitle") + args.id;
                var createFunc = function(tab, onComplete){
                    var tableViewArgs = {
                        testCases: [],
                        // important - must provide a unique id for the table. Otherwise,
                        // the cached filter mechanism in the base table will reuse the
                        // cached filters for the base TC list view.
                        tableId: "RQM-REQ-TC-TABLE-FOR-" + args.id,
                        parentItemIdArgName: ["associatedReq"],
                        parentVersionItemId: [args.itemId],
                        showQueryFilters: true,
                        queryName: name
                    }
                    // overload the parent item id arg to also include the specified
                    // tp item id. This is to constrain the tc to the specified tp parent.
                    //
                    if (args.testPlanItemId) {
                        tableViewArgs.parentItemIdArgName.push("constrainParentVTestPlan");
                        tableViewArgs.parentVersionItemId.push(args.testPlanItemId);
                    }else{
						tableViewArgs.parentItemIdArgName.push("constrainParentVTestPlan");
						tableViewArgs.parentVersionItemId.push(null);
					}
                    var list = new com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView(tableViewArgs);
                    this._testCaseTable = list;
                    onComplete(tab, list);
                }
                var loadFunc = function(tab, onComplete){
                    this._loadClassAsync("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView", dojo.hitch(this, createFunc, tab, onComplete));
                }                
                var pageName = name; 
				this._addPage( {
                    name: pageName,
                    onDeferredLoad: dojo.hitch(this, loadFunc),
                    closable: true
                });
			},
			
            // Obsolete - not used. Test case table view now handles all query filterings.
			_fetchAssociatedTestCases: function(queryArgs) {
                var self = this;
                var handler = {
                    success: function(cases) {
						self._testCaseTable.refresh(cases ? cases : []);
						self._testCaseTable.stopInProgress();
                    },
                    error: function(e) {
                        self._error(e);
                    }
                }
                
                var responseHandler = new ServiceResponseHandler(handler, "success", "error");          
                RequirementWorkItemClient.getTestCasesByRequirement(responseHandler, queryArgs);  				
			},
			
			_onClickLaunchRequirementTool: function()
			{
                this._showRequirementsUpload();
			},
			
			_showRequirementsUpload: function()
			{
                var createFunc = function(tab, onComplete){
                    var args = {
                        title: Constants.getMessage("labelUploadRequirements"),
                        artifactType: "requirement",
                        contextHelpId: "rqm_import_requirements.html",
                        cshelpLabel: Constants.getMessage("labelImportReqHelp")
                    };
                    var view = new com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementsUploader(args);
                    onComplete(tab, view);
                }
                var loadFunc = function(tab, onComplete){
                    this._loadClassAsync("com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementsUploader", dojo.hitch(this, createFunc, tab, onComplete));
                }
				var browserTitle = Constants.getMessage("labelRequirements") + Constants.getMessage("labelColen") + " " + Constants.getMessage("labelImport");
				this._addPage( {
                    name: Constants.getMessage("labelUploadRequirements"),
                    onDeferredLoad: dojo.hitch(this, loadFunc),
                    closable: true,
                    browserTitle: browserTitle
                });
			},
			
			_handleCreateTestCases: function(tableView) {
                var createTestCasesProc = dojo.hitch(this, function(selectedReqs){
                    if (!selectedReqs || selectedReqs.length == 0) {
                        return;
                    }
                    var newTCDislog = new com.ibm.rqm.requirement.web.ui.internal.view.requirement.NewTestCasesForReqs({
                        requirments: selectedReqs,
                        okCallBack: dojo.hitch(this, "openInNewTab")
                    });
                })
                // we might not have all data. Get virtual selected rows will determine
                // whether this is a select all and, if so, it will fetch all data for this action.
                // if there are deselected items from a select all, filter them by the item's id
                tableView.getVirtualSelectedRows('id', createTestCasesProc);
			},
			
			openInNewTab: function(args){
				if (args && args.length > 1) {
					var name = Constants.getMessage("labelNewCasesForReq");
                    var createFunc = function(tab, onComplete){
                        var tableViewArgs = {
                            testCases: args,
                            queryName: name,
                            showQueryFilters: false,
                            enableRowEditable: true,
                            contentIsVirtual: false
                        }
                        var list = new com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView(tableViewArgs);
                        onComplete(tab, list);
                    }
                    var loadFunc = function(tab, onComplete){
                        this._loadClassAsync("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView", dojo.hitch(this, createFunc, tab, onComplete));
                    }
					this._addPage({
                        name: name,
                        onDeferredLoad: dojo.hitch(this, loadFunc),
                        closable: true
                    });
				}
				else if(args && args.length == 1) {
					window.location.hash = "action=com.ibm.rqm.planning.home.actionDispatcher&subAction=viewTestCase&id=" + args[0].id;
				}
			}
		}
	);
}
)();
