/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.requirement.web.ui.internal.view.extension.RequirementsSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");

dojo.require("com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementSelectionDialog");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementsSource");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.table.RequirementTableView");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.requirement.NewTestCasesForReqs");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function(){ // module container
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
	var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
    var RequirementWorkItemClient = com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient;
    var RequirementTableView = com.ibm.rqm.requirement.web.ui.internal.view.table.RequirementTableView;
    
    var REQ_WORKITEM_TYPE_ID = "requirement";
    
    dojo.declare("com.ibm.rqm.requirement.web.ui.internal.view.extension.RequirementsSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: '<div class="com-ibm-rqm-test-requirement-plan-section">' +
                        '   <div dojoAttachPoint="_tableContainer"></div>' +
                        '</div>',
        
        constructor: function(args){
            this.RequirementWorkItemClient = RequirementWorkItemClient;
            this._reqTableViewer = null;
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport();
			
			//if this is a copy, must force initialization in order to create new links
			if (this.getParentEditor().isCopy()) {
				this.initialize(this.isEditable());
			}			
        },
        
        //@Overriden from AbstractArtifactEditorSection
        initialize: function(isEditable){
            this._requirements = [];
            this._requirementsAdded = [];
            this._requirementsRemoved = [];
            if (this.getArtifactType() == Constants.TYPE_PLAN) {
                this._linkType = "com.ibm.rqm.requirement.linkType.testPlan";
            }
            else {
                this._linkType = "com.ibm.rqm.requirement.linkType.testCase";
            }
            
            this.showRequirementList();
			this.initialized = true;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            
            // we need to retain the selection in the table for the
            // user after save. Get the current selected rows before
            // we refresh, and we'll set them to be selected when
            // get refreshed the list.
            if(this._retainSelection && this._reqTableViewer) {
                var rows = this._reqTableViewer.getSelectedRows();
                if(rows && dojo.isArray(rows) && rows.length) {
                    this.selectedRows = rows;
                }
                delete this._retainSelection;
            }
            // if we have not been shown, don't need to
            // create the content
            if (this.initialized) {
                this.initialize();
            }
            this.addWorkItemSupport();
            if (this._reqTableViewer) {
                this._reqTableViewer.setEditable(this.isEditable() && this.getArtifact());
            }
			this._requirementsAdded = [];
            this._requirementsRemoved = [];
        },
        
        //actions to be displayed in the table header
        getTableActions: function(){
            var actions = [];            
            if (this.getArtifactType() == Constants.TYPE_PLAN) {
				//add "create test cases" button
				var tcPerm = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEST_CASE);
				if (!tcPerm || tcPerm.create) {
					actions.push({
						label: Constants.getMessage("labelCreateTestCases"),
						actionHandler: dojo.hitch(this, "_handleCreateTestCases"),
						iconClass: "add-new-testcase-image",
						disabledOnReadOnly: true
					});
				}
                actions.push({
                    label: Constants.getMessage("labelImportRequirements"),
                    actionHandler: dojo.hitch(this, "_handleImportRequirementClicked"),
                    iconClass: "import-reqirement-image",
                    disabledOnReadOnly: true
                });
            }
            
            actions.push({
                label: Constants.getMessage("labelAssociateRequirements"),
                actionHandler: dojo.hitch(this, "_handleAssociateRequirementClicked"),
                iconClass: "add-icon-image",
                disabledOnReadOnly: true
            });
            
            if (this.getArtifactType() == Constants.TYPE_PLAN) {
                actions.push({
                    label: Constants.getMessage("labelAddNewRequirement"),
                    actionHandler: dojo.hitch(this, "_handleNewRequirementClicked"),
                    iconClass: "add-new-requirement-image",
                    disabledOnReadOnly: true
                });
            }
            
            actions.push({
                label: Constants.getMessage("labelDisassociateRequirements"),
                actionHandler: dojo.hitch(this, "_handleRemoveRequirementClicked"),
                iconClass: "remove-reqirement-image",
                disabledOnReadOnly: true
            });
			
            if (this.getArtifactType() == Constants.TYPE_PLAN) {
                actions.push({
	                label: Constants.getMessage("labelAssociatedTestCases"),
	                actionHandler: dojo.hitch(this, "_handleFindAssociatedTestCaseClicked"),
	                iconClass: "tcases-w-requirmt-icon-image",
	                disabledOnReadOnly: true
            	});
            }
            
            return actions;
        },
        
        showRequirementList: function(){
            if (!this._reqTableViewer) {
                dojox.data.dom.removeChildren(this._tableContainer);
                var args = {
                    requirements: this._requirements,
                    editable: this.isEditable(),
                    actions: this.getTableActions(),
                    parent: this,
					printable: this.isPrintView(),
                    contentIsVirtual: false,
                    // the table have actions to update the requirement's statuses,
                    // need to get our copy refreshed as well.
                    onStatusRefreshed: dojo.hitch(this, function(requirements) {
                        this._requirements = requirements;
                    }),
					refreshCallback: dojo.hitch(this, "_loadRequirements"),
					showDuplicateAction: false
                };
                this._reqTableViewer = new RequirementTableView(args);
                this._tableContainer.appendChild(this._reqTableViewer.domNode);
            }
            
            //load the table
            if (this.getArtifact()) {
                this._loadRequirements();
            }
        },
        
        _refreshRequirementList: function(requirements) {
			requirements = requirements || [];
			if (this.getParentEditor().isCopy() && !this._copyInitialized) {
				//for copied artifacts, add existing reqts (from copied artifact) as new links to new copuy 
				this.addRequirements(requirements);
				this._copyInitialized = true;
			} else {
				this._requirements = requirements;	
			}		
            
            this.refresh();			
        },
        
        _checkSelectedRowsExist: function(rows) {
            var len = rows.length;
            var list = [];
            for(var i = 0; i < len; i++) {
                var row = rows[i];
                if(this._contains(this._requirements, row)) {
                    list.push(row);
                }
            }
            return list;
        },
        
        refresh: function(){
            this._reqTableViewer.refresh(this._requirements);
            
            // we're retaining the selections for the user. Set the select
            // rows before we refresh the table's content.
            if(this.selectedRows) {
                var tableViewer = this._reqTableViewer.tableViewer;
                if (this._requirements && tableViewer) {
                    var rows = this._checkSelectedRowsExist(this.selectedRows);
                    if (rows.length > 0) {
                        tableViewer.selectRows(rows, true);
                    }
                }
                delete this.selectedRows;
            }
			this._reqTableViewer.disaccosicateButton.setDisabled(true);
			if(this._reqTableViewer.assoicatedTestPlanNode!=null)
				this._reqTableViewer.assoicatedTestPlanNode.setDisabled(true);
			if(this._reqTableViewer.assoicatedTestCaseNode!=null)
				this._reqTableViewer.assoicatedTestCaseNode.setDisabled(true);
			if(this._reqTableViewer.clearStatusActionNode!=null)
				this._reqTableViewer.clearStatusActionNode.setDisabled(true);	
		},
        
        _loadRequirements: function(){
            if (this.getArtifact() && this._reqTableViewer) {
                var responseHandler = new ServiceResponseHandler(this, "_refreshRequirementList", "_error");
                switch (this.getArtifactType()) {
                    case Constants.TYPE_PLAN:
                        this.RequirementWorkItemClient.getRequirementWorkItemDTOs(responseHandler, {
                            testPlanItemId: this.getArtifact().versionableItemId
                        });
                        break;
                    case Constants.TYPE_CASE:
                        this.RequirementWorkItemClient.getRequirementWorkItemDTOs(responseHandler, {
                            testCaseItemId: this.getArtifact().versionableItemId
                        });
                        break;
                }
            }
            else {
                //this.RequirementWorkItemClient.getAllRequirements(this, callbackFunction);
            }
        },
        
        _handleAssociateRequirementClicked: function(){
            var self = this;
            var dialogArgs = {
                object: this,
                requirements: this._requirements,
				removedReqs:this._requirementsRemoved,
                type: this.getArtifactType(),
                artifactId: this.getArtifact()? this.getArtifact().versionableItemId : null,
                onOK: function(requirements){
					if(self.getArtifactType() == Constants.TYPE_PLAN){
						var args = {
							title: Constants.getMessage("labelConfirmation"),
							message: Constants.getMessage("msgConfirmSaveRelatedTC"),
							yesCallback: function() {
								self.addRequirements(requirements, true);
							},
                			noCallback: function() {
                				self.addRequirements(requirements);
                			}
						}						
						var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
                		dialog.setSecondaryTitle(args.title);
                		dialog.open();
					} else {
						self.addRequirements(requirements);	
					}                    
                }
            }
            this.requirementDialog = new com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementSelectionDialog(dialogArgs);
			var args = {
                primaryTitle: Constants.getMessage("labelAssociateRequirement"),
                // make our z index below that of a popup tooltip. Otherwise,
                // our dialog will cover the tooltip popups
                zIndex: 998,
                forceZIndex: true,
                isClosable: true,
                contentNode: this.requirementDialog.domNode,
                _yesAction: function(){
                    // override the button action to not close the
                    // dialog immediately. the base impl does check for
                    // return value to not popdown the dialog, but it
                    // will re-enable the yes button.
                    this._yesButton.disabled = true;
                    this.okCallback();
                },                
				okCallback: dojo.hitch(this.requirementDialog, "_onOK")
            }
            this.floatingPane = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(args);
			this.floatingPane._yesButton.disabled = true;
            dojox.data.dom.removeChildren(this.floatingPane._headerText);
            var span1 = document.createElement("span");
            span1.innerHTML = Constants.getMessage("labelAssociateRequirement");
            this.floatingPane._headerText.appendChild(span1);
            var cshelp = new CSHelp({
                contextID: "rqm_assoc_req.html"
            });
            cshelp.setButtonLabel(Constants.getMessage("labelAssociateReqCSHelp"));
            var span2 = document.createElement("span");
            span2.appendChild(cshelp.domNode);
            this.floatingPane._headerText.appendChild(span2);
            
            this.floatingPane.open(false);
            this.floatingPane.domNode.style.width = "90%";
            this.requirementDialog._dialog = this.floatingPane;
			this.requirementDialog.initialize();
        },
        
        _handleImportRequirementClicked: function(){
            var self = this;
            var setTitle = function(title){
                self.importFloatingPane.setTitle(title);
            };
            var onCancel = function(){
                self.importFloatingPane.close();
            };
            var onDone = function(requirements){
                self.importFloatingPane.close();
                self.addRequirements(requirements);
            };
            this._requirementsSource = new com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementsSource({
                requirements: this._requirements,
                setTitle: setTitle,
                onCancel: onCancel,
                onDone: onDone,
                parent: self
            });
            
            var args = {
                primaryTitle: Constants.getMessage("labelImportRequirementStep1"),
                isClosable: true,
				width: "50%",
                contentNode: this._requirementsSource.domNode,
				zIndex: 9998
            }
            this.importFloatingPane = new com.ibm.asq.common.web.ui.internal.widgets._CommonDialog(args);
            dojox.data.dom.removeChildren(this.importFloatingPane._headerText);
            var span1 = document.createElement("span");
            span1.innerHTML = args.primaryTitle;
            this.importFloatingPane._headerText.appendChild(span1);
            var cshelp = new CSHelp({
                contextID: "rqm_import_req.html"
            });
            cshelp.setButtonLabel(Constants.getMessage("labelImportReqCSHelp"));
            var span2 = document.createElement("span");
            span2.appendChild(cshelp.domNode);
            this.importFloatingPane._headerText.appendChild(span2);
            
            this.importFloatingPane.open(false);
            dojo.mixin(this.importFloatingPane, {
                "setTitle": function(title){
                    dojox.data.dom.removeChildren(this._headerText);
                    var span1 = document.createElement("span");
                    span1.innerHTML = title;
                    this._headerText.appendChild(span1);
                    var cshelp = new CSHelp({
                        contextID: "rqm_import_req.html"
                    });
                    cshelp.setButtonLabel(Constants.getMessage("labelImportReqCSHelp"));
                    var span2 = document.createElement("span");
                    span2.appendChild(cshelp.domNode);
                    this._headerText.appendChild(span2);
					dijit.focus(this._headerText);
                }
            });
        },
        
        _handleNewRequirementClicked: function(){
            var self = this;
            var args = {
                callback: {
                    method: dojo.hitch(self, "addNewRequirement"),
                    returnAsDTO: true
                },
                workItem: null,
                workItemTypeId: REQ_WORKITEM_TYPE_ID,
				title: Constants.getMessage("labelAddNewRequirement"),
                cshelpId: "rqm_add_req.html"
            }
            this._wiPopup = new com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane(args);
            //this._workitemnumber.appendChild(this._wiPopup.domNode);
        },
        
        addNewRequirement: function(requirement){
            //create the link b/w the artifact and this new requirement
            if (requirement) {
                var newReq = {
                    id: requirement.id,
                    status: "",
                    itemId: requirement.itemId,
                    name: requirement.summary.content,
                    description: requirement.description.content,
                    owner: requirement.attributes.owner,
                    externalId: ""
                };
                this._requirements.push(newReq);
                this._requirementsAdded.push(newReq);
                this.getSaveContext().createLink(this.getParentEditor(), this._linkType, requirement.itemId);
                this.setDirty();
                this.refresh();
            }
        },
        
        _handleRemoveRequirementClicked: function(){
            var selectedRequirements = this._reqTableViewer.getSelectedRows();
            if (selectedRequirements && selectedRequirements.length > 0) {
				var len = selectedRequirements.length;
				var self = this;
				var args = {
					title: Constants.getMessage("labelConfirmation"),
					message: Constants.getMessage("msgConfirmDeleteRequirements",len),
					yesCallback: function(){   
						console.debug("Number of to be removed - " + len);
                		self.removeRequirements(selectedRequirements);
						return true;          
					},
					noCallback: function() {
						return false;
					}
				}
				var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
	        	dialog.setSecondaryTitle(args.title);
	        	dialog.open();
            }
            else {
                this.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, Constants.getMessage("msgNoRequirementDisassociate"));
            }
        },
        
        _handleFindAssociatedTestCaseClicked: function() {
			var selectedReqs = this._reqTableViewer.getSelectedRows();
			if(!selectedReqs || selectedReqs.length != 1) {
				return;
			}
			var selectedReq = selectedReqs[0];
			var args = {};
			args.subAction = "viewAssociatedTestCases";
			args.itemId = selectedReq.itemId;
			args.id = selectedReq.id;
			args.testPlanItemId = this.getArtifact().versionableItemId;
			var uri = "#action=com.ibm.rqm.planning.home.actionDispatcher"; //FIXME removed hard coded action dispatcher id
                
            for(var argName in args) {
                var argVal = args[argName];
                
                if(argName == "tq")
                    argVal = Date.parse(new Date().toString());
                    
                uri += ("&" + argName + "=" + argVal); 
            }
            window.location.hash = uri;
        }, 
		
		_handleCreateTestCases: function() {
            var selectedReqs = this._reqTableViewer.getSelectedRows();
            if (!selectedReqs) {
                return;
            }
            
            // since we're potentially going to modify the parent test plan with this action
            // and load the modified test plan to the parent editor, we need to ask the user
            // to save any unsaved changes first to ensure that the user wouldn't lose any
            // work. 
            var parentEditor = this.getParentEditor();
            if(parentEditor.isDirty()) {
                var dialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({
                    primaryTitle:  Constants.getMessage("labelNewTestCase"),
                    message: Constants.getMessage("msgSaveTPBeforeCreateTC")
                });
                dialog.open();
                
                // remember the selection when we update the list
                // due to save
                this._retainSelection = true;
                return;
            }
            var newTCDislog = new com.ibm.rqm.requirement.web.ui.internal.view.requirement.NewTestCasesForReqs({
                requirments: selectedReqs,
				isInSection: true,
				artifactEditor: parentEditor,
                okCallBack: dojo.hitch(this, "openInNewTab")
            });
		},
			
		openInNewTab: function(args){
            if (args && args.length > 1) {
				dojo.publish(GlobalEvents.TestCasesCreated, [ args ]);
            }
            else 
                if (args && args.length == 1) {
                    window.location.hash = "action=com.ibm.rqm.planning.home.actionDispatcher&subAction=viewTestCase&id=" + args[0].id;
                }
		},
		
        addRequirements: function(requirements, addRelatedTestCases){
			var len = requirements?requirements.length:0;
            var editor = this.getParentEditor();
			var reqtIds = [];
			for (var i = 0; i < len; i++) {
				var reqt = requirements[i];
                if (this._contains(this._requirementsRemoved, reqt)) {
                    this.getSaveContext().unremoveLink(editor, this._linkType, reqt.itemId);
                    this._requirementsRemoved = this._removeItem(this._requirementsRemoved, reqt);
                    this._requirements.push(reqt);
                
				} else if (!this._contains(this._requirements, reqt)) {
                    this.getSaveContext().createLink(editor, this._linkType, reqt.itemId);
                    this._requirementsAdded.push(reqt);
                    this._requirements.push(reqt);							
					reqtIds.push(reqt.itemId);
					
				} else {
                    editor.setHeaderStatus(Constants.getMessage("msgExistRequirement"), Constants.HEADER_STATUS_INFO);
                    this._displayAlertDialog(Constants.getMessage("labelAlert"), Constants.getMessage("msgExistRequirement"));
                }
            }
			
			if (addRelatedTestCases === true && reqtIds.length > 0) {
            	var responseHandler = new ServiceResponseHandler(this, "_addTestCases", "_error");
                var args = {
					itemIds: reqtIds
				}
				this.RequirementWorkItemClient.getTestCasesByRequirements(responseHandler, args);							
			}
								
            this.setDirty();
            this.refresh();
        },		
		
		_addTestCases: function(relatedCases){
			//ensure the parent editor contains the test case section
			var section = this.getParentEditor().getSection("com.ibm.rqm.planning.editor.section.planTestCases"); //$NON-NLS-1$
			if (section == null) {
				console.warn("Parent Editor does not contain the TestCases section."); //$NON-NLS-1$
				return;
			}
			//ensure we're only adding test cases that the section doesn't already contain
			var len = (relatedCases || []).length;
			for (var i = 0; i < len; i++) {
				var tCase = relatedCases[i];
				if (!section.containsTestCase(tCase)) {
					section.addTestCases([tCase]);
				}
			}
		},

        removeRequirements: function(requirements){
        	var msg = "";
        	var isDirty = false;
			var len = requirements?requirements.length:0;
            for (var i = 0; i < len; i++) {
            	// DOORS exported requirements are read-only, cannot remove from plan
            	if(typeof(requirements[i].externalId)!="undefined"&&requirements[i].externalId.match(/^doors\//i)&&this.getParentEditor().getArtifactType()==Constants.TYPE_PLAN) {
            		msg += Constants.getMessage("msgCannotRemoveReadOnlyRequirement", requirements[i].id) + "<br>";
            	}
            	else {
                	this._removeRequirement(requirements[i]);
                	isDirty = true;
                }
            }
            if(msg != ""){
            	var alertDialog = new AlertDialog({'message': msg});
	    	    alertDialog.setSecondaryTitle(Constants.getMessage("labelAlert"));
	        	alertDialog.open();
            }
            if (isDirty)
            	this.setDirty();
            this.refresh();
        },
        
        _contains: function(items, item){
            var id = item.id;
            var externalId = item.externalId;
            if (id) {
                return this._getRequirementById(items, id) != null;
            }
            else {
                return this._getRequirementByExternalId(items, externalId) != null;
            }
        },
        
        _getRequirementById: function(items, id){
			var len = items.length;
            for (var i = 0; i < len; i++) {
                var requirement = items[i];
                var itemId = requirement.id;
                if (itemId == id) {
                    return items[i];
                }
            }
            return null;
        },
        
        _getRequirementByExternalId: function(items, id){
			var len = items.length;
            for (var i = 0; i < len; i++) {
                var requirement = items[i];
                var itemId = requirement.externalId;
                if (itemId == id) {
                    return items[i];
                }
            }
            return null;
        },
        
        _removeRequirement: function(item){
            var updatedItems = [];
            var items = this._requirements;
			var len = items.length;
            for (var i = 0; i < len; i++) {
                if (items[i].itemId && items[i].itemId != item.itemId) {
                    updatedItems.push(items[i]);
                }
                else {
                    //handle item removal from save context   
                    var saveContext = this.getSaveContext();
                    if (this._contains(this._requirementsAdded, items[i])) {
                        //handle existing items ADDED during this saveContext session
                        this._requirementsAdded = this._removeItem(this._requirementsAdded, items[i]);
                        saveContext.uncreateLink(this.getParentEditor(), this._linkType, items[i].itemId);
                    }
                    else {
                        //handle existing references, add them 'removed'
                        this._requirementsRemoved.push(items[i]);
                        saveContext.removeLink(this.getParentEditor(), this._linkType, items[i].itemId);
                    }
                }
            }
            this._requirements = updatedItems;
        },
        
        _removeItem: function(items, item){
            var updatedArray = [];
			var len = items.length;
            for (var i = 0; i < len; i++) {
                if (items[i].itemId != item.itemId) {
                    updatedArray.push(items[i]);
                }
            }
            return updatedArray;
        },
        
        _error: function(e){
            this._displayAlertDialog(Constants.getMessage("labelError"), e.message);
        },
        
        _displayAlertDialog: function(title, message){
            var alertDialog = new AlertDialog({
                'message': message
            });
            alertDialog.setSecondaryTitle(title);
            alertDialog.open();
        }
        
    });
    
})();
