/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredPagingTableView");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.client.internal.ContributorClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormDateField");

dojo.require("com.ibm.rqm.planning.web.client.internal.RegistryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {

var ServiceResponseHandler      = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var CommonPreferenceClient		= com.ibm.asq.common.web.client.internal.CommonPreferenceClient
var TableContentFilterProvider  = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;
var TableConstants              = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;

var RegistryClient              = com.ibm.rqm.planning.web.client.internal.RegistryClient;
var UserClient                  = com.ibm.asq.common.web.client.internal.ContributorClient;
var Constants                   = com.ibm.rqm.planning.web.client.internal.Constants;
var FILTER_CACHE_OBJ_ID			= "com.ibm.rqm.planning.tableViews.filter.cache";
var DEFAULT_SCRIPT_TYPE         = "Manual";
var AlertDialog                 = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
var GlobalEvents                = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView", null,
{
    // optionally to pass in 'hidden' query parameters to constrain the results to
    // the specified parent artifact. The name is the query argument name to be 
    // used in the query. It requires a matching itemId for the argument's value.
    // These attributes can be array as well, if more then one 'parent' constraining 
    // artifacts are needed. The order of the argument & value array must match.
    // 
    parentItemIdArgName: null,
    parentVersionItemId: null,
    
	constructor: function() {
		this._typeLabel = "";
		this._searchFunction = null;
		this._searchCallback = null;
		this._searchFilterCallback = null; //allow caller to provide additional filter
		this._searchResponseHandlerType = ServiceResponseHandler;
        this._processDateTimeFilter = false; // default to not process time-based filtering unless
                                             // the filter is added and opt to process the params
		
		//get the required user preferences
		var initOpenPref = CommonPreferenceClient.getStoredUserPreferences(
			Constants.USER_UI_CUSTOMISE_QUERYLISTS_FILTER_INITOPEN_CATEGORY.key
		);
		this._initiallyOpen = initOpenPref && !dojo.isArray(initOpenPref) && initOpenPref.value == true;
		
		var onlyMinePref = CommonPreferenceClient.getStoredUserPreferences(
			Constants.USER_UI_CUSTOMISE_QUERYLISTS_ONLY_MY_ASSETS_CATEGORY.key
		);
		this._onlyMineByDefault = onlyMinePref && !dojo.isArray(onlyMinePref) && onlyMinePref.value == true;

		var runOnOpenPref = CommonPreferenceClient.getStoredUserPreferences(
			Constants.USER_UI_CUSTOMISE_QUERYLISTS_AUTORUN_MYQUERY_CATEGORY.key
		);
		this._runOnOpen = runOnOpenPref && !dojo.isArray(runOnOpenPref) && runOnOpenPref.value == true;
	},
	
	isInitiallyOpen: function() {
		return this._initiallyOpen;
	},
	
	isDefaultsToMine: function() {
		return this._onlyMineByDefault;
	},
	
    setTypeLabel: function(/* String */ label) {
        this._typeLabel = label;
    },  
		
	setSearchFunction: function(/* Function */ func) {
		this._searchFunction = func;
	},	
	
    setSearchCallback: function(/* Function */ func) {
        this._searchCallback = func;
    }, 

    setSearchFilterCallback: function(/* Function */ func) {
        this._searchFilterCallback = func;
    },
	
    setResponseHandlerType: function(/* TransportServiceResponseWrapper */ handlerType) {
        this._searchResponseHandlerType = handlerType || ServiceResponseHandler;
    },  
		
    setInProgress: function(msg) {
        if (this.viewHeaderNode) {
			this.viewHeaderNode.setProgressStart(msg);
		}
    },
    
    stopInProgress: function() {
        this.showInitLoadProgress = false;
		if (this.viewHeaderNode) {
			this.viewHeaderNode.setProgressEnd();
		}
		if (this.tableViewer) {
			this.tableViewer.endLoadingProgress();
		}
    },	
	
	cacheFilters: function(filterArgs) {
		var filterCache = dojo.getObject(FILTER_CACHE_OBJ_ID);
		if (filterCache == null) {
			filterCache = {};
		}
		filterCache[this._getCacheId()] = filterArgs;
		dojo.setObject(FILTER_CACHE_OBJ_ID, filterCache);
	},
	
	getCachedFilters: function() {
		var filterCache = dojo.getObject(FILTER_CACHE_OBJ_ID);
		if (filterCache != null) {
			return filterCache[this._getCacheId()];
		}	
		return null;	
	},
	
	_getCacheId: function() {
		var table = this.tableViewer;
		return table && table.tableId ? table.tableId : this.declaredClass;
	},
	
    _getCategoryFilters: function(cacheFilters) {
        var allCatValues = [];
        
        if (cacheFilters) {
            for (var i = 0; i < cacheFilters.length; i++) {
                var filters = cacheFilters[i];
                // all of the selected values are collapsed into a comma
                // separated string.
                if (filters && filters.length > 0) {
                    allCatValues = allCatValues.concat(filters.split(","));
                }
            }
        }   
        return allCatValues;
    },
    
    _mapFilterNameToCachedValues: function(fname, cache, cachedCategories){
        // saved category filters are collapse into a single
        // value, just map any category filter to it.
        if (fname.indexOf("categories") == 0) {
            return cachedCategories;            
        }
        // the weight filter is mapped to two values
        // in the saved query, convert it back to the
        // filter consumeable default value.
        if(fname == "weight") {
            var val = cache["weightValue"];
            var op = cache["weightComparator"];
            
            if(val) {
                return {'value': val, 'operator': op || null};
            }
        }
        // no special mapping. 
        return cache[fname];
    },
    
	initFilterProvider: function(provider) {
		var cache = this.getCachedFilters();
		if (cache != null) {
            // the category filters are all collapsed into a single value
            // in 'categories'. Process it once to get all of the default
            // values in each of the categories.
            var cachedCategories = this._getCategoryFilters(cache["categories"]);
			var filters = provider.getFilters() || [];
            
			for (var i = 0; i < filters.length; i++) {
				var f = filters[i];
                var defaultVal = this._mapFilterNameToCachedValues(f.name, cache, cachedCategories);
                
                // no cached values, no default values to set.
				if (defaultVal == null) {
					continue;
				}
				switch(f.type) {
					case TableContentFilterProvider.FIELD_TOGGLELIST:
                        // If any of this category values is in the filter list
                        // those values are the default values. Set them as the
                        // default values. Toggle list takes a list of values
                        // as defaults.
						if (defaultVal.length > 0 && f.params && f.params.options) {
                            var optDefaults = [];
							for (var j = 0; j < f.params.options.length; j++) {
								if (this._findIn(defaultVal, f.params.options[j].value) != null) {
									optDefaults.push(f.params.options[j].value);
								}
							}
                            f.defaultValue = optDefaults;
						}
						break;
						
					default:
						//set the default directly
						f.defaultValue = defaultVal;
						break;
				}
			}
		}
		
	},
	
	_findIn: function(array, value) {
		for (var i = 0; i < (array || []).length; i++) {
			if (array[i] == value) {
				return array[i];
			}
		}
		return null;
	},
    
    // Base method to prepare the query arguments
    // 
    _prepareQueryArgs: function(filterArgs, updatePageOnly) {
        if (filterArgs != null) {
            // ensure the artifact id, if in the filters, is a non-zero numeric value
            if (filterArgs.id != null) {
                var idNum = Number(filterArgs.id);
                if (isNaN(idNum) || idNum <= 0) {
                    delete filterArgs.id;
                }
            }
            // ensure the calculated risk, if specified, is numeric and not less than zero
            if (filterArgs.calculatedRisk != null) {
                var riskNum = Number(filterArgs.calculatedRisk);
                if (isNaN(riskNum) || riskNum < 0) {
                    delete filterArgs.calculatedRisk;
                }
            }
            // ensure the communityRisk risk, if specified, is numeric and not less than zero
            if (filterArgs.communityRisk != null) {
                var riskNum = Number(filterArgs.communityRisk);
                if (isNaN(riskNum) || riskNum < 0) {
                    delete filterArgs.communityRisk;
                }
            }
            // empty suspect is 'any' don't pass in any value to the query.
            if(filterArgs.suspect == "") {
                delete filterArgs.suspect;
            }
            
            if (filterArgs.weightValue != null) {
                // weight is defined, validate it is numeric value, empty string
                // means 'any' don't send any query parameter for that.
                var weightNum = Number(filterArgs.weightValue);
                if (isNaN(weightNum) || weightNum < 0 || filterArgs.weightValue == "") {
                    // invalid weight is specified, remove corresponding args
                    delete filterArgs.weightValue;
                    if (filterArgs.weightComparator) {
                        delete filterArgs.weightComparator;
                    }
                }
            }

            // save the new filter args
            //cache current filter settings (session persistence of filters)
            this.filterArgs = filterArgs;
            this.cacheFilters(this.filterArgs);
        }
        else {
            // no new filter args. just use the current one.
            filterArgs = this.filterArgs;
        }
        
        //add the current project area to the args
        filterArgs.processArea = com.ibm.team.process.web.projectArea.itemId;

        // If we have a parent item arg & itemId, use that to constraint our
        // query.
        if(this.parentItemIdArgName && this.parentVersionItemId) {
            // the parent item id queries can be more then one. process 
            // the multiple case too.
            if (dojo.isArray(this.parentItemIdArgName)) {
                if(dojo.isArray(this.parentVersionItemId) && 
                   this.parentVersionItemId.length == this.parentItemIdArgName.length) {
                    for(var i = 0; i < this.parentItemIdArgName.length; i++) {
						if(this.parentVersionItemId[i])
                        	filterArgs[this.parentItemIdArgName[i]] = this.parentVersionItemId[i];
						else{
							delete(filterArgs[this.parentItemIdArgName[i]]);
						}
                    }
                }
                else {
                    // log an error about mismatch parent parameter settings. 
                    console.error("The number of constrain artifact names does not match the number of artifact item id values");
                }
            }
            else {
                filterArgs[this.parentItemIdArgName] = this.parentVersionItemId;
            }
        }else if(this.parentItemIdArgName && this.parentVersionItemId == null)
			delete(filterArgs[this.parentItemIdArgName]);                

        // re-map composite filter params if there is any
        return this._remapQueryArgs(this.filterArgs);
    },
    
    // For filters with composite parameters, the server API would expect more than one parameters.
    // We need to cache the filter for persistence but have to re-map the single filter parameter
    // to multiple query parameters to satisfy the server API.  Take the modified time for example, the
    // composite filter parameter is re-mapped to "from" and "to" parameters before submitting the query.
    //
    _remapQueryArgs: function(cachedFilterArgs) {
        var filterArgs = cachedFilterArgs;
        
        // setup time-based filter params if filter is added and set to process dates with all required keywords
        if (this._processDateTimeFilter && this._dateTimeFilterKey && this._dateTimeFromFilterKey && this._dateTimeToFilterKey) {
            if (cachedFilterArgs[this._dateTimeFilterKey] != null) {
                filterArgs = dojo.clone(cachedFilterArgs);

                // apply the date time filter if defined
                var from = filterArgs[this._dateTimeFilterKey].fromDateTime;
                var to = filterArgs[this._dateTimeFilterKey].toDateTime;
                if (from) {
                    filterArgs[this._dateTimeFromFilterKey] = from;
                }
                if (to) {
                    filterArgs[this._dateTimeToFilterKey] = to;
                }
                delete filterArgs[this._dateTimeFilterKey];
            }
        }
        return filterArgs;
    },
    
    // Base method to prepare the table viewer before querying new results
    // 
    _prepareTableUIForQuery: function() {
        this.showInitLoadProgress = true;
        this.tableViewer.setContent([]);
        this.tableViewer.refresh();
        this.tableViewer.showLoadingProgress();
        this.setInProgress(Constants.getMessage("msgQueryTableList", this._typeLabel));        
    },

    // Base method to do additional processing on the returned results.
    // 
    _processResults: function(resultsDTO){
        // default to no work
    },
        
    // method invoked by the filter "Run" action to query with server side filtering. 
    runQuery: function(filterArgs, updatePageOnly) {
        var args = this._prepareQueryArgs(filterArgs, updatePageOnly);
        this._prepareTableUIForQuery();
        
        if(typeof updatePageOnly != "undefined") {
            this.updatePageOnly = updatePageOnly;
        }
        var self = this;
        var isNewQuery = !this.queryToken;
        var callback = {
            success: function(results){
                self.showQueryResults(results);
                if (isNewQuery && dojo.isFunction(self.onQueryComplete)) {
                    // allow optional processing after a new query is completed and result is shown
                    self.onQueryComplete();
                }
            },
            failure: function(error){
                self.showQueryError(error);
                if (isNewQuery && dojo.isFunction(self.onQueryComplete)) {
                    self.onQueryComplete();
                }
            }
        }
        var responseHandler = new this._searchResponseHandlerType(callback, "success", "failure");          
        this._searchFunction(responseHandler, args);   
    },
	
	// run the query automatically if the preference dictates
    initRunQuery: function() {  
        // the list can be override the runOnOpen preference. If it's set
        // obey the override setting.
        if (this._runOnOpen) {
			this._initRunQuery();
		}else {
			this.stopInProgress();
			if (this.tableViewer) {
				this.tableViewer.refresh();
			}
		}       
    },  
    
    _initRunQuery: function() {
        var filterArgs = this.initFilterArgs ? this.initFilterArgs : this.getCachedFilters();
        if (filterArgs == null) {
            filterArgs = {
                myItemsOnly: this._onlyMineByDefault,
                includeArchived: false
            }; 
        }
        this.runQuery(filterArgs, true);
    },	
	
	toggleQueryRunning: function(isRunning) {
		
	},
	
	// refresh by simply running the query with the current settings
	refreshQuery: function() {
        // preference is set to not auto run query and filter has never been ran to set the filterArgs
        if (this.filterArgs == undefined) {
            this._initRunQuery();
        } else if (this.filterArgs && !this._isRunningQuery) {
			this.runQuery(null, false);
		}
	},
    
	showQueryResults: function(resultsDTO) {
        this.stopInProgress();
        
        // HACK: to workaround a irregular return result from TER search...
        if(dojo.isArray(resultsDTO)) {
            resultsDTO = resultsDTO[0];
        }
		if (resultsDTO != null && 
            (resultsDTO.status == Constants.ARTIFACT_SEARCH_STATUS_OK || 
             resultsDTO.status == Constants.ARTIFACT_SEARCH_STATUS_OK_EXPIRED)) {
            
            // perform any additional processing on the result.
            this._processResults(resultsDTO);
            
            //search ok. show results
			if (this._searchFilterCallback && dojo.isFunction(this._searchFilterCallback)) {
				resultsDTO.results = this._searchFilterCallback(resultsDTO.results);
			}
			this._doSearchCallback(resultsDTO.results);
            
            // if the query has been expired. Show a message to warn the user.
            if(resultsDTO.status == Constants.ARTIFACT_SEARCH_STATUS_OK_EXPIRED) {
                var args = {
                    message: Constants.getMessage("msgQueryExpired"),
                    primaryTitle: ""
                };
                var alert = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(args);
                alert.open();
            }		  	
		} else if (resultsDTO != null && resultsDTO.status == Constants.ARTIFACT_SEARCH_STATUS_MAX_EXCEEDED) {
            //too many results, show warning dialog
            this.showQueryThresholdWarning(resultsDTO);			
		} else {
            //some error occurred
			this.showQueryError({
                message: resultsDTO ? resultsDTO.message : "Unknown Error"  
            });			
		}
	},
	
    _doSearchCallback: function(searchResults) {
		if(!this.tableViewer || this.tableViewer.uninitialized){
			console.error(this.declaredClass + ": tableviewer not found");
			return;
		}
		if (dojo.isFunction(this._searchCallback)) {
            this._searchCallback(searchResults);
        } else {
            console.error(this.declaredClass + ": no search callback secified!");
        }
    },

    showQueryError: function(error) {
		if(!this.tableViewer || this.tableViewer.uninitialized){
			console.error(this.declaredClass + ": tableviewer not found");
			return;
		}
		this.stopInProgress();
		this.tableViewer.showContentError((error && error.message) ? error.message : null);	
	},
	
	showQueryThresholdWarning: function(resultsDTO) {
        var dialogArgs = {
            primaryTitle: Constants.getMessage("labelSearchLimitWarning", resultsDTO.totalSize),
            yesLabel: Constants.getMessage("labelRefine"),
			yesCallback: dojo.hitch(this, function() {
                //set proper policy; user can refine search
				this.filterArgs.policy = Constants.ARTIFACT_SEARCH_POLICY_ENFORCE;
            }),
			noLabel: Constants.getMessage("labelLoadFirst", resultsDTO.size),
            noCallback: dojo.hitch(this, function() {
				//truncate the search to the threshold
                this.filterArgs.policy = Constants.ARTIFACT_SEARCH_POLICY_TRUNCATE;
				this.runQuery(this.filterArgs);
            }),
			cancelLabel: Constants.getMessage("labelLoadAll"),
            cancelCallback: dojo.hitch(this, function() {
                //run the query anyway, getting all results
                this._dialog.close();
				this.filterArgs.policy = Constants.ARTIFACT_SEARCH_POLICY_IGNORE;
                this.runQuery(this.filterArgs);
            }),		
			showCancel: true,	
            isClosable: false
        };                
        this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(dialogArgs);
        this._dialog.setWidth("600px");
		this._dialog._cancelButton.onclick = dialogArgs.cancelCallback;
		
        var messageNode = document.createElement("span");
        messageNode.innerHTML = Constants.getMessage("msgSearchLimitWarning", resultsDTO.totalSize);
		  	
		this._dialog.contentContainer.appendChild(messageNode);
        this._dialog.open();		
	},
	
	addArtifactStateFilter: function(filters) {
    	var stateFilter = {
			name: "stateIds", // must match key in the arg list of the API
			dbName: "state",
			title: Constants.getMessage("labelStateLabel"),
			promptTitle: Constants.getMessage("labelState"),
			type: TableContentFilterProvider.FIELD_TOGGLELIST,
			defaultValue: [],
			params: {
				options: [{
					name: Constants.getMessage("com.ibm.rqm.planning.common.new"),
					value: Constants.STATETRANSITION_STATE_DRAFT,
					checked: false
				}, {
					name: Constants.getMessage("com.ibm.rqm.planning.common.underreview"),
					value: Constants.STATETRANSITION_STATE_UNDERREVIEW,
					checked: false
				}, {
					name: Constants.getMessage("com.ibm.rqm.planning.common.approved"),
					value: Constants.STATETRANSITION_STATE_APPROVED,
					checked: false
				}, {
					name: Constants.getMessage("com.ibm.rqm.planning.common.retired"),
					value: Constants.STATETRANSITION_STATE_RETIRED,
					checked: false
				}]
			}
		}	
		filters.push(stateFilter);	
	},
	
    addRiskFilter: function(filterObj, filtersCollection) {
        var riskFilter = {
            name: filterObj.name, // must match key in the arg list of the API
            title: filterObj.label,
            promptTitle: filterObj.prompt,
            type: TableContentFilterProvider.FIELD_ENUM, 
            defaultValue: -1,
            params: [{
                name: Constants.getMessage("labelAnyCap"),
                value: -1
            }, {
                name: Constants.getMessage("msgNoRiskShort"),
                value: 0
            }, {
                name: Constants.getMessage("labelVeryHigh"),
                value: 5
            }, {
                name: Constants.getMessage("labelHigh"),
                value: 4
            }, {
                name: Constants.getMessage("labelNeutral"),
                value: 3
            }, {
                name: Constants.getMessage("labelLow"),
                value: 2
            }, {
                name: Constants.getMessage("labelVeryLow"),
                value: 1
            }]
        }   
        filtersCollection.push(riskFilter);  
    },
    
    addDateTimeRangeFilter: function(filterObj, filtersCollection) {
        this._processDateTimeFilter = filterObj.processFilter; // subclass has option to have the params processed
                                                               // here instead of handling it in the subclass
        this._dateTimeFilterKey = filterObj.name;        // these keywords must be specified in order to
        this._dateTimeFromFilterKey = filterObj.fromKey; // have the params processed here
        this._dateTimeToFilterKey = filterObj.toKey;
        
        var datetimeRangeFilter = {
            name: filterObj.name,  // there are two attributes in the filter for from and to datetime, the
                                   // consumer needs to match the name to compose proper parameters to run query
            title: filterObj.label,
            promptTitle: filterObj.prompt,
            type: TableContentFilterProvider.FIELD_TEXT_POPUP,
            defaultValue: filterObj.defaultValue,  // if defined, it should be in the format of
                                                   // {
                                                   //     fromDateTime: <date in long>,
                                                   //     toDateTime: <date in long>,
                                                   //     relative: "true" or "false" for display relative date or specific dates
                                                   // },
            params: {
                fieldWidth: 350,
                fieldClass: "com.ibm.asq.common.web.ui.internal.widgets.layout.FormDateTimeRangeField"
            }
        };
        filtersCollection.push(datetimeRangeFilter);
    },
    
	addContributorFilter: function(filterObj, filtersCollection, callback) {
        if (dojo.isArray(this.contributors) && this.contributors.length > 0) {
			this._doAddContributorFilter(this.contributors, filterObj, filtersCollection, callback);
			return;
		}
		//creates a filter with the list of contributors
        var self = this;
        var serviceObj = {
            success: function(contributors) {
				self._doAddContributorFilter(contributors, filterObj, filtersCollection, callback);
            },
            failure: function(e) {
                self._error(e);
				callback();
            }
        };
        var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
        UserClient.getContributors(responseHandler);
    },
	
    _doAddContributorFilter: function(contributors, filterObj, filtersCollection, callback) {
        contributors = contributors || [];
		this.contributors = contributors;
		for (var i = 0; i < this.contributors.length; i++) {
            if (com.ibm.asq.common.web.ui.internal.util.Utility.isUnassignedUserId(contributors[i].itemId)){
				contributors[i].name = Constants.getMessage("labelUnassigned");
			}
            contributors[i].value = contributors[i].itemId;
            contributors[i].checked = false;
        }
        
        var contributorFilter = {
            name: filterObj.name,  // must match key in the arg list of the API
            title: filterObj.label,
            promptTitle: filterObj.prompt,
            type: filterObj.type,
            defaultValue: [],
            params: {options: contributors}
        };
        filtersCollection.push(contributorFilter);
        callback();		
	},
    
    addScriptTypeFilter: function(filterObj, filtersCollection, callback) {
        if (dojo.isArray(this.exeScriptTypes) && this.exeScriptTypes.length > 0) {
            this._doAddScriptTypeFilter(this.exeScriptTypes, filterObj, filtersCollection, callback);
            return;
        }
        
        //creates a filter with the list of script types
        var self = this;
        var serviceObj = {
            success: function(types) {
                self._doAddScriptTypeFilter(types, filterObj, filtersCollection, callback);
            },
            failure: function(e) {
                self._error(e);
                callback();
            }
        };
        var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
        RegistryClient.getAllExecutionScriptTypes(responseHandler);
    },
    
    _doAddScriptTypeFilter: function(types, filterObj, filtersCollection, callback) {
        types = types || [];
        this.exeScriptTypes = types;

        // filter out scriptless type
        var length = types.length;
        var scriptTypes = [];
        for (var i = 0; i < length; i++) {
            var type = types[i];
            var name = type.name;
            if (name == "Unknown" || name == "NoScript") { continue; }
            type.displayName = (name == DEFAULT_SCRIPT_TYPE) ? Constants.getMessage("labelManual") : 
                                                               Constants.getExtensionDisplayMessage(name);
            scriptTypes.push({
                name: type.displayName,
                value: type.id,
                checked: false
            });
        }
        
        // sort the entries
        length = scriptTypes.length;
        for (var ix = 0; ix < length; ix++ ) {
            for (var iy = ix + 1; iy < length; iy++ ) {
                if (scriptTypes[iy].displayName < scriptTypes[ix].displayName) {
                    var swap = scriptTypes[ix];
                    scriptTypes[ix] = scriptTypes[iy];
                    scriptTypes[iy] = swap;
                }
            }
        }

        var scriptTypesFilter = {
            name: filterObj.name,  // must match key in the arg list of the API
            title: filterObj.label ? filterObj.label : Constants.getMessage("labelScriptType") + Constants.getMessage("labelColen"),
            promptTitle: filterObj.prompt ? filterObj.prompt : Constants.getMessage("labelScriptType"),
            type: TableContentFilterProvider.FIELD_TOGGLELIST,
            defaultValue: null,
            params: {options: scriptTypes}
        };
        
        filtersCollection.push(scriptTypesFilter);
        callback();     
    },
    
    // Summary:
    //    Returns whether the current table has selected rows.
    hasSelectedRows: function()
    {
        var selectedItems = this.tableViewer.getCheckedRows();
        return selectedItems.length > 0;
    },
    
    // Summary:
    //    Main entry point for all selection based table actions. This method assembles all of the
    //    selected item's ids and invokes the actually function that carries out the
    //    action. This structure is necessary to uniformly handle the 'select all' 
    //    case in vitrual content (paging) tables. In that situation, the table does
    //    not have all the data (ids) yet and would require to fetch them before
    //    invoking the action.
    //      idType - the type of ids to obtained for the action (id, itemId, versionableItemId)
    //      context - the object of the funcName.
    //      funcName - the action function name to be invoked with the assembled ids. The funcName must be in the context specified.
    //      param1, param2 - additional params to be passed to the function identified by funcName.
    //
    invokeAction: function(idType, context, funcName, param1, param2){
        var selectedItems = this.tableViewer.getCheckedRows();
        var itemIds = [];
        var length = selectedItems.length;
        
        for (var i = 0; i < length; i++) {
            itemIds.push(selectedItems[i][idType]);
        }
        var actionFunc = dojo.hitch(context, funcName);
        actionFunc(itemIds, param1, param2);
    },
    
    // Summary:
    //    Main entry point for all table header actions. This method prepares the
    //    the list's selected item ids and calls the abstract method 'doBatchUpdate'
    //    Specific subclasses would perform artifact specific batch update, usually
    //    by creating a Model action to carry out the change.
    //
    headerActionInvoked: function(columnIndex, actionId)
    {
        var isRiskAction = (actionId == Constants.getMessage("labelRiskAssessment"));
        var idType = isRiskAction ? 'versionableItemId' : 'itemId';
        
        this.invokeAction(idType, this, "doBatchUpdate", columnIndex, actionId)
    },

    // Summary:
    //    Abstract method to actually performs the batch change operations. Specific
    //    subclassess need to override this method to do the work on the artifact type.
    doBatchUpdate: function(uids, columnIndex, actionId) 
    {
    },    
    
    _error: function(e) {
        if (this.isQueryPage) {
            this.viewHeaderNode.setStatus("Error: " + (e.message || "Unknown error"), ViewHeaderLayout.ERROR);
        }
        console.error(this.declaredClass + ": Error: " +(e.message || "Unknown error"));
    },
    
    // This path is used by artifacts which have been implemented in the new print engine.  Not all 
    // artifact tables have been switched to the new path so we need to maintain both this method 
    // and _onPrintpdfActionClick below for now.
    _onPrintActionClick: function(printType) {
        // summary print
        if( !this.hasSelectedRows() ) {
            var content = this.tableViewer.getContent();
            var len = content ? content.length : 0;
            
            if(len > 0) {
                var ids = [];
                for (var i = 0; i < len; i++)
                    ids.push(content[i].id);
                
                this._doPrintAction(ids, printType, null);
            }
        }
        else {
            // printing detail contents. Also need to handle select all case.
            this.invokeAction('id', this, "_doPrintAction", printType, null);
        }
    },

    // Summary:
    //    Base implementation for PDF print in the table. Subclass must define a 'pdfPrintType'
    //    attribute to use this method, or define its own to handle.
    // **deprecated** use _onPrintActionClick above
    _onPrintPdfActionClick: function(/*optional*/ isHistory) {
        var isSummary = false;
        
        // no rows selected. It's a summary of all contents. Need to get all of
        // ids to print the summary of each row.
        if (!this.hasSelectedRows()) {
            isSummary = true;
            var content = this.tableViewer.getContent();
            var len = content ? content.length : 0;
            
            if(len > 0) {
                var ids = [];
                for(var i = 0; i < len; i++) {
                    ids.push(content[i].id);
                }
                this._doPrintPdfAction(ids, isSummary, isHistory);
            }
        }
        else {
            // printing detail contents. Also need to handle select all case.
            this.invokeAction('id', this, "_doPrintPdfAction", isSummary, isHistory);
        }
    },
    
    // This path is used by artifacts which have been implemented in the new print engine.  Not all 
    // artifact tables have been switched to the new path so we need to maintain both this method 
    // and _doPrintAction below for now.
    _doPrintAction: function(ids, printType, /*this is not used*/dummyParam) {
        if( !this.pdfPrintType )
            console.error("Must supply a PDF print type, or override the _onPrintPdfActionClick method!");
        
        var hyperlink = window.location.href;
        hyperlink = hyperlink.replace(/[&]/g,"&amp;");
        
        this.tableViewer.sendPrintRequestByIds(ids, this.pdfPrintType, printType, "application/pdf", this._projectAlias, hyperlink);
    },
    
    // **deprecated** use _doPrintAction above
    _doPrintPdfAction: function(ids, summary, /*optional */isHistory) {
        if(!this.pdfPrintType) {
            console.error("Must supply a PDF print type, or override the _onPrintPdfActionClick method!");
        }
        var type = this.pdfPrintType;
        var loc = window.location.href;
        loc = loc.replace(/[&]/g,"&amp;");
        this.tableViewer.sendPdfPrintRequestByIds(ids, /* itemId not required */ null, type, loc, this._projectAlias, summary, null, isHistory);   
    },
    
    _showAlertDialog: function(message){
        var alertDialog = new AlertDialog({
            'message': message
        });
        alertDialog.setSecondaryTitle("");
        alertDialog.open();
    },
    
    _showArtifactChangedMessage: function(viewHeader, artifactType, changeType){
        var typeMsg = null;
                
        if (changeType == GlobalEvents.ArtifactsAdded) {
            typeMsg = Constants.getMessage("labelArtifactsAdded");
        }
        else if (changeType == GlobalEvents.ArtifactsUpdated) {
            typeMsg = Constants.getMessage("labelArtifactsModified");
        } 
        else if (changeType == GlobalEvents.ArtifactsDeleted) {
            typeMsg = Constants.getMessage("labelArtifactsDeleted");
        }
        if (typeMsg) {
            var msg = Constants.getMessage("msgRefreshModArtifact", artifactType, typeMsg);
            viewHeader.setStatus(msg);
        }
    }
});

/*
 * Common abstract class for paging content table views. Artifacts that supports
 * paging content should subclass from this base class.
 */
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredPagingTableView", 
             com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView,
{
    contentIsVirtual: true,
    currentPage: 0,
    
    // refresh by simply running the query with the current settings
    refreshQuery: function(){
        // This is a new query, reset the query token
        this.queryToken = null;
        this.inherited(arguments);
    },
    
    // @override AbstractFilteredTableView to process paging returned values.   
    _processResults: function(resultsDTO){
        var provider = this.provider;
        
        this.queryToken = resultsDTO.queryToken;
        provider.currentPageIndex = this.currentPage = resultsDTO.currentPage;
        provider.totalPageCount = this.totalPages = resultsDTO.totalPages;
        provider.totalContentCount = this.totalSize = resultsDTO.totalSize;
    },

    // @override AbstractFilteredTableView to insert paging arguments.
    _prepareQueryArgs: function(filterArgs, updatePageOnly) {
        if(filterArgs) {
            // non-paging related query, has provided new
            // filters. Make this a new query. Our filter provider
            // could have cleared the token for us. But in case
            // it didn't, always clear the token here.
            this.queryToken = null;
        }
        var args = this.inherited("_prepareQueryArgs", arguments);

        args = dojo.clone(args);
        
        // we're still using the same query. Don't have to add in any
        // sort arguments.
        if (updatePageOnly && this.queryToken) {
            this.updatePageOnly = true;
            args["queryToken"] = this.queryToken;
            args["page"] = this.currentPage;
        }
        else {
            if (this._keepCurrentPage) {
                args["page"] = this.currentPage ? this.currentPage : 0;
                this._keepCurrentPage = false;
            }
            else {
                args["page"] = 0;
            }
            var sortArgs = this.provider.buildGroupSortServerArgs();
            if (sortArgs.length > 0) {
                args['sortColumns'] = sortArgs;
            }
            
            // apply text filter if one is specified.
            var textFilter = this.provider.getTextFilter();
            if (textFilter && textFilter.length > 0) {
                args["textSearch"] = textFilter;
            }
        }
        args["pageSize"] = this.provider.pageSize;
        
        // guard against getting 'all items' when we are
        // resetting the query. The user could have switched to
        // show all for a small query and reset the query. 
        // A new query would be indicated by the queryToken
        // being null.
        if(args.pageSize == -1 && this.queryToken == null) {
        	// WI 32649: User does not want to lose the Select All setting when 
        	// filtering, sorting or refreshing the viewer. In order to prevent
        	// performance issues, instead of Show All it will return the MAX page
        	// size.
        	if(this.tableViewer.sizeSelectedByUser) {
	            this.tableViewer.setContentPageSize(TableConstants.MAX_PAGE_SIZE);
	            args.pageSize = TableConstants.MAX_PAGE_SIZE;
        	} else {
	        	this.tableViewer.resetPageSize();
	            args.pageSize = TableConstants.MIN_PAGE_SIZE;
        	}
        }
        // added paging args.
        return args;
    },
    
    // @override AbstractFilteredTableView.
    _prepareTableUIForQuery: function() {
        // no need to prepare the table UI, if this is coming from the
        // paging contents. Otherwise, if this is a new query,
        // clear the display.
        if(this.queryToken == null && this.provider.preLoadPageContent)
            this.provider.preLoadPageContent();
        
    },

    
    // @override AbstractFilteredTableView to handle virtual select all case.
    hasSelectedRows: function()
    {
        var selectedItems = this.tableViewer.getCheckedRows();
        return (selectedItems == TableConstants.VIRTUAL_CONTENT_SELECT_ALL || selectedItems.length > 0);
    },

    _removeIdFromList: function(idType, list, id, compareFunc){
        var len = list.length;
        
        for (var i = 0; i < len; i++) {
            if(compareFunc(idType, list[i], id)) {
                list.splice(i, 1);
                return;
            }
        }
    },
    
    // Removes deselected rows from the 'all' item list. Caller must pass in
    // compare function to match the removed item. The returned list can be
    // the array of actual content objects, or just an array of ids.
    //
    _removeDeselectedItems: function(idType, allItems, compareFunc)
    {
        var deselectedItems = this.tableViewer.getVirtualDeselectedRows();
        var dlen = deselectedItems.length;
        if(dlen > 0) {
            for(var i = 0; i < dlen; i++) {
                var removeId = deselectedItems[i][idType];
                if(removeId) {
                    this._removeIdFromList(idType, allItems, removeId, compareFunc);
                }
            }
        }
        return allItems;
    },
    
    // Method to return selected all virtual contents' records. Upon returned
    // filter out any deselected items from the list. The user might have done a select
    // all and then deselect items before invoking actions.
    //
    getSelectedAllVirtualResultContents: function(idType, callback)
    {
        if (this.queryToken) {
            var args = this._prepareQueryArgs(null, true);
            
            // reset the page arguments to fetch from top and
            // to fetch all.
            args.page = 0;
            args.pageSize = -1;
            
            var handler = {
                self: this,
                'success': function(resultsDTO) {
                    // HACK: to workaround a irregular return result from TER search...
                    if(dojo.isArray(resultsDTO)) {
                        resultsDTO = resultsDTO[0];
                    }
                    // remove any delected items during a select all.
                    var contents = this.self._removeDeselectedItems(idType, resultsDTO.results, function(idType, row, id) {
                        return (row[idType] == id);
                    });
                    callback(contents);
                },
                'error': function() {
                    this.self._showAlertDialog(Constants.getMessage("msgErrorFetchAll"));
                    callback([]);
                }
            }
            var responseHandler = new this._searchResponseHandlerType(handler, "success", "error");
            this._searchFunction(responseHandler, args);
        }
    },
    
    // Method to return selected all virtual contents' item ids. Upon returned
    // filter out any deselected items from the list. The user might have done a select
    // all and then deselect items before invoking actions.
    //
    getSelectedAllVirtualContentItemIds: function(idType, context, funcName, param1, param2)
    {
        var handler = {
            context: context,
            self: this,
            idFieldName: (idType == "id") ? "ids" : "uuids",
            success: function(results){
                // remove any delected items during a select all.
                var allIds = this.self._removeDeselectedItems(idType, results[this.idFieldName], function(idType, idRec, id) {
                    return idRec == id;
                });
                var actionFunc = dojo.hitch(this.context, funcName);
                actionFunc(allIds, param1, param2);
            },
            failure: function(){
                this.self._showAlertDialog(Constants.getMessage("msgErrorFetchAll"));                
            }
        }
        var responseHandler = new ServiceResponseHandler(handler, "success", "failure");
        this._getAllResultsItemIds(responseHandler, {
            'queryToken': this.queryToken,
            'kindOfHandles': idType
        });
    },
    
    // @override AbstractFilteredTableView to handle paged content select all token
    invokeAction: function(idType, context, funcName, param1, param2)
    {
        var selectedItems = this.tableViewer.getCheckedRows();
        
        // This is a select all case, get all content's ids from the server.
        if (selectedItems == TableConstants.VIRTUAL_CONTENT_SELECT_ALL) {
            this.getSelectedAllVirtualContentItemIds(idType, context, funcName, param1, param2);
        }
        else {
            var itemIds = [];
            var length = selectedItems.length;
            
            for (var i = 0; i < length; i++) {
                itemIds.push(selectedItems[i][idType]);
            }
            var actionFunc = dojo.hitch(context, funcName);
            actionFunc(itemIds, param1, param2);
        }
    },

    // abstract method to be used to get the all results' item ids. Subclasses must
    // override. E.g., this._getAllResultsItemIds = dojo.hitch(TestCaseClient, "getAllSearchResultItemIds");
    //
    _getAllResultsItemIds: function(responseHandler, params) {
        console.error("No client method provided for: _getAllResultsItemIds");
    },
    
    // @override - to handle virtual select all.
    // This path is used by the new print engine.  Not all artifact tables support this path yet so we
    // need to maintain both this method and _onPrintPdfActionClick below for the time being.
    _onPrintActionClick: function(printType) {
        // summary print
        if( !this.hasSelectedRows() ) {
            var content = this.tableViewer.getContent();
            var len = content ? content.length : 0;
            
            // the content is virtual and we don't have all the content
            // get the itemIds to invoke pdf print.
            if (this.contentIsVirtual && this.totalSize > len)
                this.getSelectedAllVirtualContentItemIds('id', this, "_doPrintAction", printType);
            else if(len > 0) {
                // just print the content that we're showing.
                var ids = [];
                for(var i = 0; i < len; i++) {
                    ids.push(content[i].id);
                }
                this._doPrintAction(ids, printType, null);
            }
            
        }
        else {
            // printing detail contents. Also need to handle select all case.
            this.invokeAction('id', this, "_doPrintAction", printType, null);
        }
    },
    
    // @Override - to handle virtual select all.
    _onPrintPdfActionClick: function(/*optional*/ isHistory) {
        var isSummary = false;
        
        // no rows selected. It's a summary of all contents. Need to get all of
        // ids to print the summary of each row.
        if (!this.hasSelectedRows()) {
            isSummary = true;
            var content = this.tableViewer.getContent();
            var len = content ? content.length : 0;
            
            // the content is virtual and we don't have all the content
            // get the itemIds to invoke pdf print.
            if(this.contentIsVirtual && this.totalSize > len)
                this.getSelectedAllVirtualContentItemIds('id', this, "_doPrintPdfAction", isSummary, isHistory);
            else if(len > 0) {
                // just print the content that we're showing.
                var ids = [];
                for(var i = 0; i < len; i++) {
                    ids.push(content[i].id);
                }
                this._doPrintPdfAction(ids, isSummary, isHistory);
            }
            
        }
        else {
            // printing detail contents. Also need to handle select all case.
            this.invokeAction('id', this, "_doPrintPdfAction", isSummary, isHistory);
        }
    },
    
    // Summary:
    //    Returns selected rows from the table. If the selection is a select all
    //    and we are showing virtual content, this method will fetch the entire content
    //    to return. 
    //      idType - the id attribute to use to filter out deselected items in a select all context (id, itemId, etc.)
    //      callback - method to receive the returned item. The callback is called with the returned items: callback(items)
    // 
    getVirtualSelectedRows: function(idType, callback) {
        var selectrows = this.tableViewer.getCheckedRows();
        
        // not a select all, just return the rows
        // right away.
        if(dojo.isArray(selectrows)) {
            callback(selectrows);
        }
        else {
            // It's a virtual select all. Get all results contents.
            // It will remove any rows that have been deselected from
            // the select all set.
            this.getSelectedAllVirtualResultContents(idType, callback);
        }            
    },
    
    // Summary:
    //   convenient method to rerun the current query to pickup any content
    //   changes.
    _invalidateQuery: function () {
        // clear the token to invalidate last ran query
        this.queryToken = null;
        this._keepCurrentPage = true;
        this.refreshQuery(null);
    }
});

})();
