/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.table.ConfigurationTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ConfigurationCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants  	   = com.ibm.rqm.planning.web.client.internal.Constants;
var ConfigurationUriUtil = com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil;


dojo.declare("ConfigurationTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	constructor: function(parentView) {
        this.parentView = parentView;
    },
	
	setRowCheckListener: function(listener) {
		this.rowCheckListener = listener;
	},
	
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
      var selectionCount = tableViewer.getCheckedRows().length;
	  if(this.parentView.deleteActionNode)
	 		 this.parentView.deleteActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
	  if(this.parentView.oncheckListener)
	  		 this.parentView.oncheckListener.method.apply(this.parentView.oncheckListener.context, [tableViewer, rowObject, state]);   
    },
	
	// Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {
	  if (callback != null && dojo.isFunction(callback)) {
         callback();
      } 
    }
});

dojo.declare("ConfigurationTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {       
        // store the new values in our internal data model        
		  if(columnIndex == 5) {
//           rowObject.selectedScriptId = newValue;
        }
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		//handle delete action
		if (columnIndex == 5) {
		   //alert("delete clicked for TestCase: " + rowObject.id);
		}
    }
});

dojo.declare("ConfigurationTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
    }
});

dojo.declare("ConfigurationTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
	// MFV Changed on 09/05
	// If the content we are displaying contains config objects then show the name of the config as a link in the first column
	// If the content we are displaying DOES NOT contain config object then just show the columns from the catalog
    // Summary:
    //
	
	constructor: function(args){
		this.showName = false;
		this.inWizard = args.inWizard;
	},
	
    getNumColumns: function(content)
    {
		if(!this.catalog){
			this.catalog = new com.ibm.asq.common.web.client.internal.Catalog();
		}
        var attributeIds = this.catalog.getDisplayTypes();
        var count = attributeIds.length;
		
		//If we want to show the name then add one to the column count
		if (this._showName(content)){
			count++;
			this.showName = true;
		}
		return count;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
    	//Return "Name" if we are showing the name column
		if (this._showName(content) && index == 0)
			return(Constants.getMessage("labelName"));
			
		if(!this.catalog){
			this.catalog = new com.ibm.asq.common.web.client.internal.Catalog();
		}
        var headings = this.catalog.getDisplayTypes();
		
		//Use the appropriate index depending on whether we are showing the header
		if (this._showName(content))
	        return Constants.getLmMessage(headings[index-1].name);
		else
	        return Constants.getLmMessage(headings[index].name);
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
		//Don't bother grouping by the name field
		if (this._showName(content))
	        return (atColumnIndex > 0);
		else
			return true;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return (true);   
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        var nCol = this.getNumColumns(content);
		var width = Math.floor(100 / nCol);
        var remainder = 100 % nCol;
        
        // spread the remainder over the first % columns
        return width + ((atColumnIndex < remainder) ? 1 : 0);
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
		var rowDataId = "cfg_";
		if(rowData.length > 0){
			for (var i = 0; i < rowData.length; i++){
				rowDataId += rowData[i].value;
			}
		}
		if (rowData.config && rowData.config.id){
			rowDataId += rowData.config.id;
		}
        return rowDataId;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
		if (atColumnIndex == 0 && !this.inWizard && this._showName(rowData)){
			return TableConstants.RENDER_TYPE_LINK;
		}else 
			return TableConstants.RENDER_TYPE_TEXT;
		
    },

    getCellAlignment: function(rowData, atColumnIndex){
        return TableConstants.ALIGN_LEFT;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
		var showDataName = this._showName(rowData);
		var showName = this.showName;
		if (showName && showDataName && atColumnIndex == 0) 
		{
			if (rowData.config.name) 
				return rowData.config.name;
			else 
				return Constants.getMessage("labelUndefined");
		}
		else if (showName && !showDataName && atColumnIndex == 0) 
		{
			return Constants.getMessage("labelUndefined");
		} 
		else
		{
			var typeId = this.getTypeId(rowData, atColumnIndex);
			for (var i = 0; i < rowData.length; i++) 
			{
                // the rowData's key is the env type encoded object:
                // {
                //    id: <the id of the whitch object>
                //    name: <name of the catalog type>,
                //    path: <the attribute path of which type appears in a config
                //    template: <the catalog template name>
                //  }
                var key = rowData[i].key.id;
				if (key == typeId) 
				{
					return rowData[i].value;
				}
				if (i == rowData.length - 1) 
				{
					return Constants.getMessage("labelNotConfig");
				}
			}
		}
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {
		var val = null;
        if(this._showName(rowData) && atColumnIndex == 0 && !this.inWizard) {
            val = {};
            val['href'] = ConfigurationUriUtil.getEditConfigurationURI( rowData.config.id );
        }

        return val;
    },
	
	// Determine if we should be showing the "Name" column which will contain the name of the configuration
	// We want to show the name if there is a Config object associated with the data in the table.
	// This method is called with two different types of data
	//  1.  The entire content of the table in which case we check for a Config object in the 0th entry in the data
	//  2.  Row data for one row, in which we check for a Config oject in that data
	_showName: function(content)
	{
		//If the content is not null and either
		//The content is an array and the 0th index in the array contains a config object
		//or the content itself contains a cofig object
		var retval = (content != null && ((content[0] != null && content[0].config != null) || content.config != null));
		return retval;
	},
	
	// Summary:
	//   This function rerurn the type id of a Catalog type. The type id is the UUID of the WHITCH object. 
	getTypeId: function(rowData, index)
	{
		if(!this.catalog){
			this.catalog = new com.ibm.asq.common.web.client.internal.Catalog();
		}
        var types = this.catalog.getDisplayTypes();
		
		//Use the appropriate index depending on whether we are showing the header
		if (this.showName)
	        return types[index-1].id;
		else
	        return types[index].id;
	}
    
    
});

dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.table.ConfigurationTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" tabindex="-1"></div>',

    editable: true,
	printable: false,
    allowSettings: true,
    tableId: "RQM-CONFIG-TABLEVIWER",
    inWizard:false,
    constructor: function() {
	 	this.results = null;
		this.queryName = null;
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        //var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer(); 
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            //turn off all controls if in print mode             
            showPagingControl: !this.printable,
            showPagingRangeControl: !this.printable,
            showGroupFilterControls: !this.printable,
            showPagingActionsControls: !this.printable,
            allowAdjustSettings: this.allowSettings,
            autoSaveSettings: this.allowSettings && this.tableId,
            tableId: this.tableId
        }); 		       
        this._tableViewerNode.appendChild(tableViewer.domNode);
		          
        var content = this.configs ? this.configs : [];
		  
        tableViewer.setContentPageSize(10);
        tableViewer.setContent(content);
        tableViewer.setContentProvider(new ConfigurationTableView.TableProvider({inWizard:this.inWizard}));
		tableViewer.setTableListener(new ConfigurationTableView.TableListener(this));
        tableViewer.setCellModifier(new ConfigurationTableView.CellModifier());
		tableViewer.setTableRowsCheckable((this.editable && !this.printable), /* isSingleSelect */ false);
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelGeneratedTE")));

		//add actions to the table viewer
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            var action = this.actions[i];
			if( action.label == Constants.getMessage("labelDeleteConfigurations") ){
				this.deleteActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
				this.deleteActionNode.setDisabled(true);
			}else{
				tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
			}
        }
		
        if (this.printable) {
            tableViewer.setContentPageSize(-1);
        }		       
        tableViewer.show();		  
		
		this.tableViewer = tableViewer;
		this.tableViewer.setEditable(this.editable && !this.printable);
    },
	
	//API
	getTableViewer: function(){
		return this.tableViewer;
	},
	
	// API
	removeSelectedConfigs: function(){
		var selectedConfig = this.getSelectedConfigs();
		if(selectedConfig.length == 0){
			alert(Constants.getMessage("msgSelectTestEnvironments"));
		}
		for(var j = 0; j < selectedConfig.length; j++){
			this.tableViewer.deleteItem(selectedConfig[j]);
		}
		var content = this.tableViewer.getContent();
		this.tableViewer.setContent(content);
		this.tableViewer.show();
	},
	 
	// API
	getSelectedConfigs: function() {
	  return this.tableViewer.getCheckedRows();
	},
	
	 
    refresh: function(configs) {
      this.tableViewer.setContent(configs);
      this.tableViewer.show();
    },

    appendConfigs: function(configs) {
      var content = this.tableViewer.getContent();
      for (var i = 0; i < configs.length; i++) {
         content.push(configs[i]);  
      }     
      this.tableViewer.setContent(content);
      this.tableViewer.show();
    }	 
});

})();
