/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableHeaderRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GridTableHeaderRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var TOP_BORDER_CSS = "table-header-top-border-line";
var SelectUtility  = com.ibm.asq.common.web.ui.internal.util.SelectUtility;
var NoSort_Text    = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_nosort");

dojo.declare("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableHeaderRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GridTableHeaderRenderer,
{
	columnTypeEditors: [],
	
    createTableHeader: function(table)
    {
        
        // not show table heading. 
        if(!this.viewer.getIsShowingTableHeading())
            return;
                    
        var provider = this.viewer.getContentProvider();
        var content = this.viewer.getContent();
        var nColumns = provider.getNumColumns(content);
        var thead = table.createTHead();
        var headerTopTr, headerBotTr;
        var headerContentTr;
		var headerSortTr = null;                          
        var groupBy = this.viewer.getGroupByColumn();
        var atCol = 0;
        var slackWidth = 0;
        var nVisible = nColumns;
        var reduceWidth = 0;
        var tableHeaderStyle = "table-header-content";
		var doc = document;
        
        var totalWidth = 0;                
        var widthLeft = 0;
        // set table header style for CJK pictographic based locales
        if (dojo.locale.indexOf("zh") == 0 ||
            dojo.locale.indexOf("ja") == 0 ||
            dojo.locale.indexOf("ko") == 0) {
            tableHeaderStyle = "table-header-content-cjk";
        }

        // reset the header data
        this.columnActions = [];
        this.tableHeaders = {};
        
        // multi column sorting data.
        this.sortColCtrls = {};
        this.sortColumns = [];
        
        this.sortByColumn = this.viewer.getSortByColumn();
        this.sortDir = this.viewer.getSortDirection();

        // multi-column sorting. Get the array of sort columns.
        if (this.viewer.allowMultiSort) {
            this.sortColumns = this.viewer.getMultiColumSortInfo();
        }
                
        for (var i = 0; i < nColumns; i++) {
            if (!this.viewer.isColumnVisible(i) || (groupBy == i)) {
                nVisible--;
            }
            else {
                totalWidth += this.viewer.getColumnWidth(i);
            }
        }        
        if(nVisible == 0)
            return;
        
        widthLeft = 100 - totalWidth;   
        
        // we're going to remove the group column,
        // distribute its width amongst all columns
        if (groupBy > -1 && this.viewer.isColumnVisible(groupBy)) {
            slackWidth = Math.floor(this.viewer.getColumnWidth(groupBy) / nVisible);
        }    
        
		// create the header rows, top & bottom border rows
		// and content & sort rows in the middle.
        headerTopTr = table.insertRow(table.rows.length); 
        headerContentTr = table.insertRow(table.rows.length); 
        headerBotTr = table.insertRow(table.rows.length); 
        
        // The rows in the table are checkable, insert
        // a column to provide a check box for the rows
        if(this.viewer.getTableRowsCheckable()) {
            this.createHeaderBorderLine(table, headerTopTr, true);
            
            var checkbox_th = doc.createElement("th");
            dojo.addClass(checkbox_th, tableHeaderStyle);
            
            // create a check box to select/deselect all rows in the table.
            if (!this.viewer.getIsSingleSelectionMode() && this.viewer.getIsAllowSelectAll()) {
                dojo.addClass(checkbox_th, "align-center");
                checkbox_th.vAlign = "center";
                checkbox_th.title = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_selectAll");
                
                var checkState = (this.checkAllState) ? true : false;
                var button = new dijit.form.CheckBox({
                    'checked': checkState
                });
                button.headerRenderer = this;
                button.onChange = function() {
                    this.headerRenderer.checkAllState = this.checked;
                    this.headerRenderer.viewer.performSelectAll(this.checked);
                };
                checkbox_th.appendChild(button.domNode);
            }
            else {
                checkbox_th.width = 20;
            }                
            headerContentTr.appendChild(checkbox_th);
			if(headerSortTr) {
				checkbox_th = doc.createElement("th");
				dojo.addClass(checkbox_th, tableHeaderStyle);
				dojo.addClass(checkbox_th, TOP_BORDER_CSS);
				headerSortTr.appendChild(checkbox_th);
			}
            this.createHeaderBorderLine(table, headerBotTr);
        }
        
        // if there's a group by, insert a filler column for the
        // indentation.
        if(groupBy > -1) {            
            this.createHeaderBorderLine(table, headerTopTr, true);
            var gr_th = doc.createElement("th");
            dojo.addClass(gr_th, tableHeaderStyle);
            headerContentTr.appendChild(gr_th);
            gr_th.width = 8;            

            if(headerSortTr) {
				gr_th = doc.createElement("th");
                dojo.addClass(gr_th, tableHeaderStyle);
                dojo.addClass(gr_th, TOP_BORDER_CSS);
                headerSortTr.appendChild(gr_th);
            }
            this.createHeaderBorderLine(table, headerBotTr);			
        }
        // Fix for 23461
		this.columnTypeEditors = [];
        for(var i = 0; i < nColumns; i++) {
            // skip displaying the group by column and non-visible columns.
            if(groupBy == i || !this.viewer.isColumnVisible(i)) {
                continue;
            }
            this.createHeaderBorderLine(table, headerTopTr, true);
            
            var title = provider.getColumnHeading(content, i);
            var th = doc.createElement("th");
             
            dojo.addClass(th, tableHeaderStyle);    
            var currentWidth = this.viewer.getColumnWidth(i);        
            var addedPercentage=0;
            if(totalWidth < 100 && totalWidth != 0)
                addedPercentage=Math.floor((currentWidth/totalWidth)*widthLeft);
                                
            th.width = (currentWidth + slackWidth - reduceWidth + addedPercentage) + "%";	
	        th.vAlign = "center";
            var align = provider.getColumnHeadingAlignment(i);
           
            if(align == TableConstants.ALIGN_CENTER) {
               dojo.addClass(th, "align-center");
            } 
            else if(align == TableConstants.ALIGN_RIGHT) {
               dojo.addClass(th, "align-right");   
            } 
            else {
                dojo.addClass(th, "align-left")
            }
				            
            if (provider.getIsColumnSortable(content, i)) {
                var anchor = doc.createElement("a");
                var titleText = (title && title.length > 0) ? title : "\u00a0";
                var colIsSort = this.columnHasSort(i);
                
                anchor.appendChild(doc.createTextNode(titleText));
                anchor.colIndex = i;
                anchor.name = "anchor";
                anchor.className = "table-link";

                th.appendChild(anchor);
                this.headerConnect(anchor, "onclick", "_handleTableHeadSortBy");
                anchor.title = NoSort_Text;            

                // Create the sort indicator for the column
                this._createSortControls(th, i);
            } 
            else if (title != null && dojo.trim(title).length > 0){
                th.appendChild(doc.createTextNode(title));
            }
            else {
                var spacer = doc.createElement('span');
                spacer.style.width = '8px';
                th.appendChild(spacer);
            }
                        
            // if the column header has a image, create it.
            var imgRef = provider.getColumnHeadingImage(content, i);
            if(imgRef) {
                var src = (dojo.isObject(imgRef) && imgRef.src) ? imgRef.src : imgRef;
                var isAction = (dojo.isObject(imgRef) && imgRef.actionId) ? true : false;
                
                // create a margin node if we already have
                // something created before the image.
                if(th.childNodes && th.childNodes.length > 0) {
                    var spacer = doc.createElement("span");
                    spacer.className = "table-head-spacer";
                    th.appendChild(spacer);                    
                }
                
                var img = doc.createElement("img");
                img.className = "table-head-image";
                img.src = src;
                th.appendChild(img);
                
                if (isAction) {
                    this.columnActions.push(img);

                    img.colIndex = i;
                    img.name = imgRef.actionId;
                    if(imgRef.tooltip) {
                        img.title = imgRef.tooltip;
                    }
                    dojo.addClass(img, "table-head-image-action");
                    this.headerConnect(img, "onclick", "_handleColumnActionClicked");
                }
            }
			
			// 23461
			// create the column type editor
			
			var editor = document.createElement("select");
			SelectUtility.add(editor, new Option("STRING", 0));
			SelectUtility.add(editor, new Option("NUMBER", 1));
			SelectUtility.add(editor, new Option("BOOLEAN", 2));
			SelectUtility.add(editor, new Option("ENUMERATION", 3));
			editor.style.display = "block";
			th.appendChild(editor);
			this.columnTypeEditors.push(editor);
			editor.style.display = "none";

            headerContentTr.appendChild(th);
			
            // We're in multi-columns sort mode. Create the sort indicator
            // header for this sortable column.
            if(headerSortTr) {
                var sth = doc.createElement("th");
                dojo.addClass(sth, tableHeaderStyle);
                dojo.addClass(sth, TOP_BORDER_CSS);
                
                // create the sort control for this column in the
                // sort header.
				if (provider.getIsColumnSortable(content, i)) {
					this._createSortControls(sth, i);
				}
                headerSortTr.appendChild(sth);
            }   
            this.createHeaderBorderLine(table, headerBotTr);
            
            // store the column header in our associated array indexed
            // by the column position string key.
            this.tableHeaders[this._getTableHeadKey(i)] = {'thead': th};
        }
		
        thead.appendChild(headerTopTr);
        thead.appendChild(headerContentTr);    
        if (headerSortTr) {
            thead.appendChild(headerSortTr);    
        }
        thead.appendChild(headerBotTr);    
        this.thead = thead;
          
        table.appendChild(thead);    
        this.viewer.contentRowStart = table.rows.length;
        
        // initialize the column's action states.
        this.setColumnActionEnablement();    
        
        // initialize the sort indicators.
        this._initSortIndicators(groupBy);
    }
});


})();
