/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.SearchResultTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.RichHoverCellRenderer");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.WorkItemClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");




(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var WorkItemClient = com.ibm.rqm.planning.web.client.internal.WorkItemClient;
var RENDER_SEARCH_IMAGE = "RenderType.SearchImage";
var RichHoverCellRenderer = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.RichHoverCellRenderer;

dojo.declare("SearchResultTableView.ImageCellRender", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer,
{
	typeCache: {},
	
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/ cellModifier, rowObject, columnIndex){
		var img = document.createElement("img");
		var tooltip = cellData;
		var id = null;
		
		if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
			var imgSrc = (typeof extendedDisplayInfo['mouseOutImage'] != "undefined") ? extendedDisplayInfo['mouseOutImage'] : extendedDisplayInfo['image'];
			img.alt = cellData;
			img.src = imgSrc;
			img.extendData = extendedDisplayInfo;
			
			if (extendedDisplayInfo['tooltip']) 
				tooltip = extendedDisplayInfo['tooltip'];
		}
		else {
			img.src = cellData;
		}
		viewer.connect(img, "onmouseover", dojo.hitch(this, "overImage"));
		viewer.connect(img, "onmouseout", dojo.hitch(this, "outOfImage"));
		
		var dojoTooltip = this._attachTooltipToCell(img, tooltip);
		
		if (extendedDisplayInfo && typeof extendedDisplayInfo['href'] != "undefined") {
			var anchor = document.createElement("a");
			anchor.href = extendedDisplayInfo.href;
			anchor.appendChild(img);
			anchor.tooltip = dojoTooltip;
			return anchor;
		}
		img.tooltip = dojoTooltip;
		
		if (extendedDisplayInfo['id']) {
			id = extendedDisplayInfo['id'];
			this.setWorkItemType(id, tooltip, img);
		}
		return img;
	},
	
	setWorkItemType: function(id, tooltip, img) {
		if (this.typeCache[id]) {
			this.addIconInfo(this.typeCache[id], img);
			return;
		}
		var self = this;
		var handlerObj = {
			success: function(workItemDTO){
				self.typeCache[id] = {
					id: id,
					label: (workItemDTO) ? workItemDTO.workItemType.label : Constants.getMessage("labelWorkitem"),
					iconURL: (workItemDTO ? workItemDTO.workItemType.iconUrl : ImageCache.WORK_ITEM)
				}
                self.addIconInfo(self.typeCache[id], img);	
			},
			error: function(){
				console.error("Unable to retrieve WorkItemDTO for WorkItem: " + self.workItem.id);
			}
		}
		var responseHandler = new ServiceResponseHandler(handlerObj, "success", "error");
		WorkItemClient.getWorkItemDTO(responseHandler, {id: id});
	},
	
	addIconInfo: function(typeObj, img) {
        img.src = typeObj.iconURL || "";
        img.tooltip = this._attachTooltipToCell(img, (typeObj.label ?typeObj.label: ""));		
	}

});

dojo.declare("SearchResultTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        //alert("Row check state changed: " + rowObject.col1);        
    },
	
	// Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {                
        if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        } 
    }
});

dojo.declare("SearchResultTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        alert(Constants.getMessage("labelCellValueChanged",rowObject.col1,newValue));
        
        // store the new values in our internal data model
        if(columnIndex == 6) {
            if(dojo.isString(newValue)) {
                rowObject.selection = newValue;
            }
        }
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		//handle delete action
		if (columnIndex == 6) {
		   alert(Constants.getMessage("labelNotYetSupported"));
		}
    }
});

dojo.declare("SearchResultTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
        
        var moreContents = document.createElement("div");
        var toggle = document.createElement("button");
        toggle.innerHTML = Constants.getMessage("labelAddedCustomControl");
        moreContents.appendChild(toggle);
        listWidget.contentNode.appendChild(moreContents);
    }
});

dojo.declare("SearchResultTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    // Summary:
    //
    getNumColumns: function(content)
    {
        return 4;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        var headings = [
            Constants.getMessage("labelScorePlanningBody")+" (%)",
			Constants.getMessage("labelType"),
			Constants.getMessage("labelId"),
            Constants.getMessage("labelName")
        ];
        return headings[index];
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
	 	  
        if(atColumnIndex == 6) {
            return ImageCache.REMOVE_ENABLED;
        }
		  /*
        if(atColumnIndex == 4) {
            return ImageCache.REMOVE_ENABLED;
        }
        */		  
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return (atColumnIndex == 1);     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return true;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        switch(atColumnIndex) {
            case 0: 
                return 5;
            case 1: 
                return 5;
            case 2: 
                return 10;
            case 3: 
                return 80;
        }
        return 0;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return rowData.type+"_"+rowData.id;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = TableConstants.RENDER_TYPE_TEXT;

        switch(atColumnIndex) {
            case 1: 
                type = RENDER_SEARCH_IMAGE;
                break;
            case 2: 
			case 3:
				if (this.isCalmLink(rowData.type)) {
					type = TableConstants.RENDER_TYPE_RICH_HOVER;
				} else {
                	type = TableConstants.RENDER_TYPE_LINK;
				}
                break;
			default:
                type = TableConstants.RENDER_TYPE_TEXT;
				break;
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        if (atColumnIndex == 0 || atColumnIndex == 1 || atColumnIndex == 2) {
			return TableConstants.ALIGN_CENTER;
		}
        return TableConstants.ALIGN_LEFT;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
		var val = "data";
        switch(atColumnIndex) {
            case 0: 
                var score = new Number(rowData.score);
                var tmpVal = score / 1000;
                val = new Number(tmpVal.toFixed()).toString(); 
                break;
            case 1: 
                val = Utility.getDisplayNameForType(rowData.type);
                break;
            case 2: 
                val = dojo.trim(rowData.id);
                break;
            case 3: 
                val = dojo.trim(rowData.name);
                break;
        }
        return val;
    },

    getCellRawData: function(rowData, atColumnIndex)
    {
        // sort these columns by its numeric value when possible.
        if (atColumnIndex == 0) {
            // score is a number already, don't have to convert it to
            // pretty displayable value for sort.
            return rowData.score;
        }
        if(atColumnIndex == 2) {
            if(!isNaN(rowData.id)) {
                return parseInt(rowData.id);
            }
            return -1; // non-numeric id. sort them first.
        }
        return this.getCellDisplayData(rowData, atColumnIndex);
    },

    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  	    
        var val = null;
        if (atColumnIndex == 1) {
            val = {};
            val.tooltip = Utility.getDisplayNameForType(rowData.type);
			val.image = Utility.getArtifactIconURL(rowData.type);
			if (!val.image && val.image == "") {
				val.image = ImageCache.TEST_PLAN;
			}
			if (rowData.type == Constants.MODEL_TYPE_WORKITEM) {
				val.id = dojo.trim(rowData.id);
			}
        } else if(atColumnIndex == 2 || atColumnIndex == 3) {
			val = {};
			if (this.isCalmLink(rowData.type)) {
				val.uri = rowData.uri;
                val.presentationUri = rowData.uri;
                val.label = rowData.name;
			}
			else {
				val.href = this.getURI(rowData.type, rowData.id, rowData.details);
				if (atColumnIndex == 2) 
					val.linkType = Constants.getMessage("WorkItemID");
			}
        }
        
        return val;			
    },

	getURI: function(type, id, optData) {
		switch(type) {
			case Constants.MODEL_TYPE_PLAN: return Utility.getTestPlanURI({id: id});
			case Constants.MODEL_TYPE_CASE: return Utility.getTestCaseURI(id);
			case Constants.MODEL_TYPE_SCRIPT: return Utility.getTestScriptURI(id);
			case Constants.MODEL_TYPE_REMOTE_SCRIPT: return Utility.getTestScriptURI(id);
			case Constants.MODEL_TYPE_SUITE: return Utility.getTestSuiteURI(id);
			case Constants.MODEL_TYPE_WORKITEM: return Utility.getViewWorkItemURI(id);
			case Constants.MODEL_TYPE_EWI: return Utility.getEwiURI(id);
			case Constants.MODEL_TYPE_RESULT: return Utility.getResultURI({id: id});
			case Constants.MODEL_TYPE_BUILD_DEFINITION: return Utility.getBuildDefinitionURI(id);
			case Constants.MODEL_TYPE_BUILD_RECORD: return Utility.getBuildRecordURI(id);
			case Constants.MODEL_TYPE_DATAPOOL: return Utility.getViewDatapoolURI(id);
			case Constants.MODEL_TYPE_KEYWORD: return Utility.getKeywordsURI();
			case Constants.MODEL_TYPE_LAB_REQUEST: return Utility.getRequestURI(id);
			case Constants.MODEL_TYPE_TEST_CELL: return Utility.getTestCellURI(id);
            case Constants.MODEL_TYPE_TEST_ENVIRONMENT: return Utility.getTestEnvironmentURI(id);
            case Constants.MODEL_TYPE_RESOURCE_GROUP: return Utility.getResourceGroupURI(id);            
            case Constants.MODEL_TYPE_EXECUTION_SEQUENCE: return Utility.getExecutionScheduleURI(id);
            case Constants.MODEL_TYPE_LAB_RESOURCE: return Utility.getLabResourceURI(optData);            
			default: return "#";
		} 
	},
	
	isCalmLink: function(type) {
		return type == Constants.CALM_LINK_TYPE_PLAN_ITEM || type == Constants.CALM_LINK_TYPE_REQUIREMENT || type == Constants.CALM_LINK_TYPE_REQUIREMENT_SET;
	}
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.SearchResultTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" isCompact="true" "noline"=true dojoAttachPoint="viewHeaderNode">' +
                    '    <div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '  </div>' +
                    '</div>',
                    
    widgetsInTemplate: true,
    searchText: Constants.getMessage("labelUnspecified"),
    
    constructor: function() {
	 	this.refreshCallback = null;
		this.results = null;
	},
    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-SEARCH-RESULTS-TABLEVIEWER"                                
        });        
        this._tableViewerNode.appendChild(tableViewer.domNode);
		          
        var content = this.results ? this.results : [];
        this.viewHeaderNode.setTitle(Constants.getMessage("labelSearchText", this.searchText));
		  
        tableViewer.setContentPageSize(25);
        tableViewer.setContent(content);
		tableViewer.registerCellRenderer(RENDER_SEARCH_IMAGE, new SearchResultTableView.ImageCellRender());
		tableViewer.registerCellRenderer(TableConstants.RENDER_TYPE_RICH_HOVER, new RichHoverCellRenderer());
        tableViewer.setContentProvider(new SearchResultTableView.TableProvider({itemType: this.itemType}));
        tableViewer.setTableListener(new SearchResultTableView.TableListener());
        tableViewer.setCellModifier(new SearchResultTableView.CellModifier());
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", this.searchText));
		
        //add refresh action if applicable
        if (this.refreshCallback && dojo.isFunction(this.refreshCallback)) {
            this.refreshAction = tableViewer.addAction("refresh", Constants.getMessage("labelRefresh"), this, "refresh-icon-image", this.refreshCallback);
        }
				
        tableViewer.show();
		  
		this.tableViewer = tableViewer;		  
    },
    
    refresh: function(content) {
        this.results = content ? content : [];
        if (this.tableViewer) {
            this.tableViewer.setContent(this.results);
            this.tableViewer.refresh();
        }
    },

    setInProgress: function(msg) {
        this.viewHeaderNode.setProgressStart(msg);
        if (this.tableViewer) {
            this.tableViewer.showLoadingProgress();
        }
    },
    
    stopInProgress: function() {
        this.viewHeaderNode.setProgressEnd();
        if (this.tableViewer) {
            this.tableViewer.endLoadingProgress();
        }
    }
});

})();
