//debug workitemviewlet
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.WorkItemTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants 	   = com.ibm.rqm.planning.web.client.internal.Constants;
var RENDER_WRAPPED_LINK = "RenderType.WRAPPEDLINK";


dojo.declare("WorkItemTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        //alert("Row check state changed: " + rowObject.col1);        
    }
});

dojo.declare("WorkItemTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        //nothing; not implemented
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		//nothing; not implemented
    }
});

dojo.declare("WorkItemTableView.WrappedLinkCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
    	var doc=document;
        var node = doc.createElement("a");
        var divNode=doc.createElement("div");
        var textData = cellData + "";
        var linkText = null;
        
        if (extendedDisplayInfo) {			
            if (typeof extendedDisplayInfo['href'] != "undefined") {
                node.href = extendedDisplayInfo['href'];
			} else if (dojo.isFunction(extendedDisplayInfo['onclick'])) {
				node.onclick = extendedDisplayInfo['onclick'];
			}
            // NOTE: we probably should refactor this to the common table's 
            // Link renderer. Other clients might have the need for this as well.
            if(extendedDisplayInfo['linkText'])
                linkText = extendedDisplayInfo['linkText'];
		}
        // handle compact mode. This possibly clips the text and attach the
        // proper tooltip for it.
        if(viewer.isCompactMode && typeof cellWidthHint != "undefined") {
            textData = this._handleCompactCellText(viewer, textData, extendedDisplayInfo, divNode, cellWidthHint);
        }
        else {
            // the parent class's attach tooltip will not create a tooltip
            // if the string is null, or undefined. Don't have to check it here.
            node.tooltip = this._attachTooltipToCell(node, extendedDisplayInfo['tooltip']);                
        }
        var textNode = doc.createTextNode(textData);
        
        // prepend the link text to the actual displayable text.
        if(linkText)
            node.innerHTML = linkText; 

        node.appendChild(divNode);
        divNode.appendChild(textNode);
        this.cellNodes.push(node);
        return node;
    },
    
    disconnectAll: function()
    {
        this.inherited(arguments);
        //console.debug("================WrappedLinkCellRenderer disconnectAll");
        var cellNodes = this.cellNodes;
        var len = cellNodes.length;
        var d = dojo;
        for(var i = 0; i < len; i++) {
            var node = this.cellNodes[i];
            var tooltip = node.tooltip;
            if(tooltip && tooltip.destroy) {
                tooltip.destroy();
            }
            node.tooltip = null;
            node.onclick = null;
            node.onmouseover = null;
            node.onmouseout = null;
            node.onhover = null;
            node.onunhover = null;
            d._destroyElement(node);
        }        
        this.cellNodes = [];
    }
});

dojo.declare("WorkItemTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
	
	
	remote:false,
	url:"",
	
	setUrl:function(url){
		this.url=url;
	},
	
	setRemote: function(remote){
		this.remote=remote;
	},
    // Summary:
    //
    getNumColumns: function(content)
    {
        return 4;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        var headings = [
			Constants.getMessage("labelId"),
            Constants.getMessage("labelSummary"),
            Constants.getMessage("labelArtifact"),
			Constants.getMessage("labelState")
        ];
        return headings[index];
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {	  
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return false;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return true;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
	 	  var widths = [10, 40, 40, 10];
        return (atColumnIndex < widths.length ? widths[atColumnIndex] : 0);
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(results)
    {
      //convert the objects into simpler convenience summaries
      var summaries = [];
	  var resultLen = (results ? results.length : 0);
      for (var i = 0; i < resultLen; i++) {
         var result = results[i];
		 var wiSummary = result.workItemSummary;
		 var isReviewWI = (wiSummary.type2 == "review"); //$NON_NLS$
		 var summary = {
			workItemId: wiSummary.id,
			workItemType: wiSummary.typeName,
			workItemSummary: wiSummary.summary,
			workItemURI: Utility.getViewWorkItemURI(wiSummary.id, (isReviewWI ? "approvals" : null)),
			workItemState: wiSummary.stateName
		 }

		 summary.linkURI = summary.workItemURI;
         if (result.linkTypeDTOs != null) {
            var linkType = result.linkTypeDTOs[0];
			var artifactType = Constants.MODEL_PACKAGE +  "." + linkType.itemType;			
            if (linkType.linkDTOs != null) {
               var link = linkType.linkDTOs[0];
               summary.artifactId = link.id;
			   summary.artifactName = link.comment;
               summary.artifactURI = this.getItemURI(linkType, link, artifactType);
            }
			// For WIs of type TaskReview that have formal review (ie anything but scripts), point the workitem URI to the artifact. 
			if (isReviewWI) { 
				if (artifactType != Constants.MODEL_TYPE_VERSIONED_SCRIPT && artifactType != Constants.MODEL_TYPE_SCRIPT) {
					summary.linkURI = this.getItemURI(linkType, link, artifactType, "com.ibm.rqm.process.editor.section.review");
				}
			}
         }
		 summaries.push(summary);
      }  
      return summaries;
    },
	 
	getItemURI: function(linkType, link, artifactType, /*optional*/ sectionId) {
    	if("com.ibm.rqm.execution."+linkType.itemType == Constants.MODEL_TYPE_SUITE)
			return Utility.getTestSuiteURI(link.id);
        switch(artifactType) {
            case Constants.MODEL_TYPE_PLAN: 
			case Constants.MODEL_TYPE_VERSIONED_PLAN:
				var args = {id: link.id};
				if (sectionId != null) 
					args.sectionId = sectionId;
                return Utility.getTestPlanURI(args);		 
			
			case Constants.MODEL_TYPE_CASE: 
			case Constants.MODEL_TYPE_VERSIONED_CASE:
                return Utility.getTestCaseURI(link.id, null, null, (sectionId || null));
			
			case Constants.MODEL_TYPE_VERSIONED_SCRIPT:
			case Constants.MODEL_TYPE_SCRIPT: 
                return Utility.getTestScriptURI(link.id);
				
            case Constants.MODEL_PACKAGE + "." + "SizingRecord": 
                return Utility.getSizingRecordURI(link.itemId);
				
            default: 
                return null;				
        }
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_workitem_" + rowData.workItemId;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = TableConstants.RENDER_TYPE_TEXT;

        switch(atColumnIndex) {
            case 0: 
                type = RENDER_WRAPPED_LINK;
                break;
            case 1: 
                type = TableConstants.RENDER_TYPE_LINK;
                break;
            case 2: 
                type = TableConstants.RENDER_TYPE_LINK;
                break;
			case 3:
				type = TableConstants.RENDER_TYPE_TEXT;
				break;
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return TableConstants.ALIGN_LEFT;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "data";
        switch(atColumnIndex) {
            case 0: 
                // convert the id to numeric value for proper sorting.
                if (dojo.isString(rowData.workItemId)) {
                    var intVal = parseInt(rowData.workItemId);
                    val = isNaN(intVal) ? rowData.workItemId : intVal;
                }
                else {
                    val = rowData.workItemId;
                }
                break;
            case 1: 
                val = this.replaceWorkItemSummaryText(rowData.workItemSummary);
                break;
            case 2: 
                val = rowData.artifactId != null ? rowData.artifactName : "";
                break;
			case 3:
				val = rowData.workItemState;
				break;
        }
        return val;
    },
	wrapIdForA11y: function(id){
		var forA11y = "<span style='left:-3000px; position:absolute; display:inline;'>TaskID: " + id + "</span>";
		return forA11y;
	},
	replaceWorkItemSummaryText: function(summary) {
		var rawText = summary;
		rawText= rawText.replace(/\&gt;/gi,">");
		rawText= rawText.replace(/\&lt;/gi,"<");
		rawText= rawText.replace(/\&amp;/gi,"&");
		rawText= rawText.replace(/\&tilde;/gi,"~");
		rawText= rawText.replace(/\&quot;/gi,"\"");
		rawText= rawText.replace(/\&circ;/gi,"^");
		rawText= rawText.replace(/\&nbsp;/gi," ");
		rawText= rawText.replace(/<br\/>/gi,"\n");
		rawText= rawText.replace(/\&#09;/g, "\t");
		return rawText
	},
    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
    	var urlAbsolutePath = "";
    	 var val = {};
    	if(this.remote && (atColumnIndex == 0  || atColumnIndex == 1 || atColumnIndex == 2)){
    		urlAbsolutePath += this.url;    		
			val.target = "_RQMScope";
    	}       
        if (atColumnIndex == 0) {
            val.linkText = this.wrapIdForA11y(rowData.workItemId);
            val.href = urlAbsolutePath+rowData.workItemURI;			
		} else if (atColumnIndex == 1) {
            val.href = urlAbsolutePath+rowData.linkURI;
        
		} else if(atColumnIndex == 2) {
            val.href =urlAbsolutePath+( rowData.artifactURI != null ? rowData.artifactURI : rowData.linkURI);
        }
        return val;
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.WorkItemTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>',

    remote: false,
    url:"",
    
    constructor: function() {
	 	this.summaries = null;
		this.args = arguments;
	 },

    postCreate: function() 
    {
		var isCompactModeVar = typeof this.args[0].isCompactMode != "undefined"? this.args[0].isCompactMode: true; 
        this.inherited("postCreate", arguments);
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
			isCompactMode: isCompactModeVar,	
            tableId: "RQM-WI-TABLEVIEWER"                                
        });        
        this._tableViewerNode.appendChild(tableViewer.domNode);
		          
        var content = this.summaries ? this.summaries : [];
		  
        tableViewer.setContentPageSize((this.numitems) ? this.numitems : 20);
		tableViewer.setShowingPagingRangeControl( ( this.usePageRangeControl == undefined ) ? true: this.usePageRangeControl );
        tableViewer.setContent(content);
		tableViewer.registerCellRenderer(RENDER_WRAPPED_LINK, new WorkItemTableView.WrappedLinkCellRenderer());		
		var provider = new WorkItemTableView.TableProvider();
		provider.setRemote(this.remote);
		provider.setUrl(this.url);
        tableViewer.setContentProvider(provider);
        tableViewer.setTableListener(new WorkItemTableView.TableListener());
        tableViewer.setCellModifier(new WorkItemTableView.CellModifier());
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelWorkItems")));
        tableViewer.show();
		  
        this.tableViewer = tableViewer;		  
    },
	 
	 refresh: function(refreshedContent) {
       if(this.tableViewer && !this.tableViewer.uninitialized){
		   	var content = refreshedContent && dojo.isArray(refreshedContent) ? refreshedContent : [];
			this.summaries = content;
	        this.tableViewer.setContent(content);
	        this.tableViewer.refresh();
		}	 	 	
	 }
	 
   });	 
})();
