/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.viewlets.ArtifactListViewlet");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView");
dojo.require("com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet");
(function()
{
var AbstractViewlet = com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet;	
var PlanningPageClient     	= com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
var TestCaseClient			= com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var TestScriptClient		= com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
var TestPlanTableView 		= com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView;
var TestCaseTableView		= com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView;
var TestScriptTableView		= com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView;
var Constants 				= com.ibm.rqm.planning.web.client.internal.Constants;
var ServiceResponseHandler 	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.ArtifactListViewlet",	AbstractViewlet,
{
	templateString: "<div class=\"\" dojoAttachPoint=\"_contentArea\"></div>",
	
	refresh: function()
	{
		this._contentArea.innerHTML = "";
		this._contentArea.setAttribute("id", "artifactListViewlet");
		this._hasZoomableContent = false;
		this.getSite().setLoading(true);

		var loading = document.createElement("div");
		loading.innerHTML = Constants.getMessage("labelLoading");
		loading.className = "loadingText";
		this._contentArea.appendChild(loading);
		
		this._artifactType = this.getPreference("type");
		this._maxRows = this.getPreference("rows");
		if (!this._artifactType) 
		{
			this._artifactType = "plan";
		}
		
		this._loadArtifacts();
	},
	
	_loadArtifacts: function()
	{
		var self = this;
		var handler = 
		{
			success: function(results)
			{
				self._results = results;
				self._update();
				self.getSite().setLoading(false);
				self.getSite().doneRefresh();
			},
			error: function(e) 
			{
				self._error(e);
				self.getSite().setLoading(false);
				self.getSite().doneRefresh();
			}
		}
		
		var responseHandler = new ServiceResponseHandler(handler, "success", "error");
		
		switch (this._artifactType)
		{
			case "plan": this._title = Constants.getMessage("labelTestPlans");
						 PlanningPageClient.getTestPlans(responseHandler, this._title);
						 break;
	 		case "case": this._title = Constants.getMessage("labelTestCases");
						 TestCaseClient.getTestCases(responseHandler, this._title);
						 break;
	 		case "script": this._title = Constants.getMessage("labelTestScripts");
						 TestScriptClient.getTestScriptsSummary(responseHandler, this._title);
						 break;
		}			
	},

	getViewletStrId: function()
	{
		var id = "";

		switch (this._artifactType)
		{
			case "plan": id = "com.ibm.rqm.planning.web.viewlets.testplan";
						 break;
	 		case "case": id = "com.ibm.rqm.planning.web.viewlets.testcase";
						 break;
	 		case "script": id = "com.ibm.rqm.planning.web.viewlets.testscript";
						 break;
		}
		
		return(id);			
	},	

   _showTestPlanList: function(name) 
   {
      this._planTableViewer = new TestPlanTableView({plans: this._results, queryName: name, showPagingRangeControl: false, pageSize: this._maxRows});
      this._contentArea.appendChild(this._planTableViewer.domNode);
   }, 
   
   _showTestCaseList: function(name)
   {
      this._caseTableViewer = new TestCaseTableView({testCases: this._results, queryName: name, showPagingRangeControl: false, pageSize: this._maxRows});
      this._contentArea.appendChild(this._caseTableViewer.domNode);            
   },
	
   _showTestScriptList: function(name)
   {
      this._scriptTableViewer = new TestScriptTableView({scripts: this._results, queryName: name, showPagingRangeControl: false, pageSize: this._maxRows});
      this._contentArea.appendChild(this._scriptTableViewer.domNode);            
   },	
			
	_update: function()
	{
		dojox.data.dom.removeChildren(this._contentArea);

		switch (this._artifactType)
		{
			case "plan": this._showTestPlanList(this._title);
						 break;
			case "case": this._showTestCaseList(this._title);
						 break;
			case "script": this._showTestScriptList(this._title);
						 break;
		}			
	},
	
	hasZoomableContent: function()
	{
//		if (this._count && this._count > 10)
			return true;
//		return false;
	},
	
	settingsChanged: function(oldSettings, newSettings)
	{
		if ((oldSettings.preferences.type != newSettings.preferences.type) ||
			(oldSettings.preferences.rows != newSettings.preferences.rows))
		{
			this.getSite().refresh();
		}
		else
		{
			this._update();
		}
	}
});

})();
