/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.viewlets.ReviewViewlet");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.WorkItemTableView");

dojo.require("com.ibm.rqm.process.web.client.internal.approval.ProcessClient");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.QueryClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet");

(function()
{
var Constants 					= com.ibm.rqm.planning.web.client.internal.Constants;
var ProcessClient				= com.ibm.rqm.process.web.client.internal.approval.ProcessClient;
var PlanClient					= com.ibm.rqm.planning.web.client.internal.QueryClient;

var Viewlet 					= com.ibm.team.dashboard.web.ui.Viewlet;
var DashboardConstants          = com.ibm.team.dashboard.web.ui.DashboardConstants;
var QueryClient                 = com.ibm.team.workitem.web.client.internal.QueryClient;
var ServiceResponseHandler 		= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;	
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var WorkItemTableView 			= com.ibm.rqm.planning.web.ui.internal.view.table.WorkItemTableView;
var GlobalEvents 				= com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
var AbstractViewlet = 			com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet;
	
dojo.declare("com.ibm.rqm.process.web.ui.internal.view.viewlets.ReviewViewlet",	AbstractViewlet,
{
	templateString: "<div dojoAttachPoint=\"_resultSection\"></div>",
	
	_actionsDefaultLocale:[
				"My open items",
				"Already approved for me", 
				"Already rejected for me",
				"Pending approvals for me"
				],
				
    _actionsCurrentLocale: [
	           Constants.getMessage("labelOpenItemReviewlet"),
			   Constants.getMessage("labelAlreadyApprovedReviewlet"),
			   Constants.getMessage("labelAlreadyRejectedReviewlet"),
			   Constants.getMessage("labelPendingApprovalsReviewlet")
			   ],
			   
    _actionsIds:[
               "com.ibm.rqm.planning.myOpenItems",
               "com.ibm.rqm.process.myApprovedItems",
               "com.ibm.rqm.process.myRejectedItems",
               "com.ibm.team.workitem.queries.predefined.pendingApprovals"
               ],
				    
	_usingAlternativeQueries:false, 
	
	_isQuerying: false,
	
	/*
     * Called by the framework after the viewlet is instantiated, but before the
     * first refresh. Viewlets that need to initialize themselves shoudl override.
     */
	//@Override from com.ibm.team.dashboard.web.ui.Viewlet
    init: function() {
		this.inherited("init", arguments);		
				
		dojo.subscribe(GlobalEvents.DashboardFocus, this, "_refreshOnFocus");
		this._queries = [];
		
    },    
    
	//@Override from com.ibm.team.dashboard.web.ui.Viewlet
	// If there are no queries regen the queries otherwise, just run them 
	refresh: function() {
		if (!this.getProjectArea()) {
            this._resultSection.innerHTML = "";
            this._resultSection.appendChild(this.createNoScopeMsg());
            this.getSite().setSecondaryTitle("");
            this.getSite().doneRefresh();
            return;
        }
		this._projectArea = this.getProjectArea();
		this.getSite().setLoading(true);        
        this._showTable();		
		if (!this._queries || this._queries.length == 0) {
			this._getScopedQueries();
		} else {
			this._runQueries();
		}
	},
	
	//@Override from com.ibm.team.dashboard.web.ui.Viewlet
	update: function() {
		this._showTable(this._results);
	},
	
	_refreshOnFocus: function() {
		//TODO determine if a refresh *should* be performed
		//for now, always refresh
		this.refresh();  
	},

	// If there is a project area defined, use it otherwise make the user select one. 		
	_getScopedQueries: function() {
	    this._showTable();		
		var responseHandler = new ServiceResponseHandler(this, "_setScopedqueriesFromIds", "_error");
		if(this._projectArea){
			var args = {
					ids: this._actionsIds,
					includeCount:false,
					projectAreaItemId:this._projectArea.itemId
						
			};
			QueryClient.getQueries2(responseHandler, args, false);	
		} 			
		else
			this.getSite().toggleSettings();					
	},
	
	_setScopedqueriesFromIds: function(results){
		this._queries = new Array();
		//If we have the new query ids, use them but since com.ibm.team.workitem.queries.predefined.pendingApprovals is an already existing query,
		//so most likely it will find it, that's why we are validating that we have > 1
		if(results && results.length > 1){
			var length = results.length;
			for(var j = 0; j < length; j++){
				this._queries.push(results[j]);
			}
			this._runQueries();
		}
		else
			this._getAlternativeQueries();		
	},
	
	_getAlternativeQueries: function(){
		this._usingAlternativeQueries = true;
		var responseHandler = new ServiceResponseHandler(this, "_setScopedQueries", "_error");
		if(this._projectArea) 
			QueryClient.getScopedQueries(responseHandler, QueryClient.SCOPE_PROJECT_AREA, this._projectArea.itemId, true, true, this._projectArea.itemId, false);		
	},
	// Find all of the valid queries in this project area
	_setScopedQueries: function(scopedQueriesDTO) {
		if(!this.queries)
			this._queries = new Array();

		for(var j = 0; j < scopedQueriesDTO.length; j++){
			var scopedQuery = scopedQueriesDTO[j];		
			if(scopedQuery.queries){
				for (var i = 0; i < scopedQuery.queries.length; i++){
					var query = scopedQuery.queries[i];
					//search queries in english and in the user's locale 
					if(this._isInArray(this._actionsDefaultLocale, query.name) || this._isInArray(this._actionsCurrentLocale, query.name))
						this._queries.push(query);
				}
			}
		}
		this._runQueries();
  	},
	
	// Run the queries
	_runQueries: function() {
		if (this._isQuerying) {
			return;
		} else {
			this.getSite().setLoading(true);
			this._isQuerying = true;
		}
		var length = (this._queries && this._queries.length)?this._queries.length:0;
		if(this._projectArea && length >0 ){
			var queryOps = new Array();
			for(var i = 0; i < length; i++){
				var params = {
					jsonQuery:{
						itemId: this._queries[i].itemId
					},
					queryItemId: this._queries[i].itemId,
					name: this._queries[i].name,
					token: "",
					projectAreaItemId: this._projectArea.itemId
				}
				queryOps.push(params);
			}
			var param = dojo.toJson({querys: queryOps});
			var responseHandler = new ServiceResponseHandler(this, "_showResults", "_error");
			var args = {operationJson: param, 
						projectAreaItemId: params.projectAreaItemId};
			ProcessClient.postExecuteQuery(responseHandler, args);
		}	
		else {
			this._showResults([]);
		}	
	},
	
	// Show the table, note that the WorkItemTable viewer can't show  viewlet tables in 
	// compact mode. TODO investigate why this is.
	_showTable: function(summaries) {
		summaries = summaries || [];
		if (!this._tableViewer) {
			dojox.data.dom.removeChildren(this._resultSection);
			var itemCount = this.getPreference("items");
			this._tableViewer = new WorkItemTableView({
				summaries: summaries,
				numitems: itemCount,
				usePageRangeControl: false,
				url:this.getFormattedUrl(),
				remote:this.isRemote(),
				isCompactMode: false
			});
			this._resultSection.appendChild(this._tableViewer.domNode);
			this._resultSection.style.width = "100%"; // CML: hack: the table doesn't fill the viewlet otherwise, looks silly
		} else {
			this._tableViewer.refresh(summaries);
		} 		
	},
	
	_showResults: function(results) {
		results = results || [];
		this._isQuerying = false;
		
		this.getSite().setLoading(false);
		this.getSite().doneRefresh();
		
		//change workitem's summary to display what the user should do.  	
		for (var i = 0; i < results.length; i++) {
			var wiSummary = results[i].workItemSummary;
			if (results[i].linkTypeDTOs != null) {
				var linkType = results[i].linkTypeDTOs[0];
				var taskName = null;
				if(this._usingAlternativeQueries)
					taskName = this._getTaskName(linkType);  //not really useful when using ids (display name and current label may be different locales)
				if (!taskName) {
					taskName = wiSummary.summary;
				}
				if (linkType.id != "com.ibm.rqm.planning.myOpenItems" || 
						(this._usingAlternativeQueries && (this._actionsDefaultLocale[0] != linkType.displayName && this._actionsCurrentLocale[0] != linkType.displayName))) {
					//if the display name is not "My open items", this is an approval record
					//flag it as such so URI can point to the approvals ta directly
					wiSummary.isApproval = true;
				}
				wiSummary.summary = taskName;
			}
		}	
		this._results = results;		
        this._showTable(results);
	},
	
	_isInArray: function(array, item) {
		if(array != null && array.length > 0) {
			for(var i=0; i<array.length; i++){
				if(array[i]==item)
					return true;
			}
		}
		return false;
	},
		
	_getTaskName: function(linkType) {
		var name = null;
		if(linkType.displayName == this._actionsCurrentLocale[1] || linkType.displayName == this._actionsDefaultLocale[1]) {
			name = Constants.getMessage("labelApprovedReviewlet", linkType.linkDTOs[0].comment);
		}
		if(linkType.displayName == this._actionsCurrentLocale[2] || linkType.displayName == this._actionsDefaultLocale[2]) {
			name = Constants.getMessage("labelRejectedReviewlet", linkType.linkDTOs[0].comment);
		}
		if(linkType.displayName == this._actionsCurrentLocale[3] || linkType.displayName == this._actionsDefaultLocale[3]) {
			name = Constants.getMessage("labelApproveReviewReviewlet", linkType.linkDTOs[0].comment);
		}
		return name;
	},
		
	_error: function(errorObj){
        console.error(errorObj.message);
    }
});

})();
