/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.planning.web.ui.internal.view.workitem.BasicWorkItemEditor");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.workitem.BasicWorkItemEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.DateTextBox");
dojo.require("dojox.data.dom");

// Require the common widgets needed
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");

// Require local widgets
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

(function() { // module container

/* Jazz objects */
var getAuthenticatedContributor	 = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var RepositoryClient 		       = com.ibm.team.repository.web.client.internal.RepositoryClient;

/* QM objects */
var Constants    			          = com.ibm.rqm.planning.web.client.internal.Constants;
var EditorUtil     			       = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var Utility    				       = com.ibm.rqm.planning.web.ui.internal.util.Utility;

dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.workitem.BasicWorkItemEditor",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/workitem/BasicWorkItemEditor.html"),
//		templateCssPath: dojo.uri.dojoUri("../com.ibm.rqm.planning.web/ui/internal/view/templates/workitem/BasicWorkItemEditor.css"),

		labelDiscardChanges: Constants.getMessage("labelDiscardChanges"),
		labelSave: Constants.getMessage("labelSave"),
		labelClose: Constants.getMessage("labelClose"),
		labelSummary: Constants.getMessage("labelSummary"),
		labelCreator: Constants.getMessage("labelCreator"),
		labelLoading: Constants.getMessage("labelLoading"),
		labelOwner: Constants.getMessage("labelOwner"),
		labelPriority: Constants.getMessage("labelPriority"),
		labelSeverity: Constants.getMessage("labelSeverity"),
		labelPlannedFor: Constants.getMessage("labelPlannedFor"),
		labelEstimateDays: Constants.getMessage("labelEstimateDays"),
		labelDue: Constants.getMessage("labelDue"),
		labelCancel: Constants.getMessage("labelCancel"),
		labelOk: Constants.getMessage("labelOk"),
		labelClose: Constants.getMessage("labelClose"),
	
		// object initializer
		constructor: function(args) 
		{
			this._isNew = true;
			this._defaultSummary = null;
			this._workItem = null;
			this._workItemData = null;
			this._owningDialog = null;
			this._initOwner = null;
		},
		
		// populate the template with data, connect events, and show initial
		// sub-tab
		postCreate: function() {		
		},
		
		initialize: function(args) 
		{			
			if (args.parentController === null) {
				throw new Error("BasicWorkItemEditor#postCreate: Missing required parameter 'args.parentController'");
			}
			this.parentController = args.parentController;
			this._workItem = args.workItem;			
			this._owningDialog = args.dialog;
			this._defaultSummary = args.defaultSummary;
			this._initOwner = args.initOwner;
			
			EditorUtil.initContributors(this._owner, this._initOwner);
			this._isNew = this._workItem == null;
			this._summary.value = this._isNew ? this._defaultSummary : this._workItem.summary;

			//setup date picker
			dojox.data.dom.removeChildren(this._dueDate);
			//this._dueDatePicker = new dijit.form.DateTextBox({displayFormat: "MMM dd, yyyy"});
			var args = {isDate: true, containerToggle: "fade", constraints: {datePattern: "MMM d, yyyy"} };
			this._dueDatePicker = new dijit.form.DateTextBox(args, this._dueDate);			
			
			if ( this._workItem && this._workItem.dueDate ) {
				this._dueDatePicker.setValue(Utility.dateFromISOString(this._workItem.dueDate));
			} else {
				this._dueDatePicker.setValue(new Date());
			}
			//dojox.data.dom.removeChildren(this._dueDate);
			//this._dueDate.appendChild(this._dueDatePicker.domNode);			
		},
		
		synchronizeOwner: function(ownerId) 
		{
			EditorUtil.initContributors(this._owner, ownerId);
		},

		getWorkItemData: function()
		{
			return this._workItemData;
		},		

		onCancel: function(event)
		{			
			if ( this._owningDialog ) {
				//hide the dialog
				this._owningDialog.hide();
			} else {
				if ( confirm(Constants.getMessage("labelConfirmChanges")) ) {
					this._initialize();	
				}
			}	
			if(event){
                dojo.stopEvent(event);
            }
		},
								
		onSave: function(event)
		{
			//validate fields
			if (dojo.trim(this._summary.value).length == 0) {
				alert(Constants.getMessage("labelSummaryRequired"));
				return;
			}		
			
			//create the new object
			this._workItemData = {				
				summary: this._summary.value,
				owner: this._owner.value,
				ownerName: this._owner.options[this._owner.selectedIndex].text,
				dueDate: this._dueDatePicker.getValue() != null ? this._dueDatePicker.getValue().getTime() : null
			}
			if (this._workItem && this._workItem.itemId) {
				this._workItemData.workItemItemId = this._workItem.workItemItemId; //itemId;
			}
			//hide the dialog
			this._owningDialog.hide();
			if(event){
                dojo.stopEvent(event);
            }
		},

		_saveError: function(e)
		{
			this._setLastSaved(Constants.STATUS_FAILURE);
			alert("An error occurred: [" + e.message + "] @ BasicWorkItemEditor#_saveError");
		},
				
		_error: function(e)
		{
			alert("An error occurred: [" + e.message + "] @ BasicWorkItemEditor#_error");
		}
	}
);

}
)();
