//customizecatalogview
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/



/***********************
 *
 * MFV 09/17/08 
 * Doug asked that the catalog customization feature be removed for V1.  
 * I removed access to this from the flyout menu but
 * I've left this here as a possible starting point for Catalog Customization
 * 
 ***********************/



dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.CustomizeCatalogView");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.Tooltip");
dojo.require("dijit.Tree");

dojo.require("com.ibm.asq.common.web.client.internal.Catalog");

(function() {

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;

//These are the types that can be customized by the 
var typesArray = [{name:"CPU", value:"CPU"},
				  {name:"Operating System", value:"OperatingSystem"},
				  {name:"Software", value:"SoftwareInstall"}];

//an object to store the trees for each type after they have been loaded.
//We need this because once a tree has been modified we can't reload it from the catalog.
//The catalog needs to be reloaded in order for the changes to be visible.				  
var typesTrees = {};

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.CustomizeCatalogView",
    [dijit._Widget, dijit._Templated],
{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.test.lm.web", "ui/internal/views/machines/templates/CustomizeCatalogView.html"),
	widgetsInTemplate: true,

	constructor: function() {
		this.theCatalog = new com.ibm.asq.common.web.client.internal.Catalog();
	},
	
	postCreate: function() {
        com.ibm.rational.test.lm.web.ui.internal.views.machines.CustomizeCatalogView.superclass.postCreate.apply(this, arguments);
		this.viewHeaderLayoutNode.setTitle(RtlmUiUtil_getMessage("custTitle"));
		this.viewHeaderLayoutNode.setSummary(RtlmUiUtil_getMessage("custSummary"));
		
		this.typesLabel.innerHTML = RtlmUiUtil_getMessage("custTypes");
		this.treeLabel.innerHTML = RtlmUiUtil_getMessage("custParent");
		this.enterNewValLabel.innerHTML = RtlmUiUtil_getMessage("custEnterValue");
		
		this.connect(this.newVal, "onkeyup", "_onNewValChange");
		this.connect(this.typesSelector, "onchange", "_onTypesChange");
		this.connect(this.addButton, "onclick", "_onAddClick");
		this.addButton.disabled = true;

		//Load the "Types"
		this._loadTypes();
		this._initTree(this.typesSelector.options[0].value);
		this._loadTree(this.typesSelector.options[0].value);
		this.currentType = this.typesSelector.options[0].value;
	},
    
    uninitialize: function()
    {
        this.theCatalog = null;
    },
	
	_loadTypes: function()
	{
        var typesArray_length = typesArray.length;
		for (var i = 0; i < typesArray_length; i++)
		{
            var option = new Option( typesArray[i].name, typesArray[i].value );
			this.typesSelector.options[i] = option;
		}		
	},
	
	_initTree: function(templateName)
	{

		var treeData={     

			items: [
					{ id: templateName, name:templateName, type:'top'}
				   ],

			theIdentifier: 'id',
			label: 'name'   
        };

		var store_arg = {data:treeData};
        var store1=new dojo.data.ItemFileReadStore(store_arg);
       
        var partTree_args = {           
            store:store1,
            query:{type:'top'},
            labelAttr:"name",
//          label:templateName,
            typeAttr:"type",
            'onClick': dojo.hitch(this, "_onTreeClick")
        };
        this._partTree=new dijit.Tree(partTree_args, this.treeSpan);
	},
	
	_loadTree:function(templateName)
	{
		console.log(templateName);
		
		//Clear the tree
		var root = this._partTree.getChildren()[0];
		if (root)
			this._partTree.removeChild(root);


		//First, add a root node with the template name
		var rootItemData = {tree: this._partTree, label:templateName, item: {
			name: templateName,
			type: 'top'
		}};
		var rootItem = new dijit._TreeNode(rootItemData);
		this._partTree.addChild(rootItem);



		//Get the root part of the template
		var template = this.theCatalog.getTemplate(templateName);
		var rootPart = template.getRoot();

		
		//Now process all the children (recursively)
		var rootItem = this._partTree.getChildren()[0];
		
		var childParts = rootPart.getChildParts();
        var childParts_length = childParts.length;
		for (var i = 0; i < childParts_length; i++)
		{
			var childPart = childParts[i];
			this._processPart(rootItem, childPart);
		}		
	},
	
	_processPart: function(treeItem, part)
	{
		var label = part.getName();

		var itemData = {tree: this._partTree, label:label, item: {
			name: label,
			type: "instance"
		}};

		
		var item = new dijit._TreeNode(itemData);
		treeItem.addChild(item);
		
		//Now process all the children (recursively)
		var childParts = part.getChildParts();
        var childParts_length = childParts.length;
		for (var i = 0; i < childParts_length; i++)
		{
			var childPart = childParts[i];
			this._processPart(item, childPart);
		}		
	},
	_onTypesChange: function()
	{
		var root = this._partTree.getChildren()[0];
		//Save the tree to the typesTrees variable in case we need to use it again.
		//We can't reload it from the template after changes have been made.
		typesTrees[this.currentType] = root;
		
		var templateName = this.typesSelector.options[this.typesSelector.selectedIndex].value;
		
		//If we've loaded this tree before then reuse it
		//otherwise load the tree from the template
		if (typesTrees[templateName]) 
		{
			this._partTree.removeChild(root);
			this._partTree.addChild(typesTrees[templateName]);
		}
		else
		{
			//Clear the tree
			this._partTree.removeChild(root);
			this._loadTree(templateName);
			//Clear the parent text and new value fields
			this.parentText.value = "";
			this.currentItem = null;
			this.currentNode = null;
			this.newVal.value = "";
			this._checkButtonStatus();
		}

		
		this.currentType = templateName;
	},
	
	_onTreeClick:function(item, node)
	{
		this.currentItem = item;
		this.currentNode = node;
		this.parentText.value = item.name;
		this.currentParent = item.name;
		this._checkButtonStatus();
	},
	
	_onAddClick: function()
	{
		this.addButton.disabled = true;
		this.viewHeaderLayoutNode.setProgressStart("Adding new item...");
		var projectAreaItemID = com.ibm.team.process.web.projectArea.itemId;
		var newVal = this.newVal.value;
		var admin = new com.ibm.asq.common.web.client.internal.admin();
        var responseHandler = new ServiceResponseHandler(this, "_addSuccess", "_addFailure");
		admin.createNewCatalogType(responseHandler, projectAreaItemID, this.currentParent, newVal);		
	},
	
	_addSuccess: function(response)
	{
		this.viewHeaderLayoutNode.setProgressEnd();
		var newVal = this.newVal.value;

		if (response == "true") 
		{
			this.viewHeaderLayoutNode.setStatus(RtlmUiUtil_getMessage("custSuccess"), ViewHeaderLayout.SUCCESS, RtlmUiUtil_getMessage("custAddSucceeded", newVal));
			var itemData = {
				tree: this._partTree,
				label: newVal,
				item: {
					id: newVal,
					name: newVal,
					type: "instance",
					style: "com-ibm-rational-test-lm-machine-customizecatalog-new-node"
				}
			};
			this.newVal.value = "";
			var item = new dijit._TreeNode(itemData);
			this.currentNode.addChild(item);
			this._partTree.focusNode(item);
//Not working.  Trying to add styles to make the new values standout			
//			dojo.addClass(item, "com-ibm-rational-test-lm-machine-customizecatalog-newTreeLabel");
		}
		else
		{
			this.viewHeaderLayoutNode.setStatus(RtlmUiUtil_getMessage("custFailed"), ViewHeaderLayout.ERROR, RtlmUiUtil_getMessage("custAddFailed", newVal));
		}
		this._checkButtonStatus();
	},
	
	_addFailure:function(responseHandler)
	{
		this.viewHeaderLayoutNode.setProgressEnd();
		this._checkButtonStatus();
	},
	
	_onNewValChange: function()
	{
		this._checkButtonStatus();
	},
	_checkButtonStatus: function()
	{
		var disable = false;
		if (this.parentText.value == null || this.parentText.value == "" || this.newVal.value == null || this.newVal.value == "")
			disable = true;
			
		this.addButton.disabled = disable;
	}

});

})();