// ********  SIMPLE ASSETS VIEW ********* //

/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.SimpleAssetsView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.SimpleMachineSearchTableProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.MachineListBaseView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.InventoryUriUtil");
dojo.require("com.ibm.rational.test.lm.web.client.internal.reservations");
dojo.require("com.ibm.asq.common.web.client.internal.admin");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() 
{
	var MachineInfo = com.ibm.rational.test.lm.web.ui.internal.views.providers.SimpleMachineSearchInfo;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var InventoryUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.InventoryUriUtil;
	var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
	var reservationService = new com.ibm.rational.test.lm.web.client.internal.reservations();
	var adminService = new com.ibm.asq.common.web.client.internal.admin();
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
    var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
	var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.SimpleAssetsView", 
    [com.ibm.rational.test.lm.web.ui.internal.views.machines.MachineListBaseView, dijit._Widget, dijit._Templated], 
{
	//This line is NECESSARY fot the attachPoints in the PageLayout to get instantiated
	widgetsInTemplate: true,
	
	//The layout of the page.
	templateString: 
		'<div id="${id}" dojoAttachPoint="domNode">' +
		'<div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" isCompact="true" dojoAttachPoint="viewHeaderNode" noline="true">' + 		
        '    <!-- table resultarea --> '+
        '    <div dojoAttachPoint="resultDiv"> '+
        '    </div> '+
		' </div>' +
		'</div>',

    constructor: function()
    {
        this.tableId = "RQM-LM-SIMPLE-ASSET-TABLEVIEWER";       
    },
    
    postCreate: function()
    {
		this.assetType = "Machine";
		this.asset = catalog.createConfiguration(this.assetType);
		this.configPart = this.asset.getRootPart(0);
		this.reservationData = [];
		this.permissionsHash = {};
        
        com.ibm.rational.test.lm.web.ui.internal.views.machines.SimpleAssetsView.superclass.postCreate.apply(this);
		
        this.initializeViewHeader();
        
		//Create the table that will hold the reservations
		this.createTable();
		this.loadTable();
	},
    
    uninitialize: function()
    {
        delete this.reservationData;
        delete this.permissionsHash;
    },
	
    initializeViewHeader: function()
    {
        //Set the title and cs help id on the page
        this.viewHeaderNode.setTitle(RtlmUiUtil.getMessage("MachTitleAll"));
        this.viewHeaderNode.setCSHelpId("rtlm_search.html", com.ibm.rational.test.lm.web.CSHelpID);
        this.viewHeaderNode.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelHelp", RtlmUiUtil.getMessage("MachTitleAll"), ""));
        
    },
    
    createTableProvider: function()
    {
        var tableProvider = new com.ibm.rational.test.lm.web.ui.internal.views.providers.SimpleMachineSearchTableProvider( this.assetType );
        tableProvider.receiver = this;
        return tableProvider;                
    },
    
    getSearchParameters: function()
    {
        this._searchAttributeCriteria = MachineInfo.getSearchAttributeCriteria()[this.assetType];
        var searchParam = {
                    configuration: this.asset,
                    columns: ( this._searchAttributeCriteria )
        };
        return searchParam;
    },
    
	createTable: function()
	{
		this.tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: this.tableId                                
        });
		this.tableContentProvider = this.createTableProvider();
		this.tableViewer.setTableSummary(RtlmUiUtil.getMessage("labelTableSummary", RtlmUiUtil.getMessage("MachTitleAll")));
				
		this.tableViewer.setTableRowsCheckable( true );

		this.tableViewer.setContentProvider( this.tableContentProvider );
		this.tableViewer.setTableListener( this );		
		this.resultDiv.appendChild( this.tableViewer.domNode );
        var emptyContent = {Data:[]};
        this.tableViewer.setContent(emptyContent);
        this.tableViewer.show();

		this.createTableActions();
        this._updateTableAction();
        
        //Create the asset type selector
        this._constructAssetSelector();
        this._clearTable();        
	},
	
	loadTable: function()
	{
		this.refreshButton.setDisabled( true );
	   	var responseHandler = new ServiceResponseHandler(this, "_searchSuccess", "_searchError");
        
        this.viewHeaderNode.clearStatus();
        this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("srchWorking"));
        this.tableViewer.showLoadingProgress();
        		
		var searchParam = this.getSearchParameters();
        catalog.search(responseHandler, searchParam);		
	},
	
	_searchSuccess: function( response )
	{
		if (!(response.Header)){
			var colnames = this._searchAttributeCriteria.replace(/, /g,',');
			colnames = colnames.split(',');
			response.Header = colnames;
		}

		//If we are showing machines then we want to show the reservation status for the following 24 hours
		//This needs to be done via a call to the reservation service
		if (this.assetType == "Machine") 
		{
			//Save the data from this call.
			//This will be passed to the table in the reservation response handler
			this.tableData = response;
			
            var data_length = response.Data ? response.Data.length : 0;
            if (data_length > 0) 
            {
                var ids = [];
                for (var i = 0; i < data_length; i++) {
                    ids.push(response.Data[i][0]);
                }
                var responseHandler = new ServiceResponseHandler(this, "_searchResSuccess", "_searchResError");
                reservationService.getReservationsInWindow(responseHandler, ids, 1440);
            }
            else 
            {
                this.tableViewer.noItemMessage = "";
                this.refreshButton.setDisabled( false );
                this.viewHeaderNode.setProgressEnd();
                this.tableViewer.endLoadingProgress();
                
                this.tableViewer.setContent(this.tableData);
                this.tableViewer.refresh();
            }
		}
		else  //Showing Virtual images so just load the table from the response data
		{
            this.tableViewer.noItemMessage = "";
			this.refreshButton.setDisabled( false );
	        this.viewHeaderNode.setProgressEnd();
            this.tableViewer.endLoadingProgress();
	
	        this.tableViewer.setContent( response );
	        this.tableViewer.refresh();
			this._checkButtonStatus();	
		}
	},
	
	//Reload the table after reservations have been added or deleted
	_reloadTableDueToResChange: function()
	{
		var ids = [];
		//Use the table IDs that were loaded previously.
		//This method is used to update just the reservation information, not the list of machines in the table
        var data_length = this.tableData.Data ? this.tableData.Data.length : 0;
		for (var i = 0; i < data_length; i++)
		{
			ids.push(this.tableData.Data[i][0]);
		}
		//Get the updated set of reservations from the server
	   	var responseHandler = new ServiceResponseHandler(this, "_searchResSuccess", "_searchResError");
		reservationService.getReservationsInWindow(responseHandler, ids, 1440);	
	},
	
	_searchResSuccess: function(response)
	{
        this.tableViewer.noItemMessage = "";
		this.refreshButton.setDisabled( false );
        this.viewHeaderNode.setProgressEnd();
        this.tableViewer.endLoadingProgress();

		//Save the response data.  it will be used by a callback from the table provider
		//The callback will ask if a particular machine is reserved.  To answer that we need this response data
		this.reservationData = response;
		if (this.reservationData == null)
			this.reservationData = [];  //Don't like nulls
		
        this.tableViewer.setContent( this.tableData );
        this.tableViewer.refresh();
		this._checkButtonStatus();	

	},
	
	_searchResError: function(response)
	{
        this.tableViewer.noItemMessage = "";
   		this.refreshButton.setDisabled( false );
        this.viewHeaderNode.setProgressEnd();	
        this.tableViewer.endLoadingProgress();
	},
	
	_searchError: function( message, error )
	{
    	this.refreshButton.setDisabled( false );
        this.viewHeaderNode.setProgressEnd();
        this.tableViewer.endLoadingProgress();
        this.viewHeaderNode.setStatus("Error occurred. Unable to perform asset search.", ViewHeaderLayout.ERROR);		
	},
	
    _updateTableAction: function()
    {
        //Hide or show the appropriate actions depending on the asset type on display
        if( this.assetType == "Machine" )
        {
            this.configureButton.domNode.style.display = "";
            this.reserveButton.domNode.style.display = "";
            this.unReserveButton.domNode.style.display = "";
            this.deployButton.domNode.style.display = "none";
        }
        else if( this.assetType == "VirtualImage" )
        {
            this.configureButton.domNode.style.display = "none";
            this.reserveButton.domNode.style.display = "none";
            this.unReserveButton.domNode.style.display = "none";
            this.deployButton.domNode.style.display = "";
        }
    },
    
	_clearTable: function()
	{
        // loads an empty content to the table to clear the content.
        var emptyContent = {Data:[]};
        
        // don't show the standard empty content message. This is intensional
        // to have an empty content in the table.
        this.tableViewer.noItemMessage = " ";
        this.tableViewer.setContent(emptyContent);
        this.tableViewer.refresh();
	},
	
	//Asset selector methods
	_constructAssetSelector: function()
	{
        var views = [];
        var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
        this.assetTypeNames = catalog.getAssetTemplateNames();
                
        for(var i in this.assetTypeNames) {
            var opts = {
                'name': RtlmUiUtil.getMessage( this.assetTypeNames[i] ),
                'value': this.assetTypeNames[i]
            };
            views.push(opts);
        }
		this.assetType = this.assetTypeNames[0];
        this.tableViewer.setTableContentSelectors(views);
	},
	
	//Load the asset type specified in this.assetType
	_loadAssetType: function(assetType)
	{
        this.assetType = assetType;

        this.tableContentProvider = this.createTableProvider();
        this.tableViewer.setContentProvider( this.tableContentProvider );

		//A new asset type so create a new blank config based on the type
		this.asset = catalog.createConfiguration(this.assetType);
		this.configPart = this.asset.getRootPart(0);
		
		//And recreate and load the table
        this._updateTableAction();
 		this._clearTable();
		this.loadTable();
	},
		
	//Callback from the table when actions are called.  Just pass along the invocation (aka call the callback)
	tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback) 
	{
		callback(callbackData);
	},
	
    _checkActionPermissionForMachineId: function(id, action)
    {
        if(this.permissionsHash[id]) {
            return this.permissionsHash[id][action];
        }
        // optimistically set to has permission.
        return true;
    },

    // Summary:
    //    A group check box is shown when the tables rows are checkable and the in multi-select mode.
    //    This method is called when the group's check box is checked/unchecked. The affected rows and 
    //    their states are passed.
    //
    groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    {
        this.rowCheckStateChanged(tableViewer, rowObjects, state);
    },    
        
    // a row was checked so lets enable and disable actions appropriately    
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        var rows = this.tableViewer.getCheckedRows();
        var rows_length = rows.length;
        
        // deselection
        if (rows_length == 0) {
            this._checkButtonStatus();
            return;
        }
        
        var fetchPermList = [];
        for (var i = 0; i < rows_length; i++) {
            var id = rows[i][( ( this.assetType == "VirtualImage" ) ? "VirtualImage.id" : "machine.id" )];
            
            if(!this.permissionsHash[id]) {
                fetchPermList.push(id);
            }
        }            
        
        // we have all of the permissions for the rows, can check right away
        if(fetchPermList.length == 0) {
            this._checkButtonStatus();
            return;
        }
        
        var responseHandler = new ServiceResponseHandler
        (
            {
                _self: this,
                _success: function (perms) {
                    if (perms) {
                        var perms_length = perms.length;
                        // store the permissions in a table hashed by its id.
                        for (var i = 0; i < perms_length; i++) {
                            this._self.permissionsHash[perms[i]['itemId']] = perms[i];
                        }
                    }
                    this._self._checkButtonStatus();
                },
                _failure: function (errObj) {
                    // optimistically disables permission check
                    this._self._checkButtonStatus();
                }
            },
            "_success",
            "_failure"
        );
        
        // fetches the permissions for the rows, before we start the button
        // enablement check.
        adminService.getResourcePermissions(responseHandler, fetchPermList);
    },

    tableContentSelectionChanged: function(tableViewer, contentSelectionValue)
    {
        this._loadAssetType(contentSelectionValue);
    },
	
	createTableActions: function()
	{
		this.refreshButton = this.tableViewer.addAction( "Search", RtlmUiUtil.getMessage("Refresh"), null, "refresh-icon-image", dojo.hitch( this, this._searchAssets ) );
		
		this.configureButton = this.tableViewer.addAction( "Automation", RtlmUiUtil.getMessage( "Configure Automation" ), null, "configure-automation-icon-image", dojo.hitch( this, this._automateSelected ) );
		this.configureButton.setDisabled( true );
		
		this.deployButton = this.tableViewer.addAction( "Deploy", RtlmUiUtil.getMessage( "Deploy" ), null, "configure-deploy-icon-image", dojo.hitch( this, this._deploySelected ));
		this.deployButton.setDisabled( true );

		this.reserveButton = this.tableViewer.addAction( "Reserve", RtlmUiUtil.getMessage("resMachine24Hours"), null, "reserve-icon-image", dojo.hitch( this, this._reserveSelected ) );
		this.reserveButton.setDisabled( true );

		this.unReserveButton = this.tableViewer.addAction( "UnReserve", RtlmUiUtil.getMessage("unresMachine"), null, "unreserve-icon-image", dojo.hitch( this, this._unReserveSelected ) );
		this.unReserveButton.setDisabled( true );
        
        // add actions to delete/hide resources if the user has permissions
        var perm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_RESOURCE);  
        if(perm && perm[PermissionsClient.ACTION_ARCHIVE]) {
            this.hideButton = this.tableViewer.addAction("Hide", RtlmUiUtil.getMessage("labelHideButton"), "hide", "hide-icon-image", dojo.hitch(this, "_archiveSelected"));
            this.hideButton.setDisabled( true );

            this.deleteButton = this.tableViewer.addAction("Delete", RtlmUiUtil.getMessage("labelDeleteButton"), "delete", "delete-icon-image", dojo.hitch(this, "_archiveSelected"));
            this.deleteButton.setDisabled( true );
        }           
	},
		
	_checkButtonStatus: function()
	{
        if (this.assetType == "Machine") {
            this._checkReserveButtonStatus();
            this._checkUnReserveButtonStatus();
            this._checkConfigureButtonStatus();
        }
        else if( this.assetType == "VirtualImage" ) {
            this._checkDeployButtonStatus();
        }
        this._checkDeleteButtonStatus();		
        this._checkHideButtonStatus();        
	},
	
	//Enable if ONLY unreserved machines are selected
	_checkReserveButtonStatus : function()
	{
		var enable = true;
		var rows = this.tableViewer.getCheckedRows();
        var rows_length = rows.length;
		if (rows_length == 0)
			enable = false;
		else
		{
			//Loop over all the rows.  If any are already reserved then disable the reserve button
			for (var i = 0; i < rows_length; i++) 
			{
				var rowData = rows[i];
                var reservePerm = this._checkActionPermissionForMachineId(rowData["machine.id"], "canReserve");

				if ((reservePerm == false) || this._isMachineReserved(rowData["machine.id"])) 
				{
					enable = false;
					break;
				}
			}
		}
		this.reserveButton.setDisabled(!enable);
	},
	
	//Enable if ONLY reserved machines are selected
	_checkUnReserveButtonStatus : function()
	{
		var enable = true;
		var rows = this.tableViewer.getCheckedRows();
        var rows_length = rows.length;
		if (rows_length == 0)
			enable = false;
		else
		{
			//Loop over all the rows.  If any are already reserved then disable the reserve button
			for (var i = 0; i < rows_length; i++) 
			{
				var rowData = rows[i];
                var reservePerm = this._checkActionPermissionForMachineId(rowData["machine.id"], "canReserve");       
				if ((reservePerm == false) || !this._isMachineReserved(rowData["machine.id"])) 
				{
					enable = false;
					break;
				}
			}
		}
		this.unReserveButton.setDisabled(!enable);
	},
	
	_checkConfigureButtonStatus : function()
	{
		//Only enable automations if anyone of the machines applicable
		var checkedRows = this.tableViewer.getCheckedRows();
        var checkedRows_length = checkedRows.length;
        if( checkedRows_length == 0 )
        {
            this.configureButton.setDisabled(true);
            return;
        }
        
        // quick check to see if any of the selected machines that the
        // user can't run automation, before checking for automatable.
        for (var i = 0; i < checkedRows_length; i++) 
        {
            var rowData = checkedRows[i];
            var automatePerm = this._checkActionPermissionForMachineId(rowData["machine.id"], "canAutomate");
            if (automatePerm == false) 
            {
                this.configureButton.setDisabled(true);
                return;
            }
        }
		
		var inven = new com.ibm.rational.test.lm.web.client.internal.inventory();
		var numdone = 0;
		var enable = false;
		this.automatable = {};
		
		for( var iRow = 0; iRow < checkedRows_length; iRow++ )
		{
			var id = this.tableContentProvider.getRowDataId( checkedRows[ iRow ] );
			var checkHandler = new ServiceResponseHandler
			(
				{
					parent: this,
					machineid: id,
					_success: function( response )
					{
						numdone++;
						if( response && ( response.length > 0 ) )
						{
							this.parent.automatable[ this.machineid ] = true;
							enable = true;
						}
						
						if( numdone == checkedRows.length )
						{
							this.parent.configureButton.setDisabled( !enable );
						}
					},
					_failure: function( errObj )
					{
						this.parent.automatable = {};
						this.parent.configureButton.setDisabled(true);
					}
				},
				"_success",
				"_failure"
			);
			
			inven.getAutomationsForAsset( checkHandler, id );
		}
	},
	
	//Enable if ANY Virtual Images are selected
	_checkDeployButtonStatus: function()
	{
		var checkedRows = this.tableViewer.getCheckedRows();
        var virtualizePerm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_VIRTUALIZATION);  
        var canVirtualize = (!virtualizePerm || virtualizePerm[PermissionsClient.ACTION_DEPLOY]);
                    
		this.deployButton.setDisabled( checkedRows.length == 0 || !canVirtualize );
	},

    _checkDeleteButtonStatus: function()
    {
        var checkedRows = this.tableViewer.getCheckedRows();
        if(this.deleteButton)
            this.deleteButton.setDisabled( checkedRows.length == 0 );
    },

    _checkHideButtonStatus: function()
    {
        //Only enable automations if anyone of the machines applicable
        var checkedRows = this.tableViewer.getCheckedRows();
        if(this.hideButton)
            this.hideButton.setDisabled( checkedRows.length == 0 );
    },
	
	//Run automations on the selected machines
	_automateSelected: function( event )
	{
		var rows = this.tableViewer.getCheckedRows();
		var machines = { "Data": [], "Header": ["machine.id", "machine.name"] };
		var rows_length = rows.length;
		for( var iRow = 0; iRow < rows_length; iRow++ )
		{
			var id = this.tableContentProvider.getRowDataId( rows[ iRow ] );
			if( this.automatable && this.automatable[ id ] )
			{
				machines.Data.push( [ id,
								 	  this.tableContentProvider.getCellDisplayData( rows[ iRow ], this.tableContentProvider.getColumnIndex( "machine\\.name" ) ) ] );
			}
		}
		
		dojo.setObject( "com.ibm.rational.test.lm.web.machines", machines );
				
		var uri = InventoryUriUtil.getAutomationsURI();
		
		window.location.href = uri;
	},
	
	//Deploy selected virtual images
	_deploySelected: function(){
		var rows = this.tableViewer.getCheckedRows();
		var vis = [];
		var rows_length = rows.length;
		for (var iRow = 0; iRow < rows_length; iRow++) {
			vis.push(this.tableContentProvider.getRowDataId(rows[iRow]));
		}
		
		dojo.setObject( "com.ibm.rational.test.lm.web.virtualimages", vis );
		
		var uri = InventoryUriUtil.getDeploymentURI();
		
		window.location.href = uri;
	},
		
	//Reserve selected machines
	_reserveSelected : function()
	{
		var rows = this.tableViewer.getCheckedRows();
        var rows_length = rows.length;
		this.ids = [];
		for( var iRow = 0; iRow < rows_length; iRow++ )
		{
			var id = this.tableContentProvider.getRowDataId( rows[ iRow ] ) ;
			this.ids.push(id);
		}
		
		//Get the server time to use in the 24 hour time span
		var self = this;
        var ids_length = self.ids.length;
		var handler = {
			success:function(response)
			{
				var serverDate = new Date(Number(response));
				for (var i = 0; i < ids_length; i++)
				{
					self.doResAction(self.ids[i], serverDate);
				}
			},
			failure:function(response)
			{
				//Couldn't get the server time so use the client time
				var fromDate = new Date();
				for (var i = 0; i < ids_length; i++)
				{
					self.doResAction(self.ids[i], fromDate)
				}				
			}
		}
		 	
		//Get the server time in order to ensure we have the right current time
		var responseHandler = new ServiceResponseHandler(handler, "success", "failure");
		adminService.getServerTime(responseHandler);

	},
	
	//unReserve selected machines
	_unReserveSelected : function()
	{
		var rows = this.tableViewer.getCheckedRows();
        var rows_length = rows.length;
		for( var iRow = 0; iRow < rows_length; iRow++ )
		{
			this.doUnResAction(rows[iRow]);
		}	
	},
	
	
	doResAction: function(id, fromDate)
	{
		if (!this._isMachineReserved(id))  //NOT reserved already
		{
			var responseHandler = new ServiceResponseHandler(this, "_reserveSuccess", "_reserveError");
			
			//The default reservation time span is from "now" till 24 hours from now
			var toDate = new Date();
			toDate.setDate(fromDate.getDate() + 1);
			
			//Create the reservation
			this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("msgReserving"));
			reservationService.createReservation(responseHandler, id, fromDate.getTime(), toDate.getTime());
		}
	},

	doUnResAction : function(row)
	{
		var id = this.tableContentProvider.getRowDataId(row) ;

 		if (this._isMachineReserved(id))  //Is it reserved
		{
			this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("msgUnReserving"));
			var resID = this._getResID(id);
			if (resID != null)  //should never be null but check anyway
			{
				var responseHandler = new ServiceResponseHandler(this, "_unReserveSuccess", "_unReserveError");
				var reservations = new com.ibm.rational.test.lm.web.client.internal.reservations();
				this.resToUnreserve = resID;  //MFV THIS IS A HACK, THE cancelReservation method does not return the reservation or the ID of the canceled res
				reservations.cancelReservation(responseHandler, resID);
			}
		}
	},

	//Return the reservation ID for the give machineID.
	//Note that it is possible for there to be more than one reservation in the reservationData set for a machineID.
	//This could occur if someone reserved a machine for short time spans - ex. from 8-12 and 12-4 on the same day
	//In this case we will only be returning the first reservation we find for the machine	
	_getResID: function(machineID)
	{
        var res_length = this.reservationData.length;
		for (var i = 0; i < res_length; i++)
		{
			if (this.reservationData[i].resourceId == machineID)
			{
				return this.reservationData[i].reservations[0].id;
			}
		}
		return null;
	},

	//The reservation succeeded so redisplay the table using the latest reservation info		
	_reserveSuccess:function(response)
	{
		this.viewHeaderNode.setProgressEnd();
		this._reloadTableDueToResChange();
		this._checkButtonStatus();
	},
	
	_reserveError:function(response)
	{
		this.viewHeaderNode.setProgressEnd();
		this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("msgReserveMachineFailed"), ViewHeaderLayout.ERROR);		
		this._checkButtonStatus();
	},
	
	//The UnReservation succeeded so redisplay the table using the latest reservation info		
	_unReserveSuccess:function(response)
	{
		this.viewHeaderNode.setProgressEnd();
		this._reloadTableDueToResChange();
		this._checkButtonStatus();
	},

	_unReserveError:function(response)
	{
		this.viewHeaderNode.setProgressEnd();
		this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("msgUnReserveMachineFailed"), ViewHeaderLayout.ERROR);
		this._checkButtonStatus();
	},
	
	
	//Determine if there is any reservation data for the specified machineID
	_isMachineReserved:function(machineID)
	{
        var res_length = this.reservationData.length;
		for (var i = 0; i < res_length; i++)
		{
			if (this.reservationData[i].resourceId == machineID)
			{
				return true;
			}
		}
		return false;
		
	},
	
	//Return a useful string containing the name of the user who reserved the machine and the time span
	//If there is no reservation, return an empty string.
	_reservedBy: function(machineID)
	{
		//loop over the reservation data to see if the machine is reserved for the magic time period
		//If it is then return the user name of the person for whom the machine is reserved
		var retVal = "";
        var res_length = this.reservationData.length;
		for (var i = 0; i < res_length; i++)
		{
			if (this.reservationData[i].resourceId == machineID)
			{
				var resFor = this.reservationData[i].reservations[0].reservedFor.name;
				var tillDate = this.reservationData[i].reservations[0].reserveTo;
				var resTill = RtlmUiUtil.getDateDisplay(tillDate, false);
				retVal = RtlmUiUtil.getMessage("resbyuntil", resFor, resTill);
				break;
			}
		}
		return retVal;
	},
	
	//Run the search
	_searchAssets: function()
	{
        this.permissionHash = {};
        this._clearTable();
		this.loadTable();
	},
	
    // Create an query result object from the actual asset. Process all of the
    // attributes that we specified for the search result criteria.
    //
    assetObjToQueryObj: function(asset) {
        // make sure we've an asset object first
        if(!(asset instanceof com.ibm.asq.common.web.client.internal._catalog.Asset)) {
            return null;
        }
        var rootObj = asset.getRoot();
        if (!rootObj) {
            return null;
        }
        
        // get all of the attribute names that we're displaying from the
        // criteria string.
        var colnames = this._searchAttributeCriteria.replace(/, /g,',');
        colnames = colnames.split(',');
                
        var queryObj = [];
        
        // the id is always first. 
        queryObj.push(asset.getId());

        // process all of the attributes that we're showing for the asset
        // type. The attributes are specified in full path names, like 'machine.operatingsystem'.
        // extract the non-root part of the name.
        var colnames_length = colnames.length;
        for(var i = 0; i < colnames_length; i++) {
            var attrName = colnames[i].split('.')[1];
            var value = "";
            
            // the 'id' is not really an attribute in the asset model.
            // skip over it.
            if(attrName == "id")
                continue;
                
            if(rootObj.isAttributeArray(attrName)) {
                var arr = rootObj.getAttributeArrayValues(attrName);
                var arr_length = arr.length;
                for(var j = 0; j < arr_length; j++) {
                    value += arr[j].getName();
                    
                    // show the 'pipe' divider for multiple
                    // values.
                    if(j + 1 < arr_length)
                        value += "|";
                }
            }
            else {
                value = rootObj.getAttributeValue(attrName);
                if(value && rootObj.getAttribute(attrName).getType() == "PointerType") {
                    var pointer = value;
                    value = pointer.getDisplayString();
                }
            }
            queryObj.push(value);
        }
        return queryObj;
    },
    
    // handler on artifacts changed
    onArtifactChanged: function(changeType, artifactType, artifactArray) {
        var displayAssetType = (this.assetType == "Machine") ? RtlmUiUtil.TYPE_MACHINE : (this.assetType == "VirtualImage") ? RtlmUiUtil.TYPE_VM : this.assetType;

        // only care about machines or vm that is the currently displayed type
        if (artifactType == displayAssetType) {
            // convert asset objects to query result objects
            var queryObjArray = [];
            var artifactArray_length = artifactArray.length;
            for (var i = 0; i < artifactArray_length; i++) {
                var queryObj = this.assetObjToQueryObj(artifactArray[i]);
                if (queryObj) {
                    queryObjArray.push(queryObj);
                } else {
                    // can't convert asset object to query result object
                    this._searchAssets();
                    return;
                }
            }
            // get the current result table content
            var content = this.tableViewer.getContent();
            if (changeType == GlobalEvents.ArtifactsAdded) { 
                // add new assets to result table
                content.Data = content.Data.concat(queryObjArray);
                this.tableViewer.setContent(content);
                this.tableViewer.refresh();
            } else {
                // iterate through the changed assets
                var queryObjArray_length = queryObjArray.length;
                var content = content.Data;
                var content_length = content.length;
                for (var i = 0; i < queryObjArray_length; i++) {
                    // iterate through the array of assets in the table
                    for (var j = 0; j < content_length; j++) {
                        // handle the matching asset
                        if (queryObjArray[i][0] == content[j][0]) {
                            if (changeType == GlobalEvents.ArtifactsUpdated) {
                                // swap asset on update
                                content.splice(j, 1, queryObjArray[i]);
                            } else if (changeType == GlobalEvents.ArtifactsDeleted) {
                                // remove asset from table on delete
                                content.splice(j, 1);
                                this.tableViewer.setContent(content);
                                this.tableViewer.refresh();
                            }
                            break;
                        }
                    }
                }
                this.tableViewer.setContent(content);
                this.tableViewer.refresh();
            }
        }
    }

});

})();	