/*
 * IBM Confidential
 * (C) Copyright IBM Corp. 2008. All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *
 * All rights reserved
 *
 * Author: Paul Beaulieu
 */

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.DeployOperationsTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() {
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var COLLECTION = 2;
	var ASSET = 0;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var column_types = [
						TableConstants.RENDER_TYPE_LINK,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TIMESTAMP,
						TableConstants.RENDER_TYPE_TIMESTAMP,
						TableConstants.RENDER_TYPE_LINK,
						TableConstants.RENDER_TYPE_TEXT ];
						
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.DeployOperationsTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
	constructor: function( column_headings, id_fields )
	{
		this.setParameters( { "column_headings" : this.getColumnHeadings(),
							  "id_fields" : this.getIdFields(),
							  "column_types" : column_types } );
	},
	
	getColumnHeadings : function()
	{
		return( [
		  {title: RtlmUiUtil.getMessage("vmmachines"), native_column: "virtualMachineName", isSortable: true, includeInFilter: true, width: 15}, 
		  {title: RtlmUiUtil.getMessage("Virtual Machine Status"), native_column: "status", isSortable: true, includeInFilter: true, width: 15  },
		  {title: RtlmUiUtil.getMessage("Start Time"), native_column: "startTime", isSortable: true, includeInFilter: true, width: 15  },
		  {title: RtlmUiUtil.getMessage("End Time"), native_column: "endTime", isSortable: true, includeInFilter: true, width: 15  },
		  {title: RtlmUiUtil.getMessage("VirtualImages"), native_column: "virtualResourceName", isSortable: true, includeInFilter: true, width: 15  },
		  {title: RtlmUiUtil.getMessage("Provider"), native_column: "providerName", isSortable: true, includeInFilter: true, width: 25  }
		] );
	},
	
	getIdFields: function()
	{
		return( "virtualResourceUUID" );
	},
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {    
		var type = this.inherited( "getCellDisplayType", arguments );
		//Ensure that removed virtual machines are not links
		
		if( rowData[ "resourceType" ] == COLLECTION && ( atColumnIndex == 0 ) && !( rowData[ "virtualMachineCollection" ] ) )
		{
			return( TableConstants.RENDER_TYPE_TEXT );
		}
		
        return( ( ( atColumnIndex == 0 ) && ( rowData[ "status" ] == "Removed" ) ) ? TableConstants.RENDER_TYPE_TEXT : type );
    },
	
	getCellDisplayData: function(rowData, atColumnIndex)
	{
		if( rowData[ "resourceType" ] == COLLECTION && atColumnIndex == 0 )
		{
			return( ( rowData[ "virtualMachineCollection" ] ) ? rowData[ "virtualMachineCollection" ][ "name" ] : RtlmUiUtil.getMessage("Unresolved") );
		}
		else if( rowData[ "resourceType" ] == COLLECTION && atColumnIndex == 4 )
		{
			return( rowData[ "virtualResourceName" ] );			
		}
		else
		{
			return( this.inherited( "getCellDisplayData", arguments ) );
		}
	},
	
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
 		var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
        if(atColumnIndex == 0) {
            val = {};
			if (rowData["resourceType"] == COLLECTION && rowData["virtualMachineCollection"] ) {
				val['href'] = "#action=com.ibm.rqm.planning.home.actionDispatcher&subAction=viewTestCell&cellID=" + rowData["virtualMachineCollection"]["id"];
			}
			else {
				val['href'] = AssetUriUtil.getViewAssetDetailURI("Machine", rowData["virtualMachineUUID"]);
			}
        }
        else if(atColumnIndex == 4) {
            val = {};
			if (rowData["resourceType"] == COLLECTION) {
				val['href'] = "#action=com.ibm.rqm.planning.home.actionDispatcher&subAction=viewTestCell&cellID=" + rowData["virtualResourceUUID"];			
			}
			else {
				val['href'] = AssetUriUtil.getViewAssetDetailURI("VirtualImage", this.getRowDataId(rowData));
			}
        }

        return val;
    },
    
    getInitialSortDirection: function()
    {
    	return( TableConstants.SORT_DIR_DESCEND );
    },
    
    getInitialSortByColumn: function()
    {
    	return( 2 );
    }      
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.DeployOperationsTableSorter",
				com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter,
{
    sortBy: function(viewer, objects, column, sortDir)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects);
        
        this.viewer = viewer;
        this.groupCol = -1;
        this.sortCol = column;
        this.sortDir = sortDir;
        sortContainer.sort(dojo.hitch(this, "_sortComparator"));
        
        var collate = [];
        collate.push(sortContainer);
        return collate;
    },
    
    groupSortBy: function(viewer, objects, /*primary key (group by)*/column1, /*secondary key*/column2, sortDir)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects); 
        
        this.viewer = viewer;
        this.groupCol = column1;
        this.sortCol = column2;
        this.sortDir = sortDir;
        this.groupSortDir = TableConstants.SORT_DIR_DESCEND;
        sortContainer.sort(dojo.hitch(this, "_groupComparator"));
        
        var collate = [];
        var groupVal = null;
        var aGroup = null;
        
        // collate the objects into groups bins
        for(var i = 0; i < sortContainer.length; i++) {
            var val = this.viewer.getContentProvider().getCellDisplayData(sortContainer[i], this.groupCol);
            if(val != groupVal) {
                if(aGroup != null) {
                    collate.push(aGroup);
                    aGroup = null;
                }
                if(aGroup == null)
                    aGroup = [];
                groupVal = val;
            }
            aGroup.push(sortContainer[i]);
        }
        if(aGroup)
            collate.push(aGroup);
        return collate;
    },
    
    _groupComparator: function(obj1, obj2)
    {
        var val1 = obj1[ "startTime" ];
        var val2 = obj2[ "startTime" ];

        // this is a time stamp type. Convert the timestamp value to a Date
        // object for the comparison.
        //
        if (val1 && val2) {
	        var date1 = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(val1);
	        val1 = date1.getTime();
	        var date2 = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(val2);
	        val2 = date2.getTime();
        }
        
        // same group, try secondary sorting.
        if(val1 == val2) {
            // there's a secondary sort and it's not the same as the group
            // sort column, sort by it. 
            if(this.sortCol > -1 && this.groupCol != this.sortCol)
                return this._sortComparator(obj1, obj2);
                
            return 0;
        }
        
        if(this.groupSortDir== TableConstants.SORT_DIR_ASCEND) {
            return (val1 > val2) ? 1 : -1;
        }
        else {
            return (val1 > val2) ? -1 : 1;            
        }
    },
    
    _sortComparator: function(obj1, obj2)
    {
        var val1 = this.viewer.getContentProvider().getCellDisplayData(obj1, this.sortCol);   
        var val2 = this.viewer.getContentProvider().getCellDisplayData(obj2, this.sortCol);
        
        // this is a time stamp type. Convert the timestamp value to a Date
        // object for the comparison.
        //
        if(val1 && val2) {
            if(this.viewer.getContentProvider().getCellDisplayType(obj1, this.sortCol) == TableConstants.RENDER_TYPE_TIMESTAMP) {
                var date1 = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(val1);
                val1 = date1.getTime();
            }
            if(this.viewer.getContentProvider().getCellDisplayType(obj2, this.sortCol) == TableConstants.RENDER_TYPE_TIMESTAMP) {
                var date2 = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(val2);
                val2 = date2.getTime();
            }
        }
        
        if(val1 == val2)
            return 0;
        
        if(this.sortDir == TableConstants.SORT_DIR_ASCEND) {
            return (val1 > val2) ? 1 : -1;
        }
        else {
            return (val1 > val2) ? -1 : 1;            
        }
    } 
});	
})();