/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestWidget");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var LOADING = RtlmUiUtil.getMessage("reqMsgRetrievingRequestsWorking");
var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;

// Summary:
//    Provides the request query display. This view displays a list of "canned" queries
//    for requests in the system that is specific to the current user. The view displays
//    the results of the selected query in a table. When the page is constructed, it automatically
//    runs the "default" query and display the results. 
// 
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestView",
     [dijit._Widget, dijit._Templated],
{
    //The layout of the page.
    widgetsInTemplate: true,
    templateString: 
    '<div id="${id}" dojoAttachPoint="domNode">' +
    '    <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout"' +
    '         isCompact="true" noline="true" dojoAttachPoint="viewHeaderNode">' +      
    '        <!-- The table -->  '+
    '        <div dojoAttachPoint="requestsNode">'+
    '       </div>' +
    '    </div>'+
    '</div>',
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.viewHeaderNode.setProgressStart(LOADING);

        // set up the header area display of the view.
        this.viewHeaderNode.setTitle(RtlmUiUtil.getMessage("reqLabel_requests"));
        
        // set up the CS help id
        this.viewHeaderNode.setCSHelpId("rtlm_request_list.html", com.ibm.rational.test.lm.web.CSHelpID);
        this.viewHeaderNode.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelHelp", "", RtlmUiUtil.getMessage("reqLabel_requests")));
		var params = {
			processStartCallback: dojo.hitch(this.viewHeaderNode, "setProgressStart"),
			processEndCallback: dojo.hitch(this.viewHeaderNode, "setProgressEnd")};
		this.queryRequestWidget = new com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestWidget(params);
        this.requestsNode.appendChild(this.queryRequestWidget.domNode);
		this.resultTable= this.queryRequestWidget.resultTable;
    },
	
	_onRefreshActionClick: function()
	{
		this.runSelectedQuery();
	},
	
	runSelectedQuery: function() {
		this.queryRequestWidget.runSelectedQuery();
	},

    getController: function() {
        return this;
    },

    // handler on artifacts changed
    onArtifactChanged: function(changeType, artifactType, artifactArray) {
        // only care about requests
        if (artifactType == RtlmUiUtil.TYPE_REQUEST) {
            this.runSelectedQuery();
        }
    }
});

})();
