/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestWidget");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.QueryResultWidget");
dojo.require("com.ibm.rational.test.lm.web.client.internal.request");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.RequestTableProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");


(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var UriUtil = com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil;
var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;
var LOADING = RtlmUiUtil_getMessage("reqMsgRetrievingRequestsWorking");

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestWidget",
    com.ibm.rational.test.lm.web.ui.internal.widgets.QueryResultWidget,
{
	ALL_UNASSIGNED_REQUEST_QUERY_ID: 1,
	ALL_ASSIGNED_REQUEST_QUERY_ID: 2,
	ASSIGNED_TO_ME_REQUEST_QUERY_ID: 3,
	MY_OPEN_REQUEST_QUERY_ID: 4,
	MY_FULFILLED_REQUEST_QUERY_ID: 5,
	defaultQuery: null,
	processStartCallback: null,
	processEndCallback: null,
    tableSortColumn: 0,
    tableSortDirection: -1,
	
    iconMap: {'High' : 'priority_high_obj.gif',
              'Normal' : 'priority_med_obj.gif',
              'Low' : 'priority_low_obj.gif'},
	
	constructor: function() {
        this.label = RtlmUiUtil_getMessage("request_view_label");
        this.tableId = "RQM-LM-REQUEST-QUERY-TABLEVIWER";
    },
	
    postCreate: function()
    {
        this.inherited('postCreate', arguments);
        this.refreshAction = this.resultTable.addAction(
                "refresh",                                // action id
                RtlmUiUtil_getMessage("Refresh"),         // label
                null,                                     //callback data
                "refresh-icon-image",                     //icon class
                dojo.hitch(this, "runSelectedQuery"));    // callback
        
        this._projectAlias = com.ibm.asq.common.web.ui.internal.util.Utility.getCurrentProjectAreaAlias();
        this.pdfAction = this.resultTable.addAction(
                "pdf",                                // action id
                RtlmUiUtil.getRQMMessage("labelPDFView"),         // label
                null,                                     //callback data
                "pdf-icon-image",                     //icon class
                dojo.hitch(this, "printPDF"));    // callback
        
        this.resultTable.setContent( [] );
        this.resultTable.show();
    },
    
	getTableContentProvider: function() {
        var requestTableProvider = new com.ibm.rational.test.lm.web.ui.internal.views.providers.RequestTableProvider();
		return(requestTableProvider);
	},

    loadQueryCombo: function() {
		var request = new com.ibm.rational.test.lm.web.client.internal.request();
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadViewsSuccess", "_loadDataError");
        
        // loads the queries for the user.    
        request.getViewIdsForCurrentUser(responseHandler);      
    },
    
    // Summary:
    //    Callback to handle the returned canned queries for the user.
    //
    _loadViewsSuccess: function(response) {
        if(response) {
			var selectors = [];
            var response_length = response.length;
            for(var i = 0; i < response_length; i++) {
				var selector = {name:RtlmUiUtil_getMessage(response[i].name), value:response[i].id, def:response[i].id == this.defaultQuery};
				selectors.push(selector);
	        }
			this.resultTable.setTableContentSelectors(selectors);
			this.resultTable.setTableListener(this);

			if(!this.defaultQuery)
			     this.resultTable.contentSelect.options[0].selected = true;
				 
	        // initialize the page to display the default view.
	        this.runSelectedQuery();
			if(this.processEndCallback)
			    this.processEndCallback();
        }    
    },
    
    _loadDataError: function() {
        if(this.resultTable)
            this.resultTable.endLoadingProgress();
		if(this.processEndCallback)
            this.processEndCallback();
    },
	
	//Callback from the table when the view selector is changed
	tableContentSelectionChanged : function(table, selectedId)
	{
		this.runSelectedQuery(selectedId);
	},
	
	// Summary:
    //    Runs the currently selected query and displays the returned results.
    //
    runSelectedQuery: function(id) {
        if (id == null)
			id = this.getQuerySelectionId();
        
        if(id) {
            var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_getRequestQueryResultsSuccess", "_loadDataError");
            var request = new com.ibm.rational.test.lm.web.client.internal.request();
            request.getViewData(responseHandler, id);
            
            // clear the table and show loading
            this.resultTable.setContent( [] );
            this.resultTable.refresh();
            this.resultTable.showLoadingProgress();
            
			if(this.processStartCallback && this.processStartMessage)
			    this.processStartCallback(this.processStartMessage);
        }    
    },
    
    printPDF: function() {
    	var type = "request";
        var requests = this.resultTable.orderedObjects[0];
        var loc = window.location.href;
		loc = loc.replace(/[&]/g,"&amp;");
		var summary = true;
		this.resultTable.sendPdfPrintRequest(requests,type,loc,this._projectAlias, summary);
    },
	
	// Summary:
    //    Handler method for the returned query result data. This method "normalizes"
    //    the response data into the table expected row format and loads the new set of
    //    rows into the query result table.
    // 
    _getRequestQueryResultsSuccess: function(response) {
        this.resultTable.endLoadingProgress();
        this.resultTable.setContent( ( response ) ? response : [] );
		if(this.pdfAction){
			if(!response || !response.length){
				this.pdfAction.setDisabled(true);
			}else{
				this.pdfAction.setDisabled(false);
			}
		}
        this.resultTable.refresh();
        if(this.processEndCallback)
             this.processEndCallback();
    },
	
	// Summary:
    //     Returns the priority icon display name for a given priority value. Returns null if no
    //     prority is specified (null, or empty).
    //
    getPriorityDisplay: function(dataValue)
    {
        var priorityDisplay = null;                        
        var iconName = this.iconMap[dataValue];
        
        if(iconName) {
            priorityDisplay = dojo.moduleUrl("com.ibm.rational.test.lm.web", "ui/internal/images/icons/obj16/" + iconName);
        }    
        return priorityDisplay;
    },
    
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {
        if(callback) {
            callback(callbackData);
        }                     
    }
    
});

})();
