/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
DatePicker = dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");

// convenient constant to specify today and yesterday
DatePicker.today = new Date();
DatePicker.yesterday = new Date().setDate(new Date().getDate() - 1);

dojo.require("dojo.date");
dojo.require("dijit.form.DateTextBox");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.DatePicker", dijit.form.DateTextBox, {
    nonExistDateMessage: "",
    minDate: null,
    maxDate: null,
	isInDialog: false,
    dateFormat: com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatLength(),
    imgSrc: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/images/icons/elcl/calendar.gif"),
    imgSrcDisabled: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/images/icons/dlcl/calendar.gif"),
    _calendarNode: null,
    _calendarImg: null,
    _calendarImgDisabled: null,
	popupCalendarOnFocus: false,
    
    constructor: function(){
    },
    
    // Override the compare to provide a date only compare.
    compare: function(date1, date2){
        // Handling of the 'normalization' of dojo.date.compare. If date2 
        // is null, the campare method makes it to today's date. This would
        // cause selection of today's date not reported as onChange the first time. 
        // The base class's setValue handler would only trigger the onChange when the
        // last changed value and the new value are different, which the initial
        // last changed value is null.
        //
        if (!date1 && date2) {
            return -1;
            
        } 
        else if(date1 && !date2) {
            return 1;
        }
        else {
            return dojo.date.compare(date1, date2, "date");
        }
    },
    
    postCreate: function(){
        this.inherited('postCreate', arguments);

        this.constraints = {selector: 'date', formatLength: this.dateFormat};
	
        if (this.minDate)
            this.constraints['min'] = this.minDate;
        if (this.maxDate) 
            this.constraints['max'] = this.maxDate;
        
        // on IE, the mozilla inline stack style is not honored.
        // make our dom node style inline to avoid line breaks before & after    
        if(dojo.isIE) {
            dojo.style(this.domNode, "display", "inline");
        }
        // Init the widget to create a calendar pop up button to the parent node.
        this.init();
    },
    
    // Summary:
    //    Convenient method to dynamically set the date picker's min or max 
    //    constraint. 
    setConstraint: function(type, value) {
        if(type != "min" && type != "max") {
            console.debug("unsupported constraint type: " + type);
            return;
        }
        if(value) {
            this.constraints[type] = value;
        }
        else if(this.constraints[type]) {
            delete this.constraints[type];
        }
        // reset the calender's constraints with ours
        if (this._picker) {
            this._picker.constraints = this.constraints;
        }  
    },
    
    // Summary:
    //    Init the widget to create a calendar pop up button to the parent node.
    //    Note that the parent node may not exist yet so the consumer will need
    //    to invoke the init() method after the widget is attached to a parent.
    init: function(){
        var parentNode = ((dojo.isIE) ? this.domNode.parentElement : this.domNode.parentNode);
        if (parentNode) {
            this.fieldLabel = this.fieldLabel || MessageUtil.getMessage(MessageBundle.COMMON, "labelDateSelect"); 
            var calendarStr = this.fieldLabel;
            this._calendarNode = document.createElement("button");
            dojo.addClass(this._calendarNode, "icon-image-button");
            this.connect(this._calendarNode, "onclick", "_open");
            this._calendarNode.style.display = "inline";
			
            this._calendarNode.title = calendarStr;
            this._calendarImgDisabled = document.createElement("img");
            this._calendarImgDisabled.src = this.imgSrcDisabled;
            this._calendarImg = document.createElement("img");
            this._calendarImg.src = this.imgSrc;
            
            if (this.disabled) 
                this._calendarNode.appendChild(this._calendarImgDisabled);
            else 
                this._calendarNode.appendChild(this._calendarImg);
            this._calendarNode.disabled = this.disabled;

            parentNode.appendChild(this._calendarNode);
        }
    },
    
    setDisabled: function(/* Boolean*/disabled){
        if (this._calendarNode) {
            if (disabled) {
                if (!this.disabled) {
                    this._calendarNode.removeChild(this._calendarImg);
                    this._calendarNode.appendChild(this._calendarImgDisabled);
                }
            }
            else {
                if (this.disabled) {
                    this._calendarNode.removeChild(this._calendarImgDisabled);
                    this._calendarNode.appendChild(this._calendarImg);
                }
            }
            this._calendarNode.disabled = disabled;
        }
        this.inherited('setDisabled', arguments);
        if (disabled) 
            this.validate();
    },

    
    _open: function(){
        if(this.disabled){return;}

        var self = this;

        if(!this._picker){
            var popupProto=dojo.getObject(this.popupClass, false);
            
            this._picker = new popupProto({
                onValueSelected: function(value){

                    self.focus(); // focus the textbox before the popup closes to avoid reopening the popup
                    setTimeout(dojo.hitch(self, "_close"), 1); // allow focus time to take

                    // this will cause InlineEditBox and other handlers to do stuff so make sure it's last
					dijit.form.TimeTextBox.superclass._setValueAttr.call(self, value, true);
                },
                lang: this.lang,
                constraints:this.constraints,
                isDisabledDate: function(/*Date*/ date){
                    // summary:
                    //  disables dates outside of the min/max of the TimeTextBox. Specify
                    //  this is a date compare only. This removes any time component for the
                    //  comparison. 
                    return self.constraints && (dojo.date.compare(self.constraints.min,date,"date") > 0 || dojo.date.compare(self.constraints.max,date,"date") < 0);
                }
            });			
        }
        if(!this._opened){
            // if no value is in the date box, try to use the min date first
            // before resorting to the current date.
            var minDate = (this.constraints && this.constraints.min) ? this.constraints.min : new Date();
            var val = this.getValue() ? this.getValue() : minDate;
			this._picker.attr('value', val);
            dijit.popup.open({
                parent: this,
                popup: this._picker,
                around: this.domNode,
                onCancel: dojo.hitch(this, this._close),
                onClose: function(){ self._opened=false; }
            });
            this._opened=true;
        }
        
        dojo.marginBox(this._picker.domNode,{ w:this.domNode.offsetWidth });

        dijit.focus(this.domNode);
		if(this.isInDialog)
            // our common dialogs are set to 10002 on FF 3, need to 
            // make it higher then our dialogs.
			this._picker.domNode.parentNode.style.zIndex = 10003;
    },
    
    // Summary:
    //    Override the parent's method to add additional test on non-existing dates.
    //
    isValid: function(/* Boolean*/isFocused){
        var isValid = this.inherited('isValid', arguments);
        
        // add one more check to ensure that the entered string is a valid date.
        // dates like 02/30/07 passes the format validation, but the date doesn't 
        // exist. Our checkDate ensure that we keep the text box in invalid mode if
        // such a date is entered.
        if (isValid) 
            return this._checkDate();
        return isValid;
    },
    
    // Summary:
    //    Check for invalid date that passes the format validation. If a date string
    //    is valid in format, the date text box doesn't flag it as invalid. However,
    //    its date value is null. Add this check to make the date text box stay in 
    //    invalid mode.
    // 
    _checkDate: function(){
        var date = this.getValue();
        
        if (!date && this.textbox.value && this.textbox.value.length > 0) {
            this.invalidMessage = this.nonExistDateMessage;
            return false;
        }
        return true;
    },
	
	_onFocus: function( evt )
	{
		//Don't popup the calendar when the control gets the focus unless required.
		if (this.popupCalendarOnFocus) {
			this.inherited( "_onFocus", arguments );
		}
	},
	
	displayMessage: function(/*String*/message){
        if (this._message == message) {
            return;
        }
        this._message = message;

        // message is changed or maybe cleared
        if (this.tooltip) {
            this.tooltip.close();
            this.tooltip.destroy();
        }

        if (message) {
            this.tooltip = new jazz.ui.HoverPopup({html: message, openDelay: 300, around:[this.domNode]});
            this.tooltip.open();
        }
	},
	
	getValue: function() {
		return this.attr('value');
	},
	
	setValue: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
		this._setValueAttr(value, priorityChange, formattedValue);
	}	
	
});
