/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// author: vasrivas
//
// Summary: This is a widget to upload attachments
//
//

console.debug("Loading: com.ibm.asq.common.web.ui.internal.widgets.GenericAttachmentViewer");
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.GenericAttachmentViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dojo.io.iframe");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.attachmentViewer.AttachmentTableView");
dojo.require("dojo.parser");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");

(function() { // module container
var _textContent=dojo.isIE?"innerText":"textContent"; 

var Constants = com.ibm.asq.common.web.client.internal.Constants;

var AttachmentTableView			= com.ibm.asq.common.web.ui.internal.widgets.attachmentViewer.AttachmentTableView;
var TeamServerClient 			= com.ibm.team.repository.web.transport.TeamServerClient;
var getCurrentProjectArea		= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;
var UPLOAD_SERVICE_URI   		= "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/"; //RQM Upload Service
var DOWNLOAD_SERVICE_URI 		= "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/"; //RQM Downaload Service
var ACTION_PROP          		= "__jazzActionProp";
var FILE_UPLOAD_RESPONSE_PREFIX = "File Upload Response:"; // Must match response prefix in AttachmentRestService.java 
var _BYTES 						= " bytes";
var _currentBindRequest			= null;

function displayBlock(node, isDisplayed) {
	node.style.display= isDisplayed ? "block" : "none";
}

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.GenericAttachmentViewer", [dijit._Widget, dijit._Templated],
{
		templatePath: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/templates/GenericAttachmentViewer.html"),		
		/*
		 * General Summary of the widget
		 * This widget creates a upload form which can be used to select the file on a disk
		 * Creating Widget
		 	this.attTest = new com.ibm.asq.common.web.ui.internal.widgets.GenericAttachmentViewer();
			var attachmentObjects = new Array(); //Creating a mock attachment
			attachmentObjects.push({id: 1, size: 1024, fileName: "Vaibhav.txt", path: "C:\\Test\\ABC", uuid: "_ABCDEFDSAS"});
			this.attTest.initialize({isNonEditableMode: true, elements: attachmentObjects});
			this._test.appendChild(this.attTest.domNode);
		 *
		 * Getting Data Back
		 * var attachmentsArray = this.attTest.getData();
		 */
			
		labelToAddAnAttachment: Constants.getMessage("labelToAddAnAttachment"),
		labelNumber: Constants.getMessage("labelNumber"),
		
		postCreate: function() 
		{	
			this.editable = true;
			
			if(this._uploadForm != null)
			{
				this._initializeUploadInput();
				this._uploadForm_onsubmit_handle = this.connect(this._uploadForm, "onsubmit", "_performUploadSubmit");
			}
			this.reset();	
		},
		
		/*
		 * External Apis
		 */
		
		/*
		 * getAllAttachments()
		 * This can be called to get all the attachments present in the attachment table
		 * Eg. of Array objects
		 * {fileName: 'test.txt',id: 12,size: 4096,path: "C://Test/Project1"uuid: _UxtmJ01203452};
		 */
				
		getAllAttachments: function()
		{
			return this._attachmentArray;
		},
		
		/*
		 * getNewAttachments()
		 * This function will return only the newly added attachments
		 * Eg. of Array objects
		 * {fileName: 'test.txt',id: 12,size: 4096,path: "C://Test/Project1"uuid: _UxtmJ01203452};
		 */
		getNewAttachments: function()
		{
			var returnArray = []
			for(var i=0;i<this._attachmentArray.length ; i++)
			{
				if(this._attachmentArray[i].isNew)
					returnArray.push(this._attachmentArray[i]);
			}
			return returnArray;		
		},
		/*
		 * getRemovedAttachments()
		 * This function will return the attachments which are marked to be removed
		 * Eg. of Array objects
		 * {fileName: 'test.txt',id: 12,size: 4096,path: "C://Test/Project1"uuid: _UxtmJ01203452};
		 */
		getRemovedAttachments: function()
		{
			var returnArray = []
			for(var i=0;i<this._attachmentArray.length ; i++)
			{
				if(this._attachmentArray[i].isRemoved)
					returnArray.push(this._attachmentArray[i]);
			}
			return returnArray;		
		},
		
		/*
		 *initialize()
		 * First function to be called when the view is created.
		 * isPreviewMode: boolean
		 * isNonEditableMode: boolean
		 * elements: Array of Attachments
		 * Eg. of Array object
		 * {
				fileName: 'test.txt',
				id: 12,
				size: 4096,
				path: "C://Test/Project1"
				uuid: _UxtmJ01203452
			}; 
		 * downloadServiceUri: String 	eg. "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/"
		 * uploadServiceUri: String 	eg. "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/"
		 */
		
		initialize: function(args)
		{
			if(args != null)
			{
				this.parentSection = args.parentSection ? args.parentSection : null;
				this._attachmentArray = args.elements ? args.elements : [];	
				
				UPLOAD_SERVICE_URI = args.uploadServiceUri ? args.uploadServiceUri:  UPLOAD_SERVICE_URI;
				DOWNLOAD_SERVICE_URI = args.downloadServiceUri ? args.downloadServiceUri:  DOWNLOAD_SERVICE_URI;
				if(args.isPreviewMode)
				{
					this.editable = false; this.previewMode = true;
					this._showInPreviewMode();
					return;
					
				}else if(args.isNonEditableMode)
				{
					this.editable = false;
					dojox.data.dom.removeChildren(this._uploadForm);
					dojox.data.dom.removeChildren(this._message);
					dojox.data.dom.removeChildren(this._uploadDiv);
				}else //Editable Mode
				{
					this.editable = true;
				}
			}else
				this._attachmentArray = [];
			dojox.data.dom.removeChildren(this._attachmentTableBody);
			
			var showPath = true;
			if(args.showPath != undefined)
				showPath = args.showPath;
			var params = {
				attachments: this._attachmentArray,
				editable: this.editable,
				printable: this.parentSection != null ? this.parentSection.isPrintView() : false,
				showPath: showPath,
				actions: this.getTableActions(),
				removeListener: {
					context: this,
					method:dojo.hitch(this, function(rowObject, state) {
		                    this.updateAttachment(rowObject);
		                }) 
					} 
			}
			this.attachmentTableViewer = new AttachmentTableView(params);
			this._attachmentTableBody.appendChild(this.attachmentTableViewer.domNode);
		},
	
	   showContentError: function(message)
       {
           console.debug("showContentError: " + message)
           this.attachmentTableViewer.tableViewer.showContentError(message);
       },
       
		/*
		 * Internal Functions
		 */
	
		_showInPreviewMode: function()
		{
			dojox.data.dom.removeChildren(this._uploadForm);
			dojox.data.dom.removeChildren(this._message);
			dojox.data.dom.removeChildren(this._uploadDiv);
			dojox.data.dom.removeChildren(this._attachmentTable);
			this._previewSection.style.display = "inline";
			this.previewSecitonHeading.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "labelAttachment");
			this.PrevAttachNumber.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "labelNumber");
			this._noOfAttachments.innerHTML = this._attachmentArray.length;
		},
		
		destroyWidget: function() {
			this.cancelUploadInProgress();
		},
		
		reset: function() {
			this._attachmentArray= [];
			displayBlock(this._uploadForm, !isGuest());
			this._initializeUploadInput(); // reset the form
		},
		
		_initializeUploadInput: function() {
			var div= this._uploadInputDiv;
			dojox.data.dom.removeChildren(div);
			var inputLabel = document.createElement("label");
			var input1= document.createElement("INPUT");
			input1.type= "file";
			input1.name= "uploadFileInput";
			input1.className="UploadControl";
			this._uploadFileInput = input1;
			this._uploadFileInput_onchange_handle = this.connect(this._uploadFileInput, "onchange", "_performUploadSubmit");
			inputLabel.appendChild(input1);
			div.appendChild(inputLabel);
		},

		_getFileNameAndId: function(/* String id:filename*/ fileName) {
			var result = { id: " ", fileName: fileName };
			var fileNameColonIndex = fileName.indexOf(":");
			if(fileNameColonIndex > -1) {
				var idNameSplitArray = fileName.split(":");
				var attachmentId = idNameSplitArray[0];
				if(idNameSplitArray[0].indexOf("File Upload Response") > -1){//Response String added from the server, strip it
					attachmentId = idNameSplitArray[0].split("File Upload Response")[0];
				} 
				result.id = attachmentId;
				result.fileName = idNameSplitArray[idNameSplitArray.length -1];
			}
			return result;
		},

		_rebindAttachments: function() {
			this.attachmentTableViewer.clearContent();
			for (var i=0; i<this._attachmentArray.length; i++) {
				this._addAttachmentRow(this._attachmentArray[i]);
			}
			if(this.parentSection != null){
				this.parentSection.setDirty();
			}
		},

		_addAttachmentRow: function(attachment, isFirst) {
			this.attachmentTableViewer.appendAttachment(attachment);
			return;
		},
		
		_getSaveAnchor: function(attachment, text) {
			var saveAnchor= document.createElement("a");
			saveAnchor.className= "actionLink";
			saveAnchor.href = TeamServerClient._getServicePrefix() + DOWNLOAD_SERVICE_URI + attachment.uuid;
			saveAnchor[_textContent]=text;
			return saveAnchor;
		},

		updateAttachment: function(attachment)
		{
			if(attachment.action == "remove")
				this._performRemove(attachment);
			else if(attachment.action == "undoremove")
				this._performUndoRemove(attachment);
			else {
				console.error("Unknown action from genericattachmentviewer");
			}	
		},
		
		_performRemove: function(attachment) {
	 		for (var i=0; i<this._attachmentArray.length; i++) {
		 		if (this._attachmentArray[i].uuid == attachment.uuid) {
		 			if (this._attachmentArray[i].isNew) {
						this._attachmentArray.splice(i, 1);
					} else {
						this._attachmentArray[i].isRemoved = true;
					}
		 			break;
		 		}
	 		}
			this._rebindAttachments();
			if(this.parentSection != null){
				this.parentSection.setDirty();
			}
		},
		
		_performUndoRemove: function(attachment) {
	 		for (var i = 0; i< this._attachmentArray.length; i++) {
	 			if (this._attachmentArray[i].uuid == attachment.uuid) {
	 				this._attachmentArray[i].isRemoved = false;
	 				break;
	 			}
	 		}
			this._rebindAttachments();
			if(this.parentSection != null){
				this.parentSection.setDirty();
			}
		},

		_showFormHideMessage: function() {
			this._isUploadInProgress = false;
			this._initializeUploadInput(); // reset the form
		},
		
		isUploadInProgress: function() {
			return this._isUploadInProgress;			
		},
		
		_handleCancelClick: function(e) {
			e.preventDefault();
			this.cancelUploadInProgress();
		},
		
		cancelUploadInProgress: function() {
			if(this._currentBindRequest) {
				this._currentBindRequest.cancel();
				if(dojo.isIE) {
					dojo.io.iframe.setSrc (dojo.io.iframe._frame, "", true);
					this._currentBindRequest.isCancelled = true;
				} 
				if(dojo.isMozilla){
					window.stop();
				}
			}
			this._showFormHideMessage();
		},

		_performUploadSubmit: function(e, isRestart) {
			if(e) {
				e.preventDefault();	
			}
			this._isUploadInProgress = true;
			this.disconnect(this._uploadFileInput_onchange_handle);
			
			var fullPath;
			// Check to see if the file is already loaded
			if(isRestart) {
				fullPath = this._priorFullPath;
			} else {
				fullPath = this._uploadFileInput.value;
			}
			if (fullPath.length === 0) {
				this._showFormHideMessage();
				return;
			}
			this._priorFullPath = fullPath;
			var parts= fullPath.split("\\");
			var fileName= parts[parts.length-1];
			
			for (var i=0; i<this._attachmentArray.length; i++) {
				if (this._attachmentArray[i].fileName == fileName) {
					//fix defect 5940: check the fileName when uploading
					alert(Constants.getMessage("alertAttachmentHasBeenAdded", fullPath));
					this._showFormHideMessage();
					return;
				}			
			}
			var self= this;
			if(this._currentBindRequest != null) {
				this._currentBindRequest.cancel();
				this._currentBindRequest  = null;
				}
		/*
		 * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the 
		 * next one breaks. Resetting the iframe fixes the problem
		 * Unfortunately this code reaches into iframe private variables.
		 */
			if(dojo.io.iframe._frame) {
				document.body.removeChild(dojo.io.iframe._frame);				
			}
			var frameName = "attachmentIframe_" + new Date().getTime(); //$NON-NLS$
			dojo.io.iframe._iframeName = frameName;
			dojo.io.iframe._frame = null;
			
			var url= TeamServerClient._getServicePrefix() + UPLOAD_SERVICE_URI + "?projectId=" + getCurrentProjectArea().itemId;
			
			this._currentBindRequest = dojo.io.iframe.send({
				form: this._uploadForm,
				url: TeamServerClient._getServicePrefix() + UPLOAD_SERVICE_URI + "?projectId=" + getCurrentProjectArea().itemId,
				content: {
						mimetype: "text/html",
						fileFields: "uploadFileInput"
					},
				handleAs:  "html",
				handle: function(response, ioArgs) {
					if(response instanceof Error){
							if(response.dojoType && response.dojoType == "cancel") {
								return; // this is a cancelled request, ignore it
							}
							self._currentBindRequest  = null; // no need to cancel it any more
							alert(Constants.getMessage("alertErrorDuringFileUpload"));
							self._showFormHideMessage();
							console.debug("Error uploading file: " + response.message);
							return;
						}
					var data = response;
					// If the user presses cancel on Firefox or Stop, then we get an extra call on the next upload. Skip it
					// The data URI of the bad call is for the page and not the service
					if(data && data.documentURI && data.documentURI.indexOf("IAttachmentRestService") == -1) {
						self._wasUploadCancelled = true;
						self._performUploadSubmit(null, true);
						return;						
					}					
					if(self._currentBindRequest && self._currentBindRequest.isCancelled) {
						self._currentBindRequest = null; // no need to cancel it any more
						return;
						}
					self._currentBindRequest  = null; // no need to cancel it any more
					var uuidOrMessage = data.body.innerHTML; /* innerHTML OK */
					if(uuidOrMessage.indexOf(FILE_UPLOAD_RESPONSE_PREFIX) != 0) {
						alert(Constants.getMessage("alertFileUploadFailed", Constants.getMessage("alertFileUploadFailedReason")));
						console.debug("Upload failed. Response is [" + uuidOrMessage + "]");
						self._showFormHideMessage();
						return;
					}
					uuidOrMessage = uuidOrMessage.substring(FILE_UPLOAD_RESPONSE_PREFIX.length, uuidOrMessage.length);
					if (uuidOrMessage.substring(0,1) != "_") {
						// If an upload was cancelled we get one of these anomolous calls
						if(self._wasUploadCancelled) {
							self._wasUploadCancelled = false;
							return;
						}
						alert(Constants.getMessage("alertFileUploadFailed", uuidOrMessage));
						self._showFormHideMessage();
						return;
					}

					var message = uuidOrMessage.split(',');
					var size=0;
					if(message.length > 0) {
						uuidOrMessage = message[0];
						size = message[1];;
						var id = message[2];
						fileName = id +":"+fileName;
					}
					
					if(size=="0") {
						if(!confirm(Constants.getMessage("alertFileIsEmpty", fileName))) {
							self._showFormHideMessage();
							return;
						}						
					}
					
					var fileNameAndId = self._getFileNameAndId(fileName);				
					var attachment = {
						id: fileNameAndId.id,
						fileName: fileNameAndId.fileName,
						size: size,
						path: fullPath,
						uuid: uuidOrMessage,
						isNew: true
					};
					self._attachmentArray.push(attachment);
					self._rebindAttachments();
					
					self._showFormHideMessage();					
				}
			});
		},
		
		refresh: function()
		{
			//Do nothing
		},
		
		getTableActions: function() {    
            var actions = [];       
	            
            if (this.tableActions) {
                actions = actions.concat(this.tableActions);
            }

            return actions;
        },

		_error: function(errorObj)
		{
			alert(Constants.getMessage("ErrorMsg", errorObj.message));
		}
});
}
)();
