/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: Mark Victory
//
// Summary:
//

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.MachineReservationsPanel");

dojo.require("dijit.layout.ContentPane");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.rational.test.lm.web.client.internal.machine");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineReservationTableProvider"); 
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.scheduler.Scheduler");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util");
dojo.require("com.ibm.rational.test.lm.web.client.internal.reservations");
dojo.require("com.ibm.asq.common.web.client.internal.admin");

(function() 
{
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
    var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;

dojo.declare(
  "com.ibm.rational.test.lm.web.ui.internal.widgets.MachineReservationsPanel",
  [dijit.layout.ContentPane, dijit._Templated],
{
	widgetsInTemplate: true,
	templateString:
['<div id="${id}" dojoAttachPoint="domNode" style="padding:1px">' ,

' <div dojoType="com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection"', 
'         dojoAttachPoint="reservationsArea" ',
'         mainClass="actionPanel" ',
'         open="false">',
'   <table dojoAttachPoint="progressIndicator" border="0" cellpadding="0" cellspacing="0" width="100%" style="display: none">' ,
'     <tr align="left">' ,
'       <td align="right" valign="middle" width="25">' ,
'         <div class="viewProgressImage"><div>' ,
'       </td>' ,
'       <td valign="middle" width="100%">' ,
'         <span dojoAttachPoint="progressLabel" class="viewProgressLabel"></span>' ,
'       </td>' ,
'     </tr>' ,
'   </table>' ,
'   <!-- table resultarea --> ',
'   <table border="0" cellpadding="0" cellspacing="0" width="100%">' ,
'     <tr align="left">' ,
'       <td>' ,
'         <div dojoAttachPoint="resultDiv"  /div> ',
'       </td>' ,
'     </tr>' ,
'     <tr align="left">' ,
'       <td>' ,
'         <div dojoAttachPoint="statusDiv">' ,
'           <table dojoAttachPoint="statusContent" border="0" cellpadding="0" cellspacing="0" width="100%">' ,
'             <tr align="left">' ,
'               <td valign="middle" align="left">' ,
'                 <span dojoAttachPoint="statusLineNode" class="viewStatusLine"></span>' ,
'               </td>' ,
'             </tr>' ,
'           </table>' ,
'         </div>' ,
'       </td>' ,
'     </tr>' ,
'   </table>' ,
' </div> ',
'</div>'].join(""),

	//The ID of the machine we are showing reservations for
	machineID: null,
	
	//The label for the Panel
	panelLabel: null,

	postCreate: function()
	{
		var self = this;
		this.reservationsArea.toggle = function()
								  {
								  		if( !this.open )
										{
											//Load the table
											if (self.machineID != null)
											{
												self.loadTable();
											}
										}
										
										this.inherited( "toggle", arguments );
								  }
								  
		//Set the title
		if (this.panelLabel != null)
		{
			this.setLabel(this.panelLabel);
		}
		else
		{
			this.setLabel(RtlmUiUtil_getMessage("MachLabelReservations"));
		}
        
        // hide areas not being used in the section but may want to have the spacing to match other sections
        if (this.compact) {
            dojo.style(this.reservationsArea.infoNode, "display", "none");
        }
        this.reservationsArea.summaryNode.innerHTML = RtlmUiUtil_getMessage("reservationDesc");

        this.createScheduler(); 
        this.createTable(); 
	},
	
	_postPermCheck: function(permissions)
    {
        var perm = permissions;
        
        // if we are calling the permission fetch, the response is
        // an array.
        if(dojo.isArray(permissions)) {
            perm = permissions[0];
        }
        
        // if the user doesn't have permission to do reservation, Remove the reservation
        // controls from the scheduler.
        if(perm && typeof perm['canReserve'] != "undefined" && !perm.canReserve) {
            this.scheduler.hideReservationControls();
        }
        if (this.reservationsArea.open) {
            this.loadTable();
        }        
    },
    
    _postPermCheckErr: function(response)
    {
        this._postPermCheck(null);
    },
    
	load: function(id, resourceName)
	{
		if (id == null)
			return;
		
		this.machineID = id;
		this.machineName = resourceName;

        // If we have created with the permissions being already mixed in, we
        // can just use it. Otherwise, fetch it ourself.
        if(this.permissions) {
            this._postPermCheck(this.permissions);
            return;
        }
        var adminClient = new com.ibm.asq.common.web.client.internal.admin();
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_postPermCheck", "_postPermCheckErr");
        
        adminClient.getResourcePermissions(responseHandler, this.machineID);
	},
	
	createScheduler: function()
	{
//		this.scheduler = new com.ibm.asq.common.web.ui.internal.widgets.scheduler.Scheduler( null, false, "monthly" );
		this.scheduler = new com.ibm.asq.common.web.ui.internal.widgets.scheduler.Scheduler( { "entities": null, 
																							   "showentities": false, 
																							   "zoomlevel": "monthly" } );
		this.scheduler.addReserveClickListener( dojo.hitch( this, this.reserveMachine ) );
		this.resultDiv.appendChild( this.scheduler.domNode );
	},
	
	createTable: function()
	{
		this.resultTable = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-LM-ASSET-SECT-RESERVE-VIEW"                                
        });

        this.refreshActionNode = this.resultTable.addAction(
                "refresh",                       // action id
                RtlmUiUtil_getMessage("labelRefreshButton"), // label
                null,                            // callback data
                "refresh-icon-image",            // icon class
                dojo.hitch(this, "loadTable"));  // callback

        this.resultTable.setContentPageSize( 10 );
        var contentProvider = new com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineReservationTableProvider();
        this.resultTable.setContentProvider(contentProvider);
        var tableListener = new Reservation.TableListener();
        this.resultTable.setTableListener(tableListener);
        this.resultTable.setContent( [] );
		this.resultTable.setTableSummary(RtlmUiUtil_getMessage("labelTableSummary", RtlmUiUtil_getMessage("MachLabelReservations")));
        
		this.resultTable.show();
		this.resultDiv.appendChild( this.resultTable.domNode );
	},
	
	loadTable: function()
	{
		var machine = new com.ibm.rational.test.lm.web.client.internal.machine();
		var responseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadTable", "_errorFunc");
        
		this.setProgressStart(RtlmUiUtil_getMessage("resMsgRetrievingReservationsWorking"));
		
        if( this.machineID )
        {
        	machine.getReservationsFromMachine( responseHandler, this.machineID, true /*minimalData*/ );
        }
		else
			this.setProgressEnd();  //This shouldn't happen but just in case, take down the progress message		
	},
	
	_loadTable: function(response)
	{
		this.setProgressEnd();

		this.resultTable.setContent( ( response ) ? response : [] );
		this.resultTable.show();
		this.scheduler.clear();
		var entity = {name: this.machineName, id: this.MachineID};
		this.scheduler.addEntity( entity );
		
        var response_length = response ? response.length : 0;
		for( var iReservation = 0; iReservation < response_length; iReservation++ )
		{
			var start = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(response[ iReservation ].reserveFrom);
			var end = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(response[ iReservation ].reserveTo);
            var resBy = response[ iReservation ].reservedBy ? response[ iReservation ].reservedBy.name : null;
            var resFor = response[ iReservation ].reservedFor ? response[ iReservation ].reservedFor.name : null;
            
			this.scheduler.reserve( entity, start, end, false, resBy, resFor );
		}
		
		this.scheduler.update();         
	},
	
	_errorFunc: function(errorType, errorMessage) 
	{
		this.setProgressEnd();
		//Display the empty table otherwise the result is a funny looking blank area
		this.resultTable.setContent( [] );
		this.resultTable.show();         
		//Not sure what errors should do here.
	},
	
    _getDateDisplay: function(dateValue, dateOnly)
    {
        // Constant that represents the date equal to "-1".  This is the
        // convention used to represent a null date.  This convention is needed
        // so that we can indicate that we want to remove a date from a date field.
        var _NODATE = "1969-12-31T18:59:59.999-0500";
        
        if(dateValue == null)
            return "";
            
        // Check if the date is -1.  If it is we have no date specified
        // and we should display a blank.
        if( dateValue == _NODATE ) {
            return "";
        }
        var date = RtlmUiUtil.convertTimeStampToDate(dateValue);
        
        if(dateOnly)
            return dojo.date.locale.format(date, {selector:'date', formatLength:"short"});
        return RtlmUiUtil.getDisplayDateTimeString(date);
    },
	
	//Set the Panel's label
	setLabel: function(labelText)
	{
		if (labelText != null) 
		{
			this.panelLable = labelText;
			this.reservationsArea.setTitle(labelText);
		}
	},
	
	// hide the status line from the display
	//
	hideStatusLine: function() 
	{
		this.statusLineNode.style.display = "none";
	},
    
	// Summary:
	//    set the status message in the header. 
	//
	setStatus: function(status) 
	{
		this.statusLineNode.innerHTML = status;
	},
	
	// show the in-progress indicator
	//
	setProgressStart: function(message) 
	{
		this.progressLabel.innerHTML = message;
		this.progressIndicator.style.display = "inline";
	},
    
	// Hide the in-progress indicator
	//
	setProgressEnd: function() 
	{
		this.progressLabel.innerHTML = "";
		this.progressIndicator.style.display = "none";
	},
	
	reserveMachine: function( startTime, endTime )
	{
		var reservations = new com.ibm.rational.test.lm.web.client.internal.reservations();

		if (this._validateDates(startTime, endTime)) {
			var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_reserveSuccess", "_reserveError");
			reservations.fillReservationGaps(responseHandler, this.machineID, startTime.getTime(), endTime.getTime(), true);
		}
	},
	
	_validateDates :function(fromDate, toDate)
	{
		if (fromDate == null || toDate == null)
			return false;

		var fromDateLong = fromDate.getTime();
		var toDateLong = toDate.getTime();
		
		if (fromDateLong == null || toDateLong == null)
			return false;
		
		if (fromDateLong >= toDateLong)
			return false;
			
		return true;
	},
	
	_reserveSuccess:function(response)
	{
		this.scheduler.onFinishReservation( true );
		this.loadTable();
	},
	
	_reserveError:function(response)
	{
		this.scheduler.onFinishReservation( false );
	},

	getCollapsibleSection: function()
	{
		return( dijit.byId( this.reservationsArea.id ) );
	}
});

dojo.declare("Reservation.TableListener",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{    
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback) {
        callback(callbackData);
    }
});

})();
