/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The AttributeDisplay widget creates a labeled field for a given LM Catalog attribute
 //    object. The labeled input field is created based on the Catalog attribute type. It 
 //    displays the attribute value and, if editable, can accept new value for the attribute.  
 //    If the catalog attribute is a PART type, this widget will recursively 
 //    creates the sub-attribute fields if the attribute has a non-null/empty value, or it  
 //    is marked for display.
 //
 //    This widget also can have action controls. One is for adding sub-attributes if the attribute is a PART,
 //    and the other is for removing the attribute value from its parent part attribute. The action is only 
 //    created if the AttributeDisplay is editable. They can also be shown in low opacity when the field
 //    isn't under the mouse pointer.
  
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributeDisplay");

dojo.require("dijit.Tooltip");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Attribute");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Configuration");

dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.GroupUtil");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormDateField");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormLabelField");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextField");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextCheckboxField");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormEnumField");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormCheckboxLabelField");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function() {

var TYPE_LABEL='LabelType';
var TYPE_ENUM='EnumType';
var TYPE_PART='ConfigurationPartType';
var TYPE_NUMBER='NumericType';
var TYPE_STRING='StringType';
var TYPE_POINTER='PointerType';
var TYPE_TIMESTAMP='TimeStampType';
var TYPE_CHECKBOX='CheckboxType';

var RtlmUiUtil 		= com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var AttributeWidget = com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributeDisplay;
var LabelField      = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormLabelField;
var TextField       = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextField;
var EnumField       = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormEnumField;
var DateField       = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormDateField;
var NumericField    = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormNumericField;
var IPAddressField  = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormIPAddressField;
var AttributePicker = com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributePicker;
var CheckboxField   = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormCheckboxLabelField;
var FormMACAddressField = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormMACAddressField;
var ValidateTextField = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.GeneralValidateTextField;
var YesNoDialog     = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
var CommonUtility   = com.ibm.asq.common.web.ui.internal.util.Utility;
var HostNameRegExp  = "^[a-zA-Z0-9][a-zA-Z0-9.-]*[a-zA-Z0-9]$";


dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributeDisplay",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-rational-test-lm-web-ui-attributeDisplay">' +
                    '  <div class="attribute-container" dojoAttachPoint="attributePanel">' +
                    '    <div class="attribute-row" dojoAttachPoint="attributeRowNode">' +
                    '      <div class="attribute-control" dojoAttachPoint="attributeControlNode">' +
                    '      </div>' +
                    '      <div class="attribute-display" dojoAttachPoint="attributeNode">' +
                    '      </div>' +
                    '    </div>' +
                    '    <div class="sub-attribute-container" dojoAttachPoint="subAttributeContainerNode">' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',
    
    mode: "EDITABLE",
    
    reduceActionsVisibility: true,
    
    canDelete: true,
    
    catalogAttribute: null,
    
    arrayAttrTag: null,
        
    showSubAttrWithValueOnly: true,
    
    indentLevel: 0,
    
    createChildren: true,
    
    // Set this member to true if this is the root attribute and
    // it should show all asset root types, i.e,, Machine, virtualImage, etc.
    //
    showAllRootTypes: false,
    
    postCreate: function() 
    {        
        this.isArrayAttr = false;
        this.isRoot = false;
        
        if(this.catalogAttribute) {
            if(this.catalogAttribute.isArray()) {
                this.isArrayAttr = true;
                if(!this.arrayAttrTag) {
                    var firstVal = this.catalogAttribute.getArrayValue(0);
                    if(firstVal)
                        this.arrayAttrTag = firstVal.getTag();
                }
            }
            this._createAttributeDisplay();
        }
        
        // we're reducing our action's visibility. Control the area' opacity when
        // it has mouse in/out.
        if(this.reduceActionsVisibility) {
            this.connect(this.attributeRowNode, "onmouseover", "_mouseInAttrArea");
            this.connect(this.attributeRowNode, "onmouseout", "_mouseOutAttrArea");
        }
        else {
            dojo.addClass(this.attributeControlNode, "attribute-control-active"); 
        }
    },

    // Summary:
    //    If the attribute is the root attribute, based on the value of the attribute
    //    this method set and store the root asset template name. For example, if the value
    //    is 'VirtualMachine', the root asset template name is 'Machine'.
    //
    _setCurrentRootType: function(value) 
    {
        var type = this._getRootAssetTypeFromValue(value);
        this.currentAssetType = type;
    },
    
    // Summary:
    //    If the attribute is the root attribute, based on the value of the attribute
    //    this method returns the root asset template name. For example, if the value
    //    is 'VirtualMachine', the root asset template name is 'Machine'.
    //    
    _getRootAssetTypeFromValue: function(value)
    {
        var assetType = null;
        for(var type in this.assetTypeMap) {
            if(value == type) {
                assetType = type;
                break;
            }
            var subtypes = this.assetTypeMap[type];
            for(var i = 0; i < subtypes.length; i++) {
                if(value == subtypes[i]) {
                    assetType = type;
                    break;
                }
            }
            if(assetType)
                break;
        }
        return assetType;
    },
    
    _buildRootTypeChoices: function(assetRootAttr, choiceArr)
    {
        var rootName = assetRootAttr.getRootPart().getName();
        var subTypes = assetRootAttr.getAllPartTypeNames();
        var obj = {};
        
        // for root types we show the root part as well, i.e., this is
        // the 'Machine', or 'VirtualImage' root parts.
        //
        obj['name'] = RtlmUiUtil.getCatalogMessage(rootName);
        obj['value'] = rootName;
        obj['indent'] = false;
        choiceArr.push(obj);
        
        // Get all sub types of the root part.
        for(var j = 0; j < subTypes.length; j++) {
            obj = {};
            obj['name'] = RtlmUiUtil.getCatalogMessage(subTypes[j]);
            obj['value'] = subTypes[j];
            obj['indent'] = true;
            choiceArr.push(obj);
        }
    },

    _buildEnumChoices: function(enumVals)
    {
        var choices = [];
        for (var i = 0; i < enumVals.length; i++) {
            var enumVal = enumVals[i];
            var val = {
                name: RtlmUiUtil.getCatalogMessage(enumVal),
                value: enumVal
            }
            choices.push(val);
        }
        return choices;
    },
    
    // Summary:
    //    For the root attribute returns the possible values for the root attribute.
    //    If this display object is set to 'showAllRootTypes', all of the root asset 
    //    types are included in the possible choices. Otherwise, only the sub types of 
    //    current asset attribute are returned.
    // 
    _getRootTypeChoices: function()
    {
        var choices = [];
        
        // We're not showing all asset root template choices. Just 
        // show the subtypes of the current root attribute.
        // 
        if (!this.showAllRootTypes) {
            this._buildRootTypeChoices(this.catalogAttribute, choices);
            return choices;
        }
        
        var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
        this.assetTypeNames = catalog.getAssetTemplateNames();
        this.assetTypeMap = {};
        
        for(var i = 0; i < this.assetTypeNames.length; i++) {
            var asset = catalog.createConfiguration(this.assetTypeNames[i]); 
            var assetRootAttr = asset.getRootAttribute();
            var subTypes = assetRootAttr.getAllPartTypeNames();
            
            this.assetTypeMap[this.assetTypeNames[i]] = subTypes;
            this._buildRootTypeChoices(assetRootAttr, choices);
        }
        return choices;
    },
    
    // Summary:
    //    Creates the input field for the catalog attribute object.
    //
    _createAttributeDisplay: function()
    {
        var type = this.catalogAttribute.getType();
        var attrName = this.catalogAttribute.getName();
        var title = RtlmUiUtil.getCatalogMessage(attrName);
        var field = null;
        var isPart = (this.catalogAttribute.getType() == TYPE_PART);
        
        this.isRoot = (attrName == "root");
        
        // apply the supported mode to the attribute display.
        // only if we're in edit mode that we'll use the catalog's
        // type.
        if(this.mode == "READONLY") 
            type = TYPE_LABEL;
        else if(this.mode == "SELECTABLE")
            type = (this.isRoot) ? TYPE_LABEL : TYPE_CHECKBOX;             
        else if(this.catalogAttribute.isEnum())
            type = TYPE_ENUM;
        
        switch(type) {
            case TYPE_PART:
                var hasEmpty = !this.isRoot;
                var choices = (this.isRoot) ? this._getRootTypeChoices() : this.catalogAttribute.getAllPartTypeNames();
                
                if(!choices)
                    choices = [];
                    
                // include the root part if there're no sub-part types.
                if(choices.length == 0) {
                    var rootName = this.catalogAttribute.getRootPart().getName();
                    choices.unshift(rootName);
                    hasEmpty = false;
                }
                field = new EnumField({fieldLabel: title, createEmpty: hasEmpty});
                
                // root type is already using translated string. For sub types try to
                // get a translated string for them too.
                var translatedChoices = (this.isRoot) ? choices : this._buildEnumChoices(choices);
                field.setEnumChoices(translatedChoices);
                break;
            case TYPE_ENUM:
                field = new EnumField({fieldLabel: title, createEmpty: true});
                var choices = this._buildEnumChoices(this.catalogAttribute.getEnumValues());
                field.setEnumChoices(choices);
                break;
            case TYPE_NUMBER:
                field = new TextField({fieldLabel: title}); // use catalog validation.
                break;
            case TYPE_STRING:
                if(attrName == "ipAddress" || attrName == "IpV4Address" || 
                   attrName == "IpV6Address" || attrName == "SubnetAddress" || 
                   attrName == "Netmask") {
                    field = new IPAddressField({fieldLabel: title}); 
                }
                else if(attrName == "PrimaryMACAddress") {
                     field = new FormMACAddressField({fieldLabel: title}); 
                }
                else if(attrName == "hostname" || attrName == "FullyQualifiedDomainName") {
                    field = new ValidateTextField({fieldLabel: title, regExp: HostNameRegExp}); 
                }       
                else {
                    field = new TextField({fieldLabel: title}); 
                }
                break;
            case TYPE_TIMESTAMP:
                field = new DateField({fieldLabel: title}); 
                break;
            case TYPE_POINTER:
                if(attrName == "owner") {
                    var userNames = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.getUserNames();
                    field = new EnumField({fieldLabel: title, createEmpty: true});
                    field.setEnumChoices(userNames);
                    break;
                }
                else if(attrName == "group") {
                    var groupNames = GroupUtil.getAllGroupNames();
                    field = new EnumField({fieldLabel: title, createEmpty: true});
                    field.setEnumChoices(groupNames);
                    break;
                }
            case TYPE_CHECKBOX:
                field = new CheckboxField({fieldLabel: title, checked: true}); 
                break;
            default:
            case TYPE_LABEL:
                field = new LabelField({fieldLabel: title}); 
        }
        
        this.attributeNode.appendChild(field.domNode);
        this.attrField = field;
        var attrValue = this.getAttrValue(this.catalogAttribute, this.arrayAttrTag);
        
        // apply the indent level to the label of the input field.
        if(this.indentLevel > 0)
            this.attrField.setLabelIndentLevel(this.indentLevel, 1);
        
        if(this.catalogAttribute.getType() == TYPE_PART && attrValue)
            attrValue = attrValue.getName();
            
        // a pointer object has two components. Use
        // the display string for value.
        if(this.catalogAttribute.getType() == TYPE_POINTER && attrValue) {
            var pointer = attrValue;
            attrValue = pointer.getDisplayString();
        }
        
        if(attrValue)
            this.attrField.setValue(attrValue);
        this.attrField.onValueChanged = dojo.hitch(this, "attributeValueChanged");

        // We'are showing all root asset types and their sub types.
        // Record which root type we're showing currently.
        if (this.showAllRootTypes)
            this._setCurrentRootType(attrValue);
            
        // retain the current value of the attribute. 
        if (this.mode == "SELECTABLE") {
            this.attrField.onValueChanged = dojo.hitch(this, "toggleAttributeValue");
            this.attributeValue = attrValue;
        }
        
        // create the actions for the attribute if this is editable.
        if(this.mode == "EDITABLE")
            this.buildAttributeActions(isPart);
        
        // if this is a part attribute, render its attributes too.
        if(this.createChildren && isPart) {
            this.buildSubAttributes();
        }
    },
    
    // Summary:
    //    Creates the AttributeDisplay widgets for the sub-attributes of this attribute. The indent level
    //    of our sub-attributes are increased by one. All of our widget attributes, like read-only, etc,
    //    are passed down to our children AttributeDisplays.
    //    
    buildSubAttributes: function()
    {
        var name = this.catalogAttribute.getName();
        var part = this.getAttrValue(this.catalogAttribute, this.arrayAttrTag);
        var attrNames = part.getAttributeNames();
        var indentLevel = this.indentLevel + 1;
        
		//The setting of the display to none is done for IE, because otherwise it wants
		//to sometimes scroll the view to the top when the attribute display is remade.
		var display = this.subAttributeContainerNode.style.display;
        
        // unload any widgets we've first, before rebuild                
        //CommonUtility.destroyWidgetsFromNode(this.subAttributeContainerNode);
        
		this.subAttributeContainerNode.style.display = "none";
        this.subAttributeContainerNode.innerHTML = "";
        this.subAttributeFields = [];
        this.childAttributeList = [];
        
        for(var i = 0; i < attrNames.length; i++) {
            var attr = part.getAttribute(attrNames[i]);
            var attrWidget;
            
            // skip over attributes that don't have value, or we've
            // not marked (added explicitly) it as a display attribute.
            if (!attr.isMarked() && !this._attributeHasValue(attr)) 
                continue;
            
            // save our display children in the list.
            this.childAttributeList.push(attrNames[i]);
            
            if (attr.isArray()) {
                var attrVals = attr.getArrayValues();
                
                for (var j = 0; j < attrVals.length; j++) {
                    var subpart = attrVals[j];
                    attrWidget = new com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributeDisplay({
                        'catalogAttribute': attr,
                        'mode': this.mode,
                        'arrayAttrTag': subpart.getTag(),
                        'indentLevel': indentLevel,
                        'onValueChanged': dojo.hitch(this, "_handleChildAttributeValueChanged"),
                        'onRemoveAttribute': dojo.hitch(this, "_handleChildAttributeRemove"),
                        
                        // get notification on action clicked. When an action is clicked
                        // in our children attributes, that doesn't generate a _onBlur event
                        // to us. Manually add it to our children to receive the clicked event
                        // to popdown our picker.
                        'onActionClicked': dojo.hitch(this, "_onBlur")
                    });
                    this.subAttributeContainerNode.appendChild(attrWidget.domNode);
                    this.subAttributeFields.push(attrWidget);
                }
            }
            else {
                attrWidget = new com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.AttributeDisplay({
                    'catalogAttribute': attr,
                    'mode': this.mode,
                    'indentLevel': indentLevel,
                    'onValueChanged': dojo.hitch(this, "_handleChildAttributeValueChanged"),
                    'onRemoveAttribute': dojo.hitch(this, "_handleChildAttributeRemove"),

                    // get notification on action clicked. When an action is clicked
                    // in our children attributes, that doesn't generate a _onBlur event
                    // to us. Manually add it to our children to receive the clicked event
                    // to popdown our picker.
                    'onActionClicked': dojo.hitch(this, "_onBlur")
                });
                this.subAttributeContainerNode.appendChild(attrWidget.domNode);
                this.subAttributeFields.push(attrWidget);
            }
        }

		this.subAttributeContainerNode.style.display = display;
    },
    
    getAttrValue: function(attr, tag)
    {
        var val = null;
        var isArray = attr.isArray();
        
        if(isArray) {
            if(tag) {
                val = attr.getArrayValueByTag(tag);
            }
            else {
                val = attr.getArrayValues(); 
                if(val.length == 0)
                    val = null;               
            }
        } 
        else {
            val = attr.getValue();
        }
        return val;
    },
    
    buildAttributeActions: function(isPart)
    {
        var args = {
            'showLabel': false,
            'class': "com-ibm-rational-test-lm-web-ui-attributeDisplay-action"
        };
        
        if(isPart) {
            args['label'] = RtlmUiUtil.getMessage("add_attribute");
            args['iconClass'] = "add-attr-icon-image";
            args['onClick'] = dojo.hitch(this, "_openAttributePicker");
            this.addAttrButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(args);
            this.attributeControlNode.appendChild(this.addAttrButton.domNode);
            
            // add onclick handlers to popdown the add attribute picker within 
            // our widget's dom nodes. The _onBlur is only called when the user
            // is clicking outside of this widget's dom nodes. We want to pop down
            // the picker when the user click on nodes inside this widget as well.
            //
            this.connect(this.subAttributeContainerNode, "onclick", "_onBlur");
            this.connect(this.attributePanel, "onclick", "_onBlur");
            var focusNode = this.addAttrButton.focusNode;
			this.connect(focusNode, "onfocus", "_mouseInAttrArea");
			this.connect(focusNode, "onblur", "_mouseOutAttrArea");
        }
        if (this.canDelete) {
            args['label'] = RtlmUiUtil.getMessage("labelRemoveButton");
            args['iconClass'] = "delete-icon-image";
            args['onClick'] = dojo.hitch(this, function(){        
                this.onActionClicked();
                this.onRemoveAttribute(this);
            });
            
            var button = new com.ibm.asq.common.web.ui.internal.widgets.Button(args);
            this.attributeControlNode.appendChild(button.domNode);
            var focusNode = button.focusNode;
			this.connect(focusNode, "onfocus", "_mouseInAttrArea");
			this.connect(focusNode, "onblur", "_mouseOutAttrArea");
       }
    },
    
    _openAttributePicker: function()
    {
        // notify of the event.
        this.onActionClicked();
        
        if(this._opened)
            return;
            
        var part = this.getAttrValue(this.catalogAttribute, this.arrayAttrTag);
        
        this.attrPicker = new AttributePicker({
            'promptText': RtlmUiUtil.getCatalogMessage(part.getName()),
            'catalogPart': part,
            'excludeItems': this.childAttributeList,
            'onClose': dojo.hitch(this, "_closeAttributePicker"),
            'onAdd': dojo.hitch(this, "_addChildAttributes")
        });

        // popup will register this widget with the focus manager (I think).
        // By having a _onBlur method in this widget, whenever a click happens outside
        // of this widget, that method will get called.
        dijit.popup.open({
            parent: this,
            popup: this.attrPicker,
            around: this.addAttrButton.domNode
        });
		this.attrPicker.selectAndSetFocus();		
		
  		
					        
        // the popup would immediately deliver a _onBlur message to this (parent)
        // when the picker is shown. Create a flag to indicate this is first time
        // popup and don't popdown the picker when this flag is true.
        // Note: need to look into this closer in the focus manager changes in dojo.
        // to see if there are any 'magic' to avoid this.
        this._justPopup = true;         
        this._opened = true;       
    },
    
    _closeAttributePicker: function()
    {
        if(this._opened){
            dijit.popup.close(this.attrPicker);
            this._opened = false;
            this._mouseOutAttrArea();
        }           
    },

    _addChildAttributes: function(attributes)
    {
        var part = this.getAttrValue(this.catalogAttribute, this.arrayAttrTag);

        for(var i = 0; i < attributes.length; i++) {
            var attrName = attributes[i];
            var attr = part.getAttribute(attrName);
            
            if(attr.getType() == TYPE_PART) {
                var defaultValue = attr.getRootPart().getName();
                
                if(attr.isArray()) {
                    attr.addArrayValue(defaultValue);
                }
                else {
                    attr.setValue(defaultValue);
                }
            }
            attr.setMarked(true);
        }
        
        // rebuild the attribute fields based on the new attribute list
        // update the picker's list as well.
        this.buildSubAttributes();
        this.attrPicker.excludeItems = this.childAttributeList;
        this.attrPicker.refreshList();
        
        this.onValueChanged();
    },
    
    _isChangingRootType: function(newValue)
    {
        var type = this._getRootAssetTypeFromValue(newValue);
        return (type != this.currentAssetType);
    },
    
    attributeValueChanged: function(fieldRef, oldValue, newValue)
    {
        if(oldValue == newValue)
            return;

        // handle changing the root type. We've to create a new configuration
        // of the selected type.
        if(this.isRoot && this.showAllRootTypes && this._isChangingRootType(newValue)) {
            var type = this._getRootAssetTypeFromValue(newValue);
            var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
            var asset = catalog.createConfiguration(type); 
            this.catalogAttribute = asset.getRootAttribute();
            
            var parts = this.catalogAttribute.getArrayValues();
            this.arrayAttrTag = parts[0].getTag();
            this.catalogAttribute.setArrayValueByTag(this.arrayAttrTag, newValue);

            this._setCurrentRootType(newValue);            
            this.buildSubAttributes();
            
            // notify the change of asset root
            this.onRootTypeChanged(type, asset);
            return;
        }

        var value = newValue;
        var isPart = this.catalogAttribute.getType() == TYPE_PART;
        
        if(isPart && (newValue == null || newValue.length < 1)) {
            var rootPart = this.catalogAttribute.getRootPart();
            value = rootPart.getName();
        }

        var attrValid = this.catalogAttribute.VALIDATION_OK;
        
        // the root attribute doesn't have a validate method. Don't have to
        // call it, if the object doesn't have one.
        //
        if(dojo.isFunction(this.catalogAttribute.validate))
            attrValid = this.catalogAttribute.validate(value);
            
        if (attrValid != this.catalogAttribute.VALIDATION_OK) {
            var msg = this._mapAttributeValidationErrorMessage(this.catalogAttribute, attrValid);
            this.attrField.setHasError(true);
            this.attrField.setErrorMessage(msg);
        }
        else {
            this.attrField.setHasError(false);
            this.attrField.setErrorMessage(null);
            
            if (this.catalogAttribute.isArray()) {
                this.catalogAttribute.setArrayValueByTag(this.arrayAttrTag, value);
            }
            else {
                // handle the owner and group attribute differently. We have to
                // pass in the uuid as well.
                if (this.catalogAttribute.getName() == "owner" && value) {
                    var user = {
                        "ds": value,
                        "uuid": this._getUserIdByName(value)
                    };
                    value = user;
                }
                else if (this.catalogAttribute.getName() == "group" && value) {
                    var group = {
                        "ds": value,
                        "uuid": this._getGroupIdByName(value)
                    };
                    value = group;
                }
                this.catalogAttribute.setValue(value);
            }
            // reinitialize the sub attributes, part type changed.
            if (isPart) {
                this.buildSubAttributes();
            }
        }
        this.onValueChanged();
    },

    toggleAttributeValue: function(fieldRef, oldValue, newValue)
    {
        var isPart = this.catalogAttribute.getType() == TYPE_PART;
        var attrValue = (newValue) ? this.attributeValue : null;

        if(isPart && attrValue == null) {
            var rootPart = this.catalogAttribute.getRootPart();
            attrValue = rootPart.getName();
        }
        
        if (this.catalogAttribute.isArray()) {
            this.catalogAttribute.setArrayValueByTag(this.arrayAttrTag, attrValue);
        }
        else {
            this.catalogAttribute.setValue(attrValue);
        }
        
        // reinitialize the sub attributes, part type changed.
        if (isPart) {
            this.buildSubAttributes();
        }
        this.onValueChanged();
    },
    
    _mapAttributeValidationErrorMessage: function(attribute, validateVal)
    {
        switch(validateVal) {
            case attribute.VALIDAIION_INVALID_CHARACTER: return RtlmUiUtil.getMessage("msgAttrInvalidChar");
			case attribute.VALIDAIION_NONNUMERIC_CHARACTER: return RtlmUiUtil.getMessage("msgAttrNonNumericChar");			
            case attribute.VALIDATION_INVALID_SEQUENCE: return RtlmUiUtil.getMessage("msgAttrInvalidSequence");
            case attribute.VALIDATION_DUPLICATE_OPERATORS: return RtlmUiUtil.getMessage("msgAttrDupOps");
            case attribute.VALIDATION_DISJOINTED: return RtlmUiUtil.getMessage("msgAttrDisjoined");
            case attribute.VALIDATION_INCOMPLETE: return RtlmUiUtil.getMessage("msgAttrIncomplete");
            case attribute.VALIDATION_EXTRA_COMPONENTS: return RtlmUiUtil.getMessage("msgAttrExtraComp");
            default: return RtlmUiUtil.getMessage("msgInvalidValue");
        }
    },

    _getUserIdByName: function(name)
    {
        var users = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.getUsers();
        if(users) {
            for(var i = 0; i < users.length; i++) {
                if(users[i].name == name)
                    return users[i].itemId;
            }
        }
        return null;
    },
    
    _getGroupIdByName: function(name)
    {
        var groups = GroupUtil.getAllGroups();
        if(groups) {
            for(var i = 0; i < groups.length; i++) {
                if(groups[i].name == name)
                    return groups[i].itemId;
            }
        }
        return null;
    },
    
    _attributeHasValue: function(attr)
    {
        if(attr.isArray()) {
            return (attr.getArrayValues().length > 0);
        }
        else if (attr.getValue() != null) {
            var val = attr.getValue();
            
            if (attr.getType() == TYPE_PART) {
                // not the root type, user had changed it. this type has value.
                if (val.getName() != attr.getRootPart().getName()) {
                    return true;
                }
                // this is still a root type, check to see if the type
                // have any value set in them, if it does, it has value.
                var attrNames = val.getAttributeNames();
                for (var i = 0; i < attrNames.length; i++) {
                    var attr = val.getAttribute(attrNames[i]);
                    
                    // recursively to find out whether this attribute
                    // has value. We only need to find 1 attr that has
                    // value.
                    if (this._attributeHasValue(attr)) {
                        return true;
                    }
                }
            }
            // if the value is an object, it has value, 
            // or if it is a string, ensure that it's
            // not empty.
            else if (dojo.isObject(val) || (dojo.isString(val) && val.length > 0)) {
				return true;
            }
        }            
        return false;
    },

    _mouseInAttrArea: function()
    {
        dojo.addClass(this.attributeControlNode, "attribute-control-active"); 
    },

    _mouseOutAttrArea: function()
    {
        if(this._opened || !this.reduceActionsVisibility) {
            return;            
        }
        dojo.removeClass(this.attributeControlNode, "attribute-control-active"); 
    },
    
    // Summary:
    //    Internal method to receive children attribute's remove event.
    //
    _handleChildAttributeRemove: function(attrWidget)
    {
		var self = this;
        var catName = attrWidget.catalogAttribute.getName();
		var attrName = RtlmUiUtil.getCatalogMessage("pl_" + catName);

        // this is a custom attribute, just use the name from catalog.
        if(attrName.indexOf("pl_") > -1) {
            attrName = catName;
        }
        
		var args = {
			title: RtlmUiUtil.getMessage("msgConfirmTitle"),
			message: RtlmUiUtil.getMessage("msgConfirmRemoveAttribute", attrName),
			yesCallback: function(){
		        var attr = attrWidget.catalogAttribute;
		        var removeFromList = true;
		        
		        if(attr.isArray()) {
		            attr.removeArrayValueByTag(attrWidget.arrayAttrTag);
		            if(self._attributeHasValue(attr))
		                removeFromList = false;
		        }
		        else {
		            attr.setValue(null);
		        }
		        
		        if (removeFromList) {
		            attr.setMarked(false);
		        }
		        
		        // just rebuild our children attribute display
		        self.buildSubAttributes();  
		        self.onValueChanged();
				return true;          
			},
			noCallback: function() {
				return false;
			}
		}

		var dialog = new YesNoDialog(args);
        dialog.setSecondaryTitle(args.title);
        dialog.open();
    },

    // Summary:
    //    Internal method to receive children attribute's value changed event.
    //
    _handleChildAttributeValueChanged: function(attrWidget)
    {
        this.onValueChanged();
    },
    
    // Summary:
    //    Notification when a child attribute is removed. Client can override this
    //    method to receive the notification.
    //
    onRemoveAttribute: function()
    {
    },
    
    // Summary:
    //    Notification when the input field value has changed. This includes any children 
    //    attribute down the recurse level. Client can override this method to receive the 
    //    notification.
    //
    onValueChanged: function()
    {
    },
    
    // Summary:
    //    Specific notification for root type changed events. This is only called when the
    //    display's member 'showAllRootTypes' is set to true and the attribute is the root,
    //    and that the user has changed the root asset type to the one other then that was set. 
    //
    onRootTypeChanged: function(assetType, asset)
    {
    },

    // Summary:
    //    Returns true if there's input error in the input field. This reports any children 
    //    attribute errors down the recurse level.
    //
    hasError: function()
    {
        if(this.attrField.getHasError())
            return true;

        if(this.subAttributeFields) {
            for(var i = 0; i < this.subAttributeFields.length; i++) {
                var subAttr = this.subAttributeFields[i];
                
                if(subAttr.hasError())
                    return true;
            }
        }
        return false;
    },
    
    // Summary:
    //    Notification on this display's action clicked. Override this method to receive 
    //    notification when this display's actions have been invoked.
    //
    onActionClicked: function()
    {
        
    },
    
    // summary: 
    //    Called by the focus manager in focus.js when focus moves outside of the
    //    AttributeDisplay widget (or it's descendants).
    //
    _onBlur: function()
    {
        // we just popped up. Don't popdown right away. The shift of focus
        // was to our picker.
        if(this._justPopup) {
            this._justPopup = false;
            return;
        }
        this._closeAttributePicker();

        if(this.attrPicker) {
            this.attrPicker.destroy();
            delete this.attrPicker;
        }
        this.inherited(arguments);
    }
});

})();
