/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Summary:
//      Subclassed from the dijit.form.DateTextBox to remove the auto popup of the calendar
//      when the text field received focus. 
//
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.datetextbox.DateTextBox");
dojo.require("dijit.form.DateTextBox");

dojo.declare(
    "com.ibm.asq.common.web.ui.internal.widgets.datetextbox.DateTextBox",
    dijit.form.DateTextBox,
{  
    nonExistDateMessage: "",
     
    _open: function() 
    {
        // override to do nothing.
    },
    
    // Summary:
    //    Override the parent's method to add additional test on non-existing dates.
    //
    isValid: function(/* Boolean*/ isFocused)
    {
        var isValid = com.ibm.asq.common.web.ui.internal.widgets.datetextbox.DateTextBox.superclass.isValid.call(this, isFocused);
        
        // add one more check to ensure that the entered string is a valid date.
        // dates like 02/30/07 passes the format validation, but the date doesn't 
        // exist. Our checkDate ensure that we keep the text box in invalid mode if
        // such a date is entered.
        if(isValid)
            return this._checkDate();
        return isValid;
    },
    
    // Summary:
    //    Check for invalid date that passes the format validation. If a date string
    //    is valid in format, the date text box doesn't flag it as invalid. However,
    //    its date value is null. Add this check to make the date text box stay in 
    //    invalid mode.
    // 
    _checkDate: function()
    {
        var date = this.getValue();
        
        if(!date && this.domNode.value && this.domNode.value.length > 0) {
            this.invalidMessage = this.nonExistDateMessage;
            return false;                        
        }
        return true;
    }
    
});
