/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.grid.GridColumnEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");


dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");


(function() { // module container
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.grid.GridColumnEditor",
[dijit._Widget, dijit._Templated],
{
    templateString: '<table class="settings-table" dojoAttachPoint="_columnsDomNode">'+
                    '<tr>'+
                     '<th nowrap dojoAttachPoint="columnHead" class="settings-table-header"></th>'+
                     '<th nowrap dojoAttachPoint="columnNameHead" class="settings-table-header"></th>'+
                     '<th nowrap dojoAttachPoint="actionsHead" class="settings-table-header" style="align:center"></th>'+ 
                '</tr>'+
                '</table>',
                    
    constructor: function() {
        
    },
    
    postCreate: function() {
        
        this.table = this._columnsDomNode;
        this.columnCount =0;
        
        var addArg = {
                label: Constants.getMessage("labelAddColumn"),
                showLabel: false,
                iconClass: "add-icon-image",
                parent: this,
                onClick: dojo.hitch(this, "_addColumn")
            };
       this.addColumn = new com.ibm.asq.common.web.ui.internal.widgets.Button(addArg);
        
       //Set columns titles
       this.columnHead.appendChild(document.createTextNode(Constants.getMessage("labelColumn")));
       this.columnNameHead.appendChild(document.createTextNode(Constants.getMessage("labelColumnNames")));
       this.actionsHead.appendChild(this.addColumn.domNode);
        
       var columns = this.columnNames; 
       var nColumns = columns.length;
        this.columnCount = nColumns;
        //Populate table with the existing data         
        for (var i = 0; i < nColumns; i++) {
            //general informacion of the row
            var tr = this.table.insertRow(-1);
            this.connect(tr, "onmouseover", "_onMouseRowHandler");
            this.connect(tr, "onmouseout", "_onMouseRowHandler");
            //original position of this column
            tr.originalPosition = i;
            //adding one to the position since the first row has the columns titles
            tr.columnIndex = i + 1;
                
             //information of the first column, whitch displays the column number
             var td = tr.insertCell(0);
             td.align = "center";
             // if this is CJK locale add in our cjk locale specific CSS
             if (com.ibm.asq.common.web.ui.internal.util.Utility.isCJKLocale()) {
                           dojo.addClass(td, "settings-table-td-cjk");
              }
               else {
                dojo.addClass(td, "settings-table-td");
            }
            var label = document.createElement("span");
            label.innerHTML = "" + (i + 1);
            td.appendChild(label);
                
            //information of the second column, whitch displays the column name and allows to be edited
            var input = document.createElement("input");
            td = tr.insertCell(1);
            input.type = "text";
            input.value = columns[i];//this.viewer.getColumnWidth(i);
            input.size = 15;
            input.style.paddingRight = "2px";
            dojo.addClass(td, "settings-table-td");
            td.appendChild(input);
                
            //information of the third column, whitch displays Remove button
            td = tr.insertCell(2);
            td.align = "center";
            var removeButton = document.createElement("img");
            if (!dojo.isIE) {
                removeButton.src = "";
             }
             else{
                removeButton.src = ImageCache.REMOVE_DISABLED;
             }
             dojo.addClass(removeButton, "remove-action");
             removeButton.style.cursor = "pointer";
             this.connect(removeButton, "onclick", "_removeColumn");
             td.appendChild(removeButton);
         }
         if(nColumns == 0)
            this._addColumn();
        
         
       },
        
        //add a new row to this dialog (each row correspond a column of the DynamicGrid ) 
        _addColumn : function(){
            //general informacion of the row
            var tableSize = this.table.rows.length;
            var tr = this.table.insertRow(-1);
            //original position of this column
            tr.originalPosition = this.NEW_VALUE;
            //adding one to the position since the first row has the columns titles
            tr.columnIndex = tableSize;
            this.connect(tr, "onmouseover", "_onMouseRowHandler");
            this.connect(tr, "onmouseout", "_onMouseRowHandler");
                
            //information of the first column, whitch displays the column number
            var td = tr.insertCell(0);
            td.align = "center";
            // if this is CJK locale add in our cjk locale specific CSS
            if (com.ibm.asq.common.web.ui.internal.util.Utility.isCJKLocale()) {
                dojo.addClass(td, "settings-table-td-cjk");
            } else {
                dojo.addClass(td, "settings-table-td");
            }
            var label = document.createElement("span");
            label.innerHTML = "" + (tableSize);
            td.appendChild(label);
                
            //information of the second column, whitch displays the column name and allows to be edited
            td = tr.insertCell(1);
            var input = document.createElement("input");
            input.type = "text";
            input.value = Constants.getMessage("defaultColumnHeader",""+(++this.columnCount));
            input.size = 15;
            input.style.paddingRight = "2px";
            dojo.addClass(td, "settings-table-td");
            td.appendChild(input);
                
            //information of the third column, whitch displays Remove button
            td = tr.insertCell(2);
            td.align = "center";
            var removeButton = document.createElement("img");
            if (!dojo.isIE) {
                removeButton.src = "";
             }
             else{
                removeButton.src = ImageCache.REMOVE_DISABLED;
             }
              dojo.addClass(removeButton, "remove-action");
            removeButton.cursor = "pointer";
            this.connect(removeButton, "onclick", "_removeColumn");
            td.appendChild(removeButton);
                
         },
         
         //when the mouse pass in a specific row this method is called to shown the Remove action
        _onMouseRowHandler: function(event){
            event = event || window.event;
            
            //remove the image when this row is not selected
            if( event.type == "mouseout"){
                if (!dojo.isIE) {
                    event.currentTarget.childNodes[2].childNodes[0].src = "";
                }
                else{
                    event.currentTarget.childNodes[2].childNodes[0].src = ImageCache.REMOVE_DISABLED;
                }
            }
            //add the image when the row is selected
            else if( event.type == "mouseover"){
                event.currentTarget.childNodes[2].childNodes[0].src = ImageCache.REMOVE_ENABLED;
            }
        
         },
     
        //remove the row of this dialog (each row correspond a column of the DynamicGrid ) 
        _removeColumn : function(event){
            
            event = event || window.event;
            if(!event || !event.currentTarget)
                return;
            
            //extract the number of the row that is been edited to be removed 
            var colIndex = event.currentTarget.parentNode.parentNode.columnIndex;
            this.table.deleteRow(colIndex); 
             
            //adjust the column index of each row 
            for(var i = 1; i < this.table.rows.length ; i++){
               this.table.rows[i].columnIndex = i;  
               this.table.rows[i].childNodes[0].childNodes[0].innerHTML = i; 
            }
        },
        
        //Validates the values on this panel 
        checkColumnNames: function()
        {   
            var hasErr = false;
            var width = 0;
            var length = this.table.rows.length;
            
            this.errorMsg = "";
            for (var i = 1; i < length; i++) {
                var val = 0;
                var row = this.table.rows[i];
                // update the column's error status.
                var value= row./*column*/childNodes[1].childNodes[0].value;
                var labelNum = row.childNodes[0].childNodes[0];
                
                //Add the message when a column has an empty name
                if (!value || dojo.trim(value).length == 0) {
                    hasErr = true;
                    dojo.addClass(labelNum, "settings-column-label-attention");
                    this.errorMsg = Constants.getMessage("msgGenericColumnNameInvalid");
                }
                else {
                    dojo.removeClass(labelNum, "settings-column-label-attention");
                }
            }
            
            //Add a message when there is no Columns names    
            if (length < 2) {
                hasErr = true;
                this.errorMsg = Constants.getMessage("msgColumnNumberInvalid"); 
            }
                 
            return hasErr;
        },
        
        getErrorMessage: function(){
          return this.errorMsg;
        },
        
        getColumnNames: function(){
         
        var columns = [];
        //populate with the values remaining in the new order
        for(var i = 1; i <  this.table.rows.length ; i++)
        {
            var element = {
                columnNumber: this.table.rows[i].columnIndex, 
                columnBefore: this.table.rows[i].originalPosition,
                value: this.table.rows[i]./*column*/childNodes[1]./*input*/childNodes[0].value
            };
            columns.push(element);
        }   
            return columns;
        }
    
    
    });

})();