/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTree");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.layout._LayoutWidget");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.Panel");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTreeHeader");

(function() {
    
var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var PanelLayout = com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelLayout;
var Panel = com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.Panel;
var LEFT_RIGHT_TABS_WIDTH = 28;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTree",
             [dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-widget-htree">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTreeHeader" dojoAttachEvent="firstClicked:_doFirst,leftClicked:_doPrev,rightClicked:_doNext,lastClicked:_doLast" dojoAttachPoint="header">' +
                    '  </div>' +
                    '  <div class="com-ibm-asq-common-widget-htree-client-area">' +
                    '    <span class="com-ibm-asq-common-widget-htree-panel-container-tab left-tab" style="visibility:hidden;" dojoAttachEvent="onclick:_doPrev" dojoAttachPoint="leftTabNode">' +
                    '      <img src=${prevPageIcon}></img>' +
                    '      <span dojoAttachPoint="leftTabCountNode"></span>' +
                    '    </span>' +
                    '    <div class="com-ibm-asq-common-widget-htree-panel-container" dojoAttachPoint="panelViewport">' +
                    '      <div tabindex="-1" class="com-ibm-asq-common-widget-htree-panel-contents" dojoAttachPoint="panelContents,containerNode">' +
                    '      </div>' +
                    '    </div>' +
                    '    <span class="com-ibm-asq-common-widget-htree-panel-container-tab right-tab"  dojoAttachEvent="onclick:_doNext" style="visibility:hidden;" dojoAttachPoint="rightTabNode">' +
                    '      <img src=${nextPageIcon}></img>' +
                    '      <span dojoAttachPoint="rightTabCountNode"></span>' +
                    '    </span>' +
                    '  </div>' +
                    '</div>',
                    
    widgetsInTemplate: true,
    nextPageIcon: ImageCache.NEXT_PAGE_ICON,
    prevPageIcon: ImageCache.PREV_PAGE_ICON,
    title: "",
    model: null,
    width: "500px",
    height: "400px",
    panelWidth: 140,
    nVisiblePanelsHint: 0,
    allowAutoResize: true,
    isCompactMode: true,
    
    postCreate: function()
    {
        this.inherited('postCreate', arguments);
        
        this.header.panelContainer = this;
        this.layoutManager = new PanelLayout({panelContainer: this});
        this.panels = [];
        this.firstPanelPos = 0;
        this.inLoading = null;
        this.focusPanel = null;
        this._started = false;
        this.validSize = false;
        this.ruler = dojo.byId("com-ibm-rqm-app-ruler-node");
        
        dojo.style(this.domNode, "width", this.width);
        dojo.style(this.domNode, "height", this.height);
        
        // wire the panel visibility changed to the navigation buttons
        dojo.connect(this, "setFirstPanelPosition", this.header, "updateNavigationButtonsState");
        
        // connect to the window resize event and refresh our display.
        if (this.allowAutoResize) {
            this.connect(dojo.global, 'onresize', function(){
                // call resize with no params
                this.resize();
            });
        }
		
        this.connectKeyNavHandlers([dojo.keys.LEFT_ARROW], [dojo.keys.RIGHT_ARROW]);
    },

    _doFirst: function(event)
    {
        var panelPos = this.getFirstPanelPosition();
        
        if(panelPos > 0)
            this.layoutManager.setFirstVisiblePanel(0);    
        if(event){
            dojo.stopEvent(event);
        }    
    },

    _doLast: function(event)
    {
        var panelPos = this.getFirstPanelPosition();
        var nVisible = this.getNumVisiblePanels();
        var nPanels = this.getPanelCount();
        
        if(nPanels > (panelPos + nVisible))
            this.layoutManager.setFirstVisiblePanel(nPanels - nVisible);  
        if(event){
            dojo.stopEvent(event);
        }            
    },
    
	_doNext: function(event)
	{
        var panelPos = this.getFirstPanelPosition();
        var nVisible = this.getNumVisiblePanels();
        var nPanels = this.getPanelCount();
        var newPos = (panelPos + 1 > nPanels) ? panelPos : panelPos + 1;
        this.layoutManager.setFirstVisiblePanel(newPos);   
        if(event){
            dojo.stopEvent(event);
        }         
	},

    _doPrev: function(event)
    {
        var panelPos = this.getFirstPanelPosition();
        var newPos = (panelPos > 0) ? panelPos - 1 : 0;
        this.layoutManager.setFirstVisiblePanel(newPos);
        if(event){
            dojo.stopEvent(event);
        }
    },
	
    load: function()
    {
        if (!this._started) {
            setTimeout(dojo.hitch(this, function(){
                this._computeSize();
                this._started = true;
                this.load();
            }), 5);
            return;
        }
        this._clearAll();
        if(this.model) {
            // on model value changes, get notification to update our
            // panels.
            this.model.onChange = dojo.hitch(this, "handleModelChanged");
			this.model.onDelete = dojo.hitch(this, "handleModelDelete");
            this.model.getRoot(dojo.hitch(this, function(item) {
                this._createPanel(item, false);
                
                // initialize the keyboard navigation focus child to the first
                // child. 
                this.focusedChild = this.panels[0];
            }));
        }
    },
    
    handleModelChanged: function(item) 
    {
        var panel = this.getPanelByModelItem(item);
        if(panel)
            panel.update();
    },

	handleModelDelete : function(item)
	{
		this.removeItem(item.parent, item);		
	},   
	 
    resize: function(sizeHint)
    {        
        var mb = dojo.marginBox(this.panelViewport);
        
        // if we're not visible, or the size hint is invalid, don't do anything.
        if(mb.w < 1 || mb.h < 1 || (sizeHint && sizeHint.w < 1)) {
            return;
        }
        
        if (!sizeHint) {
	        // the viewport was set to a fixed width after layout.
	        // set its width to 100% again and redo the layout 
	        // with the new width.
	        //
			dojo.style(this.domNode, "visibility", "hidden");
			this.panelViewport.style.width = "100%";
			this.panelViewport.style.height = "100%";
			
			setTimeout(dojo.hitch(this, function(){
				this._computeSize();
				this.layoutManager.doLayoutAllChildren();
				dojo.style(this.domNode, "visibility", "");
				this.layoutManager.setFirstVisiblePanel(this.firstPanelPos, /*force*/ true);
			}), 5);
		}
		else {
	       this._computeSize(sizeHint);
           this.layoutManager.doLayoutAllChildren();
           this.layoutManager.setFirstVisiblePanel(this.firstPanelPos, /*force*/ true);
		}
    },

    hasValidSize: function()
    {
        return this.validSize;
    },
    
    _computeSize: function(sizeHint)
    {
        var mb = sizeHint || dojo.marginBox(this.panelViewport);
        var hmb = dojo.marginBox(this.header.domNode);
        var tabsWidth = LEFT_RIGHT_TABS_WIDTH;
		
        this.contentBox = dijit.layout.marginBox2contentBox(this.panelViewport, mb);
        this.contentBox.w = this.contentBox.w - tabsWidth;
		
        // size the view port to show only n-th panels based on the panel width
        if(this.nVisiblePanelsHint > 0) {
            this.nVisiblePanels = this.nVisiblePanelsHint;
            this.panelWidth = Math.floor(this.contentBox.w / this.nVisiblePanels);
        }
        else {
            this.nVisiblePanels = Math.floor(this.contentBox.w / this.panelWidth);            
        }
        // Make the panels narrower on IE to avoid the horizontal scrollbar
        // to show up.
        this.panelWidth = (dojo.isIE) ? this.panelWidth - 4 : this.panelWidth;
        this.contentBox.w = (this.nVisiblePanels * (this.panelWidth - 2)) + 1;
        this.contentBox.h = this.contentBox.h - hmb.h;
        this.header.resize({w: this.contentBox.w + tabsWidth});
        dojo.marginBox(this.panelViewport, {w: this.contentBox.w, h: this.contentBox.h});
        
        // record whether we have a valid size.
        this.validSize = (this.contentBox.w > 0);
    },
    
    setDisabled: function(disabled)
    {
        dojo.forEach(this.panels, function(panel){
            panel.setDisabled(disabled);
        })
    },
    
    _clearAll: function()
    {
        dojo.forEach(this.panels, function(panel) {
            panel.destroy();
        });
        this.panels = [];
        this.panelContents.innerHTML = "";
        this.header.updateNavigationButtonsState();
    },
    
    _createPanel: function(item, animate)
    {
        var panel = new Panel({
            model: this.model,
            modelItem: item,
            width: this.panelWidth,
            ruler: this.ruler,
            getIconClass: this.getIconClass,
            getLabelClass: this.getLabelClass,
            actionRegistry: this.actionRegistry,
			onLoadingCanceled: dojo.hitch(this, "_handlePanelLoadCanceled"),
            onItemOpen: dojo.hitch(this, "_handleItemOpen")
        });
        this.panels.push(panel);
        this.layoutManager.doLayout(panel);
        
        this.focusPanel = panel;
        
        this.addChild(panel);
        
        this.layoutManager.setFirstVisiblePanel();
        this.inLoading = this.focusPanel;
        var doAnimate = (typeof animate == "undefined") ? true : animate;
        panel.startup(dojo.hitch(this, function(){
            this.focusPanel.setStartLoading();
            this.model.getChildren(item, dojo.hitch(this, "_loadPanelContents"));
        }), doAnimate);
    },

    _handlePanelLoadCanceled: function(panel)
	{
		if(this.inLoading != null && this.inLoading === panel) {
	        this.focusPanel.setEndLoading();
	        this.inLoading = null;
		}
	},
	
    _loadPanel: function(item)
    {
        this.focusPanel.setModelItem(item);
        
        this.inLoading = this.focusPanel;
        this.focusPanel.setStartLoading();
        this.model.getChildren(item, dojo.hitch(this, "_loadPanelContents"));     
    },
    
    _loadPanelContents: function(items)
    {
		// the load was canceled. Don't load these items
		if(this.inLoading == null || this.inLoading != this.focusPanel) {
			return;
		}
        this.focusPanel.setEndLoading();
        this.focusPanel.addItems(items);
        this.inLoading = null;
    },
        
    _findPanelPosById : function(id)
    {
		var len = this.panels.length;
        for(var i = 0; i < len; i++) {
            if(id == this.panels[i].identity) {
                return i;
            }
        }
        return -1;
    },
    
    _handleItemOpen: function(panel, item)
    {
        // short circuit any other clicks while we're loading.
        if(this.inLoading != null)
            return;
        
        var panelPos;
        
        this.currentItemContext = item;
        if (!this.model.mayHaveChildren(item)) {
            panelPos = this._findPanelPosById(panel.identity);
            if(panelPos < 0) {
                // shouldn't happen
                console.debug("Unknown panel detected, identity: " + panel.identity);
                return;
            }
            if (panelPos < this.panels.length - 1) {
                this._closePanels(panelPos);
            }
        }
        else if(panel === this.focusPanel) {
            this._createPanel(item);
        }
        else {
            panelPos = this._findPanelPosById(panel.identity);
            if(panelPos < 0) {
                // shouldn't happen
                console.debug("Unknown panel detected, identity: " + panel.identity);
                return;
            }
            // close the panels down to the next panel of the
            // clicked panel.
            this._closePanels(panelPos + 1);
            this._loadPanel(item);
			this._adjustFirstPanelPos();
        }
        // explicitly call update navigation button states. Even if 
        // panels were closed the first panel position may not need 
        // to be adjusted. If that happens, the nav button states will
        // not be called.
        this.header.updateNavigationButtonsState();
    },
    
    _closePanels: function(toPanelPos, chainCb)
    {
        var delay = 0;
        var removedPanel = false;
		var len = this.panels.length;
        for(var i = len - 1; i > toPanelPos; i--) {
            //this.panels[i].collapse(delay);
            //delay += 150;
            dojo.style(this.panels[i].domNode, "display", "none");
            this.removeChild(this.panels[i]);
            this.panels[i].destroy();
            this.panels.pop();
            removedPanel = true;
        }
        // panels were removed. Update the content's container size
        // and the navigate button states.
        if(removedPanel) {
            this.layoutManager.adjustContentContainerSize();
            this.focusPanel = this.panels[toPanelPos];
        }
        this._adjustFirstPanelPos();
    },
    
    _adjustFirstPanelPos: function()
    {
        var curFirstPos = this.getFirstPanelPosition();
        var nVisible = this.getNumVisiblePanels();
        var focusPos = this._findPanelPosById(this.focusPanel.identity);
        var nPanels = this.getPanelCount();
        var newPos = -1;
        
        if(nPanels <= nVisible && curFirstPos > 0) {
            newPos = 0;
        }
        else if (focusPos > (curFirstPos + nVisible - 1)) {
            newPos = curFirstPos + (focusPos - (curFirstPos + nVisible - 1));
        }
        if(newPos != -1)
            this.layoutManager.setFirstVisiblePanel(newPos);
    },
    
    getViewport: function()
    {
        return this.panelViewport;
    },
    
    getContent: function()
    {
        return this.panelContents;
    },
    
    getPanelCount: function()
    {
        return this.panels.length;  
    },
    
    getPanelAt: function(nPanel)
    {
        if(nPanel >= 0 && nPanel < this.panels.length) {
            return this.panels[nPanel];
        } 
        return null; 
    },
    
    setFirstPanelPosition: function(firstPanelPos)
    {
        this.firstPanelPos = firstPanelPos;
    },

    getFirstPanelPosition: function()
    {
        return this.firstPanelPos;
    },
    
    getNumVisiblePanels: function()
    {
        return this.nVisiblePanels;
    },
    
    getPanelByModelItem: function(item)
    {
        var id = this.model.getIdentity(item);
        var panelPos = this._findPanelPosById(id);
        
        if(panelPos < 0) {
            return null;
        }
        return this.getPanelAt(panelPos);
    },
    
    removeItem: function(parentItem, itemToRemove)
    {
        var panel = this.getPanelByModelItem(parentItem);
        
        if (panel) {
            var panelPos = this._findPanelPosById(this.model.getIdentity(parentItem));
            panel.removeItem(itemToRemove);
            this._closePanels(panelPos);
        }
    },
    
    addItem: function(parentItem, newItem, atPos)
    {
        var panel = this.getPanelByModelItem(parentItem);
        
        if (panel) {
            panel.addItem(newItem, atPos, /*setFocus*/true);
        }
        else {
            // the parent item to have a new item is not in our panel
            // stack. The item must be the current selection context.
            var currId = this.model.getIdentity(this.currentItemContext);
            var pid = this.model.getIdentity(parentItem);
            
            if(pid == currId) {
                this.focusPanel.updateItem(parentItem);
                this.handleItemOpen(this.focusPanel, parentItem);
            }
        }       
    },
    
    updateItem: function(parentItem, item)
    {
        var panel = this.getPanelByModelItem(parentItem);
        
        if (panel) {
            panel.updateItem(item);

            // the panel's item changed.
            var panelPos = this._findPanelPosById(this.model.getIdentity(parentItem));
            if(panelPos >= 0) {
				var len = this.panels.length;
                for(var i = panelPos; i < len; i++) {
                    this.panels[i].updateItem();
                }
            }
        }
    },
    
    updateAllPanelActions: function()
    {
		var len = this.panels.length;
        for(var i = 0; i < len; i++) {
            this.panels[i].updatePanelActionStates();
        }
        
    },
    
    getUINodeForItem: function(parentItem, item) 
    {
        var panel = this.getPanelByModelItem(parentItem);
        
        if (panel) {
            return panel.getUIForItem(item);
        }
        return null;
    },
    
    _getFocusChildPos: function()
    {
        // focusedChild is managed by our parent class _KeyNavContainer 
        // it's the current child in navigation focus.
        var focusedChild = this.focusedChild || this.focusPanel;
        return this._findPanelPosById(focusedChild.identity);
    },


    // @override from dijit._KeyNavContainer
    focus: function()
    {
        this.inherited(arguments);
    },
    
    // @override from dijit._KeyNavContainer
    focusNext: function()
    {
        var firstPanelPos = this.getFirstPanelPosition();
        var nVisible = this.getNumVisiblePanels();
        var focusPos = this._getFocusChildPos();
        
        // if the next child is not in view, scroll to it first.
        if(focusPos == (firstPanelPos + nVisible)) {
            this.header.rightClicked();
        }
        this.inherited(arguments);
    },

    // @override from dijit._KeyNavContainer
    focusPrev: function()
    {
        var firstPanelPos = this.getFirstPanelPosition();
        var focusPos = this._getFocusChildPos();

        // if the prev child is not in view, scroll to it first.
        if(firstPanelPos > 0 && focusPos == firstPanelPos) {
            this.header.leftClicked();
        }
        this.inherited(arguments);
    },
    dummy:null

});

})();
