/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTreeHeader");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

(function() {

var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Constants = com.ibm.asq.common.web.client.internal.Constants;

var DisableCSS = "com-ibm-asq-common-widget-htree-header-action-disabled";
var EnableCSS = "com-ibm-asq-common-widget-htree-header-action";
var Spacing = 10;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTreeHeader",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div tabindex="-1" id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-widget-htree-header">' +
                    '  <div class="navigator">' +
                    '  <span dojoAttachPoint="prevGroupNode" class="prev-navigator-container">' +
                    '    <a href="javascript:{}" class="view-page-action" dojoAttachPoint="viewFirstNode" dojoAttachEvent="onclick:_handleFirstClicked">' +
                    '      <img src=${firstPageIcon}></img>' +
                    '    </a>' +
                    '    <a href="javascript:{}" dojoAttachEvent="onclick:_handleLeftClicked" dojoAttachPoint="viewLeftNode">${moveLeftLabel}' +
                    '    </a>' +
                    '  </span>' +
                    '  <span dojoAttachPoint="nextGroupNode" class="next-navigator-container">' +
                    '    <a href="javascript:{}" dojoAttachEvent="onclick:_handleRightClicked" dojoAttachPoint="viewRightNode">${moveRightLabel}' +
                    '    </a>' +
                    '    <a href="javascript:{}" class="view-page-action" dojoAttachPoint="viewLastNode" dojoAttachEvent="onclick:_handleLastClicked">' +
                    '      <img src=${lastPageIcon}></img>' +
                    '    </a>' +
                    '  </span>' +
                    '  </div>' +
                    '</div>',
  
    widgetsInTemplate: true,
    firstPageIcon: ImageCache.FIRST_PAGE_ICON,
	lastPageIcon: ImageCache.LAST_PAGE_ICON,
    moveLeftLabel: Constants.getMessage("Previous"),
    moveRightLabel: Constants.getMessage("Next"),
	
	// our parent container.
    panelContainer: null,
      
    postCreate: function()
    {
        this.inherited('postCreate', arguments);
        this.updateNavigationButtonsState();
		this._tickleContainer = false;
    },
    
    updateNavigationButtonsState: function()
    {
        if(this.panelContainer) {
            var panelPos = this.panelContainer.getFirstPanelPosition();
            var nVisiblePanels = this.panelContainer.getNumVisiblePanels();
            var nPanels = this.panelContainer.getPanelCount();    
            var canSlideLeft = ((panelPos + nVisiblePanels) < nPanels);
			var disableLeft = (panelPos == 0);
          
			dojo.toggleClass(this.viewFirstNode, DisableCSS, disableLeft);
            dojo.toggleClass(this.viewLastNode, DisableCSS, !canSlideLeft);
            dojo.toggleClass(this.viewLeftNode, DisableCSS, disableLeft);
            dojo.toggleClass(this.viewRightNode, DisableCSS, !canSlideLeft);
			
			// update the parent's tabs as well. They can do navigation as well.
			dojo.style(this.panelContainer.leftTabNode, "visibility", disableLeft ? "hidden" : "visible");
            dojo.style(this.panelContainer.rightTabNode, "visibility", canSlideLeft ? "visible" : "hidden");
			
			if(!disableLeft) {
				this.panelContainer.leftTabCountNode.innerHTML = (panelPos + "");
//				if(!this._tickleContainer) {
//					this.panelContainer.resize();
//					this._tickleContainer = true;
//				}
			}
            if(canSlideLeft) {
                this.panelContainer.rightTabCountNode.innerHTML = ((nPanels - (panelPos + nVisiblePanels)) + "");
            }
        }
    },
    
    resize: function(dim)
    {
        dojo.marginBox(this.domNode, {w: dim.w});
        
        var center = Math.round(dim.w/2);
		
		// line up the prev + next groups to the center of the
		// header.
        var left = (center - this.prevGroupNode.offsetWidth);
        this.prevGroupNode.style.left = (left - 11) + "px";
        this.nextGroupNode.style.left = (left - 7) + "px";
        
    },
    
    leftClicked: function()
    {   
    },

    rightClicked: function()
    {
    },

    firstClicked: function()
    {   
    },

    lastClicked: function()
    {
    },
	
	_handleLeftClicked: function(e)
	{
        // stop propagate event to body. This may cause Jazz's
        // body handler to trigger unnecessary dialog.
        dojo.stopEvent(e);
        
        var panelPos = this.panelContainer.getFirstPanelPosition();
        if(panelPos > 0)
		  this.leftClicked();
	},
	
    _handleRightClicked: function(e)
    {
        // stop propagate event to body. This may cause Jazz's
        // body handler to trigger unnecessary dialog.
        dojo.stopEvent(e);

        var panelPos = this.panelContainer.getFirstPanelPosition();
        var nVisiblePanels = this.panelContainer.getNumVisiblePanels();
        var nPanels = this.panelContainer.getPanelCount();    
        var canSlideLeft = ((panelPos + nVisiblePanels) < nPanels);

        if(canSlideLeft)
		  this.rightClicked();        
    },

    _handleFirstClicked: function(e)
    {
        // stop propagate event to body. This may cause Jazz's
        // body handler to trigger unnecessary dialog.
        dojo.stopEvent(e);

        var panelPos = this.panelContainer.getFirstPanelPosition();
        if(panelPos > 0)
          this.firstClicked();        
    },
    
    _handleLastClicked: function(e)
    {
        // stop propagate event to body. This may cause Jazz's
        // body handler to trigger unnecessary dialog.
        dojo.stopEvent(e);

        var panelPos = this.panelContainer.getFirstPanelPosition();
        var nVisiblePanels = this.panelContainer.getNumVisiblePanels();
        var nPanels = this.panelContainer.getPanelCount();    
        var canSlideLeft = ((panelPos + nVisiblePanels) < nPanels);

        if(canSlideLeft)
          this.lastClicked();  
    }
});

})();
