/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelItem");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.InlineEditBox");
dojo.require("dojo.fx");

dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.ActionRegistry");

(function() {

var ARROW_WIDTH = 12;
var ICON_WIDTH  = 20;
var SLACK_WIDTH = 30; // TODO: make the slack width width in case there's scrollbar. If we implement panel paging, this would not be necessary.

dojo.declare("PanelItem.InlineEditBox",
             [dijit.InlineEditBox],
{
    postCreate: function()
    {
        this.inherited("postCreate", arguments);
    },
    
    focus: function()
    {
        this.inFocusing = true;
        this._onClick();
        this.inFocusing = false;
    },
    
    _onClick: function()
    {
        this.preEdit();
        this.inherited("_onClick", arguments);
        
        // don't generate clicked event when we're only focusing.
        if(!this.inFocusing)
            this.clicked();   
    },
    
    _showText: function()
    {
        this.inherited("_showText", arguments);
        this.postEdit();
    },
    
    edit: function()
    {
        // maximize the width of the editor to at least match
        // our display node's width.
        var minWidth = this.minWidthHint || 0;
        this.width = Math.max(this.domNode.offsetWidth, minWidth) + "px";
        
        this.inherited("edit", arguments);
    },
    
    clicked: function()
    {
    },
    
    preEdit: function()
    {
    },
    
    postEdit: function()
    {
    }
    
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelItem",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<li id="${id}" dojoAttachPoint="domNode,focusNode" waiRole="button" dojoAttachEvent="onclick:_openHandler,onkeypress:_keyPressHandler" >' + 
                    '  <div id="${id}" dojoAttachPoint="containerNode" waiRole="presentation" class="com-ibm-asq-common-widget-htree-panel-item">' +
                    '    <span waiRole="presentation" class="com-ibm-asq-common-widget-htree-panel-arrow" dojoAttachPoint="arrowIcon">' +
                    '    </span>' +
                    '    <img waiRole="presentation" src="${_blankGif}" class="com-ibm-asq-common-widget-htree-panel-icon" dojoAttachPoint="iconLabelNode" style="display:none">' +
                    '    </img>' +
                    '    <span waiRole="presentation" class="com-ibm-asq-common-widget-htree-panel-label" dojoAttachPoint="labelNode">' +
                    '    </span>' +
                    '    <span dojoAttachPoint="labelEditNode" dojoType="PanelItem.InlineEditBox" dojoAttachEvent="clicked:_openHandler,onChange:_nameChanged,preEdit:_handlePreEdit,postEdit:_handlePostEdit">' +
                    '    </span>' +
                    '  </div>' +
                    '</li>',

    widgetsInTemplate: true,
    label: "",
    model: null,
    modelItem: null,
    selectedState: false,
    openState: false,
    iconClass: null,
    labelClass: null,
    ruler: null,
    widthHint: null,
    
    postCreate: function()
    {
        this.inherited('postCreate', arguments);
        this.editable = false;
        this.disabled = false;
        
        if(this.actionRegistry) {
            var actionDesc = this.actionRegistry.getContextActions(this.model, this.modelItem);
            if(actionDesc) {
                this.actionDesc = this.actionDesc;
                var menu = this.actionRegistry.getContextMenu(actionDesc, dojo.hitch(this, function(action) {
                    // delegate the action handling to our creator.
                    if(this.disabled) {
                        return;
                    }
                    this.onContextActionInvoke(action, this.modelItem);
                }));
                if(menu) {
                    this.menu = menu;
                    
                    // binds the menu and connect the menu's popup event to set our item
                    // to be in focus and to be opened. 
                    this.menuConnect = this.connect(this.menu, "_openMyself", "_handleContextMenuOpen");
                    this.menu.bindDomNode(this.domNode);
                }
            }
        }
        this.update();
    },

    isFocusable: function()
    {
        return true;
    },
    
    update: function()
    {
        this.label = this.model.getLabel(this.modelItem);
        this._applyIconClass();  
        this.setLabel(this.label);
        
        if (this.model.mayHaveChildren(this.modelItem)) {
            dojo.addClass(this.arrowIcon, "children-indicator-unselect-image");
        } 
        else {
            if(dojo.hasClass(this.arrowIcon, "children-indicator-unselect-image"))
                dojo.removeClass(this.arrowIcon, "children-indicator-unselect-image");
            if(dojo.hasClass(this.arrowIcon, "children-indicator-selected-image"))
                dojo.removeClass(this.arrowIcon, "children-indicator-selected-image");
        }   
    },
    
    resize: function(widthHint)
    {
        this.widthHint = widthHint;    
        this._drawLabel();
    },
    
    setDisabled: function(disabled)
    {
        this.disabled = disabled;
        this.setLabel(this.label);
    },
    
    _applyIconClass: function()
    {
        var iconClass = null;
        if(this.getIconClass && dojo.isFunction(this.getIconClass)) {
            iconClass = this.getIconClass(this.modelItem, this.openState);
        }
         this.setIcon(iconClass);
    },
    
    _applyLabelClass: function()
    {
        var labelClass = null;
        if(this.getLabelClass && dojo.isFunction(this.getLabelClass)) {
            labelClass = this.getLabelClass(this.modelItem);
        }
       
        if(!this.editable) {
            if(this.labelClass)
                dojo.removeClass(this.labelNode, this.labelClass);
            dojo.addClass(this.labelNode, labelClass);
        }
        else {
            if(this.labelClass)
                dojo.removeClass(this.labelEditNode.domNode, this.labelClass);
            dojo.addClass(this.labelEditNode.domNode, labelClass);
        }
        this.labelClass = labelClass;
    },
    
    setLabel: function(label)
    {        
        // check for editability
        if(this.model.isLabelEditable && dojo.isFunction(this.model.isLabelEditable)) {
            this.editable = this.model.isLabelEditable(this.modelItem);
        }
        this.label = label;
        
        if(!this.editable || this.disabled) {
            dojo.style(this.labelNode, "display", "");
            dojo.style(this.labelEditNode.domNode, "display", "none");
            this._drawLabel();
        }
        else {
            dojo.style(this.labelNode, "display", "none");
            dojo.style(this.labelEditNode.domNode, "display", "inline");
            this.labelEditNode.attr("noValueIndicator", this.label);
            this.labelEditNode.attr('value', this.label);
        }
        this._applyLabelClass();
    },
    
    _getMaxWidth: function()
    {
        var iconWidth = (this.iconLabelNode.style.display == "none") ? 0 : ICON_WIDTH;
        var arrowWidth = (this.arrowIcon.style.display == "none") ? 0 : ARROW_WIDTH;
        return this.widthHint - iconWidth - arrowWidth - SLACK_WIDTH; // more slack to not get too close to the edge.
        
    },
    
    _drawLabel: function()
    {
        var iconWidth = (this.iconLabelNode.style.display == "none") ? 0 : ICON_WIDTH;
        var arrowWidth = (this.arrowIcon.style.display == "none") ? 0 : ARROW_WIDTH;
        var width = this._getMaxWidth();
        var val = com.ibm.asq.common.web.ui.internal.util.wordWrapping.clipText(this.label, this.labelNode.className, width, this.ruler, true);

        // clear the label node first and redraw.
        this.labelNode.innerHTML = "";
        this.labelNode.appendChild(document.createTextNode(val.text));
        if(val.clipped) {
            this.labelNode.title = this.label;
        }
    },
    
    setIcon: function(iconClass)
    {
        if(this.iconClass == iconClass)
            return;
            
        var lastClass = this.iconClass;
        this.iconClass = iconClass;

        if(lastClass)
            dojo.removeClass(this.iconLabelNode, lastClass);
        
        this._setIcon(iconClass);
    },
    
    _setIcon: function(iconClass)
    {            
        if(iconClass) {
            dojo.style(this.iconLabelNode, "display", "");
            dojo.addClass(this.iconLabelNode, iconClass);
        }
        else {
            dojo.style(this.iconLabelNode, "display", "none");
        }        
    },

    _handleContextMenuOpen: function()
    {
        this.menu.getChildren().forEach(dojo.hitch(this, function(item){
            var action = item.action;
            var enabled = true;
            
            if(this.disabled) {
                item.attr("disabled", true);
            }
            else if(action && action.enablementCallback && dojo.isFunction(action.enablementCallback)) {
                // get the action's enablement state, if one is specified.
                enabled = action.enablementCallback(this.model, this.modelItem)
                item.attr("disabled", !enabled);
            }
        }));

        // trigger the open handler to get the item selected when the user
        // right click on the item.
        this._openHandler();
    },
    
    _keyPressHandler: function(event) 
    {
        if(event.ctrlKey || event.altKey){ return; }
        if(event.charOrCode == dojo.keys.ENTER || event.charOrCode == dojo.keys.SPACE) {
            this._openHandler(event);
        }
          
    },
    
    _openHandler: function(event)
    {
        if (!this.disabled) {
            if (this.openState) 
                return;
            
            this.setOpenState(true);
            this.onOpen(this.modelItem);
        }
        if(event){
            dojo.stopEvent(event);
        }
    },

    _nameChanged: function(newName)
    {
        // notify the model of the label changed.
        this.model.labelChanged(this.modelItem, newName);
        this.label = this.model.getLabel(this.modelItem);
         
        // the inline editbox is 'recoiling' at this point.
        // need to wait until the dust settles before applying
        // text style on it.
        setTimeout(dojo.hitch(this, "_applyLabelClass"), 200);
    },

    _handlePreEdit: function(event)
    {
        this.labelEditNode.minWidthHint = this._getMaxWidth();

        // remove any text style before we entering edit.
        if(this.labelClass)
            dojo.removeClass(this.labelEditNode.domNode, this.labelClass);
            
        if(event){
            dojo.stopEvent(event);
        }
    },

    _handlePostEdit: function(event)
    {
        // restore the text style after we edit.
        if(this.labelClass)
            dojo.addClass(this.labelEditNode.domNode, this.labelClass);
          if(event){
            dojo.stopEvent(event);
        }
    },
     
    setOpenState: function(openState)
    {
        this.openState = openState;
        
        this._applyIconClass();
        if (this.openState) {
            dojo.addClass(this.containerNode, "com-ibm-asq-common-widget-htree-panel-item-open");
            if (this.model.mayHaveChildren(this.modelItem)) {
               if(dojo.hasClass(this.arrowIcon, "children-indicator-unselect-image"))
                    dojo.removeClass(this.arrowIcon, "children-indicator-unselect-image");
                dojo.addClass(this.arrowIcon, "children-indicator-selected-image");
            }
        }
        else {
            dojo.removeClass(this.containerNode, "com-ibm-asq-common-widget-htree-panel-item-open");
            if (this.model.mayHaveChildren(this.modelItem)) {
               if(dojo.hasClass(this.arrowIcon, "children-indicator-selected-image"))
                    dojo.removeClass(this.arrowIcon, "children-indicator-selected-image");
                dojo.addClass(this.arrowIcon, "children-indicator-unselect-image");
            }
        }
    },
    
    
    setSelectState: function(selectedState)
    {
        this.selectedState = selectedState;
        
        if(this.selectedState)
            dojo.addClass(this.containerNode, "com-ibm-asq-common-widget-htree-panel-item-select");
        else 
            dojo.removeClass(this.containerNode, "com-ibm-asq-common-widget-htree-panel-item-select");
    },
    
    onOpen: function()
    {
        
    },
    
    onContextActionInvoke: function(action, item)
    {
        
    },
    
    focus: function()
    {
        if(this.editable) {
            this.labelEditNode.focus();
        }
        else {
            this.focusNode.focus();
        }
    },

    uninitialize: function()
    {
        var menu = this.menu;
        
        // clean up the menu if one is attached.
        if(menu) {
            menu.unBindDomNode(this.domNode);
            menu.destroyRecursive();
        }  
    },
    dummy:null
});

})();
