/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.FormDateField");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.datetimepicker.DateTimePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField");
dojo.require("jazz.ui.HoverPopup");

(function() {
var Constants = com.ibm.asq.common.web.client.internal.Constants;

var YEAR   = 31536000000; // 365 * 24 * 60 * 60 * 1000
var MONTH  = 2592000000; // 30 * 24 * 60 * 60 * 1000
var WEEK   = 604800000; // 7 * 24 * 60 * 60 * 1000
var DAY    = 86400000; // 24 * 60 * 60 * 1000
var HOUR   = 3600000; // 60 * 60 * 1000
var MINUTE = 60000; //  60 * 1000

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormDateField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    currentValue: null,
    disabled: false,
    
    textFieldInvalidFormatMsg: "",
    postCreate: function() {    
        this.inherited("postCreate", arguments);
        this.dateField = new com.ibm.asq.common.web.ui.internal.widgets.DatePicker({
                    fieldLabel: this.fieldLabel,
                    name: "dateField",
                    value: "",
                    invalidMessage: Constants.getMessage("dtpInvalidDate"),
                    onChange: dojo.hitch(this, "_valueChangedHandler"),
                    onkeyup: dojo.hitch(this, "_valueChangedHandler"),
                    'class': this._getBaseClassName()
                });
        this.fieldHolderNode.appendChild(this.dateField.domNode);
        if(this.defaultValue) {
            this.setValue(this.defaultValue);
        }
        this.dateField.init();
        this.formField = this.dateField.domNode;

        if (this.dateField.textbox) {
            this._attachLabelToField(this.dateField.textbox, this.fieldLabel + "-input");
        }    
        
    },

    _valueChangedHandler: function()
    {
        if(this.inSetValue)
            return;
            
        var oldValue = this.currentValue;
        this.currentValue = this.getValue(); 
        
        this.setHasError(!this.dateField.isValid());
        this.onValueChanged(this, oldValue, this.currentValue);
    },
    
    setHasError: function(hasError) {
        if (!hasError && this.textFieldInvalidFormatMsg != "")
            // restore the original invalid format message on no error
            this.setErrorMessage(this.textFieldInvalidFormatMsg);
            
        this.inherited("setHasError", arguments);
    },
    
    setErrorMessage: function(message)
    {
        if (this.textFieldInvalidFormatMsg == "")
            // save the original invalid format message
            this.textFieldInvalidFormatMsg = this.dateField.getErrorMessage();
            
        this.dateField.invalidMessage = message;
    },
    
    setEdited: function(isEdited)
    {
        if (!this.dateField.isValid()) {
            // use error condition for invalid entry 
            this.setHasError(true);
        } else {
            this.inherited("setEdited", arguments);
        }
    },
    
    setModified: function(isEdited)
    {
        if (!this.dateField.isValid()) {
            // use error condition for invalid entry 
            this.setHasError(true);
        } else {
            this.inherited("setModified", arguments);
        }
    },
    
    setValue: function(value)
    {
        var val = (value) ? value : "";
        var intVal = parseInt(val);
        
        // It's an empty value, don't have to
        // set it to the date box.
        if(intVal <= 0)
            return;
            
        var date = new Date();
        date.setTime(intVal);
        
        this.inSetValue = true;
        this.dateField.setValue(date);
        this.inSetValue = false;
    },

    // Summary:
    //    Returns the value from the field. Subclass must override. A null is returned if
    //    no value is specified.
    //
    getValue: function()
    {
        var val = this.dateField.getValue();
        if(val)
            return val.getTime() + "";
        return "";    
    },
    
    // Summary:
    //    This method is called when the field's value has been changed. Consumer of the widget 
    //    overrides this method to receive notification of the value changed event.
    //
    onValueChanged: function(oldValue, newValue)
    {
    },
    
    _getBaseClassName: function()
    {
        return "date-field";
    },
    
    clearValue: function(){    
        this.dateField.textbox.value  = "";
    }
});

// A widget with two display modes for specifying a relate date from now or a range
// of from and to dates with time.
//
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout._DateTimeRangeField", 
    [dijit._Widget, dijit._Templated],
{
    widgetsInTemplate: true,
    minutes: Constants.getMessage("labelFromMinutesAgo", ""),
    hours: Constants.getMessage("labelFromHoursAgo", ""),
    days: Constants.getMessage("labelFromDaysAgo", ""),
    weeks: Constants.getMessage("labelFromWeeksAgo", ""),
    months: Constants.getMessage("labelFromMonthsAgo", ""),
    years: Constants.getMessage("labelFromYearsAgo", ""),
    relativeDate: Constants.getMessage("labelRelativeDate"),
    specificDate: Constants.getMessage("labelSpecificDate"),
    selectLabel: Constants.getMessage("msgSelectDateType"),
    
    templateString: ['<table id="${id}" dojoAttachPoint="domNode" class="asq-section-form-field" tabindex="-1" border="0" cellpadding="0" cellspacing="0" width="100%">' ,
                    '  <tr>' ,
                    '    <td tabindex="-1" colspan="2" dojoAttachPoint="fieldRelDateSpacerNode" style="height:2em !important"></td>' ,
                    '    <td tabindex="-1" colspan="2" dojoAttachPoint="fieldFromNode" style="white-space:nowrap;height:2em !important"></td>' ,
                    '  </tr>' ,
                    '  <tr>' ,
                    '    <td tabindex="-1" dojoAttachPoint="fieldRelDateNode" style="white-space:nowrap; height:2em !important"></td>' ,
                    '    <td tabindex="-1" dojoAttachPoint="fieldRelDateSelectorNode" >' ,
                    '      <label for="${id}_date_unit_select"><span style="display:none">${selectLabel}</span></label>' ,
                    '        <select id="${id}_date_unit_select" dojoAttachPoint="relDateUnitSelector" dojoAttachEvent="onchange:_dateValueChanged" style="padding:1px">' ,
                    '          <option value="minutes">${minutes}</option>' ,
                    '          <option value="hours">${hours}</option>' ,
                    '          <option value="days">${days}</option>' ,
                    '          <option value="weeks" selected="true">${weeks}</option>' ,
                    '          <option value="months">${months}</option>' ,
                    '          <option value="years">${years}</option>' ,
                    '        </select>' ,
                    '    </td>' ,
                    '    <td tabindex="-1" colspan="2" dojoAttachPoint="fieldToNode" style="white-space:nowrap; height:2em !important"></td>' ,
                    '  </tr>' ,
                    '  <tr>' ,
                    '    <td tabindex="-1" colspan="2" style="white-space:nowrap; padding-top:10px">' ,
                    '      <input type="radio" name="dateType" value="REL" checked="checked" dojoAttachPoint="relDateNode" dojoAttachEvent="onclick:_onDateTypeChange"> ${relativeDate} &nbsp;' ,
                    '      <input type="radio" name="dateType" value="SPEC" dojoAttachPoint="specDateNode" dojoAttachEvent="onclick:_onDateTypeChange"> ${specificDate}' ,
                    '    </td>' ,
                    '  </tr>' ,
                    '</table>'].join(""),

    constructor: function() {
        // default to use relative date of one week ago
        this.relative = true;
        this.currentValue = {
            fromDateTime: 0,
            toDateTime: 0,
            relative: "true"  // use by UI only to maintain display state on table viewer refresh
        };
    },
    
    postCreate: function() {
        var now = new Date();
        // specific dates have from and to date time pickers
        this.fromDateField = new com.ibm.asq.common.web.ui.internal.widgets.datetimepicker.DateTimePicker({
                                 label: Constants.getMessage("labelFrom"),
                                 name: "fromDateTime",
                                 invalidMessage: Constants.getMessage("dtpInvalidDate"),
                                 onDateChange: dojo.hitch(this, "_onFromDateChanged"),
                                 onTimeChange: dojo.hitch(this, "_onFromTimeChanged"),
                                 onKeyUp: dojo.hitch(this, "_onFromDateChanged"),
                                 "dateClass": "text-field"
                             });
        dojo.style(this.fromDateField.labelNode, "width", "4em");
        this.fieldFromNode.appendChild(this.fromDateField.domNode);

        this.toDateField = new com.ibm.asq.common.web.ui.internal.widgets.datetimepicker.DateTimePicker({
                               label: Constants.getMessage("labelTo"),
                               name: "toDateTime",
                               invalidMessage: Constants.getMessage("dtpInvalidDate"),
                               onDateChange: dojo.hitch(this, "_onToDateChanged"),
                               onTimeChange: dojo.hitch(this, "_onToTimeChanged"),
                               onKeyUp: dojo.hitch(this, "_onToDateChanged"),
                               "dateClass": "text-field"
                           });
        dojo.style(this.toDateField.labelNode, "width", "4em");
        this.fieldToNode.appendChild(this.toDateField.domNode);

        // relative date has a numeric text field and date type drop-down select
        this.relDate = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField({
                           fieldLabel: Constants.getMessage("labelFromDate"),  // special "From" label when used with "ago" in the drop down
                           maxLength: 4,                                       // to address string concatenation from different UI elements
                           onChange: dojo.hitch(this, "_dateValueChanged")
                       });
        this.fieldRelDateNode.appendChild(this.relDate.domNode);        

        // initialize the showing of relative date, relative date radio button is selected by default 
        this._onDateTypeChange();
    },

    uninitialize: function() {
        this.fromDateField.destroy();
        this.toDateField.destroy();
        this.relDate.destroy();
    },

    // relative and specific dates radio buttons change handler to hide and show
    // appropriate display
    //
    _onDateTypeChange: function(event) {
        this.relative = this.relDateNode.checked;
        var relativeDisplay = this.relDateNode.checked ? "" : "none";
        var specificDisplay = this.specDateNode.checked ? "" : "none";
        dojo.style(this.fieldRelDateSpacerNode, "display", relativeDisplay);
        dojo.style(this.fieldRelDateNode, "display", relativeDisplay);
        dojo.style(this.fieldRelDateSelectorNode, "display", relativeDisplay);
        dojo.style(this.fieldFromNode, "display", specificDisplay);
        dojo.style(this.fieldToNode, "display", specificDisplay);

        // set the current date widget
        if (this.relative) {
            this.dateFieldChanged = this.relDate;
            this.invalidRange = false;  // invalid date range does not apply to relative date
        } else {
            this.dateFieldChanged = this.fromDateField;
        }
        
    },
    
    // change handler for the from date field to change the min constraint value of the
    // to date field with the new from value
    //
    _onFromDateChanged: function() {
        if (this.fromDateField && this.fromDateField.isValid()) {
            this.dateFieldChanged = this.fromDateField;
            this.timeFieldChanged = false;  // change is not from the time field - use in invalid range error tool tip
        
            var fromDateTime = this.fromDateField.getValue();
            var minDate = new Date();
            if (fromDateTime) {
                var fromTime = this.fromDateField.timeField.getTime();
                fromDateTime.setHours(fromTime.getHours(), fromTime.getMinutes(), 0, 0);
                minDate.setTime(fromDateTime.getTime());
            } else {
                // from date is blank, no min constraint for the to date field
                minDate = -Infinity;
            }
            this.toDateField.dateNode.constraints['min'] = minDate;
                
            // force an update of the calendar in the to field to pick up the new min date
            if (this.toDateField.dateNode._picker) {
                var picker = this.toDateField.dateNode._picker;
                delete picker;
                this.toDateField.dateNode._picker = null;
            }
            this.toDateField.dateNode.isValid(false);
            this._dateValueChanged();
        }
    },

    // change handler for the to date field to change the max constraint value of the
    // from date field with the new to value
    //
    _onToDateChanged: function() {
        if (this.toDateField && this.toDateField.isValid()) {
            this.dateFieldChanged = this.toDateField;
            this.timeFieldChanged = false;  // change is not from the time field - use in invalid range error tool tip 
        
            var toDateTime = this.toDateField.getValue();
            var maxDate = new Date();
            if (toDateTime) {
                var toTime = this.toDateField.timeField.getTime();
                toDateTime.setHours(toTime.getHours(), toTime.getMinutes(), 0, 0);
                maxDate.setTime(toDateTime.getTime());
            } else {
                // to date is blank, no max constraint for the from date field
                maxDate = +Infinity;
            }
            this.fromDateField.dateNode.constraints['max'] = maxDate;
                
            // force an update of the calendar in the from field to pick up the new max date
            if (this.fromDateField.dateNode._picker) {
                var picker = this.fromDateField.dateNode._picker;
                delete picker;
                this.fromDateField.dateNode._picker = null;
            }
            this.fromDateField.dateNode.isValid(false);
            this._dateValueChanged();
        }
    },

    _onFromTimeChanged: function() {
        if (this.fromDateField && this.fromDateField.isValid()) {
            this.dateFieldChanged = this.fromDateField;
            this.timeFieldChanged = true;  // change is from the time field - use in invalid range error tool tip
            this._dateValueChanged();
        }
    },
    
    _onToTimeChanged: function() {
        if (this.toDateField && this.toDateField.isValid()) {
            this.dateFieldChanged = this.toDateField;
            this.timeFieldChanged = true;  // change is from the time field - use in invalid range error tool tip
            this._dateValueChanged();
        }
    },
    
    // validate if the time range specified is valid
    //
    _isDateTimeRangeValid: function() {
        if (this.fromDateField.isValid() && this.toDateField.isValid()) {
            var fromDate = this.fromDateField.getValue();
            var toDate = this.toDateField.getValue();
            if (fromDate && toDate) {
                // both of the from and to dates are specified
                var fromDateTime = fromDate.getTime();
                var toDateTime = toDate.getTime();
                return (fromDateTime < toDateTime);
            } else {
                // one or both of the from and to dates are empty 
                return true;
            }
        }
        return false;
    },
    
    _dateValueChanged: function(event) {
        var oldValue = this.currentValue;
        this.currentValue = this.getValue(); 
        
        if (!this.relative) {
            // reset invalid range error tool tip if there is any 
            if (this.invalidRabgeTooltip) {
                this.invalidRabgeTooltip.close();
                this.invalidRabgeTooltip.destroy();
            }

            if (!this._isDateTimeRangeValid()) {
                this.invalidRange = true;
                this.invalidRabgeTooltip = new jazz.ui.HoverPopup({
                    html: Constants.getMessage("msgInvalidTimeRange"),
                    openDelay: 300,
                    around: this.timeFieldChanged ? [this.dateFieldChanged.timeField.domNode] : [this.dateFieldChanged.dateNode.domNode]
                });
                this.invalidRabgeTooltip.open();
            } else {
                this.invalidRange = false;
            }
        }
        this.onValueChanged(this, oldValue, this.currentValue);
         if(event){
            dojo.stopEvent(event);
        }
    },
    
    setValue: function(value) {
        if (value) {
            var fromVal = value.fromDateTime ? value.fromDateTime : "";
            var toVal = value.toDateTime ? value.toDateTime : "";
            this.relative = value.relative ? (value.relative == "true") : true;
            
            this.relDateNode.checked = this.relative;
            this.specDateNode.checked = !this.relative;

            // update the date display
            this._onDateTypeChange();
            
            var intFromVal = parseInt(fromVal);
            var intToVal = parseInt(toVal);
            
            if (fromVal > 0) {
                var fromDate = new Date();
                fromDate.setTime(intFromVal);
            }
            
            if (toVal > 0) {
                var toDate = new Date();
                toDate.setTime(intToVal);
            }
            
            if (this.relative) {
                // setting a date range does not apply to relative date,
                // ignore the to date and use rough approximation
                var now = new Date().getTime();
                var dateDiff = now - intFromVal;
                
                if (dateDiff >= YEAR) {
                    this.relDate.setValue(Math.floor(dateDiff / YEAR));
                    this.relDateUnitSelector.selectedIndex = 5;
                }
                else if (dateDiff >= MONTH) {
                    this.relDate.setValue(Math.floor(dateDiff / MONTH));
                    this.relDateUnitSelector.selectedIndex = 4;
                }
                else if (dateDiff >= WEEK) {
                    this.relDate.setValue(Math.floor(dateDiff / WEEK));
                    this.relDateUnitSelector.selectedIndex = 3;
                }
                else if (dateDiff >= DAY) {
                    this.relDate.setValue(Math.floor(dateDiff / DAY));
                    this.relDateUnitSelector.selectedIndex = 2;
                }
                else if (dateDiff >= HOUR) {
                    this.relDate.setValue(Math.floor(dateDiff / HOUR));
                    this.relDateUnitSelector.selectedIndex = 1;
                }
                else {
                    this.relDate.setValue(Math.floor(dateDiff / MINUTE));
                    this.relDateUnitSelector.selectedIndex = 0;
                }
            } else {
                if (fromVal > 0) {
                    this.fromDateField.dateNode.setValue(fromDate);
                    this.fromDateField.timeField.setTime(fromDate);
                } else {
                    this.fromDateField.clear();
                }
                if (toVal > 0) {
                    this.toDateField.dateNode.setValue(toDate);
                    this.toDateField.timeField.setTime(toDate);
                } else {
                    this.toDateField.clear();
                }
            }
            this._dateValueChanged();
        }
    },

    getValue: function() {
        var val; 
        if (this.relative) {
            // relative date is determined by the specific number of date units from now
            var from = this.relDate.textField.isValid() ? this.relDate.getValue() : 0;
            from = parseInt(from);
            if (from > 0) {
                var selectedDateUnitIndex = this.relDateUnitSelector.selectedIndex;
                switch (this.relDateUnitSelector.options[selectedDateUnitIndex].value) {
                    case "minutes":
                        from = from * MINUTE;
                        break;
                    case "hours":
                        from = from * HOUR;
                        break;
                    case "days":
                        from = from * DAY;
                        break;
                    case "weeks":
                        from = from * WEEK;
                        break;
                    case "months":
                        from = from * MONTH;
                        break;
                    case "years":
                        from = from * YEAR;
                        break;
                };
                var now = new Date().getTime();
                val = {
                    fromDateTime: Math.max(now - from, 0),
                    toDateTime: 0,
                    relative: "true"
                }
            } else {
                // relative number is blank
                val = {
                    fromDateTime: 0,
                    toDateTime: 0,
                    relative: "true"
                }
            }
        } else {
            var fromDate = this.fromDateField.isValid() ? this.fromDateField.getValue() : null;
            var toDate = this.toDateField.isValid() ? this.toDateField.getValue() : null;
            val = {
                fromDateTime: fromDate ? fromDate.getTime() : 0,
                toDateTime: toDate ? toDate.getTime() : 0,
                relative: "false"
            }
        }
        return val;
    },
    
    getValueInString: function() {
        var val = this.getValue();
            
        if (this.relative) {
            var from = this.relDate.textField.isValid() ? this.relDate.getValue() : 0;
            from = parseInt(from);
            if (from > 0) {
                var selectedDateUnitIndex = this.relDateUnitSelector.selectedIndex;
                switch (this.relDateUnitSelector.options[selectedDateUnitIndex].value) {
                    case "minutes":
                        return Constants.getMessage("labelMinutesAgo", from);
                    case "hours":
                        return Constants.getMessage("labelHoursAgo", from);
                    case "days":
                        return Constants.getMessage("labelDaysAgo", from);
                    case "weeks":
                        return Constants.getMessage("labelWeeksAgo", from);
                    case "months":
                        return Constants.getMessage("labelMonthsAgo", from);
                    case "years":
                        return Constants.getMessage("labelYearsAgo", from);
                };
            } else {
                return Constants.getMessage("labelAny");
            }
        } else if (!this.fromDateField.isValid() || !this.toDateField.isValid()) {
            return Constants.getMessage("msgInvalidDate");
        } else if (this.invalidRange) {
            return Constants.getMessage("msgInvalidTimeRange");
        } else {
            if (val.fromDateTime && val.toDateTime) {
                var fromDate = new Date();
                fromDate.setTime(val.fromDateTime);
                var fromStr = dojo.date.locale.format(fromDate, {formatLength: "medium"});
                var toDate = new Date();
                toDate.setTime(val.toDateTime);
                var toStr = dojo.date.locale.format(toDate, {formatLength: "medium"});
                return Constants.getMessage("msgFromDateToDate", fromStr, toStr);
            } else if (val.fromDateTime) {
                var fromDate = new Date();
                fromDate.setTime(val.fromDateTime);
                var fromStr = dojo.date.locale.format(fromDate, {formatLength: "medium"});
                return Constants.getMessage("msgFromDateToNow", fromStr);
            } else if (val.toDateTime){
                var toDate = new Date();
                toDate.setTime(val.toDateTime);
                var toStr = dojo.date.locale.format(toDate, {formatLength: "medium"});
                return Constants.getMessage("msgFromBeforeToDate", toStr);
            } else {
                return Constants.getMessage("labelAny");
            }
        }
        return "";   
    },

    clearValue: function() {
        // the last display mode is cached in order to restore to the same display mode when
        // the table viewer refresh table with a clear and set of the widget
        var currentDisplayState = this.currentValue ? this.currentValue.relative : "true";
        this.currentValue = {
            fromDateTime: 0,
            toDateTime: 0,
            relative: currentDisplayState  // use by UI only to maintain display state on table viewer refresh
        };
        this.setValue(this.currentValue);
    },
    
    isValid: function() {
        if (this.relative) {
            return this.relDate.textField.isValid();
        } else {
            return this.fromDateField.isValid() && this.toDateField.isValid() && !this.invalidRange;
        }
    }
});

// a form field that uses the _DateTimeRangeField above.
//
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormDateTimeRangeField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    currentValue: null,
    disabled: false,
    
    textFieldInvalidFormatMsg: Constants.getMessage("dtpInvalidDate"),
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.dateTimeField = new com.ibm.asq.common.web.ui.internal.widgets.layout._DateTimeRangeField({
                    onValueChanged: dojo.hitch(this, "onValueChanged")
                });
        this.fieldHolderNode.appendChild(this.dateTimeField.domNode);
        if (this.defaultValue) {
            this.dateTimeField.setValue(this.defaultValue);
        }
        this.formField = this.dateTimeField.domNode;
    },

    uninitialize: function() {
        this.dateTimeField.destroy();
    },

    setHasError: function(hasError) {
        if (!hasError && this.textFieldInvalidFormatMsg != "") {
            // restore the original invalid format message on no error
            this.setErrorMessage(this.textFieldInvalidFormatMsg);
        }
        this.inherited("setHasError", arguments);
    },
    
    setErrorMessage: function(message) {
        if (this.textFieldInvalidFormatMsg == "") {
            // save the original invalid format message
            this.textFieldInvalidFormatMsg = this.dateField.getErrorMessage();
        }
        if (this.dateTimeField.dateFieldChanged) {
            this.dateTimeField.dateFieldChanged.invalidMessage = message;
        }
    },
    
    setEdited: function(isEdited) {
        if (!this.dateTimeField.isValid()) {
            // use error condition for invalid entry 
            this.setHasError(true);
        } else {
            this.inherited("setEdited", arguments);
        }
    },
    
    setModified: function(isEdited) {
        if (!this.dateTimeField.isValid()) {
            // use error condition for invalid entry 
            this.setHasError(true);
        } else {
            this.inherited("setModified", arguments);
        }
    },
    
    setValue: function(value) {
        this.dateTimeField.setValue(value);
    },

    getValue: function() {
        return this.dateTimeField.getValue();
    },
    
    getValueInString: function() {
        return this.dateTimeField.getValueInString();
    },

    clearValue: function(){    
        this.dateTimeField.clearValue();
    }
});

})();
