/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.FormEnumField");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField");

(function() {
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormEnumField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
	// Summary:
	//    if set to true, an empty choice (option) will always be created
	//    at the first option in the dropdown list. 
	createEmpty: false,
    value: null,
	choices: null,
	currentValue: null,
	disabled: false,
    
    // if there's no default selection and no empty item, indicate to still select
    // the first item. Override to true to select first item with no default value & no empty item.
    defaultSelectFirstItem: false,
	
    postCreate: function() {
        this.inherited("postCreate", arguments);
		
        this.selectField = document.createElement("select");
        this.fieldHolderNode.appendChild(this.selectField);
		
        this.formField = this.selectField;
        this.formField.className =  this._getBaseClassName();
        this.connect(this.formField, "onchange", "_valueChangedHandler");
		this.formField.disabled = this.disabled;

        this._attachLabelToField(this.selectField, this.fieldLabel + "-select");
        
        if(this.choices) {
            this.setEnumChoices(this.choices);
        }
    },
    
	_valueChangedHandler: function()
	{
		var oldValue = this.currentValue;
		this.currentValue = this._getSelectValue();

		this.onValueChanged(this, oldValue, this.currentValue);
	},
	
    setValue: function(value)
    {
		this.value = value;
		this.selectField.length = 0;
		this.setEnumChoices(this.choices);
    },

    clearValue: function() {
        this.defaultValue = null;
        this.setValue(null);
    },

    // Summary:
    //    Returns the value from the field. Subclass must override. A null is returned if
    //    no value is specified.
    //
    getValue: function()
    {
        return this._getSelectValue();
    },
	
    // Summary:
    //    Set the choices for the enum field. The data can a simple array of strings. Or
    //    individual choices can be an object with these attributes: {'name':'xxx', 'value':'xxx','indent':true|false}
    //    the 'name' member must be set if the choice is an object. The client can have mixed strings
    //    and objects in the choices array.
    //
	setEnumChoices: function(/* array of strings | objects */choices)
	{
        this.choices = choices;
		
        var pos = 0;

        // create an empty selection
        if(this.createEmpty) {
            this.selectField.options[0] = new Option("");
            pos = 1;
        }
		
        var currentValue = this.value || this.defaultValue;
        
        if(choices && choices.length > 0) {
            for (var i = 0; i < choices.length; i++) {
                var name = choices[i];
                var value = name;
                var indent = false;
                
                // if the choice is an object, the client can specify
                // name, value and indent level. 
                if(dojo.isObject(choices[i])) {
                    name = choices[i].name;
                    value = (choices[i].value != null) ? choices[i].value : choices[i].name; 
                    indent = (choices[i].indent || false);
                }
                
                // IE and others don't support css in Option.
                // Need to indent the string here.
                if (indent && !dojo.isFF) {
                    name = "    " + name;
                }
                
                // always creates the option w/ both display name and value
                // 
                var option = new Option(name, value);
                
                if (indent && !dojo.isIE) {
                    option.className = "enum-field-option-indent";
                }
                this.selectField.options[i+pos] = option;
                
                // make this option selected (shown) if a 
                // selectValue is specified and it matches this
                // option.
                if(currentValue && value == currentValue)
                    this.selectField.options[i+pos].selected = true;
            }
        }
        // don't select anything if we don't have a default value
        // and there's not an empty option.
        if (!currentValue && !this.createEmpty) {
            this.selectField.selectedIndex = (this.defaultSelectFirstItem) ? 0 : -1;
        }
	},

    _getSelectValue: function() 
    {
        for(var i = 0; i < this.selectField.options.length; i++) {
            if(this.selectField.options[i].selected) {
                return this.selectField.options[i].value;
            }
        }
        return null;
    },

    // Summary:
    //    This method is called when the field's value has been changed. Consumer of the widget 
    //    overrides this method to receive notification of the value changed event.
    //
    onValueChanged: function(oldValue, newValue)
    {
    },

    _getBaseClassName: function()
    {
        return "enum-field";
    }
});

})();
