/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");

dojo.require("dijit.form.TextBox");
dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.ASQValidateTextBox");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField");

(function() {

var ValidateTextBox = com.ibm.asq.common.web.ui.internal.widgets.layout.ASQValidateTextBox;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    currentValue: null,
    disabled: false,
	
    textFieldInvalidFormatMsg: "",
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        var textFieldArgs = {
            name: "textField",
            externalValidation: true,
            reportChangeWhenEdit: true,
            value: "",
            'class': this._getBaseClassName(),
            required: this.required ? true : false,
            disabled: this.disabled
        };
        if (this.maxLength) {
            textFieldArgs.maxLength = this.maxLength;
        }
        this.textField = new ValidateTextBox(textFieldArgs);
        if(this.regExp) {
            this.textField.regExp = this.regExp;
            this.textField.externalValidation = false;
        }
        var parent = (this.alterHolderNode || this.fieldHolderNode);
        parent.appendChild(this.textField.domNode);
        this.connect(this.textField, "onChange", "_valueChangedHandler");
        
        if(this.defaultValue) {
            this.setValue(this.defaultValue);
        }
        this.formField = this.textField.domNode;
    },
    
    uninitialize: function() {
    	this.textField.destroy();
    },

    _valueChangedHandler: function()
    {
        var oldValue = this.currentValue;
        this.currentValue = this.getValue(); 
        
        this.setHasError(!this.textField.isValid());
        this.onValueChanged(this, oldValue, this.currentValue);
        if (this.textField.externalValidation)
            // for external validation, use validate() will implicitly show the error tooltip
            this.textField.validate(true);
    },
    
    setHasError: function(hasError) {
        if (!hasError && this.textFieldInvalidFormatMsg != "")
            // restore the original invalid format message on no error
            this.setErrorMessage(this.textFieldInvalidFormatMsg);
            
        this.inherited("setHasError", arguments);
        this.textField.setIsValid(!hasError);
    },
    
    setErrorMessage: function(message)
    {
        if (this.textFieldInvalidFormatMsg == "")
            // save the original invalid format message
            this.textFieldInvalidFormatMsg = this.textField.getErrorMessage();
            
        this.textField.invalidMessage = message;
    },
	
	setPromptMessage: function(message)
	{
		this.textField.promptMessage = message;
	},
    
    setEdited: function(isEdited)
    {
        if (!this.textField.isValid()) {
            // use error condition for invalid entry 
            this.setHasError(true);
        } else {
            this.inherited("setEdited", arguments);
        }
    },
    
	setModified: function(isEdited)
	{
		if (!this.textField.isValid()) {
			// use error condition for invalid entry 
			this.setHasError(true);
		} else {
			this.inherited("setModified", arguments);
		}
    },
	
    setValue: function(value)
    {
        var val = (value) ? value : "";
        this.textField.attr('value',val);
    },

    clearValue: function() {
        this.setValue("");
    },

    // Summary:
    //    Returns the value from the field. Subclass must override. A null is returned if
    //    no value is specified.
    //
    getValue: function()
    {
        var val = this.textField.getValue();
        return val;
    },
    
    // Summary:
    //    This method is called when the field's value has been changed. Consumer of the widget 
    //    overrides this method to receive notification of the value changed event.
    //
    onValueChanged: function(oldValue, newValue)
    {
    },
    
    _getBaseClassName: function()
    {
        return "text-field";
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormLabelField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    showFieldLabel: false,

    // label only
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.labelField = document.createElement("div");
        this.fieldHolderNode.appendChild(this.labelField);
        this.formField = this.labelField;
        this.formField.className = this.className ? this.className : "field-label";
        if(this.fieldLabel) {
            this.labelField.innerHTML = this.fieldLabel;
        }
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractDigitField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField,
{
    opFieldTemplate: 
              '<table dojoAttachPoint="domNode" class="text-field-inner-table">' +
              ' <tr>' +
              '  <td>' +
              '    <select dojoAttachPoint="opNode">' +
              '    </select>' +
              '  </td>' +
              '  <td width="100%" dojoAttachPoint="fieldNode"></td>' +
              ' </tr>' +
              '</table>',

	_lastSelectedNumericOps: "=",
	
    postCreate: function() {
        // if we're showing a numeric operator for the value, create
        // a inner field holder for both operator and field.
        //
        if(this.showNumericOperators) {
            this.holder = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(this.opFieldTemplate);            
            this.numericOps = this.holder.opNode;
            this.numericOps.options[0] = new Option(" = ", "=", false, true);
            this.numericOps.options[1] = new Option(" < ", "<");
            this.numericOps.options[2] = new Option(" <= ", "<=");
            this.numericOps.options[3] = new Option(" > ", ">");
            this.numericOps.options[4] = new Option(" >= ", ">=");
            this.numericOps.style.marginRight = "2px";
            
            this.fieldHolderNode.appendChild(this.holder.domNode);
            this.alterHolderNode = this.holder.fieldNode;
        }
        this.inherited("postCreate", arguments);        
    },
    
    _getBaseClassName: function()
    {
        // we're showing the operator. The input field
        // lives inside a table, make it fully occupy the
        // entire inner space.
        if(this.showNumericOperators)
            return "numeric-op-text-field";
        return "text-field";
    },

    // Override get base getValue to add operator value
    setValue: function(value)
    {
        var val = "";
        
        if(dojo.isObject(value)) {
            if(value.value) {
                val = value.value;
            }
            if (value.operator) {
                for (var i = 0; i < this.numericOps.options.length; i++) {
                    this.numericOps.options[i].selected = false;
                    if (this.numericOps.options[i].value == value.operator) {
                        this.numericOps.options[i].selected = true;
                    }
                }
            }
        }
        else if(dojo.isString(value) || value instanceof Number ||  typeof value == "number") {
            val = value;
        }
        this.textField.attr('value', val);
    },
    
    // Override get base getValue to add operator value
    getValue: function()
    {
        var val = this.textField.getValue();

        if (this.showNumericOperators) {
            var numericPart = val;
            val = {
                'value': numericPart,
                'operator': this._getOperatorValue()
            };
        }
        return val;
    },
    
    getValueInString: function()
    {
        var val = this.getValue();
        
        if(dojo.isObject(val)) {
            val = (val.value && val.value.length > 0) ? (val.operator + val.value) : "";
        }
        return val;   
    },
    
    _getOperatorValue: function()
    {
        for(var i = 0; i < this.numericOps.options.length; i++) {
            if(this.numericOps.options[i].selected) {
                this._lastSelectedNumericOps = this.numericOps.options[i].value;
                break;
            }
        }
        return this._lastSelectedNumericOps;
    } 
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractDigitField,
{
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.textField.regExp = "|0|[1-9][0-9]*";
        this.textField.externalValidation = false;
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormDecimalField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractDigitField,
{
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.textField.regExp = "|0|[1-9][0-9]*|0\.[1-9]|[1-9][0-9]*\.[0-9]*[1-9]+";
        this.textField.externalValidation = false;
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormIPAddressField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField,
{
    postCreate: function() {
        this.inherited("postCreate", arguments);
        
        var ipv4 = "((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))";
        var ipv6 = "(([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){7})";
        var ipv6Abbr0 = "(::([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,6})";
        var ipv6Abbr1 = "(([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,5}))";
        var ipv6Abbr2 = "((([0-9a-fA-F]{1,4}|0)\:)([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,4}))";
        var ipv6Abbr3 = "((([0-9a-fA-F]{1,4}|0)\:){2}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,3}))";
        var ipv6Abbr4 = "((([0-9a-fA-F]{1,4}|0)\:){3}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,2}))";
        var ipv6Abbr5 = "((([0-9a-fA-F]{1,4}|0)\:){4}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,1}))";
        var ipv6Abbr6 = "((([0-9a-fA-F]{1,4}|0)\:){5}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)))";
        var ipv6Abbr7 = "((([0-9a-fA-F]{1,4}|0)\:){6}([0-9a-fA-F]{1,4}|0)\:\:)";
        this.textField.regExp = "|" + ipv4 + "|" + ipv6 +
                                "|" + ipv6Abbr0 + "|" + ipv6Abbr1 + "|" + ipv6Abbr2 + "|" + ipv6Abbr3 +
                                "|" + ipv6Abbr4 + "|" + ipv6Abbr5 + "|" + ipv6Abbr6 + "|" + ipv6Abbr7;
        this.textField.externalValidation = false;
    }
});

// a FormTextField with a FormCheckboxField to override the value in the text field when checked
//
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormCheckboxOverrideTextField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    currentValue: null,
    disabled: false,
    checkboxLabel: "",
    checkboxValue: "",
    checkboxDisplayValue: null,  // optional, use checkboxValue if null
    
    textFieldInvalidFormatMsg: "",
    FormWidgetTemplate: '<table dojoAttachPoint="domNode" width="100%">' +
                        '  <tr>' +
                        '    <td dojoAttachPoint="checkboxNode" style="white-space:nowrap"></td>' +
                        '  </tr>' +
                        '  <tr>' +
                        '    <td dojoAttachPoint="textFieldNode" style="white-space:nowrap"></td>' +
                        '  </tr>' +
                        '</table>',
                        
    constructor: function() {
        this.currentValue = null;
    },
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        
        this.containerField = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(this.FormWidgetTemplate);

        this.checkbox = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormCheckboxField({
            fieldLabel: this.checkboxLabel,
            defaultValue: false,
            onValueChanged: dojo.hitch(this, "_onCheckboxChanged")
        });
        this.containerField.checkboxNode.appendChild(this.checkbox.domNode);
        
        this.textField = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField({
            showFieldLabel: false,
            defaultValue: "",
            onValueChanged: dojo.hitch(this, "_onTextFieldChanged")
        });
        this.containerField.textFieldNode.appendChild(this.textField.domNode);

        var parent = (this.alterHolderNode || this.fieldHolderNode);
        parent.appendChild(this.containerField.domNode);
        
        if (this.defaultValue) {
            this.setValue(this.defaultValue);
        }
        this.formField = this.containerField.domNode;
    },
    
    uninitialize: function() {
        this.checkbox.destroy();
        this.textField.destroy();
    },

    _onCheckboxChanged: function(widget, oldValue, newValue) {
        if (newValue) {
            // clear the text field on check box checked
            this.textField.clearValue();
            this._valueChangedHandler();
        }
    },

    _onTextFieldChanged: function(widget, oldValue, newValue) {
        if (newValue.length > 0) {
            // uncheck the check box when user typed something in the text field
            this.checkbox.clearValue();
            this._valueChangedHandler();
        }
    },

    _valueChangedHandler: function() {
        var oldValue = this.currentValue;
        this.currentValue = this.getValue(); 
        
        this.onValueChanged(this, oldValue, this.currentValue);
    },
    
    setValue: function(value) {
        if (dojo.isString(value) && value.toLowerCase() == this.checkboxValue) {
            this.checkbox.setValue(true);
            this.textField.clearValue();
        } else {
            this.checkbox.setValue(true);
            this.textField.setValue(value);
        }
    },

    getValue: function() {
        if (this.checkbox.getValue()) {
            return this.checkboxValue;
        } else {
            var value = this.textField.getValue();
            if (value.toLowerCase() == this.checkboxValue) {
                value = this.checkboxValue;
            }
            return value;
        }
    },
    
    getValueInString: function() {
        val = this.getValue();
        if (this.checkboxDisplayValue && (val == this.checkboxValue)) {
            val = this.checkboxDisplayValue;
        }
        return val;
    },
    
    clearValue: function() {
        this.checkbox.clearValue();
        this.textField.clearValue();
    },

    _getBaseClassName: function() {
        return "text-field";
    }
});

})();
