/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008,2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// Summary:
//    RichTablePlugin extends dojo's dijit._editor._Plugin to provide table edit enhancements 
//     for rich text editing across the QM application.  
//       
//

RichTablePlugin = dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.RichTablePlugin");
RichTablePlugin.PLUGIN_ID = 'inserttable';

// require the editor plugins to be used...
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._editor._Plugin");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.RichTablePopup");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

( function() {

	var Constants = com.ibm.asq.common.web.client.internal.Constants;

    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.RichTablePlugin", [dijit._editor._Plugin], {
        useDefaultCommand: false,
                
        _initButton: function(){
            var _this = this;
            this.tag =  'table';
            this.command = RichTablePlugin.PLUGIN_ID;
            this.toolTipDialog = new dijit.TooltipDialog({
                title: Constants.getMessage("InsertTable"),
                onOpen: function(){
                    _this._onOpenDialog();
                    dijit.TooltipDialog.prototype.onOpen.apply(this, arguments);
					//remove the 'V'
					dojo.removeClass(this.domNode,"dijitTooltipConnector");
					dojo.removeClass(this.domNode,"dijitTooltipBelow");
					dojo.removeClass(this.domNode,"dijitTooltipAbove");
					dojo.removeClass(this.containerNode,"dijitTooltipFocusNode");
                },
                onCancel: function(){
                    setTimeout(dojo.hitch(_this, "_onCloseDialog"), 0);
                },
                onClose: dojo.hitch(this, "_onCloseDialog")
            });
            
            this.button = new dijit.form.DropDownButton({
                label: Constants.getMessage("InsertTable"),
                showLabel: false,
                dropDown: this.toolTipDialog,
                iconClass: "InsertTableIcon InsertTableIconPos",
                tabIndex: "-1"
            });
        },
        
        setToolbar: function(){
            this.inherited("setToolbar", arguments);
        },
        
        setValue: function(args){
            //TODO: prevent closing popup if the text is empty
            this._onCloseDialog();
            if (dojo.isIE) {
                var a = dojo.withGlobal(this.editor.window, "getAncestorElement", dijit._editor.selection, [this.tag]);
                if (a) {
                    dojo.withGlobal(this.editor.window, "selectElement", dijit._editor.selection, [a]);
                }
            }
            var template = '&nbsp;<table summary="This is a table in Rich Text Editor" border="' + args.border + '" style="width:60%;"><tbody>';
            if (args.row) {
                for (var i = 0; i < args.row; i++) {
                    template += '<tr>';
                    if (args.col) {
                        for (var j = 0; j < args.col; j++) {
                            template += '<td>    </td>';
                        }
                    }
                    template += '</tr>';
                }
            }
            template += '</tbody></table>&nbsp;';
            this.editor.execCommand('inserthtml', template);
        },
        
        _onCloseDialog: function(){
            // FIXME: IE is really messed up here!!
            if (dojo.isIE) {
                if ("_savedSelection" in this) {
                    var b = this._savedSelection;
                    delete this._savedSelection;
                    this.editor.focus();
                    var range = this.editor.document.selection.createRange();
                    range.moveToBookmark(b);
                    range.select();
                }
            }
            else {
                this.editor.focus();
            }
			if (this.dia) {
				this.dia.destroyRecursive();
				this.dia = null;
			}
			this.dia = null;
        },
        
        _onOpenDialog: function(){
            var a = dojo.withGlobal(this.editor.window, "getAncestorElement", dijit._editor.selection, [this.tag]);
            var url, text;
            // FIXME: IE is *really* b0rken
            if (dojo.isIE) {
                this._savedSelection = this.editor.document.selection.createRange().getBookmark();
            }
			var params = {
                parentWidget: this
            }
            this.dia = new com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.RichTablePopup();
            this.dia.initialize({
                parent: this
            });
            this.toolTipDialog._setContentAttr(this.dia.domNode);
			this.toolTipDialog.domNode.style.overflow="auto";
            this.toolTipDialog.startup();
        }
    });
    
})();

dojo.subscribe("dijit.Editor.getPlugin", null, function(o){
    if (o.plugin) {
        return;
    }
    switch (o.args.name) {
        case RichTablePlugin.PLUGIN_ID:
            o.plugin = new com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.RichTablePlugin({
                command: o.args.name
            });
    }
});
