/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    The comment cell renderer provides an input element and an edit image action.
//    In editable mode, user could input short comment directly in the input element, 
//     or in comment pop-up by clicking edit action.  
//    In read-only mode, display the first 30 chars, the whole comment is in tooltip.  
//

dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.CommentCellRenderer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");

(function(){
	var _textContent=dojo.isIE?"innerText":"textContent"; 

    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var Constants = com.ibm.asq.common.web.client.internal.Constants;
    
    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.CommentCellRenderer", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer, {
        // Summary:
        //    Renders the cell content. 
        //    
        renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/ cellModifier, rowObject, columnIndex, cellWidthHint){
            var doc = document;
			var parentNode = doc.createElement("div");
            var node;
            var contentProvider = viewer.getContentProvider();
            this.editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
            this.viewer = viewer;
            this.rowObject = rowObject;
            this.columnIndex = columnIndex;
            if (this.editable) {
                cellData = this.restoreTags(cellData);
                var inputLabel = doc.createElement("label");
                node = doc.createElement("textarea");
                node.rows = 1;
                //node.cols = 40;
                node.value = cellData;
                if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo) && extendedDisplayInfo['textWidth']) 
                    node.style.width = extendedDisplayInfo['textWidth'];
                
                var f = dojo.hitch({
                	'cellModifier': cellModifier,
	            	'rowObject': rowObject,
            		'columnIndex':columnIndex,
            		'oldValue': cellData
            	},this.handleEdit);            
                
                this.connect(node, "onkeypress", f);
				this.connect(node, "onchange", f);
                
                inputLabel.appendChild(node);
                parentNode.appendChild(inputLabel);
                
                
                //create edit action
                var img = doc.createElement("img");
                img.alt = "Edit";
                img.src = ImageCache.EDIT_FIELD_ICON;
                var actionLink = doc.createElement("a");
                actionLink.appendChild(img);
                actionLink.inputNode = node;
	    	    // use the viewer's connect as it will manage the lifetime of this connection.
    	    	// use hitch to provide this cell renderer to the function.
                this.connect(actionLink, "onclick", function(e){
                    var node = e.currentTarget;
                    if(node) {
                        var moreComment = new CommentCellRenderer.CommentDialog({ 'node': node.inputNode, 'parent': this });
                    }
                });
				var tooltip = Constants.getMessage("labelCommentsImageTooltip");
				img.title = tooltip;
				
                parentNode.appendChild(actionLink);
            }
            else {
                var shortComment = this.restoreTags(cellData);
                node = doc.createElement("div");
                if (viewer.isCompactMode && typeof cellWidthHint != "undefined") {
                    // clear the tooltip text here. It's the same as the cell data. Otherwise,
                    // the compact cell handling would concat both the cell data & the tooltip
                    // text and combine them into one tt message.
					var tooltip = "";
					if(extendedDisplayInfo['tooltip']){
						tooltip = extendedDisplayInfo['tooltip'];
					}
                    extendedDisplayInfo['tooltip'] = null;
                    shortComment = this._handleCompactCellText(viewer, shortComment, extendedDisplayInfo, node, cellWidthHint);
					// FF2 clips the tooltip so user has no way of seeing whole content so dont loose the tooltip 
					//and use attachTooltipToCell. Also because it can handle tooltip containing tags -- WI 10946
					if(tooltip != ""){
						this._attachTooltipToCell(node, tooltip);
						node.title = "";
					}
                }
                else if (shortComment && shortComment.length > 30) {
                    shortComment = shortComment.substr(0, 30) + "...";
                    if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) 
                        this._attachTooltipToCell(parentNode, extendedDisplayInfo['tooltip']);
                }    
                node[_textContent]=shortComment ? shortComment : " ";
                parentNode.appendChild(node);
            }
            return parentNode;
        },
        
        handleEdit: function(e){
            var node = e.target;
            if(node) {
                var oldValue = this.oldValue;
                this.oldValue = node.value;
                
                if (this.cellModifier) 
                    this.cellModifier.cellValueChanged(oldValue, this.oldValue, this.rowObject, this.columnIndex);
             }
        },
        
        restoreTags: function(value){
			if (value) {
				var rawText = value;
				rawText = rawText.replace(/\&gt;/gi, ">");
				rawText = rawText.replace(/\&lt;/gi, "<");
				rawText = rawText.replace(/\&amp;/gi, "&");
				rawText = rawText.replace(/\&tilde;/gi, "~");
				rawText = rawText.replace(/\&quot;/gi, "\"");
				rawText = rawText.replace(/\&circ;/gi, "^");
				rawText = rawText.replace(/\&nbsp;/gi, " ");
				rawText = rawText.replace(/<br\/>/gi, "\n");
				rawText = rawText.replace(/\&#09;/g, "\t");
				return rawText;
			}
			return "";
        }
    });
    
    dojo.declare("CommentCellRenderer.CommentDialog", [dijit._Widget, dijit._Templated], {
        templateString: '<div style="font-size: 10pt; padding-left: 0.5em; overflow: hidden" dojoAttachPoint="domNode">' +
        '  <div style="padding: 0.5em;" dojoAttachPoint="textArea">' +
        '   <label><textarea style="font-size: 10pt; width: 375px;" dojoattachpoint="_commentArea" rows="3" cols="50" maxlength="255"></textarea></label> ' +
        '  </div>' +
        '</div>',
        
        
        labelInputCommentHere: Constants.getMessage("labelInputCommentHere"),
        actionLabel_cancel: Constants.getMessage("actionLabel_cancel"),
        actionLabel_ok: Constants.getMessage("actionLabel_ok"),
        
        
        constructor: function(args){
            this._dialog = null;
            this.node = args.node;
            this.parent = args.parent;
        },
        
        postCreate: function(){
            this._initialize();
        },
        
        _initialize: function(){
            this._commentArea.value = this.node.value;
            var self = this;
            if (this._dialog == null) {
                var args = {
                    isClosable: true,
                    contentNode: this.domNode,
					primaryTitle: Constants.getMessage("labelComments"),
                	okCallback: dojo.hitch(this, "onSave"),
                	cancelCallback: dojo.hitch(this, "onCancel"),
					_truncate: function() {} 
                }
                this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(args);
				this._dialog.setWidth("412px");	
				this._dialog.domNode.style.minWidth = "412px";
				this._dialog.getOkButton().disabled = false;
            }
            this._dialog.open();
        },
        
        onCancel: function(){
        },
        
        onSave: function(){
            var newValue = this._commentArea.value;
            this.node.value = newValue;
            
            // notify the cell renderer (parent) of the value 
            // changed.
            this.parent.handleEdit({'target': this.node});
        }
    
    });
})();
