/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DateCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility")
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");


(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DateCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
		return this._renderReadOnly(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint);
    },

	/**
	 * Render the Date cell in a 'readonly' mode
	 */	
	_renderReadOnly: function(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint) {
        var str = cellData;
        
        if (cellData instanceof Date) {
            str = com.ibm.asq.common.web.ui.internal.util.Utility.formatDate(cellData);
        }
        var node = document.createElement("div");
        this._renderCellText(viewer, str, extendedDisplayInfo, node, cellWidthHint);
        return node;		
	}
    
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.EditableDateCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DateCellRenderer,
{

	// Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var provider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (provider ? provider.getIsCellEditable(rowObject, columnIndex) : false);
		if (!editable) {
			return this._renderReadOnly(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint);
		} else {
			return this._renderEditable(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint);
		}
    },
	
	/**
	 * Render the Date cell in a 'editable' mode
	 */	
	_renderEditable: function(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint) {
        // display the date in the preferred format.            
        var args = {
            isDate: true,
            containerToggle: "fade",
            constraints: {
                formatLength: com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatLength()
            },
			popupCalendarOnFocus: true
        };
        var datePicker = new com.ibm.asq.common.web.ui.internal.widgets.DatePicker(args);
		datePicker.domNode.firstChild.firstChild.style.display = "none";
        if (cellData) {
            var dateValue = com.ibm.asq.common.web.ui.internal.util.Utility.dateFromISOString(cellData);
            datePicker.setValue(dateValue);
        }

        // restrict the size of the box to the width hint.
        if (cellWidthHint) {
			datePicker.domNode.style.width = cellWidthHint + "px";
		}
        datePicker.onChange = function(){
			//invoke the cellmodifier callback
            if (cellModifier) {
                // normalize the value to an empty string. Our client expects empty string
                // instead of null.
                var val = this.getValue() || "";
				cellModifier.cellValueChanged(null, val, rowObject, columnIndex);
			}
        }
		datePicker.onblur = function(evt) {
			//by input from keyboard
			var val = (this.textbox != null) ? this.textbox.value : (this.focusNode != null) ? this.focusNode.value : "";
			if (this.cellModifier) {
				this.cellModifier.cellValueChanged(null, val, rowObject, columnIndex);
			}
		}		
        return datePicker.domNode;
	}
});

})();
