/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    The dropdown menu cell renderer creates an icon button (a down arrow icon) that
//    when clicked will popup a menu. The menu entries are specified via the extended display
//    info data's "actions" field. Please sse the table content provider for additional 
//    details. The user can invoke the menu item by clicking on it. A cellModifier message
//    will generated by this user action. If the table viewer is set to non-editable mode,
//    the dropdown menu is still created, however, the menu entries will all become disabled.
//  
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DropdownMenuCellRenderer");

dojo.require("dijit.form.Button");
dojo.require("dijit.Menu");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("DropdownMenuCellRenderer.DropdownButton",
             dijit.form.DropDownButton,
{

    templateString:'<div class="com-ibm-asq-common-tableViewer-dropdown-cell" dojoAttachPoint="domNode" dojoAttachEvent="onclick:_onArrowClick,onkeypress:_onKey,onmouseover:_onMouse,onmouseout:_onMouse,onmousedown:_onMouse">' +
                   '  <button class="button-container" dojoAttachPoint="focusNode,titleNode" tabIndex="${tabIndex}" id="${id}" name="${name}" waiRole="button" waiState="haspopup-true,labelledby-${id}_label">' +
                   '    <div nowrap class="label-text" dojoAttachPoint="containerNode,popupStateNode" id=\"${id}_label\">${label}</div>' +
                   '    <div class="down-arrow-icon">&nbsp;</div>' +
                   '  </button>' +
                   '</div>',

    cellModifier: null,
    rowObject: null,
    colIndex: -1,
    
    postCreate: function() 
    {
        this.inherited('postCreate', arguments);
        this.dropDown = new dijit.Menu();
    },
    
    addAction: function(label, actionId, /* optional */ iconClass, editable) {
        var arg = {
            'label': label,
            'disabled': !editable,
            'onClick': function() {
                if(this.parent.cellModifier) {
                    this.parent.cellModifier.cellMenuActionInvoked(this.actionId, this.parent.rowObject, this.parent.colIndex);
                }
            }
        };
        
        if (iconClass) {
            arg['iconClass'] = iconClass;
        }
        var item = new dijit.MenuItem(arg);
        item.actionId = actionId;
        item.parent = this;
        
        this.dropDown.addChild(item);
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DropdownMenuCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex)
    {
        var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        var node;
        
        var dropDownButton = new DropdownMenuCellRenderer.DropdownButton(
                {'cellModifier': cellModifier,
                 'rowObject': rowObject,
                 'colIndex' : columnIndex});

        node = dropDownButton.domNode;
        
        if (extendedDisplayInfo && extendedDisplayInfo['actions']) {
            var actions = extendedDisplayInfo['actions'];
            for(var i = 0; i < actions.length; i++) {
                dropDownButton.addAction(actions[i].label, actions[i].id, actions[i].iconClass, editable);
            }
        }
        return node;
    }
});

})();
