/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.EditableTextCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {
var _textContent=dojo.isIE?"innerText":"textContent"; 

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.EditableTextCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{
    // Indicate what types of text input. Subclass can override the text input 
    // type of the text edit box. Default to plain text input. Other supported 
    // type is 'password'.
    inputType: "text",
        
    // Summary:
    //    Renders the cell content. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node;
        var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        var d = dojo;
        var doc = document;
        
        if (editable) {
			var span = doc.createElement("span");
			node = doc.createElement("input");
            node.type = this.inputType;
			node.value = cellData;
            var widthStyled = false;
            
            if(extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
                // Ignore sizing information if the cell size hint is provided.
                // the cell's size hint takes precedance.
    			if (!cellWidthHint && extendedDisplayInfo['textWidth']) {
                    node.style.width = extendedDisplayInfo['textWidth'];
                    widthStyled = true;
                }    
    			if (!cellWidthHint && extendedDisplayInfo['size']) {
                    node.size = extendedDisplayInfo['size'];
                    widthStyled = true;
                }
                if(extendedDisplayInfo['class'])
                    d.addClass(node, extendedDisplayInfo['class']);
            }
            // by default, if not IE always add style to stretch the field
            // to the table cell width. 
            if(!widthStyled && !d.isIE) {
                d.addClass(node, "editText-cell");
            }
            d.addClass(node, "editText-cell-border");
            // restrict the size of the box to the width hint.
            if(cellWidthHint)
                node.style.width = (cellWidthHint - 4) + "px";
                
                    
            this.connect(node, "onkeyup", dojo.hitch(
            	{	'cellModifier': cellModifier,
	            	'rowObject': rowObject,
            		'columnIndex':columnIndex,
            		'oldValue': cellData
            	},this._handleValueChanged));            
			
			var cellLabel = this.getCellLabel(viewer, rowObject, columnIndex);
			if(cellLabel){
				var label = document.createElement("label");
				label[_textContent]=cellLabel;
				label.style.display = "none";
				span.appendChild(label);
			}
			
			span.appendChild(node);
			if(extendedDisplayInfo && d.isObject(extendedDisplayInfo))
            	this._attachTooltipToCell(node, extendedDisplayInfo['tooltip']);
			return span;
        } 
        else {
            node = doc.createElement("div");
            this._renderCellText(viewer, cellData, extendedDisplayInfo, node, cellWidthHint);
			return node;
        }
    },
    
    _handleValueChanged: function(e)
    {
        var node = e.target;
        if (node) {
            var oldValue = this.oldValue;
            this.oldValue = node.value;
            
            var cellModifier = this.cellModifier;
            if (oldValue != this.oldValue && cellModifier) 
                cellModifier.cellValueChanged(oldValue, this.oldValue, this.rowObject, this.columnIndex);
        }
    } 
});

})();
