/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GridTableHeaderRenderer");

dojo.require("dojo.fx");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DynamicFilterControl"); 
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");

(function() {
var _textContent=dojo.isIE?"innerText":"textContent"; 

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var TOP_BORDER_CSS = "table-header-top-border-line";
var BOTTOM_BORDER_CSS = "table-header-bot-border-line";
var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;
var AttributeBuilder = com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder;

var AscendSort_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortAscend");
var DescendSort_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortDescend");
var NoSort_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_nosort");
var OpenFilterCtrl_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_openFilterHeader");
var CloseFilterCtrl_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_closeFilterHeader");
var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
var DynamicFilterControl = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DynamicFilterControl;
var WidthSlack = (dojo.isIE) ? 16 : 12;

// Simple widget to display the sort direction and for multi-column sorting
// the n-th position of the sort.
dojo.declare("GridTableHeaderRenderer.SortIndicator",
             [dijit._Widget, dijit._Templated],
{
    templateString: '<span class="sort-indicator" dojoAttachPoint="domNode">' +
                    '  <span class="sort-indicator-text" dojoAttachPoint="textNode">' +
                    '  </span>' +
                    '</span>',
    
    columnPos: null,
    
    setSortAttributes: function(sortDir, nSort)
    {
        // clear all of the sort indications first. sort dir & nth sort
        // could be null.
        var imgClass = null;
        this.domNode.className = "sort-indicator";
        
        if(sortDir == TableConstants.SORT_DIR_ASCEND) {
            imgClass = 'sort-ascend-image';
        }
        else if(sortDir == TableConstants.SORT_DIR_DESCEND){
            imgClass = 'sort-descend-image';
        }
        if(imgClass)
            dojo.addClass(this.domNode, imgClass);
            
        this.textNode.innerHTML = nSort ? (nSort + "") : "";
    }  
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GridTableHeaderRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer,
{
    tableHeaders: null,
	
    // Summary:
    //    Reset any states that the header may have cached. This is usually called when
    //    the table viewer's content has been changed and any old state may not be applicable.
    //
    resetStates: function()
    {
        this.checkAllState = false;
        this.setColumnActionEnablement();
    },
    
    createTableControls: function(pageRangeControlNode, pageNode, footerPageNode, filterControlNode, groupControlNode, viewModeControlNode)
    {   
		if(this.viewer.getIsShowingPagingRangeControl()) {
			this._createPageRangeControl(pageRangeControlNode);
		}
		
        if(this.viewer.getIsShowingPagingControl()) {
            this._createPageControls(pageNode);
            this._createPageControls(footerPageNode);
        }

        this._constructGroupControl(groupControlNode);
        
        var sep = document.createElement("span");
        sep.className = "controls-spacing";

        if(this.viewer.getIsShowingFilterControl()) {
            dojo.style(filterControlNode, "display", "");
            this._constructFilterControls(filterControlNode);            
        }   
        else {
            dojo.style(filterControlNode, "display", "none");
        }

        if(this.viewer.getIsShowingViewModeControl()) {
            dojo.style(viewModeControlNode, "display", "");
            this._constructViewModeControl(viewModeControlNode);            
        }   
        else {
            dojo.style(viewModeControlNode, "display", "none");
        }
    },

    createHeaderBorderLine: function(table, borderRow, isTop)
    {
        var line = document.createElement("th");
        var borderCss = (isTop) ? TOP_BORDER_CSS : BOTTOM_BORDER_CSS;

        line.innerHTML = "&nbsp;"
        dojo.addClass(line, borderCss);
        borderRow.appendChild(line);
    },
    
    destroyTableHeader: function()
    {
        // before we destroy the header, check to see if there are
        // filters set. If so, keep them and restore them when the 
        // header gets recreated again. The table can rebuild the header
        // number of times. We need to keep the current state of the
        // filters for the table.
        this.getSelectedFilters();
        
    	this.inherited(arguments);
        
        if (this.thead) {
            Utility.destroyWidgetsFromNode(this.thead);
        }
        if (this.tableHeaders) {
            for (var i in this.tableHeaders) {
                var th = this.tableHeaders[i].thead;
                var imgs = dojo.query("img", th);
                
                for (var j = 0; j < imgs.length; j++) {
                    imgs[j].src = null;
                }
            }
        }
        this.headerDisconnectAll();
        this.sortColCtrls = null;
        this.columnActions = null;
        this.tableHeaders = null;
        this.thead = null;
    },
    
    createTableHeader: function(table)
    {
        // not show table heading. 
        if(!this.viewer.getIsShowingTableHeading())
            return;
                    
        var provider = this.viewer.getContentProvider();
        var content = this.viewer.getContent();
        var nColumns = provider.getNumColumns(content);
        var thead = table.createTHead();
        var headerTopTr, headerBotTr;
        var headerContentTr;
		var headerFilterTr = null;
        var headerActionTr;                          
        var groupBy = this.viewer.getGroupByColumn();
        var atCol = 0;
        var slackWidth = 0;
        var nVisible = nColumns;
        var reduceWidth = 0;
        var tableHeaderStyle = "table-header-content";
		var doc = document;
        var totalWidth = 0;                
        var widthLeft = 0;
        var showFiltersHeader = this.viewer.showColumnFilters && this.viewer.filterAttrProvider;
        var _dojo = dojo;
        // set table header style for CJK pictographic based locales
        if (_dojo.locale.indexOf("zh") == 0 ||
            _dojo.locale.indexOf("ja") == 0 ||
            _dojo.locale.indexOf("ko") == 0) {
            tableHeaderStyle = "table-header-content-cjk";
        }

        // reset the header data
        this.columnActions = [];
        this.tableHeaders = {};
        this.filterCtrls = null;
        this.moreFiltersCtrl = null;
        
        // multi column sorting data.
        this.sortColCtrls = {};
        this.sortColumns = [];
        
        this.sortByColumn = this.viewer.getSortByColumn();
        this.sortDir = this.viewer.getSortDirection();

        // multi-column sorting. Get the array of sort columns.
        if (this.viewer.allowMultiSort) {
            this.sortColumns = this.viewer.getMultiColumSortInfo();
        }
                
        for (var i = 0; i < nColumns; i++) {
            if (!this.viewer.isColumnVisible(i) || (groupBy == i)) {
                nVisible--;
            }
            else {
                totalWidth += this.viewer.getColumnWidth(i);
            }
        }        
        if(nVisible == 0)
            return;
        
        widthLeft = 100 - totalWidth;   
        
        // we're going to remove the group column,
        // distribute its width amongst all columns
        if (groupBy > -1 && this.viewer.isColumnVisible(groupBy)) {
            slackWidth = Math.floor(this.viewer.getColumnWidth(groupBy) / nVisible);
        }    
        
		// create the header rows, top & bottom border rows
		// and content & sort rows in the middle.
        headerTopTr = table.insertRow(table.rows.length); 
        headerContentTr = table.insertRow(table.rows.length); 
        
        // add filter controls' row
        if(showFiltersHeader) {
            headerFilterTr = table.insertRow(table.rows.length);
            headerActionTr = table.insertRow(table.rows.length);
            _dojo.addClass(headerFilterTr, "table-filter-header-row");
            _dojo.addClass(headerActionTr, "table-filter-header-row");
            this.filterHeaders = {};
        }
        
        headerBotTr = table.insertRow(table.rows.length); 
        
        // The rows in the table are checkable, insert
        // a column to provide a check box for the rows
        if(this.viewer.getTableRowsCheckable()) {
            this.createHeaderBorderLine(table, headerTopTr, true);
            
            var checkbox_th = doc.createElement("th");
            _dojo.addClass(checkbox_th, tableHeaderStyle);
            
            // create a check box to select/deselect all rows in the table.
            if (!this.viewer.getIsSingleSelectionMode() && this.viewer.getIsAllowSelectAll()) {
                _dojo.addClass(checkbox_th, "align-center");
                checkbox_th.vAlign = "center";
                checkbox_th.title = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_selectAll");
                
                var checkState = (this.checkAllState) ? true : false;
                var button = new dijit.form.CheckBox({
                    'checked': checkState
                });
                button.headerRenderer = this;
                button.onChange = function() {
                    this.headerRenderer.checkAllState = this.checked;
                    this.headerRenderer.viewer.performSelectAll(this.checked);
                };
                checkbox_th.appendChild(button.domNode);
            }
            else {
                checkbox_th.width = 20;
            }                
            headerContentTr.appendChild(checkbox_th);
            
			if(headerFilterTr) {
				checkbox_th = doc.createElement("th");
				_dojo.addClass(checkbox_th, TOP_BORDER_CSS);
				headerFilterTr.appendChild(checkbox_th);
			}
            this.createHeaderBorderLine(table, headerBotTr);
        }
        
        // if there's a group by, insert a filler column for the
        // indentation.
        if(groupBy > -1) {            
            this.createHeaderBorderLine(table, headerTopTr, true);
            var gr_th = doc.createElement("th");
            _dojo.addClass(gr_th, tableHeaderStyle);
            headerContentTr.appendChild(gr_th);
            gr_th.width = 8;            

            if(headerFilterTr) {
				gr_th = doc.createElement("th");
                _dojo.addClass(gr_th, TOP_BORDER_CSS);
                headerFilterTr.appendChild(gr_th);
            }
            this.createHeaderBorderLine(table, headerBotTr);			
        }
        
        for(var i = 0; i < nColumns; i++) {
            // skip displaying the group by column and non-visible columns.
            if(groupBy == i || !this.viewer.isColumnVisible(i)) {
                continue;
            }
            this.createHeaderBorderLine(table, headerTopTr, true);
            
            var title = provider.getColumnHeading(content, i);
            var th = doc.createElement("th");
             
            _dojo.addClass(th, tableHeaderStyle);    
            var currentWidth = this.viewer.getColumnWidth(i);        
            var addedPercentage=0;
            if(totalWidth < 100 && totalWidth != 0)
                addedPercentage=Math.floor((currentWidth/totalWidth)*widthLeft);
            var width = (currentWidth + slackWidth - reduceWidth + addedPercentage);
			if(width <= 0)
				width = 1;//IE doesnt like 0 width                    
            th.width = width + "%";	
	        th.vAlign = "center";
            var align = provider.getColumnHeadingAlignment(i);
           
            if(align == TableConstants.ALIGN_CENTER) {
               _dojo.addClass(th, "align-center");
            } 
            else if(align == TableConstants.ALIGN_RIGHT) {
               _dojo.addClass(th, "align-right");   
            } 
            else {
                _dojo.addClass(th, "align-left")
            }
				            
            if (provider.getIsColumnSortable(content, i)) {
                var anchor = doc.createElement("a");
                var titleText = (title && title.length > 0) ? title : "\u00a0";
                var colIsSort = this.columnHasSort(i);
                
                anchor[_textContent]=titleText;
                anchor.colIndex = i;
                anchor.name = "anchor";
                anchor.className = "table-link";

                th.appendChild(anchor);
                this.headerConnect(anchor, "onclick", "_handleTableHeadSortBy");
                anchor.title = NoSort_Text;            

                // Create the sort indicator for the column
                this._createSortControls(th, i);
                
                // this is the current sort column provide the right
                // tool tip.
                if (colIsSort) {
                    this.setColumnTitleSortTooltip(i, anchor);
                }
            } 
            else if (title != null && _dojo.trim(title).length > 0){
                th[_textContent]=title;
            }
            else {
                var spacer = doc.createElement('span');
                spacer.style.width = '8px';
                th.appendChild(spacer);
            }
                        
            // if the column header has a image, create it.
            var imgRef = provider.getColumnHeadingImage(content, i);
            if(imgRef) {
                var src = (_dojo.isObject(imgRef) && imgRef.src) ? imgRef.src : imgRef;
                var isAction = (_dojo.isObject(imgRef) && imgRef.actionId) ? true : false;
                
                // create a margin node if we already have
                // something created before the image.
                if(th.childNodes && th.childNodes.length > 0) {
                    var spacer = doc.createElement("span");
                    spacer.className = "table-head-spacer";
                    th.appendChild(spacer);                    
                }
                
                var img = doc.createElement("img");
                img.className = "table-head-image";
                img.src = src;
                th.appendChild(img);
                
                if (isAction) {
                    this.columnActions.push(img);

                    img.colIndex = i;
                    img.name = imgRef.actionId;

                    _dojo.addClass(img, "table-head-image-action");
                    this.headerConnect(img, "onclick", "_handleColumnActionClicked");
                }
				
                if(imgRef.tooltip) {
                    img.title = imgRef.tooltip;
                }				
            }

            headerContentTr.appendChild(th);
			
            // We have column filters, create the th holder for the filter.
            if(headerFilterTr) {
                var fth = doc.createElement("th");
                _dojo.addClass(fth, TOP_BORDER_CSS);
                
                var colPos = i + "";
                this.filterHeaders[colPos] = fth;
                headerFilterTr.appendChild(fth);
            }   
            this.createHeaderBorderLine(table, headerBotTr);
            
            // store the column header in our associated array indexed
            // by the column position string key.
            this.tableHeaders[this._getTableHeadKey(i)] = {'thead': th};
        }

        // create a drop down column to open/collapse the filter header
        if(headerFilterTr) {
            this.createHeaderBorderLine(table, headerTopTr, true);
            this._createFilterHeaderExpandControls(table, headerContentTr, headerFilterTr);
            this.createHeaderBorderLine(table, headerBotTr);
        }
		
        thead.appendChild(headerTopTr);
        thead.appendChild(headerContentTr);  
          
        if (headerFilterTr) {
            thead.appendChild(headerFilterTr);
            this._createFilterActionsHeader(headerFilterTr, headerActionTr);  
            thead.appendChild(headerActionTr);
            
            this.multiFilterHeader = headerFilterTr;
            this.multiFilterActionHeader = headerActionTr;
            var headerState = this.viewer.filterAttrProvider.getIsSectionOpenInitially() || this.filterHeaderOpenState;
            
            this._toggleFilterHeader(headerState);
        }
        thead.appendChild(headerBotTr);    
        this.thead = thead;
          
        table.appendChild(thead);    
        this.viewer.contentRowStart = table.rows.length;
        
        // initialize the column's action states.
        this.setColumnActionEnablement();    
        
        // initialize the sort indicators.
        this._initSortIndicators(groupBy);
    },

    _createSortControls: function(th, pos)
	{
        var sortCtrl = new GridTableHeaderRenderer.SortIndicator({
            columnPos: pos
        });
        this.sortColCtrls['col' + pos] = sortCtrl;
		th.appendChild(sortCtrl.domNode);
	},
	
    setColumnActionEnablement: function()
    {
        if (!this.columnActions) {
            return;
        }
        var disable = this.viewer.getContentItemCount() == 0 || this.viewer.getCheckedRows().length == 0;
        for(var i = 0; i < this.columnActions.length; i++) {
            var action = this.columnActions[i];
            action.disabled = disable;
            if(disable)
                dojo.addClass(action, "table-head-image-action-disable");
            else
                dojo.removeClass(action, "table-head-image-action-disable");
        }          
    },
    
    getColumnHeaderWidthInPixel: function(columnIndex)
    {
        var header = this.tableHeaders[this._getTableHeadKey(columnIndex)];
        if(!header.width) {
			var w = dojo.marginBox(header.thead).w;
			// only cache the width if we actually has a width.
            // reduce the column width to ensure that the column's 
            // desired width would not be excceeded
			if(w > 0)
                header['width'] = (w <= WidthSlack) ? w : (w - WidthSlack);
	        else 
			    return w;
        }
        return header.width;
    },
    
    getColumnHeaderAt: function(columnIndex)
    {
        return this.tableHeaders[this._getTableHeadKey(columnIndex)].thead;
    },
    
    _handleColumnActionClicked: function(event)
    {
        event = event || window.event;
        var column = event.currentTarget.colIndex;
        var actionId = event.currentTarget.name;
        
        if(event.currentTarget.disabled)
            return;
            
        var listener = this.viewer.getTableListener();
        if(listener) {
            listener.tableHeaderColumnActionInvoked(this.viewer, column, actionId);
        }
    },
    
    // Summary:
    //    Based on the sort direction of the provided column position, returns the
    //    the next sort direction as follows:
    //       none -> ascending
    //       ascending -> descending
    //       descending -> none.
    //
    _getNextSortDirByColumn: function(colPos)
    {
        var curDir = null;

        if (this.viewer.allowMultiSort) {
            for(var i = 0; i < this.sortColumns.length; i++) {
                if(colPos == this.sortColumns[i].col) {
                    curDir = this.sortColumns[i].dir;
                }
            }
        }
        else if(colPos == this.sortByColumn) {
            curDir = this.sortDir;
        }
        
        var newDir = null;
        if(curDir == null) {
            newDir = TableConstants.SORT_DIR_ASCEND;
        }
        else if(curDir == TableConstants.SORT_DIR_ASCEND) {
            newDir = TableConstants.SORT_DIR_DESCEND;
        }
        else if(curDir == TableConstants.SORT_DIR_DESCEND) {
            newDir = null;
        }
        return newDir;
    },
    
    // Summary:
    //    Handles a column's sort control being cleared for sort. Update
    //    the sort indicator & the header title's sort tooltip and
    //    perform the sort.
    _handleClearSortCol: function(sortCtrl)
    {
        var colPos = sortCtrl.columnPos;
        
        // clear this column's sort indication and the main header's
        // title's sort tooltip.  
        sortCtrl.setSortAttributes(null, null);
        
        if (this.viewer.allowMultiSort) {
            // remove this column from sort participations.
            for (var i = 0; i < this.sortColumns.length; i++) {
                if (colPos == this.sortColumns[i].col) {
                    break;
                }
            }
            this.sortColumns.splice(i, 1);
            
            var th, anchor;
            
            // move column's sort position back up one.
            for (var j = i; j < this.sortColumns.length; j++) {
                this._updateSortIndicators(this.sortColumns[j].col, this.sortColumns[j].dir, j + 1);
                
                // update the column title's sort tooltip
                th = this.getColumnHeaderAt(this.sortColumns[j].col);
                anchor = this._getHeaderTitleLink(th);
                if (anchor) 
                    this.setColumnTitleSortTooltip(this.sortColumns[j].col, anchor);
            }
        }
        
        // clear the sort tooltip in the column being cleared.
        th = this.getColumnHeaderAt(colPos);
        anchor = this._getHeaderTitleLink(th);
        if (anchor) 
            this.setColumnTitleSortTooltip(colPos, anchor);
    },
    
    // Summary:
    //    Handler for a sortable column's header click. Updates the sort direction
    //    and order on the click and performs the sorting.
    //
    _handleTableHeadSortBy: function(event)
    {
        event = event || window.event;
        dojo.stopEvent(event);
        
        var lastSortCol = this.sortByColumn;
        var sortColIndex = event.currentTarget.colIndex;
        var sortDir = this._getNextSortDirByColumn(sortColIndex);
        var multiSort = this.viewer.allowMultiSort;
       
        this.sortByColumn = sortColIndex;
        this.sortDir = sortDir;
        
        // not in multi-column sort and the user had selected a different 
        // column to sort. Clear the last column's sort indicator.
        if(!multiSort && lastSortCol > -1 && lastSortCol != sortColIndex) {
            var ctrl = this.sortColCtrls['col' + lastSortCol];
            this._handleClearSortCol(ctrl);            
        }

        // apply the new sort direction.                 
        if (sortDir == null) {
            var ctrl = this.sortColCtrls['col' + sortColIndex];
            this._handleClearSortCol(ctrl);
            
            // single column sort, clear the current sort column.
            if(!multiSort) {
                this.sortByColumn = -1;
            }
        }
        else {
            this._updateSortColumn(sortColIndex, sortDir);
        }
        
        // update the sort tooltip on the header 
        var th = this.getColumnHeaderAt(sortColIndex);
        var anchor = this._getHeaderTitleLink(th);
        if (anchor) {
            this.setColumnTitleSortTooltip(sortColIndex, anchor);
        }
        
        // apply the sort
        if (multiSort) {
            this.viewer.performMultiColumnSort(this.sortColumns);
        }
        else {
            this.viewer.performSortBy(this.sortByColumn, this.sortDir);
        }
    },
    
    _getHeaderTitleLink: function(th)
    {
        var anchor = null;
        if(th.childNodes.length) {
            for(var i = 0; i < th.childNodes.length; i++) {
                if(th.childNodes[i].name == "anchor") {
                    anchor = th.childNodes[i];
                    break;
                }
            }
        }
        return anchor;        
    },
    
    columnHasSort: function(colPos)
    {
        if (this.viewer.allowMultiSort) {
            for(var i = 0; i < this.sortColumns.length; i++) {
                if(colPos == this.sortColumns[i].col) {
                    return true;
                }
            }
        }
        else {
            return colPos == this.sortByColumn;
        }
        return false;
    },
    
    setColumnTitleSortTooltip: function(colPos, anchor)
    {
        var sortDir = null;
        
        if (this.viewer.allowMultiSort) {
            for(var i = 0; i < this.sortColumns.length; i++) {
                if(colPos == this.sortColumns[i].col) {
                    sortDir = this.sortColumns[i].dir;
                    break;
                }
            }
        }
        else if(colPos == this.sortByColumn) {
            sortDir = this.sortDir;
        }
        anchor.title = (sortDir) ? ((sortDir == TableConstants.SORT_DIR_ASCEND) ? AscendSort_Text : DescendSort_Text) : NoSort_Text;
    },
    
    _initSortIndicators: function(groupBy)
    {
        if (this.viewer.allowMultiSort) {
            for (var i = 0; i < this.sortColumns.length; i++) {
                var sortInfo = this.sortColumns[i];
                
                // skip the group by column.
                if (groupBy > -1 && groupBy == sortInfo.col) 
                    continue;
                
                this._updateSortIndicators(sortInfo.col, sortInfo.dir, i + 1);
            }
        }
        else if(this.sortByColumn > -1 && this.sortByColumn != groupBy){
            this._updateSortIndicators(this.sortByColumn, this.sortDir, null);
        }
    },
    
    _updateSortColumn: function(colPos, sortDir) 
    {
        var sortPos = null;
        
        if (this.viewer.allowMultiSort) {
            var sortCol = null;
            for (var i = 0; i < this.sortColumns.length; i++) {
                if (colPos == this.sortColumns[i].col) {
                    sortCol = this.sortColumns[i];
                    break;
                }
            }
            if (!sortCol) {
                sortCol = {
                    col: colPos,
                    dir: sortDir
                };
                this.sortColumns.push(sortCol);
            }
            sortCol.dir = sortDir;
            sortPos = i + 1;
        }
        this._updateSortIndicators(colPos, sortDir, sortPos);
    },
    
    _updateSortIndicators: function(pos, sortDir, nSort)
    {
        var ctrl = this.sortColCtrls['col' + pos];
        if(ctrl)
            ctrl.setSortAttributes(sortDir, nSort);
    },
    
    _getTableHeadKey: function(atCol)  
    {
        return "col-" + atCol;
    },

    // Summary:
    //    Internal method to connect header node's event to an internal function. 
    //
    headerConnect: function(node, event, func)
    {
        if(!this.headConnect) {
            this.headConnect = [];
        }
        var cnt = dojo.connect(node, event, this, func);  
        this.headConnect.push(cnt);  
    },
    
    headerDisconnectAll: function()
    {
        if(!this.headConnect) return;
        
        var len = this.headConnect.length;
        for(var i = 0; i < len; i++) {
            dojo.disconnect(this.headConnect[i]);
        }
        delete this.headConnect;    
    },
   
    // Summary:
    //   Clears all filter's input values and reset them to empty.
    clearFilters: function()
    {
        if (this.filterCtrls) {
            if (this.moreFiltersCtrl) {
                this.moreFiltersCtrl.clearFilters();
            }
            for (var name in this.filterCtrls) {
                var field = this.filterCtrls[name];
                field.clearValue();
            }
            this._savedFilters = null;
        }
    },
   
    // Summary:
    //   Returns the filters' values. This method collects all filter input 
    //   values and returns them in an associate array indexed by the filter's
    //   name.
    getSelectedFilters: function()
    {
        if (!this.filterCtrls) return null;
        
        var values = {};
        
        for(var name in this.filterCtrls) {
            var field = this.filterCtrls[name];
            values[name] = field.getValue();
        }
        // attach any more filter values.
        if(this.moreFiltersCtrl) {
            var mValues = this.moreFiltersCtrl.getSelectedFilters();
            for(var name in mValues) {
                values[name] = mValues[name];
            }
        }
        // retain the last set of filter values. If the header
        // get destroy when the content is being updated, etc.
        // we can restore the selected filter values. This method
        // would be called before the header is destroyed.
        this._savedFilters = values;
        return values;
    },
    
    _createFilterActionsHeader: function(headerFilterTr, headerActionTr)
    {
        var tableHeaderStyle = "table-header-content";
        var doc = document;
        var nCol = headerFilterTr.cells.length;
        var th = doc.createElement("th");
        var actionContainer = doc.createElement('div');

        //dojo.addClass(actionContainer, tableHeaderStyle);
        var filterProvider = this.viewer.filterAttrProvider;
        var actions = filterProvider.getActionsInfo();
        
        if(actions && dojo.isArray(actions)) {
            for(var i = 0; i < actions.length; i++) {
                var action = actions[i];
                var button = document.createElement("button");

                button.innerHTML = action.title;
                button.name = action.id;
                
                actionContainer.appendChild(button);
                this.headerConnect(button, "onclick", function(event) {
                    var evt = event || window.event;
                    if (evt.target) {
                        var id = evt.target.name;
                        var listener = this.viewer.getTableListener();
                        var filters = this.getSelectedFilters();
                        var filterProvider = this.viewer.filterAttrProvider;
                        var actions = filterProvider.getActionsInfo();
                        
                        for(var i = 0; i < actions.length; i++) {
                            var action = actions[i];
                            if(action.id == id) {
                                if (listener) {
                                    listener.tableContentFilterActionInvoked(this.viewer, id, filters, action.callback, action.callbackData);
                                }
                                break;
                            }
                        }
                    }
                });
            }
        }
        
        dojo.style(actionContainer, "float", "right");
        dojo.style(actionContainer, "margin", "0.5em 0.2em");
        th.colSpan = nCol;
        th.appendChild(actionContainer);
        headerActionTr.appendChild(th);
    },

    // Summary:
    //    Creates a last column in the table to hold an action to open/close
    //    the filter header row. If there're hidden filters (not shown in
    //    individual columns), create a more column in the filter row to popup
    //    more filters.
    //
    _createFilterHeaderExpandControls: function(table, headerContentTr, headerFilterTr)
    {            
        var doc = document;
        var tableHeaderStyle = "table-header-content";
        var th = doc.createElement("th");
        var dropDown = doc.createElement("span");
        this.openFilterColAction = dropDown;
        
        dojo.addClass(dropDown, "dropdown-arrow-action");
        dojo.addClass(dropDown, "filter-expand-image");
        this.headerConnect(dropDown, "onclick", function(e){
            dojo.stopEvent(e);
            var newState = !this.filterHeaderOpenState;
            this._toggleFilterHeader(newState, true);
        });
        th.appendChild(dropDown);
        th.width = 16;
        th.align = "center";
        dojo.addClass(th, tableHeaderStyle);
        headerContentTr.appendChild(th);

        th = doc.createElement("th");
        dojo.addClass(th, TOP_BORDER_CSS); 
        headerFilterTr.appendChild(th);
    },

    _getFilterIndexByColumnIndex: function(columnIndex)
    {
        var provider = this.viewer.getContentProvider();
        var colName = provider.getColumnIdName(columnIndex);
        
        if (colName) {
            var filterProvider = this.viewer.filterAttrProvider;
            var nFilters = filterProvider.getFilterCount();
            for (var i = 0; i < nFilters; i++) {
                var name = filterProvider.getFilterIdName(i);
                if(colName == name)
                    return i;
            }
        }
        return -1;
    },
    
    // Summary:
    //    Given the column find and create a filter input field.
    //
    _createFilterControl: function(th, columnIndex)
    {
        var filterProvider = this.viewer.filterAttrProvider;
        var filterPos = this._getFilterIndexByColumnIndex(columnIndex);
        var curFilterValues = this._savedFilters;
        
        if (filterPos > -1) {
            var type = filterProvider.getFilterType(filterPos);
            var defaultValue = filterProvider.getFilterDefaultValue(filterPos);
            var params = filterProvider.getFilterParams(filterPos);
            var name = filterProvider.getFilterName(filterPos);
            var title = filterProvider.getFilterTitle(filterPos);
            var field = null;
            
            // if the filter currently has value(s), apply the value
            if(curFilterValues && typeof curFilterValues[name] != 'undefined') {
                defaultValue = curFilterValues[name];
            }
            
            // create a 'std' field, or if the client specified a custom field,
            // get it from the provider.
            if (type == TableContentFilterProvider.FIELD_CUSTOM) {
                field = filterProvider.getCustomField(filterPos);
            }
            else {
                var fieldArgs = {
                    widthHint: this.getColumnHeaderWidthInPixel(columnIndex),
                    showFieldLabel: false,
                    fieldLabel: title,
                    filterName: name,
                    defaultValue: defaultValue
                };
                for(var key in params) {
                    fieldArgs[key] = params[key];
                }
                var fieldCls = AttributeBuilder.FIELD_TYPE_CLASS_MAP[type];
                field = new fieldCls(fieldArgs);
                if(type == AttributeBuilder.FIELD_ENUM) {
                    // make the field to select the first item if there's no
                    // default value. Our filter always select the first item
                    // (usually is any (the same as empty).
                    field.defaultSelectFirstItem = true;
                    field.setEnumChoices(params);
                }
            }
            th.appendChild(field.domNode);
            
            // save the filter control, indexed by its name
            this.filterCtrls[name] = field;
        } else {
            // create a N/A label filter field if the column hs no filter
            var provider = this.viewer.getContentProvider();
            var content = this.viewer.getContent();
            var heading = provider.getColumnHeading(content, columnIndex);
            if (heading && heading != "") {
                // no need if there is no heading such as icon column
                var naField = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormLabelField({
                    fieldLabel: "&nbsp;",
                    className: "grey-center-field-label",
                    disabled: true
                });
                th.appendChild(naField.domNode);
            }
        }
    },

    _initColumnFilters: function()
    {
        var provider = this.viewer.getContentProvider();
        var content = this.viewer.getContent();
        var nColumns = provider.getNumColumns(content);
        var groupBy = this.viewer.getGroupByColumn();
        
        this.filterCtrls = {};
        for (var i = 0; i < nColumns; i++) {
            // skip displaying the group by column and non-visible columns.
            if (groupBy == i || !this.viewer.isColumnVisible(i)) {
                continue;
            }
            var colPos = i + "";
            var fth = this.filterHeaders[colPos];
            this._createFilterControl(fth, i);
        }

        // look at the full filter list and see if any filters that 
        // we have not shown in the individual columns. Certain columns
        // could be hidden by the user, or a filter don't map directly
        // to a column. We need to show them in a 'more filters' option.
        //
        var non_col_filters = [];        
        var filterProvider = this.viewer.filterAttrProvider;
        var nFilters = filterProvider.getFilterCount();
        var curFilterValues = this._savedFilters;

        for(var i = 0; i < nFilters; i++) {
            var name = filterProvider.getFilterName(i);
            
            if(!this.filterCtrls[name]) {
                var defValue = filterProvider.getFilterDefaultValue(i);
                
                // if the filter currently has value(s), apply the value
                if(curFilterValues && typeof curFilterValues[name] != 'undefined') {
                    defValue = curFilterValues[name];
                }
                non_col_filters.push({
                    'type': filterProvider.getFilterType(i),
                    'title': filterProvider.getFilterTitle(i),
                    'name': name,
                    'defaultValue': defValue,
                    'params': filterProvider.getFilterParams(i)
                });
            }
        }

        // create the 'more filters' control, if we have unused
        // filters.
        if(non_col_filters.length > 0) {
            this.moreFiltersCtrl = new DynamicFilterControl({
                viewer: this.viewer,
                filters: non_col_filters
            });
            var lastCol = this.multiFilterHeader.cells.length - 1;
            var th = this.multiFilterHeader.cells[lastCol];
            th.appendChild(this.moreFiltersCtrl.domNode);
        }       
    },
    
    _toggleFilterHeader: function(open, animate)
    {
        this.openFilterColAction.className = "dropdown-arrow-action " + (open ? "filter-collpase-image" : "filter-expand-image");
        this.openFilterColAction.title = (open) ? CloseFilterCtrl_Text : OpenFilterCtrl_Text;
        this.filterHeaderOpenState = open;
        
        // create the filter controls, if we have not created it.
        if (open && !this.filterCtrls) {
            this._initColumnFilters();
        }
        if (animate) {
            if (open) {
                // we have to make the filter header rows normal display first
                // but make its transparent. The fade in will make it 
                // completely visible.
                dojo.style(this.multiFilterHeader, "opacity", 0);
                dojo.style(this.multiFilterActionHeader, "opacity", 0);
                dojo.style(this.multiFilterHeader, "display", "");
                dojo.style(this.multiFilterActionHeader, "display", "");

                dojo.fx.combine([
                    dojo.fadeIn({
                        node: this.multiFilterHeader
                    }), dojo.fadeIn({
                        node: this.multiFilterActionHeader
                    })
                ]).play();
            }
            else {
                dojo.fx.combine([
                    dojo.fadeOut({
                        node: this.multiFilterHeader,
                        onEnd: dojo.hitch(this, function() {
                            dojo.style(this.multiFilterHeader, "display", "none");
                        })
                    }), dojo.fadeOut({
                        node: this.multiFilterActionHeader,
                        onEnd: dojo.hitch(this, function() {
                            dojo.style(this.multiFilterActionHeader, "display", "none");
                        })
                    })
                ]).play();
            }
        }
        else {
            dojo.style(this.multiFilterHeader, "display", open ? "" : "none");
            dojo.style(this.multiFilterActionHeader, "display", open ? "" : "none");            
        }
    }
});


})();
