/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.LinkCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {
var _textContent=dojo.isIE?"innerText":"textContent"; 

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.LinkCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint)
    {
    	var doc = document;
		var linkExtHTML = null;
        
		//Fix wi 7584, add link type info
		if (extendedDisplayInfo) {			
            if (typeof extendedDisplayInfo['linkType'] != "undefined") {
				linkExtHTML = "<span style='left:-3000px; position:absolute; display:inline;'>" + extendedDisplayInfo['linkType'] + "</span>";
			} 
		}
		
        // empty link. Cannot be clicked. Just create a text node.
        if(cellData == null || dojo.isString(cellData) && cellData.length == 0) {
            return doc.createTextNode(" ");
        }
        
        // don't display as link if the table is displayed in a popup
        var non_link = viewer.displayInPopup == true;
        var node = (non_link) ? doc.createElement('div') : doc.createElement("a");
        var divNode = doc.createElement('div');
        var text = cellData;

        if (viewer.isCompactMode && typeof cellWidthHint != "undefined" && dojo.isString(cellData)) {
            text = this._handleCompactCellText(viewer, text, extendedDisplayInfo, divNode, cellWidthHint);
        }
        else {
            //If cellData is a number, it wouldn't into the handleCompactCellText Method (due to the isString validation),
            // hence the numbers will be in "raw" mode, making IE to act funny and wrap longer numbers into a single cell (23741).
            if(!isNaN(cellData))
              node.style.whiteSpace = "nowrap";        
            if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
               this._attachTooltipToCell(node, extendedDisplayInfo['tooltip']);                            
            }            
        } 
        
        if(linkExtHTML)
            node.innerHTML = linkExtHTML; 

        divNode[_textContent]=text;
        node.appendChild(divNode);

        if (!non_link && extendedDisplayInfo) {			
            if (typeof extendedDisplayInfo['href'] != "undefined") {
                node.href = extendedDisplayInfo['href'];
			} 
            else if (dojo.isFunction(extendedDisplayInfo['onclick'])) {
                node.setAttribute("href", "javascript:{}");
                
                // make sure the link onclick event isn't propagated
                // up. Jazz has a onclick handler that checks for link 
                // clicked to put up a leave page confirm. 
                this.connect(node, "onclick", dojo.hitch({'extendedDisplayInfo': extendedDisplayInfo},this._onClick));
			}
            if (typeof extendedDisplayInfo['target'] != "undefined") {
                node.target = extendedDisplayInfo['target'];
			} 
		}
        return node;
    },
    _onClick: function(e){
       // local event don't propagate the event up.
        dojo.stopEvent(e);
        var func = this.extendedDisplayInfo['onclick'];
        func();
     }
});

})();
