/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008 - 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The Table Content Filter Provider is the interface class that the client of the TableViewer
 //    can implement and installed to a TableViewer instance. It's used to describe the filtering
 //    attributes for the table's content. When it's provided, the TableViewer creates a section
 //    with the specified input fields that would allow the user to enter filtering values. 
 //    
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder");

(function() {
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants; 

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider", null,
{
    // Summary:
    //    Returns the title for the filter section. The filter input fields are shown in a panel
    //    above the table controls. This panel can display a title. 
    //
    getTitle: function()
    {
        return null;
    },
    
    // Summary:
    //    Returns the summary text for the filter section.
    //
    getSummaryText: function()
    {
        return "";
    },
    
	// Summary:
	//    The client can control whether the filter section is initially open or closed.
	//
    getIsSectionOpenInitially: function()
    {
	    // return true to open, false to close initially
        return true;	
    },
	
    // Summary:
    //    The filter section has the same size width as the width of the table. The filter input
    //    fields are layout in a single column inside the panel. Adjust the width of the fields
    //    as a percentage of the width of the panel. If 50 is specified, the fields right edge will
    //    be on the mid-point of the panel, etc.
    //
    getFilterFieldAreaWidth: function()
    {
        return 65;
    },

    // Summary:
    //    The input field column consists of label and the input field. Specifies the width of the field
    //    labels as a percentage of the input field column. Please note that this is the percentage of the
    //    field area width specified above.
    //
    getFilterFieldLabelWidth: function()
    {
        return 20;
    },
    
    // Summary:
    //    Returns the initial number of visible attributes in the filter section. 
    //    If the total attributes exceeds this count, they will not be shown initially.
    //    The user can add these hidden attributes in the filter section. 
    // 
    getInitialVisibleFilterCount: function()
    {
        // -1 means no hidden attributes. Show all.
        return -1;    
    },
    
    // Summary:
    //    Returns the number of filter fields.
    //
    getFilterCount: function()
    {
        return 0;
    },

    // Summary:
    //    Returns an array of objects representing the filter definitions for this provider.
    //
    getFilters: function()
    {
        // e.g. "owner"
        return [];        
    },
	    
    // Summary:
    //    Returns the name of a filter input field. The name is the id for the field.
    //    Index indicates which filter input field. It's 0 based and up to the filter count - 1.
    //
    getFilterName: function(index)
    {
        // e.g. "owner"
        return "";        
    },
    
    // Summary:
    //    Returns the id name of the filter. The ID name is the server side identifier
    //    of the filter. If this name matches the ID name of a column in the content provider,
    //    this filter is displayed in the column's filter input area.
    //
    getFilterIdName: function(index)
    {
        // default to the 'name of the filter. The name usually is the 
        // id name of the filter.
        return this.getFilterName(index);        
    },

    // Summary:
    //    Returns the title of a filter input field. It's the visible display label for the field.
    //    Index indicates which filter input field. It's 0 based and up to the filter count - 1.
    //
    getFilterTitle: function(index)
    {
        // e.g. "Owner:"
        return "";        
    },
    
    // Summary:
    //    Returns the prompt title of a filter input field. If the filter is set to have hidden filter
    //    fields, the user can be prompted to add the hidden attribute to the visible attributes. The prompt
    //    title for the filter field should be different from the field's label, i.e., without any colon. 
    //    Index indicates which filter input field. It's 0 based and up to the filter count - 1.
    //
    getFilterPromptTitle: function(index)
    {
        // e.g. "Owner"
        return "";        
    },
    
    // Summary:
    //    Returns the type of a filter input field. It's one of the input field type specified below. 
    //    Index indicates which filter input field. It's 0 based and up to the filter count - 1.
    //
    getFilterType: function(index)
    {
        return null;        
    },
    
    // Summary:
    //    Returns the default value of a filter input field. The initial value being shown in the field, if any.
    //    Index indicates which filter input field. It's 0 based and up to the filter count - 1.
    //
    getFilterDefaultValue: function(index)
    {
        return null;        
    },
    
    // Summary:
    //    Returns additionl parameters for a filter input field, like the enum type would require the choices.
    //    Index indicates which filter input field. It's 0 based and up to the filter count - 1.
    //
    getFilterParams: function(index)
    {
        return null;        
    },
    
    // Summary:
    //    Returns the custom field created by the client. This method is only called when the specified
    //    type is a custom. This field must be a subclass of AbstractFormField. 
    //    Index indicates which filter input field. It's 0 based and up to the filter count - 1.
    //
    getCustomField: function(index)
    {
        return null;        
    },
    
    // Summary:
    //    Returns an array of action descriptors with the following attributes:
    //    {
    //      id: 'action name', 
    //      title: 'the display name of the action', 
    //      callback: callback function, can be null
    //      callbackData: callback data, or null
    //      isDefault: true | false to indicate whether the action is the default.
    //    }  
    getActionsInfo: function()
    {
        return null;
    }    
});

var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;
var AttributeBuilder = com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder;

// Define the supported filter field types
TableContentFilterProvider.FIELD_TEXT = AttributeBuilder.FIELD_TEXT;
TableContentFilterProvider.FIELD_LABEL = AttributeBuilder.FIELD_LABEL;
TableContentFilterProvider.FIELD_ENUM = AttributeBuilder.FIELD_ENUM;
TableContentFilterProvider.FIELD_NUMERIC = AttributeBuilder.FIELD_NUMERIC;
TableContentFilterProvider.FIELD_CHECKBOX = AttributeBuilder.FIELD_CHECKBOX;
TableContentFilterProvider.FIELD_TOGGLELIST = AttributeBuilder.FIELD_TOGGLELIST;
TableContentFilterProvider.FIELD_TEXT_POPUP = AttributeBuilder.FIELD_TEXT_POPUP;
TableContentFilterProvider.FIELD_DATE = AttributeBuilder.FIELD_DATE;
TableContentFilterProvider.FIELD_DATETIME_RANGE = AttributeBuilder.FIELD_DATETIME_RANGE;
TableContentFilterProvider.FIELD_IP = AttributeBuilder.FIELD_IP;
TableContentFilterProvider.FIELD_CUSTOM = "Custom";

})();
