/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.wizard.WizardDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.ProgressBar")

dojo.require("com.ibm.team.repository.web.foundation.ProgressMonitor");

dojo.require("dojox.data.dom");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function(){
	
var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
var AlertDialog   = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
	
dojo.declare("com.ibm.rqm.defects.web.ui.internal.wizard.WizardDialog", 
			 [dijit._Widget, dijit._Templated], {

	labelBack: DefectsUIUtil.getMessage("labelBack"),
	labelNext: DefectsUIUtil.getMessage("labelNext"),
	labelFinish: DefectsUIUtil.getMessage("labelFinish"),
	labelCancel: DefectsUIUtil.getMessage("labelCancel"),
	
	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_dialogRoot: null,
	_titleContainer: null,
	_subTitleContainer: null,
	_content: null,
	_progressSection: null,
	_progressBar: null,
	_progressMessage: null,
	_backButton: null,
	_nextButton: null,
	_finishButton: null,
	_cancelButton: null,

	_width: "40%",
	_progressBarWidget: null,
	_currentPage: null,
	
	_wizard: null,
	
	constructor: function() {
	},
	
	// ---- api ------------------------------------------------------------------------------------------------------------	

	// ---- WizardDialog api -----------------------------------------------------------------------------------------------	

	updateButtons: function() {
		this._enableButton(this._backButton, this._wizard.getPreviousPage(this._currentPage) != null);
		this._enableButton(this._nextButton, this._wizard.getNextPage(this._currentPage) != null && this._currentPage.isPageComplete());
		this._enableButton(this._finishButton, this._wizard.canFinish());
	},
	
	runWithProgress: function(progressMonitoredCode) {
		var monitor= new com.ibm.rqm.defects.web.ui.internal.wizard.WizardDialog._ProgressMonitor(this);
		this._progressSection.style.visibility= "visible";
		progressMonitoredCode(monitor);
	},

	// ---- events ---------------------------------------------------------------------------------------------------------	

	onFinish: function() {
	},

	onCancel: function() {
		
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.rqm.defects.web", "ui/internal/wizard/templates/WizardDialog.html"),

	postCreate: function() {
		dojo.forEach(this._wizard.getPages(), dojo.hitch(this, function(page) {
			page.setWizardDialog(this);
		}));

		this._progressBarWidget= new dijit.ProgressBar({}, this._progressBar);
		this._progressBarWidget.update({indeterminate: true});
		
		this._dialog = new jazz.ui.Dialog({
			contentNode: this._dialogRoot,
			primaryTitle: this._wizard.getTitle(),
			width: this._width,
			isClosable: false
		});
		
		this.connect(this._backButton, "onclick", "_onBack");
		this.connect(this._nextButton, "onclick", "_onNext");
		this.connect(this._finishButton, "onclick", "_onFinish");
		this.connect(this._cancelButton, "onclick", "_onCancel");

		this._showPage(this._wizard.getInitialPage());
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_showPage: function(page) {
		if (this._currentPage != null) {
			this._currentPage.aboutToHide();
		}

		page.aboutToShow(dojo.hitch(this, function(status) {
			if (status.isOK()) {
				this._currentPage= page;
				dojox.data.dom.replaceChildren(this._titleContainer, document.createTextNode(page.getTitle()));
				dojox.data.dom.replaceChildren(this._subTitleContainer, document.createTextNode(page.getDescription()));
				dojox.data.dom.replaceChildren(this._content, page.getContentNode());
				
				this.updateButtons();
			}
		}));
	},

	_onBack: function() {
		var previousPage= this._wizard.getPreviousPage(this._currentPage);
		if (previousPage != null)
			this._showPage(previousPage);
	},

	_onNext: function() {
		if (this._currentPage.isPageComplete()) {
			var nextPage= this._wizard.getNextPage(this._currentPage);
			if (nextPage != null)
				this._showPage(nextPage);
		}
	},

	_onFinish: function() {
		if (this._currentPage != null) {
			this._currentPage.aboutToHide();
		}

		this.runWithProgress(dojo.hitch(this, function(monitor) {
			this._wizard.performFinish(dojo.hitch(this, function(status) {
				monitor.done();
				if (status.isOK()) {
					this.onFinish();
					this._dialog.close();
					this._dialog.destroy();
				} else {
					var alertDialog = new AlertDialog({
		                'message': status.getMessage()
		            });
		            alertDialog.open();
				}
			}), monitor);
		}));
	},

	_onCancel: function() {
		this.onCancel();
		this._dialog.close();
		this._dialog.destroy();
	},

	_enableButton: function(button, enabled) {
		if (enabled) {
			button.removeAttribute("disabled");
		} else {
			button.setAttribute("disabled", "disabled");
		}
	},
	
	_notifyBeginTask: function(name) {
		dojox.data.dom.replaceChildren(this._progressMessage, document.createTextNode(name));
	},

	_notifyDone: function() {
		this._progressSection.style.visibility= "hidden";
		dojox.data.dom.removeChildren(this._progressMessage);
	},


	__sentinel: null // terminates this class definition
});

com.ibm.rqm.defects.web.ui.internal.wizard.WizardDialog.create= function(wizard) {
	return new com.ibm.rqm.defects.web.ui.internal.wizard.WizardDialog({
		_wizard: wizard
	}, null);
};


dojo.declare("com.ibm.rqm.defects.web.ui.internal.wizard.WizardDialog._ProgressMonitor",
			 [com.ibm.team.repository.web.foundation.ProgressMonitor], {

	constructor: function(wizardDialog) {
		this._wizardDialog= wizardDialog;
	},

	// ---- ProgressMonitor implementation ---------------------------------------------------------------------------------

	beginTask: function(name) {
		this._wizardDialog._notifyBeginTask(name);
	},

	done: function() {
		this._wizardDialog._notifyDone();
	},

	__sentinel: null // terminates this class definition
});

})();
