/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.wizard.WizardReview");

dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardReview");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {

var WizardReview = com.ibm.team.repository.web.ui.internal.wizard.WizardReview;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;

var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

var Constants 				= com.ibm.rqm.planning.web.client.internal.Constants;

var QM_REPO_INIT_SERVICE_URI = "com.ibm.rqm.process.deployment.rest.IQMRepoInitializerRestService";

var CONFIG_COUNT = 4;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.wizard.WizardReview",  [WizardReview], {

	templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/wizard/templates/WizardReview.html"),

	labelSummary: Constants.getMessage("labelSummary"),
	
	constructor: function() {
		this._projectArea = null;
	},

	postCreate: function() {
		this.inherited('postCreate', arguments);	
	},
	
	//@Override
	_incrementStatusReturnCount: function () {
		this._statusReturnCount++;
		if (this._statusReturnCount == CONFIG_COUNT) {
			this.header.clearMessages();
		}
	},	
	
	//@Override
	_refresh: function () {
		this.inherited('_refresh', arguments);
		this._fetchDefaultProjectArea();
	},
	
	_fetchDefaultProjectArea: function() {
		var responseHandler = new ServiceResponseHandler(this, "_fetchProjectSuccess", "_fetchProjectFailure");
		var serviceRequest= new ServiceRequest(QM_REPO_INIT_SERVICE_URI, "getDefaultProjectArea", {});
		TeamServerClient.invokeService(serviceRequest, responseHandler);		
	},
	
	_fetchProjectSuccess: function(projectArea) {
		this._incrementStatusReturnCount();
		if (projectArea != null) {
			this._projectArea = projectArea;
			this._setupCompleted();
		}
	},
	
	_fetchProjectFailure: function(error) {
		this.header.displayServiceCallbackErrorMessage(error);
	},
	
	_setupCompleted: function() {
		//add item to summary list
		var projectSummary = this._createConfigSummaryItem(Constants.getMessage("labelProjectArea"), this._projectArea.name);
		this._summaryList.appendChild(projectSummary);
		
		//show completion message & update button states
		this.header.showStatus({
			type: WizardStepHeader.STATUS_TYPES.OK,
			summary: Constants.getMessage("msgSetupCompleted")
		});		
		this.triggerButtonStateUpdate();		
	},
	
	//override from "com.ibm.team.repository.web.ui.internal.wizard.WizardReview"
	//add title for link elements
	_createConfigSummaryItem: function(label, summary, props){
		if (props != null && props.length > 0) {
			var item = document.createElement("a");
			item.href="#";
			
			//defect 17443, add title param
			item.title = label;
		} else {
			var item = document.createElement("div");
		}
		item.className = "summaryItem";
		
		var labelDiv = document.createElement("div");
		labelDiv.className = "summaryItemLabel";
		labelDiv.innerHTML = label;
		item.appendChild(labelDiv);
		
		var statusDiv = document.createElement("div");
		statusDiv.className = "summaryItemStatus";
		statusDiv.innerHTML = summary;
		item.appendChild(statusDiv);
		
		var itemClear = document.createElement("div");
		itemClear.className = "summaryItemClear";
		
		if (props != null && props.length > 0) {
			var itemProperties = this._createDetailedProperties(props);
			itemProperties.className = "summaryItemProperties";
			itemClear.appendChild(itemProperties);
			item._detailedProps = itemProperties;
			item._detailedProps.style.display = "none";
			
			this.connect(item, "onclick", function(event){
				dojo.stopEvent(event);
				if (item._detailedProps.style.display === "none") {
					var newDisplay = "";
					dojo.addClass(item, "expanded");
				} else {
					var newDisplay = "none";
					dojo.removeClass(item, "expanded");
				}
				item._detailedProps.style.display = newDisplay;
			});
		} 
		item.appendChild(itemClear);
		
		return item;
	}, 
	
	_doFinish: function() {
		if (this._projectArea != null) {
			document.location.href = net.jazz.ajax._contextRoot + "/web/console/" + encodeURIComponent(this._projectArea.name);
		} else {
			document.location.href = net.jazz.ajax._contextRoot + "/admin";
		}
	},
		
	// SUPER WizardStep functions
	focusStep: function () {
		this._refresh();
		if (this.wizard) {
			this.wizard.processFinish = dojo.hitch(this, "_doFinish");
		}		
	}, 
	
	canFinish: function() {
		return (this._projectArea != null);
	}
	
});

})();