/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.workitems.AssociatedWorkItemViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.app.ResourceLink");

dojo.require("com.ibm.team.repository.web.foundation.ProgressMonitor");

dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.defects.web.client.internal.CmIntegrationClient");

(function() { // module container

var ProgressMonitor				= com.ibm.team.repository.web.foundation.ProgressMonitor;
var DefectsUIUtil 				= com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
var CmIntegrationClient			= com.ibm.rqm.defects.web.client.internal.CmIntegrationClient;

dojo.declare("com.ibm.rqm.defects.web.ui.workitems.AssociatedWorkItemViewer",
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_dialogRoot: null,
	_defectsContainer: null,
	_defectsTable: null,
	_defectsTableHeader: null,
	_defectsTableBody: null,
	_progressSection: null,
	_progressBar: null,
	_jobName: null,
	_taskName: null,

	labelId: DefectsUIUtil.getMessage("labelId"),
	labelSummary: DefectsUIUtil.getMessage("labelSummary"),

	_ewiItemId: null,
//	_defectsProvider: null,
	_draftUriFilter: null,
	_isExecutionResultRelated: null,
	_linkType: null,

	constructor: function() {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------	

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.rqm.defects.web", "ui/workitems/templates/AssociatedWorkItemViewer.html"),

	postCreate: function() {
		this._progressBarWidget= new dijit.ProgressBar({}, this._progressBar);
		this._progressBarWidget.update({indeterminate: true});

		this._dialog = new jazz.ui.Dialog({
			width: "800px",
			contentNode: this._dialogRoot,
			primaryTitle: DefectsUIUtil.getMessage("labelAssociatedDefects")
		});

		this._runWithProgress(DefectsUIUtil.getMessage("labelLoadAssociatedDefects"), dojo.hitch(this, function(monitor) {
			
			
			var workItemIdentification = { 
					linkType: this._linkType,
					isExecutionResultRelated: false, 
					itemIdOrUriArray: [this._ewiItemId] 
			};
			CmIntegrationClient.postAssociatedWorkItems(workItemIdentification, dojo.hitch(this, function(status, response) {
				try {
					if (status.isOK()) {
						dojo.forEach(response, dojo.hitch(this, function(workItem) {
							var _draftUriFilter = this._draftUriFilter;
							if(_draftUriFilter && dojo.isArray(_draftUriFilter)){
								for(var i = 0; i < _draftUriFilter.length; i++){
									if (_draftUriFilter[i].indexOf(workItem.uri) == 0){
										this._addWorkItemRow(workItem);
									}
								}
							}else{
								this._addWorkItemRow(workItem);
							}
						}));
					}
				} finally {
					monitor.done();
				}
			}));
		}));
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_addWorkItemRow: function(workItem) {
		var row= document.createElement("tr");
		dojo.addClass(row, "item-table-row");

		var idCell= document.createElement("td");
		dojo.addClass(idCell, "table-cell");
		
		var anchor = document.createElement("a");
		anchor.href= workItem.reference;
		anchor.target= "_blank";
		
		var resourceLinkArgs= { 
				uri: workItem.uri,
				presentationUri: workItem.uri,
				label: workItem.id
			};
		new jazz.app.ResourceLink(resourceLinkArgs, anchor);

		var id= workItem.id;
		if (!id)
			id= "---";


		idCell.appendChild(anchor);
		row.appendChild(idCell);

		var summaryCell= document.createElement("td");
		dojo.addClass(summaryCell, "table-cell");
		
		var summary= this._restoreTags(workItem.summary);
		if (!summary)
			summary= DefectsUIUtil.getMessage("labelAssociatedDefect");
		
		summaryCell.appendChild(document.createTextNode(summary));
		row.appendChild(summaryCell);

		this._defectsTableBody.appendChild(row);
	},
	
	_restoreTags: function(value) {
		var rawText= value;
		rawText= rawText.replace(/\&gt;/gi,">");
		rawText= rawText.replace(/\&lt;/gi,"<");
		rawText= rawText.replace(/\&amp;/gi,"&");
		rawText= rawText.replace(/\&tilde;/gi,"~");
		rawText= rawText.replace(/\&quot;/gi,"\"");
		rawText= rawText.replace(/\&circ;/gi,"^");
		rawText= rawText.replace(/\&nbsp;/gi," ");
		rawText= rawText.replace(/<br\/>/gi,"\n");
		rawText= rawText.replace(/\&#09;/g, "\t");
		return rawText
	},


	_runWithProgress: function(jobName, progressMonitoredCode) {
		var monitor= new ProgressMonitor(dojo.hitch(this, this._notifyBeginTask), dojo.hitch(this, this._notifyDone));
		dojox.data.dom.replaceChildren(this._jobName, document.createTextNode(jobName));
		this._progressSection.style.visibility= "visible";
		progressMonitoredCode(monitor);
	},

	_notifyBeginTask: function(name) {
		dojox.data.dom.replaceChildren(this._taskName, document.createTextNode(name));
	},

	_notifyDone: function() {
		this._progressSection.style.visibility= "hidden";
		dojox.data.dom.removeChildren(this._jobName);
		dojox.data.dom.removeChildren(this._taskName);
	},

	__sentinel: null // terminates this class definition
});

com.ibm.rqm.defects.web.ui.workitems.AssociatedWorkItemViewer.create= function(ewiItemId, linkType, isExecutionResultRelated, draftUriFilter) {
	return new com.ibm.rqm.defects.web.ui.workitems.AssociatedWorkItemViewer({
//		_defectsProvider: defectsProvider,
		_ewiItemId: ewiItemId,
		_draftUriFilter: draftUriFilter,
		_isExecutionResultRelated: isExecutionResultRelated,
		_linkType: linkType
	}, null);
}

})();
