/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2008. All
 * Rights Reserved.
 * 
 * U.S. Government Users Restricted Rights: Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
// -*- tab-width: 4 -*-
package com.ibm.rational.test.lm.spi.bf;

import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import com.ibm.rational.test.lm.spi.IAutomationSPI;
import com.ibm.rational.test.lm.spi.IInventorySPI;
import com.ibm.rational.test.lm.spi.ITracer;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIAuthenticationException;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIClientConnectionException;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIException;
import com.ibm.rational.test.lm.spi.bf.internal.BFAPIClientConnection;
import com.ibm.rational.test.lm.spi.common.IRestAPIResponseCodes;
import com.ibm.rational.test.lm.spi.common.IRestResponse;
import com.ibm.rational.test.lm.spi.common.SPIAuthenticationException;
import com.ibm.rational.test.lm.spi.common.SPIClientConnectionException;
import com.ibm.rational.test.lm.spi.common.SPIException;
import com.ibm.rational.test.lm.spi.common.SPIResourceNotFoundException;
import com.ibm.rational.test.lm.spi.common.SPIURI;
import com.ibm.rational.test.lm.spi.common.URIResourceType;

public class BFRestSPIImpl implements IInventorySPI, IAutomationSPI {
	private static ITracer g_tracer;
	private InventoryService _inventoryService = InventoryService.INSTANCE;
	private AutomationService _automationService = AutomationService.INSTANCE;


	/*
	 * Holding user name of every Build Forge instance on this version. But for
	 * current Build Forge implementation, all of connections are system
	 * connection, so user name is not necessary for current implementation.
	 * 
	 * Key: hostname:port Value: username
	 */
	private static Map<String, String> usernamesMap = new HashMap<String, String>();

	/*
	 * Holding password of every Build Forge instance on this version. All of
	 * passwords are system user password for current implementation.
	 * 
	 * Key: hostname:port Value: password
	 */
	private static Map<String, String> passwordsMap = new HashMap<String, String>();

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.rational.test.lm.spi.AbstractRestSPI#init(java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	public void init(String initProperties, ITracer tracer) throws Exception {
	    String meth_name = "init"; ////$NON-NLS-N$
		g_tracer = tracer;

		String oldUserName = null;
		String oldPwd = null;
		// Initial usernamesMap and passwordsMap by parsing
		// initProperties XML doclet.
		SAXBuilder xmlIn = new SAXBuilder();
		StringReader reader = new StringReader(initProperties);
		try {
			Document providerDoclet = xmlIn.build(reader);
			List<Element> providerInstances = providerDoclet.getRootElement()
					.getChildren(INSTANCE_ELEMENT_NAME);
			for (Element instance : providerInstances) {
				List<Element> credentials = instance
						.getChildren(CREDENTIAL_ELEMENT_NAME);
				if (credentials.size() > 0) {
					String hostname = instance
							.getChildText(HOSTNAME_ELEMENT_NAME);
					String port = instance.getChildText(PORT_ELEMENT_NAME);
					String username = credentials.get(0).getChildText(
							USERNAME_ELEMENT_NAME);
					String password = credentials.get(0).getChildText(
							PASSWORD_ELEMENT_NAME);
					StringBuffer mapKeyBuf = new StringBuffer(hostname);
					mapKeyBuf.append(COLON);
					mapKeyBuf.append(port);
					synchronized (usernamesMap) {
						oldUserName = usernamesMap.get(mapKeyBuf.toString());							
						usernamesMap.put(mapKeyBuf.toString(), username);
					}
					synchronized (passwordsMap) {
						oldPwd = passwordsMap.get(mapKeyBuf.toString());						
						passwordsMap.put(mapKeyBuf.toString(), password);
					}
					// ping the provider server to check the
					// username/password/version
					boolean isUserSame = (username == null)? false:
						(username.equalsIgnoreCase(oldUserName));
					boolean isPwdSame = (password == null)? false:
						(password.equals(oldPwd));
					if ((!isUserSame) || (!isPwdSame))
						ping(hostname, port);
				}
			}
		} catch (JDOMException e) {
			// TODO: add descriptive TMS error here
		    tracer.trace(ITracer.TraceLevel.ERROR, meth_name, "Problem parsing initial properties in BF Impl:"	+ e.toString());////$NON-NLS-N$
		}
	}

	public static ITracer getTracer()
	{
		return g_tracer;
		
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.rational.test.lm.spi.AbstractRestSPI#get(java.net.URI)
	 */
	public IRestResponse get(java.net.URI uri) {
		// Get the meta data from the URI
		SPIURI parser = SPIURI.parse(uri);
		String providerHostName = parser.getProviderHostname();
		int providerPort = parser.getProviderPort();
		URIResourceType resourceType = parser.getResourceType();
		String resourceId = parser.getResourceID();
		URIResourceType subResourceType = parser.getSubResourceType();
		String xml = null;
		RestResponse response = new RestResponse();

		// By default, set the Response code to SC_OK.
		response.setResponseCode(IRestAPIResponseCodes.SC_OK);

		switch (resourceType) {
		case asset:
			try {

				if (resourceId == null || "".equals(resourceId)) {
					xml = _inventoryService.listAllAssetIds(providerHostName,
							providerPort);
				} else if (resourceId.equals("*")) {
					xml = _inventoryService.updateAllAssets(providerHostName,
							providerPort);
				} else {
					xml = _inventoryService.updateAssetById(resourceId,
							providerHostName, providerPort);
				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case discoveryControl:
			try {
				if (resourceId == null || resourceId.length() == 0) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
				} else {
					xml = _inventoryService.getDiscoveryRunStatus(
							providerHostName, providerPort, resourceId);

				}
				if (xml == null) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_NO_CONTENT);
					return response;
				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case assetStatus:
			try {
				xml = _inventoryService.getStatusPollById(resourceId,
						providerHostName, providerPort);
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case jobDefinition:
			try {
				if (resourceId == null || "".equals(resourceId)) {
					// List all job definition id
					xml = _automationService.listAllJobDefIds(providerHostName,
							providerPort);
				} else {
					xml = _automationService.getJobDef(resourceId,
							providerHostName, providerPort);
				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case jobExecution:

			// job result and job status info are all in JobDetails ,resolving
			// it can get needed data.
			try {
				if (resourceId == null || "".equals(resourceId)) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
				} else if (subResourceType == null) {
					xml = _automationService.getJobDetailsByJobId(resourceId,
							providerHostName, providerPort);
				} else if (subResourceType == URIResourceType.log) {
					xml = _automationService.getJobLogByJobId(resourceId,
							providerHostName, providerPort);
				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		default:
			response.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
			break;
		}

		// Set the result
		response.setXMLResponse(xml);
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.rational.test.lm.spi.AbstractRestSPI#post(java.net.URI,
	 *      java.lang.StringBuffer)
	 */
	public IRestResponse post(java.net.URI uri, String XMLBody) {
		// Get the meta data from the URI
		SPIURI parser = SPIURI.parse(uri);
		String providerHostName = parser.getProviderHostname();
		int providerPort = parser.getProviderPort();
		URIResourceType resourceType = parser.getResourceType();
		String resourceId = parser.getResourceID();
		String xml = null;
		RestResponse response = new RestResponse();

		// By default, set the Response code to SC_OK.
		response.setResponseCode(IRestAPIResponseCodes.SC_OK);

		switch (resourceType) {
		case asset:
			try {
				xml = _inventoryService.addAsset(XMLBody, providerHostName,
						providerPort);
				// xml is null when adding resource failed.
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case discoveryControl:
			try {
				if ("*".equals(resourceId)) {
					xml = _inventoryService.discoveryAll(providerHostName,
							providerPort);
				} else {
					xml = _inventoryService.discoveryById(providerHostName,
							providerPort, resourceId);
				}
				if (xml == null) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_NO_CONTENT);
					return response;
				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case assetStatus:
			try {
				if (resourceId == null || "".equals(resourceId)) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
				} else {

					xml = _inventoryService.startStatusPollById(resourceId,
							providerHostName, providerPort);
				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case jobDefinition:
			response.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
			break;

		case jobExecution:
			try {
				// TODO We need a new Zephyr TDM Domain to wrapper the
				// JobDefinition Id and
				// Automation Parameters so that we could use it to execute the
				// job.
				// Modify it later to use the
				// _automationService.executeJob(XMLBody,providerHostName,
				// providerPort);
				xml = _automationService.executeJobDef(XMLBody,
						providerHostName, providerPort);
				if (xml == null) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_NO_CONTENT);
					return response;
				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		default:
			response.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
			break;
		}

		// Set the result
		response.setXMLResponse(xml);
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.rational.test.lm.spi.AbstractRestSPI#delete(java.net.URI)
	 */
	public IRestResponse delete(java.net.URI uri) {
		// Get the meta data from the URI
		SPIURI parser = SPIURI.parse(uri);
		String providerHostName = parser.getProviderHostname();
		int providerPort = parser.getProviderPort();
		URIResourceType resourceType = parser.getResourceType();
		String resourceId = parser.getResourceID();
		String xml = null;
		RestResponse response = new RestResponse();

		// By default, set the Response code to SC_OK.
		response.setResponseCode(IRestAPIResponseCodes.SC_OK);

		switch (resourceType) {
		case asset:
			try {
				if (resourceId == null || resourceId.length() == 0) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
				} else {
					_inventoryService.deleteAsset(providerHostName,
							providerPort, resourceId);

				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case discoveryControl:
			try {
				if (resourceId == null || resourceId.length() == 0) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
				} else {
					_inventoryService.cancelDiscovery(providerHostName,
							providerPort, resourceId);

				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case assetStatus:

			try {
				if (resourceId == null || resourceId.length() == 0) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
				} else {
					_inventoryService.cancelStatusPoll(resourceId,
							providerHostName, providerPort);

				}
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		case jobDefinition:
			response.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
			break;

		case jobExecution:
			try {
				_automationService.cancelJob(resourceId, providerHostName,
						providerPort);
			} catch (Exception e) {
				return handleResponseException(response, e);
			}
			break;

		default:
			response.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
			break;
		}

		// Set the result
		response.setXMLResponse(xml);
		return response;
	}

	class RestResponse implements IRestResponse {
		private int responseCode;
		private String xmlResponse;

		public int getResponseCode() {
			return responseCode;
		}

		public String getXMLResponse() {
			return xmlResponse;
		}

		public void setResponseCode(int code) {
			responseCode = code;
		}

		public void setXMLResponse(String XMLString) {
			xmlResponse = XMLString;
		}
	}

	/**
	 * Get user name of the Build Forge instance which is identified by
	 * hostname:port
	 * 
	 * @param hostname
	 *            hostname of the Build Forge instance
	 * @param port
	 *            port of the Build Forge instance
	 * @return user name of the Build Forge instance or null if not found
	 */
	public static String getUsername(String hostname, int port) {
		StringBuffer keyBuf = new StringBuffer();
		keyBuf.append(hostname);
		keyBuf.append(COLON);
		keyBuf.append(port);
		synchronized (usernamesMap) {
			return usernamesMap.get(keyBuf.toString());
		}
	}

	/**
	 * Get system password of the Build Forge instance which is identified by
	 * hostname:port
	 * 
	 * @param hostname
	 *            hostname of the Build Forge instance
	 * @param port
	 *            port of the Build Forge instance
	 * @return system password of the Build Forge instance or null if not found
	 */
	public static String getPassword(String hostname, int port) {
		StringBuffer keyBuf = new StringBuffer();
		keyBuf.append(hostname);
		keyBuf.append(COLON);
		keyBuf.append(port);
		synchronized (passwordsMap) {
			return passwordsMap.get(keyBuf.toString());
		}
	}

	private RestResponse handleResponseException(RestResponse response,
			Exception ex) {
		if (ex instanceof SPIClientConnectionException) {
			response.setResponseCode(IRestAPIResponseCodes.SC_GATEWAY_TIMEOUT);
		} else if (ex instanceof SPIAuthenticationException) {
			response.setResponseCode(IRestAPIResponseCodes.SC_FORBIDDEN);
		} else if (ex instanceof SPIResourceNotFoundException) {
			response.setResponseCode(IRestAPIResponseCodes.SC_NOT_FOUND);
		} else {
			response
					.setResponseCode(IRestAPIResponseCodes.SC_INTERNAL_SERVER_ERROR);
		}
		if (ex.getMessage().contains("BuildEnvMustChange")) {
			response.setXMLResponse("One of the automation parameters must be changed");
		}
		else { 
			response.setXMLResponse(ex.toString());
		}
		return response;
	}

	private void ping(String hostname, String port) throws Exception {
		BFAPIClientConnection conn = null;
		try {
			// ping the provider server to check the
			// username/password/version
			 conn = BFAPIClientConnection
					.getConnection(hostname, Integer.valueOf(port));
//			conn.connect();
			
		} catch (Exception ex) {
			if (ex instanceof BFAPIAuthenticationException) {
				throw new SPIAuthenticationException(ex.getMessage());
			} else if (ex instanceof BFAPIClientConnectionException) {
				throw new SPIClientConnectionException(ex.getMessage());
			} else if (ex instanceof BFAPIException) {
				throw new SPIException(ex.getMessage());
			} else {
				throw new Exception(ex.getMessage());
			}
		}
		finally {
			if (conn != null) {
				try {
					conn.close();
				} catch (BFAPIClientConnectionException e) {
					//log 
				}
			}
		}
	}
}
