/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2008. All
 * Rights Reserved.
 * 
 * U.S. Government Users Restricted Rights: Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
// -*- tab-width: 4 -*-

package com.ibm.rational.test.lm.spi.bf.internal;

import com.buildforge.services.client.api.APIClientConnection;
import com.ibm.rational.test.lm.spi.bf.BFRestSPIImpl;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIException;
import com.ibm.rational.test.lm.spi.bf.common.BFConstants;

public class ConnectionController {

	public static final Class<ConnectionController> CLASS = ConnectionController.class;

	private static boolean buildForgeInitialized = false;
	private static final String defaultBfServerSystemPassword = "styX1xyzzy"; //$NON-NLS-1$
	private static boolean buildClassesInitialized = false;
	private static boolean buildCollectorInitialized = false;

	public static BFAPIClientConnection getAuthConnection(
			String bfServerHostname, int bfServerPort, String domain,
			String loginName, String password) {
		initBuildForge(bfServerHostname, bfServerPort);
		return new BFAPIClientConnection(bfServerHostname, bfServerPort,
				domain, loginName, password);

	}

	public static BFAPIClientConnection getSystemConnection(
			String bfServerHostname, int bfServerPort) {
		initBuildForge(bfServerHostname, bfServerPort);
		String systemPassword = getBFSystemPassword(bfServerHostname,
				bfServerPort);
		return getSystemConnectionRaw(bfServerHostname, bfServerPort,
				systemPassword);
	}

	private static BFAPIClientConnection getSystemConnectionRaw(
			String bfServerHostname, int bfServerPort, String systemPassword) {
		return new BFAPIClientConnection(bfServerHostname, bfServerPort, null,
				systemPassword);
	}

	public static void initBuildForge(String bfServerHostname, int bfServerPort) {
		if (!buildForgeInitialized) {
			String systemPassword = getBFSystemPassword(bfServerHostname,
					bfServerPort);
			BFAPIClientConnection systemConn = getSystemConnectionRaw(
					bfServerHostname, bfServerPort, systemPassword);
			try {
				systemConn.connect();
				try {
					APIClientConnection apiConn = systemConn.getConnection();
					// init roles for RTLM
					BuildForgeArtifacts.initRoles(apiConn);
					// init build class for RTLM
					initBuildClasses(apiConn);
					// init Collector for RTLM
					initCollector(apiConn, BFConstants.agentCollectorId);
					buildForgeInitialized = true;
				} finally {
					systemConn.close();
				}
			} catch (BFAPIException e2) {

			}
		}
	}

	public static BFAPIException bfApiException(Class<?> c, String methName,
			String msg, Throwable e) {
		if (msg == null)
			msg = "Internal BuildForge Error"; //$NON-NLS-1$
		StringBuffer sb = new StringBuffer(msg); //$NON-NLS-1$
		sb
				.append(" in method ").append(c.getName()).append('.').append(methName); //$NON-NLS-1$
		BFAPIException apie = new BFAPIException(sb.toString());
		if (e != null)
			apie.initCause(e);
		return apie;
	}

	private static String getBFSystemPassword(String hostname, int port) {
		String systemPassword = BFRestSPIImpl.getPassword(hostname, port);
		return (systemPassword != null) ? systemPassword
				: defaultBfServerSystemPassword;
	}

	/**
	 * Initialize the Build Forge build class for RTLM
	 * 
	 * @param apiConn
	 * @throws BFAPIException
	 */
	private static void initBuildClasses(APIClientConnection apiConn)
			throws BFAPIException {
		if (buildClassesInitialized)
			return;
		buildClassesInitialized = true;

		BuildForgeArtifacts.initBuildClass(apiConn,
				BFConstants.rtlmJobBuildClass, BFConstants.bfJobPurgeDays);

	}

	/**
	 * Initialize the Build Forge collector for RTLM
	 * 
	 * @param apiConn
	 * @param collectorId
	 * @throws BFAPIException
	 */
	public static void initCollector(APIClientConnection apiConn,
			String collectorId) throws BFAPIException {
		if (buildCollectorInitialized)
			return;
		buildCollectorInitialized = true;

		BuildForgeArtifacts.createCollector(apiConn,
				BFConstants.agentCollectorId);
	}

}
