/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.rational.test.lm.spi.util.topology;

import java.text.BreakIterator;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * Name mapping is used to map the name which is from the external system to
 * RTLM's asset name
 * 
 */
public class AssetNameMapping {

	// The following array comes from RTLM catalog schema xml file
	private final static String[] rtlm_OsList = { "SCO OpenServer",
			"SCO 6 OpenServer ", "i5 OperatingSystem", "i5/OS 5.3",
			"i5/OS 5.4", "i5/OS 6.1", "OS/390", "OS/390 2.8", "OS/390 2.10",
			"OS/390 2.9", "Novell Netware", "Novell Netware 5.1",
			"Novell Netware 4.11", "Novell Netware 6.0", "Novell Netware 6.5",
			"Solaris", "Solaris 5.10", "Solaris 5.8", "Solaris 5.6",
			"Solaris 5.9", "Solaris 5.7", "Solaris Zones", "HP-UX",
			"HP-UX 11i v2", "HP-UX 11i v3", "SCO Unixware",
			"SCO Unixware 7.1.3", "SCO Unixware 7.1.4", "SCO Unixware 7.0.1",
			"SCO Unixware 7.1.1", "FreeBSD", "Free BSD 4", "Free BSD 5",
			"Free BSD 7", "Free BSD 6", "AIX", "AIX 5L", "AIX 5.1", "AIX 5.2",
			"AIX 5.3", "AIX 4.3.3", "DOS", "MS Dos 3.1", "z/OS", "z/OS 1.10",
			"z/OS 1.5", "z/OS 1.6", "z/OS 1.2", "z/OS 1.9", "z/OS 1.3",
			"z/OS 1.8", "z/OS 1.1", "z/OS e System", "z/OS 1.4", "z/OS 1.7",
			"Mac OS", "Max OS 10.3", "Max OS 10.4", "Max OS 10.2",
			"Max OS 10.5", "Max OS 8", "Max OS 10.1", "Max OS 9", "z/VM",
			"z/VM 4.4", "z/VM 5.1", "z/VM 5.2", "z/VM 4.3", "True 64 Unix",
			"Tru64 5.1", "Tru64 5.1b", "Tru64 5.1a", "Darwin", "IRIX",
			"IRIX 6.5", "Windows", "Windows Vista Business", "Windows 98",
			"Windows Server 2003 Enterprise Edition",
			"Windows 2000 Advanced Server",
			"Windows Server 2003 Datacenter Edition",
			"Windows Server 2008 Enterprise Edition",
			"Windows Millennium Edition", "Windows 2000 DataCenter Server",
			"Windows Server 2003", "Windows 2000 Professional",
			"Windows Vista Ultimate", "Windows Server 2003 Standard Edition",
			"Vista", "Windows Vista Enterprise", "Windows XP",
			"Windows Server 2008 Standard Edition",
			"Windows Server 2008 Datacenter Edition",
			"Windows XP Professional", "Windows NT", "Windows Server 1998",
			"OS/2", "OS/2 for eBusiness Warp Server", "OS/2 4.0 Warp ",
			"Linux", "Novell Linux", "Novell Linux Desktop 9.0", "Mandravia",
			"Mandravia 2007.1", "Mandravia 2006.0", "Mandravia 2008.0",
			"Mandravia 2007.0", "Fedora", "Fedora 8", "Fedora 5", "Fedora 6",
			"Fedora 7", "Gentoo", "Gentoo 2007.0", "Gentoo 2005.0",
			"Gentoo 2006.0", "Gentoo 2006.1", "Gentoo 2005.1-1", "OpenLinux",
			"OpenLinux Workstation 3.1.1", "United Linux", "United Linux 1.0",
			"SuSE Linux", "SLES", "SLES 11.0", "SLES 10.0", "SLES 7.0",
			"SLES 9.0", "SLES 9.1", "SLES 8.0", "SuSE Linux Desktop",
			"SuSE Linux Desktop 1.0", "SLED", "SLED 11.0", "SLED 10.0",
			"openSuSE", "openSuSE 10.x", "openSuSE 8.x", "openSuSE 7.x",
			"openSuSE 9.x", "Asianux Linux", "Asianux Linux 3.0",
			"Asianux Linux 2.0", "RedHat Linux", "RHEL 5.0 WS",
			"RHEL 2.1 WS", "RHEL 5.0 AS/ES", "RHEL 3.0 WS", "RedHat 8.0",
			"RHEL 4.0 AS/ES", "RHEL 6.0 AS/ES", "RHEL 2.1 AS/ES", "RedHat 7.2",
			"RHEL 4.0 WS", "RHEL 6.0 WS", "RedHat 7.1", "RHEL 3.0 AS/ES",
			"RedHat 9.0", "RedHat 7.3", "Ubuntu", "Ubuntu Server 7.10",
			"Ubuntu Desktop 7.10", "Red Flag", "Red Flag 5.0 Advanced Server",
			"Red Flag 4.0 DC Server", "Red Flag 4.1 DC Server",
			"Red Flag 4.0 Advanced Server", "Red Flag 4.1 Advanced Server",
			"Red Flag 5.0 DC Server", "CentOS", "CentOS 5", "CentOS 4.6",
			"CentOS 5.1", "CentOS 3.1", "Debian", "Turbo Linux",
			"Turbo Linux 7", "Turbo Linux 8", "Turbo Linux 6", "Slackware",
			"Slackware 10.1", "Slackware 10.2", "Slackware 12.0",
			"Slackware 11.0", "Slackware 10.0", "VMWare ESX Server",
			"VMWare ESX Server 3.0", "VMWare ESX Server 2.5", "Sequent DYNIX",
			"Sequent DYNIX 4.5", "Sequent DYNIX 4.4", "Sequent DYNIX 4.6",
			"Reliant Unix", "Reliant Unix 5.44", "Reliant Unix 5.45" };

	private final static String[] rtlm_CpuTypeList = {"Sparc", "x86", "x86-32", 
	        "x86-64", "EM64T", "AMD64", "Power", "IA64", "System I", 
	        "System z", "PA-RISC", "Alpha"};       
	
	private final static String[] rtlm_CpuArchList = { "x86-32", "x86-64",
			"IA64", "SPARC", "POWER", "PowerPC", "PA-RISC", "System i",
			"System z" };

	private final static String[] rtlm_SoftwareInstallList = { "Browsers",
			"Firefox", "Firefox 2.0", "Internet Explorer",
			"Internet Explorer 7", "Internet Explorer 6", "Safari",
			"Safari 2.0", "Safari 3.0", "Test Adapter", "Rational Manual Tester", 
			"Rational Functional Tester", "Rational Performance Tester",
			"Rational Service Tester", "Rational Test RealTime",
			"Rational Robot", "Rational AppScan Tester Edition", 
			"Sample: Command Line Adapter", "Tomcat", "Tomcat 6.0",
			"Tomcat 4.1", "Tomcat 5.5", "BEA", "Weblogic Server 10.3",
			"Weblogic 9.2", "Weblogic Server 10.0", "WAS", "WAS 6.1",
			"WAS 7.0", "WAS 6.0", "RPT", "RPT 7.x", "RPT 6.x", "DataBase",
			"Oracle", "Oracle 11g", "Oracle 10g", "Oracle 9i", "Sybase",
			"SQL Server", "SQL Server 2008", "SQL Server 2000",
			"SQL Server 2005", "Informix", "DB2", "DB2 7.x", "DB2 8.x",
			"DB2 9.x", "Build Forge Agent"};

	private final static String[] rtlm_VirtualImageTypes = {
        "Microsoft Hyper-V", //$NON-NLS-N$
        "Microsoft Virtual Server", //$NON-NLS-N$
        "VMware ESX", //$NON-NLS-N$
        "VMware Workstation", //$NON-NLS-N$
        "XenSource"  //$NON-NLS-N$
	};
	
	// Handle the speical case, for example, "SunOS" equals with "Solaris"
	private final static HashMap<String, String> specialValue = new HashMap<String, String>();

	static {
		// Keys are converted to lower case to ignore the case when compare
		specialValue.put("SunOS".toLowerCase(), "Solaris");
		specialValue.put("X86".toLowerCase(), "x86-32");
		specialValue.put("WebSphere Application Server".toLowerCase(), "WAS");
		specialValue.put("Rational Functional Tester".toLowerCase(), "RFT");
	}

	/**
     * Get RTLM's CPU Type name
     * 
     * @param cpu
     * @return the CPU type name used in RTLM
     */
    public static String getRTLMCPUTypeName(String cpu) {
        if (cpu == null || cpu.length() == 0)
            return cpu;
        else {
            return AssetNameMapping.getMatchedResult(cpu,
                    AssetNameMapping.rtlm_CpuTypeList);
        }
    }
	
	/**
	 * Get RTLM's CPU Architecture name
	 * 
	 * @param cpu
	 * @return the CPU architecture name used in RTLM
	 */
	public static String getRTLMCPUArchName(String cpu) {
		if (cpu == null || cpu.length() == 0)
			return cpu;
		else {
			String cpuArch = AssetNameMapping.getMatchedResult(cpu,
					AssetNameMapping.rtlm_CpuArchList);
			/*
			 * if ("x86".equalsIgnoreCase(cpuArch)) cpuArch = "x86-32";
			 */
			return cpuArch;
		}
	}

	/**
	 * Get RTLM's SoftwareInstall name
	 * 
	 * @param softwareName
	 * @return the SoftwareInstall name used in RTLM
	 */
	public static String getRTLMSoftwareInstallName(String softwareName) {
		if (softwareName == null || "".equalsIgnoreCase(softwareName))
			return softwareName;
		else {
			String softwareInstall = AssetNameMapping.getMatchedResult(
					softwareName, AssetNameMapping.rtlm_SoftwareInstallList);
			return softwareInstall;
		}
	}

	/**
	 * Get RTLM's OS name
	 * 
	 * @param osName
	 * @return the OS name used in RTLM
	 */
	public static String getRTLMOSName(String osName) {
		if (osName == null || "".equalsIgnoreCase(osName))
			return osName;
		else
			return AssetNameMapping.getMatchedResult(osName,
					AssetNameMapping.rtlm_OsList);
	}

	/**
     * Get RTLM's Image Type constant
     * 
     * @param ImageType
     * @return the Image Type name used in RTLM
     */
    public static String getRTLMImageType(String imageType) {
        if (imageType == null || imageType.length() == 0)
            return imageType;
        else
            return AssetNameMapping.getMatchedResult(
                    imageType, AssetNameMapping.rtlm_VirtualImageTypes);
    }
	
	private static String getMatchedResult(String target, String[] sourceList) {
		int score = 0;
		String result = null;

		// Get the list of all the words in the target like "Windows 2003
		// Enterprise" = Windows, 2003, Enterprise"
		ArrayList<String> targetList = new ArrayList<String>();

		BreakIterator boundary = BreakIterator.getWordInstance();
		boundary.setText(target);

		int start = boundary.first();
		for (int end = boundary.next(); end != BreakIterator.DONE; start = end, end = boundary
				.next()) {
			String content = target.substring(start, end);
			if (!content.equalsIgnoreCase(" "))
				targetList.add(content);
		}

		// begin to search
		for (String rtlmName : sourceList) {
			if (target.equalsIgnoreCase(rtlmName)) {
				return rtlmName;
			} else {
				int currentScore = 0;
				if (rtlmName.toLowerCase().contains(
						targetList.get(0).toLowerCase())) {
					for (String content : targetList) {
						if (rtlmName.toLowerCase().contains(
								content.toLowerCase()))
							currentScore++;
					}
				}

				if (currentScore > score) {
					score = currentScore;
					result = rtlmName;
				}
			}
		}

		// handle the special case
		if (result == null) {
			if (specialValue.containsKey(target.toLowerCase())) {
				result = specialValue.get(target.toLowerCase());
			}
		}
		return result;
	}
}
