<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/">
    <xsl:output method="xml" indent="no"/>
    
    <!-- parameters -->
    <xsl:param name="expand-testcase"/>
    <xsl:param name="expand-testsuitelog"/>
    <xsl:param name="expand-testscript"/>
    <xsl:param name="expand-remotescript"/>
    <xsl:param name="expand-labresource"/>
    <xsl:param name="expand-ewi"/>
    <xsl:param name="expand-testphase"/>
    <xsl:param name="expand-buildrecord"/>
    <xsl:param name="expand-workitem"/>
    <xsl:param name="expand-testplan"/>
    
    <!-- MAIN TEMPLATE -->
    <xsl:template match="/">
        <xsl:apply-templates/>
    </xsl:template>
    
    <!--
      This template essentially catches all nodes that do not need to be expanded and passes them 
      directly into the result document. (more or less)
      -->
    <xsl:template match="*">
        <xsl:copy>
            <!-- copy attributes -->
            <xsl:copy-of select="@*"/>
            
            <!-- copy or expand children -->
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
    
    <!-- Testcase -->
    <xsl:template match="rqm:executionresult/rqm:testcase">
        <xsl:choose>
            <xsl:when test="$expand-testcase">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Testsuite Log -->
    <xsl:template match="rqm:executionresult/rqm:testsuitelog">
        <xsl:choose>
            <xsl:when test="$expand-testsuitelog">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Testscript -->
    <xsl:template match="rqm:executionresult/rqm:testscript">
        <xsl:choose>
            <xsl:when test="$expand-testscript">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Remote Script -->
    <xsl:template match="rqm:executionresult/rqm:remotescript">
        <xsl:choose>
            <xsl:when test="$expand-remotescript">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Lab Resource -->
    <xsl:template match="rqm:executionresult/rqm:labresource">
        <xsl:choose>
            <xsl:when test="$expand-labresource">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Execution Work Item -->
    <xsl:template match="rqm:executionresult/rqm:executionworkitem">
        <xsl:choose>
            <xsl:when test="$expand-ewi">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Test Phase -->
    <xsl:template match="rqm:executionresult/rqm:testphase">
        <xsl:choose>
            <xsl:when test="$expand-testphase">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Build Record -->
    <xsl:template match="rqm:executionresult/rqm:buildrecord">
        <xsl:choose>
            <xsl:when test="$expand-buildrecord">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Work Item -->
    <xsl:template match="rqm:executionresult/rqm:defect[not(@rel)]">
        <xsl:choose>
            <xsl:when test="$expand-workitem">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Testplan -->
    <xsl:template match="rqm:executionresult/rqm:testplan">
        <xsl:choose>
            <xsl:when test="$expand-testplan">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>