<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" 
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:purl="http://purl.org/dc/elements/1.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/">
    <xsl:output method="xml" indent="no"/>

    <!-- included stylesheets -->
    <xsl:include href="cache://stylesheets/execution-sequence-result/table.xsl"/>
    <!--<xsl:include href="table-view.xsl"/>-->

    <!-- parameters -->
    <xsl:param name="locale"/>
	<xsl:param name="font"/>
    
    <xsl:param name="custom-header"/>
    <xsl:param name="custom-footer"/>
    
    <xsl:param name="hyperlink"/>

    <!-- variables -->
    <xsl:variable name="strings" select="document('cache://fopResources/fopmessages.xml')/fopmessages//*[name()=$locale]"/>
    
    <!-- DOCUMENT LAYOUT -->
    <xsl:template match="/">
        <fo:root font-family="{$font}">
            <fo:layout-master-set>
                <fo:simple-page-master master-name="standard-landscape-page" 
                    page-width="8.5in" page-height="11in"
                    margin="1in">
                    <fo:region-body/>
                    <fo:region-before extent="0in"/>
                    <fo:region-after extent="0in"/>
                </fo:simple-page-master>
            </fo:layout-master-set>
            
            <fo:page-sequence master-reference="standard-landscape-page"
                initial-page-number="1" format="1">
                <!-- Header -->
                <fo:static-content flow-name="xsl-region-before">
                    <fo:block><xsl:value-of select="$custom-header"/></fo:block>
                </fo:static-content>
                
                <!-- Footer -->
                <fo:static-content flow-name="xsl-region-after">
                    <fo:block><xsl:value-of select="$custom-footer"/></fo:block>
                    <fo:block text-align="center"><fo:leader leader-pattern="rule" leader-length="60%"/></fo:block>
                    <fo:block text-align="center">
                        <fo:basic-link xsl:use-attribute-sets="footer-link" external-destination="{$hyperlink}">
                            <xsl:value-of select="$hyperlink"/>
                        </fo:basic-link>
                    </fo:block>
                </fo:static-content>
                
                <!-- Body -->
                <fo:flow flow-name="xsl-region-body">
                    <!-- get associated sequence -->
                    <xsl:variable name="sequenceURI" select="list/rqm:executionsequenceresult[position()=1]/rqm:executionSequence/@href"/>
                    <xsl:variable name="sequence" select="document( concat('rest://', $sequenceURI) )/rqm:executionsequence"/>
                    
                    <!-- page title -->
                    <fo:block xsl:use-attribute-sets="page-title"><xsl:value-of select="$sequence/dc:title"/></fo:block>
                    <fo:block xsl:use-attribute-sets="page-subtitle"><xsl:value-of select="$strings/titleResultsTableView"/></fo:block>
                    
                    <!-- results table -->
                    <xsl:call-template name="buildExecutionSequenceResultTable">
                        <xsl:with-param name="strings" select="$strings"/>
                        <xsl:with-param name="sequence-results" select="list/rqm:executionsequenceresult"/>
                        
<!--                    <xsl:with-param name="render-build-record" select="'true'"/>-->
                        <xsl:with-param name="render-state" select="true()"/>
                        <xsl:with-param name="render-start-time" select="true()"/>
                        <xsl:with-param name="render-end-time" select="true()"/>
                        <xsl:with-param name="render-summary-result" select="true()"/>
                    </xsl:call-template>
                </fo:flow>
            </fo:page-sequence>
        </fo:root>    
    </xsl:template>
    
</xsl:stylesheet>