<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/">
    <xsl:output method="xml" indent="no"/>
    
    <!-- parameters -->
    <xsl:param name="expand-testcase"/>
    <xsl:param name="expand-testscript"/>
    <xsl:param name="expand-remotescript"/>
    <xsl:param name="expand-testplan"/>
    <xsl:param name="expand-testphase"/>
    <xsl:param name="expand-configuration"/>
    <xsl:param name="expand-currentexecutionresult"/>
    <xsl:param name="expand-executionresult"/>
    <xsl:param name="expand-blockingdefects"/>
    
    <!-- MAIN TEMPLATE -->
    <xsl:template match="/">
        <xsl:apply-templates/>
    </xsl:template>
    
    <!--
      This template essentially catches all nodes that do not need to be expanded and passes them 
      directly into the result document. (more or less)
      -->
    <xsl:template match="*">
        <xsl:copy>
            <xsl:copy-of select="@*"/>                <!-- copy attributes -->
            <xsl:apply-templates/>                    <!-- copy or expand children -->
        </xsl:copy>
    </xsl:template>
    
    <!-- Testcase -->
    <xsl:template match="rqm:executionworkitem/rqm:testcase">
        <xsl:choose>
            <xsl:when test="$expand-testcase">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Testscript -->
    <xsl:template match="rqm:executionworkitem/rqm:testscript">
        <xsl:choose>
            <xsl:when test="$expand-testscript">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Remote Script -->
    <xsl:template match="rqm:executionworkitem/rqm:remotescript">
        <xsl:choose>
            <xsl:when test="$expand-remotescript">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Testplan -->
    <xsl:template match="rqm:executionworkitem/rqm:testplan">
        <xsl:choose>
            <xsl:when test="$expand-testplan">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Test Phase -->
    <xsl:template match="rqm:executionworkitem/rqm:testphase">
        <xsl:choose>
            <xsl:when test="$expand-testphase">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- Configuration (Test Environment) -->
    <xsl:template match="rqm:executionworkitem/rqm:configuration">
        <xsl:choose>
            <xsl:when test="$expand-configuration">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Current Execution Result -->
    <xsl:template match="rqm:executionworkitem/rqm:currentexecutionresult">
        <xsl:choose>
            <xsl:when test="$expand-currentexecutionresult">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Execution Result -->
    <xsl:template match="rqm:executionworkitem/rqm:executionresult">
        <xsl:choose>
            <xsl:when test="$expand-executionresult">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Blocking Defect -->
    <xsl:template match="rqm:executionworkitem/rqm:blockingdefects">
        <xsl:choose>
            <xsl:when test="$expand-blockingdefects">
                <xsl:copy-of select="document(concat('rest://', @href))"/></xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>