<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:atom="http://www.w3.org/2005/Atom" 
	xmlns:xhtml="http://www.w3.org/1999/xhtml"
	xmlns:cleaner="xalan://com.ibm.rqm.xslfo.service.extension.XmlCleanerExtension"
	xmlns:time="xalan://com.ibm.rqm.xslfo.service.extension.TimeUtilsExtension"
	exclude-result-prefixes="xsl atom xhtml">
	
	<!-- included stylesheets -->
	<xsl:include href="cache://stylesheets/attribute-sets.xsl"/>
	<xsl:include href="cache://fopResources/wrapping-rules.xsl"/>
	<xsl:include href="cache://fopResources/xhtml-to-xslfo-ns.xsl"/>
	
	<!-- parameters -->
	<xsl:param name="locale"/>
	<xsl:param name="font"/>
	
	<xsl:param name="custom-header"/>
	<xsl:param name="custom-footer"/>
	
	<xsl:param name="hyperlink"/>
	
	<!-- variables -->
	<xsl:variable name="img-scale">.4</xsl:variable>
	<xsl:variable name="strings" select="document('cache://fopResources/fopmessages.xml')/fopmessages//*[name()=$locale]"/>
	
	<!-- DOCUMENT LAYOUT -->
	<xsl:template match="/">
		<fo:root font-family="{$font}">
			<fo:layout-master-set>
				<fo:simple-page-master master-name="standard-letter-page" 
					page-height="11in" page-width="8.5in"
					margin="1in">
					<fo:region-body/>
					<fo:region-before extent="0in"/>
					<fo:region-after extent="0in"/>
				</fo:simple-page-master>
			</fo:layout-master-set>
			
			<fo:page-sequence master-reference="standard-letter-page"
				initial-page-number="1" format="1">
				<!-- Header -->
				<fo:static-content flow-name="xsl-region-before">
					<fo:block><xsl:value-of select="$custom-header"/></fo:block>
				</fo:static-content>
				
				<!-- Footer -->
				<fo:static-content flow-name="xsl-region-after">
					<fo:block><xsl:value-of select="$custom-footer"/></fo:block>
					<fo:block text-align="center"><fo:leader leader-pattern="rule" leader-length="60%"/></fo:block>
					<fo:block text-align="center">
						<fo:basic-link xsl:use-attribute-sets="footer-link" external-destination="{$hyperlink}">
							<xsl:value-of select="$hyperlink"/>
						</fo:basic-link>
					</fo:block>
				</fo:static-content>
				
				<!-- Body -->
				<fo:flow flow-name="xsl-region-body">
					<xsl:apply-templates select="atom:feed"/>
				</fo:flow>
			</fo:page-sequence>
		</fo:root>    
	</xsl:template>
	
	<!-- MAIN CONTENT -->
	<xsl:template match="atom:feed">
		<!-- page title -->
		<fo:block xsl:use-attribute-sets="page-title" space-after="8mm"><xsl:value-of select="atom:title"/></fo:block>
			
		<!-- entries -->
		<xsl:for-each select="atom:entry">
			<fo:block space-after="4mm" font-size="8pt">
				<!-- entry header -->
				<fo:block font-weight="bold"><xsl:value-of select="atom:title"/></fo:block>
				<fo:block space-after="2mm"><xsl:value-of select="time:formatTimestamp(atom:updated)"/></fo:block>
				
				<!-- entry summary -->
				<xsl:choose>
					<xsl:when test="atom:summary[@type='xhtml']">
						<xsl:apply-templates select="atom:summary"/></xsl:when>
					<xsl:otherwise>
						<xsl:variable name="html" select="."/>
						<fo:block><xsl:apply-templates select="cleaner:tidy($html)"/></fo:block>
					</xsl:otherwise>
				</xsl:choose>
			</fo:block>	
		</xsl:for-each>
	</xsl:template>
	
	<!--
	  Seems like we need special handling for tables that were created in the rich text editor (though I don't really
	  understand why).
	  -->
	<xsl:template match="xhtml:table[@summary='This is a table in Rich Text Editor']">
		<fo:table>
			<fo:table-body>
				<xsl:apply-templates/>
			</fo:table-body>
		</fo:table>
	</xsl:template>
	
	<!-- For tables that were not created in RQM, we need to do some explicit checks to ensure that we do not create 
		tables without table data. Otherwise, FOP will throw an HTTP 500 error due to missing child elements.
		See defect 31547 for more details. -->
	<xsl:template match="xhtml:table[not(@summary)]">
		<fo:table>
			<xsl:attribute name="border">
				<xsl:value-of select="concat(@border,'pt solid black')"/>
			</xsl:attribute>
			<xsl:attribute name="padding">
				<xsl:value-of select="@cellpadding"/>
			</xsl:attribute>
			<xsl:if test="xhtml:tr/xhtml:th">
				<fo:table-header>
					<xsl:apply-templates select="xhtml:tr/xhtml:th"/>
				</fo:table-header>
			</xsl:if>
			<fo:table-body>
				<xsl:choose>
					<xsl:when test="xhtml:tr">
						<xsl:apply-templates select="xhtml:tr" />
					</xsl:when>
					<xsl:when test="xhtml:tbody/xhtml:tr">
						<xsl:apply-templates select="xhtml:tbody/xhtml:tr"/>
					</xsl:when>
				</xsl:choose>
			</fo:table-body>
		</fo:table>
	</xsl:template>
	
</xsl:stylesheet>