<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    
    xmlns:time="xalan://com.ibm.rqm.xslfo.service.extension.TimeUtilsExtension">
    
    <!-- included stylesheets -->
    <xsl:include href="cache://stylesheets/attribute-sets.xsl"/>
    <xsl:include href="cache://fopResources/wrapping-rules.xsl"/>
    <!--<xsl:include href="../attribute-sets.xsl"/>-->
    
    <!--  
        This template will generate a table consisting of the given testplans.  The table will 
        only render the columns for which some value was set.
    -->
    <xsl:template name="buildTestplanTable">
        <xsl:param name="strings"/>
        <xsl:param name="testplans"/>
        
        <xsl:param name="render-id"/>
        <xsl:param name="render-name"/>
        <xsl:param name="render-description"/>
        <xsl:param name="render-last-modified"/>
        <xsl:param name="render-state"/>
        <xsl:param name="render-creator"/>
        <xsl:param name="render-owner"/>
        <xsl:param name="render-risk"/>
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <!-- id -->
                <xsl:if test="$render-id">
                    <fo:table-cell xsl:use-attribute-sets="border id-header">
                        <fo:block><xsl:apply-templates select="$strings/labelId"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
                <!-- name -->
                <xsl:if test="$render-name">
                    <fo:table-cell xsl:use-attribute-sets="border" width="75">
                        <fo:block><xsl:apply-templates select="$strings/labelName"/></fo:block>
                    </fo:table-cell>  
                </xsl:if>
                <!-- description -->
                <xsl:if test="$render-description">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labeldescription"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- last updated -->
                <xsl:if test="$render-last-modified">
                    <fo:table-cell xsl:use-attribute-sets="border timestamp-header">
                        <fo:block><xsl:apply-templates select="$strings/reqStatusUpdated"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- state -->
                <xsl:if test="$render-state">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelState"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- creator -->
                <xsl:if test="$render-creator">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/creator"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- owner -->
                <xsl:if test="$render-owner">
                    <fo:table-cell xsl:use-attribute-sets="border">
                        <fo:block><xsl:apply-templates select="$strings/labelOwner"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
                <!-- risk -->
                <xsl:if test="$render-risk">
                    <fo:table-cell xsl:use-attribute-sets="border" width="20">
                        <fo:block><xsl:apply-templates select="$strings/labelRisk"/></fo:block>
                    </fo:table-cell>    
                </xsl:if>
            </fo:table-header>
            <fo:table-body>
                <xsl:for-each select="$testplans">
                    <fo:table-row>
                        <!-- id -->
                        <xsl:if test="$render-id">
                            <fo:table-cell xsl:use-attribute-sets="border id-cell">
                                <fo:block><xsl:value-of select="substring-after(dc:identifier, 'testplan:')"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- name -->
                        <xsl:if test="$render-name">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:apply-templates select="dc:title"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- description -->
                        <xsl:if test="$render-description">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:value-of select="dc:description"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- last modified -->
                        <xsl:if test="$render-last-modified">
                            <fo:table-cell xsl:use-attribute-sets="border timestamp-cell">
                                <fo:block><xsl:value-of select="time:formatTimestamp(alm:updated)"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- state -->
                        <xsl:if test="$render-state">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <xsl:variable name="state" select="alm:state"/>
                                
                                <xsl:choose>
                                    <!-- new -->
                                    <xsl:when test="$state='com.ibm.rqm.planning.common.new'">
                                        <fo:block><xsl:value-of select="$strings/com.ibm.rqm.planning.common.new"/></fo:block></xsl:when>
                                    <!-- under review -->
                                    <xsl:when test="$state='com.ibm.rqm.planning.common.underreview'">
                                        <fo:block><xsl:value-of select="$strings/com.ibm.rqm.planning.common.underreview"/></fo:block></xsl:when>
                                    <!-- approved -->
                                    <xsl:when test="$state='com.ibm.rqm.planning.common.approved'">
                                        <fo:block><xsl:value-of select="$strings/com.ibm.rqm.planning.common.approved"/></fo:block></xsl:when>
                                    <!-- retired -->
                                    <xsl:when test="$state='com.ibm.rqm.planning.common.retired'">
                                        <fo:block><xsl:value-of select="$strings/com.ibm.rqm.planning.common.retired"/></fo:block></xsl:when>
                                    <!-- default -->
                                    <xsl:otherwise>
                                        <fo:block><xsl:apply-templates select="alm:state"/></fo:block>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- creator -->
                        <xsl:if test="$render-creator">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:value-of select="dc:creator"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- owner -->
                        <xsl:if test="$render-owner">
                            <fo:table-cell xsl:use-attribute-sets="border">
                                <fo:block><xsl:value-of select="alm:owner"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                        <!-- risk -->
                        <xsl:if test="$render-risk">
                            <fo:table-cell xsl:use-attribute-sets="border" text-align="center">
                                <fo:block><xsl:value-of select="rqm:risk/rqm:riskLevel"/></fo:block>
                            </fo:table-cell>
                        </xsl:if>
                    </fo:table-row>
                </xsl:for-each>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
</xsl:stylesheet>