<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/">
    
    <!-- included stylesheets -->
    <xsl:include href="cache://stylesheets/attribute-sets.xsl"/>
    <xsl:include href="cache://fopResources/wrapping-rules.xsl"/>
    <!--<xsl:include href="../attribute-sets.xsl"/>-->
    
    <xsl:template name="buildWorkItemTable">
        <xsl:param name="strings"/>
        <xsl:param name="item-list"/>
        
        <xsl:param name="render-id"/>
        <xsl:param name="render-type"/>
        <xsl:param name="render-headline"/>
        <xsl:param name="render-state"/>
        
        <xsl:if test="count($item-list) > 0">
            <fo:table xsl:use-attribute-sets="artifact-table">
                <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                    <!-- WI# -->
                    <xsl:if test="$render-id">
                        <fo:table-cell xsl:use-attribute-sets="border" width="25">
                            <fo:block></fo:block>
                        </fo:table-cell>
                    </xsl:if>
                    <!-- Type -->
                    <xsl:if test="$render-type">
                        <fo:table-cell xsl:use-attribute-sets="border" width="50">
                            <fo:block><xsl:value-of select="$strings/labelType"/></fo:block>
                        </fo:table-cell>  
                    </xsl:if>
                    <!-- Headline -->
                    <xsl:if test="$render-headline">
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:value-of select="$strings/labelHeadline"/></fo:block>
                        </fo:table-cell>    
                    </xsl:if>
                    <!-- State -->
                    <xsl:if test="$render-state">
                        <fo:table-cell xsl:use-attribute-sets="border" width="50">
                            <fo:block><xsl:value-of select="$strings/labelState"/></fo:block>
                        </fo:table-cell>    
                    </xsl:if>
                </fo:table-header>
                <fo:table-body>
                    <xsl:for-each select="$item-list">
                        <fo:table-row>
                            <!-- WI# -->
                            <xsl:if test="$render-id">
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <xsl:variable name="id" select="dc:identifier"/>
                                    <fo:block><xsl:value-of select="substring-after($id,'workitem:')"/></fo:block>
                                </fo:table-cell>
                            </xsl:if>
                            <!-- Type -->
                            <xsl:if test="$render-type">
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block><xsl:value-of select="rqm:type"/></fo:block>
                                </fo:table-cell>
                            </xsl:if>
                            <!-- Headline -->
                            <xsl:if test="$render-headline">
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block><xsl:apply-templates select="rqm:summary"/></fo:block>
                                </fo:table-cell>
                            </xsl:if>
                            <!-- State -->
                            <xsl:if test="$render-state">
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block><xsl:value-of select="alm:state"/></fo:block>
                                </fo:table-cell>
                            </xsl:if>
                        </fo:table-row>
                    </xsl:for-each>
                </fo:table-body>
            </fo:table>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="buildCalmWorkItemTable">
        <xsl:param name="strings"/>
        <xsl:param name="item-list"/>
        
        <xsl:param name="render-relationship"/>
        <xsl:param name="render-summary"/>
        <xsl:param name="link"/>
        
        <xsl:if test="count($item-list) > 0">
            <fo:table xsl:use-attribute-sets="artifact-table">
                <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                    <!-- relationship -->
                    <xsl:if test="$render-relationship">
                        <fo:table-cell xsl:use-attribute-sets="border" width="50">
                            <fo:block><xsl:value-of select="$strings/Relationship"/></fo:block>
                        </fo:table-cell>
                    </xsl:if>
                    <!-- summary -->
                    <xsl:if test="$render-summary">
                        <fo:table-cell xsl:use-attribute-sets="border">
                            <fo:block><xsl:value-of select="$strings/labelSummary"/></fo:block>
                        </fo:table-cell>  
                    </xsl:if>
                </fo:table-header>
                <fo:table-body>
                    <xsl:for-each select="$item-list">
                        <fo:table-row>
                            <!-- relationship -->
                            <xsl:if test="$render-relationship">
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block><xsl:value-of select="@rel"/></fo:block>
                                </fo:table-cell>
                            </xsl:if>
                            <!-- summary -->
                            <xsl:if test="$render-summary">
                                <fo:table-cell xsl:use-attribute-sets="border">
                                    <fo:block>
                                        <xsl:choose>
                                            <!-- link -->
                                            <xsl:when test="$link">
                                                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="@href"><xsl:apply-templates select="@summary"/></fo:basic-link>
                                            </xsl:when>
                                            <!-- plain text -->
                                            <xsl:otherwise>
                                                <xsl:value-of select="@summary"/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </fo:block>
                                </fo:table-cell>
                            </xsl:if>
                        </fo:table-row>
                    </xsl:for-each>
                </fo:table-body>
            </fo:table>
        </xsl:if>
    </xsl:template>
    
</xsl:stylesheet>